@@ -1,16 +1,13 @@
 /**
  *
- * $Id: BOP_Merge.cpp 11139 2007-07-01 14:12:10Z bdiego $
+ * $Id: BOP_Merge.cpp 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,7 +25,7 @@
  *
  * Contributor(s): Marc Freixas, Ken Hughes
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
  
 #include "BOP_Merge.h"
@@ -60,32 +57,6 @@ void BOP_Merge::mergeFaces(BOP_Mesh *m, BOP_Index v)
 	// Merge faces
 	mergeFaces();
 
-	/*
-	 * HACK: somehow triangular faces are being created with two vertices the
-	 * same.  If it's happening in BOP_Mesh::replaceVertexIndex() we should
-	 * be catching it, so either it's not happening there or we aren't
-	 * catching it (duh).  Until we figure this out, this hack cleans things.
-	 * 
-	 * Test for any invalid faces: if any two vertices are the same of a
-	 * triangle, the face is broken.  Further, I don't believe it's possible
-	 * to have any quads at this point, so if we find one send a message
-	 * to stdout.
-	 */
-
-	BOP_Faces faces = m_mesh->getFaces();
-	const BOP_IT_Faces ifacesIEnd = (faces.end());
-	for(BOP_IT_Faces faceI=faces.begin();faceI!=ifacesIEnd;faceI++) {
-		if ((*faceI)->getTAG() != BROKEN ) {
-			BOP_Index i1 = (*faceI)->getVertex(0);
-			BOP_Index i2 = (*faceI)->getVertex(1);
-			BOP_Index i3 = (*faceI)->getVertex(2);
-			if ( (*faceI)->size() == 4)
-				cout << "BOP_Merge::mergeFaces found a quad: this is an error" << endl;
-			if (i1 == i2 || i2 == i3 || i3 == i1 )
-				(*faceI)->setTAG(BROKEN);
-		}
-	}
-
 	do {
 		// Add quads ...
 		cont = createQuads();
