@@ -13,8 +13,7 @@
 #include <algorithm>
 #include <stdio.h>
 
-// sirdude fix for solaris
-#if !defined(linux) && (defined (__sparc) || defined (__sparc__))
+#if (defined (__sun__) || defined (__sun)) || (!defined(linux) && (defined (__sparc) || defined (__sparc__)))
 #include <ieeefp.h>
 #endif
 
