@@ -1,7 +1,7 @@
 /******************************************************************************
  *
  * El'Beem - Free Surface Fluid Simulation with the Lattice Boltzmann Method
- * Copyright 2003,2004 Nils Thuerey
+ * Copyright 2003-2006 Nils Thuerey
  *
  * a geometry object
  * all other geometry objects are derived from this one
@@ -16,6 +16,8 @@
 // for FGI
 #include "elbeem.h"
 
+#define TRI_UVOFFSET (1./4.)
+//#define TRI_UVOFFSET (1./3.)
 
 
 /*****************************************************************************/
@@ -29,13 +31,14 @@ ntlGeometryObject::ntlGeometryObject() :
 	mInitialVelocity(0.0), mcInitialVelocity(0.0), mLocalCoordInivel(false),
 	mGeoInitIntersect(false),
 	mGeoPartSlipValue(0.0),
+	mcGeoImpactFactor(1.),
 	mVolumeInit(VOLUMEINIT_VOLUME),
 	mInitialPos(0.),
 	mcTrans(0.), mcRot(0.), mcScale(1.),
 	mIsAnimated(false),
-	mMovPoints(), //mMovNormals(),
+	mMovPoints(), mMovNormals(),
 	mHaveCachedMov(false),
-	mCachedMovPoints(), //mCachedMovNormals(),
+	mCachedMovPoints(), mCachedMovNormals(),
 	mTriangleDivs1(), mTriangleDivs2(), mTriangleDivs3(),
 	mMovPntsInited(-100.0), mMaxMovPnt(-1),
 	mcGeoActive(1.)
@@ -62,6 +65,7 @@ bool ntlGeometryObject::checkIsAnimated() {
 	    || (mcRot.accessValues().size()>1) 
 	    || (mcScale.accessValues().size()>1) 
 	    || (mcGeoActive.accessValues().size()>1) 
+			// mcGeoImpactFactor only needed when moving
 	    || (mcInitialVelocity.accessValues().size()>1) 
 		) {
 		mIsAnimated = true;
@@ -71,6 +75,7 @@ bool ntlGeometryObject::checkIsAnimated() {
 	if(mGeoInitType==FGI_FLUID) {
 		mIsAnimated=false;
 	}
+	//errMsg("ntlGeometryObject::checkIsAnimated","obj="<<getName()<<" debug: trans:"<<mcTrans.accessValues().size()<<" rot:"<<mcRot.accessValues().size()<<" scale:"<<mcScale.accessValues().size()<<" active:"<<mcGeoActive.accessValues().size()<<" inivel:"<<mcInitialVelocity.accessValues().size()<<". isani?"<<mIsAnimated ); // DEBUG
 	return mIsAnimated;
 }
 
@@ -78,7 +83,7 @@ bool ntlGeometryObject::checkIsAnimated() {
 /* Init attributes etc. of this object */
 /*****************************************************************************/
 #define GEOINIT_STRINGS  9
-static char *initStringStrs[GEOINIT_STRINGS] = {
+static const char *initStringStrs[GEOINIT_STRINGS] = {
 	"fluid",
 	"bnd_no","bnd_noslip",
 	"bnd_free","bnd_freeslip",
@@ -139,6 +144,13 @@ void ntlGeometryObject::initialize(ntlRenderGlobals *glob)
 	mVolumeInit     = mpAttrs->readInt("geoinit_volumeinit", mVolumeInit,"ntlGeometryObject", "mVolumeInit", false);
 	if((mVolumeInit<VOLUMEINIT_VOLUME)||(mVolumeInit>VOLUMEINIT_BOTH)) mVolumeInit = VOLUMEINIT_VOLUME;
 
+	// moving obs correction factor
+	float impactfactor=1.;
+	impactfactor = (float)mpAttrs->readFloat("impactfactor", impactfactor,"ntlGeometryObject", "impactfactor", false);
+	if(getAttributeList()->exists("impactfactor") || (!mcGeoImpactFactor.isInited()) ) {
+		mcGeoImpactFactor = mpAttrs->readChannelSinglePrecFloat("impactfactor");
+	}
+
 	// override cfg types
 	mVisible = mpAttrs->readBool("visible", mVisible,"ntlGeometryObject", "mVisible", false);
 	mReceiveShadows = mpAttrs->readBool("recv_shad", mReceiveShadows,"ntlGeometryObject", "mReceiveShadows", false);
@@ -162,21 +174,13 @@ void ntlGeometryObject::initialize(ntlRenderGlobals *glob)
 	}
 
 	float geoactive=1.;
-	geoactive = mpAttrs->readFloat("geoactive", geoactive,"ntlGeometryObject", "geoactive", false);
+	geoactive = (float)mpAttrs->readFloat("geoactive", geoactive,"ntlGeometryObject", "geoactive", false);
 	if(getAttributeList()->exists("geoactive") || (!mcGeoActive.isInited()) ) {
-		mcGeoActive = mpAttrs->readChannelFloat("geoactive");
+		mcGeoActive = mpAttrs->readChannelSinglePrecFloat("geoactive");
 	}
 	// always use channel
-	if(!mcGeoActive.isInited()) { mcGeoActive = AnimChannel<double>(geoactive); }
+	if(!mcGeoActive.isInited()) { mcGeoActive = AnimChannel<float>(geoactive); }
 
-	//if(    (mcTrans.accessValues().size()>1)  // VALIDATE
-	    //|| (mcRot.accessValues().size()>1) 
-	    //|| (mcScale.accessValues().size()>1) 
-	    //|| (mcGeoActive.accessValues().size()>1) 
-	    //|| (mcInitialVelocity.accessValues().size()>1) 
-		//) {
-		//mIsAnimated = true;
-	//}
 	checkIsAnimated();
 
 	mIsInitialized = true;
@@ -317,12 +321,12 @@ void ntlGeometryObject::sceneAddTriangleNoVert(int *trips,
 		(dst) = AnimChannel< ntlVec3Gfx >(vals,time); 
 
 #define ADD_CHANNEL_FLOAT(dst,nvals,val) \
-		valsd.clear(); time.clear(); elbeemSimplifyChannelFloat(val,&nvals); \
+		valsfloat.clear(); time.clear(); elbeemSimplifyChannelFloat(val,&nvals); \
 		for(int i=0; i<(nvals); i++) { \
-			valsd.push_back( (val)[i*2+0] ); \
+			valsfloat.push_back( (val)[i*2+0] ); \
 			time.push_back( (val)[i*2+1] ); \
 		} \
-		(dst) = AnimChannel< double >(valsd,time); 
+		(dst) = AnimChannel< float >(valsfloat,time); 
 
 void ntlGeometryObject::initChannels(
 		int nTrans, float *trans, int nRot, float *rot, int nScale, float *scale,
@@ -332,7 +336,7 @@ void ntlGeometryObject::initChannels(
 	if(debugInitc) { debMsgStd("ntlGeometryObject::initChannels",DM_MSG,"nt:"<<nTrans<<" nr:"<<nRot<<" ns:"<<nScale, 10); 
 	                 debMsgStd("ntlGeometryObject::initChannels",DM_MSG,"na:"<<nAct<<" niv:"<<nIvel<<" ", 10); }
 	vector<ntlVec3Gfx> vals;
-	vector<double> valsd;
+	vector<float>  valsfloat;
 	vector<double> time;
 	if((trans)&&(nTrans>0)) {  ADD_CHANNEL_VEC(mcTrans, nTrans, trans); }
 	if((rot)&&(nRot>0)) {      ADD_CHANNEL_VEC(mcRot, nRot, rot); }
@@ -340,14 +344,6 @@ void ntlGeometryObject::initChannels(
 	if((act)&&(nAct>0)) {      ADD_CHANNEL_FLOAT(mcGeoActive, nAct, act); }
 	if((ivel)&&(nIvel>0)) {    ADD_CHANNEL_VEC(mcInitialVelocity, nIvel, ivel); }
 
-	//if(    (mcTrans.accessValues().size()>1)  // VALIDATE
-	    //|| (mcRot.accessValues().size()>1) 
-	    //|| (mcScale.accessValues().size()>1) 
-	    //|| (mcGeoActive.accessValues().size()>1) 
-	    //|| (mcInitialVelocity.accessValues().size()>1) 
-		//) {
-		//mIsAnimated = true;
-	//}
 	checkIsAnimated();
 	if(debugInitc) { 
 		debMsgStd("ntlGeometryObject::initChannels",DM_MSG,getName()<<
@@ -399,7 +395,7 @@ void ntlGeometryObject::applyTransformation(double t, vector<ntlVec3Gfx> *verts,
 		ntlMat4Gfx rotMat;
 		rotMat.initRotationXYZ(rot[0],rot[1],rot[2]);
 		pos += mInitialPos;
-		//errMsg("ntlGeometryObject::applyTransformation","obj="<<getName()<<" t"<<pos<<" r"<<rot<<" s"<<scale);
+		errMsg("ntlGeometryObject::applyTransformation","obj="<<getName()<<" t"<<pos<<" r"<<rot<<" s"<<scale);
 		for(int i=vstart; i<vend; i++) {
 			(*verts)[i] *= scale;
 			(*verts)[i] = rotMat * (*verts)[i];
@@ -413,7 +409,7 @@ void ntlGeometryObject::applyTransformation(double t, vector<ntlVec3Gfx> *verts,
 		}
 	} else {
 		// not animated, cached points were already returned
-		//errMsg ("ntlGeometryObject::applyTransformation","Object "<<getName()<<" used cached points ");
+		errMsg ("ntlGeometryObject::applyTransformation","Object "<<getName()<<" used cached points ");
 	}
 }
 
@@ -422,6 +418,9 @@ void ntlGeometryObject::calcTriangleDivs(vector<ntlVec3Gfx> &verts, vector<ntlTr
 	mTriangleDivs1.resize( tris.size() );
 	mTriangleDivs2.resize( tris.size() );
 	mTriangleDivs3.resize( tris.size() );
+
+	//fsTri *= 2.; // DEBUG! , wrong init!
+
 	for(size_t i=0; i<tris.size(); i++) {
 		const ntlVec3Gfx p0 = verts[ tris[i].getPoints()[0] ];
 		const ntlVec3Gfx p1 = verts[ tris[i].getPoints()[1] ];
@@ -432,17 +431,7 @@ void ntlGeometryObject::calcTriangleDivs(vector<ntlVec3Gfx> &verts, vector<ntlTr
 		int divs1=0, divs2=0, divs3=0;
 		if(normNoSqrt(side1) > fsTri*fsTri) { divs1 = (int)(norm(side1)/fsTri); }
 		if(normNoSqrt(side2) > fsTri*fsTri) { divs2 = (int)(norm(side2)/fsTri); }
-		//if(normNoSqrt(side3) > fsTri*fsTri) { divs3 = (int)(norm(side3)/fsTri); }
-		/*if(getMeshAnimated()) {
-			vector<ntlSetVec3f> *verts =mcAniVerts.accessValues();
-			for(int s=0; s<verts->size(); s++) {
-				int tdivs1=0, tdivs2=0, tdivs3=0;
-				if(normNoSqrt(side1) > fsTri*fsTri) { tdivs1 = (int)(norm(side1)/fsTri); }
-				if(normNoSqrt(side2) > fsTri*fsTri) { tdivs2 = (int)(norm(side2)/fsTri); }
-				if(tdivs1>divs1) divs1=tdivs1;
-				if(tdivs2>divs2) divs2=tdivs2;
-			}
-		}*/
+
 		mTriangleDivs1[i] = divs1;
 		mTriangleDivs2[i] = divs2;
 		mTriangleDivs3[i] = divs3;
@@ -454,15 +443,14 @@ void ntlGeometryObject::initMovingPoints(double time, gfxReal featureSize) {
 	if((mMovPntsInited==featureSize)&&(!getMeshAnimated())) return;
 	const bool debugMoinit=false;
 
-	//vector<ntlVec3Gfx> movNormals;
 	vector<ntlTriangle> triangles; 
 	vector<ntlVec3Gfx> vertices; 
 	vector<ntlVec3Gfx> vnormals; 
 	int objectId = 1;
 	this->getTriangles(time, &triangles,&vertices,&vnormals,objectId);
 	
-	mMovPoints.clear(); //= vertices;
-	//movNormals.clear(); //= vnormals;
+	mMovPoints.clear();
+	mMovNormals.clear();
 	if(debugMoinit) errMsg("ntlGeometryObject::initMovingPoints","Object "<<getName()<<" has v:"<<vertices.size()<<" t:"<<triangles.size() );
 	// no points?
 	if(vertices.size()<1) {
@@ -497,8 +485,8 @@ void ntlGeometryObject::initMovingPoints(double time, gfxReal featureSize) {
 			if(divs1+divs2 > 0) {
 				for(int u=0; u<=divs1; u++) {
 					for(int v=0; v<=divs2; v++) {
-						const gfxReal uf = (gfxReal)(u+0.25) / (gfxReal)(divs1+0.0);
-						const gfxReal vf = (gfxReal)(v+0.25) / (gfxReal)(divs2+0.0);
+						const gfxReal uf = (gfxReal)(u+TRI_UVOFFSET) / (gfxReal)(divs1+0.0);
+						const gfxReal vf = (gfxReal)(v+TRI_UVOFFSET) / (gfxReal)(divs2+0.0);
 						if(uf+vf>1.0) continue;
 						countp+=2;
 					}
@@ -523,8 +511,8 @@ void ntlGeometryObject::initMovingPoints(double time, gfxReal featureSize) {
 			if(dot(mInitialVelocity,n)<0.0) continue;
 		}
 		mMovPoints.push_back(p);
-		//movNormals.push_back(n);
-		//errMsg("ntlGeometryObject::initMovingPoints","std"<<i<<" p"<<p<<" n"<<n<<" ");
+		mMovNormals.push_back(n);
+		if(debugMoinit) errMsg("ntlGeometryObject::initMovingPoints","std"<<i<<" p"<<p<<" n"<<n<<" ");
 	}
 	// init points & refine...
 	for(size_t i=0; i<triangles.size(); i++) {
@@ -533,18 +521,18 @@ void ntlGeometryObject::initMovingPoints(double time, gfxReal featureSize) {
 		const ntlVec3Gfx side1 = vertices[ trips[1] ] - p0;
 		const ntlVec3Gfx side2 = vertices[ trips[2] ] - p0;
 		int divs1=mTriangleDivs1[i], divs2=mTriangleDivs2[i];
-		//if(normNoSqrt(side1) > fsTri*fsTri) { divs1 = (int)(norm(side1)/fsTri); }
-		//if(normNoSqrt(side2) > fsTri*fsTri) { divs2 = (int)(norm(side2)/fsTri); }
-		const ntlVec3Gfx trinorm = getNormalized(cross(side1,side2))*0.5*featureSize;
+		
+		const ntlVec3Gfx trinormOrg = getNormalized(cross(side1,side2));
+		const ntlVec3Gfx trinorm = trinormOrg*0.25*featureSize;
 		if(discardInflowBack) { 
 			if(dot(mInitialVelocity,trinorm)<0.0) continue;
 		}
-		//errMsg("ntlGeometryObject::initMovingPoints","Tri1 "<<vertices[trips[0]]<<","<<vertices[trips[1]]<<","<<vertices[trips[2]]<<" "<<divs1<<","<<divs2 );
+		if(debugMoinit) errMsg("ntlGeometryObject::initMovingPoints","Tri1 "<<vertices[trips[0]]<<","<<vertices[trips[1]]<<","<<vertices[trips[2]]<<" "<<divs1<<","<<divs2 );
 		if(divs1+divs2 > 0) {
 			for(int u=0; u<=divs1; u++) {
 				for(int v=0; v<=divs2; v++) {
-					const gfxReal uf = (gfxReal)(u+0.25) / (gfxReal)(divs1+0.0);
-					const gfxReal vf = (gfxReal)(v+0.25) / (gfxReal)(divs2+0.0);
+					const gfxReal uf = (gfxReal)(u+TRI_UVOFFSET) / (gfxReal)(divs1+0.0);
+					const gfxReal vf = (gfxReal)(v+TRI_UVOFFSET) / (gfxReal)(divs2+0.0);
 					if(uf+vf>1.0) continue;
 					ntlVec3Gfx p = 
 						vertices[ trips[0] ] * (1.0-uf-vf)+
@@ -557,25 +545,16 @@ void ntlGeometryObject::initMovingPoints(double time, gfxReal featureSize) {
 					//normalize(n);
 					// discard inflow backsides
 
-					mMovPoints.push_back(p);
+					mMovPoints.push_back(p + trinorm); // NEW!?
 					mMovPoints.push_back(p - trinorm);
-					//movNormals.push_back(n);
-					//movNormals.push_back(trinorm);
+					mMovNormals.push_back(trinormOrg);
+					mMovNormals.push_back(trinormOrg); 
 					//errMsg("TRINORM","p"<<p<<" n"<<n<<" trin"<<trinorm);
 				}
 			}
 		}
 	}
 
-	// duplicate insides
-	//size_t mpsize = mMovPoints.size();
-	//for(size_t i=0; i<mpsize; i++) {
-		//normalize(vnormals[i]);
-		//errMsg("TTAT"," moved:"<<(mMovPoints[i] - mMovPoints[i]*featureSize)<<" org"<<mMovPoints[i]<<" norm"<<mMovPoints[i]<<" fs"<<featureSize);
-		//mMovPoints.push_back(mMovPoints[i] - movNormals[i]*0.5*featureSize);
-		//movNormals.push_back(movNormals[i]);
-	//}
-
 	// find max point
 	mMaxMovPnt = 0;
 	gfxReal dist = normNoSqrt(mMovPoints[0]);
@@ -594,7 +573,7 @@ void ntlGeometryObject::initMovingPoints(double time, gfxReal featureSize) {
 		// also do trafo...
 	} else {
 		mCachedMovPoints = mMovPoints;
-		//mCachedMovNormals = movNormals;
+		mCachedMovNormals = mMovNormals;
 		//applyTransformation(time, &mCachedMovPoints, &mCachedMovNormals, 0, mCachedMovPoints.size(), true);
 		applyTransformation(time, &mCachedMovPoints, NULL, 0, mCachedMovPoints.size(), true);
 		mHaveCachedMov = true;
@@ -610,31 +589,27 @@ void ntlGeometryObject::initMovingPoints(double time, gfxReal featureSize) {
 void ntlGeometryObject::initMovingPointsAnim(
 		 double srctime, vector<ntlVec3Gfx> &srcmovPoints,
 		 double dsttime, vector<ntlVec3Gfx> &dstmovPoints,
+		 vector<ntlVec3Gfx> *dstmovNormals,
 		 gfxReal featureSize,
 		 ntlVec3Gfx geostart, ntlVec3Gfx geoend
 		 ) {
 	const bool debugMoinit=false;
 
-	//vector<ntlVec3Gfx> srcmovNormals;
-	//vector<ntlVec3Gfx> dstmovNormals;
-
 	vector<ntlTriangle> srctriangles; 
 	vector<ntlVec3Gfx> srcvertices; 
 	vector<ntlVec3Gfx> unused_normals; 
 	vector<ntlTriangle> dsttriangles; 
 	vector<ntlVec3Gfx> dstvertices; 
-	//vector<ntlVec3Gfx> dstnormals; 
+	vector<ntlVec3Gfx> dstnormals; 
 	int objectId = 1;
 	// TODO optimize? , get rid of normals?
 	unused_normals.clear();
 	this->getTriangles(srctime, &srctriangles,&srcvertices,&unused_normals,objectId);
 	unused_normals.clear();
-	this->getTriangles(dsttime, &dsttriangles,&dstvertices,&unused_normals,objectId);
+	this->getTriangles(dsttime, &dsttriangles,&dstvertices,&dstnormals,objectId);
 	
 	srcmovPoints.clear();
 	dstmovPoints.clear();
-	//srcmovNormals.clear();
-	//dstmovNormals.clear();
 	if(debugMoinit) errMsg("ntlGeometryObject::initMovingPointsAnim","Object "<<getName()<<" has srcv:"<<srcvertices.size()<<" srct:"<<srctriangles.size() );
 	if(debugMoinit) errMsg("ntlGeometryObject::initMovingPointsAnim","Object "<<getName()<<" has dstv:"<<dstvertices.size()<<" dstt:"<<dsttriangles.size() );
 	// no points?
@@ -668,7 +643,7 @@ void ntlGeometryObject::initMovingPointsAnim(
 	}
 	for(size_t i=0; i<dstvertices.size(); i++) {
 		dstmovPoints.push_back(dstvertices[i]);
-		//dstmovNormals.push_back(dstnormals[i]);
+		if(dstmovNormals) (*dstmovNormals).push_back(dstnormals[i]);
 	}
 	if(debugMoinit) errMsg("ntlGeometryObject::initMovingPointsAnim","stats src:"<<srcmovPoints.size()<<" dst:"<<dstmovPoints.size()<<" " );
 	// init points & refine...
@@ -684,13 +659,14 @@ void ntlGeometryObject::initMovingPointsAnim(
 			const ntlVec3Gfx dstp0 =    dstvertices[ dsttrips[0] ];
 			const ntlVec3Gfx dstside1 = dstvertices[ dsttrips[1] ] - dstp0;
 			const ntlVec3Gfx dstside2 = dstvertices[ dsttrips[2] ] - dstp0;
-			const ntlVec3Gfx src_trinorm = getNormalized(cross(srcside1,srcside2))*0.5*featureSize;
-			const ntlVec3Gfx dst_trinorm = getNormalized(cross(dstside1,dstside2))*0.5*featureSize;
+			const ntlVec3Gfx src_trinorm    = getNormalized(cross(srcside1,srcside2))*0.25*featureSize;
+			const ntlVec3Gfx dst_trinormOrg = getNormalized(cross(dstside1,dstside2));
+			const ntlVec3Gfx dst_trinorm    = dst_trinormOrg                         *0.25*featureSize;
 			//errMsg("ntlGeometryObject::initMovingPointsAnim","Tri1 "<<srcvertices[srctrips[0]]<<","<<srcvertices[srctrips[1]]<<","<<srcvertices[srctrips[2]]<<" "<<divs1<<","<<divs2 );
 			for(int u=0; u<=divs1; u++) {
 				for(int v=0; v<=divs2; v++) {
-					const gfxReal uf = (gfxReal)(u+0.25) / (gfxReal)(divs1+0.0);
-					const gfxReal vf = (gfxReal)(v+0.25) / (gfxReal)(divs2+0.0);
+					const gfxReal uf = (gfxReal)(u+TRI_UVOFFSET) / (gfxReal)(divs1+0.0);
+					const gfxReal vf = (gfxReal)(v+TRI_UVOFFSET) / (gfxReal)(divs2+0.0);
 					if(uf+vf>1.0) continue;
 					ntlVec3Gfx srcp = 
 						srcvertices[ srctrips[0] ] * (1.0-uf-vf)+
@@ -709,60 +685,19 @@ void ntlGeometryObject::initMovingPointsAnim(
 					if((srcp[1]>geoend[1]  ) && (dstp[1]>geoend[1]  )) continue;
 					if((srcp[2]>geoend[2]  ) && (dstp[2]>geoend[2]  )) continue;
 					
-					//ntlVec3Gfx srcn = 
-						//srcnormals[ srctriangles[i].getPoints()[0] ] * (1.0-uf-vf)+
-						//srcnormals[ srctriangles[i].getPoints()[1] ] * uf +
-						//srcnormals[ srctriangles[i].getPoints()[2] ] * vf;
-					//normalize(srcn);
-					srcmovPoints.push_back(srcp);
+					srcmovPoints.push_back(srcp+src_trinorm); // SURFENHTEST
 					srcmovPoints.push_back(srcp-src_trinorm);
-					//srcmovNormals.push_back(srcn);
 
-					//ntlVec3Gfx dstn = 
-						//dstnormals[ dsttriangles[i].getPoints()[0] ] * (1.0-uf-vf)+
-						//dstnormals[ dsttriangles[i].getPoints()[1] ] * uf +
-						//dstnormals[ dsttriangles[i].getPoints()[2] ] * vf;
-					//normalize(dstn);
-					dstmovPoints.push_back(dstp);
+					dstmovPoints.push_back(dstp+dst_trinorm); // SURFENHTEST
 					dstmovPoints.push_back(dstp-dst_trinorm);
-					//dstmovNormals.push_back(dstn);
-
-					/*if(debugMoinit && (i>=0)) errMsg("ntlGeometryObject::initMovingPointsAnim"," "<<
-					//srcmovPoints[ srcmovPoints.size()-1 ]<<","<<
-					//srcmovNormals[ srcmovNormals.size()-1 ]<<"   "<<
-					//dstmovPoints[ dstmovPoints.size()-1 ]<<","<<
-					//dstmovNormals[ dstmovNormals.size()-1 ]<<" " 
-					(srcmovNormals[ srcmovPoints.size()-1 ]-
-					dstmovNormals[ dstmovPoints.size()-1 ])<<" "
-					);
-					// */
+					if(dstmovNormals) {
+						(*dstmovNormals).push_back(dst_trinormOrg);
+						(*dstmovNormals).push_back(dst_trinormOrg); }
 				}
 			}
 		}
 	}
 
-	/*if(debugMoinit) errMsg("ntlGeometryObject::initMovingPointsAnim","stats src:"<<srcmovPoints.size()<<","<<srcmovNormals.size()<<" dst:"<<dstmovPoints.size()<<","<<dstmovNormals.size() );
-	// duplicate insides
-	size_t mpsize = srcmovPoints.size();
-	for(size_t i=0; i<mpsize; i++) {
-		srcmovPoints.push_back(srcmovPoints[i] - srcmovNormals[i]*1.0*featureSize);
-		//? srcnormals.push_back(srcnormals[i]);
-	}
-	mpsize = dstmovPoints.size();
-	for(size_t i=0; i<mpsize; i++) {
-		dstmovPoints.push_back(dstmovPoints[i] - dstmovNormals[i]*1.0*featureSize);
-		//? dstnormals.push_back(dstnormals[i]);
-	}
-	// */
-
-	/*if(debugMoinit) errMsg("ntlGeometryObject::initMovingPointsAnim","stats src:"<<srcmovPoints.size()<<","<<srcmovNormals.size()<<" dst:"<<dstmovPoints.size()<<","<<dstmovNormals.size() );
-	for(size_t i=0; i<srcmovPoints.size(); i++) {
-		ntlVec3Gfx p1 = srcmovPoints[i];
-		ntlVec3Gfx p2 = dstmovPoints[i];
-	  gfxReal len = norm(p1-p2);
-		if(len>0.01) { errMsg("ntlGeometryObject::initMovingPointsAnim"," i"<<i<<" "<< p1<<" "<<p2<<" "<<len); }
-	} // */
-
 	// find max point not necessary
 	debMsgStd("ntlGeometryObject::initMovingPointsAnim",DM_MSG,"Object "<<getName()<<" inited v:"<<srcvertices.size()<<"->"<<srcmovPoints.size()<<","<<dstmovPoints.size() , 5);
 }
@@ -772,8 +707,8 @@ void ntlGeometryObject::getMovingPoints(vector<ntlVec3Gfx> &ret, vector<ntlVec3G
 	if(mHaveCachedMov) {
 		ret = mCachedMovPoints;
 		if(norms) { 
-			//*norms = mCachedMovNormals; 
-			errMsg("ntlGeometryObject","getMovingPoints - Normals currently unused!");
+			*norms = mCachedMovNormals; 
+			//errMsg("ntlGeometryObject","getMovingPoints - Normals currently unused!");
 		}
 		//errMsg ("ntlGeometryObject::getMovingPoints","Object "<<getName()<<" used cached points "); // DEBUG
 		return;
@@ -781,8 +716,8 @@ void ntlGeometryObject::getMovingPoints(vector<ntlVec3Gfx> &ret, vector<ntlVec3G
 
 	ret = mMovPoints;
 	if(norms) { 
-		errMsg("ntlGeometryObject","getMovingPoints - Normals currently unused!");
-		//*norms = mMovNormals; 
+		//errMsg("ntlGeometryObject","getMovingPoints - Normals currently unused!");
+		*norms = mMovNormals; 
 	}
 }
 
@@ -799,7 +734,7 @@ ntlVec3Gfx ntlGeometryObject::calculateMaxVel(double t1, double t2) {
 	applyTransformation(t2,&verts2,NULL, 0,verts2.size(), true);
 
 	vel = (verts2[0]-verts1[0]); // /(t2-t1);
-	errMsg("ntlGeometryObject::calculateMaxVel","t1="<<t1<<" t2="<<t2<<" p1="<<verts1[0]<<" p2="<<verts2[0]<<" v="<<vel);
+	//errMsg("ntlGeometryObject::calculateMaxVel","t1="<<t1<<" t2="<<t2<<" p1="<<verts1[0]<<" p2="<<verts2[0]<<" v="<<vel);
 	return vel;
 }
 
@@ -835,7 +770,6 @@ ntlVec3Gfx ntlGeometryObject::getTranslation(double t) {
 }
 /*! get active flag time t*/
 float ntlGeometryObject::getGeoActive(double t) {
-	//float act = mcGeoActive.getConstant(t);
 	float act = mcGeoActive.get(t); // if <= 0.0 -> off
 	return act;
 }
