@@ -1,14 +1,11 @@
 /**
- * $Id: MEM_guardedalloc.h 9080 2006-11-27 13:59:55Z ton $
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * $Id: MEM_guardedalloc.h 14444 2008-04-16 22:40:48Z hos $
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,12 +23,12 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 /**
 
- * $Id: MEM_guardedalloc.h 9080 2006-11-27 13:59:55Z ton $
+ * $Id: MEM_guardedalloc.h 14444 2008-04-16 22:40:48Z hos $
  * Copyright (C) 2001 NaN Technologies B.V.
  * Guarded memory (de)allocation
  *
@@ -99,10 +96,17 @@ extern "C" {
 		* */
 	void *MEM_mapallocN(unsigned int len, const char * str);
 
+	/** Print a list of the names and sizes of all allocated memory
+	 * blocks. as a python dict for easy investigation */ 
+	void MEM_printmemlist_pydict(void);
+
 	/** Print a list of the names and sizes of all allocated memory
 	 * blocks. */ 
 	void MEM_printmemlist(void);
 
+	/** Print statistics about memory usage */
+	void MEM_printmemlist_stats(void);
+	
 	/** Set the callback function for error output. */
 	void MEM_set_error_callback(void (*func)(char *));
 
