@@ -1,14 +1,11 @@
 /**
- * $Id: IK_QSegment.h 5229 2005-08-31 22:09:44Z blendix $
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * $Id: IK_QSegment.h 14444 2008-04-16 22:40:48Z hos $
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  * Original author: Laurence
  * Contributor(s): Brecht
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifndef NAN_INCLUDED_IK_QSegment_h
@@ -165,6 +162,13 @@ public:
 
 	virtual void SetBasis(const MT_Matrix3x3& basis) { m_basis = basis; }
 
+	// functions needed for pole vector constraint
+	void PrependBasis(const MT_Matrix3x3& mat);
+	void Reset();
+
+	// scale
+	virtual void Scale(float scale);
+
 protected:
 
 	// num_DoF: number of degrees of freedom
@@ -331,6 +335,8 @@ public:
 	void SetWeight(int axis, MT_Scalar weight);
 	void SetLimit(int axis, MT_Scalar lmin, MT_Scalar lmax);
 
+	void Scale(float scale);
+
 private:
 	int m_axis[3];
 	bool m_axis_enabled[3], m_limit[3];
