@@ -4,17 +4,14 @@
  * This is external code. Sets some compression related options
  * (width, height quality, framerate).
  *
- * $Id: options.c 125 2002-11-25 12:02:15Z mein $ 
+ * $Id: options.c 14444 2008-04-16 22:40:48Z hos $ 
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,7 +29,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  *  */
 
 #include "AVI_avi.h"
@@ -99,8 +96,8 @@ AviError AVI_set_compress_option (AviMovie *movie, int option_type, int stream,
 				break;
 				
 			case AVI_OPTION_FRAMERATE:
-				if (1000000/(*((int *) opt_data)))
-					movie->header->MicroSecPerFrame = 1000000/(*((int *) opt_data));					
+				if (1000000/(*((double *) opt_data)))
+					movie->header->MicroSecPerFrame = 1000000/(*((double *) opt_data));					
 
 				for (i=0; i < movie->header->Streams; i++) {
 					if (avi_get_format_type(movie->streams[i].format) == FCC("vids")) {
