@@ -1,15 +1,12 @@
 /**
- * $Id: BKE_DerivedMesh.h 9302 2006-12-12 21:29:09Z blendix $
+ * $Id: BKE_DerivedMesh.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifndef BKE_DERIVEDMESH_H
@@ -57,6 +54,7 @@ struct Mesh;
 struct EditMesh;
 struct ModifierData;
 struct MCol;
+struct ColorBand;
 
 /* number of sub-elements each mesh element has (for interpolation) */
 #define SUB_ELEMS_VERT 0
@@ -69,6 +67,7 @@ struct DerivedMesh {
 	CustomData vertData, edgeData, faceData;
 	int numVertData, numEdgeData, numFaceData;
 	int needsFree; /* checked on ->release, is set to 0 for cached results */
+	int deformedOnly; /* set by modifier stack if only deformed from original */
 
 	/* Misc. Queries */
 
@@ -395,6 +394,9 @@ void DM_interp_face_data(struct DerivedMesh *source, struct DerivedMesh *dest,
 
 void DM_swap_face_data(struct DerivedMesh *dm, int index, int *corner_indices);
 
+/* Temporary? A function to give a colorband to derivedmesh for vertexcolor ranges */
+void vDM_ColorBand_store(struct ColorBand *coba);
+
 /* Simple function to get me->totvert amount of vertices/normals,
    correctly deformed and subsurfered. Needed especially when vertexgroups are involved.
    In use now by vertex/weigt paint and particles */
@@ -425,6 +427,11 @@ DerivedMesh *editmesh_get_derived_cage(CustomDataMask dataMask);
 DerivedMesh *editmesh_get_derived_cage_and_final(DerivedMesh **final_r,
                                                  CustomDataMask dataMask);
 
+/* returns an array of deform matrices for crazyspace correction, and the
+   number of modifiers left */
+int editmesh_get_first_deform_matrices(float (**deformmats)[3][3],
+                                       float (**deformcos)[3]);
+
 void weight_to_rgb(float input, float *fr, float *fg, float *fb);
 
 /* determines required DerivedMesh data according to view and edit modes */
