@@ -4,17 +4,14 @@
  *
  *	Reevan McKay
  *
- * $Id: BKE_action.h 8825 2006-11-11 16:45:17Z ton $
+ * $Id: BKE_action.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,7 +27,7 @@
  *
  * Contributor(s): Full recode, Ton Roosendaal, Crete 2005
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifndef BKE_ACTION_H
@@ -60,6 +57,11 @@ extern "C" {
  */
 void free_pose_channels(struct bPose *pose);
 
+/** 
+ * Removes and deallocates all data from a pose, and also frees the pose.
+ */
+void free_pose(struct bPose *pose);
+
 /**
  * Allocate a new pose on the heap, and copy the src pose and it's channels
  * into the new pose. *dst is set to the newly allocated structure, and assumed to be NULL.
@@ -101,6 +103,9 @@ struct bPoseChannel *verify_pose_channel(struct bPose* pose,
 /* sets constraint flags */
 void update_pose_constraint_flags(struct bPose *pose);
 
+/* clears BONE_UNKEYED flags for frame changing */
+void framechange_poses_clear_unkeyed(void);
+
 /**
  * Allocate a new bAction on the heap and copy 
  * the contents of src into it. If src is NULL NULL is returned.
@@ -116,8 +121,12 @@ void calc_action_range(const struct bAction *act, float *start, float *end, int
 /**
  * Set the pose channels from the given action.
  */
-void extract_pose_from_action(struct bPose *pose, struct bAction *act,
-                                                  float ctime);
+void extract_pose_from_action(struct bPose *pose, struct bAction *act, float ctime);
+
+/**
+ * Get the effects of the given action using a workob 
+ */
+void what_does_obaction(struct Object *ob, struct bAction *act, float cframe);
 
 /**
  * Iterate through the action channels of the action
@@ -153,11 +162,5 @@ float get_action_frame_inv(struct Object *ob, float cframe);
 };
 #endif
 
-/* nla strip->mode, for action blending */
-enum	{
-			POSE_BLEND		= 0,
-			POSE_ADD
-};
-
 #endif
 
