@@ -1,4 +1,4 @@
-/* $Id: CCGSubSurf.c 11032 2007-06-23 15:02:49Z campbellbarton $ */
+/* $Id: CCGSubSurf.c 14193 2008-03-21 03:08:04Z nicholasbishop $ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -1293,7 +1293,7 @@ static void ccgSubSurf__sync(CCGSubSurf *ss) {
 			VertDataMulN(nCo, 1.0f/numEdges);
 		}
 
-		if ((sharpCount>1 && v->numFaces) || seam) {
+		if (sharpCount>1 || seam) {
 			VertDataZero(q);
 
 			if (seam) {
