@@ -1,5 +1,5 @@
 /*
-* $Id: customdata.c 10633 2007-05-02 00:01:23Z campbellbarton $
+* $Id: customdata.c 14853 2008-05-15 19:40:09Z blendix $
 *
 * ***** BEGIN GPL LICENSE BLOCK *****
 *
@@ -97,11 +97,16 @@ static void layerCopy_mdeformvert(const void *source, void *dest,
 
 	for(i = 0; i < count; ++i) {
 		MDeformVert *dvert = (MDeformVert *)((char *)dest + i * size);
-		MDeformWeight *dw = MEM_callocN(dvert->totweight * sizeof(*dw),
-		                                "layerCopy_mdeformvert dw");
 
-		memcpy(dw, dvert->dw, dvert->totweight * sizeof(*dw));
-		dvert->dw = dw;
+		if(dvert->totweight) {
+			MDeformWeight *dw = MEM_callocN(dvert->totweight * sizeof(*dw),
+											"layerCopy_mdeformvert dw");
+
+			memcpy(dw, dvert->dw, dvert->totweight * sizeof(*dw));
+			dvert->dw = dw;
+		}
+		else
+			dvert->dw = NULL;
 	}
 }
 
@@ -271,7 +276,7 @@ static void layerSwap_tface(void *data, int *corner_indices)
 
 static void layerDefault_tface(void *data, int count)
 {
-	static MTFace default_tf = {{{0, 1}, {0, 0}, {1, 0}, {1, 1}}, NULL,
+	static MTFace default_tf = {{{0, 0}, {1, 0}, {1, 1}, {0, 1}}, NULL,
 	                           0, 0, TF_DYNAMIC, 0, 0};
 	MTFace *tf = (MTFace*)data;
 	int i;
@@ -280,6 +285,83 @@ static void layerDefault_tface(void *data, int count)
 		tf[i] = default_tf;
 }
 
+static void layerCopy_origspace_face(const void *source, void *dest, int count)
+{
+	const OrigSpaceFace *source_tf = (const OrigSpaceFace*)source;
+	OrigSpaceFace *dest_tf = (OrigSpaceFace*)dest;
+	int i;
+
+	for(i = 0; i < count; ++i)
+		dest_tf[i] = source_tf[i];
+}
+
+static void layerInterp_origspace_face(void **sources, float *weights,
+							  float *sub_weights, int count, void *dest)
+{
+	OrigSpaceFace *osf = dest;
+	int i, j, k;
+	float uv[4][2];
+	float *sub_weight;
+
+	if(count <= 0) return;
+
+	memset(uv, 0, sizeof(uv));
+
+	sub_weight = sub_weights;
+	for(i = 0; i < count; ++i) {
+		float weight = weights ? weights[i] : 1;
+		OrigSpaceFace *src = sources[i];
+
+		for(j = 0; j < 4; ++j) {
+			if(sub_weights) {
+				for(k = 0; k < 4; ++k, ++sub_weight) {
+					float w = (*sub_weight) * weight;
+					float *tmp_uv = src->uv[k];
+
+					uv[j][0] += tmp_uv[0] * w;
+					uv[j][1] += tmp_uv[1] * w;
+				}
+			} else {
+				uv[j][0] += src->uv[j][0] * weight;
+				uv[j][1] += src->uv[j][1] * weight;
+			}
+		}
+	}
+
+	*osf = *(OrigSpaceFace *)sources[0];
+	for(j = 0; j < 4; ++j) {
+		osf->uv[j][0] = uv[j][0];
+		osf->uv[j][1] = uv[j][1];
+	}
+}
+
+static void layerSwap_origspace_face(void *data, int *corner_indices)
+{
+	OrigSpaceFace *osf = data;
+	float uv[4][2];
+	int j;
+
+	for(j = 0; j < 4; ++j) {
+		uv[j][0] = osf->uv[corner_indices[j]][0];
+		uv[j][1] = osf->uv[corner_indices[j]][1];
+	}
+	memcpy(osf->uv, uv, sizeof(osf->uv));
+}
+
+static void layerDefault_origspace_face(void *data, int count)
+{
+	static OrigSpaceFace default_osf = {{{0, 0}, {1, 0}, {1, 1}, {0, 1}}};
+	OrigSpaceFace *osf = (OrigSpaceFace*)data;
+	int i;
+
+	for(i = 0; i < count; i++)
+		osf[i] = default_osf;
+}
+/* --------- */
+
+
+
+
 static void layerInterp_mcol(void **sources, float *weights,
                              float *sub_weights, int count, void *dest)
 {
@@ -367,23 +449,32 @@ const LayerTypeInfo LAYERTYPEINFO[CD_NUMTYPES] = {
 	/* 3 floats per normal vector */
 	{sizeof(float)*3, "", 0, NULL, NULL, NULL, NULL, NULL, NULL},
 	{sizeof(int), "", 0, NULL, NULL, NULL, NULL, NULL, NULL},
+	{sizeof(MFloatProperty), "MFloatProperty",1,"Float",NULL,NULL,NULL,NULL},
+	{sizeof(MIntProperty), "MIntProperty",1,"Int",NULL,NULL,NULL,NULL},
+	{sizeof(MStringProperty), "MStringProperty",1,"String",NULL,NULL,NULL,NULL},
+	{sizeof(OrigSpaceFace), "OrigSpaceFace", 1, "UVTex", layerCopy_origspace_face, NULL,
+	 layerInterp_origspace_face, layerSwap_origspace_face, layerDefault_origspace_face},
+	{sizeof(float)*3, "", 0, NULL, NULL, NULL, NULL, NULL, NULL}
 };
 
 const char *LAYERTYPENAMES[CD_NUMTYPES] = {
 	"CDMVert", "CDMSticky", "CDMDeformVert", "CDMEdge", "CDMFace", "CDMTFace",
-	"CDMCol", "CDOrigIndex", "CDNormal", "CDFlags"};
+	"CDMCol", "CDOrigIndex", "CDNormal", "CDFlags","CDMFloatProperty",
+	"CDMIntProperty","CDMStringProperty", "CDOrigSpace", "CDOrco"};
 
 const CustomDataMask CD_MASK_BAREMESH =
 	CD_MASK_MVERT | CD_MASK_MEDGE | CD_MASK_MFACE;
 const CustomDataMask CD_MASK_MESH =
 	CD_MASK_MVERT | CD_MASK_MEDGE | CD_MASK_MFACE |
-	CD_MASK_MSTICKY | CD_MASK_MDEFORMVERT | CD_MASK_MTFACE | CD_MASK_MCOL;
+	CD_MASK_MSTICKY | CD_MASK_MDEFORMVERT | CD_MASK_MTFACE | CD_MASK_MCOL |
+	CD_MASK_PROP_FLT | CD_MASK_PROP_INT | CD_MASK_PROP_STR;
 const CustomDataMask CD_MASK_EDITMESH =
 	CD_MASK_MSTICKY | CD_MASK_MDEFORMVERT | CD_MASK_MTFACE |
-	CD_MASK_MCOL;
+	CD_MASK_MCOL|CD_MASK_PROP_FLT | CD_MASK_PROP_INT | CD_MASK_PROP_STR;
 const CustomDataMask CD_MASK_DERIVEDMESH =
 	CD_MASK_MSTICKY | CD_MASK_MDEFORMVERT | CD_MASK_MTFACE |
-	CD_MASK_MCOL | CD_MASK_ORIGINDEX;
+	CD_MASK_MCOL | CD_MASK_ORIGINDEX | CD_MASK_PROP_FLT | CD_MASK_PROP_INT |
+	CD_MASK_PROP_STR | CD_MASK_ORIGSPACE | CD_MASK_ORCO;
 
 static const LayerTypeInfo *layerType_getInfo(int type)
 {
@@ -540,6 +631,29 @@ int CustomData_get_render_layer_index(const CustomData *data, int type)
 	return -1;
 }
 
+int CustomData_get_active_layer(const CustomData *data, int type)
+{
+	int i;
+
+	for(i=0; i < data->totlayer; ++i)
+		if(data->layers[i].type == type)
+			return data->layers[i].active;
+
+	return -1;
+}
+
+int CustomData_get_render_layer(const CustomData *data, int type)
+{
+	int i;
+
+	for(i=0; i < data->totlayer; ++i)
+		if(data->layers[i].type == type)
+			return data->layers[i].active_rnd;
+
+	return -1;
+}
+
+
 void CustomData_set_layer_active(CustomData *data, int type, int n)
 {
 	int i;
@@ -558,6 +672,25 @@ void CustomData_set_layer_render(CustomData *data, int type, int n)
 			data->layers[i].active_rnd = n;
 }
 
+/* for using with an index from CustomData_get_active_layer_index and CustomData_get_render_layer_index */
+void CustomData_set_layer_active_index(CustomData *data, int type, int n)
+{
+	int i;
+
+	for(i=0; i < data->totlayer; ++i)
+		if(data->layers[i].type == type)
+			data->layers[i].active = n-i;
+}
+
+void CustomData_set_layer_render_index(CustomData *data, int type, int n)
+{
+	int i;
+
+	for(i=0; i < data->totlayer; ++i)
+		if(data->layers[i].type == type)
+			data->layers[i].active_rnd = n-i;
+}
+
 
 void CustomData_set_layer_flag(struct CustomData *data, int type, int flag)
 {
@@ -1108,7 +1241,8 @@ void CustomData_em_copy_data(const CustomData *source, CustomData *dest,
 		if(dest_i >= dest->totlayer) return;
 
 		/* if we found a matching layer, copy the data */
-		if(dest->layers[dest_i].type == source->layers[src_i].type) {
+		if(dest->layers[dest_i].type == source->layers[src_i].type &&
+			strcmp(dest->layers[dest_i].name, source->layers[src_i].name) == 0) {
 			char *src_data = (char*)src_block + source->layers[src_i].offset;
 			char *dest_data = (char*)*dest_block + dest->layers[dest_i].offset;
 
@@ -1335,6 +1469,13 @@ const char *CustomData_layertype_name(int type)
 	return layerType_getName(type);
 }
 
+static int  CustomData_is_property_layer(int type)
+{
+	if((type == CD_PROP_FLT) || (type == CD_PROP_INT) || (type == CD_PROP_STR))
+		return 1;
+	return 0;
+}
+
 void CustomData_set_layer_unique_name(CustomData *data, int index)
 {
 	char tempname[64];
@@ -1355,9 +1496,17 @@ void CustomData_set_layer_unique_name(CustomData *data, int index)
 	/* see if there is a duplicate */
 	for(i=0; i<data->totlayer; i++) {
 		layer = &data->layers[i];
-
-		if(i!=index && layer->type==type && strcmp(layer->name, name)==0)
-			break;
+		
+		if(CustomData_is_property_layer(type)){
+			if(i!=index && CustomData_is_property_layer(layer->type) && 
+				strcmp(layer->name, name)==0)
+					break;	
+		
+		}
+		else{
+			if(i!=index && layer->type==type && strcmp(layer->name, name)==0)
+				break;
+		}
 	}
 
 	if(i == data->totlayer)
@@ -1373,8 +1522,16 @@ void CustomData_set_layer_unique_name(CustomData *data, int index)
 		for(i=0; i<data->totlayer; i++) {
 			layer = &data->layers[i];
 			
-			if(i!=index && layer->type==type && strcmp(layer->name, tempname)==0)
+			if(CustomData_is_property_layer(type)){
+				if(i!=index && CustomData_is_property_layer(layer->type) && 
+					strcmp(layer->name, tempname)==0)
+
 				break;
+			}
+			else{
+				if(i!=index && layer->type==type && strcmp(layer->name, tempname)==0)
+					break;
+			}
 		}
 
 		if(i == data->totlayer) {
