@@ -4,17 +4,14 @@
  * 
  *	Reevan McKay
  *
- * $Id: deform.c,v 1.33 2007/04/27 11:16:34 aligorith Exp $
+ * $Id: deform.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,7 +29,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <string.h>
@@ -113,14 +110,31 @@ bDeformGroup *get_named_vertexgroup (Object *ob, char *name)
 	return NULL;
 }
 
-int get_defgroup_num (Object *ob, bDeformGroup	*dg)
+int get_named_vertexgroup_num (Object *ob, char *name)
+{
+	/* Return the location of the named deform group within the list of
+	 * deform groups. This function is a combination of get_defgroup_num and
+	 * get_named_vertexgroup. The other two could be called instead, but that
+	 * require looping over the vertexgroups twice.
+	 */
+	bDeformGroup *curdef;
+	int def_nr;
+	
+	for (curdef=ob->defbase.first, def_nr=0; curdef; curdef=curdef->next, def_nr++) {
+		if (!strcmp(curdef->name, name))
+			return def_nr;
+	}
+	
+	return -1;
+}
+
+int get_defgroup_num (Object *ob, bDeformGroup *dg)
 {
 	/* Fetch the location of this deform group
 	 * within the linked list of deform groups.
 	 * (this number is stored in the deform
 	 * weights of the deform verts to link them
-	 * to this deform group) deform deform
-	 * deform blah blah deform
+	 * to this deform group).
 	 */
 
 	bDeformGroup *eg;
@@ -129,8 +143,7 @@ int get_defgroup_num (Object *ob, bDeformGroup	*dg)
 	eg = ob->defbase.first;
 	def_nr = 0;
 
-	/* loop through all deform groups
-	 */
+	/* loop through all deform groups */
 	while (eg != NULL) {
 
 		/* if the current deform group is
