@@ -1,15 +1,12 @@
 /**
- * $Id: depsgraph.c,v 1.63 2007/04/12 17:22:08 ton Exp $
+ * $Id: depsgraph.c 14764 2008-05-09 13:04:36Z blendix $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +22,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
  
 #include <stdio.h>
@@ -44,10 +41,12 @@
 #include "DNA_action_types.h"
 #include "DNA_armature_types.h"
 #include "DNA_curve_types.h"
+#include "DNA_camera_types.h"
 #include "DNA_ID.h"
 #include "DNA_effect_types.h"
 #include "DNA_group_types.h"
 #include "DNA_lattice_types.h"
+#include "DNA_lamp_types.h"
 #include "DNA_key_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_modifier_types.h"
@@ -56,6 +55,7 @@
 #include "DNA_object_force.h"
 #include "DNA_object_fluidsim.h"
 #include "DNA_oops_types.h"
+#include "DNA_particle_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_space_types.h"
@@ -71,8 +71,10 @@
 #include "BKE_mball.h"
 #include "BKE_modifier.h"
 #include "BKE_object.h"
-#include "BKE_scene.h"
+#include "BKE_particle.h"
+#include "BKE_pointcache.h"
 #include "BKE_utildefines.h"
+#include "BKE_scene.h"
 
 #include "MEM_guardedalloc.h"
 #include "blendef.h"
@@ -323,9 +325,9 @@ static void dag_add_driver_relation(Ipo *ipo, DagForest *dag, DagNode *node, int
 							ob = *oba;
 							node1 = dag_get_node(dag, ob);
 							if (ob->type == OB_ARMATURE)
-								dag_add_relation(dag, node1, node, isdata?DAG_RL_DATA_DATA:DAG_RL_DATA_OB);
+								dag_add_relation(dag, node1, node, isdata?DAG_RL_DATA_DATA:DAG_RL_DATA_OB, "Python Ipo Driver");
 							else
-								dag_add_relation(dag, node1, node, isdata?DAG_RL_OB_DATA:DAG_RL_OB_OB);
+								dag_add_relation(dag, node1, node, isdata?DAG_RL_OB_DATA:DAG_RL_OB_OB, "Python Ipo Driver");
 							oba++;
 						}
 
@@ -336,9 +338,27 @@ static void dag_add_driver_relation(Ipo *ipo, DagForest *dag, DagNode *node, int
 			else if (icu->driver->ob) {
 				node1 = dag_get_node(dag, icu->driver->ob);
 				if(icu->driver->blocktype==ID_AR)
-					dag_add_relation(dag, node1, node, isdata?DAG_RL_DATA_DATA:DAG_RL_DATA_OB);
+					dag_add_relation(dag, node1, node, isdata?DAG_RL_DATA_DATA:DAG_RL_DATA_OB, "Ipo Driver");
 				else
-					dag_add_relation(dag, node1, node, isdata?DAG_RL_OB_DATA:DAG_RL_OB_OB);
+					dag_add_relation(dag, node1, node, isdata?DAG_RL_OB_DATA:DAG_RL_OB_OB, "Ipo Driver");
+			}
+		}
+	}
+}
+
+static void dag_add_collision_field_relation(DagForest *dag, Object *ob, DagNode *node)
+{
+	Base *base;
+	DagNode *node2;
+
+	// would be nice to have a list of colliders here
+	// so for now walk all objects in scene check 'same layer rule'
+	for(base = G.scene->base.first; base; base= base->next) {
+		if((base->lay & ob->lay) && base->object->pd) {
+			Object *ob1= base->object;
+			if((ob1->pd->deflect || ob1->pd->forcefield) && (ob1 != ob))  {
+				node2 = dag_get_node(dag, ob1);					
+				dag_add_relation(dag, node2, node, DAG_RL_DATA_DATA|DAG_RL_OB_DATA, "Field Collision");
 			}
 		}
 	}
@@ -347,17 +367,19 @@ static void dag_add_driver_relation(Ipo *ipo, DagForest *dag, DagNode *node, int
 static void build_dag_object(DagForest *dag, DagNode *scenenode, Object *ob, int mask)
 {
 	bConstraint *con;
+	bConstraintChannel *conchan;
 	DagNode * node;
 	DagNode * node2;
 	DagNode * node3;
 	Key *key;
+	ParticleSystem *psys;
 	int addtoroot= 1;
 	
 	node = dag_get_node(dag, ob);
 	
 	if ((ob->data) && (mask&DAG_RL_DATA)) {
 		node2 = dag_get_node(dag,ob->data);
-		dag_add_relation(dag,node,node2,DAG_RL_DATA);
+		dag_add_relation(dag,node,node2,DAG_RL_DATA, "Object-Data Relation");
 		node2->first_ancestor = ob;
 		node2->ancestor_count += 1;
 	}
@@ -366,25 +388,34 @@ static void build_dag_object(DagForest *dag, DagNode *scenenode, Object *ob, int
 		if (ob->pose){
 			bPoseChannel *pchan;
 			bConstraint *con;
-			Object * target;
-			char *subtarget;
 			
-			for (pchan = ob->pose->chanbase.first; pchan; pchan=pchan->next){
-				for (con = pchan->constraints.first; con; con=con->next){
-					if (constraint_has_target(con)) {
+			for (pchan = ob->pose->chanbase.first; pchan; pchan=pchan->next) {
+				for (con = pchan->constraints.first; con; con=con->next) {
+					bConstraintTypeInfo *cti= constraint_get_typeinfo(con);
+					ListBase targets = {NULL, NULL};
+					bConstraintTarget *ct;
+					
+					if (cti && cti->get_constraint_targets) {
+						cti->get_constraint_targets(con, &targets);
 						
-						target = get_constraint_target(con, &subtarget);
-						if (target!=ob) {
-							// fprintf(stderr,"armature %s target :%s \n", ob->id.name, target->id.name);
-							node3 = dag_get_node(dag, target);
-							
-							if(subtarget && subtarget[0])
-								dag_add_relation(dag,node3,node, DAG_RL_OB_DATA|DAG_RL_DATA_DATA);
-							else
-								dag_add_relation(dag,node3,node, DAG_RL_OB_DATA);
-							
+						for (ct= targets.first; ct; ct= ct->next) {
+							if (ct->tar && ct->tar != ob) {
+								// fprintf(stderr,"armature %s target :%s \n", ob->id.name, target->id.name);
+								node3 = dag_get_node(dag, ct->tar);
+								
+								if (ct->subtarget[0])
+									dag_add_relation(dag,node3,node, DAG_RL_OB_DATA|DAG_RL_DATA_DATA, cti->name);
+								else if(ELEM(con->type, CONSTRAINT_TYPE_FOLLOWPATH, CONSTRAINT_TYPE_CLAMPTO)) 	
+									dag_add_relation(dag,node3,node, DAG_RL_DATA_DATA|DAG_RL_OB_DATA, cti->name);
+								else
+									dag_add_relation(dag,node3,node, DAG_RL_OB_DATA, cti->name);
+							}
 						}
+						
+						if (cti->flush_constraint_targets)
+							cti->flush_constraint_targets(con, &targets, 1);
 					}
+					
 				}
 			}
 		}
@@ -398,9 +429,12 @@ static void build_dag_object(DagForest *dag, DagNode *scenenode, Object *ob, int
 	if(key && key->ipo)
 		dag_add_driver_relation(key->ipo, dag, node, 1);
 	
+	for (conchan=ob->constraintChannels.first; conchan; conchan=conchan->next)
+		if(conchan->ipo)
+			dag_add_driver_relation(conchan->ipo, dag, node, 0);
+
 	if(ob->action) {
 		bActionChannel *chan;
-		bConstraintChannel *conchan;
 		for (chan = ob->action->chanbase.first; chan; chan=chan->next){
 			if(chan->ipo)
 				dag_add_driver_relation(chan->ipo, dag, node, 1);
@@ -422,7 +456,7 @@ static void build_dag_object(DagForest *dag, DagNode *scenenode, Object *ob, int
 				for(amod= strip->modifiers.first; amod; amod= amod->next) {
 					if(amod->ob) {
 						node2 = dag_get_node(dag, amod->ob);
-						dag_add_relation(dag, node2, node, DAG_RL_DATA_DATA|DAG_RL_OB_DATA);
+						dag_add_relation(dag, node2, node, DAG_RL_DATA_DATA|DAG_RL_OB_DATA, "NLA Strip Modifier");
 					}
 				}
 			}
@@ -442,42 +476,57 @@ static void build_dag_object(DagForest *dag, DagNode *scenenode, Object *ob, int
 		
 		switch(ob->partype) {
 			case PARSKEL:
-				dag_add_relation(dag,node2,node,DAG_RL_DATA_DATA|DAG_RL_OB_OB);
+				dag_add_relation(dag,node2,node,DAG_RL_DATA_DATA|DAG_RL_OB_OB, "Parent");
 				break;
 			case PARVERT1: case PARVERT3: case PARBONE:
-				dag_add_relation(dag,node2,node,DAG_RL_DATA_OB|DAG_RL_OB_OB);
+				dag_add_relation(dag,node2,node,DAG_RL_DATA_OB|DAG_RL_OB_OB, "Vertex Parent");
 				break;
 			default:
 				if(ob->parent->type==OB_LATTICE) 
-					dag_add_relation(dag,node2,node,DAG_RL_DATA_DATA|DAG_RL_OB_OB);
+					dag_add_relation(dag,node2,node,DAG_RL_DATA_DATA|DAG_RL_OB_OB, "Lattice Parent");
 				else if(ob->parent->type==OB_CURVE) {
 					Curve *cu= ob->parent->data;
 					if(cu->flag & CU_PATH) 
-						dag_add_relation(dag,node2,node,DAG_RL_DATA_OB|DAG_RL_OB_OB);
+						dag_add_relation(dag,node2,node,DAG_RL_DATA_OB|DAG_RL_OB_OB, "Curve Parent");
 					else
-						dag_add_relation(dag,node2,node,DAG_RL_OB_OB);
+						dag_add_relation(dag,node2,node,DAG_RL_OB_OB, "Curve Parent");
 				}
 					else
-						dag_add_relation(dag,node2,node,DAG_RL_OB_OB);
+						dag_add_relation(dag,node2,node,DAG_RL_OB_OB, "Curve Parent");
 		}
 		/* exception case: parent is duplivert */
 		if(ob->type==OB_MBALL && (ob->parent->transflag & OB_DUPLIVERTS)) {
-			dag_add_relation(dag, node2, node, DAG_RL_DATA_DATA|DAG_RL_OB_OB);
+			dag_add_relation(dag, node2, node, DAG_RL_DATA_DATA|DAG_RL_OB_OB, "Duplivert");
 		}
 		
 		addtoroot = 0;
 	}
 	if (ob->track) {
 		node2 = dag_get_node(dag,ob->track);
-		dag_add_relation(dag,node2,node,DAG_RL_OB_OB);
+		dag_add_relation(dag,node2,node,DAG_RL_OB_OB, "Track To");
 		addtoroot = 0;
 	}
 	if (ob->proxy) {
 		node2 = dag_get_node(dag, ob->proxy);
-		dag_add_relation(dag, node, node2, DAG_RL_DATA_DATA|DAG_RL_OB_OB);
+		dag_add_relation(dag, node, node2, DAG_RL_DATA_DATA|DAG_RL_OB_OB, "Proxy");
 		/* inverted relation, so addtoroot shouldn't be set to zero */
 	}
-	
+	if (ob->type==OB_CAMERA) {
+		Camera *cam = (Camera *)ob->data;
+		if (cam->ipo) {
+			dag_add_driver_relation(cam->ipo, dag, node, 1);
+		}
+		if (cam->dof_ob) {
+			node2 = dag_get_node(dag, cam->dof_ob);
+			dag_add_relation(dag,node2,node,DAG_RL_OB_OB, "Camera DoF");
+		}
+	}
+	if (ob->type==OB_LAMP) {
+		Lamp *la = (Lamp *)ob->data;
+		if (la->ipo) {
+			dag_add_driver_relation(la->ipo, dag, node, 1);
+		}
+	}
 	if (ob->transflag & OB_DUPLI) {
 		if((ob->transflag & OB_DUPLIGROUP) && ob->dup_group) {
 			GroupObject *go;
@@ -485,46 +534,33 @@ static void build_dag_object(DagForest *dag, DagNode *scenenode, Object *ob, int
 				if(go->ob) {
 					node2 = dag_get_node(dag, go->ob);
 					/* node2 changes node1, this keeps animations updated in groups?? not logical? */
-					dag_add_relation(dag, node2, node, DAG_RL_OB_OB);
+					dag_add_relation(dag, node2, node, DAG_RL_OB_OB, "Dupligroup");
 				}
 			}
 		}
 	}
     
 	/* softbody collision  */
-	if((ob->type==OB_MESH) || (ob->type==OB_CURVE) || (ob->type==OB_LATTICE)) {
-		Base *base;
-		if(modifiers_isSoftbodyEnabled(ob)){
-			// would be nice to have a list of colliders here
-			// so for now walk all objects in scene check 'same layer rule'
-			for(base = G.scene->base.first; base; base= base->next) {
-				if( (base->lay & ob->lay) && base->object->pd) {
-					Object *ob1= base->object;
-					if((ob1->pd->deflect) && (ob1 != ob))  {
-						node2 = dag_get_node(dag, ob1);					
-						dag_add_relation(dag, node2, node, DAG_RL_DATA_DATA|DAG_RL_OB_DATA);						
-					}
-				}
-			}
-		}
-	}
+	if((ob->type==OB_MESH) || (ob->type==OB_CURVE) || (ob->type==OB_LATTICE))
+		if(modifiers_isSoftbodyEnabled(ob) || modifiers_isClothEnabled(ob))
+			dag_add_collision_field_relation(dag, ob, node);
 		
 	if (ob->type==OB_MBALL) {
 		Object *mom= find_basis_mball(ob);
 		if(mom!=ob) {
 			node2 = dag_get_node(dag, mom);
-			dag_add_relation(dag,node,node2,DAG_RL_DATA_DATA|DAG_RL_OB_DATA);  // mom depends on children!
+			dag_add_relation(dag,node,node2,DAG_RL_DATA_DATA|DAG_RL_OB_DATA, "Metaball");  // mom depends on children!
 		}
 	}
 	else if (ob->type==OB_CURVE) {
 		Curve *cu= ob->data;
 		if(cu->bevobj) {
 			node2 = dag_get_node(dag, cu->bevobj);
-			dag_add_relation(dag,node2,node,DAG_RL_DATA_DATA|DAG_RL_OB_DATA);
+			dag_add_relation(dag,node2,node,DAG_RL_DATA_DATA|DAG_RL_OB_DATA, "Curve Bevel");
 		}
 		if(cu->taperobj) {
 			node2 = dag_get_node(dag, cu->taperobj);
-			dag_add_relation(dag,node2,node,DAG_RL_DATA_DATA|DAG_RL_OB_DATA);
+			dag_add_relation(dag,node2,node,DAG_RL_DATA_DATA|DAG_RL_OB_DATA, "Curve Taper");
 		}
 		if(cu->ipo)
 			dag_add_driver_relation(cu->ipo, dag, node, 1);
@@ -534,59 +570,108 @@ static void build_dag_object(DagForest *dag, DagNode *scenenode, Object *ob, int
 		Curve *cu= ob->data;
 		if(cu->textoncurve) {
 			node2 = dag_get_node(dag, cu->textoncurve);
-			dag_add_relation(dag,node2,node,DAG_RL_DATA_DATA|DAG_RL_OB_DATA);
+			dag_add_relation(dag,node2,node,DAG_RL_DATA_DATA|DAG_RL_OB_DATA, "Texture On Curve");
 		}
 	}
-	else if(ob->type==OB_MESH) {
-		PartEff *paf= give_parteff(ob);
-		if(paf) {
-			ListBase *listb;
-			pEffectorCache *ec;
+	
+	psys= ob->particlesystem.first;
+	if(psys) {
+		ParticleEffectorCache *nec;
+		GroupObject *go;
+
+		for(; psys; psys=psys->next) {
+			ParticleSettings *part= psys->part;
 			
-			/* ob location depends on itself */
-			if((paf->flag & PAF_STATIC)==0)
-				dag_add_relation(dag, node, node, DAG_RL_OB_DATA);
-			
-			listb= pdInitEffectors(ob, paf->group);		/* note, makes copy... */
-			if(listb) {
-				for(ec= listb->first; ec; ec= ec->next) {
-					Object *ob1= ec->ob;
-					PartDeflect *pd= ob1->pd;
-						
-					if(pd->forcefield) {
+			dag_add_relation(dag, node, node, DAG_RL_OB_DATA, "Particle-Object Relation");
+
+			if(part->phystype==PART_PHYS_KEYED && psys->keyed_ob &&
+			   BLI_findlink(&psys->keyed_ob->particlesystem,psys->keyed_psys-1)) {
+				node2 = dag_get_node(dag, psys->keyed_ob);
+				dag_add_relation(dag, node2, node, DAG_RL_DATA_DATA, "Particle Keyed Physics");
+			}
+
+			if(part->draw_as == PART_DRAW_OB && part->dup_ob) {
+				node2 = dag_get_node(dag, part->dup_ob);
+				dag_add_relation(dag, node, node2, DAG_RL_OB_OB, "Particle Object Visualisation");
+				if(part->dup_ob->type == OB_MBALL)
+					dag_add_relation(dag, node, node2, DAG_RL_DATA_DATA, "Particle Object Visualisation");
+			}
+
+			if(part->draw_as == PART_DRAW_GR && part->dup_group) {
+				for(go=part->dup_group->gobject.first; go; go=go->next) {
+					node2 = dag_get_node(dag, go->ob);
+					dag_add_relation(dag, node, node2, DAG_RL_OB_OB, "Particle Group Visualisation");
+				}
+			}
+
+			if(psys->effectors.first)
+				psys_end_effectors(psys);
+			psys_init_effectors(ob,psys->part->eff_group,psys);
+
+			if(psys->effectors.first) {
+				for(nec= psys->effectors.first; nec; nec= nec->next) {
+					Object *ob1= nec->ob;
+
+					if(nec->type & PSYS_EC_EFFECTOR) {
 						node2 = dag_get_node(dag, ob1);
-						if(pd->forcefield==PFIELD_GUIDE)
-							dag_add_relation(dag, node2, node, DAG_RL_DATA_DATA|DAG_RL_OB_DATA);
+						if(ob1->pd->forcefield==PFIELD_GUIDE)
+							dag_add_relation(dag, node2, node, DAG_RL_DATA_DATA|DAG_RL_OB_DATA, "Particle Field");
 						else
-							dag_add_relation(dag, node2, node, DAG_RL_OB_DATA);
+							dag_add_relation(dag, node2, node, DAG_RL_OB_DATA, "Particle Field");
+					}
+					else if(nec->type & PSYS_EC_DEFLECT) {
+						node2 = dag_get_node(dag, ob1);
+						dag_add_relation(dag, node2, node, DAG_RL_DATA_DATA|DAG_RL_OB_DATA, "Particle Collision");
+					}
+					else if(nec->type & PSYS_EC_PARTICLE) {
+						node2 = dag_get_node(dag, ob1);
+						dag_add_relation(dag, node2, node, DAG_RL_DATA_DATA, "Particle Field");
+					}
+					
+					if(nec->type & PSYS_EC_REACTOR) {
+						node2 = dag_get_node(dag, ob1);
+						dag_add_relation(dag, node, node2, DAG_RL_DATA_DATA, "Particle Reactor");
 					}
 				}
-				
-				pdEndEffectors(listb);	/* restores copy... */
 			}
 		}
 	}
 	
-	for (con = ob->constraints.first; con; con=con->next){
-		if (constraint_has_target(con)) {
-			char *str;
-			Object *obt= get_constraint_target(con, &str);
+	for (con = ob->constraints.first; con; con=con->next) {
+		bConstraintTypeInfo *cti= constraint_get_typeinfo(con);
+		ListBase targets = {NULL, NULL};
+		bConstraintTarget *ct;
+		
+		if (cti && cti->get_constraint_targets) {
+			cti->get_constraint_targets(con, &targets);
 			
-			node2 = dag_get_node(dag, obt);
-			if(con->type==CONSTRAINT_TYPE_FOLLOWPATH)
-				dag_add_relation(dag, node2, node, DAG_RL_DATA_OB|DAG_RL_OB_OB);
-			else {
-				if(obt->type==OB_ARMATURE && str[0])
-					dag_add_relation(dag, node2, node, DAG_RL_DATA_OB|DAG_RL_OB_OB);
+			for (ct= targets.first; ct; ct= ct->next) {
+				Object *obt;
+				
+				if (ct->tar)
+					obt= ct->tar;
 				else
-					dag_add_relation(dag, node2, node, DAG_RL_OB_OB);
+					continue;
+				
+				node2 = dag_get_node(dag, obt);
+				if (ELEM(con->type, CONSTRAINT_TYPE_FOLLOWPATH, CONSTRAINT_TYPE_CLAMPTO))
+					dag_add_relation(dag, node2, node, DAG_RL_DATA_OB|DAG_RL_OB_OB, cti->name);
+				else {
+					if (ELEM3(obt->type, OB_ARMATURE, OB_MESH, OB_LATTICE) && (ct->subtarget[0]))
+						dag_add_relation(dag, node2, node, DAG_RL_DATA_OB|DAG_RL_OB_OB, cti->name);
+					else
+						dag_add_relation(dag, node2, node, DAG_RL_OB_OB, cti->name);
+				}
+				addtoroot = 0;
 			}
-			addtoroot = 0;
+			
+			if (cti->flush_constraint_targets)
+				cti->flush_constraint_targets(con, &targets, 1);
 		}
 	}
 
 	if (addtoroot == 1 )
-		dag_add_relation(dag,scenenode,node,DAG_RL_SCENE);
+		dag_add_relation(dag,scenenode,node,DAG_RL_SCENE, "Scene Relation");
 }
 
 struct DagForest *build_dag(struct Scene *sce, short mask) 
@@ -777,7 +862,7 @@ DagNode * dag_get_sub_node (DagForest *forest,void * fob)
 	return node;
 }
 
-void dag_add_relation(DagForest *forest, DagNode *fob1, DagNode *fob2, short rel) 
+void dag_add_relation(DagForest *forest, DagNode *fob1, DagNode *fob2, short rel, char *name) 
 {
 	DagAdjList *itA = fob1->child;
 	
@@ -795,10 +880,11 @@ void dag_add_relation(DagForest *forest, DagNode *fob1, DagNode *fob2, short rel
 	itA->type = rel;
 	itA->count = 1;
 	itA->next = fob1->child;
+	itA->name = name;
 	fob1->child = itA;
 }
 
-static void dag_add_parent_relation(DagForest *forest, DagNode *fob1, DagNode *fob2, short rel) 
+static void dag_add_parent_relation(DagForest *forest, DagNode *fob1, DagNode *fob2, short rel, char *name) 
 {
 	DagAdjList *itA = fob2->parent;
 	
@@ -816,9 +902,66 @@ static void dag_add_parent_relation(DagForest *forest, DagNode *fob1, DagNode *f
 	itA->type = rel;
 	itA->count = 1;
 	itA->next = fob2->parent;
+	itA->name = name;
 	fob2->parent = itA;
 }
 
+static char *dag_node_name(DagNode *node)
+{
+	if(node->ob == NULL)
+		return "null";
+	else if(ugly_hack_sorry)
+		return ((ID*)(node->ob))->name+2;
+	else
+		return ((bPoseChannel*)(node->ob))->name;
+}
+
+#if 0
+static void dag_node_print_dependencies(DagNode *node)
+{
+	DagAdjList *itA;
+
+	printf("%s depends on:\n", dag_node_name(node));
+
+	for(itA= node->parent; itA; itA= itA->next)
+		printf("  %s through %s\n", dag_node_name(itA->node), itA->name);
+	printf("\n");
+}
+#endif
+
+static int dag_node_print_dependency_recurs(DagNode *node, DagNode *endnode)
+{
+	DagAdjList *itA;
+
+	if(node->color == DAG_BLACK)
+		return 0;
+
+	node->color= DAG_BLACK;
+
+	if(node == endnode)
+		return 1;
+
+	for(itA= node->parent; itA; itA= itA->next) {
+		if(dag_node_print_dependency_recurs(itA->node, endnode)) {
+			printf("  %s depends on %s through %s.\n", dag_node_name(node), dag_node_name(itA->node), itA->name);
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
+static void dag_node_print_dependency_cycle(DagForest *dag, DagNode *startnode, DagNode *endnode, char *name)
+{
+	DagNode *node;
+
+    for(node = dag->DagNode.first; node; node= node->next)
+		node->color= DAG_WHITE;
+
+	printf("  %s depends on %s through %s.\n", dag_node_name(endnode), dag_node_name(startnode), name);
+	dag_node_print_dependency_recurs(startnode, endnode);
+	printf("\n");
+}
 
 /*
  * MainDAG is the DAG of all objects in current scene
@@ -1548,11 +1691,12 @@ static void flush_update_node(DagNode *node, unsigned int layer, int curtime)
 	ob= node->ob;
 	if(ob && (ob->recalc & OB_RECALC)) {
 		all_layer= ob->lay;
+
 		/* got an object node that changes, now check relations */
 		for(itA = node->child; itA; itA= itA->next) {
 			all_layer |= itA->lay;
 			/* the relationship is visible */
-			if(itA->lay & layer) {
+			if((itA->lay & layer) || (itA->node->ob == G.obedit)) {
 				if(itA->node->type==ID_OB) {
 					obc= itA->node->ob;
 					oldflag= obc->recalc;
@@ -1583,7 +1727,7 @@ static void flush_update_node(DagNode *node, unsigned int layer, int curtime)
 			}
 		}
 		/* even nicer, we can clear recalc flags...  */
-		if((all_layer & layer)==0) {
+		if((all_layer & layer)==0 && (ob != G.obedit)) {
 			/* but existing displaylists or derivedmesh should be freed */
 			if(ob->recalc & OB_RECALC_DATA)
 				object_free_display(ob);
@@ -1597,7 +1741,7 @@ static void flush_update_node(DagNode *node, unsigned int layer, int curtime)
 	/* could merge this in with loop above...? (ton) */
 	for(itA = node->child; itA; itA= itA->next) {
 		/* the relationship is visible */
-		if(itA->lay & layer) {
+		if((itA->lay & layer) || (itA->node->ob == G.obedit)) {
 			if(itA->node->type==ID_OB) {
 				obc= itA->node->ob;
 				/* child moves */
@@ -1621,18 +1765,24 @@ static void flush_update_node(DagNode *node, unsigned int layer, int curtime)
 }
 
 /* node was checked to have lasttime != curtime , and is of type ID_OB */
-static unsigned int flush_layer_node(DagNode *node, int curtime)
+static unsigned int flush_layer_node(Scene *sce, DagNode *node, int curtime)
 {
+	Base *base;
 	DagAdjList *itA;
 	
 	node->lasttime= curtime;
-	node->lay= ((Object *)node->ob)->lay;
+	node->lay= 0;
+	for(base= sce->base.first; base; base= base->next) {
+		if(node->ob == base->object) {
+			node->lay= ((Object *)node->ob)->lay;
+			break;
+		}
+	}
 	
 	for(itA = node->child; itA; itA= itA->next) {
 		if(itA->node->type==ID_OB) {
 			if(itA->node->lasttime!=curtime) {
-				itA->lay= flush_layer_node(itA->node, curtime);  // lay is only set once for each relation
-				//printf("layer %d for relation %s to %s\n", itA->lay, ((Object *)node->ob)->id.name, ((Object *)itA->node->ob)->id.name);
+				itA->lay= flush_layer_node(sce, itA->node, curtime);  // lay is only set once for each relation
 			}
 			else itA->lay= itA->node->lay;
 			
@@ -1643,11 +1793,38 @@ static unsigned int flush_layer_node(DagNode *node, int curtime)
 	return node->lay;
 }
 
+/* node was checked to have lasttime != curtime , and is of type ID_OB */
+static void flush_pointcache_reset(DagNode *node, int curtime, int reset)
+{
+	DagAdjList *itA;
+	Object *ob;
+	
+	node->lasttime= curtime;
+	
+	for(itA = node->child; itA; itA= itA->next) {
+		if(itA->node->type==ID_OB) {
+			if(itA->node->lasttime!=curtime) {
+				ob= (Object*)(node->ob);
+
+				if(reset || (ob->recalc & OB_RECALC)) {
+					if(BKE_ptcache_object_reset(ob, PTCACHE_RESET_DEPSGRAPH))
+						ob->recalc |= OB_RECALC_DATA;
+
+					flush_pointcache_reset(itA->node, curtime, 1);
+				}
+				else
+					flush_pointcache_reset(itA->node, curtime, 0);
+			}
+		}
+	}
+}
+
 /* flushes all recalc flags in objects down the dependency tree */
-void DAG_scene_flush_update(Scene *sce, unsigned int lay)
+void DAG_scene_flush_update(Scene *sce, unsigned int lay, int time)
 {
 	DagNode *firstnode;
 	DagAdjList *itA;
+	Object *ob;
 	int lasttime;
 	
 	if(sce->theDag==NULL) {
@@ -1656,21 +1833,43 @@ void DAG_scene_flush_update(Scene *sce, unsigned int lay)
 	}
 	
 	firstnode= sce->theDag->DagNode.first;  // always scene node
+
+	for(itA = firstnode->child; itA; itA= itA->next)
+		itA->lay= 0;
 	
 	/* first we flush the layer flags */
 	sce->theDag->time++;	// so we know which nodes were accessed
 	lasttime= sce->theDag->time;
-	for(itA = firstnode->child; itA; itA= itA->next) {
+
+	for(itA = firstnode->child; itA; itA= itA->next)
 		if(itA->node->lasttime!=lasttime && itA->node->type==ID_OB) 
-			flush_layer_node(itA->node, lasttime);
-	}
+			flush_layer_node(sce, itA->node, lasttime);
 	
 	/* then we use the relationships + layer info to flush update events */
 	sce->theDag->time++;	// so we know which nodes were accessed
 	lasttime= sce->theDag->time;
-	for(itA = firstnode->child; itA; itA= itA->next) {
-		if(itA->node->lasttime!=lasttime && itA->node->type==ID_OB) 
+	for(itA = firstnode->child; itA; itA= itA->next)
+		if(itA->node->lasttime!=lasttime && itA->node->type==ID_OB)
 			flush_update_node(itA->node, lay, lasttime);
+
+	/* if update is not due to time change, do pointcache clears */
+	if(!time) {
+		sce->theDag->time++;	// so we know which nodes were accessed
+		lasttime= sce->theDag->time;
+		for(itA = firstnode->child; itA; itA= itA->next) {
+			if(itA->node->lasttime!=lasttime && itA->node->type==ID_OB)  {
+				ob= (Object*)(itA->node->ob);
+
+				if(ob->recalc & OB_RECALC) {
+					if(BKE_ptcache_object_reset(ob, PTCACHE_RESET_DEPSGRAPH))
+						ob->recalc |= OB_RECALC_DATA;
+
+					flush_pointcache_reset(itA->node, lasttime, 1);
+				}
+				else
+					flush_pointcache_reset(itA->node, lasttime, 0);
+			}
+		}
 	}
 }
 
@@ -1705,10 +1904,23 @@ static void dag_object_time_update_flags(Object *ob)
 	if(ob->ipo) ob->recalc |= OB_RECALC_OB;
 	else if(ob->constraints.first) {
 		bConstraint *con;
-		for (con = ob->constraints.first; con; con=con->next){
-			if (constraint_has_target(con)) {
-				ob->recalc |= OB_RECALC_OB;
-				break;
+		for (con = ob->constraints.first; con; con=con->next) {
+			bConstraintTypeInfo *cti= constraint_get_typeinfo(con);
+			ListBase targets = {NULL, NULL};
+			bConstraintTarget *ct;
+			
+			if (cti && cti->get_constraint_targets) {
+				cti->get_constraint_targets(con, &targets);
+				
+				for (ct= targets.first; ct; ct= ct->next) {
+					if (ct->tar) {
+						ob->recalc |= OB_RECALC_OB;
+						break;
+					}
+				}
+				
+				if (cti->flush_constraint_targets)
+					cti->flush_constraint_targets(con, &targets, 1);
 			}
 		}
 	}
@@ -1736,6 +1948,7 @@ static void dag_object_time_update_flags(Object *ob)
 	}
 	else if(modifiers_isSoftbodyEnabled(ob)) ob->recalc |= OB_RECALC_DATA;
 	else if(object_modifiers_use_time(ob)) ob->recalc |= OB_RECALC_DATA;
+	else if((ob->pose) && (ob->pose->flag & POSE_CONSTRAINTS_TIMEDEPEND)) ob->recalc |= OB_RECALC_DATA;
 	else {
 		Mesh *me;
 		Curve *cu;
@@ -1750,21 +1963,14 @@ static void dag_object_time_update_flags(Object *ob)
 						ob->shapeflag &= ~OB_SHAPE_TEMPLOCK;
 					}
 				}
-				else if(ob->effect.first) {
-					Effect *eff= ob->effect.first;
-					PartEff *paf= give_parteff(ob);
-					
-					if(eff->type==EFF_WAVE) 
-						ob->recalc |= OB_RECALC_DATA;
-					else if(paf && paf->keys==NULL)
-						ob->recalc |= OB_RECALC_DATA;
-				}
 				if((ob->fluidsimFlag & OB_FLUIDSIM_ENABLE) && (ob->fluidsimSettings)) {
 					// fluidsimSettings might not be initialized during load...
 					if(ob->fluidsimSettings->type & (OB_FLUIDSIM_DOMAIN|OB_FLUIDSIM_PARTICLE)) {
 						ob->recalc |= OB_RECALC_DATA; // NT FSPARTICLE
 					}
 				}
+				if(ob->particlesystem.first)
+					ob->recalc |= OB_RECALC_DATA;
 				break;
 			case OB_CURVE:
 			case OB_SURF:
@@ -1794,6 +2000,17 @@ static void dag_object_time_update_flags(Object *ob)
 				if(ob->transflag & OB_DUPLI) ob->recalc |= OB_RECALC_DATA;
 				break;
 		}
+
+		if(ob->particlesystem.first) {
+			ParticleSystem *psys= ob->particlesystem.first;
+
+			for(; psys; psys=psys->next) {
+				if(psys_check_enabled(ob, psys)) {
+					ob->recalc |= OB_RECALC_DATA;
+					break;
+				}
+			}
+		}
 	}		
 }
 
@@ -1829,7 +2046,7 @@ void DAG_scene_update_flags(Scene *scene, unsigned int lay)
 	}
 	
 	for(sce= scene; sce; sce= sce->set)
-		DAG_scene_flush_update(sce, lay);
+		DAG_scene_flush_update(sce, lay, 1);
 	
 	/* test: set time flag, to disable baked systems to update */
 	for(SETLOOPER(scene, base)) {
@@ -1879,7 +2096,9 @@ void DAG_object_flush_update(Scene *sce, Object *ob, short flag)
 {
 	
 	if(ob==NULL || sce->theDag==NULL) return;
+
 	ob->recalc |= flag;
+	BKE_ptcache_object_reset(ob, PTCACHE_RESET_DEPSGRAPH);
 	
 	/* all users of this ob->data should be checked */
 	/* BUT! displists for curves are still only on cu */
@@ -1892,8 +2111,9 @@ void DAG_object_flush_update(Scene *sce, Object *ob, short flag)
 				else {
 					Object *obt;
 					for (obt=G.main->object.first; obt; obt= obt->id.next) {
-						if (obt->data==ob->data) {
+						if (obt != ob && obt->data==ob->data) {
 							obt->recalc |= OB_RECALC_DATA;
+							BKE_ptcache_object_reset(obt, PTCACHE_RESET_DEPSGRAPH);
 						}
 					}
 				}
@@ -1902,9 +2122,9 @@ void DAG_object_flush_update(Scene *sce, Object *ob, short flag)
 	}
 	
 	if(G.curscreen)
-		DAG_scene_flush_update(sce, dag_screen_view3d_layers());
+		DAG_scene_flush_update(sce, dag_screen_view3d_layers(), 0);
 	else
-		DAG_scene_flush_update(sce, sce->lay);
+		DAG_scene_flush_update(sce, sce->lay, 0);
 }
 
 /* recursively descends tree, each node only checked once */
@@ -1952,8 +2172,8 @@ void DAG_object_update_flags(Scene *sce, Object *ob, unsigned int lay)
 	
 	/* object not in scene? then handle group exception. needs to be dagged once too */
 	if(node==NULL) {
-		Group *group= find_group(ob);
-		if(group) {
+		Group *group= NULL;
+		while( (group = find_group(ob, group)) ) {
 			GroupObject *go;
 			/* primitive; tag all... this call helps building groups for particles */
 			for(go= group->gobject.first; go; go= go->next)
@@ -1989,16 +2209,21 @@ void DAG_object_update_flags(Scene *sce, Object *ob, unsigned int lay)
 static int node_recurs_level(DagNode *node, int level)
 {
 	DagAdjList *itA;
-	
+	int newlevel;
+
 	node->color= DAG_BLACK;	/* done */
-	level++;
+	newlevel= ++level;
 	
 	for(itA= node->parent; itA; itA= itA->next) {
-		if(itA->node->color==DAG_WHITE)
+		if(itA->node->color==DAG_WHITE) {
 			itA->node->ancestor_count= node_recurs_level(itA->node, level);
+			newlevel= MAX2(newlevel, level+itA->node->ancestor_count);
+		}
+		else
+			newlevel= MAX2(newlevel, level+itA->node->ancestor_count);
 	}
 	
-	return level;
+	return newlevel;
 }
 
 static void pose_check_cycle(DagForest *dag)
@@ -2023,9 +2248,10 @@ static void pose_check_cycle(DagForest *dag)
 				bPoseChannel *pchan= (bPoseChannel *)node->ob;
 				bPoseChannel *parchan= (bPoseChannel *)itA->node->ob;
 				
-				if(pchan && parchan) 
-					if(pchan->parent!=parchan)
-						printf("Cycle in %s to %s\n", pchan->name, parchan->name);
+				if(pchan && parchan)  {
+					printf("Cycle detected:\n");
+					dag_node_print_dependency_cycle(dag, itA->node, node, itA->name);
+				}
 			}
 		}
 	}
@@ -2059,51 +2285,81 @@ void DAG_pose_sort(Object *ob)
 		
 		if(pchan->parent) {
 			node2 = dag_get_node(dag, pchan->parent);
-			dag_add_relation(dag, node2, node, 0);
-			dag_add_parent_relation(dag, node2, node, 0);
+			dag_add_relation(dag, node2, node, 0, "Parent Relation");
+			dag_add_parent_relation(dag, node2, node, 0, "Parent Relation");
 			addtoroot = 0;
 		}
-		for (con = pchan->constraints.first; con; con=con->next){
-			if (constraint_has_target(con)) {
-				char *subtarget;
-				Object *target = get_constraint_target(con, &subtarget);
-				
-				if (target==ob && subtarget) {
-					bPoseChannel *target= get_pose_channel(ob->pose, subtarget);
-					if(target) {
-						node2= dag_get_node(dag, target);
-						dag_add_relation(dag, node2, node, 0);
-						dag_add_parent_relation(dag, node2, node, 0);
-						
-						if(con->type==CONSTRAINT_TYPE_KINEMATIC) {
-							bKinematicConstraint *data = (bKinematicConstraint*)con->data;
-							bPoseChannel *parchan;
-							int segcount= 0;
-							
-							/* exclude tip from chain? */
-							if(!(data->flag & CONSTRAINT_IK_TIP))
-								parchan= pchan->parent;
-							else
-								parchan= pchan;
-							
-							/* Walk to the chain's root */
-							while (parchan){
-								node3= dag_get_node(dag, parchan);
-								dag_add_relation(dag, node2, node3, 0);
-								dag_add_parent_relation(dag, node2, node3, 0);
-								
-								segcount++;
-								if(segcount==data->rootbone || segcount>255) break; // 255 is weak
-								parchan= parchan->parent;
-							}
+		for (con = pchan->constraints.first; con; con=con->next) {
+			bConstraintTypeInfo *cti= constraint_get_typeinfo(con);
+			ListBase targets = {NULL, NULL};
+			bConstraintTarget *ct;
+			
+			if(con->ipo) {
+				IpoCurve *icu;
+				for(icu= con->ipo->curve.first; icu; icu= icu->next) {
+					/* icu->driver->ob should actually point to ob->proxy if it
+					 * is a proxy, but since it wasn't set correct it older
+					 * files comparing with ob->proxy makes it work for those */
+					if(icu->driver && (icu->driver->ob==ob || icu->driver->ob==ob->proxy)) {
+						bPoseChannel *target= get_pose_channel(ob->pose, icu->driver->name);
+						if(target) {
+							node2 = dag_get_node(dag, target);
+							dag_add_relation(dag, node2, node, 0, "Ipo Driver");
+							dag_add_parent_relation(dag, node2, node, 0, "Ipo Driver");
+
+							/* uncommented this line, results in dependencies
+							 * not being added properly for this constraint,
+							 * what is the purpose of this? - brecht */
+							/*cti= NULL;*/	/* trick to get next loop skipped */
 						}
 					}
 				}
 			}
+			
+			if (cti && cti->get_constraint_targets) {
+				cti->get_constraint_targets(con, &targets);
+				
+				for (ct= targets.first; ct; ct= ct->next) {
+					if (ct->tar==ob && ct->subtarget[0]) {
+						bPoseChannel *target= get_pose_channel(ob->pose, ct->subtarget);
+						if (target) {
+							node2= dag_get_node(dag, target);
+							dag_add_relation(dag, node2, node, 0, "IK Constraint");
+							dag_add_parent_relation(dag, node2, node, 0, "IK Constraint");
+
+							if (con->type==CONSTRAINT_TYPE_KINEMATIC) {
+								bKinematicConstraint *data = (bKinematicConstraint *)con->data;
+								bPoseChannel *parchan;
+								int segcount= 0;
+								
+								/* exclude tip from chain? */
+								if(!(data->flag & CONSTRAINT_IK_TIP))
+									parchan= pchan->parent;
+								else
+									parchan= pchan;
+								
+								/* Walk to the chain's root */
+								while (parchan) {
+									node3= dag_get_node(dag, parchan);
+									dag_add_relation(dag, node2, node3, 0, "IK Constraint");
+									dag_add_parent_relation(dag, node2, node3, 0, "IK Constraint");
+									
+									segcount++;
+									if (segcount==data->rootbone || segcount>255) break; // 255 is weak
+									parchan= parchan->parent;
+								}
+							}
+						}
+					}
+				}
+				
+				if (cti->flush_constraint_targets)
+					cti->flush_constraint_targets(con, &targets, 1);
+			}
 		}
 		if (addtoroot == 1 ) {
-			dag_add_relation(dag, rootnode, node, 0);
-			dag_add_parent_relation(dag, rootnode, node, 0);
+			dag_add_relation(dag, rootnode, node, 0, "Root Bone Relation");
+			dag_add_parent_relation(dag, rootnode, node, 0, "Root Bone Relation");
 		}
 	}
 
