@@ -2,17 +2,14 @@
 /*  key.c      
  *  
  * 
- * $Id: key.c 9724 2007-01-12 04:47:38Z nicholasbishop $
+ * $Id: key.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,7 +27,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <math.h>
@@ -545,26 +542,26 @@ static void cp_key(int start, int end, int tot, char *poin, Key *key, KeyBlock *
 			case IPO_FLOAT:
 				
 				if(weights) {
-					memcpy(poin, kref, 4*cp[0]);
+					memcpy(poin, kref, sizeof(float)*cp[0]);
 					if(*weights!=0.0f)
 						rel_flerp(cp[0], (float *)poin, (float *)kref, (float *)k1, *weights);
 					weights++;
 				}
 				else 
-					memcpy(poin, k1, 4*cp[0]);
+					memcpy(poin, k1, sizeof(float)*cp[0]);
 
 				poin+= ofsp[0];
 
 				break;
 			case IPO_BPOINT:
-				memcpy(poin, k1, 3*4);
-				memcpy(poin+16, k1+12, 4);
+				memcpy(poin, k1, 3*sizeof(float));
+				memcpy(poin+4*sizeof(float), k1+3*sizeof(float), sizeof(float));
 				
 				poin+= ofsp[0];				
 
 				break;
 			case IPO_BEZTRIPLE:
-				memcpy(poin, k1, 4*12);
+				memcpy(poin, k1, sizeof(float)*10);
 				poin+= ofsp[0];	
 
 				break;
@@ -672,19 +669,21 @@ static void do_rel_key(int start, int end, int tot, char *basispoin, Key *key, i
 	
 	/* step 2: do it */
 	
-	kb= key->block.first;
-	while(kb) {
-		
+	for(kb=key->block.first; kb; kb=kb->next) {
 		if(kb!=key->refkey) {
 			float icuval= kb->curval;
 			
 			/* only with value, and no difference allowed */
-			if(icuval!=0.0f && kb->totelem==tot) {
+			if(!(kb->flag & KEYBLOCK_MUTE) && icuval!=0.0f && kb->totelem==tot) {
+				KeyBlock *refb;
 				float weight, *weights= kb->weights;
 				
 				poin= basispoin;
-				reffrom= key->refkey->data;
 				from= kb->data;
+				/* reference now can be any block */
+				refb= BLI_findlink(&key->block, kb->relative);
+				if(refb==NULL) continue;
+				reffrom= refb->data;
 				
 				poin+= start*ofs[0];
 				reffrom+= key->elemsize*start;	// key elemsize yes!
@@ -734,7 +733,6 @@ static void do_rel_key(int start, int end, int tot, char *basispoin, Key *key, i
 				}
 			}
 		}
-		kb= kb->next;
 	}
 }
 
@@ -1025,7 +1023,7 @@ static int do_mesh_key(Object *ob, Mesh *me)
 		
 		for(a=0; a<me->totvert; a+=step, cfra+= delta) {
 			
-			ctime= bsystem_time(0, 0, cfra, 0.0);
+			ctime= bsystem_time(0, cfra, 0.0);
 			if(calc_ipo_spec(me->key->ipo, KEY_SPEED, &ctime)==0) {
 				ctime /= 100.0;
 				CLAMP(ctime, 0.0, 1.0);
@@ -1060,7 +1058,7 @@ static int do_mesh_key(Object *ob, Mesh *me)
 			}
 		}
 		else {
-			ctime= bsystem_time(ob, 0, G.scene->r.cfra, 0.0);
+			ctime= bsystem_time(ob, G.scene->r.cfra, 0.0);
 
 			if(calc_ipo_spec(me->key->ipo, KEY_SPEED, &ctime)==0) {
 				ctime /= 100.0;
@@ -1180,7 +1178,7 @@ static int do_curve_key(Curve *cu)
 		
 		for(a=0; a<tot; a+=step, cfra+= delta) {
 			
-			ctime= bsystem_time(0, 0, cfra, 0.0);
+			ctime= bsystem_time(0, cfra, 0.0);
 			if(calc_ipo_spec(cu->key->ipo, KEY_SPEED, &ctime)==0) {
 				ctime /= 100.0;
 				CLAMP(ctime, 0.0, 1.0);
@@ -1202,7 +1200,7 @@ static int do_curve_key(Curve *cu)
 	}
 	else {
 		
-		ctime= bsystem_time(NULL, 0, (float)G.scene->r.cfra, 0.0);
+		ctime= bsystem_time(NULL, (float)G.scene->r.cfra, 0.0);
 		
 		if(cu->key->type==KEY_RELATIVE) {
 			do_rel_cu_key(cu, ctime);
@@ -1244,7 +1242,7 @@ static int do_latt_key(Object *ob, Lattice *lt)
 		
 		for(a=0; a<tot; a++, cfra+= delta) {
 			
-			ctime= bsystem_time(0, 0, cfra, 0.0);
+			ctime= bsystem_time(0, cfra, 0.0);
 			if(calc_ipo_spec(lt->key->ipo, KEY_SPEED, &ctime)==0) {
 				ctime /= 100.0;
 				CLAMP(ctime, 0.0, 1.0);
@@ -1261,7 +1259,7 @@ static int do_latt_key(Object *ob, Lattice *lt)
 		}		
 	}
 	else {
-		ctime= bsystem_time(NULL, 0, (float)G.scene->r.cfra, 0.0);
+		ctime= bsystem_time(NULL, (float)G.scene->r.cfra, 0.0);
 	
 		if(lt->key->type==KEY_RELATIVE) {
 			KeyBlock *kb;
@@ -1308,6 +1306,9 @@ int do_ob_key(Object *ob)
 	if(ob->shapeflag & (OB_SHAPE_LOCK|OB_SHAPE_TEMPLOCK)) {
 		KeyBlock *kb= BLI_findlink(&key->block, ob->shapenr-1);
 
+		if(kb && (kb->flag & KEYBLOCK_MUTE))
+			kb= key->refkey;
+
 		if(kb==NULL) {
 			kb= key->block.first;
 			ob->shapenr= 1;
@@ -1342,7 +1343,7 @@ int do_ob_key(Object *ob)
 		if(ob->ipoflag & OB_ACTION_KEY)
 			do_all_object_actions(ob);
 		else {
-			calc_ipo(key->ipo, bsystem_time(ob, 0, G.scene->r.cfra, 0.0));
+			calc_ipo(key->ipo, bsystem_time(ob, G.scene->r.cfra, 0.0));
 			execute_ipo((ID *)key, key->ipo);
 		}
 		
@@ -1386,3 +1387,23 @@ KeyBlock *ob_get_keyblock(Object *ob)
 
 	return NULL;
 }
+
+/* get the appropriate KeyBlock given an index */
+KeyBlock *key_get_keyblock(Key *key, int index)
+{
+	KeyBlock *kb;
+	int i;
+	
+	if (key) {
+		kb= key->block.first;
+		
+		for (i= 1; i < key->totkey; i++) {
+			kb= kb->next;
+			
+			if (index==i)
+				return kb;
+		}
+	}
+	
+	return NULL;
+}
