@@ -2,17 +2,14 @@
  * lattice.c
  *
  *
- * $Id: lattice.c,v 1.40 2007/04/04 13:18:28 campbellbarton Exp $
+ * $Id: lattice.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,7 +27,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <stdio.h>
@@ -483,6 +480,7 @@ static int where_on_path_deform(Object *ob, float ctime, float *vec, float *dir)
 	
 	/* test for cyclic */
 	bl= cu->bev.first;
+	if (!bl->nr) return 0;
 	if(bl && bl->poly> -1) cycl= 1;
 
 	if(cycl==0) {
@@ -518,7 +516,7 @@ static int where_on_path_deform(Object *ob, float ctime, float *vec, float *dir)
 	/* co: local coord, result local too */
 	/* returns quaternion for rotation, using cd->no_rot_axis */
 	/* axis is using another define!!! */
-static float *calc_curve_deform(Object *par, float *co, short axis, CurveDeform *cd)
+static int calc_curve_deform(Object *par, float *co, short axis, CurveDeform *cd, float *quatp)
 {
 	Curve *cu= par->data;
 	float fac, loc[4], dir[3], cent[3];
@@ -548,7 +546,7 @@ static float *calc_curve_deform(Object *par, float *co, short axis, CurveDeform
 	/* to be sure, mostly after file load */
 	if(cu->path==NULL) {
 		makeDispListCurveTypes(par, 0);
-		if(cu->path==NULL) return NULL;	// happens on append...
+		if(cu->path==NULL) return 0;	// happens on append...
 	}
 	
 	/* options */
@@ -574,14 +572,13 @@ static float *calc_curve_deform(Object *par, float *co, short axis, CurveDeform
 	}
 	
 	if( where_on_path_deform(par, fac, loc, dir)) {	/* returns OK */
-		float q[4], mat[3][3];
-		float *quat;
+		float q[4], mat[3][3], quat[4];
 		
 		if(cd->no_rot_axis)	/* set by caller */
 			dir[cd->no_rot_axis-1]= 0.0f;
 		
 		/* -1 for compatibility with old track defines */
-		quat= vectoquat(dir, axis-1, upflag);	/* gives static quat */
+		vectoquat(dir, axis-1, upflag, quat);
 		
 		/* the tilt */
 		if(loc[3]!=0.0) {
@@ -601,18 +598,26 @@ static float *calc_curve_deform(Object *par, float *co, short axis, CurveDeform
 		/* translation */
 		VECADD(co, cent, loc);
 		
-		return quat;
+		if(quatp)
+			QUATCOPY(quatp, quat);
+		
+		return 1;
 	}
-	return NULL;
+	return 0;
 }
 
 void curve_deform_verts(Object *cuOb, Object *target, DerivedMesh *dm, float (*vertexCos)[3], int numVerts, char *vgroup, short defaxis)
 {
-	Curve *cu = cuOb->data;
-	int a, flag = cu->flag;
+	Curve *cu;
+	int a, flag;
 	CurveDeform cd;
 	int use_vgroups;
-	
+
+	if(cuOb->type != OB_CURVE)
+		return;
+
+	cu = cuOb->data;
+	flag = cu->flag;
 	cu->flag |= (CU_PATH|CU_FOLLOW); // needed for path & bevlist
 
 	init_curve_deform(cuOb, target, &cd, (cu->flag & CU_STRETCH)==0);
@@ -667,7 +672,7 @@ void curve_deform_verts(Object *cuOb, Object *target, DerivedMesh *dm, float (*v
 				for(j = 0; j < dvert->totweight; j++) {
 					if(dvert->dw[j].def_nr == index) {
 						VECCOPY(vec, vertexCos[a]);
-						calc_curve_deform(cuOb, vec, defaxis, &cd);
+						calc_curve_deform(cuOb, vec, defaxis, &cd, NULL);
 						VecLerpf(vertexCos[a], vertexCos[a], vec,
 						         dvert->dw[j].weight);
 						Mat4MulVecfl(cd.objectspace, vertexCos[a]);
@@ -685,7 +690,7 @@ void curve_deform_verts(Object *cuOb, Object *target, DerivedMesh *dm, float (*v
 		}
 
 		for(a = 0; a < numVerts; a++) {
-			calc_curve_deform(cuOb, vertexCos[a], defaxis, &cd);
+			calc_curve_deform(cuOb, vertexCos[a], defaxis, &cd, NULL);
 			Mat4MulVecfl(cd.objectspace, vertexCos[a]);
 		}
 	}
@@ -698,8 +703,13 @@ void curve_deform_verts(Object *cuOb, Object *target, DerivedMesh *dm, float (*v
 void curve_deform_vector(Object *cuOb, Object *target, float *orco, float *vec, float mat[][3], int no_rot_axis)
 {
 	CurveDeform cd;
-	float *quat;
+	float quat[4];
 	
+	if(cuOb->type != OB_CURVE) {
+		Mat3One(mat);
+		return;
+	}
+
 	init_curve_deform(cuOb, target, &cd, 0);	/* 0 no dloc */
 	cd.no_rot_axis= no_rot_axis;				/* option to only rotate for XY, for example */
 	
@@ -708,8 +718,7 @@ void curve_deform_vector(Object *cuOb, Object *target, float *orco, float *vec,
 
 	Mat4MulVecfl(cd.curvespace, vec);
 	
-	quat= calc_curve_deform(cuOb, vec, target->trackflag+1, &cd);
-	if(quat) {
+	if(calc_curve_deform(cuOb, vec, target->trackflag+1, &cd, quat)) {
 		float qmat[3][3];
 		
 		QuatToMat3(quat, qmat);
@@ -728,6 +737,9 @@ void lattice_deform_verts(Object *laOb, Object *target, DerivedMesh *dm,
 	int a;
 	int use_vgroups;
 
+	if(laOb->type != OB_LATTICE)
+		return;
+
 	init_latt_deform(laOb, target);
 
 	/* check whether to use vertex groups (only possible if target is a Mesh)
