@@ -1,15 +1,12 @@
 /** 
- * $Id: library.c 11236 2007-07-12 04:56:00Z bdiego $
+ * $Id: library.c 14669 2008-05-04 09:41:15Z campbellbarton $
  * 
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 /*
@@ -78,6 +75,8 @@
 #include "DNA_nla_types.h"
 #include "DNA_effect_types.h"
 #include "DNA_brush_types.h"
+#include "DNA_particle_types.h"
+#include "BKE_particle.h"
 
 #include "BLI_blenlib.h"
 #include "BLI_dynstr.h"
@@ -112,7 +111,7 @@
 #include "BKE_brush.h"
 #include "BKE_idprop.h"
 
-#include "BPI_script.h"
+#include "DNA_space_types.h"
 
 #define MAX_IDPUP		60	/* was 24 */
 
@@ -194,6 +193,8 @@ ListBase *wich_libbase(Main *mainlib, short type)
 			return &(mainlib->nodetree);
 		case ID_BR:
 			return &(mainlib->brush);
+		case ID_PA:
+			return &(mainlib->particle);
 	}
 	return 0;
 }
@@ -254,16 +255,17 @@ int set_listbasepointers(Main *main, ListBase **lb)
 	lb[18]= &(main->nodetree);
 	lb[19]= &(main->brush);
 	lb[20]= &(main->script);
+	lb[21]= &(main->particle);
 	
-	lb[21]= &(main->world);
-	lb[22]= &(main->screen);
-	lb[23]= &(main->object);
-	lb[24]= &(main->scene);
-	lb[25]= &(main->library);
+	lb[22]= &(main->world);
+	lb[23]= &(main->screen);
+	lb[24]= &(main->object);
+	lb[25]= &(main->scene);
+	lb[26]= &(main->library);
 	
-	lb[26]= NULL;
+	lb[27]= NULL;
 
-	return 26;
+	return 27;
 }
 
 /* *********** ALLOC AND FREE *****************
@@ -359,6 +361,9 @@ static ID *alloc_libblock_notest(short type)
 		case ID_BR:
 			id = MEM_callocN(sizeof(Brush), "brush");
 			break;
+		case ID_PA:
+			id = MEM_callocN(sizeof(ParticleSettings), "ParticleSettings");
+  			break;
 	}
 	return id;
 }
@@ -503,6 +508,9 @@ void free_libblock(ListBase *lb, void *idv)
 		case ID_BR:
 			free_brush((Brush *)id);
 			break;
+		case ID_PA:
+			psys_free_settings((ParticleSettings *)id);
+			break;
 	}
 
 	if (id->properties) {
@@ -966,7 +974,7 @@ static void image_fix_relative_path(Image *ima)
 {
 	if(ima->id.lib==NULL) return;
 	if(strncmp(ima->name, "//", 2)==0) {
-		BLI_convertstringcode(ima->name, ima->id.lib->filename, 0);
+		BLI_convertstringcode(ima->name, ima->id.lib->filename);
 		BLI_makestringcode(G.sce, ima->name);
 	}
 }
@@ -983,7 +991,7 @@ static void lib_indirect_test_id(ID *id)
 		Object *ob= (Object *)id;
 		bActionStrip *strip;
 		Mesh *me;
-		PartEff *paf;
+
 		int a;
 
 		for (strip=ob->nlastrips.first; strip; strip=strip->next){
@@ -995,10 +1003,6 @@ static void lib_indirect_test_id(ID *id)
 		for(a=0; a<ob->totcol; a++) {
 			LIBTAG(ob->mat[a]);
 		}
-		
-		paf = give_parteff(ob);
-		if (paf) 
-			LIBTAG(paf->group);
 	
 		LIBTAG(ob->dup_group);
 		LIBTAG(ob->proxy);
