@@ -1,17 +1,14 @@
 /* texture.c
  *
  *
- * $Id: texture.c,v 1.35 2007/04/16 10:55:59 ton Exp $
+ * $Id: texture.c 14463 2008-04-18 19:10:03Z sirdude $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,7 +26,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <stdio.h>
@@ -55,6 +52,7 @@
 #include "DNA_image_types.h"
 #include "DNA_world_types.h"
 #include "DNA_brush_types.h"
+#include "DNA_node_types.h"
 
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"
@@ -75,6 +73,7 @@
 #include "BKE_icons.h"
 #include "BKE_ipo.h"
 #include "BKE_brush.h"
+#include "BKE_node.h"
 
 
 /* ------------------------------------------------------------------------- */
@@ -107,6 +106,7 @@ void open_plugin_tex(PluginTex *pit)
 	pit->result= 0;
 	pit->cfra= 0;
 	pit->version= 0;
+	pit->instance_init= 0;
 	
 	/* clear the error list */
 	PIL_dynlib_get_error_as_string(NULL);
@@ -126,7 +126,7 @@ void open_plugin_tex(PluginTex *pit)
 		
 		if (version != 0) {
 			pit->version= version();
-			if (pit->version>=2 && pit->version<=5) {
+			if( pit->version >= 2 && pit->version <=6) {
 				int (*info_func)(PluginInfo *);
 				PluginInfo *info= (PluginInfo*) MEM_mallocN(sizeof(PluginInfo), "plugin_info"); 
 
@@ -238,13 +238,10 @@ void init_mapping(TexMapping *texmap)
 
 /* ****************** COLORBAND ******************* */
 
-ColorBand *add_colorband(int rangetype)
+void init_colorband(ColorBand *coba, int rangetype)
 {
-	ColorBand *coba;
 	int a;
 	
-	coba= MEM_callocN( sizeof(ColorBand), "colorband");
-	
 	coba->data[0].pos= 0.0;
 	coba->data[1].pos= 1.0;
 	
@@ -281,6 +278,15 @@ ColorBand *add_colorband(int rangetype)
 	
 	coba->tot= 2;
 	
+}
+
+ColorBand *add_colorband(int rangetype)
+{
+	ColorBand *coba;
+	
+	coba= MEM_callocN( sizeof(ColorBand), "colorband");
+	init_colorband(coba, rangetype);
+	
 	return coba;
 }
 
@@ -390,6 +396,7 @@ void free_texture(Tex *tex)
 	free_plugin_tex(tex->plugin);
 	if(tex->coba) MEM_freeN(tex->coba);
 	if(tex->env) BKE_free_envmap(tex->env);
+	BKE_previewimg_free(&tex->preview);
 	BKE_icon_delete((struct ID*)tex);
 	tex->id.icon_id = 0;
 }
@@ -404,7 +411,7 @@ void default_tex(Tex *tex)
 
 	tex->stype= 0;
 	tex->flag= TEX_CHECKER_ODD;
-	tex->imaflag= TEX_INTERPOL+TEX_MIPMAP;
+	tex->imaflag= TEX_INTERPOL+TEX_MIPMAP+TEX_USEALPHA;
 	tex->extend= TEX_REPEAT;
 	tex->cropxmin= tex->cropymin= 0.0;
 	tex->cropxmax= tex->cropymax= 1.0;
@@ -418,7 +425,8 @@ void default_tex(Tex *tex)
 	tex->turbul= 5.0;
 	tex->nabla= 0.025;	// also in do_versions
 	tex->bright= 1.0;
-	tex->contrast= tex->filtersize= 1.0;
+	tex->contrast= 1.0;
+	tex->filtersize= 1.0;
 	tex->rfac= 1.0;
 	tex->gfac= 1.0;
 	tex->bfac= 1.0;
@@ -462,6 +470,8 @@ void default_tex(Tex *tex)
 	tex->iuser.fie_ima= 2;
 	tex->iuser.ok= 1;
 	tex->iuser.frames= 100;
+	
+	tex->preview = NULL;
 }
 
 /* ------------------------------------------------------------------------- */
@@ -507,6 +517,7 @@ void default_mtex(MTex *mtex)
 	mtex->norfac= 0.5;
 	mtex->varfac= 1.0;
 	mtex->dispfac=0.2;
+	mtex->normapspace= MTEX_NSPACE_TANGENT;
 }
 
 
@@ -541,6 +552,8 @@ Tex *copy_texture(Tex *tex)
 	if(texn->coba) texn->coba= MEM_dupallocN(texn->coba);
 	if(texn->env) texn->env= BKE_copy_envmap(texn->env);
 	
+	if(tex->preview) texn->preview = BKE_previewimg_copy(tex->preview);
+
 	return texn;
 }
 
@@ -715,6 +728,7 @@ Tex *give_current_texture(Object *ob, int act)
 	Lamp *la = 0;
 	MTex *mtex = 0;
 	Tex *tex = 0;
+	bNode *node;
 	
 	if(ob==0) return 0;
 	if(ob->totcol==0) return 0;
@@ -725,7 +739,6 @@ Tex *give_current_texture(Object *ob, int act)
 			mtex= la->mtex[(int)(la->texact)];
 			if(mtex) tex= mtex->tex;
 		}
-		else tex= 0;
 	} else {
 		if(act>ob->totcol) act= ob->totcol;
 		else if(act==0) act= 1;
@@ -738,13 +751,25 @@ Tex *give_current_texture(Object *ob, int act)
 			
 			if(matarar && *matarar) ma= (*matarar)[act-1];
 			else ma= 0;
-			
+		}
+
+		if(ma && ma->use_nodes && ma->nodetree) {
+			node= nodeGetActiveID(ma->nodetree, ID_TE);
+
+			if(node) {
+				tex= (Tex *)node->id;
+				ma= NULL;
+			}
+			else {
+				node= nodeGetActiveID(ma->nodetree, ID_MA);
+				if(node)
+					ma= (Material*)node->id;
+			}
 		}
 		if(ma) {
 			mtex= ma->mtex[(int)(ma->texact)];
 			if(mtex) tex= mtex->tex;
 		}
-		else tex= 0;
 	}
 	
 	return tex;
