@@ -2,17 +2,14 @@
  * Functions for writing avi-format files.
  * Added interface for generic movie support (ton)
  *
- * $Id: writeavi.c 8683 2006-10-26 23:34:44Z blendix $
+ * $Id: writeavi.c 14669 2008-05-04 09:41:15Z campbellbarton $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,7 +27,7 @@
  *
  * Contributor(s): Robert Wenzlaff
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  * 
  */
 
@@ -123,7 +120,7 @@ void makeavistring (RenderData *rd, char *string)
 	if (string==0) return;
 
 	strcpy(string, rd->pic);
-	BLI_convertstringcode(string, G.sce, rd->cfra);
+	BLI_convertstringcode(string, G.sce);
 
 	BLI_make_existing_file(string);
 
@@ -138,7 +135,8 @@ void start_avi(RenderData *rd, int rectx, int recty)
 	int x, y;
 	char name[256];
 	AviFormat format;
-	int quality, framerate;
+	int quality;
+	double framerate;
 	
 	makeavistring(rd, name);
 
@@ -147,7 +145,7 @@ void start_avi(RenderData *rd, int rectx, int recty)
 	y = recty;
 
 	quality= rd->quality;
-	framerate= rd->frs_sec;
+	framerate= (double) rd->frs_sec / (double) rd->frs_sec_base;
 	
 	avi = MEM_mallocN (sizeof(AviMovie), "avimovie");
 
