@@ -20,11 +20,20 @@
 #include <string.h>
 #include <stdio.h>
 
+#if defined(_WIN32) && defined(_DEBUG) && !defined(__MINGW32__) && !defined(__CYGWIN__)
+/* This does not seem necessary or present on MSVC 8, but may be needed in earlier versions? */
+#if _MSC_VER < 1400
 #include <stdint.h>
+#endif
+#endif
+
 #include <stdlib.h>
+
 #include <ffmpeg/avformat.h>
 #include <ffmpeg/avcodec.h>
 #include <ffmpeg/rational.h>
+#include <ffmpeg/swscale.h>
+#include <ffmpeg/opt.h>
 
 #if LIBAVFORMAT_VERSION_INT < (49 << 16)
 #define FFMPEG_OLD_FRAME_RATE 1
@@ -33,6 +42,16 @@
 #define FFMPEG_CODEC_TIME_BASE  1
 #endif
 
+#if LIBAVFORMAT_VERSION_INT >= (52 << 16)
+#define OUTFILE_PB (outfile->pb)
+#else
+#define OUTFILE_PB (&outfile->pb)
+#endif
+
+#if defined(WIN32) && (!(defined snprintf))
+#define snprintf _snprintf
+#endif
+
 #include "BKE_writeffmpeg.h"
 
 #include "MEM_guardedalloc.h"
@@ -40,6 +59,7 @@
 
 #include "BKE_bad_level_calls.h"
 #include "BKE_global.h"
+#include "BKE_idprop.h"
 
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"
@@ -70,6 +90,7 @@ static AVFormatContext* outfile = 0;
 static AVStream* video_stream = 0;
 static AVStream* audio_stream = 0;
 static AVFrame* current_frame = 0;
+static struct SwsContext *img_convert_ctx = 0;
 
 static uint8_t* video_buffer = 0;
 static int video_buffersize = 0;
@@ -132,7 +153,7 @@ static int write_audio_frame(void)
 
 	pkt.stream_index = audio_stream->index;
 	pkt.flags |= PKT_FLAG_KEY;
-	if (av_write_frame(outfile, &pkt) != 0) {
+	if (av_interleaved_write_frame(outfile, &pkt) != 0) {
 		error("Error writing audio packet");
 		return -1;
 	}
@@ -201,6 +222,14 @@ static const char** get_file_extensions(int format)
 		static const char * rv[] = { ".avi", NULL };
 		return rv;
 	}
+	case FFMPEG_FLV: {
+		static const char * rv[] = { ".flv", NULL };
+		return rv;
+	}
+	case FFMPEG_MKV: {
+		static const char * rv[] = { ".mkv", NULL };
+		return rv;
+	}
 	default:
 		return NULL;
 	}
@@ -235,7 +264,7 @@ static void write_video_frame(AVFrame* frame)
 		packet.stream_index = video_stream->index;
 		packet.data = video_buffer;
 		packet.size = outsize;
-		ret = av_write_frame(outfile, &packet);
+		ret = av_interleaved_write_frame(outfile, &packet);
 	} else ret = 0;
 	if (ret != 0) {
 		G.afbreek = 1;
@@ -306,13 +335,83 @@ static AVFrame* generate_video_frame(uint8_t* pixels)
 	}
 
 	if (c->pix_fmt != PIX_FMT_RGBA32) {
-		img_convert((AVPicture*)current_frame, c->pix_fmt, 
-			(AVPicture*)rgb_frame, PIX_FMT_RGBA32, width, height);
+		sws_scale(img_convert_ctx, rgb_frame->data,
+			  rgb_frame->linesize, 0, c->height, 
+			  current_frame->data, current_frame->linesize);
 		delete_picture(rgb_frame);
 	}
 	return current_frame;
 }
 
+static void set_ffmpeg_property_option(AVCodecContext* c, IDProperty * prop)
+{
+	char name[128];
+	char * param;
+	const AVOption * rv = NULL;
+
+	fprintf(stderr, "FFMPEG expert option: %s: ", prop->name);
+
+	strncpy(name, prop->name, 128);
+
+	param = strchr(name, ':');
+
+	if (param) {
+		*param++ = 0;
+	}
+
+	switch(prop->type) {
+	case IDP_STRING:
+		fprintf(stderr, "%s.\n", IDP_String(prop));
+		rv = av_set_string(c, prop->name, IDP_String(prop));
+		break;
+	case IDP_FLOAT:
+		fprintf(stderr, "%g.\n", IDP_Float(prop));
+		rv = av_set_double(c, prop->name, IDP_Float(prop));
+		break;
+	case IDP_INT:
+		fprintf(stderr, "%d.\n", IDP_Int(prop));
+		
+		if (param) {
+			if (IDP_Int(prop)) {
+				rv = av_set_string(c, name, param);
+			} else {
+				return;
+			}
+		} else {
+			rv = av_set_int(c, prop->name, IDP_Int(prop));
+		}
+		break;
+	}
+
+	if (!rv) {
+		fprintf(stderr, "ffmpeg-option not supported: %s! Skipping.\n",
+			prop->name);
+	}
+}
+
+static void set_ffmpeg_properties(AVCodecContext* c, const char * prop_name)
+{
+	IDProperty * prop;
+	void * iter;
+	IDProperty * curr;
+
+	if (!G.scene->r.ffcodecdata.properties) {
+		return;
+	}
+	
+	prop = IDP_GetPropertyFromGroup(
+		G.scene->r.ffcodecdata.properties, (char*) prop_name);
+	if (!prop) {
+		return;
+	}
+
+	iter = IDP_GetGroupIterator(prop);
+
+	while ((curr = IDP_GroupIterNext(iter)) != NULL) {
+		set_ffmpeg_property_option(c, curr);
+	}
+}
+
 /* prepare a video stream for the output file */
 
 static AVStream* alloc_video_stream(int codec_id, AVFormatContext* of,
@@ -341,18 +440,26 @@ static AVStream* alloc_video_stream(int codec_id, AVFormatContext* of,
 	if (ffmpeg_type == FFMPEG_DV && G.scene->r.frs_sec != 25) {
 		c->time_base.den = 2997;
 		c->time_base.num = 100;
-	} else {
+	} else if ((double) ((int) G.scene->r.frs_sec_base) == 
+		   G.scene->r.frs_sec_base) {
 		c->time_base.den = G.scene->r.frs_sec;
-		c->time_base.num = 1;
+		c->time_base.num = (int) G.scene->r.frs_sec_base;
+	} else {
+		c->time_base.den = G.scene->r.frs_sec * 100000;
+		c->time_base.num = ((double) G.scene->r.frs_sec_base) * 100000;
 	}
 #else
 	/* FIXME: Really bad hack (tm) for NTSC support */
 	if (ffmpeg_type == FFMPEG_DV && G.scene->r.frs_sec != 25) {
 		c->frame_rate = 2997;
 		c->frame_rate_base = 100;
-	} else {
+	} else if ((double) ((int) G.scene->r.frs_sec_base) == 
+		   G.scene->r.frs_sec_base) {
 		c->frame_rate = G.scene->r.frs_sec;
-		c->frame_rate_base = 1;
+		c->frame_rate_base = G.scene->r.frs_sec_base;
+	} else {
+		c->frame_rate = G.scene->r.frs_sec * 100000;
+		c->frame_rate_base = ((double) G.scene->r.frs_sec_base)*100000;
 	}
 #endif
 	
@@ -391,13 +498,18 @@ static AVStream* alloc_video_stream(int codec_id, AVFormatContext* of,
 	}
 	
 	/* Determine whether we are encoding interlaced material or not */
-	if (G.scene->r.mode & (1 << 6)) {
+	if (G.scene->r.mode & R_FIELDS) {
 		fprintf(stderr, "Encoding interlaced video\n");
 		c->flags |= CODEC_FLAG_INTERLACED_DCT;
 		c->flags |= CODEC_FLAG_INTERLACED_ME;
-	}	
-	c->sample_aspect_ratio.num = G.scene->r.xasp;
-	c->sample_aspect_ratio.den = G.scene->r.yasp;
+	}
+
+	/* xasp & yasp got float lately... */
+
+	c->sample_aspect_ratio = av_d2q(
+		((double) G.scene->r.xasp / (double) G.scene->r.yasp), 255);
+
+	set_ffmpeg_properties(c, "video");
 	
 	if (avcodec_open(c, codec) < 0) {
 		error("Couldn't initialize codec");
@@ -409,6 +521,13 @@ static AVStream* alloc_video_stream(int codec_id, AVFormatContext* of,
 					     "FFMPEG video buffer");
 	
 	current_frame = alloc_picture(c->pix_fmt, c->width, c->height);
+
+	img_convert_ctx = sws_getContext(c->width, c->height,
+					 PIX_FMT_RGBA32,
+					 c->width, c->height,
+					 c->pix_fmt,
+					 SWS_BICUBIC,
+					 NULL, NULL, NULL);
 	return st;
 }
 
@@ -435,13 +554,23 @@ static AVStream* alloc_audio_stream(int codec_id, AVFormatContext* of)
 		error("Couldn't find a valid audio codec");
 		return NULL;
 	}
+
+	set_ffmpeg_properties(c, "audio");
+
 	if (avcodec_open(c, codec) < 0) {
 		error("Couldn't initialize audio codec");
 		return NULL;
 	}
 
 	/* FIXME: Should be user configurable */
-	audio_outbuf_size = 10000;
+	if (ffmpeg_type == FFMPEG_DV) {
+		/* this is a hack around the poor ffmpeg dv multiplexer. */
+		/* only fixes PAL for now 
+		   (NTSC is a lot more complicated here...)! */
+		audio_outbuf_size = 7680;
+	} else {
+		audio_outbuf_size = 10000;
+	}
 	audio_output_buffer = (uint8_t*)MEM_mallocN(
 		audio_outbuf_size, "FFMPEG audio encoder input buffer");
 
@@ -471,7 +600,7 @@ static AVStream* alloc_audio_stream(int codec_id, AVFormatContext* of)
 }
 /* essential functions -- start, append, end */
 
-static void start_ffmpeg_impl(RenderData *rd, int rectx, int recty)
+void start_ffmpeg_impl(struct RenderData *rd, int rectx, int recty)
 {
 	/* Handle to the output file */
 	AVFormatContext* of;
@@ -557,6 +686,9 @@ static void start_ffmpeg_impl(RenderData *rd, int rectx, int recty)
 	case FFMPEG_XVID:
 		fmt->video_codec = CODEC_ID_XVID;
 		break;
+	case FFMPEG_FLV:
+		fmt->video_codec = CODEC_ID_FLV1;
+		break;
 	case FFMPEG_MPEG4:
 	default:
 		fmt->video_codec = CODEC_ID_MPEG4;
@@ -582,6 +714,9 @@ static void start_ffmpeg_impl(RenderData *rd, int rectx, int recty)
 			return;
 		}
 	}
+	
+	fmt->audio_codec = ffmpeg_audio_codec;
+
 	if (ffmpeg_type == FFMPEG_DV) {
 		fmt->audio_codec = CODEC_ID_PCM_S16LE;
 		if (ffmpeg_multiplex_audio 
@@ -643,7 +778,8 @@ void makeffmpegstring(char* string) {
 	if (!string || !exts) return;
 
 	strcpy(string, G.scene->r.pic);
-	BLI_convertstringcode(string, G.sce, G.scene->r.cfra);
+	BLI_convertstringcode(string, G.sce);
+	BLI_convertstringframe(string, G.scene->r.cfra);
 
 	BLI_make_existing_file(string);
 
@@ -685,24 +821,37 @@ void start_ffmpeg(RenderData *rd, int rectx, int recty)
 
 void end_ffmpeg(void);
 
+static void write_audio_frames()
+{
+	int finished = 0;
+
+	while (ffmpeg_multiplex_audio && !finished) {
+		double a_pts = ((double)audio_stream->pts.val 
+				* audio_stream->time_base.num 
+				/ audio_stream->time_base.den);
+		double v_pts = ((double)video_stream->pts.val 
+				* video_stream->time_base.num 
+				/ video_stream->time_base.den);
+		
+		if (a_pts < v_pts) {
+			write_audio_frame();
+		} else {
+			finished = 1;
+		}
+	}
+}
+
 void append_ffmpeg(int frame, int *pixels, int rectx, int recty) 
 {
 	fprintf(stderr, "Writing frame %i, "
 		"render width=%d, render height=%d\n", frame,
 		rectx, recty);
-	while (ffmpeg_multiplex_audio && 
-	       (((double)audio_stream->pts.val 
-		 * audio_stream->time_base.num / audio_stream->time_base.den)
-		< 
-		((double)video_stream->pts.val 
-		 * video_stream->time_base.num / video_stream->time_base.den)
-		       )) {
-		write_audio_frame();
-	}
+
+	write_audio_frames();
 	write_video_frame(generate_video_frame((unsigned char*) pixels));
 
 	if (ffmpeg_autosplit) {
-		if (url_ftell(&outfile->pb) > FFMPEG_AUTOSPLIT_SIZE) {
+		if (url_ftell(OUTFILE_PB) > FFMPEG_AUTOSPLIT_SIZE) {
 			end_ffmpeg();
 			ffmpeg_autosplit_count++;
 			start_ffmpeg_impl(ffmpeg_renderdata,
@@ -718,6 +867,10 @@ void end_ffmpeg(void)
 	
 	fprintf(stderr, "Closing ffmpeg...\n");
 
+	if (audio_stream) {
+		write_audio_frames();
+	}
+	
 	if (outfile) {
 		av_write_trailer(outfile);
 	}
@@ -745,7 +898,7 @@ void end_ffmpeg(void)
 	}
 	if (outfile && outfile->oformat) {
 		if (!(outfile->oformat->flags & AVFMT_NOFILE)) {
-			url_fclose(&outfile->pb);
+			url_fclose(OUTFILE_PB);
 		}
 	}
 	if (outfile) {
@@ -764,6 +917,11 @@ void end_ffmpeg(void)
 		MEM_freeN(audio_input_buffer);
 		audio_input_buffer = 0;
 	}
+
+	if (img_convert_ctx) {
+		sws_freeContext(img_convert_ctx);
+		img_convert_ctx = 0;
+	}
 }
 #endif
 
