@@ -1,15 +1,12 @@
 /*
- * $Id: BLO_readfile.h 10318 2007-03-21 02:23:28Z khughes $
+ * $Id: BLO_readfile.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  * external readfile function prototypes
  */
 #ifndef BLO_READFILE_H
@@ -38,12 +35,15 @@ extern "C" {
 #endif
 
 struct SpaceFile;
+struct SpaceImaSel;
+struct FileList;
 struct LinkNode;
 struct Main;
 struct UserDef;
 struct bScreen;
 struct Scene;
 struct MemFile;
+struct direntry;
 
 typedef struct BlendHandle	BlendHandle;
 
@@ -191,6 +191,21 @@ BLO_blendhandle_get_datablock_names(
 	BlendHandle *bh, 
 	int ofblocktype);
 
+/**
+ * Gets the previews of all the datablocks in a file
+ * of a certain type (ie. All the scene names in
+ * a file).
+ * 
+ * @param bh The blendhandle to access.
+ * @param ofblocktype The type of names to get.
+ * @return A BLI_linklist of PreviewImage. The PreviewImage links
+ * should be freed with malloc.
+ */
+	struct LinkNode*
+BLO_blendhandle_get_previews(
+	BlendHandle *bh, 
+	int ofblocktype);
+
 /**
  * Gets the names of all the datablock groups in a
  * file. (ie. file contains Scene, Mesh, and Lamp
@@ -219,7 +234,8 @@ BLO_blendhandle_close(
 char *BLO_gethome(void);
 int BLO_has_bfile_extension(char *str);
 void BLO_library_append(struct SpaceFile *sfile, char *dir, int idcode);
-void BLO_script_library_append(BlendHandle *bh, char *dir, char *name, int idcode, short flag, struct Scene *scene);
+void BLO_library_append_(BlendHandle **libfiledata, struct direntry* filelist, int totfile, char *dir, char* file, short flag, int idcode);
+void BLO_script_library_append(BlendHandle **bh, char *dir, char *name, int idcode, short flag, struct Scene *scene);
 
 BlendFileData* blo_read_blendafterruntime(int file, int actualsize, BlendReadError *error_r);
 
