@@ -1,15 +1,12 @@
 /**
- * $Id: readblenentry.c 10602 2007-04-28 16:15:00Z ton $
+ * $Id: readblenentry.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  * .blend file reading entry point
  */
 
@@ -50,6 +47,7 @@
 #include "DNA_space_types.h"
 #include "DNA_userdef_types.h"
 #include "DNA_ID.h"
+#include "DNA_material_types.h"
 
 #include "BKE_utildefines.h" // for ENDB
 
@@ -61,6 +59,7 @@
 #include "BLO_undofile.h"
 
 #include "readfile.h"
+#include "genfile.h"
 
 #include "BLO_readblenfile.h"
 
@@ -218,6 +217,70 @@ LinkNode *BLO_blendhandle_get_datablock_names(BlendHandle *bh, int ofblocktype)
 	return names;
 }
 
+LinkNode *BLO_blendhandle_get_previews(BlendHandle *bh, int ofblocktype) 
+{
+	FileData *fd= (FileData*) bh;
+	LinkNode *previews= NULL;
+	BHead *bhead;
+	int looking=0;
+	int npreviews = 0;
+	PreviewImage* prv = NULL;
+	PreviewImage* new_prv = NULL;
+	
+	for (bhead= blo_firstbhead(fd); bhead; bhead= blo_nextbhead(fd, bhead)) {
+		if (bhead->code==ofblocktype) {
+			ID *id= (ID*) (bhead+1);
+			if ( (GS(id->name) == ID_MA) || (GS(id->name) == ID_TE)) {
+				new_prv = MEM_callocN(sizeof(PreviewImage), "newpreview");
+				BLI_linklist_prepend(&previews, new_prv);
+				looking = 1;
+			}
+		} else if (bhead->code==DATA) {
+			if (looking) {
+				if (bhead->SDNAnr == dna_findstruct_nr(fd->filesdna, "PreviewImage") ) {
+					prv = (PreviewImage*) (bhead+1);
+					npreviews = 0;				
+					memcpy(new_prv, prv, sizeof(PreviewImage));
+					if (prv->rect[0]) {
+						unsigned int *rect = NULL;
+						int rectlen = 0;
+						new_prv->rect[0] = MEM_callocN(new_prv->w[0]*new_prv->h[0]*sizeof(unsigned int), "prvrect");
+						bhead= blo_nextbhead(fd, bhead);
+						rect = (unsigned int*)(bhead+1);
+						rectlen = new_prv->w[0]*new_prv->h[0]*sizeof(unsigned int);
+						memcpy(new_prv->rect[0], rect, bhead->len);					
+					} else {
+						new_prv->rect[0] = NULL;
+					}
+					
+					if (prv->rect[1]) {
+						unsigned int *rect = NULL;
+						int rectlen = 0;
+						new_prv->rect[1] = MEM_callocN(new_prv->w[1]*new_prv->h[1]*sizeof(unsigned int), "prvrect");
+						bhead= blo_nextbhead(fd, bhead);
+						rect = (unsigned int*)(bhead+1);
+						rectlen = new_prv->w[1]*new_prv->h[1]*sizeof(unsigned int);					
+						memcpy(new_prv->rect[1], rect, bhead->len);							
+					} else {
+						new_prv->rect[1] = NULL;
+					}
+				}
+			}
+		} else if (bhead->code==ENDB) {
+			break;
+		} else if (bhead->code==DATA) {
+			/* DATA blocks between IDBlock and Preview */
+		} else {
+			looking = 0;
+			new_prv = NULL;
+			prv = NULL;
+		}
+		
+	}
+	
+	return previews;
+}
+
 LinkNode *BLO_blendhandle_get_linkable_groups(BlendHandle *bh) 
 {
 	FileData *fd= (FileData*) bh;
