@@ -73,17 +73,14 @@
  *     Additionally, it needs the types from the jpeg lib.
  */
 /*
- * $Id: IMB_imbuf.h,v 1.29 2007/04/04 13:18:29 campbellbarton Exp $ 
+ * $Id: IMB_imbuf.h 14444 2008-04-16 22:40:48Z hos $ 
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -101,7 +98,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 #ifndef IMB_IMBUF_H
 #define IMB_IMBUF_H
@@ -158,7 +155,7 @@ struct ImBuf *IMB_testiffname(char *naam,int flags);
  *
  * @attention Defined in readimage.c
  */
-struct ImBuf *IMB_loadiffname(char *naam, int flags);
+struct ImBuf *IMB_loadiffname(const char *naam, int flags);
 
 /**
  *
@@ -258,7 +255,7 @@ int IMB_anim_get_duration(struct anim *anim);
  *
  * @attention Defined in anim.c
  */
-struct anim * IMB_open_anim(char * name, int ib_flags);
+struct anim * IMB_open_anim(const char * name, int ib_flags);
 void IMB_close_anim(struct anim * anim);
 
 /**
@@ -277,6 +274,13 @@ int IMB_anim_get_preseek(struct anim * anim);
 
 struct ImBuf * IMB_anim_absolute(struct anim * anim, int position);
 
+/**
+ *
+ * @attention Defined in anim.c
+ * fetches a define previewframe, usually half way into the movie
+ */
+struct ImBuf * IMB_anim_previewframe(struct anim * anim);
+
 /**
  *
  * @attention Defined in anim.c
@@ -314,7 +318,7 @@ void IMB_antialias(struct ImBuf * ibuf);
  */
 void IMB_filter(struct ImBuf *ibuf);
 void IMB_filterN(struct ImBuf *out, struct ImBuf *in);
-void IMB_filter_extend(struct ImBuf *ibuf);
+void IMB_filter_extend(struct ImBuf *ibuf, char *mask);
 void IMB_makemipmap(struct ImBuf *ibuf, int use_filter);
 
 /**
@@ -395,7 +399,13 @@ void IMB_float_from_rect(struct ImBuf *ibuf);
  * @attention Defined in imageprocess.c
  */
 void IMB_convert_rgba_to_abgr(struct ImBuf *ibuf);
-
+/**
+ *
+ * @attention defined in imageprocess.c
+ */
+void bicubic_interpolation(struct ImBuf *in, struct ImBuf *out, float x, float y, int xout, int yout);
+void neareast_interpolation(struct ImBuf *in, struct ImBuf *out, float u, float v, int xout, int yout);
+void bilinear_interpolation(struct ImBuf *in, struct ImBuf *out, float u, float v, int xout, int yout);
 /**
  * Change the ordering of the color bytes pointed to by rect from
  * rgba to abgr. size * 4 color bytes are reordered.
@@ -511,6 +521,7 @@ extern float rgb_to_bw[4][4];
  *
  * @attention Defined in rotate.c
  */
+void IMB_flipx(struct ImBuf *ibuf);
 void IMB_flipy(struct ImBuf * ibuf);
 
 /**
@@ -531,6 +542,13 @@ void IMB_freezbuffloatImBuf(struct ImBuf * ibuf);
  * @attention Defined in rectop.c
  */
 void IMB_rectfill(struct ImBuf *drect, float col[4]);
+void IMB_rectfill_area(struct ImBuf *ibuf, float *col, int x1, int y1, int x2, int y2);
+
+/* this should not be here, really, we needed it for operating on render data, IMB_rectfill_area calls it */
+void buf_rectfill_area(unsigned char *rect, float *rectf, int width, int height, float *col, int x1, int y1, int x2, int y2);
+
+/* defined in imginfo.c */
+int IMB_imginfo_change_field(struct ImBuf *img, const char *key, const char *field);
 
 /* exported for image tools in blender, to quickly allocate 32 bits rect */
 short imb_addrectImBuf(struct ImBuf * ibuf);
