@@ -1,14 +1,11 @@
 /**
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,10 +23,10 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  * rotate.c
  *
- * $Id: rotate.c 6568 2006-01-28 18:33:28Z hos $
+ * $Id: rotate.c 14444 2008-04-16 22:40:48Z hos $
  */
 
 #include "BLI_blenlib.h"
@@ -43,46 +40,85 @@
 
 void IMB_flipy(struct ImBuf * ibuf)
 {
-	short x, y;
-	unsigned int *top, *bottom, do_float=0, *line;
-	float *topf=NULL, *bottomf=NULL, *linef=NULL;
+	int x, y;
 
 	if (ibuf == NULL) return;
-	if (ibuf->rect == NULL) return;
-	
-	if (ibuf->rect_float) do_float =1;
 
-	x = ibuf->x;
-	y = ibuf->y;
+	if (ibuf->rect) {
+		unsigned int *top, *bottom, *line;
 
-	top = ibuf->rect;
-	bottom = top + ((y-1) * x);
-	line= MEM_mallocN(x*sizeof(int), "linebuf");
+		x = ibuf->x;
+		y = ibuf->y;
+
+		top = ibuf->rect;
+		bottom = top + ((y-1) * x);
+		line= MEM_mallocN(x*sizeof(int), "linebuf");
 	
-	if (do_float) {
+		y >>= 1;
+
+		for(;y>0;y--) {
+			memcpy(line, top, x*sizeof(int));
+			memcpy(top, bottom, x*sizeof(int));
+			memcpy(bottom, line, x*sizeof(int));
+			bottom -= x;
+			top+= x;
+		}
+
+		MEM_freeN(line);
+	}
+
+	if (ibuf->rect_float) {
+		float *topf=NULL, *bottomf=NULL, *linef=NULL;
+
+		x = ibuf->x;
+		y = ibuf->y;
+
 		topf= ibuf->rect_float;
 		bottomf = topf + 4*((y-1) * x);
 		linef= MEM_mallocN(4*x*sizeof(float), "linebuff");
-	}
-	y >>= 1;
 
-	for(;y>0;y--) {
-		
-		memcpy(line, top, x*sizeof(int));
-		memcpy(top, bottom, x*sizeof(int));
-		memcpy(bottom, line, x*sizeof(int));
-		bottom -= x;
-		top+= x;
-		
-		if(do_float) {
+		y >>= 1;
+
+		for(;y>0;y--) {
 			memcpy(linef, topf, 4*x*sizeof(float));
 			memcpy(topf, bottomf, 4*x*sizeof(float));
 			memcpy(bottomf, linef, 4*x*sizeof(float));
 			bottomf -= 4*x;
 			topf+= 4*x;
 		}
+
+		MEM_freeN(linef);
+	}
+}
+
+void IMB_flipx(struct ImBuf * ibuf)
+{
+	short x, y, xr, xl, yi;
+	unsigned int px;
+	float px_f[4];
+	
+	if (ibuf == NULL) return;
+
+	x = ibuf->x;
+	y = ibuf->y;
+
+	if (ibuf->rect) {
+		for(yi=y-1;yi>=0;yi--) {
+			for(xr=x-1, xl=0; xr>=xl; xr--, xl++) {
+				px = ibuf->rect[(x*yi)+xr];
+				ibuf->rect[(x*yi)+xr] =	ibuf->rect[(x*yi)+xl];
+				ibuf->rect[(x*yi)+xl] =	px;		
+			}
+		}
 	}
 	
-	MEM_freeN(line);
-	if(linef) MEM_freeN(linef);
+	if (ibuf->rect_float) {
+		for(yi=y-1;yi>=0;yi--) {
+			for(xr=x-1, xl=0; xr>=xl; xr--, xl++) {
+				memcpy(&px_f, &ibuf->rect_float[((x*yi)+xr)*4], 4*sizeof(float));
+				memcpy(&ibuf->rect_float[((x*yi)+xr)*4], &ibuf->rect_float[((x*yi)+xl)*4], 4*sizeof(float));
+				memcpy(&ibuf->rect_float[((x*yi)+xl)*4], &px_f, 4*sizeof(float));
+			}
+		}
+	}
 }
