@@ -1,15 +1,12 @@
 /**
- * $Id: BDR_drawmesh.h 9072 2006-11-26 23:14:52Z blendix $
+ * $Id: BDR_drawmesh.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifndef BDR_DRAWMESH_H
@@ -36,6 +33,7 @@
 struct Image;
 struct MTFace;
 struct Object;
+struct DerivedMesh;
 struct Mesh;
 struct EdgeHash;
 
@@ -75,7 +73,7 @@ int set_tpage(struct MTFace *tface);
 void texpaint_enable_mipmap(void);
 void texpaint_disable_mipmap(void);
 
-void draw_tface_mesh(struct Object *ob, struct Mesh *me, int dt);
+void draw_mesh_textured(struct Object *ob, struct DerivedMesh *dm, int facesel);
 struct EdgeHash *get_tface_mesh_marked_edge_info(struct Mesh *me);
 void init_realtime_GL(void); 
 
