@@ -1,15 +1,12 @@
 /**
- * $Id: BDR_editcurve.h 8765 2006-11-06 18:20:56Z ton $
+ * $Id: BDR_editcurve.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifndef BDR_EDITCURVE_H
@@ -78,11 +75,13 @@ void extrude_nurb(void);
 void makecyclicNurb(void);
 void selectconnected_nurb(void);
 void selectrow_nurb(void);
-void selectends_nurb(int selFirst);
+void selectend_nurb(short selfirst, short doswap, short selstatus);
 void select_more_nurb(void);
 void select_less_nurb(void);
 void select_next_nurb(void);
 void select_prev_nurb(void);
+void select_random_nurb(void);
+void select_every_nth_nurb(void);
 void adduplicate_nurb(void);
 void delNurb(void);
 void nurb_set_smooth(short event);
@@ -94,8 +93,10 @@ void add_primitiveNurb(int type);
 void clear_tilt(void);
 void clever_numbuts_curve(void);         
 int bezt_compare (const void *e1, const void *e2);
+void setweightNurb( void );
+void setradiusNurb( void );
+void smoothradiusNurb( void );
 
 extern void undo_push_curve(char *name);
 
 #endif  /*  BDR_EDITCURVE_H */
-
