@@ -1,15 +1,12 @@
 /**
- * $Id: BDR_editmball.h 3856 2005-02-21 10:40:30Z jiri $
+ * $Id: BDR_editmball.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifndef BDR_EDITMBALL_H
@@ -42,6 +39,8 @@ void load_editMball(void);
  */
 void add_primitiveMball(int);
 void deselectall_mball(void);
+void selectinverse_mball(void);
+void selectrandom_mball(void);
 void mouse_mball(void);
 void adduplicate_mball(void);
 void delete_mball(void); 
