@@ -1,15 +1,12 @@
 /**
- * $Id: BDR_editobject.h 10571 2007-04-22 22:08:19Z campbellbarton $
+ * $Id: BDR_editobject.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifndef BDR_EDITOBJECT_H
@@ -65,6 +62,8 @@ void exit_editmode(int flag);
 void check_editmode(int type);
 void enter_editmode(int wc);
 
+void exit_paint_modes(void);
+
 void docenter(int centermode);
 void docenter_new(void);
 void docenter_cursor(void);
@@ -77,6 +76,8 @@ void link_to_scene(unsigned short nr);
 void make_links_menu(void);
 void make_links(short event);
 void make_duplilist_real(void);
+void apply_objects_locrot(void);
+void apply_objects_visual_tx(void);
 void apply_object(void);
 
 /* old transform */
@@ -93,6 +94,7 @@ void single_object_users(int flag);
 void new_id_matar(struct Material **matar, int totcol);
 void single_obdata_users(int flag);
 void single_mat_users(int flag);
+void single_ipo_users(int flag);
 void do_single_tex_user(struct Tex **from);
 void single_tex_users_expand(void);
 void single_mat_users_expand(void);
@@ -107,16 +109,20 @@ void set_ob_ipoflags(void);
 void select_select_keys(void);
 int vergbaseco(const void *a1, const void *a2);
 void auto_timeoffs(void);
+void ofs_timeoffs(void);
+void rand_timeoffs(void);
 void texspace_edit(void);
 void flip_subdivison(int);
 void mirrormenu(void);
 void hookmenu(void); /* object mode hook menu */
 
 
-void add_hook(void);
+void add_hook_menu(void);
+void add_hook(int mode);
 void hook_select(struct HookModifierData *hmd);
 int hook_getIndexArray(int *tot, int **indexar, char *name, float *cent_r);
 
+int object_is_libdata(struct Object *ob);
 int object_data_is_libdata(struct Object *ob);	
 void hide_objects(int select);
 void show_objects(void);
