@@ -1,15 +1,12 @@
 /**
- * $Id: BIF_butspace.h 9376 2006-12-20 17:57:56Z ton $
+ * $Id: BIF_butspace.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 #ifndef BIF_BUTSPACE_H
 #define BIF_BUTSPACE_H
@@ -83,11 +80,13 @@ extern void validate_editbonebutton_cb(void *bonev, void *namev);
 
 #define TAB_OBJECT_OBJECT	0
 #define TAB_OBJECT_PHYSICS 	1
+#define TAB_OBJECT_PARTICLE	2
 
 #define TAB_SCENE_RENDER	0
-#define TAB_SCENE_WORLD		1
+#define TAB_SCENE_WORLD     	1
 #define TAB_SCENE_ANIM		2
 #define TAB_SCENE_SOUND		3
+#define TAB_SCENE_SEQUENCER	4
 
 
 /* buts->scaflag */		
@@ -102,6 +101,38 @@ extern void validate_editbonebutton_cb(void *bonev, void *namev);
 #define BUTS_ACT_LINK		256
 
 
+/* buttons grid */
+#define PANELX		320
+#define PANELY		0
+#define PANELW		318
+#define PANELH		204
+  
+#define BUTW1		300
+#define BUTW2		145
+#define BUTW3		93
+#define BUTW4		67
+#define ICONBUTW	20
+#define BUTH		22
+ 
+#define YSPACE		6
+#define XSPACE		10
+#define PANEL_YMAX	210
+#define PANEL_XMAX	310
+ 
+#define X1CLM1		10
+ 
+#define X2CLM1		X1CLM1
+#define X2CLM2		165
+ 
+#define X3CLM1		X1CLM1
+#define X3CLM2		113
+#define X3CLM3		217
+
+#define X4CLM1		X1CLM1
+#define X4CLM2		77
+#define X4CLM3		165
+#define X4CLM4		232
+ 
 
 #endif
 
