@@ -1,15 +1,12 @@
 /**
- * $Id: BIF_drawtext.h,v 1.8 2005/05/13 16:11:27 ton Exp $
+ * $Id: BIF_drawtext.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifndef BIF_DRAWTEXT_H
@@ -47,7 +44,7 @@ void add_text_fs(char *file);
 void free_txt_data(void);
 void pop_space_text(struct SpaceText *st);
 
-void get_format_string(void);
+void get_format_string(struct SpaceText *st);
 void do_brackets(void);
 
 #endif
