@@ -1,15 +1,12 @@
 /**
- * $Id: BIF_editarmature.h 10455 2007-04-04 13:18:41Z campbellbarton $
+ * $Id: BIF_editarmature.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 #ifndef BIF_EDITARMATURE_H
 #define BIF_EDITARMATURE_H
@@ -84,12 +81,16 @@ void	delete_armature(void);
 void	deselectall_armature(int toggle, int doundo);
 void	deselectall_posearmature (struct Object *ob, int test, int doundo);
 int		draw_armature(struct Base *base, int dt);
+
 void	extrude_armature(int forked);
-void	subdivide_armature(void);
+void	subdivide_armature(int numcuts);
+void	fill_bones_armature(void);
+void	merge_armature(void);
 
 void	free_editArmature(void);
 
 int		join_armature(void);
+void 	separate_armature(void);
 void	load_editArmature(void);
 
 void	make_bone_parent(void);
@@ -101,26 +102,32 @@ void	make_trans_bones (char mode);
 
 int		do_pose_selectbuffer(struct Base *base, unsigned int *buffer, short hits);
 
+void generateSkeleton(void);
+
 void	mouse_armature(void);
 void	remake_editArmature(void);
 void	selectconnected_armature(void);
 void	selectconnected_posearmature(void);
 void	select_bone_parent(void);
-void    unique_editbone_name (struct ListBase *ebones, char* name);
+void	setflag_armature(short mode);
+void    unique_editbone_name (struct ListBase *ebones, char *name);
 
-void	auto_align_armature(void);
-void	create_vgroups_from_armature(Object *ob, Object *par);
+void	auto_align_armature(short mode);
+
+void	create_vgroups_from_armature(struct Object *ob, struct Object *par);
+void 	add_verts_to_dgroups(struct Object *ob, struct Object *par, int heat, int mirror);
 
 void	hide_selected_pose_bones(void);
 void	hide_unselected_pose_bones(void);
 void	show_all_pose_bones(void);
 
-int		bone_looper(Object *ob, struct Bone *bone, void *data,
-				int (*bone_func)(Object *, struct Bone *, void *));
+int		bone_looper(struct Object *ob, struct Bone *bone, void *data,
+				int (*bone_func)(struct Object *, struct Bone *, void *));
 
 void	undo_push_armature(char *name);
 void	armature_bone_rename(struct bArmature *arm, char *oldname, char *newname);
 void	armature_flip_names(void);
+void 	armature_autoside_names(short axis);
 EditBone *armature_bone_get_mirrored(EditBone *ebo);
 void	transform_armature_mirror_update(void);
 
@@ -137,3 +144,4 @@ void	show_all_armature_bones(void);
 
 #endif
 
+
