@@ -1,15 +1,12 @@
 /**
- * $Id: BIF_editdeform.h 10590 2007-04-27 11:16:35Z aligorith $
+ * $Id: BIF_editdeform.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifndef BIF_DEFORM_H
@@ -50,6 +47,7 @@ void del_defgroup (struct Object *ob);
 void duplicate_defgroup ( struct Object *ob );
 void assign_verts_defgroup (void);
 void remove_verts_defgroup (int allverts);
+void remove_verts_defgroups (int allverts);
 void sel_verts_defgroup (int select);
 
 struct MDeformWeight *get_defweight (struct MDeformVert *dv, int defgroup);
@@ -61,11 +59,16 @@ void add_vert_to_defgroup (struct Object *ob, struct bDeformGroup *dg,
 						   int assignmode);
 void remove_vert_defgroup (struct Object *ob, struct bDeformGroup  *dg, 
 						   int vertnum);
+float get_vert_defgroup	(struct Object *ob, struct bDeformGroup  *dg,
+                         int vertnum);
 void create_dverts(ID *id);
 
 void vertexgroup_select_by_name(struct Object *ob, char *name);
 
 extern void object_apply_deform(struct Object *ob);
 
+void vgroup_assign_with_menu(void);
+void vgroup_operation_with_menu(void);
+
 #endif
 
