@@ -1,15 +1,12 @@
 /**
- * $Id: BIF_interface.h,v 1.43 2007/04/04 13:18:29 campbellbarton Exp $
+ * $Id: BIF_interface.h 14742 2008-05-08 08:59:36Z aligorith $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifndef BIF_INTERFACE_H
@@ -36,6 +33,7 @@
 struct ID;
 struct ListBase;
 struct ScrArea;
+struct AutoComplete;
 
 /* uiBlock->dt */
 #define UI_EMBOSS		0	/* use one of the themes for drawing */
@@ -165,6 +163,8 @@ struct ScrArea;
 #define BUT_CURVE (32<<9)
 #define BUT_TOGDUAL (33<<9)
 #define ICONTOGN (34<<9)
+#define FTPREVIEW (35<<9)
+#define NUMABS	(36<<9)
 #define BUTTYPE	(63<<9)
 
 
@@ -186,7 +186,7 @@ void uiGetMouse(int win, short *adr);
 void uiComposeLinks(uiBlock *block);
 void uiSetButLock(int val, char *lockstr);
 void uiClearButLock(void);
-int uiDoBlocks(struct ListBase *lb, int event);
+int uiDoBlocks(struct ListBase *lb, int event, int movemouse_quit);
 void uiSetCurFont(uiBlock *block, int index);
 void uiDefFont(unsigned int index, void *xl, void *large, void *medium, void *small);
 void uiFreeBlock(uiBlock *block);
@@ -319,5 +319,11 @@ extern void *uiSetCurFont_ext(float aspect);
 
 void shade_buttons_change_3d(void);
 
+typedef struct AutoComplete AutoComplete;
+
+AutoComplete *autocomplete_begin(char *startname, int maxlen);
+void autocomplete_do_name(AutoComplete *autocpl, const char *name);
+void autocomplete_end(AutoComplete *autocpl, char *autoname);
+
 #endif /*  BIF_INTERFACE_H */
 
