@@ -1,15 +1,12 @@
 /**
- * $Id: BIF_language.h,v 1.7 2007/03/15 12:34:44 sirdude Exp $
+ * $Id: BIF_language.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,12 +24,14 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifndef BIF_LANGUAGE_H
 #define BIF_LANGUAGE_H
 
+#include "DNA_vec_types.h"
+
 struct BMF_Font;
 
 int  read_languagefile(void);		/* usiblender.c */
@@ -47,6 +46,7 @@ char *fontsize_pup(void);
 
 int BIF_DrawString(struct BMF_Font* font, char *str, int translate);
 float BIF_GetStringWidth(struct BMF_Font* font, char *str, int translate);
+void BIF_GetBoundingBox(struct BMF_Font* font, char* str, int translate, rctf* bbox);
 
 void BIF_RasterPos(float x, float y);
 void BIF_SetScale(float aspect);
