@@ -1,15 +1,12 @@
 /**
- * $Id: BIF_resources.h 9653 2007-01-07 19:33:00Z theeth $
+ * $Id: BIF_resources.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  * 
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifndef BIF_RESOURCES_H
@@ -58,7 +55,7 @@ typedef enum {
 	ICON_SPACE4,
 	ICON_TRIA_LEFT,
 	ICON_TRIA_UP,
-	ICON_BLANK3,
+	ICON_FONTPREVIEW,
 	ICON_BLANK4,
 	ICON_BLANK5,
 	ICON_BLANK6,
@@ -66,16 +63,16 @@ typedef enum {
 	ICON_ORTHO,
 	ICON_PERSP,
 	ICON_CAMERA,
-	ICON_EFFECTS,
+	ICON_PARTICLES,
 	ICON_BBOX,
 	ICON_WIRE,
 	ICON_SOLID,
 	ICON_SMOOTH,
 	ICON_POTATO,
 	ICON_MARKER_HLT,
-	ICON_NORMALVIEW,
-	ICON_LOCALVIEW,
-	ICON_UNUSEDVIEW,
+	ICON_PMARKER_ACT,
+	ICON_PMARKER_SEL,
+	ICON_PMARKER,
 	ICON_VIEWZOOM,
 	ICON_SORTALPHA,
 	ICON_SORTTIME,
@@ -84,10 +81,10 @@ typedef enum {
 	ICON_SHORTDISPLAY,
 	ICON_TRIA_DOWN,
 	ICON_TRIA_RIGHT,
-	ICON_BLANK7,
-	ICON_BLANK8,
-	ICON_BLANK9,
-	ICON_BLANK10,
+	ICON_NDOF_TURN,
+	ICON_NDOF_FLY,
+	ICON_NDOF_TRANS,
+	ICON_NDOF_DOM,
 
 	ICON_VIEW_AXIS_ALL,
 	ICON_VIEW_AXIS_NONE,
@@ -123,9 +120,9 @@ typedef enum {
 	ICON_AXIS_SIDE,
 	ICON_AXIS_FRONT,
 	ICON_AXIS_TOP,
-	ICON_DRAW_UVFACES,
-	ICON_STICKY_UVS,
-	ICON_STICKY2_UVS,
+	ICON_STICKY_UVS_LOC,
+	ICON_STICKY_UVS_DISABLE,
+	ICON_STICKY_UVS_VERT,
 	ICON_PREV_KEYFRAME,
 	ICON_NEXT_KEYFRAME,
 	ICON_ENVMAP,
@@ -164,7 +161,7 @@ typedef enum {
 	ICON_FACESEL_DEHLT,
 	ICON_FACESEL_HLT,
 	ICON_EDIT_DEHLT,
-	ICON_BLANK19,
+	ICON_BOOKMARKS,
 	ICON_BLANK20,
 	ICON_BLANK21,
 	ICON_BLANK22,
@@ -268,7 +265,7 @@ typedef enum {
 	ICON_EYEDROPPER,
 	ICON_WINDOW_WINDOW,
 	ICON_PANEL_CLOSE,
-	ICON_BLANK35,
+	ICON_PHYSICS,
 	ICON_BLANK36,
 	ICON_BLANK37,
 	ICON_BLANK38,
@@ -317,13 +314,13 @@ typedef enum {
 	ICON_HOME,
 	ICON_CLIPUV_DEHLT,
 	ICON_CLIPUV_HLT,
-	ICON_SOME_WACKY_VERTS_AND_LINES,
-	ICON_A_WACKY_VERT_AND_SOME_LINES,
+	ICON_BLANK2,
+	ICON_BLANK3,
 	ICON_VPAINT_COL,
 	ICON_RESTRICT_SELECT_OFF,
 	ICON_RESTRICT_SELECT_ON,
-	ICON_BLANK45,
-	ICON_BLANK46,
+	ICON_MUTE_IPO_OFF,
+	ICON_MUTE_IPO_ON,
 	
 	ICON_MAN_TRANS,
 	ICON_MAN_ROT,
@@ -494,6 +491,7 @@ enum {
 	TH_NORMAL,
 	TH_FACE_DOT,
 	TH_FACEDOT_SIZE,
+	TH_CFRAME,
 
 	TH_SYNTAX_B,
 	TH_SYNTAX_V,
@@ -525,6 +523,7 @@ enum {
 	TH_SEQ_META,
 	
 	TH_EDGE_SHARP,
+	TH_EDITMESH_ACTIVE,
 };
 /* XXX WARNING: previous is saved in file, so do not change order! */
 
