@@ -1,15 +1,12 @@
 /**
- * $Id: BIF_space.h,v 1.42 2006/12/29 04:46:46 broken Exp $
+ * $Id: BIF_space.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifndef BIF_SPACE_H
@@ -55,6 +52,7 @@ struct SpaceOops;
 #define VIEW3D_HANDLER_OBJECT		3
 #define VIEW3D_HANDLER_PREVIEW		4
 #define VIEW3D_HANDLER_MULTIRES         5
+#define VIEW3D_HANDLER_TRANSFORM	6
 
 /* ipo handler codes */
 #define IPO_HANDLER_PROPERTIES	20
@@ -65,6 +63,8 @@ struct SpaceOops;
 #define IMAGE_HANDLER_CURVES		32
 #define IMAGE_HANDLER_PREVIEW		33
 #define IMAGE_HANDLER_GAME_PROPERTIES	34
+#define IMAGE_HANDLER_VIEW_PROPERTIES	35
+/*#define IMAGE_HANDLER_TRANSFORM_PROPERTIES	36*/
 
 /* action handler codes */
 #define ACTION_HANDLER_PROPERTIES	40
@@ -75,6 +75,9 @@ struct SpaceOops;
 /* sequence handler codes */
 #define SEQ_HANDLER_PROPERTIES	60
 
+/* imasel handler codes */
+#define IMASEL_HANDLER_IMAGE	70
+
 /* theme codes */
 #define B_ADD_THEME 	3301
 #define B_DEL_THEME 	3302
@@ -89,6 +92,7 @@ struct SpaceOops;
 #define B_RECALCLIGHT 	3310
 
 
+void	scrarea_do_winprefetchdraw	(struct ScrArea *sa);
 void	scrarea_do_windraw		(struct ScrArea *sa);
 void	scrarea_do_winchange	(struct ScrArea *sa);
 void	scrarea_do_winhandle	(struct ScrArea *sa, struct BWinEvent *evt);
@@ -114,6 +118,7 @@ extern		 void force_draw_plus(int type, int header);
 extern       void freespacelist(struct ScrArea *sa);
 extern       void handle_view3d_around(void);
 extern       void handle_view3d_lock(void);
+extern		 void handle_view_middlemouse(void);
 extern       void init_v2d_oops(struct ScrArea *, struct SpaceOops *);
 extern       void initipo(struct ScrArea *sa);
 extern       void newspace(struct ScrArea *sa, int type);
