@@ -1,15 +1,12 @@
 /**
- * $Id: BIF_transform.h,v 1.43 2007/03/24 13:38:50 theeth Exp $
+ * $Id: BIF_transform.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifndef BIF_TRANSFORM_H
@@ -55,6 +52,14 @@
 #define TFM_BONE_ENVELOPE	16
 #define TFM_CURVE_SHRINKFATTEN		17
 #define TFM_BONE_ROLL		18
+#define TFM_TIME_TRANSLATE	19	
+#define TFM_TIME_SLIDE		20
+#define	TFM_TIME_SCALE		21
+#define TFM_TIME_EXTEND		22
+#define TFM_BAKE_TIME		23
+#define TFM_BEVEL			24
+#define TFM_BWEIGHT			25
+#define TFM_ALIGN			26
 
 /* TRANSFORM CONTEXTS */
 #define CTX_NONE			0
@@ -62,10 +67,14 @@
 #define CTX_EDGE			2
 #define CTX_NO_PET			4
 #define CTX_TWEAK			8
+#define CTX_NO_MIRROR		16
+#define CTX_AUTOCONFIRM		32
+#define CTX_BMESH			64
+#define CTX_NDOF			128
 
 void initTransform(int mode, int context);
 void Transform(void);
-void Mirror(short mode);
+void NDofTransform();
 
 /* Standalone call to get the transformation center corresponding to the current situation
  * returns 1 if successful, 0 otherwise (usually means there's no selection)
@@ -75,15 +84,34 @@ int calculateTransformCenter(int centerMode, float *vec);
 
 struct TransInfo;
 struct ScrArea;
+struct Base;
+struct Scene;
 
 struct TransInfo * BIF_GetTransInfo(void);
 void BIF_setSingleAxisConstraint(float vec[3], char *text);
 void BIF_setDualAxisConstraint(float vec1[3], float vec2[3], char *text);
 void BIF_setLocalAxisConstraint(char axis, char *text);
 void BIF_setLocalLockConstraint(char axis, char *text);
+
+int BIF_snappingSupported(void);
+
+struct TransformOrientation;
+
+void BIF_clearTransformOrientation(void);
+void BIF_removeTransformOrientation(struct TransformOrientation *ts);
+void BIF_manageTransformOrientation(int confirm, int set);
+int BIF_menuselectTransformOrientation(void);
+void BIF_selectTransformOrientation(struct TransformOrientation *ts);
+void BIF_selectTransformOrientationFromIndex(int index);
+
+char * BIF_menustringTransformOrientation(char *title); /* the returned value was allocated and needs to be freed after use */
+int BIF_countTransformOrientation();
+
+/* Drawing callbacks */
 void BIF_drawConstraint(void);
 void BIF_drawPropCircle(void);
 void BIF_drawSnap(void);
+
 void BIF_getPropCenter(float *center);
 
 void BIF_TransformSetUndo(char *str);
