@@ -9,17 +9,14 @@
  *   mouse / timer / window: until 0x020
  *   custom codes: 0x4...
  * 
- * $Id: mydevice.h 6855 2006-02-19 14:55:16Z ton $
+ * $Id: mydevice.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -37,7 +34,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 /* MOUSE : 0x00x */
@@ -70,6 +67,10 @@
 #define WINQUIT			0x018	/* signal from user that app is to go away */
 #define Q_FIRSTTIME		0x019	/* on startup */
 
+/* N-degre of freedom device : 500 */
+#define NDOFMOTION 500
+#define NDOFBUTTON 501
+
 /* standard keyboard */
 
 #define AKEY		'a'
@@ -253,6 +254,8 @@
 #define REDRAWANIM			0x4037
 #define REDRAWNODE			0x4038
 #define RECALC_COMPOSITE	0x4039
+#define REDRAWMARKER		0x4040 /* all views that display markers */
+#define REDRAWVIEW3D_IMAGE	0x4041
 
 #endif	/* !__MYDEVICE_H__ */
 
