@@ -1,15 +1,12 @@
 /**
- * $Id: transform.h 10455 2007-04-04 13:18:41Z campbellbarton $
+ * $Id: transform.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifndef TRANSFORM_H
@@ -44,7 +41,16 @@ struct NumInput;
 struct Object;
 struct View3D;
 struct ScrArea;
+struct bPose;
+struct bConstraint;
+struct BezTriple;
 
+typedef struct NDofInput {
+	int		flag;
+	int		axis;
+	float	fval[7];
+	float	factor[3];
+} NDofInput;
 
 typedef struct NumInput {
     short  idx;
@@ -120,7 +126,7 @@ typedef struct TransDataExtension {
     float  iquat[4];	 /* Initial rotation quaternion                                                    */
     float *size;         /* Size of the data to transform (Faculative)                                     */
     float  isize[3];	 /* Initial size                                                                   */
-	float  obmat[3][3];	 /* Object matrix */  
+	float  obmat[4][4];	 /* Object matrix */  
 } TransDataExtension;
 
 typedef struct TransData2D {
@@ -128,6 +134,13 @@ typedef struct TransData2D {
 	float *loc2d;		/* Pointer to real 2d location of data */
 } TransData2D;
 
+/* we need to store 2 handles for each transdata incase the other handle wasnt selected */
+typedef struct TransDataCurveHandleFlags {
+	short ih1, ih2;
+	short *h1, *h2;
+} TransDataCurveHandleFlags;
+
+
 typedef struct TransData {
 	float  dist;         /* Distance needed to affect element (for Proportionnal Editing)                  */
 	float  rdist;        /* Distance to the nearest element (for Proportionnal Editing)                    */
@@ -141,8 +154,10 @@ typedef struct TransData {
     float  smtx[3][3];   /* Transformation matrix from global space to data space                          */
 	float  axismtx[3][3];/* Axis orientation matrix of the data                                            */
 	struct Object *ob;
+	struct bConstraint *con;	/* for objects/bones, the first constraint in its constraint stack */
 	TransDataExtension *ext;	/* for objects, poses. 1 single malloc per TransInfo! */
 	TransDataIpokey *tdi;		/* for objects, ipo keys. per transdata a malloc */
+	TransDataCurveHandleFlags *hdata; /* for curves, stores handle flags for modification/cancel */
 	void *tdmir;		 /* mirrored element pointer, in editmode mesh to EditVert */
     short  flag;         /* Various flags */
 	short  protectflag;	 /* If set, copy of Object or PoseChannel protection */
@@ -160,6 +175,8 @@ typedef struct TransInfo {
     float       fac;            /* factor for distance based transform  */
     int       (*transform)(struct TransInfo *, short *);
                                 /* transform function pointer           */
+	int       (*handleEvent)(struct TransInfo *, unsigned short event, short val);
+								/* event handler function pointer  RETURN 1 if redraw is needed */
     int         total;          /* total number of transformed data     */
     TransData  *data;           /* transformed data (array)             */
 	TransDataExtension *ext;	/* transformed data extension (array)   */
@@ -167,6 +184,7 @@ typedef struct TransInfo {
     TransCon    con;            /* transformed constraint               */
     TransSnap	tsnap;
     NumInput    num;            /* numerical input                      */
+    NDofInput   ndof;           /* ndof input                           */
     char        redraw;         /* redraw flag                          */
 	float		propsize;		/* proportional circle radius           */
 	char		proptext[20];	/* proportional falloff text			*/
@@ -193,6 +211,8 @@ typedef struct TransInfo {
 	char		spacename[32];	/* name of the current space				*/
 	
 	struct Object *poseobj;		/* if t->flag & T_POSE, this denotes pose object */
+	
+	void       *customData;		/* Per Transform custom data */
 } TransInfo;
 
 
@@ -205,38 +225,49 @@ typedef struct TransInfo {
 #define NUM_NO_FRACTION		16
 #define	NUM_AFFECT_ALL		32
 
+/* NDOFINPUT FLAGS */
+#define NDOF_INIT			1
+
 /* transinfo->state */
 #define TRANS_RUNNING	0
 #define TRANS_CONFIRM	1
 #define TRANS_CANCEL	2
 
 /* transinfo->flag */
-#define T_OBJECT		1
-#define T_EDIT			2
-#define T_POSE			4
-#define T_TEXTURE		8
-#define T_CAMERA		16
+#define T_OBJECT		(1 << 0)
+#define T_EDIT			(1 << 1)
+#define T_POSE			(1 << 2)
+#define T_TEXTURE		(1 << 3)
+#define T_CAMERA		(1 << 4)
 		// when shift pressed, higher resolution transform. cannot rely on G.qual, need event!
-#define T_SHIFT_MOD		32
+#define T_SHIFT_MOD		(1 << 5)
 	 	// trans on points, having no rotation/scale 
-#define T_POINTS		64
+#define T_POINTS		(1 << 6)
 		// for manipulator exceptions, like scaling using center point, drawing help lines
-#define T_USES_MANIPULATOR	128
+#define T_USES_MANIPULATOR	(1 << 7)
 
-/* restrictions flags */
-#define T_ALL_RESTRICTIONS	(256|512|1024)
-#define T_NO_CONSTRAINT		256
-#define T_NULL_ONE			512
-#define T_NO_ZERO			1024
+	/* restrictions flags */
+#define T_ALL_RESTRICTIONS	((1 << 8)|(1 << 9)|(1 << 10))
+#define T_NO_CONSTRAINT		(1 << 8)
+#define T_NULL_ONE			(1 << 9)
+#define T_NO_ZERO			(1 << 10)
 
-#define T_PROP_EDIT			2048
-#define T_PROP_CONNECTED	4096
+#define T_PROP_EDIT			(1 << 11)
+#define T_PROP_CONNECTED	(1 << 12)
 
-/* if MMB is pressed or not */
-#define	T_MMB_PRESSED		8192
-#define T_V3D_ALIGN			16384
-#define T_2D_EDIT			32768 /* for 2d views like uv or ipo */
-#define T_CLIP_UV			65536
+	/* if MMB is pressed or not */
+#define	T_MMB_PRESSED		(1 << 13)
+
+#define T_V3D_ALIGN			(1 << 14)
+	/* for 2d views like uv or ipo */
+#define T_2D_EDIT			(1 << 15) 
+#define T_CLIP_UV			(1 << 16)
+
+#define T_FREE_CUSTOMDATA	(1 << 17)
+	/* auto-ik is on */
+#define T_AUTOIK			(1 << 18)
+
+/* ******************************************************************************** */
 
 /* transinfo->con->mode */
 #define CON_APPLY		1
@@ -250,14 +281,20 @@ typedef struct TransInfo {
 
 /* transdata->flag */
 #define TD_SELECTED			1
-#define	TD_NOACTION			2
-#define	TD_USEQUAT			4
-#define TD_NOTCONNECTED		8
-#define TD_SINGLESIZE		16	/* used for scaling of MetaElem->rad */
+#define TD_ACTIVE			(1 << 1)
+#define	TD_NOACTION			(1 << 2)
+#define	TD_USEQUAT			(1 << 3)
+#define TD_NOTCONNECTED		(1 << 4)
+#define TD_SINGLESIZE		(1 << 5)	/* used for scaling of MetaElem->rad */
 #ifdef WITH_VERSE
-#define TD_VERSE_OBJECT		32
-#define TD_VERSE_VERT		64
+	#define TD_VERSE_OBJECT		(1 << 6)
+	#define TD_VERSE_VERT		(1 << 7)
 #endif
+#define TD_TIMEONLY			(1 << 8)
+#define TD_NOCENTER			(1 << 9)
+#define TD_NO_EXT			(1 << 10)	/* ext abused for particle key timing */
+#define TD_SKIP				(1 << 11)	/* don't transform this data */
+#define TD_BEZTRIPLE		(1 << 12)	/* if this is a bez triple, we need to restore the handles, if this is set transdata->misc.hdata needs freeing */
 
 /* transsnap->status */
 #define SNAP_ON			1
@@ -272,6 +309,7 @@ typedef struct TransInfo {
 #define SNAP_CLOSEST		0
 #define SNAP_CENTER			1
 #define SNAP_MEDIAN			2
+#define SNAP_ACTIVE			3
 
 void checkFirstTime(void);
 
@@ -284,9 +322,11 @@ void convertVecToDisplayNum(float *vec, float *num);
 void convertDisplayNumToVec(float *num, float *vec);
 
 void initWarp(TransInfo *t);
+int handleEventWarp(TransInfo *t, unsigned short event, short val);
 int Warp(TransInfo *t, short mval[2]);
 
 void initShear(TransInfo *t);
+int handleEventShear(TransInfo *t, unsigned short evenl, short val);
 int Shear(TransInfo *t, short mval[2]);
 
 void initResize(TransInfo *t);
@@ -316,6 +356,13 @@ int Trackball(TransInfo *t, short mval[2]);
 void initPushPull(TransInfo *t);
 int PushPull(TransInfo *t, short mval[2]);
 
+void initBevel(TransInfo *t);
+int handleEventBevel(TransInfo *t, unsigned short evenl, short val);
+int Bevel(TransInfo *t, short mval[2]);
+
+void initBevelWeight(TransInfo *t);
+int BevelWeight(TransInfo *t, short mval[2]);
+
 void initCrease(TransInfo *t);
 int Crease(TransInfo *t, short mval[2]);
 
@@ -328,15 +375,35 @@ int BoneEnvelope(TransInfo *t, short mval[2]);
 void initBoneRoll(TransInfo *t);
 int BoneRoll(TransInfo *t, short mval[2]);
 
+void initTimeTranslate(TransInfo *t);
+int TimeTranslate(TransInfo *t, short mval[2]);
+
+void initTimeSlide(TransInfo *t);
+int TimeSlide(TransInfo *t, short mval[2]);
+
+void initTimeScale(TransInfo *t);
+int TimeScale(TransInfo *t, short mval[2]);
+
+void initBakeTime(TransInfo *t);
+int BakeTime(TransInfo *t, short mval[2]);
+
+void initMirror(TransInfo *t);
+int Mirror(TransInfo *t, short mval[2]);
+
+void initAlign(TransInfo *t);
+int Align(TransInfo *t, short mval[2]);
+
 /*********************** transform_conversions.c ********** */
 struct ListBase;
+void flushTransIpoData(TransInfo *t);
 void flushTransUVs(TransInfo *t);
+void flushTransParticles(TransInfo *t);
 int clipUVTransform(TransInfo *t, float *vec, int resize);
 
 /*********************** exported from transform_manipulator.c ********** */
 void draw_manipulator_ext(struct ScrArea *sa, int type, char axis, int col, float vec[3], float mat[][3]);
 int calc_manipulator_stats(struct ScrArea *sa);
-float get_drawsize(struct View3D *v3d);
+float get_drawsize(struct View3D *v3d, float *co);
 
 /*********************** TransData Creation and General Handling *********** */
 void createTransData(TransInfo *t);
@@ -344,6 +411,13 @@ void sort_trans_data_dist(TransInfo *t);
 void add_tdi_poin(float *poin, float *old, float delta);
 void special_aftertrans_update(TransInfo *t);
 
+void transform_autoik_update(TransInfo *t, short mode);
+
+/* auto-keying stuff used by special_aftertrans_update */
+short autokeyframe_cfra_can_key(struct Object *ob);
+void autokeyframe_ob_cb_func(struct Object *ob, int tmode);
+void autokeyframe_pose_cb_func(struct Object *ob, int tmode, short targetless_ik);
+
 /*********************** Constraints *****************************/
 
 void getConstraintMatrix(TransInfo *t);
@@ -392,6 +466,8 @@ void postTrans (TransInfo *t);
 
 void drawLine(float *center, float *dir, char axis, short options);
 
+TransDataCurveHandleFlags *initTransDataCurveHandes(TransData *td, struct BezTriple *bezt);
+
 /* DRAWLINE options flags */
 #define DRAWLIGHT	1
 #define DRAWDASHED	2
@@ -407,6 +483,7 @@ void calculateCenterBound(TransInfo *t);
 void calculateCenterMedian(TransInfo *t);
 void calculateCenterCursor(TransInfo *t);
 
+void calculateCenterCursor2D(TransInfo *t);
 void calculatePropRatio(TransInfo *t);
 
 void getViewVector(float coord[3], float vec[3]);
@@ -420,5 +497,44 @@ short hasNumInput(NumInput *n);
 void applyNumInput(NumInput *n, float *vec);
 char handleNumInput(NumInput *n, unsigned short event);
 
+/*********************** NDofInput ********************************/
+
+void initNDofInput(NDofInput *n);
+int hasNDofInput(NDofInput *n);
+void applyNDofInput(NDofInput *n, float *vec);
+int handleNDofInput(NDofInput *n, unsigned short event, short val);
+
+/* handleNDofInput return values */
+#define NDOF_REFRESH	1
+#define NDOF_NOMOVE		2
+#define NDOF_CONFIRM	3
+#define NDOF_CANCEL		4
+
+
+/*********************** TransSpace ******************************/
+
+int manageObjectSpace(int confirm, int set);
+int manageMeshSpace(int confirm, int set);
+
+/* Those two fill in mat and return non-zero on success */
+int createSpaceNormal(float mat[3][3], float normal[3]);
+int createSpaceNormalTangent(float mat[3][3], float normal[3], float tangent[3]);
+
+int addMatrixSpace(float mat[3][3], char name[]);
+int addObjectSpace(struct Object *ob);
+void applyTransformOrientation(void);
+
+
+#define ORIENTATION_NONE	0
+#define ORIENTATION_NORMAL	1
+#define ORIENTATION_VERT	2
+#define ORIENTATION_EDGE	3
+#define ORIENTATION_FACE	4
+
+int getTransformOrientation(float normal[3], float plane[3], int activeOnly);
+int createSpaceNormal(float mat[3][3], float normal[3]);
+int createSpaceNormalTangent(float mat[3][3], float normal[3], float tangent[3]);
+
 #endif
 
+
