@@ -3,17 +3,14 @@
  *
  * ID and Library types, which are fundamental for sdna,
  *
- * $Id: DNA_ID.h 11236 2007-07-12 04:56:00Z bdiego $
+ * $Id: DNA_ID.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,7 +28,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 #ifndef DNA_ID_H
 #define DNA_ID_H
@@ -120,6 +117,18 @@ typedef struct Library {
 	struct Library *parent;	/* for outliner, showing dependency */
 } Library;
 
+#define PREVIEW_MIPMAPS 2
+#define PREVIEW_MIPMAP_ZERO 0
+#define PREVIEW_MIPMAP_LARGE 1
+
+typedef struct PreviewImage {
+	unsigned int w[2];
+	unsigned int h[2];	
+	short changed[2];
+	short pad0, pad1;
+	unsigned int * rect[2];
+} PreviewImage;
+
 /**
  * Defines for working with IDs.
  *
@@ -128,7 +137,7 @@ typedef struct Library {
  *
  **/
 
-#if defined(__sgi) || defined(__sparc) || defined(__sparc__) || defined (__PPC__) || defined (__ppc__) || defined (__BIG_ENDIAN__)
+#if defined(__sgi) || defined(__sparc) || defined(__sparc__) || defined (__PPC__) || defined (__ppc__)  || defined (__hppa__) || defined (__BIG_ENDIAN__)
 /* big endian */
 #define MAKE_ID2(c, d)		( (c)<<8 | (d) )
 #define MOST_SIG_BYTE				0
@@ -171,6 +180,7 @@ typedef struct Library {
 #define ID_SCRIPT	MAKE_ID2('P', 'Y')
 #define ID_NT		MAKE_ID2('N', 'T')
 #define ID_BR		MAKE_ID2('B', 'R')
+#define ID_PA		MAKE_ID2('P', 'A')
 
 	/* NOTE! Fake IDs, needed for g.sipo->blocktype or outliner */
 #define ID_SEQ		MAKE_ID2('S', 'Q')
