@@ -1,15 +1,12 @@
 /**
- * $Id: DNA_customdata_types.h 10633 2007-05-02 00:01:23Z campbellbarton $
+ * $Id: DNA_customdata_types.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 #ifndef DNA_CUSTOMDATA_TYPES_H
 #define DNA_CUSTOMDATA_TYPES_H
@@ -64,7 +61,12 @@ typedef struct CustomData {
 #define CD_ORIGINDEX	7
 #define CD_NORMAL		8
 #define CD_FLAGS		9
-#define CD_NUMTYPES		10
+#define CD_PROP_FLT		10
+#define CD_PROP_INT		11
+#define CD_PROP_STR		12
+#define CD_ORIGSPACE	13 /* for modifier stack face location mapping */
+#define CD_ORCO			14
+#define CD_NUMTYPES		15
 
 /* Bits for CustomDataMask */
 #define CD_MASK_MVERT		(1 << CD_MVERT)
@@ -77,6 +79,12 @@ typedef struct CustomData {
 #define CD_MASK_ORIGINDEX	(1 << CD_ORIGINDEX)
 #define CD_MASK_NORMAL		(1 << CD_NORMAL)
 #define CD_MASK_FLAGS		(1 << CD_FLAGS)
+#define CD_MASK_PROP_FLT	(1 << CD_PROP_FLT)
+#define CD_MASK_PROP_INT	(1 << CD_PROP_INT)
+#define CD_MASK_PROP_STR	(1 << CD_PROP_STR)
+#define CD_MASK_ORIGSPACE	(1 << CD_ORIGSPACE)
+#define CD_MASK_ORCO		(1 << CD_ORCO)
+
 
 /* CustomData.flag */
 
