@@ -1,16 +1,13 @@
 /**
  *
- * $Id: DNA_object_force.h,v 1.17 2006/11/16 20:57:02 bjornmose Exp $
+ * $Id: DNA_object_force.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,7 +25,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 #ifndef DNA_OBJECT_FORCE_H
 #define DNA_OBJECT_FORCE_H
@@ -41,32 +38,57 @@ typedef struct PartDeflect {
 	short deflect;		/* Deflection flag - does mesh deflect particles*/
 	short forcefield;	/* Force field type, do the vertices attract / repel particles ? */
 	short flag;			/* general settings flag */
-	short pad;
+	short falloff;		/* fall-off type*/
 	
 	float pdef_damp;	/* Damping factor for particle deflection       */
 	float pdef_rdamp;	/* Random element of damping for deflection     */
 	float pdef_perm;	/* Chance of particle passing through mesh      */
+	float pdef_frict;	/* Friction factor for particle deflection		*/
+	float pdef_rfrict;	/* Random element of friction for deflection	*/
 
 	float f_strength;	/* The strength of the force (+ or - )       */
 	float f_power;		/* The power law - real gravitation is 2 (square)  */
+	float f_dist;
+	float f_damp;		/* The dampening factor, currently only for harmonic force	*/
 	float maxdist;		/* if indicated, use this maximum */
+	float mindist;		/* if indicated, use this minimum */
+	float maxrad;		/* radial versions of above */
+	float minrad;
+	float f_power_r;	/* radial fall-off power*/
 	
 	float pdef_sbdamp;	/* Damping factor for softbody deflection       */
 	float pdef_sbift;	/* inner face thickness for softbody deflection */
 	float pdef_sboft;	/* outer face thickness for softbody deflection */
+
+	/* variables for guide curve */
+	float clump_fac, clump_pow;
+	float kink_freq, kink_shape, kink_amp, free_end;
+
+	float tex_nabla;
+	short tex_mode, kink, kink_axis, rt2;
+	struct Tex *tex;	/* Texture of the texture effector */
 } PartDeflect;
 
+typedef struct PointCache {
+	int flag;		/* generic flag */
+	int simframe;	/* current frame of simulation (only if SIMULATION_VALID) */
+	int startframe;	/* simulation start frame */
+	int endframe;	/* simulation end frame */
+	int editframe;	/* frame being edited (runtime only) */
+} PointCache;
 
 typedef struct SBVertex {
 	float vec[4];
 } SBVertex;
 
 typedef struct SoftBody {
+	struct ParticleSystem *particles;	/* particlesystem softbody */
+
 	/* dynamic data */
 	int totpoint, totspring;
 	struct BodyPoint *bpoint;		/* not saved in file */
 	struct BodySpring *bspring;		/* not saved in file */
-	float ctime;					/* last time calculated */
+	float pad;
 	
 	/* part of UI: */
 	
@@ -96,8 +118,10 @@ typedef struct SoftBody {
 	int interval;
 	short local, solverflags;		/* local==1: use local coords for baking */
 	
+	/* -- these must be kept for backwards compatibility -- */
 	SBVertex **keys;			/* array of size totpointkey */
 	int totpointkey, totkey;	/* if totpointkey != totpoint or totkey!- (efra-sfra)/interval -> free keys */
+	/* ---------------------------------------------------- */
 	float secondspring;
 
 	/* self collision*/
@@ -109,10 +133,15 @@ typedef struct SoftBody {
 		minloops,
 		maxloops,
 		choke,
-		pad3,pad4,pad5
+		solver_ID,
+		plastic,springpreload
 		;   
 
 	struct SBScratch *scratch;	/* scratch pad/cache on live time not saved in file */
+	float shearstiff;
+	float inpush;
+
+	struct PointCache *pointcache;
 
 } SoftBody;
 
@@ -122,11 +151,45 @@ typedef struct SoftBody {
 #define PFIELD_MAGNET	3
 #define PFIELD_WIND		4
 #define PFIELD_GUIDE	5
+#define PFIELD_TEXTURE	6
+#define PFIELD_HARMONIC	7
+#define PFIELD_NUCLEAR	8
+#define PFIELD_MDIPOLE	9
+
 
 /* pd->flag: various settings */
 #define PFIELD_USEMAX			1
 #define PDEFLE_DEFORM			2
 #define PFIELD_GUIDE_PATH_ADD	4
+#define PFIELD_PLANAR			8
+#define PDEFLE_KILL_PART		16
+#define PFIELD_POSZ				32
+#define PFIELD_TEX_OBJECT		64
+#define PFIELD_TEX_2D			128
+#define PFIELD_USEMIN			256
+#define PFIELD_USEMAXR			512
+#define PFIELD_USEMINR			1024
+#define PFIELD_TEX_ROOTCO		2048
+
+/* pd->falloff */
+#define PFIELD_FALL_SPHERE		0
+#define PFIELD_FALL_TUBE		1
+#define PFIELD_FALL_CONE		2
+//reserved for near future
+//#define PFIELD_FALL_INSIDE		3
+
+/* pd->tex_mode */
+#define PFIELD_TEX_RGB	0
+#define PFIELD_TEX_GRAD	1
+#define PFIELD_TEX_CURL	2
+
+/* pointcache->flag */
+#define PTCACHE_BAKED				1
+#define PTCACHE_OUTDATED			2
+#define PTCACHE_SIMULATION_VALID	4
+#define PTCACHE_BAKING				8
+#define PTCACHE_BAKE_EDIT			16
+#define PTCACHE_BAKE_EDIT_ACTIVE	32
 
 /* ob->softflag */
 #define OB_SB_ENABLE	1
@@ -134,18 +197,28 @@ typedef struct SoftBody {
 #define OB_SB_EDGES		4
 #define OB_SB_QUADS		8
 #define OB_SB_POSTDEF	16
-#define OB_SB_REDO		32
-#define OB_SB_BAKESET	64
-#define OB_SB_BAKEDO	128
-#define OB_SB_RESET		256
+// #define OB_SB_REDO		32
+// #define OB_SB_BAKESET	64
+// #define OB_SB_BAKEDO	128
+// #define OB_SB_RESET		256
 #define OB_SB_SELF		512
 #define OB_SB_FACECOLL  1024
 #define OB_SB_EDGECOLL  2048
 #define OB_SB_COLLFINAL 4096
+//#define OB_SB_PROTECT_CACHE	8192
+#define OB_SB_AERO_ANGLE	16384
 
+/* sb->solverflags */
 #define SBSO_MONITOR    1 
 #define SBSO_OLDERR     2 
 
+/* sb->sbc_mode */
+#define SBC_MODE_MANUAL		0
+#define SBC_MODE_AVG		1
+#define SBC_MODE_MIN		2
+#define SBC_MODE_MAX		3
+#define SBC_MODE_AVGMINMAX	4
+
 #ifdef __cplusplus
 }
 #endif
