@@ -1,15 +1,12 @@
 /**
- * $Id: makesdna.c 10574 2007-04-24 14:52:35Z ton $
+ * $Id: makesdna.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  *
  * Struct muncher for making SDNA
  *
@@ -49,7 +46,7 @@
  * numbers give more output.
  * */
 
-#define DNA_VERSION_DATE "$Id: makesdna.c 10574 2007-04-24 14:52:35Z ton $"
+#define DNA_VERSION_DATE "$Id: makesdna.c 14444 2008-04-16 22:40:48Z hos $"
 
 #include <string.h>
 #include <stdlib.h>
@@ -127,6 +124,8 @@ char *includefiles[] = {
 	"DNA_color_types.h",
 	"DNA_brush_types.h",
 	"DNA_customdata_types.h",
+	"DNA_particle_types.h",
+	"DNA_cloth_types.h",
 	// if you add files here, please add them at the end
 	// of makesdna.c (this file) as well
 
@@ -714,8 +713,10 @@ static int calculate_structlens(int firststruct)
 						
 						/* struct alignment */
 						if(type >= firststruct) {
-							if(sizeof(void *)==8 && (len % 8) )
+							if(sizeof(void *)==8 && (len % 8) ) {
 								printf("Align struct error: %s %s\n", types[structtype],cp);
+								dna_error = 1;
+							}
 						}
 						
 						/* 2-4 aligned/ */
@@ -1144,4 +1145,6 @@ int main(int argc, char ** argv)
 #include "DNA_color_types.h"
 #include "DNA_brush_types.h"
 #include "DNA_customdata_types.h"
+#include "DNA_particle_types.h"
+#include "DNA_cloth_types.h"
 /* end of list */
