@@ -1,5 +1,5 @@
 /**
- * $Id: CMP_node.h 10516 2007-04-13 04:22:32Z scourage $
+ * $Id: CMP_node.h 14061 2008-03-11 14:40:27Z blendix $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -53,15 +53,21 @@ extern bNodeType cmp_node_output_file;
 extern bNodeType cmp_node_curve_rgb;
 extern bNodeType cmp_node_mix_rgb;
 extern bNodeType cmp_node_hue_sat;
+extern bNodeType cmp_node_brightcontrast;
+extern bNodeType cmp_node_gamma;
+extern bNodeType cmp_node_invert;
 extern bNodeType cmp_node_alphaover;
 extern bNodeType cmp_node_zcombine;
 
 extern bNodeType cmp_node_normal;
 extern bNodeType cmp_node_curve_vec;
 extern bNodeType cmp_node_map_value;
+extern bNodeType cmp_node_normalize;
 
 extern bNodeType cmp_node_filter;
 extern bNodeType cmp_node_blur;
+extern bNodeType cmp_node_dblur;
+extern bNodeType cmp_node_bilateralblur;
 extern bNodeType cmp_node_vecblur;
 extern bNodeType cmp_node_dilateerode;
 extern bNodeType cmp_node_defocus;
@@ -79,6 +85,7 @@ extern bNodeType cmp_node_sepyuva;
 extern bNodeType cmp_node_combyuva;
 extern bNodeType cmp_node_sepycca;
 extern bNodeType cmp_node_combycca; 
+extern bNodeType cmp_node_premulkey;
 
 extern bNodeType cmp_node_diff_matte;
 extern bNodeType cmp_node_chroma;
@@ -90,12 +97,12 @@ extern bNodeType cmp_node_translate;
 extern bNodeType cmp_node_rotate;
 extern bNodeType cmp_node_scale;
 extern bNodeType cmp_node_flip;
+extern bNodeType cmp_node_crop;
 extern bNodeType cmp_node_displace;
 extern bNodeType cmp_node_mapuv;
 
-extern bNodeType cmp_node_brightcontrast;
-extern bNodeType cmp_node_gamma;
+extern bNodeType cmp_node_glare;
+extern bNodeType cmp_node_tonemap;
+extern bNodeType cmp_node_lensdist;
 
 #endif
-
-
