@@ -1,5 +1,5 @@
 /**
- * $Id: SHD_node.h 10378 2007-03-26 15:07:38Z ton $
+ * $Id: SHD_node.h 13620 2008-02-09 23:17:15Z jesterking $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -56,6 +56,12 @@ extern bNodeType sh_node_curve_rgb;
 extern bNodeType sh_node_math;
 extern bNodeType sh_node_vect_math;
 extern bNodeType sh_node_squeeze;
+extern bNodeType node_dynamic_typeinfo;
+extern bNodeType sh_node_material_ext;
+extern bNodeType sh_node_invert;
+extern bNodeType sh_node_seprgb;
+extern bNodeType sh_node_combrgb;
+extern bNodeType sh_node_hue_sat;
 
 #endif
 
