@@ -1,15 +1,12 @@
 /* 
- * $Id: Bone.c 11583 2007-08-14 06:39:07Z campbellbarton $
+ * $Id: Bone.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +22,7 @@
  *
  * Contributor(s): Joseph Gilbert
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
 */
 
 #include "Bone.h"
@@ -88,9 +85,9 @@ static PyObject *EditBone_hasParent(BPy_EditBone *self)
 {
 	if (self->editbone){
 		if (self->editbone->parent)
-			return EXPP_incr_ret(Py_True);
+			Py_RETURN_TRUE;
 		else
-			return EXPP_incr_ret(Py_False);
+			Py_RETURN_FALSE;
 	}else{
 		goto AttributeError;
 	}
@@ -105,7 +102,7 @@ static PyObject *EditBone_clearParent(BPy_EditBone *self)
 	if (self->editbone){
 		if (self->editbone->parent)
 			self->editbone->parent = NULL;
-		return EXPP_incr_ret(Py_None);
+		Py_RETURN_NONE;
 	}else{
 		goto AttributeError;
 	}
@@ -406,9 +403,10 @@ static PyObject *EditBone_getOptions(BPy_EditBone *self, void *closure)
 				goto RuntimeError;
 	}
 
-	return EXPP_incr_ret(list);
+	return list;
 
 RuntimeError:
+	Py_XDECREF( list );
 	return EXPP_objError(PyExc_RuntimeError, "%s%s%s", 
 		sEditBoneError, ".options: ", "Internal failure!");
 }
@@ -515,9 +513,9 @@ static PyObject *EditBone_getParent(BPy_EditBone *self, void *closure)
 		if (self->editbone->parent)
 			return PyEditBone_FromEditBone(self->editbone->parent);
 		else
-			return EXPP_incr_ret(Py_None);
+			Py_RETURN_NONE;
 	}else{
-		return EXPP_incr_ret(Py_None); //not in the list yet can't have a parent
+		Py_RETURN_NONE; //not in the list yet can't have a parent
 	}
 }
 //------------------------EditBone.parent (set)
@@ -708,6 +706,40 @@ AttributeError:
 		sEditBoneError, ".tailRadius: ", "expects a float");
 }
 
+//------------------------Bone.layerMask (get)
+static PyObject *EditBone_getLayerMask(BPy_EditBone *self)
+{
+	/* do this extra stuff because the short's bits can be negative values */
+	unsigned short laymask = 0;
+	if (self->editbone)	laymask |= self->editbone->layer;
+	else				laymask |= self->layer;
+	return PyInt_FromLong((int)laymask);
+}
+//------------------------Bone.layerMask (set)
+static int EditBone_setLayerMask(BPy_EditBone *self, PyObject *value)
+{
+	int laymask;
+	if (!PyInt_Check(value)) {
+		return EXPP_ReturnIntError( PyExc_AttributeError,
+									"expected an integer (bitmask) as argument" );
+	}
+	
+	laymask = PyInt_AsLong(value);
+
+	if (laymask <= 0 || laymask > (1<<16) - 1)
+		return EXPP_ReturnIntError( PyExc_AttributeError,
+									"bitmask must have from 1 up to 16 bits set");
+	
+	if (self->editbone) {
+		self->editbone->layer = 0;
+		self->editbone->layer |= laymask;
+	} else {
+		self->layer = 0;
+		self->layer |= laymask;
+	}
+	
+	return 0;
+}
 
 //------------------TYPE_OBECT IMPLEMENTATION--------------------------
 //------------------------tp_methods
@@ -748,6 +780,8 @@ static PyGetSetDef BPy_EditBone_getset[] = {
 		"Set the radius of this bones tip", NULL},
 	{"headRadius", (getter)EditBone_getHeadRadius, (setter)EditBone_setHeadRadius, 
 		"Set the radius of this bones head", NULL},
+	{"layerMask", (getter)EditBone_getLayerMask, (setter)EditBone_setLayerMask, 
+		"Layer bitmask", NULL },
 	{NULL, NULL, NULL, NULL,NULL}
 };
 
@@ -802,6 +836,7 @@ static PyObject *EditBone_new(PyTypeObject *type, PyObject *args, PyObject *kwds
 	py_editBone->rad_head= 0.10f;
 	py_editBone->rad_tail= 0.05f;
 	py_editBone->segments= 1;
+	py_editBone->layer= 1;
 	py_editBone->flag = 0;
 	py_editBone->roll = 0.0f;
 
@@ -837,7 +872,7 @@ PyTypeObject EditBone_Type = {
 	0,											//tp_getattr
 	0,											//tp_setattr
 	(cmpfunc)EditBone_compare,					//tp_compare
-	(reprfunc)EditBone_repr,					//tp_repr
+	(reprfunc)EditBone_repr,			//tp_repr
 	0,											//tp_as_number
 	0,											//tp_as_sequence
 	0,											//tp_as_mapping
@@ -886,45 +921,47 @@ static int PyBone_ChildrenAsList(PyObject *list, ListBase *bones){
 		py_bone = PyBone_FromBone(bone);
 		if (py_bone == NULL)
 			return 0;
-
+		
 		if(PyList_Append(list, py_bone) == -1){
-			goto RuntimeError;
+			return 0;
 		}
+		Py_DECREF(py_bone);
 		if (bone->childbase.first) 
-			PyBone_ChildrenAsList(list, &bone->childbase);
+			if (!PyBone_ChildrenAsList(list, &bone->childbase))
+				return 0;
 	}
 	return 1;
-
-RuntimeError:
-	return EXPP_intError(PyExc_RuntimeError, "%s%s", 
-		sBoneError, "Internal error trying to wrap blender bones!");
 }
 //-------------------------Bone.hasParent()
 static PyObject *Bone_hasParent(BPy_Bone *self)
 {
 	if (self->bone->parent)
-		return EXPP_incr_ret(Py_True);
+		Py_RETURN_TRUE;
 	else
-		return EXPP_incr_ret(Py_False);
+		Py_RETURN_FALSE;
 }
 //-------------------------Bone.hasChildren()
 static PyObject *Bone_hasChildren(BPy_Bone *self)
 {
 	if (self->bone->childbase.first)
-		return EXPP_incr_ret(Py_True);
+		Py_RETURN_TRUE;
 	else
-		return EXPP_incr_ret(Py_False);
+		Py_RETURN_FALSE;
 }
 //-------------------------Bone.getAllChildren()
 static PyObject *Bone_getAllChildren(BPy_Bone *self)
 {
 	PyObject *list = PyList_New(0);
-
-	if (self->bone->childbase.first)
-		if (!PyBone_ChildrenAsList(list, &self->bone->childbase))
-			return NULL;
-	return EXPP_incr_ret(list);
+	if (!self->bone->childbase.first) {
+		/* do nothing */
+	} else if (!PyBone_ChildrenAsList(list, &self->bone->childbase)) {
+		Py_XDECREF(list);
+		EXPP_objError(PyExc_RuntimeError, "%s%s", 
+				sBoneError, "Internal error trying to wrap blender bones!");
+	}
+	return list;
 }
+
 //------------------ATTRIBUTE IMPLEMENTATIONS-----------------------------
 //------------------------Bone.name (get)
 static PyObject *Bone_getName(BPy_Bone *self, void *closure)
@@ -940,10 +977,10 @@ static int Bone_setName(BPy_Bone *self, PyObject *value, void *closure)
 }
 //------------------------Bone.roll (get)
 static PyObject *Bone_getRoll(BPy_Bone *self, void *closure)
-{
-	return Py_BuildValue("{s:O, s:O}", 
-		"BONESPACE", PyFloat_FromDouble((self->bone->roll * (180/Py_PI))),
-		"ARMATURESPACE", PyFloat_FromDouble((boneRoll_ToArmatureSpace(self->bone) * (180/Py_PI))));
+{	
+	return Py_BuildValue("{s:f, s:f}", 
+		"BONESPACE", self->bone->roll * (180/Py_PI),
+		"ARMATURESPACE", boneRoll_ToArmatureSpace(self->bone) * (180/Py_PI));
 }
 //------------------------Bone.roll (set)
 static int Bone_setRoll(BPy_Bone *self, PyObject *value, void *closure)
@@ -954,9 +991,14 @@ static int Bone_setRoll(BPy_Bone *self, PyObject *value, void *closure)
 //------------------------Bone.head (get)
 static PyObject *Bone_getHead(BPy_Bone *self, void *closure)
 {
-	return Py_BuildValue("{s:O, s:O}", 
-		"BONESPACE", newVectorObject(self->bone->head, 3, Py_WRAP),
-		"ARMATURESPACE", newVectorObject(self->bone->arm_head, 3, Py_WRAP));
+	PyObject *val1 = newVectorObject(self->bone->head, 3, Py_WRAP);
+	PyObject *val2 = newVectorObject(self->bone->arm_head, 3, Py_WRAP);
+	PyObject *ret =	Py_BuildValue(
+			"{s:O, s:O}", "BONESPACE", val1, "ARMATURESPACE", val2);
+	
+	Py_DECREF(val1);
+	Py_DECREF(val2);
+	return ret;
 }
 //------------------------Bone.head (set)
 static int Bone_setHead(BPy_Bone *self, PyObject *value, void *closure)
@@ -967,9 +1009,14 @@ static int Bone_setHead(BPy_Bone *self, PyObject *value, void *closure)
 //------------------------Bone.tail (get)
 static PyObject *Bone_getTail(BPy_Bone *self, void *closure)
 {
-    return Py_BuildValue("{s:O, s:O}", 
-		"BONESPACE", newVectorObject(self->bone->tail, 3, Py_WRAP),
-		"ARMATURESPACE", newVectorObject(self->bone->arm_tail, 3, Py_WRAP));
+	PyObject *val1 = newVectorObject(self->bone->tail, 3, Py_WRAP);
+	PyObject *val2 = newVectorObject(self->bone->arm_tail, 3, Py_WRAP);
+	PyObject *ret =	Py_BuildValue("{s:O, s:O}", 
+		"BONESPACE", val1, "ARMATURESPACE", val2);
+	
+	Py_DECREF(val1);
+	Py_DECREF(val2);
+	return ret;
 }
 //------------------------Bone.tail (set)
 static int Bone_setTail(BPy_Bone *self, PyObject *value, void *closure)
@@ -1052,9 +1099,10 @@ static PyObject *Bone_getOptions(BPy_Bone *self, void *closure)
 			EXPP_GetModuleConstant("Blender.Armature", "TIP_SELECTED")) == -1)
 			goto RuntimeError;
 
-	return EXPP_incr_ret(list);
-
+	return list;
+	
 RuntimeError:
+	Py_XDECREF(list);
 	return EXPP_objError(PyExc_RuntimeError, "%s%s%s", 
 		sBoneError, "getOptions(): ", "Internal failure!");
 }
@@ -1070,7 +1118,7 @@ static PyObject *Bone_getParent(BPy_Bone *self, void *closure)
 	if (self->bone->parent)
 		return PyBone_FromBone(self->bone->parent);
 	else
-		return EXPP_incr_ret(Py_None);
+		Py_RETURN_NONE;
 }
 //------------------------Bone.parent (set)
 static int Bone_setParent(BPy_Bone *self, PyObject *value, void *closure)
@@ -1089,15 +1137,17 @@ static PyObject *Bone_getChildren(BPy_Bone *self, void *closure)
 		for (bone = self->bone->childbase.first; bone; bone = bone->next){
 			py_bone = PyBone_FromBone(bone);
 			if (py_bone == NULL)
-				return 0;
-			if(PyList_Append(list, py_bone) == -1){
 				goto RuntimeError;
-			}
+			if (PyList_Append(list, py_bone) == -1)
+				goto RuntimeError;
+			Py_DECREF(py_bone);
 		}
 	}
-	return EXPP_incr_ret(list);
+	return list;
 	
 RuntimeError:
+	Py_XDECREF(list);
+	Py_XDECREF(py_bone);
 	return EXPP_objError(PyExc_RuntimeError, "%s%s", 
 		sBoneError, "Internal error trying to wrap blender bones!");
 }
@@ -1110,9 +1160,15 @@ static int Bone_setChildren(BPy_Bone *self, PyObject *value, void *closure)
 //------------------------Bone.matrix (get)
 static PyObject *Bone_getMatrix(BPy_Bone *self, void *closure)
 {
-    return Py_BuildValue("{s:O, s:O}", 
-		"BONESPACE", newMatrixObject((float*)self->bone->bone_mat, 3,3, Py_WRAP),
-		"ARMATURESPACE", newMatrixObject((float*)self->bone->arm_mat, 4,4, Py_WRAP));
+	PyObject *val1 = newMatrixObject((float*)self->bone->bone_mat, 3,3, Py_WRAP);
+	PyObject *val2 = newMatrixObject((float*)self->bone->arm_mat, 4,4, Py_WRAP);
+	PyObject *ret =	Py_BuildValue("{s:O, s:O}", 
+		"BONESPACE", val1, "ARMATURESPACE", val2);
+	Py_DECREF(val1);
+	Py_DECREF(val2);
+	return ret;
+    
+    
 }
 //------------------------Bone.matrix (set)
 static int Bone_setMatrix(BPy_Bone *self, PyObject *value, void *closure)
@@ -1181,6 +1237,35 @@ AttributeError:
 		sEditBoneError, ".headRadius: ", "expects a float");
 }
 
+//------------------------Bone.layerMask (get)
+static PyObject *Bone_getLayerMask(BPy_Bone *self)
+{
+	/* do this extra stuff because the short's bits can be negative values */
+	unsigned short laymask = 0;
+	laymask |= self->bone->layer;
+	return PyInt_FromLong((int)laymask);
+}
+//------------------------Bone.layerMask (set)
+static int Bone_setLayerMask(BPy_Bone *self, PyObject *value)
+{
+	int laymask;
+	if (!PyInt_Check(value)) {
+		return EXPP_ReturnIntError( PyExc_AttributeError,
+									"expected an integer (bitmask) as argument" );
+	}
+	
+	laymask = PyInt_AsLong(value);
+
+	if (laymask <= 0 || laymask > (1<<16) - 1)
+		return EXPP_ReturnIntError( PyExc_AttributeError,
+									"bitmask must have from 1 up to 16 bits set");
+
+	self->bone->layer = 0;
+	self->bone->layer |= laymask;
+
+	return 0;
+}
+
 //------------------TYPE_OBECT IMPLEMENTATION--------------------------
 //------------------------tp_methods
 //This contains a list of all methods the object contains
@@ -1224,6 +1309,8 @@ static PyGetSetDef BPy_Bone_getset[] = {
 		"Set the radius of this bones tip", NULL},
 	{"headRadius", (getter)Bone_getHeadRadius, (setter)Bone_setHeadRadius, 
 		"Set the radius of this bones head", NULL},
+	{"layerMask", (getter)Bone_getLayerMask, (setter)Bone_setLayerMask, 
+		"Layer bitmask", NULL },
 	{NULL, NULL, NULL, NULL,NULL}
 };
 //------------------------tp_repr
@@ -1252,14 +1339,14 @@ static char BPy_Bone_doc[] = "This object wraps a Blender Boneobject.\n\
 PyTypeObject Bone_Type = {
 	PyObject_HEAD_INIT(NULL)   //tp_head
 	0,										//tp_internal
-	"Bone",								//tp_name
-	sizeof(BPy_Bone),					//tp_basicsize
+	"Bone",									//tp_name
+	sizeof(BPy_Bone),						//tp_basicsize
 	0,										//tp_itemsize
-	(destructor)Bone_dealloc,		//tp_dealloc
+	(destructor)Bone_dealloc,				//tp_dealloc
 	0,										//tp_print
 	0,										//tp_getattr
 	0,										//tp_setattr
-	(cmpfunc)Bone_compare,					//tp_compare
+	(cmpfunc) Bone_compare,					//tp_compare
 	(reprfunc) Bone_repr,					//tp_repr
 	0,										//tp_as_number
 	0,										//tp_as_sequence
@@ -1321,19 +1408,11 @@ RuntimeError:
 //Converts a struct Bone to a BPy_Bone
 PyObject *PyBone_FromBone(struct Bone *bone)
 {
-	BPy_Bone *py_Bone = NULL;
-
-	py_Bone = (BPy_Bone*)Bone_Type.tp_alloc(&Bone_Type, 0); //*new*
-	if (py_Bone == NULL)
-		goto RuntimeError;
-
+	BPy_Bone *py_Bone = ( BPy_Bone * ) PyObject_NEW( BPy_Bone, &Bone_Type );
+	
 	py_Bone->bone = bone;
 
 	return (PyObject *) py_Bone;
-
-RuntimeError:
-	return EXPP_objError(PyExc_RuntimeError, "%s%s%s", 
-		sBoneError, "PyBone_FromBone: ", "Internal Error Ocurred");
 }
 //-----------------(internal)
 //Converts a PyBone to a bBone
