@@ -1,14 +1,11 @@
 /*
- * $Id: CurNurb.c 11328 2007-07-21 16:35:55Z campbellbarton $
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * $Id: CurNurb.c 14456 2008-04-17 21:14:55Z campbellbarton $
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +23,7 @@
  *
  * Contributor(s): Stephen Swaney, Campbell Barton, Ken Hughes
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include "CurNurb.h" /*This must come first */
@@ -37,6 +34,11 @@
 #include "gen_utils.h"
 #include "BezTriple.h"
 
+#include "BKE_utildefines.h"
+
+/* Only for ME_SMOOTH */
+#include "DNA_meshdata_types.h"
+
 /*
  * forward declarations go here
  */
@@ -69,7 +71,8 @@ static PyObject *CurNurb_isCyclic( BPy_CurNurb * self );
 static PyObject *CurNurb_dump( BPy_CurNurb * self );
 static PyObject *CurNurb_switchDirection( BPy_CurNurb * self );
 static PyObject *CurNurb_recalc( BPy_CurNurb * self );
-
+static PyObject *CurNurb_getFlagBits( BPy_CurNurb * self, void *type );
+static int CurNurb_setFlagBits( BPy_CurNurb * self, PyObject *value, void *type );
 char M_CurNurb_doc[] = "CurNurb";
 
 
@@ -185,7 +188,10 @@ static PyGetSetDef BPy_CurNurb_getseters[] = {
 	 (getter)CurNurb_getKnotsV, (setter)NULL,
 	 "The The knot vector in the V direction",
 	 NULL},
-
+	{"smooth",
+	 (getter)CurNurb_getFlagBits, (setter)CurNurb_setFlagBits,
+	 "The smooth bool setting",
+	 (void *)ME_SMOOTH},
 	{NULL,NULL,NULL,NULL,NULL}  /* Sentinel */
 };
 
@@ -420,13 +426,20 @@ static PyObject *CurNurb_getKnotsV( BPy_CurNurb * self )
 
 static PyObject *CurNurb_getPoints( BPy_CurNurb * self )
 {
-	PyObject *attr = PyInt_FromLong( ( long ) self->nurb->pntsu );
+	return PyInt_FromLong( ( long ) self->nurb->pntsu );
+}
 
-	if( attr )
-		return attr;
+static PyObject *CurNurb_getFlagBits( BPy_CurNurb * self, void *type )
+{
+	return EXPP_getBitfield( (void *)&self->nurb->flag,
+							  GET_INT_FROM_POINTER(type), 'h' );
+}
 
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-			"could not get number of points" );
+static int CurNurb_setFlagBits( BPy_CurNurb * self, PyObject *value,
+									void *type )
+{
+	return EXPP_setBitfield( value, (void *)&self->nurb->flag,
+							 GET_INT_FROM_POINTER(type), 'h' );
 }
 
 /*
@@ -437,9 +450,7 @@ static PyObject *CurNurb_getPoints( BPy_CurNurb * self )
 
 PyObject *CurNurb_append( BPy_CurNurb * self, PyObject * value )
 {
-	Nurb *nurb = self->nurb;
-
-	return CurNurb_appendPointToNurb( nurb, value );
+	return CurNurb_appendPointToNurb( self->nurb, value );
 }
 
 
@@ -456,6 +467,10 @@ PyObject *CurNurb_appendPointToNurb( Nurb * nurb, PyObject * value )
 	int size;
 	int npoints = nurb->pntsu;
 
+	/*
+	   do we have a list of four floats or a BezTriple?
+	*/
+	
 	/* if curve is empty, adjust type depending on input type */
 	if (nurb->bezt==NULL && nurb->bp==NULL) {
 		if (BPy_BezTriple_Check( value ))
@@ -760,9 +775,9 @@ static PyObject *CurNurb_isNurb( BPy_CurNurb * self )
 	 */
 
 	if( self->nurb->bp ) {
-		return EXPP_incr_ret_True();
+		Py_RETURN_TRUE;
 	} else {
-		return EXPP_incr_ret_False();
+		Py_RETURN_FALSE;
 	}
 }
 
@@ -776,9 +791,9 @@ static PyObject *CurNurb_isCyclic( BPy_CurNurb * self )
         /* supposing that the flagu is always set */ 
 
 	if( self->nurb->flagu & CU_CYCLIC ) {
-		return EXPP_incr_ret_True();
+		Py_RETURN_TRUE;
 	} else {
-		return EXPP_incr_ret_False();
+		Py_RETURN_FALSE;
 	}
 }
 
@@ -936,8 +951,7 @@ PyObject *CurNurb_pointAtIndex( Nurb * nurb, int index )
 
 		for( i = 0; i < 4; i++ ) {
 			PyList_SetItem( pyo, i,
-					PyFloat_FromDouble( nurb->bp[index].
-							    vec[i] ) );
+					PyFloat_FromDouble( nurb->bp[index].vec[i] ) );
 		}
 
 		/* add Tilt only if curve is 3D */
