@@ -1,14 +1,12 @@
 /* 
- * $Id: Curve.c,v 1.62 2007/03/26 02:10:21 campbellbarton Exp $
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * $Id: Curve.c 14444 2008-04-16 22:40:48Z hos $
+ *
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,11 +24,12 @@
  *
  * Contributor(s): Jacques Guignot, Stephen Swaney
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include "Curve.h" /*This must come first*/
 
+#include "BLI_blenlib.h"
 #include "BKE_main.h"
 #include "BKE_displist.h"
 #include "BKE_global.h"
@@ -122,6 +121,7 @@ static PyObject *Curve_getIter( BPy_Curve * self );
 static PyObject *Curve_iterNext( BPy_Curve * self );
 
 PyObject *Curve_getNurb( BPy_Curve * self, int n );
+static int Curve_setNurb( BPy_Curve * self, int n, PyObject * value );
 static int Curve_length( PyInstanceObject * inst );
 
 
@@ -138,13 +138,7 @@ struct chartrans *text_to_curve( Object * ob, int mode );
 
 PyObject *Curve_getName( BPy_Curve * self )
 {
-	PyObject *attr = PyString_FromString( self->curve->id.name + 2 );
-
-	if( attr )
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					"couldn't get Curve.name attribute" );
+	return PyString_FromString( self->curve->id.name + 2 );
 }
 
 static int Curve_newsetName( BPy_Curve * self, PyObject * args )
@@ -164,13 +158,7 @@ static int Curve_newsetName( BPy_Curve * self, PyObject * args )
 
 static PyObject *Curve_getPathLen( BPy_Curve * self )
 {
-	PyObject *attr = PyInt_FromLong( ( long ) self->curve->pathlen );
-
-	if( attr )
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-			"couldn't get Curve.pathlen attribute" );
+	return PyInt_FromLong( ( long ) self->curve->pathlen );
 }
 
 
@@ -191,25 +179,13 @@ static int Curve_newsetPathLen( BPy_Curve * self, PyObject * args )
 
 static PyObject *Curve_getTotcol( BPy_Curve * self )
 {
-	PyObject *attr = PyInt_FromLong( ( long ) self->curve->totcol );
-
-	if( attr )
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-			"couldn't get Curve.totcol attribute" );
+	return PyInt_FromLong( ( long ) self->curve->totcol );
 }
 
 
 PyObject *Curve_getMode( BPy_Curve * self )
 {
-	PyObject *attr = PyInt_FromLong( ( long ) self->curve->flag );
-
-	if( attr )
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-			"couldn't get Curve.flag attribute" );
+	return PyInt_FromLong( ( long ) self->curve->flag );
 }
 
 
@@ -230,13 +206,7 @@ static int Curve_newsetMode( BPy_Curve * self, PyObject * args )
 
 PyObject *Curve_getBevresol( BPy_Curve * self )
 {
-	PyObject *attr = PyInt_FromLong( ( long ) self->curve->bevresol );
-
-	if( attr )
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-			"couldn't get Curve.bevresol attribute" );
+	return PyInt_FromLong( ( long ) self->curve->bevresol );
 }
 
 static int Curve_newsetBevresol( BPy_Curve * self, PyObject * args )
@@ -263,13 +233,7 @@ static int Curve_newsetBevresol( BPy_Curve * self, PyObject * args )
 
 PyObject *Curve_getResolu( BPy_Curve * self )
 {
-	PyObject *attr = PyInt_FromLong( ( long ) self->curve->resolu );
-
-	if( attr )
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-			"couldn't get Curve.resolu attribute" );
+	return PyInt_FromLong( ( long ) self->curve->resolu );
 }
 
 
@@ -301,13 +265,7 @@ static int Curve_newsetResolu( BPy_Curve * self, PyObject * args )
 
 PyObject *Curve_getResolv( BPy_Curve * self )
 {
-	PyObject *attr = PyInt_FromLong( ( long ) self->curve->resolv );
-
-	if( attr )
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-			"couldn't get Curve.resolv attribute" );
+	return PyInt_FromLong( ( long ) self->curve->resolv );
 }
 
 static int Curve_newsetResolv( BPy_Curve * self, PyObject * args )
@@ -333,13 +291,7 @@ static int Curve_newsetResolv( BPy_Curve * self, PyObject * args )
 
 PyObject *Curve_getWidth( BPy_Curve * self )
 {
-	PyObject *attr = PyFloat_FromDouble( ( double ) self->curve->width );
-
-	if( attr )
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-			"couldn't get Curve.width attribute" );
+	return PyFloat_FromDouble( ( double ) self->curve->width );
 }
 
 
@@ -367,13 +319,7 @@ static int Curve_newsetWidth( BPy_Curve * self, PyObject * args )
 
 PyObject *Curve_getExt1( BPy_Curve * self )
 {
-	PyObject *attr = PyFloat_FromDouble( ( double ) self->curve->ext1 );
-
-	if( attr )
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-			"couldn't get Curve.ext1 attribute" );
+	return PyFloat_FromDouble( ( double ) self->curve->ext1 );
 }
 
 
@@ -390,9 +336,9 @@ static int Curve_newsetExt1( BPy_Curve * self, PyObject * args )
 	value = (float)PyFloat_AS_DOUBLE( num );
 	Py_DECREF( num );
 
-	if(value > 5.0f || value < 0.0f)
+	if(value > 100.0f || value < 0.0f)
 		return EXPP_ReturnIntError( PyExc_ValueError,
-				"acceptable values are between 0.0 and 5.0" );
+				"acceptable values are between 0.0 and 100.0" );
 	self->curve->ext1 = value;
 
 	return 0;
@@ -400,13 +346,7 @@ static int Curve_newsetExt1( BPy_Curve * self, PyObject * args )
 
 PyObject *Curve_getExt2( BPy_Curve * self )
 {
-	PyObject *attr = PyFloat_FromDouble( ( double ) self->curve->ext2 );
-
-	if( attr )
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-			"couldn't get Curve.ext2 attribute" );
+	return PyFloat_FromDouble( ( double ) self->curve->ext2 );
 }
 
 
@@ -481,7 +421,7 @@ static PyObject *Curve_setControlPoint( BPy_Curve * self, PyObject * args )
 
 static PyObject *Curve_getControlPoint( BPy_Curve * self, PyObject * args )
 {
-	PyObject *liste = PyList_New( 0 );	/* return values */
+	PyObject *liste;
 	PyObject *item;
 
 	Nurb *ptrnurb;
@@ -498,7 +438,7 @@ static PyObject *Curve_getControlPoint( BPy_Curve * self, PyObject * args )
 
 	/* if no nurbs in this curve obj */
 	if( !self->curve->nurb.first )
-		return liste;
+		return PyList_New( 0 );
 
 	/* walk the list of nurbs to find requested numcourbe */
 	ptrnurb = self->curve->nurb.first;
@@ -513,7 +453,8 @@ static PyObject *Curve_getControlPoint( BPy_Curve * self, PyObject * args )
 	if( numpoint >= ptrnurb->pntsu )
 		return ( EXPP_ReturnPyObjError( PyExc_ValueError,
 						"point index out of range" ) );
-
+	
+	liste = PyList_New( 0 );
 	if( ptrnurb->bp ) {	/* if we are a nurb curve, you get 4 values */
 		for( i = 0; i < 4; i++ ) {
 			item = PyFloat_FromDouble( ptrnurb->bp[numpoint].vec[i] );
@@ -534,14 +475,8 @@ static PyObject *Curve_getControlPoint( BPy_Curve * self, PyObject * args )
 
 static PyObject *Curve_getLoc( BPy_Curve * self )
 {
-	PyObject *attr = Py_BuildValue( "[f,f,f]", self->curve->loc[0],
+	return Py_BuildValue( "[f,f,f]", self->curve->loc[0],
 				self->curve->loc[1], self->curve->loc[2] );
-
-	if( attr )
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-			"couldn't get Curve.loc attribute" );
 }
 
 static int Curve_newsetLoc( BPy_Curve * self, PyObject * args )
@@ -572,14 +507,8 @@ TypeError:
 
 static PyObject *Curve_getRot( BPy_Curve * self )
 {
-	PyObject *attr = Py_BuildValue( "[f,f,f]", self->curve->rot[0],
+	return Py_BuildValue( "[f,f,f]", self->curve->rot[0],
 				self->curve->rot[1], self->curve->rot[2] );
-
-	if( attr )
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-			"couldn't get Curve.rot attribute" );
 }
 
 static int Curve_newsetRot( BPy_Curve * self, PyObject * args )
@@ -610,14 +539,8 @@ TypeError:
 
 static PyObject *Curve_getSize( BPy_Curve * self )
 {
-	PyObject *attr = Py_BuildValue( "[f,f,f]", self->curve->size[0],
+	return Py_BuildValue( "[f,f,f]", self->curve->size[0],
 				self->curve->size[1], self->curve->size[2] );
-
-	if( attr )
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-			"couldn't get Curve.size attribute" );
 }
 
 static int Curve_newsetSize( BPy_Curve * self, PyObject * args )
@@ -844,8 +767,6 @@ static PyObject *Curve_appendPoint( BPy_Curve * self, PyObject * args )
 	int i;
 	int nurb_num;		/* index of curve we append to */
 	PyObject *coord_args;	/* coords for new point */
-	PyObject *retval = NULL;
-	PyObject *valtuple;
 	Nurb *nurb = self->curve->nurb.first;	/* first nurb in Curve */
 
 /* fixme - need to malloc new Nurb */
@@ -867,14 +788,7 @@ static PyObject *Curve_appendPoint( BPy_Curve * self, PyObject * args )
 			return EXPP_ReturnPyObjError( PyExc_ValueError,
 					"curve index out of range" );
 	}
-
-	/* rebuild our arg tuple for appendPointToNurb() */
-	valtuple = Py_BuildValue( "(O)", coord_args );
-	
-	retval =  CurNurb_appendPointToNurb( nurb, valtuple );
-	Py_DECREF( valtuple );
-
-	return retval;
+	return CurNurb_appendPointToNurb( nurb, coord_args );
 }
 
 
@@ -884,29 +798,16 @@ static PyObject *Curve_appendPoint( BPy_Curve * self, PyObject * args )
   returns a refernce to the newly created nurb.
 *****/
 
-static PyObject *Curve_appendNurb( BPy_Curve * self, PyObject * args )
+static PyObject *Curve_appendNurb( BPy_Curve * self, PyObject * value )
 {
-	Nurb *nurb_ptr = self->curve->nurb.first;
-	Nurb **pptr = ( Nurb ** ) & ( self->curve->nurb.first );
 	Nurb *new_nurb;
-
-
-	/* walk to end of nurblist */
-	if( nurb_ptr ) {
-		while( nurb_ptr->next ) {
-			nurb_ptr = nurb_ptr->next;
-		}
-		pptr = &nurb_ptr->next;
-	}
-
 	/* malloc new nurb */
 	new_nurb = ( Nurb * ) MEM_callocN( sizeof( Nurb ), "appendNurb" );
 	if( !new_nurb )
 		return EXPP_ReturnPyObjError
 			( PyExc_MemoryError, "unable to malloc Nurb" );
-
-	if( CurNurb_appendPointToNurb( new_nurb, args ) ) {
-		*pptr = new_nurb;
+	
+	if( CurNurb_appendPointToNurb( new_nurb, value ) ) {
 		new_nurb->resolu = self->curve->resolu;
 		new_nurb->resolv = self->curve->resolv;
 		new_nurb->hide = 0;
@@ -917,9 +818,9 @@ static PyObject *Curve_appendNurb( BPy_Curve * self, PyObject * args )
 			new_nurb->type = CU_BEZIER;
 			new_nurb->bezt->h1 = HD_ALIGN;
 			new_nurb->bezt->h2 = HD_ALIGN;
-			new_nurb->bezt->f1 = 1;
-			new_nurb->bezt->f2 = 1;
-			new_nurb->bezt->f3 = 1;
+			new_nurb->bezt->f1 = SELECT;
+			new_nurb->bezt->f2 = SELECT;
+			new_nurb->bezt->f3 = SELECT;
 			new_nurb->bezt->hide = 0;
 			/* calchandlesNurb( new_nurb ); */
 		} else {	/* set up bp */
@@ -933,6 +834,7 @@ static PyObject *Curve_appendNurb( BPy_Curve * self, PyObject * args )
 			new_nurb->knotsu = 0;
 			/*makenots( new_nurb, 1, new_nurb->flagu >> 1); */
 		}
+		BLI_addtail( &self->curve->nurb, new_nurb);
 
 	} else {
 		freeNurb( new_nurb );
@@ -1207,6 +1109,42 @@ PyObject *Curve_getNurb( BPy_Curve * self, int n )
 
 }
 
+/*
+ * Curve_setNurb
+ * In this case only remove the item, we could allow adding later.
+ */
+static int Curve_setNurb( BPy_Curve * self, int n, PyObject * value )
+{
+	Nurb *pNurb;
+	int i;
+
+	/* bail if index < 0 */
+	if( n < 0 )
+		return ( EXPP_ReturnIntError( PyExc_IndexError,
+				 "index less than 0" ) );
+	/* bail if no Nurbs in Curve */
+	if( self->curve->nurb.first == 0 )
+		return ( EXPP_ReturnIntError( PyExc_IndexError,
+				 "no Nurbs in this Curve" ) );
+	/* set pointer to nth Nurb */
+	for( pNurb = self->curve->nurb.first, i = 0;
+			pNurb != 0 && i < n; pNurb = pNurb->next, ++i )
+		/**/;
+
+	if( !pNurb )		/* we came to the end of the list */
+		return ( EXPP_ReturnIntError( PyExc_IndexError,
+				 "index out of range" ) );
+	
+	if (value) {
+		return ( EXPP_ReturnIntError( PyExc_RuntimeError,
+				 "assigning curves is not yet supported" ) );
+	} else {
+		BLI_remlink(&self->curve->nurb, pNurb);
+		freeNurb(pNurb);
+	}
+	return 0;
+}
+
 /*****************************************************************************/
 /* Function:    Curve_compare		                                         */
 /* Description: This compares 2 curve python types, == or != only.			 */
@@ -1396,7 +1334,7 @@ static PyObject *M_Curve_Get( PyObject * self, PyObject * args )
 							      &Curve_Type );
 			found_cur->curve = curv_iter;
 			PyList_Append( curvlist, ( PyObject * ) found_cur );
-
+			Py_DECREF(found_cur);
 			curv_iter = curv_iter->id.next;
 		}
 
@@ -1492,7 +1430,7 @@ Sets a control point "},
 	 "(nothing or integer) - returns the number of points of the specified curve"},
 	{"appendPoint", ( PyCFunction ) Curve_appendPoint, METH_VARARGS,
 	 "( int numcurve, list of coordinates) - adds a new point to end of curve"},
-	{"appendNurb", ( PyCFunction ) Curve_appendNurb, METH_VARARGS,
+	{"appendNurb", ( PyCFunction ) Curve_appendNurb, METH_O,
 	 "( new_nurb ) - adds a new nurb to the Curve"},
 	{"update", ( PyCFunction ) Curve_update, METH_NOARGS,
 	 "( ) - updates display lists after changes to Curve"},
@@ -1526,7 +1464,7 @@ static PySequenceMethods Curve_as_sequence = {
 	( intargfunc ) 0,	/* sq_repeat */
 	( intargfunc ) Curve_getNurb,	/* sq_item */
 	( intintargfunc ) 0,	/* sq_slice */
-	0,			/* sq_ass_item */
+	( intobjargproc ) Curve_setNurb,	/* sq_ass_item - only so you can do del curve[i] */
 	0,			/* sq_ass_slice */
 	( objobjproc ) 0,	/* sq_contains */
 	0,
