@@ -1,15 +1,12 @@
 /* 
- * $Id: Geometry.c 10597 2007-04-27 20:48:23Z campbellbarton $
+ * $Id: Geometry.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): Joseph Gilbert, Campbell Barton
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include "Geometry.h"
@@ -47,28 +44,33 @@
  
 #include "BKE_utildefines.h"
 #include "BLI_boxpack2d.h"
+#include "BLI_arithb.h"
 
 #define SWAP_FLOAT(a,b,tmp) tmp=a; a=b; b=tmp
 #define eul 0.000001
 
 /*-- forward declarations -- */
-static PyObject *M_Geometry_PolyFill( PyObject * self, PyObject * args );
+static PyObject *M_Geometry_PolyFill( PyObject * self, PyObject * polyLineSeq );
 static PyObject *M_Geometry_LineIntersect2D( PyObject * self, PyObject * args );
+static PyObject *M_Geometry_ClosestPointOnLine( PyObject * self, PyObject * args );
 static PyObject *M_Geometry_PointInTriangle2D( PyObject * self, PyObject * args );
 static PyObject *M_Geometry_BoxPack2D( PyObject * self, PyObject * args );
 
+
 /*-------------------------DOC STRINGS ---------------------------*/
 static char M_Geometry_doc[] = "The Blender Geometry module\n\n";
 static char M_Geometry_PolyFill_doc[] = "(veclist_list) - takes a list of polylines (each point a vector) and returns the point indicies for a polyline filled with triangles";
 static char M_Geometry_LineIntersect2D_doc[] = "(lineA_p1, lineA_p2, lineB_p1, lineB_p2) - takes 2 lines (as 4 vectors) and returns a vector for their point of intersection or None";
+static char M_Geometry_ClosestPointOnLine_doc[] = "(pt, line_p1, line_p2) - takes a point and a line and returns a (Vector, Bool) for the point on the line, and the bool so you can know if the point was between the 2 points";
 static char M_Geometry_PointInTriangle2D_doc[] = "(pt, tri_p1, tri_p2, tri_p3) - takes 4 vectors, one is the point and the next 3 define the triabgle, only the x and y are used from the vectors";
 static char M_Geometry_BoxPack2D_doc[] = "";
 /*-----------------------METHOD DEFINITIONS ----------------------*/
 struct PyMethodDef M_Geometry_methods[] = {
-	{"PolyFill", ( PyCFunction ) M_Geometry_PolyFill, METH_VARARGS, M_Geometry_PolyFill_doc},
+	{"PolyFill", ( PyCFunction ) M_Geometry_PolyFill, METH_O, M_Geometry_PolyFill_doc},
 	{"LineIntersect2D", ( PyCFunction ) M_Geometry_LineIntersect2D, METH_VARARGS, M_Geometry_LineIntersect2D_doc},
+	{"ClosestPointOnLine", ( PyCFunction ) M_Geometry_ClosestPointOnLine, METH_VARARGS, M_Geometry_ClosestPointOnLine_doc},
 	{"PointInTriangle2D", ( PyCFunction ) M_Geometry_PointInTriangle2D, METH_VARARGS, M_Geometry_PointInTriangle2D_doc},
-	{"BoxPack2D", ( PyCFunction ) M_Geometry_BoxPack2D, METH_VARARGS, M_Geometry_BoxPack2D_doc},
+	{"BoxPack2D", ( PyCFunction ) M_Geometry_BoxPack2D, METH_O, M_Geometry_BoxPack2D_doc},
 	{NULL, NULL, 0, NULL}
 };
 /*----------------------------MODULE INIT-------------------------*/
@@ -83,10 +85,10 @@ PyObject *Geometry_Init(void)
 
 /*----------------------------------Geometry.PolyFill() -------------------*/
 /* PolyFill function, uses Blenders scanfill to fill multiple poly lines */
-static PyObject *M_Geometry_PolyFill( PyObject * self, PyObject * args )
+static PyObject *M_Geometry_PolyFill( PyObject * self, PyObject * polyLineSeq )
 {
 	PyObject *tri_list; /*return this list of tri's */
-	PyObject *polyLineSeq, *polyLine, *polyVec;
+	PyObject *polyLine, *polyVec;
 	int i, len_polylines, len_polypoints;
 	
 	/* display listbase */
@@ -99,7 +101,7 @@ static PyObject *M_Geometry_PolyFill( PyObject * self, PyObject * args )
 	dispbase.first= dispbase.last= NULL;
 	
 	
-	if(!PyArg_ParseTuple ( args, "O", &polyLineSeq) || !PySequence_Check(polyLineSeq)) {
+	if(!PySequence_Check(polyLineSeq)) {
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected a sequence of poly lines" );
 	}
@@ -277,6 +279,40 @@ static PyObject *M_Geometry_LineIntersect2D( PyObject * self, PyObject * args )
 	Py_RETURN_NONE;
 }
 
+static PyObject *M_Geometry_ClosestPointOnLine( PyObject * self, PyObject * args )
+{
+	VectorObject *pt, *line_1, *line_2;
+	float pt_in[3], pt_out[3], l1[3], l2[3];
+	float lambda;
+	PyObject *ret;
+	
+	if( !PyArg_ParseTuple ( args, "O!O!O!",
+	&vector_Type, &pt,
+	&vector_Type, &line_1,
+	&vector_Type, &line_2)
+	  )
+		return ( EXPP_ReturnPyObjError
+				( PyExc_TypeError, "expected 3 vector types\n" ) );
+	
+	/* accept 2d verts */
+	if (pt->size==3) { VECCOPY(pt_in, pt->vec);}
+	else { pt_in[2]=0.0;	VECCOPY2D(pt_in, pt->vec) }
+	
+	if (line_1->size==3) { VECCOPY(l1, line_1->vec);}
+	else { l1[2]=0.0;	VECCOPY2D(l1, line_1->vec) }
+	
+	if (line_2->size==3) { VECCOPY(l2, line_2->vec);}
+	else { l2[2]=0.0;	VECCOPY2D(l2, line_2->vec) }
+	
+	/* do the calculation */
+	lambda = lambda_cp_line_ex(pt_in, l1, l2, pt_out);
+	
+	ret = PyTuple_New(2);
+	PyTuple_SET_ITEM( ret, 0, newVectorObject(pt_out, 3, Py_NEW) );
+	PyTuple_SET_ITEM( ret, 1, PyFloat_FromDouble(lambda) );
+	return ret;
+}
+
 #define SIDE_OF_LINE(pa,pb,pp)	((pa[0]-pp[0])*(pb[1]-pp[1]))-((pb[0]-pp[0])*(pa[1]-pp[1]))
 #define POINT_IN_TRI(p0,p1,p2,p3)	((SIDE_OF_LINE(p1,p2,p0)>=0) && (SIDE_OF_LINE(p2,p3,p0)>=0) && (SIDE_OF_LINE(p3,p1,p0)>=0))
 
@@ -361,18 +397,16 @@ void boxPack_ToPyObject(PyObject * value, boxPack **boxarray)
 }
 
 
-static PyObject *M_Geometry_BoxPack2D( PyObject * self, PyObject * args )
+static PyObject *M_Geometry_BoxPack2D( PyObject * self, PyObject * boxlist )
 {
-	PyObject *boxlist; /*return this list of tri's */
 	boxPack *boxarray;
 	float tot_width, tot_height;
 	int len;
 	int error;
 	
-	if(!PyArg_ParseTuple ( args, "O", &boxlist) || !PyList_Check(boxlist)) {
+	if(!PyList_Check(boxlist))
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected a sequence of boxes [[x,y,w,h], ... ]" );
-	}
 	
 	len = PyList_Size( boxlist );
 	
