@@ -1,15 +1,12 @@
 /**
  * $Id: IDProp.c
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +22,7 @@
  *
  * Contributor(s): Joseph Eagar
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
  
 #include "DNA_ID.h"
@@ -49,7 +46,7 @@ extern PyTypeObject IDGroup_Iter_Type;
 
 PyObject *IDGroup_repr( BPy_IDProperty *self )
 {
-	return Py_BuildValue( "s", "(ID Property)" );
+	return PyString_FromString( "(ID Property)" );
 }
 
 extern PyTypeObject IDGroup_Type;
@@ -58,11 +55,11 @@ PyObject *BPy_IDGroup_WrapData( ID *id, IDProperty *prop )
 {
 	switch ( prop->type ) {
 		case IDP_STRING:
-			return Py_BuildValue( "s", prop->data.pointer );
+			return PyString_FromString( prop->data.pointer );
 		case IDP_INT:
-			return Py_BuildValue( "i", prop->data.val );
+			return PyInt_FromLong( (long)prop->data.val );
 		case IDP_FLOAT:
-			return Py_BuildValue( "f", *(float*)(&prop->data.val) );
+			return PyFloat_FromDouble( (double)(*(float*)(&prop->data.val)) );
 		case IDP_GROUP:
 			/*blegh*/
 			{
@@ -86,9 +83,6 @@ PyObject *BPy_IDGroup_WrapData( ID *id, IDProperty *prop )
 				array->prop = prop;
 				return (PyObject*) array;
 			}
-		case IDP_MATRIX:
-		case IDP_VECTOR:
-			break;
 	}
 	Py_RETURN_NONE;
 }
@@ -141,12 +135,12 @@ int BPy_IDGroup_SetData(BPy_IDProperty *self, IDProperty *prop, PyObject *value)
 	return 0;
 }
 
-PyObject *BPy_IDGroup_GetName(BPy_IDProperty *self)
+PyObject *BPy_IDGroup_GetName(BPy_IDProperty *self, void *bleh)
 {
-	return Py_BuildValue("s", self->prop->name);
+	return PyString_FromString(self->prop->name);
 }
 
-int BPy_IDGroup_SetName(BPy_IDProperty *self, PyObject *value)
+int BPy_IDGroup_SetName(BPy_IDProperty *self, PyObject *value, void *bleh)
 {
 	char *st;
 	if (!PyString_Check(value))
@@ -162,7 +156,7 @@ int BPy_IDGroup_SetName(BPy_IDProperty *self, PyObject *value)
 
 PyObject *BPy_IDGroup_GetType(BPy_IDProperty *self)
 {
-	return Py_BuildValue("i", self->prop->type);
+	return PyInt_FromLong((long)self->prop->type);
 }
 
 static PyGetSetDef BPy_IDGroup_getseters[] = {
@@ -206,7 +200,7 @@ PyObject *BPy_IDGroup_Map_GetItem(BPy_IDProperty *self, PyObject *item)
 /*returns NULL on success, error string on failure*/
 char *BPy_IDProperty_Map_ValidateAndCreate(char *name, IDProperty *group, PyObject *ob)
 {
-	IDProperty *prop = NULL, *prop2=NULL, *prev=NULL;
+	IDProperty *prop = NULL;
 	IDPropertyTemplate val = {0};
 	
 	if (PyFloat_Check(ob)) {
@@ -285,15 +279,7 @@ char *BPy_IDProperty_Map_ValidateAndCreate(char *name, IDProperty *group, PyObje
 		Py_XDECREF(vals);
 	} else return "invalid property value";
 	
-	prop2 = IDP_GetPropertyFromGroup(group, prop->name);
-	if (prop2) {
-		prev=prop2->prev; /*we want to insert new property in same place as old*/
-		IDP_RemFromGroup(group, prop2);
-		IDP_FreeProperty(prop2);
-		MEM_freeN(prop2);
-	}
-
-	IDP_InsertToGroup(group, prev, prop);
+	IDP_ReplaceInGroup(group, prop);
 	return NULL;
 }
 
@@ -343,13 +329,13 @@ PyObject *BPy_IDGroup_MapDataToPy(IDProperty *prop)
 {
 	switch (prop->type) {
 		case IDP_STRING:
-			return Py_BuildValue("s", prop->data.pointer);
+			return PyString_FromString(prop->data.pointer);
 			break;
 		case IDP_FLOAT:
-			return Py_BuildValue("f", *((float*)&prop->data.val));
+			return PyFloat_FromDouble(*((float*)&prop->data.val));
 			break;
 		case IDP_INT:
-			return Py_BuildValue("i", prop->data.val);
+			return PyInt_FromLong( (long)prop->data.val );
 			break;
 		case IDP_ARRAY:
 		{
@@ -362,8 +348,11 @@ PyObject *BPy_IDGroup_MapDataToPy(IDProperty *prop)
 			
 			for (i=0; i<prop->len; i++) {
 				if (prop->subtype == IDP_FLOAT)
-					PyList_SetItem(seq, i, Py_BuildValue("f", ((float*)prop->data.pointer)[i]));
-				else PyList_SetItem(seq, i, Py_BuildValue("i",   ((int*)prop->data.pointer)[i]));
+						PyList_SetItem(seq, i,
+						PyFloat_FromDouble(((float*)prop->data.pointer)[i]));
+				
+				else 	PyList_SetItem(seq, i,
+						PyInt_FromLong(((int*)prop->data.pointer)[i]));
 			}
 			return seq;
 		}
@@ -392,16 +381,15 @@ PyObject *BPy_IDGroup_MapDataToPy(IDProperty *prop)
 					   "eek!! a property exists with a bad type code!!!" );
 }
 
-PyObject *BPy_IDGroup_Pop(BPy_IDProperty *self, PyObject *vars)
+PyObject *BPy_IDGroup_Pop(BPy_IDProperty *self, PyObject *value)
 {
 	IDProperty *loop;
 	PyObject *pyform;
-	char *name;
-	int ok = PyArg_ParseTuple(vars, "s", &name);
+	char *name = PyString_AsString(value);
 	
-	if (!ok) {
+	if (!name) {
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
-		   "pop expected at least 1 arguments, got 0" );
+		   "pop expected at least 1 argument, got 0" );
 	}
 	
 	for (loop=self->prop->data.group.first; loop; loop=loop->next) {
@@ -440,17 +428,34 @@ PyObject *BPy_IDGroup_IterItems(BPy_IDProperty *self)
 
 PyObject *BPy_IDGroup_GetKeys(BPy_IDProperty *self)
 {
-	PyObject *seq = PyList_New(self->prop->len), *st;
+	PyObject *seq = PyList_New(self->prop->len);
 	IDProperty *loop;
-	int i;
+	int i, j;
 
 	if (!seq) 
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 		   "PyList_New() failed" );
 		   
-	for (i=0, loop=self->prop->data.group.first; loop; loop=loop->next, i++) {
-		st = Py_BuildValue("s", loop->name);
-		PyList_SetItem(seq, i, st);
+	for (i=0, loop=self->prop->data.group.first; loop; loop=loop->next, i++)
+		PyList_SetItem(seq, i, PyString_FromString(loop->name));
+	
+	if (i != self->prop->len) {
+		printf("ID Property Error found and corrected in BPy_IDGroup_GetKeys!\n");
+		
+		/*fill rest of list with valid references to None*/
+		for (j=i; j<self->prop->len; j++) {
+			Py_INCREF(Py_None);
+			PyList_SetItem(seq, j, Py_None);
+		}
+		
+		/*set correct group length*/
+		self->prop->len = i;
+		
+		/*free the old list*/
+		Py_DECREF(seq);
+		
+		/*call self again*/
+		return BPy_IDGroup_GetKeys(self);		
 	}
 	
 	return seq;
@@ -460,7 +465,7 @@ PyObject *BPy_IDGroup_GetValues(BPy_IDProperty *self)
 {
 	PyObject *seq = PyList_New(self->prop->len);
 	IDProperty *loop;
-	int i;
+	int i, j;
 
 	if (!seq) 
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
@@ -469,16 +474,35 @@ PyObject *BPy_IDGroup_GetValues(BPy_IDProperty *self)
 	for (i=0, loop=self->prop->data.group.first; loop; loop=loop->next, i++) {
 		PyList_SetItem(seq, i, BPy_IDGroup_WrapData(self->id, loop));
 	}
-	
+
+	if (i != self->prop->len) {
+		printf("ID Property Error found and corrected in BPy_IDGroup_GetValues!\n");
+		
+		/*fill rest of list with valid references to None*/
+		for (j=i; j<self->prop->len; j++) {
+			Py_INCREF(Py_None);
+			PyList_SetItem(seq, j, Py_None);
+		}
+		
+		/*set correct group length*/
+		self->prop->len = i;
+		
+		/*free the old list*/
+		Py_DECREF(seq);
+		
+		/*call self again*/
+		return BPy_IDGroup_GetValues(self);		
+	}
+		
 	return seq;
 }
 
-PyObject *BPy_IDGroup_HasKey(BPy_IDProperty *self, PyObject *vars)
+PyObject *BPy_IDGroup_HasKey(BPy_IDProperty *self, PyObject *value)
 {
 	IDProperty *loop;
-	char *name;
+	char *name = PyString_AsString(value);
 	
-	if (!PyArg_ParseTuple(vars, "s", &name))
+	if (!name)
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 		   "expected a string");
 		   
@@ -492,7 +516,7 @@ PyObject *BPy_IDGroup_HasKey(BPy_IDProperty *self, PyObject *vars)
 PyObject *BPy_IDGroup_Update(BPy_IDProperty *self, PyObject *vars)
 {
 	PyObject *pyob, *pkey, *pval;
-	int i=0;
+	Py_ssize_t i=0;
 	
 	if (PySequence_Size(vars) != 1)
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
@@ -517,7 +541,7 @@ PyObject *BPy_IDGroup_ConvertToPy(BPy_IDProperty *self)
 }
 
 static struct PyMethodDef BPy_IDGroup_methods[] = {
-	{"pop", (PyCFunction)BPy_IDGroup_Pop, METH_VARARGS,
+	{"pop", (PyCFunction)BPy_IDGroup_Pop, METH_O,
 		"pop an item from the group; raises KeyError if the item doesn't exist."},
 	{"iteritems", (PyCFunction)BPy_IDGroup_IterItems, METH_NOARGS,
 		"iterate through the items in the dict; behaves like dictionary method iteritems."},
@@ -525,7 +549,7 @@ static struct PyMethodDef BPy_IDGroup_methods[] = {
 		"get the keys associated with this group as a list of strings."},
 	{"values", (PyCFunction)BPy_IDGroup_GetValues, METH_NOARGS,
 		"get the values associated with this group."},
-	{"has_key", (PyCFunction)BPy_IDGroup_HasKey, METH_VARARGS,
+	{"has_key", (PyCFunction)BPy_IDGroup_HasKey, METH_O,
 		"returns true if the group contains a key, false if not."},
 	{"update", (PyCFunction)BPy_IDGroup_Update, METH_VARARGS,
 		"updates the values in the group with the values of another or a dict."},
@@ -623,18 +647,18 @@ PyObject *BPy_Wrap_IDProperty(ID *id, IDProperty *prop, IDProperty *parent)
 
 PyObject *IDArray_repr(BPy_IDArray *self)
 {
-	return Py_BuildValue("s", "(ID Array)");
+	return PyString_FromString("(ID Array)");
 }
 
 
 PyObject *BPy_IDArray_GetType(BPy_IDArray *self)
 {
-	return Py_BuildValue("i", self->prop->subtype);
+	return PyInt_FromLong( (long)self->prop->subtype );
 }
 
 PyObject *BPy_IDArray_GetLen(BPy_IDArray *self)
 {
-	return Py_BuildValue("i", self->prop->len);
+	return PyInt_FromLong( (long)self->prop->len );
 }
 
 static PyGetSetDef BPy_IDArray_getseters[] = {
@@ -662,10 +686,10 @@ PyObject *BPy_IDArray_GetItem(BPy_IDArray *self, int index)
 
 	switch (self->prop->subtype) {
 		case IDP_FLOAT:
-			return Py_BuildValue("f", ((float*)self->prop->data.pointer)[index]);
+			return PyFloat_FromDouble( (double)(((float*)self->prop->data.pointer)[index]));
 			break;
 		case IDP_INT:
-			return Py_BuildValue("i", ((int*)self->prop->data.pointer)[index]);
+			return PyInt_FromLong( (long)((int*)self->prop->data.pointer)[index] );
 			break;
 	}
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
@@ -814,20 +838,25 @@ PyObject *IDGroup_Iter_iterself(PyObject *self)
 
 PyObject *IDGroup_Iter_repr(BPy_IDGroup_Iter *self)
 {
-	return Py_BuildValue("s", "(ID Property Group)");
+	return PyString_FromString("(ID Property Group)");
 }
 
 PyObject *BPy_Group_Iter_Next(BPy_IDGroup_Iter *self)
 {
 	IDProperty *cur=NULL;
+	PyObject *tmpval;
+	PyObject *ret;
 
 	if (self->cur) {
 		cur = self->cur;
 		self->cur = self->cur->next;
 		if (self->mode == IDPROP_ITER_ITEMS) {
-			return Py_BuildValue("[s, O]", cur->name, BPy_IDGroup_WrapData(self->group->id, cur));
+			tmpval = BPy_IDGroup_WrapData(self->group->id, cur);
+			ret = Py_BuildValue("[s, O]", cur->name, tmpval);
+			Py_DECREF(tmpval);
+			return ret;
 		} else {
-			return Py_BuildValue("s", cur->name);
+			return PyString_FromString(cur->name);
 		}
 	} else {
 		return EXPP_ReturnPyObjError( PyExc_StopIteration,
