@@ -1,15 +1,12 @@
 /* 
- * $Id: Key.c 11965 2007-09-08 00:44:41Z bdiego $
+ * $Id: Key.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,7 +25,7 @@
  * Contributor(s): Pontus Lidman, Johnny Matthews, Ken Hughes,
  *   Michael Reimpell
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include "Key.h" /*This must come first*/
@@ -41,6 +38,7 @@
 #include <BKE_main.h>
 #include <BKE_curve.h>
 #include <BKE_library.h>
+#include <BKE_utildefines.h>
 #include "BIF_space.h"
 
 #include "Ipocurve.h"
@@ -60,9 +58,6 @@
 #define KEY_TYPE_CURVE   1
 #define KEY_TYPE_LATTICE 2
 
-/* macro from blenkernel/intern/key.c:98 */
-#define GS(a)	(*((short *)(a)))
-
 static int Key_compare( BPy_Key * a, BPy_Key * b );
 static PyObject *Key_repr( BPy_Key * self );
 static void Key_dealloc( BPy_Key * self );
