@@ -1,15 +1,12 @@
 /* 
- * $Id: Lattice.c 10942 2007-06-16 04:20:19Z campbellbarton $
+ * $Id: Lattice.c 14456 2008-04-17 21:14:55Z campbellbarton $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): Joseph Gilbert
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include "Lattice.h" /*This must come first*/
@@ -69,7 +66,7 @@ static char M_Lattice_doc[] = "The Blender Lattice module\n\n";
 
 static char M_Lattice_New_doc[] = "() - return a new Lattice object";
 
-static char M_Lattice_Get_doc[] = "() - geta a Lattice from blender";
+static char M_Lattice_Get_doc[] = "() - get a Lattice from blender";
 
 /*****************************************************************************/
 /* Python method structure definition for Blender.Lattice module:	*/
@@ -679,7 +676,7 @@ static PyObject *Lattice_getLatSize(BPy_Lattice * self)
 static PyObject *Lattice_getAxisType(BPy_Lattice * self, void * type)
 {
 	char interp_type = 0;
-	switch ( (int)type ) {
+	switch ( GET_INT_FROM_POINTER(type) ) {
 	case 0:
 		interp_type = self->lattice->typeu;
 		break;
@@ -708,13 +705,13 @@ static PyObject *Lattice_getAxisType(BPy_Lattice * self, void * type)
 static PyGetSetDef BPy_Lattice_getseters[] = {
 	GENERIC_LIB_GETSETATTR,
 	{"width", (getter)Lattice_getWidth, (setter)NULL,
-	 "lattice U sibdivision ", NULL},
+	 "lattice U subdivision ", NULL},
 	{"height", (getter)Lattice_getHeight, (setter)NULL,
-	 "lattice V sibdivision", NULL},
+	 "lattice V subdivision", NULL},
 	{"depth", (getter)Lattice_getDepth, (setter)NULL,
-	 "lattice W sibdivision", NULL},
+	 "lattice W subdivision", NULL},
 	{"latSize", (getter)Lattice_getLatSize, (setter)NULL,
-	 "lattice W sibdivision", NULL},	 
+	 "lattice W subdivision", NULL},	 
 	 
 	{"widthType", (getter)Lattice_getAxisType, NULL,
 	 "lattice U interpolation type", (void *)0},
