@@ -1,18 +1,15 @@
 /**
- * $Id: Library.c,v 1.23 2007/04/20 18:48:30 campbellbarton Exp $
+ * $Id: Library.c 14669 2008-05-04 09:41:15Z campbellbarton $
  *
  * Blender.Library BPython module implementation.
  * This submodule has functions to append data from .blend files.
  * 
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,7 +27,7 @@
  *
  * Contributor(s): Willian P. Germano, Campbell Barton, Ken Hughes
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
 */
 
 /************************************************************/
@@ -67,7 +64,7 @@ static PyObject *M_Library_Open( PyObject * self, PyObject * args );
 static PyObject *M_Library_Close( PyObject * self );
 static PyObject *M_Library_GetName( PyObject * self );
 static PyObject *M_Library_Update( PyObject * self );
-static PyObject *M_Library_Datablocks( PyObject * self, PyObject * args );
+static PyObject *M_Library_Datablocks( PyObject * self, PyObject * value );
 static PyObject *oldM_Library_Load( PyObject * self, PyObject * args );
 static PyObject *M_Library_LinkableGroups( PyObject * self );
 static PyObject *M_Library_LinkedLibs( PyObject * self );
@@ -123,14 +120,14 @@ static char Library_LinkedLibs_doc[] =
  * Python method structure definition for Blender.Library submodule.
  */
 struct PyMethodDef oldM_Library_methods[] = {
-	{"Open", M_Library_Open, METH_VARARGS, Library_Open_doc},
+	{"Open", M_Library_Open, METH_O, Library_Open_doc},
 	{"Close", ( PyCFunction ) M_Library_Close, METH_NOARGS,
 	 Library_Close_doc},
 	{"GetName", ( PyCFunction ) M_Library_GetName, METH_NOARGS,
 	 Library_GetName_doc},
 	{"Update", ( PyCFunction ) M_Library_Update, METH_NOARGS,
 	 Library_Update_doc},
-	{"Datablocks", M_Library_Datablocks, METH_VARARGS,
+	{"Datablocks", M_Library_Datablocks, METH_O,
 	 Library_Datablocks_doc},
 	{"Load", oldM_Library_Load, METH_VARARGS, Library_Load_doc},
 	{"LinkableGroups", ( PyCFunction ) M_Library_LinkableGroups,
@@ -147,9 +144,9 @@ struct PyMethodDef oldM_Library_methods[] = {
  * Only one can be open at a time, so this function also closes
  * the previously opened file, if any.
  */
-static PyObject *M_Library_Open( PyObject * self, PyObject * args )
+static PyObject *M_Library_Open( PyObject * self, PyObject * value )
 {
-	char *fname = NULL;
+	char *fname = PyString_AsString(value);
 	char filename[FILE_MAXDIR+FILE_MAXFILE];
 	char fname1[FILE_MAXDIR+FILE_MAXFILE];
 	
@@ -157,7 +154,7 @@ static PyObject *M_Library_Open( PyObject * self, PyObject * args )
 
 	bpy_relative= 0; /* assume non relative each time we load */
 	
-	if( !PyArg_ParseTuple( args, "s", &fname ) ) {
+	if( !fname ) {
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected a .blend filename" );
 	}
@@ -169,13 +166,13 @@ static PyObject *M_Library_Open( PyObject * self, PyObject * args )
 	
 	/* copy the name to make it absolute so BLO_blendhandle_from_file doesn't complain */
 	BLI_strncpy(fname1, fname, sizeof(fname1)); 
-	BLI_convertstringcode(fname1, G.sce, 0); /* make absolute */
+	BLI_convertstringcode(fname1, G.sce); /* make absolute */
 	
    	/* G.sce = last file loaded, save for UI and restore after opening file */
 	BLI_strncpy(filename, G.sce, sizeof(filename));
 	bpy_openlib = BLO_blendhandle_from_file( fname1 );
 	BLI_strncpy(G.sce, filename, sizeof(filename)); 
-	
+
 	if( !bpy_openlib )
 		return EXPP_ReturnPyObjError( PyExc_IOError, "file not found" );
 
@@ -190,7 +187,7 @@ static PyObject *M_Library_Open( PyObject * self, PyObject * args )
 	if( bpy_openlibname )
 		PyOS_snprintf( bpy_openlibname, len, "%s", fname1 );
 
-	return Py_BuildValue( "i", 1 );
+	Py_RETURN_TRUE;
 }
 
 /**
@@ -244,9 +241,9 @@ static PyObject *M_Library_GetName( PyObject * self )
  * Return a list with all items of a given datablock type
  * (like 'Object', 'Mesh', etc.) in the open library file.
  */
-static PyObject *M_Library_Datablocks( PyObject * self, PyObject * args )
+static PyObject *M_Library_Datablocks( PyObject * self, PyObject * value )
 {
-	char *name = NULL;
+	char *name = PyString_AsString(value);
 	int blocktype = 0;
 	LinkNode *l = NULL, *names = NULL;
 	PyObject *list = NULL;
@@ -256,7 +253,7 @@ static PyObject *M_Library_Datablocks( PyObject * self, PyObject * args )
 					      "no library file: open one first with Blender.Lib_Open(filename)" );
 	}
 
-	if( !PyArg_ParseTuple( args, "s", &name ) ) {
+	if( !name ) {
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected a string (datablock type) as argument." );
 	}
@@ -275,16 +272,15 @@ static PyObject *M_Library_Datablocks( PyObject * self, PyObject * args )
 		list = PyList_New( BLI_linklist_length( names ) );
 		for( l = names; l; l = l->next ) {
 			PyList_SET_ITEM( list, counter,
-					 Py_BuildValue( "s",
-							( char * ) l->link ) );
+					PyString_FromString( ( char * ) l->link ) );
 			counter++;
 		}
 		BLI_linklist_free( names, free );	/* free linklist *and* each node's data */
-		return list;
+	} else {
+		list = PyList_New( 0 );
 	}
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	return list;
 }
 
 /**
@@ -346,6 +342,7 @@ static PyObject *oldM_Library_Load( PyObject * self, PyObject * args )
 	int blocktype = 0;
 	int linked = 0;
 
+	
 	if( !bpy_openlib ) {
 		return EXPP_ReturnPyObjError( PyExc_IOError,
 					      "no library file: you need to open one, first." );
@@ -361,12 +358,19 @@ static PyObject *oldM_Library_Load( PyObject * self, PyObject * args )
 	if( !blocktype )
 		return EXPP_ReturnPyObjError( PyExc_NameError,
 					      "no such Blender datablock type" );
-	
+
 	if (linked)
-		BLO_script_library_append( bpy_openlib, bpy_openlibname, name, blocktype, FILE_LINK, G.scene);
+		BLO_script_library_append( &bpy_openlib, bpy_openlibname, name, blocktype, FILE_LINK, G.scene);
 	else
-		BLO_script_library_append( bpy_openlib, bpy_openlibname, name, blocktype, 0, G.scene);
-	
+		BLO_script_library_append( &bpy_openlib, bpy_openlibname, name, blocktype, 0, G.scene);
+
+	/*
+		NOTE:  BLO_script_library_append() can close the library if there is
+		an endian issue.  if this happened, reopen for the next call.
+	*/
+	if ( !bpy_openlib )
+		bpy_openlib = BLO_blendhandle_from_file( bpy_openlibname );
+
 	if( update ) {
 		M_Library_Update( self );
 		Py_DECREF( Py_None );	/* incref'ed by above function */
@@ -479,7 +483,7 @@ static BlendHandle *open_library( char *filename, char *longFilename )
 
 	/* get complete file name if necessary */
 	BLI_strncpy( longFilename, filename, FILE_MAX ); 
-	BLI_convertstringcode( longFilename, G.sce, 0 );
+	BLI_convertstringcode( longFilename, G.sce );
 
 	/* throw exceptions for wrong file type, cyclic reference */
 	if( !BLO_has_bfile_extension(longFilename) ) {
@@ -511,11 +515,12 @@ static BlendHandle *open_library( char *filename, char *longFilename )
  */
 
 static PyObject *CreatePyObject_LibData( int idtype, int kind,
-		void *name, void *iter, char *filename )
+		void *name, void *iter, char *filename, int rel )
 {
 	BPy_LibraryData *seq = PyObject_NEW( BPy_LibraryData, &LibraryData_Type);
 	seq->iter = iter;		/* the name list (for iterators) */
 	seq->type = idtype;		/* the Blender ID type */
+	seq->rel =  rel;		/* relative or absolute library */
 	seq->kind = kind;		/* used by Blender Objects */
 	seq->name = name; 		/* object name, iterator name list, or NULL */
 							/* save the library name */
@@ -528,13 +533,13 @@ static PyObject *CreatePyObject_LibData( int idtype, int kind,
  * which can be linked/appended to a scene.
  */
 
-static PyObject *lib_link_or_append( BPy_LibraryData *self, PyObject * args,
+static PyObject *lib_link_or_append( BPy_LibraryData *self, PyObject * value,
 		int mode )
 {
-	char *name;
+	char *name = PyString_AsString(value);
 
 	/* get the name of the data used wants to append */
-	if( !PyArg_ParseTuple( args, "s", &name ) )
+	if( !name )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 			"expected a string" );
 
@@ -556,7 +561,8 @@ static PyObject *lib_link_or_append( BPy_LibraryData *self, PyObject * args,
 		/* otherwise, create a pseudo object ready for appending or linking */
 
 		return CreatePyObject_LibData( ID_OB, mode, 
-				BLI_strdupn( name, strlen( name ) ), NULL, self->filename );
+				BLI_strdupn( name, strlen( name ) ), NULL, self->filename,
+				self->rel );
 	}
 }
 
@@ -582,6 +588,9 @@ PyObject *LibraryData_importLibData( BPy_LibraryData *self, char *name,
 	if( !openlib )
 		return NULL;
 
+	/* fix any /foo/../foo/ */
+	BLI_cleanup_file(NULL, longFilename); 
+
 	/* find all datablocks for the specified type */
 	names = BLO_blendhandle_get_datablock_names ( openlib, self->type ); 
 
@@ -612,15 +621,15 @@ PyObject *LibraryData_importLibData( BPy_LibraryData *self, char *name,
 	}
 
 	/* import from the libary */
-	BLO_script_library_append( openlib, longFilename, name, self->type, mode,
-			scene );
+	BLO_script_library_append( &openlib, longFilename, name, self->type, 
+			mode | self->rel, scene );
 
 	/*
 	 * locate the library.  If this is an append, make the data local.  If it
 	 * is link, we need the library for later
 	 */
 	for( lib = G.main->library.first; lib; lib = lib->id.next )
-		if( strcmp( longFilename, lib->name ) == 0 ) {
+		if( strcmp( longFilename, lib->filename ) == 0) {
 			if( mode != FILE_LINK ) {
 				all_local( lib, 1 );
 				/* important we unset, otherwise these object wont
@@ -674,16 +683,16 @@ PyObject *LibraryData_importLibData( BPy_LibraryData *self, char *name,
 
 /* .append(): make a local copy of the library's data (except for objects) */
 
-static PyObject *LibraryData_getAppend( BPy_LibraryData *self, PyObject * args)
+static PyObject *LibraryData_getAppend( BPy_LibraryData *self, PyObject * value)
 {
-	return lib_link_or_append( self, args, OBJECT_IS_APPEND );
+	return lib_link_or_append( self, value, OBJECT_IS_APPEND );
 }
 
 /* .link(): make a link to the library's data (except for objects) */
 
-static PyObject *LibraryData_getLink( BPy_LibraryData *self, PyObject * args)
+static PyObject *LibraryData_getLink( BPy_LibraryData *self, PyObject * value)
 {
-	return lib_link_or_append( self, args, OBJECT_IS_LINK );
+	return lib_link_or_append( self, value, OBJECT_IS_LINK );
 }
 
 /************************************************************************
@@ -713,7 +722,7 @@ static PyObject *LibraryData_getIter( BPy_LibraryData * self )
 
 	/* build an iterator object for the name list */
 	return CreatePyObject_LibData( self->type, OTHER, names,
-			names, self->filename );
+			names, self->filename, self->rel );
 }
 
 /* Return next name. */
@@ -747,9 +756,9 @@ static PyObject *LibraryData_nextIter( BPy_LibraryData * self )
  ************************************************************************/
 
 static struct PyMethodDef BPy_LibraryData_methods[] = {
-	{"append", (PyCFunction)LibraryData_getAppend, METH_VARARGS,
+	{"append", (PyCFunction)LibraryData_getAppend, METH_O,
 	 "(str) - create new data from library"},
-	{"link", (PyCFunction)LibraryData_getLink, METH_VARARGS,
+	{"link", (PyCFunction)LibraryData_getLink, METH_O,
 	 "(str) - link data from library"},
 	{NULL, NULL, 0, NULL}
 };
@@ -937,8 +946,8 @@ PyTypeObject LibraryData_Type = {
 
 static PyObject *LibraryData_CreatePyObject( BPy_Library *self, void *mode )
 {
-	return CreatePyObject_LibData( (int)mode, OTHER, NULL, NULL,
-			self->filename );
+	return CreatePyObject_LibData( GET_INT_FROM_POINTER(mode), OTHER, NULL, NULL,
+			self->filename, self->rel);
 }
 
 /************************************************************
@@ -968,6 +977,39 @@ static int Library_setFilename( BPy_Library * self, PyObject * args )
 	return 0;
 }
 
+/*
+ * Return the library's name.  The format depends on whether the library is 
+ * accessed as relative or absolute.
+ */
+
+static PyObject *Library_getName( BPy_Library * self )
+{
+	Library *lib;
+	BlendHandle *openlib;
+	char longFilename[FILE_MAX];
+
+	/* try to open the library */
+	openlib = open_library( self->filename, longFilename );
+	if( openlib ) {
+		BLO_blendhandle_close( openlib );
+		/* remove any /../ or /./ junk */
+		BLI_cleanup_file(NULL, longFilename); 
+
+		/* search the loaded libraries for a match */
+		for( lib = G.main->library.first; lib; lib = lib->id.next )
+			if( strcmp( longFilename, lib->filename ) == 0) {
+				return PyString_FromString( lib->name );
+			}
+
+		/* library not found in memory */
+		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
+				"library not loaded" );
+	}
+	/* could not load library */
+	return EXPP_ReturnPyObjError( PyExc_IOError, "library not found" );
+}
+
+
 /************************************************************************
  * Python Library_type attributes get/set structure
  ************************************************************************/
@@ -977,6 +1019,10 @@ static PyGetSetDef Library_getseters[] = {
 	 (getter)Library_getFilename, (setter)Library_setFilename,
 	 "library filename",
 	 NULL},
+	{"name",
+	 (getter)Library_getName, (setter)NULL,
+	 "library name (as used by Blender)",
+	 NULL},
 	{"objects",
 	 (getter)LibraryData_CreatePyObject, (setter)NULL,
 	 "objects from the library",
@@ -1064,18 +1110,25 @@ static PyGetSetDef Library_getseters[] = {
 
 static PyObject *M_Library_Load(PyObject *self, PyObject * args)
 {
-	char *filename;
+	char *filename = NULL;
+	PyObject *relative = NULL;
 	BPy_Library *lib;
 
-	if( !PyArg_ParseTuple( args, "s", &filename ) )
+	if( !PyArg_ParseTuple( args, "s|O", &filename, &relative ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
-			"expected a string" );
+			"expected strings and optional bool as arguments." );
 
 	/* try to create a new object */
 	lib = (BPy_Library *)PyObject_NEW( BPy_Library, &Library_Type );
 	if( !lib )
 		return NULL;
 
+	/* save relative flag value */
+	if( relative && PyObject_IsTrue(relative) )
+		lib->rel = FILE_STRINGCODE;
+	else
+		lib->rel = 0;
+
 	/* assign the library filename for future use, then return */
 	BLI_strncpy( lib->filename, filename, sizeof(lib->filename) );
 
