@@ -1,15 +1,12 @@
 /*
- * $Id: Modifier.c,v 1.26 2007/04/30 16:49:12 artificer Exp $
+ * $Id: Modifier.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,7 +21,7 @@
  *
  * Contributor(s): Ken Hughes, Campbell Barton
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 /* TODO, accessing a modifier sequence of a deleted object will crash blender at the moment, not sure how to fix this. */
@@ -44,6 +41,7 @@
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
 #include "MEM_guardedalloc.h"
+#include "BDR_editobject.h"
 #include "butspace.h"
 #include "blendef.h"
 #include "mydevice.h"
@@ -78,34 +76,36 @@ enum mod_constants {
 	EXPP_MOD_ENABLE_Z, /*SMOOTH, CAST*/
 	EXPP_MOD_TYPES, /*SUBSURF, CAST*/
 	
-	/*SUBSURF SPESIFIC*/
+	/*SUBSURF SPECIFIC*/
 	EXPP_MOD_LEVELS,
 	EXPP_MOD_RENDLEVELS,
 	EXPP_MOD_OPTIMAL,
 	EXPP_MOD_UV,
 	
-	/*ARMATURE SPESIFIC*/
+	/*ARMATURE SPECIFIC*/
 	EXPP_MOD_ENVELOPES,
 	
-	/*ARRAY SPESIFIC*/
+	/*ARRAY SPECIFIC*/
 	EXPP_MOD_OBJECT_OFFSET,
 	EXPP_MOD_OBJECT_CURVE,
 	EXPP_MOD_OFFSET_VEC,
 	EXPP_MOD_SCALE_VEC,
 	EXPP_MOD_MERGE_DIST,
 	
-	/*BUILD SPESIFIC*/
+	/*BUILD SPECIFIC*/
 	EXPP_MOD_START,
 	EXPP_MOD_SEED,
 	EXPP_MOD_RANDOMIZE,
 
-	/*MIRROR SPESIFIC*/
-	EXPP_MOD_AXIS,
+	/*MIRROR SPECIFIC*/
+	EXPP_MOD_AXIS_X,
+	EXPP_MOD_AXIS_Y,
+	EXPP_MOD_AXIS_Z,
 
-	/*DECIMATE SPESIFIC*/
+	/*DECIMATE SPECIFIC*/
 	EXPP_MOD_RATIO,
 
-	/*WAVE SPESIFIC*/
+	/*WAVE SPECIFIC*/
 	EXPP_MOD_STARTX,
 	EXPP_MOD_STARTY,
 	EXPP_MOD_HEIGHT,
@@ -116,10 +116,10 @@ enum mod_constants {
 	EXPP_MOD_LIFETIME,
 	EXPP_MOD_TIMEOFFS,
 	
-	/*BOOLEAN SPESIFIC*/
+	/*BOOLEAN SPECIFIC*/
 	EXPP_MOD_OPERATION,
 
-	/*EDGE SPLIT SPESIFIC */
+	/*EDGE SPLIT SPECIFIC */
 	EXPP_MOD_EDGESPLIT_ANGLE,
 	EXPP_MOD_EDGESPLIT_FROM_ANGLE,
 	EXPP_MOD_EDGESPLIT_FROM_SHARP,
@@ -515,8 +515,15 @@ static PyObject *mirror_getter( BPy_Modifier * self, int type )
 		return PyFloat_FromDouble( (double)md->tolerance );
 	case EXPP_MOD_FLAG:
 		return PyBool_FromLong( (long)( md->flag & MOD_MIR_CLIPPING ) ) ;
-	case EXPP_MOD_AXIS:
-		return PyInt_FromLong( (long)md->axis );
+	case EXPP_MOD_AXIS_X:
+		return PyBool_FromLong( ( long ) 
+				( md->flag & MOD_MIR_AXIS_X ) ) ;
+	case EXPP_MOD_AXIS_Y:
+		return PyBool_FromLong( ( long ) 
+				( md->flag & MOD_MIR_AXIS_Y ) ) ;
+	case EXPP_MOD_AXIS_Z:
+		return PyBool_FromLong( ( long ) 
+				( md->flag & MOD_MIR_AXIS_Z ) ) ;
 	default:
 		return EXPP_ReturnPyObjError( PyExc_KeyError, "key not found" );
 	}
@@ -531,8 +538,12 @@ static int mirror_setter( BPy_Modifier *self, int type, PyObject *value )
 		return EXPP_setFloatClamped( value, &md->tolerance, 0.0, 1.0 );
 	case EXPP_MOD_FLAG:
 		return EXPP_setBitfield( value, &md->flag, MOD_MIR_CLIPPING, 'i' );
-	case EXPP_MOD_AXIS:
-		return EXPP_setIValueRange( value, &md->axis, 0, 2, 'h' );
+	case EXPP_MOD_AXIS_X:
+		return EXPP_setBitfield( value, &md->flag, MOD_MIR_AXIS_X, 'h' );
+	case EXPP_MOD_AXIS_Y:
+		return EXPP_setBitfield( value, &md->flag, MOD_MIR_AXIS_Y, 'h' );
+	case EXPP_MOD_AXIS_Z:
+		return EXPP_setBitfield( value, &md->flag, MOD_MIR_AXIS_Z, 'h' );
 	default:
 		return EXPP_ReturnIntError( PyExc_KeyError, "key not found" );
 	}
@@ -736,11 +747,11 @@ static int wave_setter( BPy_Modifier *self, int type, PyObject *value )
 	case EXPP_MOD_SPEED:
 		return EXPP_setFloatClamped( value, &md->speed, -2.0, 2.0 );
 	case EXPP_MOD_DAMP:
-		return EXPP_setFloatClamped( value, &md->damp, -1000.0, 1000.0 );
+		return EXPP_setFloatClamped( value, &md->damp, -MAXFRAMEF, MAXFRAMEF );
 	case EXPP_MOD_LIFETIME:
-		return EXPP_setFloatClamped( value, &md->lifetime, -1000.0, 1000.0 );
+		return EXPP_setFloatClamped( value, &md->lifetime, -MAXFRAMEF, MAXFRAMEF );
 	case EXPP_MOD_TIMEOFFS:
-		return EXPP_setFloatClamped( value, &md->timeoffs, -1000.0, 1000.0 );
+		return EXPP_setFloatClamped( value, &md->timeoffs, -MAXFRAMEF, MAXFRAMEF );
 	case EXPP_MOD_FLAG:
 		return EXPP_setIValueRange( value, &md->flag, 0, 
 				MOD_WAVE_X | MOD_WAVE_Y | MOD_WAVE_CYCL, 'h' );
@@ -763,8 +774,6 @@ static PyObject *array_getter( BPy_Modifier * self, int type )
 		return PyFloat_FromDouble( md->length );
 	else if( type == EXPP_MOD_MERGE_DIST )
 		return PyFloat_FromDouble( md->merge_dist );
-	else if( type == EXPP_MOD_MERGE_DIST )
-		return PyFloat_FromDouble( md->merge_dist );
 	else if( type == EXPP_MOD_OFFSET_VEC)
 		return newVectorObject( md->offset, 3, Py_NEW );
 	else if( type == EXPP_MOD_SCALE_VEC)
@@ -974,7 +983,7 @@ static PyObject *Modifier_getData( BPy_Modifier * self, PyObject * key )
 {
 	int setting;
 
-	if( !PyInt_CheckExact( key ) )
+	if( !PyInt_Check( key ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 				"expected an int arg as stored in Blender.Modifier.Settings" );
 
@@ -1248,24 +1257,23 @@ static PySequenceMethods ModSeq_as_sequence = {
  * helper function to check for a valid modifier argument
  */
 
-static ModifierData *locate_modifier( BPy_ModSeq *self, PyObject * args )
+static ModifierData *locate_modifier( BPy_ModSeq *self, BPy_Modifier * value )
 {
-	BPy_Modifier *pyobj;
 	ModifierData *md;
 
 	/* check that argument is a modifier */
-	if( !PyArg_ParseTuple( args, "O!", &Modifier_Type, &pyobj ) )
+	if( !BPy_Modifier_Check(value) )
 		return (ModifierData *)EXPP_ReturnPyObjError( PyExc_TypeError,
 				"expected an modifier as an argument" );
 
 	/* check whether modifier has been removed */
-	if( !pyobj->md )
+	if( !value->md )
 		return (ModifierData *)EXPP_ReturnPyObjError( PyExc_RuntimeError,
 				"This modifier has been removed!" );
 
 	/* find the modifier in the object's list */
 	for( md = self->object->modifiers.first; md; md = md->next )
-		if( md == pyobj->md )
+		if( md == value->md )
 			return md;
 
 	/* return exception if we can't find the modifier */
@@ -1275,18 +1283,14 @@ static ModifierData *locate_modifier( BPy_ModSeq *self, PyObject * args )
 
 /* create a new modifier at the end of the list */
 
-static PyObject *ModSeq_append( BPy_ModSeq *self, PyObject *args )
+static PyObject *ModSeq_append( BPy_ModSeq *self, PyObject *value )
 {
-	int type;
-
-	if( !PyArg_ParseTuple( args, "i", &type ) )
-		return EXPP_ReturnPyObjError( PyExc_TypeError,
-				"expected int argument" );
-
+	int type = PyInt_AsLong(value);
+	
 	/* type 0 is eModifierType_None, should we be able to add one of these? */
 	if( type <= 0 || type >= NUM_MODIFIER_TYPES )
 		return EXPP_ReturnPyObjError( PyExc_ValueError,
-				"int argument out of range, expected an int from Blender.Modifier.Type" );
+				"Not an int or argument out of range, expected an int from Blender.Modifier.Type" );
 	
 	BLI_addtail( &self->object->modifiers, modifier_new( type ) );
 	return Modifier_CreatePyObject( self->object, self->object->modifiers.last );
@@ -1294,10 +1298,9 @@ static PyObject *ModSeq_append( BPy_ModSeq *self, PyObject *args )
 
 /* remove an existing modifier */
 
-static PyObject *ModSeq_remove( BPy_ModSeq *self, PyObject *args )
+static PyObject *ModSeq_remove( BPy_ModSeq *self, BPy_Modifier *value )
 {
-	ModifierData *md = locate_modifier( self, args );
-	BPy_Modifier *py_obj;
+	ModifierData *md = locate_modifier( self, value );
 
 	/* if we can't locate the modifier, return (exception already set) */
 	if( !md )
@@ -1308,17 +1311,16 @@ static PyObject *ModSeq_remove( BPy_ModSeq *self, PyObject *args )
 	modifier_free( md );
 
 	/* erase the link to the modifier */
-	py_obj = ( BPy_Modifier * )PyTuple_GET_ITEM( args, 0 );
-	py_obj->md = NULL;
+	value->md = NULL;
 
 	Py_RETURN_NONE;
 }
 
 /* move the modifier up in the stack */
 
-static PyObject *ModSeq_moveUp( BPy_ModSeq * self, PyObject * args )
+static PyObject *ModSeq_moveUp( BPy_ModSeq * self, BPy_Modifier * value )
 {
-	ModifierData *md = locate_modifier( self, args );
+	ModifierData *md = locate_modifier( self, value );
 
 	/* if we can't locate the modifier, return (exception already set) */
 	if( !md )
@@ -1333,9 +1335,9 @@ static PyObject *ModSeq_moveUp( BPy_ModSeq * self, PyObject * args )
 
 /* move the modifier down in the stack */
 
-static PyObject *ModSeq_moveDown( BPy_ModSeq * self, PyObject *args )
+static PyObject *ModSeq_moveDown( BPy_ModSeq * self, BPy_Modifier *value )
 {
-	ModifierData *md = locate_modifier( self, args );
+	ModifierData *md = locate_modifier( self, value );
 
 	/* if we can't locate the modifier, return (exception already set) */
 	if( !md )
@@ -1348,19 +1350,33 @@ static PyObject *ModSeq_moveDown( BPy_ModSeq * self, PyObject *args )
 	Py_RETURN_NONE;
 }
 
+
+/* quick hack for ZanQdo: add new hook modifier for selected verts */
+static PyObject *ModSeq_ZanQdoHack(BPy_ModSeq *self)
+{
+	/* this should add the hook (assumes that modifier stack is on same ob!) */
+	if ((self) && (G.obedit) && (self->object==G.obedit)) {
+		add_hook(1);
+	}
+	
+	Py_RETURN_NONE;
+}
+
 /*****************************************************************************/
 /* Python BPy_ModSeq methods table:                                       */
 /*****************************************************************************/
 static PyMethodDef BPy_ModSeq_methods[] = {
 	/* name, method, flags, doc */
-	{"append", ( PyCFunction ) ModSeq_append, METH_VARARGS,
+	{"append", ( PyCFunction ) ModSeq_append, METH_O,
 	 "(type) - add a new modifier, where type is the type of modifier"},
-	{"remove", ( PyCFunction ) ModSeq_remove, METH_VARARGS,
+	{"remove", ( PyCFunction ) ModSeq_remove, METH_O,
 	 "(modifier) - remove an existing modifier, where modifier is a modifier from this object."},
-	{"moveUp", ( PyCFunction ) ModSeq_moveUp, METH_VARARGS,
+	{"moveUp", ( PyCFunction ) ModSeq_moveUp, METH_O,
 	 "(modifier) - Move a modifier up in stack"},
-	{"moveDown", ( PyCFunction ) ModSeq_moveDown, METH_VARARGS,
+	{"moveDown", ( PyCFunction ) ModSeq_moveDown, METH_O,
 	 "(modifier) - Move a modifier down in stack"},
+	{"ZanQdoHack", (PyCFunction)ModSeq_ZanQdoHack, METH_NOARGS,
+	 "while in editmode, adds a hook for the selected verts (adds new modifier, and deselects object)"},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -1499,6 +1515,8 @@ static PyObject *M_Modifier_TypeDict( void )
 				PyInt_FromLong( eModifierType_Smooth ) );
 		PyConstant_Insert( d, "CAST",
 				PyInt_FromLong( eModifierType_Cast ) );
+		PyConstant_Insert( d, "DISPLACE",
+				PyInt_FromLong( eModifierType_Displace ) );
 	}
 	return S;
 }
@@ -1595,8 +1613,12 @@ for var in st.replace(',','').split('\n'):
 				PyInt_FromLong( EXPP_MOD_SEED ) );
 			PyConstant_Insert( d, "RANDOMIZE", 
 				PyInt_FromLong( EXPP_MOD_RANDOMIZE ) );
-			PyConstant_Insert( d, "AXIS", 
-				PyInt_FromLong( EXPP_MOD_AXIS ) );
+			PyConstant_Insert( d, "AXIS_X", 
+				PyInt_FromLong( EXPP_MOD_AXIS_X ) );
+			PyConstant_Insert( d, "AXIS_Y", 
+				PyInt_FromLong( EXPP_MOD_AXIS_Y ) );
+			PyConstant_Insert( d, "AXIS_Z", 
+				PyInt_FromLong( EXPP_MOD_AXIS_Z ) );
 			PyConstant_Insert( d, "RATIO", 
 				PyInt_FromLong( EXPP_MOD_RATIO ) );
 			PyConstant_Insert( d, "STARTX", 
