@@ -1,15 +1,12 @@
 /* 
- * $Id: NLA.c 11886 2007-08-29 18:47:10Z campbellbarton $
+ * $Id: NLA.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): Joseph Gilbert
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
 */
 
 #include "NLA.h" /*This must come first*/
@@ -87,11 +84,11 @@ struct PyMethodDef M_NLA_methods[] = {
 /*****************************************************************************/
 static PyObject *Action_setActive( BPy_Action * self, PyObject * args );
 static PyObject *Action_getFrameNumbers(BPy_Action *self);
-static PyObject *Action_getChannelIpo( BPy_Action * self, PyObject * args );
+static PyObject *Action_getChannelIpo( BPy_Action * self, PyObject * value );
 static PyObject *Action_getChannelNames( BPy_Action * self );
 static PyObject *Action_renameChannel( BPy_Action * self, PyObject * args );
-static PyObject *Action_verifyChannel( BPy_Action * self, PyObject * args );
-static PyObject *Action_removeChannel( BPy_Action * self, PyObject * args );
+static PyObject *Action_verifyChannel( BPy_Action * self, PyObject * value );
+static PyObject *Action_removeChannel( BPy_Action * self, PyObject * value );
 static PyObject *Action_getAllChannelIpos( BPy_Action * self );
 
 /*****************************************************************************/
@@ -107,15 +104,15 @@ static PyMethodDef BPy_Action_methods[] = {
 	 "(str) -set this action as the active action for an object"},
 	{"getFrameNumbers", (PyCFunction) Action_getFrameNumbers, METH_NOARGS,
 	"() - get the frame numbers at which keys have been inserted"},
-	{"getChannelIpo", ( PyCFunction ) Action_getChannelIpo, METH_VARARGS,
+	{"getChannelIpo", ( PyCFunction ) Action_getChannelIpo, METH_O,
 	 "(str) -get the Ipo from a named action channel in this action"},
 	{"getChannelNames", ( PyCFunction ) Action_getChannelNames, METH_NOARGS,
 	 "() -get the channel names for this action"},
 	 {"renameChannel", ( PyCFunction ) Action_renameChannel, METH_VARARGS,
 		 "(from, to) -rename the channel from string to string"},
-	{"verifyChannel", ( PyCFunction ) Action_verifyChannel, METH_VARARGS,
+	{"verifyChannel", ( PyCFunction ) Action_verifyChannel, METH_O,
 	 "(str) -verify the channel in this action"},
-	{"removeChannel", ( PyCFunction ) Action_removeChannel, METH_VARARGS,
+	{"removeChannel", ( PyCFunction ) Action_removeChannel, METH_O,
 	 "(str) -remove the channel from the action"},
 	{"getAllChannelIpos", ( PyCFunction ) Action_getAllChannelIpos,
 	 METH_NOARGS,
@@ -270,12 +267,12 @@ static PyObject *Action_setActive( BPy_Action * self, PyObject * args )
 	Py_RETURN_NONE;
 }
 
-static PyObject *Action_getChannelIpo( BPy_Action * self, PyObject * args )
+static PyObject *Action_getChannelIpo( BPy_Action * self, PyObject * value )
 {
-	char *chanName;
+	char *chanName = PyString_AsString(value);
 	bActionChannel *chan;
 
-	if( !PyArg_ParseTuple( args, "s", &chanName ) )
+	if( !chanName )
 		return EXPP_ReturnPyObjError( PyExc_AttributeError,
 				       "string expected" );
 
@@ -330,16 +327,16 @@ static PyObject *Action_renameChannel( BPy_Action * self, PyObject * args )
 }
 
 /*----------------------------------------------------------------------*/
-static PyObject *Action_verifyChannel( BPy_Action * self, PyObject * args )
+static PyObject *Action_verifyChannel( BPy_Action * self, PyObject * value )
 {
-	char *chanName;
+	char *chanName = PyString_AsString(value);
 	bActionChannel *chan;
 
 	if( !self->action )
 		( EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					 "couldn't create channel for a NULL action" ) );
 
-	if( !PyArg_ParseTuple( args, "s", &chanName ) )
+	if( !chanName )
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
 						"expected string argument" ) );
 
@@ -349,16 +346,15 @@ static PyObject *Action_verifyChannel( BPy_Action * self, PyObject * args )
 }
 
 
-static PyObject *Action_removeChannel( BPy_Action * self, PyObject * args )
+static PyObject *Action_removeChannel( BPy_Action * self, PyObject * value )
 {
-	char *chanName;
+	char *chanName = PyString_AsString(value);
 	bActionChannel *chan;
 
-	if( !PyArg_ParseTuple( args, "s", &chanName ) ) {
-		EXPP_ReturnPyObjError( PyExc_AttributeError,
-				       "string expected" );
-		return NULL;
-	}
+	if( !chanName )
+		return (EXPP_ReturnPyObjError( PyExc_AttributeError,
+				       "string expected" ));
+
 
 	chan = get_action_channel( self->action, chanName );
 	if( chan == NULL ) {
@@ -806,7 +802,8 @@ static int ActionStrip_setBlendMode( BPy_ActionStrip * self, PyObject * value )
  */
 
 #define ACTIONSTRIP_MASK (ACTSTRIP_SELECT | ACTSTRIP_USESTRIDE \
-		| ACTSTRIP_HOLDLASTFRAME | ACTSTRIP_ACTIVE | ACTSTRIP_LOCK_ACTION)
+		| ACTSTRIP_HOLDLASTFRAME | ACTSTRIP_ACTIVE | ACTSTRIP_LOCK_ACTION \
+		| ACTSTRIP_MUTE)
 
 static PyObject *ActionStrip_getFlag( BPy_ActionStrip * self )
 {
@@ -1182,6 +1179,8 @@ static PyObject *M_ActionStrip_FlagsDict( void )
 				PyInt_FromLong( ACTSTRIP_ACTIVE ) );
 		PyConstant_Insert( d, "LOCK_ACTION",
 				PyInt_FromLong( ACTSTRIP_LOCK_ACTION ) );
+		PyConstant_Insert( d, "MUTE",
+				PyInt_FromLong( ACTSTRIP_MUTE ) );
 	}
 	return S;
 }
