@@ -1,16 +1,13 @@
 /* 
  *
- * $Id: Scene.c,v 1.98 2007/04/30 08:00:48 campbellbarton Exp $
+ * $Id: Scene.c 14456 2008-04-17 21:14:55Z campbellbarton $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,7 +26,7 @@
  * Contributor(s): Willian P. Germano, Jacques Guignot, Joseph Gilbert,
  * Campbell Barton, Ken Hughes
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
 */
 struct View3D;
 
@@ -42,6 +39,7 @@ struct View3D;
 #include "DNA_screen_types.h"
 #include "DNA_userdef_types.h" /* U.userdefs */
 #include "DNA_object_types.h" /* SceneObSeq_new */
+#include "BKE_armature.h"
 #include "BKE_depsgraph.h"
 #include "BKE_library.h"
 #include "BKE_object.h"
@@ -51,12 +49,15 @@ struct View3D;
 #include "BLI_blenlib.h" /* only for SceneObSeq_new */
 #include "BSE_drawview.h"	/* for play_anim */
 #include "BSE_headerbuttons.h"	/* for copy_scene */
+#include "BSE_sequence.h"	/* to clear_scene_in_allseqs */
+#include "BSE_node.h"	/* to clear_scene_in_nodes */
 #include "BIF_drawscene.h"	/* for set_scene */
 #include "BIF_space.h"		/* for copy_view3d_lock() */
 #include "BIF_screen.h"		/* curarea */
 #include "BDR_editobject.h"		/* free_and_unlink_base() */
 #include "mydevice.h"		/* for #define REDRAW */
 #include "DNA_view3d_types.h"
+
 /* python types */
 #include "Object.h"
 #include "Camera.h"
@@ -145,7 +146,7 @@ static PyObject *Scene_getCurrentCamera( BPy_Scene * self );
 static PyObject *Scene_setCurrentCamera( BPy_Scene * self, PyObject * args );
 static PyObject *Scene_getRenderingContext( BPy_Scene * self );
 static PyObject *Scene_getRadiosityContext( BPy_Scene * self );
-static PyObject *Scene_getScriptLinks( BPy_Scene * self, PyObject * args );
+static PyObject *Scene_getScriptLinks( BPy_Scene * self, PyObject * value );
 static PyObject *Scene_getSequence( BPy_Scene * self );
 static PyObject *Scene_addScriptLink( BPy_Scene * self, PyObject * args );
 static PyObject *Scene_clearScriptLinks( BPy_Scene * self, PyObject * args );
@@ -194,7 +195,7 @@ static PyMethodDef BPy_Scene_methods[] = {
 	{"getCurrentCamera", ( PyCFunction ) Scene_getCurrentCamera,
 	 METH_NOARGS,
 	 "() - Return current active Camera"},
-	{"getScriptLinks", ( PyCFunction ) Scene_getScriptLinks, METH_VARARGS,
+	{"getScriptLinks", ( PyCFunction ) Scene_getScriptLinks, METH_O,
 	 "(eventname) - Get a list of this scene's scriptlinks (Text names) "
 	 "of the given type\n"
 	 "(eventname) - string: FrameChanged, OnLoad, OnSave, Redraw or Render."},
@@ -249,7 +250,7 @@ static int Scene_setLayerMask( BPy_Scene * self, PyObject * value )
 	
 	SCENE_DEL_CHECK_INT(self);
 	
-	if (!PyInt_CheckExact(value)) {
+	if (!PyInt_Check(value)) {
 		return EXPP_ReturnIntError( PyExc_AttributeError,
 			"expected an integer (bitmask) as argument" );
 	}
@@ -282,11 +283,13 @@ static int Scene_setLayerMask( BPy_Scene * self, PyObject * value )
 
 static PyObject *Scene_getLayerList( BPy_Scene * self )
 {
-	PyObject *laylist = PyList_New( 0 ), *item;
+	PyObject *laylist, *item;
 	int layers, bit = 0, val = 0;
 	
 	SCENE_DEL_CHECK_PY(self);
 	
+	laylist = PyList_New( 0 );
+	
 	if( !laylist )
 		return ( EXPP_ReturnPyObjError( PyExc_MemoryError,
 			"couldn't create pylist!" ) );
@@ -550,6 +553,7 @@ PyObject *Scene_Init( void )
 	PyDict_SetItemString( dict, "Render", Render_Init(  ) );
 	PyDict_SetItemString( dict, "Radio", Radio_Init(  ) );
 	PyDict_SetItemString( dict, "Sequence", Sequence_Init(  ) );
+	PyDict_SetItemString( dict, "TimeLine", TimeLine_Init(  ) );
 	
 	return submodule;
 }
@@ -677,11 +681,12 @@ static PyObject *M_Scene_Get( PyObject * self, PyObject * args )
 		while( scene_iter ) {
 			pyobj = Scene_CreatePyObject( scene_iter );
 
-			if( !pyobj )
+			if( !pyobj ) {
+				Py_DECREF(sce_pylist);
 				return ( EXPP_ReturnPyObjError
 					 ( PyExc_MemoryError,
 					   "couldn't create PyString" ) );
-
+			}
 			PyList_SET_ITEM( sce_pylist, index, pyobj );
 
 			scene_iter = scene_iter->id.next;
@@ -715,8 +720,9 @@ static PyObject *M_Scene_Unlink( PyObject * self, PyObject * args )
 {
 	PyObject *pyobj;
 	BPy_Scene *pyscn;
-	Scene *scene;
-
+	Scene *scene, *sce;
+	bScreen *sc;
+	
 	if( !PyArg_ParseTuple( args, "O!", &Scene_Type, &pyobj ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected Scene PyType object" );
@@ -730,6 +736,23 @@ static PyObject *M_Scene_Unlink( PyObject * self, PyObject * args )
 		return EXPP_ReturnPyObjError( PyExc_SystemError,
 					      "current Scene cannot be removed!" );
 
+	/* Copied from header_info.c */
+	
+	/* check all sets */
+	for (sce= G.main->scene.first; sce; sce= sce->id.next) {
+		if(sce->set == scene) sce->set= 0;
+	}
+	
+	/* check all sequences */
+	clear_scene_in_allseqs(scene);
+
+	/* check render layer nodes in other scenes */
+	clear_scene_in_nodes(scene);
+	
+	for (sc= G.main->screen.first; sc; sc= sc->id.next ) {
+		if(sc->scene == scene) sc->scene= G.scene;
+	}
+	
 	free_libblock( &G.main->scene, scene );
 	
 	pyscn->scene= NULL;
@@ -750,16 +773,16 @@ static PyObject *Scene_oldsetLayers( BPy_Scene * self, PyObject * args )
 /*-----------------------Scene.copy()------------------------------------*/
 static PyObject *Scene_copy( BPy_Scene * self, PyObject * args )
 {
-	short dup_objs = 1;
+	short dup_objs = 2;
 	Scene *scene = self->scene;
 
 	SCENE_DEL_CHECK_PY(self);
 
-	if( !PyArg_ParseTuple( args, "|h", &dup_objs ) )
+	if( !PyArg_ParseTuple( args, "|h", &dup_objs ) || dup_objs < 0 || dup_objs > 2)
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected int in [0,2] or nothing as argument" );
-
-	return Scene_CreatePyObject( copy_scene( scene, dup_objs ) );
+	
+	return Scene_CreatePyObject( copy_scene( scene, dup_objs+1 ) );
 }
 
 /*-----------------------Scene.makeCurrent()-----------------------------*/
@@ -802,10 +825,18 @@ static PyObject *Scene_update( BPy_Scene * self, PyObject * args )
 	if( !full )
 		DAG_scene_sort( scene );
 
-	else if( full == 1 )
+	else if( full == 1 ) {
+		int enablescripts = G.f & G_DOSCRIPTLINKS;
+		
+		/*Disable scriptlinks to prevent firing off newframe scriptlink
+		  events.*/
+		G.f &= ~G_DOSCRIPTLINKS;
 		set_scene_bg( scene );
-
-	else
+		scene_update_for_newframe( scene, scene->lay );
+		
+		/*re-enabled scriptlinks if necassary.*/
+		if (enablescripts) G.f |= G_DOSCRIPTLINKS;
+	} else
 		return EXPP_ReturnPyObjError( PyExc_ValueError,
 					      "in method scene.update(full), full should be:\n"
 					      "0: to only sort scene elements (old behavior); or\n"
@@ -927,7 +958,7 @@ static PyObject *Scene_unlink( BPy_Scene * self, PyObject * args )
 static PyObject *Scene_getChildren( BPy_Scene * self )
 {
 	Scene *scene = self->scene;
-	PyObject *pylist = PyList_New( 0 );
+	PyObject *pylist;
 	PyObject *bpy_obj;
 	Object *object;
 	Base *base;
@@ -940,7 +971,8 @@ static PyObject *Scene_getChildren( BPy_Scene * self )
 
 	SCENE_DEL_CHECK_PY(self);
 
-
+	pylist = PyList_New( 0 );
+	
 	base = scene->base.first;
 
 	while( base ) {
@@ -948,12 +980,13 @@ static PyObject *Scene_getChildren( BPy_Scene * self )
 		
 		bpy_obj = Object_CreatePyObject( object );
 		
-		if( !bpy_obj )
+		if( !bpy_obj ) {
+			Py_DECREF(pylist);
 			return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 						      "couldn't create new object wrapper" );
-
+		}
 		PyList_Append( pylist, bpy_obj );
-		Py_XDECREF( bpy_obj );	/* PyList_Append incref'ed it */
+		Py_DECREF( bpy_obj );	/* PyList_Append incref'ed it */
 		
 		base = base->next;
 	}
@@ -994,9 +1027,6 @@ static PyObject *Scene_getActiveObject(BPy_Scene *self)
 /*-----------------------Scene.getCurrentCamera()------------------------*/
 static PyObject *Scene_getCurrentCamera( BPy_Scene * self )
 {
-	Object *cam_obj;
-	PyObject *pyob;
-	Scene *scene = self->scene;
 	static char warning = 1;
 	
 	if( warning ) {
@@ -1005,18 +1035,8 @@ static PyObject *Scene_getCurrentCamera( BPy_Scene * self )
 	}
 
 	SCENE_DEL_CHECK_PY(self);
-
-	cam_obj = scene->camera;
-
-	if( cam_obj ) {		/* if found, return a wrapper for it */
-		pyob = Object_CreatePyObject( cam_obj );
-		if (!pyob)
-			return EXPP_ReturnPyObjError(PyExc_MemoryError,
-					"couldn't create new object wrapper!");
-		return pyob;
-	}
-
-	Py_RETURN_NONE;	/* none found */
+	/* None is ok */
+	return Object_CreatePyObject( self->scene->camera );
 }
 
 /*-----------------------Scene.setCurrentCamera()------------------------*/
@@ -1071,10 +1091,7 @@ static PyObject *Scene_getRadiosityContext( BPy_Scene * self )
 static PyObject *Scene_getSequence( BPy_Scene * self )
 {
 	SCENE_DEL_CHECK_PY(self);
-	if (self->scene->ed) /* we should create this if its not there :/ */
-		return SceneSeq_CreatePyObject( self->scene, NULL );
-	else
-		Py_RETURN_NONE;
+	return SceneSeq_CreatePyObject( self->scene, NULL );
 }
 
 /* scene.addScriptLink */
@@ -1104,7 +1121,7 @@ static PyObject *Scene_clearScriptLinks( BPy_Scene * self, PyObject * args )
 }
 
 /* scene.getScriptLinks */
-static PyObject *Scene_getScriptLinks( BPy_Scene * self, PyObject * args )
+static PyObject *Scene_getScriptLinks( BPy_Scene * self, PyObject * value )
 {
 	Scene *scene = self->scene;
 	ScriptLink *slink = NULL;
@@ -1114,7 +1131,7 @@ static PyObject *Scene_getScriptLinks( BPy_Scene * self, PyObject * args )
 
 	slink = &( scene )->scriptlink;
 
-	ret = EXPP_getScriptLinks( slink, args, 1 );
+	ret = EXPP_getScriptLinks( slink, value, 1 );
 
 	if( ret )
 		return ret;
@@ -1170,7 +1187,7 @@ static PyObject *Scene_play( BPy_Scene * self, PyObject * args )
 		areawinset( sa->win );
 
 	/* play_anim returns 0 for normal exit or 1 if user canceled it */
-	ret = Py_BuildValue( "i", play_anim( mode ) );
+	ret = PyInt_FromLong( (long)play_anim( mode ) );
 
 	if( sa )
 		areawinset( oldsa->win );
@@ -1221,7 +1238,7 @@ int SceneObSeq_setObjects( BPy_SceneObSeq *self, PyObject *value, void *_mode_)
 	Scene *scene= self->bpyscene->scene;
 	Object *blen_ob;
 	Base *base;
-	int size, mode = (int)_mode_;
+	int size, mode = GET_INT_FROM_POINTER(_mode_);
 	
 	SCENE_DEL_CHECK_INT(self->bpyscene);
 	
@@ -1328,7 +1345,6 @@ static int SceneObSeq_len( BPy_SceneObSeq * self )
 static PyObject *SceneObSeq_item( BPy_SceneObSeq * self, int i )
 {
 	int index=0;
-	PyObject *bpy_obj;
 	Base *base= NULL;
 	Scene *scene= self->bpyscene->scene;
 	
@@ -1338,28 +1354,24 @@ static PyObject *SceneObSeq_item( BPy_SceneObSeq * self, int i )
 	if (self->mode==EXPP_OBSEQ_NORMAL)
 		for (base= scene->base.first; base && i!=index; base= base->next, index++) {}
 	/* selected */
-	else if (self->mode==EXPP_OBSEQ_SELECTED)
+	else if (self->mode==EXPP_OBSEQ_SELECTED) {
 		for (base= scene->base.first; base && i!=index; base= base->next)
 			if (base->flag & SELECT)
 				index++;
+	}
 	/* context */
-	else if (self->mode==EXPP_OBSEQ_CONTEXT)
+	else if (self->mode==EXPP_OBSEQ_CONTEXT) {
 		if (G.vd)
 			for (base= scene->base.first; base && i!=index; base= base->next)
 				if TESTBASE(base)
 					index++;
+	}
 	
 	if (!(base))
 		return EXPP_ReturnPyObjError( PyExc_IndexError,
 					      "array index out of range" );
 	
-	bpy_obj = Object_CreatePyObject( base->object );
-
-	if( !bpy_obj )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"PyObject_New() failed" );
-
-	return (PyObject *)bpy_obj;
+	return Object_CreatePyObject( base->object );
 }
 
 static PySequenceMethods SceneObSeq_as_sequence = {
@@ -1549,8 +1561,11 @@ typeError:
 	object->flag = SELECT;
 	
 	/* creates the curve for the text object */
-	if (type == OB_FONT) 
+	if (type == OB_FONT) {
 		text_to_curve(object, 0);
+	} else if (object->type == OB_ARMATURE) {
+		armature_rebuild_pose(object, (bArmature *)data);
+	}
 	
 	/* link to scene */
 	base = MEM_callocN( sizeof( Base ), "pynewbase" );
@@ -1623,9 +1638,7 @@ static PyObject *SceneObSeq_unlink( BPy_SceneObSeq * self, PyObject *args )
 
 PyObject *SceneObSeq_getActive(BPy_SceneObSeq *self)
 {
-	PyObject *pyob;
 	Base *base;
-	
 	SCENE_DEL_CHECK_PY(self->bpyscene);
 	
 	if (self->mode!=EXPP_OBSEQ_NORMAL)
@@ -1636,13 +1649,7 @@ PyObject *SceneObSeq_getActive(BPy_SceneObSeq *self)
 	if (!base)
 		Py_RETURN_NONE;
 	
-	pyob = Object_CreatePyObject( base->object );
-	
-	if (!pyob)
-		return EXPP_ReturnPyObjError(PyExc_MemoryError,
-					"couldn't create new object wrapper!");
-	
-	return pyob;
+	return Object_CreatePyObject( base->object );
 }
 
 static int SceneObSeq_setActive(BPy_SceneObSeq *self, PyObject *value)
@@ -1676,26 +1683,13 @@ static int SceneObSeq_setActive(BPy_SceneObSeq *self, PyObject *value)
 
 PyObject *SceneObSeq_getCamera(BPy_SceneObSeq *self)
 {
-	PyObject *pyob;
-	Object *ob;
-	
 	SCENE_DEL_CHECK_PY(self->bpyscene);
 	
 	if (self->mode!=EXPP_OBSEQ_NORMAL)
 			return (EXPP_ReturnPyObjError( PyExc_TypeError,
 						"cannot get camera from objects.selected or objects.context" ));
 	
-	ob= self->bpyscene->scene->camera;
-	if (!ob)
-		Py_RETURN_NONE;
-	
-	pyob = Object_CreatePyObject( ob );
-	
-	if (!pyob)
-		return EXPP_ReturnPyObjError(PyExc_MemoryError,
-					"couldn't create new object wrapper!");
-	
-	return pyob;
+	return Object_CreatePyObject( self->bpyscene->scene->camera );
 }
 
 static int SceneObSeq_setCamera(BPy_SceneObSeq *self, PyObject *value)
