@@ -1,15 +1,12 @@
 /* 
- * $Id: Sound.c 10942 2007-06-16 04:20:19Z campbellbarton $
+ * $Id: Sound.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): Chris Keith
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
 */
 
 #include "Sound.h" /*This must come first*/
@@ -59,7 +56,7 @@
 /* Python API function prototypes for the Sound module.		*/
 /*****************************************************************************/
 static PyObject *M_Sound_Get( PyObject * self, PyObject * args );
-static PyObject *M_Sound_Load( PyObject * self, PyObject * args );
+static PyObject *M_Sound_Load( PyObject * self, PyObject * value );
 
 /************************************************************************/
 /* The following string definitions are used for documentation strings.	*/
@@ -82,7 +79,7 @@ returns None if not found.";
 /*****************************************************************************/
 struct PyMethodDef M_Sound_methods[] = {
 	{"Get", M_Sound_Get, METH_VARARGS, M_Sound_Get_doc},
-	{"Load", M_Sound_Load, METH_VARARGS, M_Sound_Load_doc},
+	{"Load", M_Sound_Load, METH_O, M_Sound_Load_doc},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -92,29 +89,24 @@ struct PyMethodDef M_Sound_methods[] = {
 static int Sound_compare( BPy_Sound * a, BPy_Sound * b );
 static PyObject *Sound_repr( BPy_Sound * self );
 
-#define SOUND_FLOAT_METHODS(funcname, varname)			\
-static PyObject *Sound_get ## funcname(BPy_Sound *self) {	\
-    char e[256];						\
-    PyObject *attr = PyFloat_FromDouble(self->sound->varname);	\
-    if (attr) return attr;					\
-    sprintf(e, "couldn't get Sound.%s attribute", #varname);	\
-    return EXPP_ReturnPyObjError (PyExc_RuntimeError, e);	\
-}								\
+#define SOUND_FLOAT_METHODS(funcname, varname)					\
+static PyObject *Sound_get ## funcname(BPy_Sound *self) {		\
+    return PyFloat_FromDouble(self->sound->varname);			\
+}																\
 static PyObject *Sound_set ## funcname(BPy_Sound *self, PyObject *args) { \
-    float	f = 0;						\
-    if (!PyArg_ParseTuple(args, "f", &f))			\
-	    return (EXPP_ReturnPyObjError (PyExc_TypeError,	\
-		    "expected float argument"));		\
-    self->sound->varname = EXPP_ClampFloat(f,\
-			EXPP_SND_##varname##_MIN, EXPP_SND_##varname##_MAX);\
-    Py_INCREF(Py_None);						\
-    return Py_None;						\
+    float	f = 0;													\
+    if (!PyArg_ParseTuple(args, "f", &f))							\
+	    return (EXPP_ReturnPyObjError (PyExc_TypeError,				\
+		    "expected float argument"));							\
+    self->sound->varname = EXPP_ClampFloat(f,						\
+			EXPP_SND_##varname##_MIN, EXPP_SND_##varname##_MAX);	\
+    Py_RETURN_NONE;													\
 }
 
-#define SOUND_FLOAT_METHOD_FUNCS(varname)			\
-{"get"#varname, (PyCFunction)Sound_get ## varname, METH_NOARGS,	\
-"() - Return Sound object "#varname},				\
-{"set"#varname, (PyCFunction)Sound_set ## varname, METH_VARARGS, \
+#define SOUND_FLOAT_METHOD_FUNCS(varname)							\
+{"get"#varname, (PyCFunction)Sound_get ## varname, METH_NOARGS,		\
+"() - Return Sound object "#varname},								\
+{"set"#varname, (PyCFunction)Sound_set ## varname, METH_VARARGS,	\
 "(float) - Change Sound object "#varname},
 
 
@@ -265,13 +257,13 @@ static PyObject *M_Sound_Get( PyObject * self, PyObject * args )
 /* Description:		Receives a string and returns the Sound object	 */
 /*			whose filename matches the string.		 */
 /*****************************************************************************/
-static PyObject *M_Sound_Load( PyObject * self, PyObject * args )
+static PyObject *M_Sound_Load( PyObject * self, PyObject * value )
 {
-	char *fname;
+	char *fname = PyString_AsString(value);
 	bSound *snd_ptr;
 	BPy_Sound *snd;
 
-	if( !PyArg_ParseTuple( args, "s", &fname ) )
+	if( !fname )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 						"expected string argument" ) );
 
@@ -354,24 +346,12 @@ bSound *Sound_FromPyObject( PyObject * pyobj )
 /*****************************************************************************/
 static PyObject *Sound_getName( BPy_Sound * self )
 {
-	PyObject *attr = PyString_FromString( self->sound->id.name + 2 );
-
-	if( attr )
-		return attr;
-
-	return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					"couldn't get Sound.name attribute" ) );
+	return PyString_FromString( self->sound->id.name + 2 );
 }
 
 static PyObject *Sound_getFilename( BPy_Sound * self )
 {
-	PyObject *attr = PyString_FromString( self->sound->name );
-
-	if( attr )
-		return attr;
-
-	return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					"couldn't get Sound.filename attribute" ) );
+	return PyString_FromString( self->sound->name );
 }
 
 static PyObject *Sound_getPacked( BPy_Sound * self )
