@@ -1,15 +1,12 @@
 /* 
- * $Id: Sys.c,v 1.29 2007/03/10 11:47:24 campbellbarton Exp $
+ * $Id: Sys.c 14709 2008-05-06 17:54:55Z campbellbarton $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): Willian P. Germano, Campbell Barton
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
 */
 
 #include "Sys.h" /*This must come first*/
@@ -51,16 +48,17 @@
 /*****************************************************************************/
 /* Python API function prototypes for the sys module.                        */
 /*****************************************************************************/
-static PyObject *M_sys_basename( PyObject * self, PyObject * args );
-static PyObject *M_sys_dirname( PyObject * self, PyObject * args );
+static PyObject *M_sys_basename( PyObject * self, PyObject * value );
+static PyObject *M_sys_dirname( PyObject * self, PyObject * value );
 static PyObject *M_sys_join( PyObject * self, PyObject * args );
-static PyObject *M_sys_splitext( PyObject * self, PyObject * args );
+static PyObject *M_sys_splitext( PyObject * self, PyObject * value );
 static PyObject *M_sys_makename( PyObject * self, PyObject * args,
 	PyObject * kw );
-static PyObject *M_sys_exists( PyObject * self, PyObject * args );
+static PyObject *M_sys_exists( PyObject * self, PyObject * value );
 static PyObject *M_sys_time( PyObject * self );
 static PyObject *M_sys_sleep( PyObject * self, PyObject * args );
-static PyObject *M_sys_expandpath( PyObject *self, PyObject *args);
+static PyObject *M_sys_expandpath( PyObject *self, PyObject *value);
+static PyObject *M_sys_cleanpath( PyObject *self, PyObject *value);
 
 /*****************************************************************************/
 /* The following string definitions are used for documentation strings.      */
@@ -123,25 +121,29 @@ static char M_sys_expandpath_doc[] =
 (path) - the string path to convert.\n\n\
 Note: internally Blender paths can contain two special character sequences:\n\
 - '//' (at start) for base path directory (the current .blend's dir path);\n\
-- '#' (at ending) for current frame number.\n\n\
+- '#' characters in the filename will be replaced by the frame number.\n\n\
 This function expands these to their actual content, returning a valid path.\n\
 If the special chars are not found in the given path, it is simply returned.";
 
+static char M_sys_cleanpath_doc[] =
+"(path) - Removes parts of a path that are not needed paths such as '../foo/../bar/' and '//./././'";
+
 /*****************************************************************************/
 /* Python method structure definition for Blender.sys module:                */
 /*****************************************************************************/
 struct PyMethodDef M_sys_methods[] = {
-	{"basename", M_sys_basename, METH_VARARGS, M_sys_basename_doc},
-	{"dirname", M_sys_dirname, METH_VARARGS, M_sys_dirname_doc},
+	{"basename", M_sys_basename, METH_O, M_sys_basename_doc},
+	{"dirname", M_sys_dirname, METH_O, M_sys_dirname_doc},
 	{"join", M_sys_join, METH_VARARGS, M_sys_join_doc},
-	{"splitext", M_sys_splitext, METH_VARARGS, M_sys_splitext_doc},
+	{"splitext", M_sys_splitext, METH_O, M_sys_splitext_doc},
 	{"makename", ( PyCFunction ) M_sys_makename,
 	 METH_VARARGS | METH_KEYWORDS,
 	 M_sys_makename_doc},
-	{"exists", M_sys_exists, METH_VARARGS, M_sys_exists_doc},
+	{"exists", M_sys_exists, METH_O, M_sys_exists_doc},
 	{"sleep", M_sys_sleep, METH_VARARGS, M_sys_sleep_doc},
 	{"time", ( PyCFunction ) M_sys_time, METH_NOARGS, M_sys_time_doc},
-	{"expandpath", M_sys_expandpath, METH_VARARGS, M_sys_expandpath_doc},
+	{"expandpath", M_sys_expandpath, METH_O, M_sys_expandpath_doc},
+	{"cleanpath", M_sys_cleanpath, METH_O, M_sys_cleanpath_doc},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -165,12 +167,13 @@ PyObject *sys_Init( void )
 	return submodule;
 }
 
-static PyObject *M_sys_basename( PyObject * self, PyObject * args )
+static PyObject *M_sys_basename( PyObject * self, PyObject * value )
 {
-	char *name, *p, basename[FILE_MAXDIR + FILE_MAXFILE];
+	char *name = PyString_AsString(value);
+	char *p, basename[FILE_MAXDIR + FILE_MAXFILE];
 	int n, len;
 
-	if( !PyArg_ParseTuple( args, "s", &name ) )
+	if( !name )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected string argument" );
 
@@ -196,12 +199,13 @@ static PyObject *M_sys_basename( PyObject * self, PyObject * args )
 	return PyString_FromString( name );
 }
 
-static PyObject *M_sys_dirname( PyObject * self, PyObject * args )
+static PyObject *M_sys_dirname( PyObject * self, PyObject * value )
 {
-	char *name, *p, dirname[FILE_MAXDIR + FILE_MAXFILE];
+	char *name = PyString_AsString(value);
+	char *p, dirname[FILE_MAXDIR + FILE_MAXFILE];
 	int n;
 
-	if( !PyArg_ParseTuple( args, "s", &name ) )
+	if( !name )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected string argument" );
 
@@ -254,12 +258,13 @@ static PyObject *M_sys_join( PyObject * self, PyObject * args )
 	return PyString_FromString( filename );
 }
 
-static PyObject *M_sys_splitext( PyObject * self, PyObject * args )
+static PyObject *M_sys_splitext( PyObject * self, PyObject * value )
 {
-	char *name, *dot, *p, path[FILE_MAXDIR + FILE_MAXFILE], ext[FILE_MAXDIR + FILE_MAXFILE];
+	char *name = PyString_AsString(value);
+	char *dot, *p, path[FILE_MAXDIR + FILE_MAXFILE], ext[FILE_MAXDIR + FILE_MAXFILE];
 	int n, len;
 
-	if( !PyArg_ParseTuple( args, "s", &name ) )
+	if( !name )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected string argument" );
 
@@ -358,12 +363,13 @@ static PyObject *M_sys_sleep( PyObject * self, PyObject * args )
 	return EXPP_incr_ret( Py_None );
 }
 
-static PyObject *M_sys_exists( PyObject * self, PyObject * args )
+static PyObject *M_sys_exists( PyObject * self, PyObject * value )
 {
-	char *fname = NULL;
+	char *fname = PyString_AsString(value);
+	
 	int mode = 0, i = -1;
 
-	if( !PyArg_ParseTuple( args, "s", &fname ) )
+	if( !fname )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected string (pathname) argument" );
 
@@ -380,17 +386,39 @@ static PyObject *M_sys_exists( PyObject * self, PyObject * args )
 	return PyInt_FromLong(i);
 }
 
-static PyObject *M_sys_expandpath( PyObject * self, PyObject * args )
+static PyObject *M_sys_expandpath( PyObject * self, PyObject * value )
 {
-	char *path = NULL;
+	char *path = PyString_AsString(value);
 	char expanded[FILE_MAXDIR + FILE_MAXFILE];
 
-	if (!PyArg_ParseTuple( args, "s", &path))
+	if (!path)
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 			"expected string argument" );
 	
 	BLI_strncpy(expanded, path, FILE_MAXDIR + FILE_MAXFILE);
-	BLI_convertstringcode(expanded, G.sce, G.scene->r.cfra);
+	BLI_convertstringcode(expanded, G.sce);
+	BLI_convertstringframe(expanded, G.scene->r.cfra);
 
 	return PyString_FromString(expanded);
 }
+
+static PyObject *M_sys_cleanpath( PyObject * self, PyObject * value )
+{
+	char *path = PyString_AsString(value);
+	char cleaned[FILE_MAXDIR + FILE_MAXFILE];
+	int trailing_slash = 0;
+	if (!path)
+		return EXPP_ReturnPyObjError( PyExc_TypeError,
+			"expected string argument" );
+	if (strstr(path, "/") || strstr(path, "\\")) {
+		trailing_slash = 1;
+	}
+	BLI_strncpy(cleaned, path, FILE_MAXDIR + FILE_MAXFILE);
+	BLI_cleanup_file(NULL, cleaned);
+	
+	if (trailing_slash) {
+		BLI_add_slash(cleaned);
+	}
+	
+	return PyString_FromString(cleaned);
+}
