@@ -1,15 +1,12 @@
 /* 
- * $Id: Text.c,v 1.34 2007/03/26 02:10:22 campbellbarton Exp $
+ * $Id: Text.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): Willian P. Germano
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
 */
 
 #include "Text.h" /*This must come first*/
@@ -51,7 +48,7 @@
 /*****************************************************************************/
 static PyObject *M_Text_New( PyObject * self, PyObject * args);
 static PyObject *M_Text_Get( PyObject * self, PyObject * args );
-static PyObject *M_Text_Load( PyObject * self, PyObject * args );
+static PyObject *M_Text_Load( PyObject * self, PyObject * value );
 static PyObject *M_Text_unlink( PyObject * self, PyObject * args );
 
 /*****************************************************************************/
@@ -81,7 +78,7 @@ struct PyMethodDef M_Text_methods[] = {
 	{"New", M_Text_New, METH_VARARGS, M_Text_New_doc},
 	{"Get", M_Text_Get, METH_VARARGS, M_Text_Get_doc},
 	{"get", M_Text_Get, METH_VARARGS, M_Text_Get_doc},
-	{"Load", M_Text_Load, METH_VARARGS, M_Text_Load_doc},
+	{"Load", M_Text_Load, METH_O, M_Text_Load_doc},
 	{"unlink", M_Text_unlink, METH_VARARGS, M_Text_unlink_doc},
 	{NULL, NULL, 0, NULL}
 };
@@ -93,7 +90,7 @@ struct PyMethodDef M_Text_methods[] = {
 static PyObject *Text_getFilename( BPy_Text * self );
 static PyObject *Text_getNLines( BPy_Text * self );
 static PyObject *Text_clear( BPy_Text * self );
-static PyObject *Text_write( BPy_Text * self, PyObject * args );
+static PyObject *Text_write( BPy_Text * self, PyObject * value );
 static PyObject *Text_set( BPy_Text * self, PyObject * args );
 static PyObject *Text_asLines( BPy_Text * self );
 
@@ -112,7 +109,7 @@ static PyMethodDef BPy_Text_methods[] = {
 	 "(str) - Change Text Object name"},
 	{"clear", ( PyCFunction ) Text_clear, METH_NOARGS,
 	 "() - Clear Text buffer"},
-	{"write", ( PyCFunction ) Text_write, METH_VARARGS,
+	{"write", ( PyCFunction ) Text_write, METH_O,
 	 "(line) - Append string 'str' to Text buffer"},
 	{"set", ( PyCFunction ) Text_set, METH_VARARGS,
 	 "(name, val) - Set attribute 'name' to value 'val'"},
@@ -218,11 +215,12 @@ static PyObject *M_Text_Get( PyObject * self, PyObject * args )
 		while( txt_iter ) {
 			pyobj = Text_CreatePyObject( txt_iter );
 
-			if( !pyobj )
+			if( !pyobj ) {
+				Py_DECREF(txtlist);
 				return ( EXPP_ReturnPyObjError
 					 ( PyExc_MemoryError,
 					   "couldn't create PyString" ) );
-
+			}
 			PyList_SET_ITEM( txtlist, index, pyobj );
 
 			txt_iter = txt_iter->id.next;
@@ -239,14 +237,14 @@ static PyObject *M_Text_Get( PyObject * self, PyObject * args )
 /* Description:           Receives a filename and returns the text object    */
 /*                        created from the corresponding file.               */
 /*****************************************************************************/
-static PyObject *M_Text_Load( PyObject * self, PyObject * args )
+static PyObject *M_Text_Load( PyObject * self, PyObject * value )
 {
-	char *fname = NULL;
+	char *fname = PyString_AsString(value);
 	char fpath[FILE_MAXDIR + FILE_MAXFILE];
 	Text *txt_ptr = NULL;
 	unsigned int maxlen = FILE_MAXDIR + FILE_MAXFILE;
 
-	if( !PyArg_ParseTuple( args, "s", &fname ) )
+	if( !fname )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 						"expected string argument" ) );
 
@@ -288,6 +286,7 @@ static PyObject *M_Text_unlink( PyObject * self, PyObject * args )
 					      "this text was already unlinked!" );
 
 	BPY_clear_bad_scriptlinks( text );
+	BPY_free_pyconstraint_links( text );
 	free_text_controllers( text );
 	unlink_text( text );
 
@@ -337,26 +336,16 @@ PyObject *Text_CreatePyObject( Text * txt )
 /*****************************************************************************/
 static PyObject *Text_getFilename( BPy_Text * self )
 {
-	PyObject *attr;
-	char *name = self->text->name;
-
-	if( name )
-		attr = PyString_FromString( self->text->name );
-	else
-		attr = Py_None;
-
-	if( attr )
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				      "couldn't get Text.filename attribute" );
+	if( self->text->name )
+		return PyString_FromString( self->text->name );
+	
+	Py_RETURN_NONE;
 }
 
 static PyObject *Text_getNLines( BPy_Text * self )
 {				/* text->nlines isn't updated in Blender (?) */
 	int nlines = 0;
 	TextLine *line;
-	PyObject *attr;
 
 	line = self->text->lines.first;
 
@@ -367,13 +356,7 @@ static PyObject *Text_getNLines( BPy_Text * self )
 
 	self->text->nlines = nlines;	/* and update Blender, too (should we?) */
 
-	attr = PyInt_FromLong( nlines );
-
-	if( attr )
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				      "couldn't get Text.nlines attribute" );
+	return PyInt_FromLong( nlines );
 }
 
 static PyObject *Text_clear( BPy_Text * self)
@@ -412,16 +395,16 @@ static PyObject *Text_set( BPy_Text * self, PyObject * args )
 	Py_RETURN_NONE;
 }
 
-static PyObject *Text_write( BPy_Text * self, PyObject * args )
+static PyObject *Text_write( BPy_Text * self, PyObject * value )
 {
-	char *str;
+	char *str = PyString_AsString(value);
 	int oldstate;
 
 	if( !self->text )
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "This object isn't linked to a Blender Text Object" );
 
-	if( !PyArg_ParseTuple( args, "s", &str ) )
+	if( !str )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected string argument" );
 
@@ -436,7 +419,7 @@ static PyObject *Text_write( BPy_Text * self, PyObject * args )
 static PyObject *Text_asLines( BPy_Text * self )
 {
 	TextLine *line;
-	PyObject *list, *ob;
+	PyObject *list, *tmpstr;
 
 	if( !self->text )
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
@@ -450,8 +433,9 @@ static PyObject *Text_asLines( BPy_Text * self )
 					      "couldn't create PyList" );
 
 	while( line ) {
-		ob = Py_BuildValue( "s", line->line );
-		PyList_Append( list, ob );
+		tmpstr = PyString_FromString( line->line );
+		PyList_Append( list, tmpstr );
+		Py_DECREF(tmpstr);
 		line = line->next;
 	}
 
