@@ -1,15 +1,12 @@
 /*
- * $Id: Text3d.c 10942 2007-06-16 04:20:19Z campbellbarton $
+ * $Id: Text3d.c 14456 2008-04-17 21:14:55Z campbellbarton $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,7 +26,7 @@
  *                 Johnny Matthews
  *                 Campbell BArton
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include "Text3d.h" /*This must come first*/
@@ -40,6 +37,7 @@
 #include "BKE_library.h"
 #include "BKE_global.h"
 #include "BKE_main.h"
+#include "BKE_utildefines.h"
 #include "BIF_editfont.h"	/* do_textedit() */
 #include "Curve.h"
 #include "constant.h"
@@ -83,7 +81,7 @@ static PyObject *generate_ModuleIntConstant(char *name, int value);
 struct PyMethodDef M_Text3d_methods[] = {
 	{"New", ( PyCFunction ) M_Text3d_New, METH_VARARGS, NULL},
 	{"Get", ( PyCFunction ) M_Text3d_Get, METH_VARARGS, NULL},
-	{"LoadFont", ( PyCFunction ) M_Text3d_LoadFont, METH_VARARGS, NULL},
+	{"LoadFont", ( PyCFunction ) M_Text3d_LoadFont, METH_O, NULL},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -102,7 +100,7 @@ static int Text3d_compare( BPy_Text3d * a, BPy_Text3d * b );
 /*PyObject *Text3d_getType(BPy_Text3d *self);*/
 static PyObject *Text3d_getName( BPy_Text3d * self );
 static PyObject *Text3d_setName( BPy_Text3d * self, PyObject * args );
-static PyObject *Text3d_setText( BPy_Text3d * self, PyObject * args );
+static PyObject *Text3d_setText( BPy_Text3d * self, PyObject * value );
 static PyObject *Text3d_getText( BPy_Text3d * self );
 static PyObject *Text3d_getDrawMode( BPy_Text3d * self );
 static PyObject *Text3d_setDrawMode( BPy_Text3d * self, PyObject * args );
@@ -149,7 +147,7 @@ static PyMethodDef BPy_Text3d_methods[] = {
 	{"setName", ( PyCFunction ) Text3d_setName,
 	 METH_VARARGS, "() - Sets Text3d Data name"},
 	{"setText", ( PyCFunction ) Text3d_setText,
-	 METH_VARARGS, "() - Sets Text3d Data"},
+	 METH_O, "() - Sets Text3d Data"},
 	{"getText", ( PyCFunction ) Text3d_getText,
 	 METH_NOARGS, "() - Gets Text3d Data"},		 
 	{"getDrawMode", ( PyCFunction ) Text3d_getDrawMode,
@@ -258,7 +256,7 @@ static PyObject *getFloatAttr( BPy_Text3d *self, void *type )
 	float param;
 	struct Curve *curve= self->curve;
 	
-	switch( (int)type ) {
+	switch( GET_INT_FROM_POINTER(type) ) {
 	case EXPP_T3D_ATTR_FRAME_WIDTH: 
 		param = curve->tb[curve->actbox-1].w;
 		break;
@@ -285,7 +283,7 @@ static int setFloatAttrClamp( BPy_Text3d *self, PyObject *value, void *type )
 	struct Curve *curve= self->curve;
 	float min, max;
 
-	switch( (int)type ) {
+	switch( GET_INT_FROM_POINTER(type) ) {
 	case EXPP_T3D_ATTR_FRAME_WIDTH:
 		min = 0.0;
 		max = 50.0;
@@ -664,11 +662,11 @@ static PyObject *Text3d_setName( BPy_Text3d * self, PyObject * args )
 	return Curve_setName( (BPy_Curve*)self,args );
 }
 
-static PyObject *Text3d_setText( BPy_Text3d * self, PyObject * args )
+static PyObject *Text3d_setText( BPy_Text3d * self, PyObject * value )
 {
-	char *text;
+	char *text = PyString_AsString(value);
 
-	if( !PyArg_ParseTuple( args, "s", &text ) )
+	if( !text )
 		return EXPP_ReturnPyObjError( PyExc_AttributeError,
 				"expected string argument" );
 
@@ -1173,13 +1171,13 @@ static PyObject *Text3d_removeFrame( BPy_Text3d * self, PyObject * args )
 }
 
 
-PyObject *M_Text3d_LoadFont( PyObject * self, PyObject * args )
+PyObject *M_Text3d_LoadFont( PyObject * self, PyObject * value )
 {
-	char *fontfile= NULL;
+	char *fontfile= PyString_AsString(value);
 	FILE *file= NULL;
 	VFont *vf= NULL;
 
-	if( !PyArg_ParseTuple( args, "s", &fontfile ) )
+	if( !fontfile )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected a string" );
 	vf= exist_vfont(fontfile);
