@@ -1,15 +1,12 @@
 /*  
- *  $Id: Texture.c 11338 2007-07-22 22:10:54Z campbellbarton $
+ *  $Id: Texture.c 14456 2008-04-17 21:14:55Z campbellbarton $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,11 +24,12 @@
  *
  * Contributor(s): Alex Mole, Nathan Letwory, Joilnen B. Leite, Ken Hughes
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
 */
 #include "Texture.h" /*This must come first*/
 
 #include "BKE_global.h"
+#include "BKE_image.h"
 #include "BKE_main.h"
 #include "BKE_idprop.h"
 #include "BKE_library.h"
@@ -203,11 +201,14 @@ static const EXPP_map_pair tex_type_map[] = {
 
 static const EXPP_map_pair tex_flag_map[] = {
 /* NOTE "CheckerOdd" and "CheckerEven" are new */
-    {"ColorBand",  TEX_COLORBAND },
+	{"ColorBand",  TEX_COLORBAND },
 	{"FlipBlend", TEX_FLIPBLEND},
 	{"NegAlpha", TEX_NEGALPHA},
 	{"CheckerOdd",TEX_CHECKER_ODD},
 	{"CheckerEven",TEX_CHECKER_EVEN},
+	{"PreviewAlpha",TEX_PRV_ALPHA},
+	{"RepeatXMirror",TEX_REPEAT_XMIR},
+	{"RepeatYMirror",TEX_REPEAT_YMIR}, 
 	{NULL, 0}
 };
 
@@ -438,7 +439,10 @@ GETFUNC( getWeight1 );
 GETFUNC( getWeight2 );
 GETFUNC( getWeight3 );
 GETFUNC( getWeight4 );
+#if 0
+/* not defined */
 GETFUNC( getUsers );
+#endif
 
 OLDSETFUNC( setDistMetric );
 OLDSETFUNC( setDistNoise );	/* special case used for ".noisebasis = ...  */
@@ -498,7 +502,7 @@ static int Texture_setNoiseBasis2( BPy_Texture *self, PyObject *args,
 								
 static PyObject *Texture_getColorband( BPy_Texture * self);
 int Texture_setColorband( BPy_Texture * self, PyObject * value);
-static PyObject *Texture_evaluate( BPy_Texture *self, PyObject *args );
+static PyObject *Texture_evaluate( BPy_Texture *self, PyObject *value );
 static PyObject *Texture_copy( BPy_Texture *self );
 
 /*****************************************************************************/
@@ -542,7 +546,7 @@ static PyMethodDef BPy_Texture_methods[] = {
 	 "(s) - Set Dist Noise"},
 	{"setDistMetric", ( PyCFunction ) Texture_oldsetDistMetric, METH_VARARGS,
 	 "(s) - Set Dist Metric"},
-	{"evaluate", ( PyCFunction ) Texture_evaluate, METH_VARARGS,
+	{"evaluate", ( PyCFunction ) Texture_evaluate, METH_O,
 	 "(vector) - evaluate the texture at this position"},
 	{"__copy__", ( PyCFunction ) Texture_copy, METH_NOARGS,
 	 "() - return a copy of the the texture"},
@@ -687,10 +691,6 @@ static PyGetSetDef BPy_Texture_getseters[] = {
 	 (getter)Texture_getType, (setter)Texture_setType,
 	 "Texture's 'Type' mode",
 	 NULL},
-	{"users",
-	 (getter)Texture_getUsers, (setter)NULL,
-	 "Number of texture users",
-	 NULL},
 	{"weight1",
 	 (getter)Texture_getWeight1, (setter)Texture_setWeight1,
 	 "Weight 1 (for Voronoi textures)",
@@ -772,8 +772,8 @@ static PyGetSetDef BPy_Texture_getseters[] = {
 	 (void *)TEX_NORMALMAP},
 	{"colorband",
 	 (getter)Texture_getColorband, (setter)Texture_setColorband,
-	 "Use of image RGB values for normal mapping enabled ('ImageFlags')",
-	 (void *)TEX_NORMALMAP},
+	 "The colorband for this texture",
+	 NULL},
 	{NULL,NULL,NULL,NULL,NULL}  /* Sentinel */
 };
 
@@ -1163,6 +1163,9 @@ static PyObject *M_Texture_FlagsDict( void )
 		PyConstant_Insert(d, "NEGALPHA", PyInt_FromLong(TEX_NEGALPHA));
 		PyConstant_Insert(d, "CHECKER_ODD", PyInt_FromLong(TEX_CHECKER_ODD)); 
 		PyConstant_Insert(d, "CHECKER_EVEN", PyInt_FromLong(TEX_CHECKER_EVEN));
+		PyConstant_Insert(d, "PREVIEW_ALPHA", PyInt_FromLong(TEX_PRV_ALPHA));
+		PyConstant_Insert(d, "REPEAT_XMIR", PyInt_FromLong(TEX_REPEAT_XMIR));
+		PyConstant_Insert(d, "REPEAT_YMIR", PyInt_FromLong(TEX_REPEAT_YMIR));
 	}
 	return Flags;
 }
@@ -1340,18 +1343,14 @@ Tex *Texture_FromPyObject( PyObject * pyobj )
 
 static PyObject *Texture_getExtend( BPy_Texture * self )
 {
-	PyObject *attr = NULL;
 	const char *extend = NULL;
 
 	if( EXPP_map_getStrVal
 	    ( tex_extend_map, self->texture->extend, &extend ) )
-		attr = PyString_FromString( extend );
+		return PyString_FromString( extend );
 
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					      "invalid internal extend mode" );
-
-	return attr;
+	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
+				      "invalid internal extend mode" );
 }
 
 static PyObject *Texture_getImage( BPy_Texture * self )
@@ -1367,7 +1366,6 @@ static PyObject *Texture_getImage( BPy_Texture * self )
 
 static PyObject *Texture_oldgetSType( BPy_Texture * self )
 {
-	PyObject *attr = NULL;
 	const char *stype = NULL;
 	int n_stype;
 
@@ -1384,35 +1382,29 @@ static PyObject *Texture_oldgetSType( BPy_Texture * self )
 
 	if( EXPP_map_getStrVal( tex_stype_map[self->texture->type],
 				n_stype, &stype ) )
-		attr = PyString_FromString( stype );
+		return PyString_FromString( stype );
 
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
+	
+	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "invalid texture stype internally" );
-
-	return attr;
 }
 
 static PyObject *Texture_oldgetType( BPy_Texture * self )
 {
-	PyObject *attr = NULL;
 	const char *type = NULL;
 
 	if( EXPP_map_getStrVal( tex_type_map, self->texture->type, &type ) )
-		attr = PyString_FromString( type );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
+		return PyString_FromString( type );
+	
+	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "invalid texture type internally" );
-
-	return attr;
 }
 
 static int Texture_setAnimFrames( BPy_Texture * self, PyObject * value )
 {
 	return EXPP_setIValueClamped ( value, &self->texture->iuser.frames,
 								EXPP_TEX_ANIMFRAME_MIN,
-								EXPP_TEX_ANIMFRAME_MAX, 'i' );
+								EXPP_TEX_ANIMFRAME_MAX, 'h' );
 }
 
 static int Texture_setIUserCyclic( BPy_Texture * self, PyObject * value )
@@ -1435,7 +1427,7 @@ static int Texture_setAnimLength( BPy_Texture * self, PyObject * value )
 {
 	return EXPP_setIValueClamped ( value, &self->texture->len,
 								EXPP_TEX_ANIMLEN_MIN,
-								EXPP_TEX_ANIMLEN_MAX, 'i' );
+								EXPP_TEX_ANIMLEN_MAX, 'h' );
 }
 
 /* this is too simple to keep supporting? disabled for time being (ton) */
@@ -1469,14 +1461,14 @@ static int Texture_setAnimOffset( BPy_Texture * self, PyObject * value )
 {
 	return EXPP_setIValueClamped ( value, &self->texture->iuser.offset,
 								EXPP_TEX_ANIMOFFSET_MIN,
-								EXPP_TEX_ANIMOFFSET_MAX, 'i' );
+								EXPP_TEX_ANIMOFFSET_MAX, 'h' );
 }
 
 static int Texture_setAnimStart( BPy_Texture * self, PyObject * value )
 {
 	return EXPP_setIValueClamped ( value, &self->texture->iuser.sfra,
 								EXPP_TEX_ANIMSTART_MIN,
-								EXPP_TEX_ANIMSTART_MAX, 'i' );
+								EXPP_TEX_ANIMSTART_MAX, 'h' );
 }
 
 static int Texture_setBrightness( BPy_Texture * self, PyObject * value )
@@ -1540,20 +1532,15 @@ static int Texture_setFilterSize( BPy_Texture * self, PyObject * value )
 static int Texture_setFlags( BPy_Texture * self, PyObject * value )
 {
 	int param;
-	int bitmask = TEX_FLIPBLEND
-					| TEX_COLORBAND
-					| TEX_NEGALPHA
-					| TEX_CHECKER_ODD
-					| TEX_CHECKER_EVEN;
 
-	if( !PyInt_CheckExact ( value ) ) {
+	if( !PyInt_Check( value ) ) {
 		char errstr[128];
-		sprintf ( errstr , "expected int bitmask of 0x%08x", bitmask );
+		sprintf ( errstr , "expected int bitmask of 0x%08x", TEX_FLAG_MASK );
 		return EXPP_ReturnIntError( PyExc_TypeError, errstr );
 	}
 	param = PyInt_AS_LONG ( value );
 
-	if ( ( param & bitmask ) != param )
+	if ( ( param & TEX_FLAG_MASK ) != param )
 		return EXPP_ReturnIntError( PyExc_ValueError,
 						"invalid bit(s) set in mask" );
 
@@ -1573,16 +1560,24 @@ static int Texture_setImage( BPy_Texture * self, PyObject * value )
 {
 	Image *blimg = NULL;
 
-	if( !BPy_Image_Check (value) )
+	if ( value != Py_None && !BPy_Image_Check (value) )
 		return EXPP_ReturnIntError( PyExc_TypeError,
-					      "expected an Image" );
-	blimg = Image_FromPyObject( value );
+					      "expected an Image or None" );
+
 
 	if( self->texture->ima ) {
 		self->texture->ima->id.us--;
+		self->texture->ima = NULL;
 	}
 
+	if ( value == Py_None )
+		return 0;
+
+	blimg = Image_FromPyObject( value );
+
 	self->texture->ima = blimg;
+	self->texture->type = TEX_IMAGE;
+	BKE_image_signal(blimg, &self->texture->iuser, IMA_SIGNAL_RELOAD );
 	id_us_plus( &blimg->id );
 
 	return 0;
@@ -1598,10 +1593,10 @@ static int Texture_setImageFlags( BPy_Texture * self, PyObject * value,
 	 * so set/clear the bit in the bitfield based on the type
 	 */
 
-	if( (int)type ) {
+	if( GET_INT_FROM_POINTER(type) ) {
 		int err;
 		param = self->texture->imaflag;
-		err = EXPP_setBitfield( value, &param, (int)type, 'h' );
+		err = EXPP_setBitfield( value, &param, GET_INT_FROM_POINTER(type), 'h' );
 		if( err )
 			return err;
 
@@ -1618,7 +1613,7 @@ static int Texture_setImageFlags( BPy_Texture * self, PyObject * value,
 					| TEX_CALCALPHA
 					| TEX_NORMALMAP;
 
-		if( !PyInt_CheckExact ( value ) ) {
+		if( !PyInt_Check( value ) ) {
 			char errstr[128];
 			sprintf ( errstr , "expected int bitmask of 0x%08x", bitmask );
 			return EXPP_ReturnIntError( PyExc_TypeError, errstr );
@@ -1645,9 +1640,9 @@ static int Texture_setIUserFlags( BPy_Texture * self, PyObject * value,
 				"expected True/False or 0/1" );
 	
 	if( param )
-		self->texture->iuser.flag |= (int)flag;
+		self->texture->iuser.flag |= GET_INT_FROM_POINTER(flag);
 	else
-		self->texture->iuser.flag &= ~(int)flag;
+		self->texture->iuser.flag &= ~GET_INT_FROM_POINTER(flag);
 	return 0;
 }
 
@@ -1695,7 +1690,7 @@ static int Texture_setNoiseBasis( BPy_Texture * self, PyObject * value )
 {
     int param;
 
-	if( !PyInt_CheckExact ( value ) )
+	if( !PyInt_Check( value ) )
 		return EXPP_ReturnIntError( PyExc_TypeError, 
 				"expected int (see 'Noise' constant dictionary)" );
 
@@ -1719,9 +1714,9 @@ static int Texture_setNoiseBasis2( BPy_Texture * self, PyObject * value,
 	 * attribute, so check the range and set the whole value
 	 */
 
-	if( (int)type == EXPP_TEX_NOISEBASIS2 ) {
+	if( GET_INT_FROM_POINTER(type) == EXPP_TEX_NOISEBASIS2 ) {
     	int param;
-		if( !PyInt_CheckExact ( value ) )
+		if( !PyInt_Check( value ) )
 			return EXPP_ReturnIntError( PyExc_TypeError, 
 					"expected int (see 'Noise' constant dictionary)" );
 
@@ -1741,7 +1736,7 @@ static int Texture_setNoiseBasis2( BPy_Texture * self, PyObject * value,
 	 */
 
 	} else {
-		if( !PyInt_CheckExact ( value ) )
+		if( !PyInt_Check( value ) )
 			return EXPP_ReturnIntError( PyExc_TypeError, 
 					"expected int value of 1" );
 
@@ -1749,7 +1744,7 @@ static int Texture_setNoiseBasis2( BPy_Texture * self, PyObject * value,
 			return EXPP_ReturnIntError( PyExc_ValueError,
 							  "expected int value of 1" );
 
-		self->texture->noisebasis2 = (short)(int)type;
+		self->texture->noisebasis2 = (short)GET_INT_FROM_POINTER(type);
 	}
 	return 0;
 }
@@ -1793,7 +1788,7 @@ static int Texture_setSType( BPy_Texture * self, PyObject * value )
 	short param;
 	const char *dummy = NULL;
 
-	if( !PyInt_CheckExact ( value ) )
+	if( !PyInt_Check( value ) )
 		return EXPP_ReturnIntError( PyExc_TypeError,
 					      "expected int argument" );
 
@@ -2002,13 +1997,7 @@ static int Texture_setIpo( BPy_Texture * self, PyObject * value )
 
 static PyObject *Texture_getAnimFrames( BPy_Texture *self )
 {
-	PyObject *attr = PyInt_FromLong( self->texture->iuser.frames );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyInt_FromLong( self->texture->iuser.frames );
 }
 
 static PyObject *Texture_getIUserCyclic( BPy_Texture *self )
@@ -2023,20 +2012,12 @@ static PyObject *Texture_getIUserCyclic( BPy_Texture *self )
 /* disabled. this option was too stupid! (ton) */
 static PyObject *Texture_getAnimLength( BPy_Texture *self )
 {
-	PyObject *attr = PyInt_FromLong( self->texture->len );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-		"couldn't get attribute" );
-
-	return attr;
+	return PyInt_FromLong( self->texture->len );
 }
 
 static PyObject *Texture_getAnimMontage( BPy_Texture *self )
-{
-	PyObject *attr;
-	
-	attr = Py_BuildValue( "((i,i),(i,i),(i,i),(i,i))",
+{	
+	return Py_BuildValue( "((i,i),(i,i),(i,i),(i,i))",
 						self->texture->fradur[0][0],
 						self->texture->fradur[0][1],
 						self->texture->fradur[1][0],
@@ -2045,187 +2026,93 @@ static PyObject *Texture_getAnimMontage( BPy_Texture *self )
 						self->texture->fradur[2][1],
 						self->texture->fradur[3][0],
 						self->texture->fradur[3][1] );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
 }
 #endif
 
 static PyObject *Texture_getAnimOffset( BPy_Texture *self )
 {
-	PyObject *attr = PyInt_FromLong( self->texture->iuser.offset );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyInt_FromLong( self->texture->iuser.offset );
 }
 
 static PyObject *Texture_getAnimStart( BPy_Texture *self )
 {
-	PyObject *attr = PyInt_FromLong( self->texture->iuser.sfra );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyInt_FromLong( self->texture->iuser.sfra );
 }
 
 static PyObject *Texture_getBrightness( BPy_Texture *self )
 {
-	PyObject *attr = PyFloat_FromDouble ( self->texture->bright );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyFloat_FromDouble ( self->texture->bright );
 }
 
 static PyObject *Texture_getContrast( BPy_Texture *self )
 {
-	PyObject *attr = PyFloat_FromDouble( self->texture->contrast );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyFloat_FromDouble( self->texture->contrast );
 }
 
 static PyObject *Texture_getCrop( BPy_Texture *self )
 {
-	PyObject *attr;
-
-	attr = Py_BuildValue( "(f,f,f,f)",
+	return Py_BuildValue( "(f,f,f,f)",
 							self->texture->cropxmin,
 							self->texture->cropymin,
 							self->texture->cropxmax,
 							self->texture->cropymax );
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
 }
 
 static PyObject *Texture_getDistAmnt( BPy_Texture *self )
 {
-	PyObject *attr = PyFloat_FromDouble( self->texture->dist_amount );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyFloat_FromDouble( self->texture->dist_amount );
 }
 
 static PyObject *Texture_getDistMetric( BPy_Texture *self )
 {
-	PyObject *attr = PyInt_FromLong( self->texture->vn_distm );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyInt_FromLong( self->texture->vn_distm );
 }
 
 static PyObject *Texture_getExp( BPy_Texture *self )
 {
-	PyObject *attr = PyFloat_FromDouble( self->texture->vn_mexp );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyFloat_FromDouble( self->texture->vn_mexp );
 }
 
 static PyObject *Texture_getIntExtend( BPy_Texture * self )
 {
-	PyObject *attr = PyInt_FromLong( self->texture->extend );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyInt_FromLong( self->texture->extend );
 }
 
 static PyObject *Texture_getFieldsPerImage( BPy_Texture *self )
 {
-	PyObject *attr = PyInt_FromLong( self->texture->iuser.fie_ima );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyInt_FromLong( self->texture->iuser.fie_ima );
 }
 
 static PyObject *Texture_getFilterSize( BPy_Texture *self )
 {
-	PyObject *attr = PyFloat_FromDouble( self->texture->filtersize );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyFloat_FromDouble( self->texture->filtersize );
 }
 
 static PyObject *Texture_getFlags( BPy_Texture *self )
 {
-	PyObject *attr = PyInt_FromLong( self->texture->flag );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyInt_FromLong( self->texture->flag );
 }
 
 static PyObject *Texture_getHFracDim( BPy_Texture *self )
 {
-	PyObject *attr = PyInt_FromLong( (long)self->texture->mg_H );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyInt_FromLong( (long)self->texture->mg_H );
 }
 
 static PyObject *Texture_getImageFlags( BPy_Texture *self, void *type )
 {
-	PyObject *attr;
-
-	/*
-	 * type == 0 means attribute "imageFlags"
+	/* type == 0 means attribute "imageFlags"
 	 * other types means attribute "mipmap", "calcAlpha", etc
 	 */
 
-	if( (int)type )
-		attr = EXPP_getBitfield( &self->texture->imaflag, (int)type, 'h' );
+	if( GET_INT_FROM_POINTER(type) )
+		return EXPP_getBitfield( &self->texture->imaflag, GET_INT_FROM_POINTER(type), 'h' );
 	else
-		attr = PyInt_FromLong( self->texture->imaflag );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+		return PyInt_FromLong( self->texture->imaflag );
 }
 
 static PyObject *Texture_getIUserFlags( BPy_Texture *self, void *flag )
 {
-	if( self->texture->iuser.flag & (int)flag )
+	if( self->texture->iuser.flag & GET_INT_FROM_POINTER(flag) )
 		Py_RETURN_TRUE;
 	else
 		Py_RETURN_FALSE;
@@ -2233,231 +2120,108 @@ static PyObject *Texture_getIUserFlags( BPy_Texture *self, void *flag )
 
 static PyObject *Texture_getIScale( BPy_Texture *self )
 {
-	PyObject *attr = PyFloat_FromDouble( self->texture->ns_outscale );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyFloat_FromDouble( self->texture->ns_outscale );
 }
 
 static PyObject *Texture_getLacunarity( BPy_Texture *self )
 {
-	PyObject *attr = PyFloat_FromDouble( self->texture->mg_lacunarity );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyFloat_FromDouble( self->texture->mg_lacunarity );
 }
 
 static PyObject *Texture_getNoiseBasis( BPy_Texture *self )
 {
-	PyObject *attr = PyInt_FromLong( self->texture->noisebasis );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyInt_FromLong( self->texture->noisebasis );
 }
 
 static PyObject *Texture_getNoiseBasis2( BPy_Texture *self, void *type )
 {
-	PyObject *attr;
-
-	/*
-	 * type == EXPP_TEX_NOISEBASIS2 means attribute "noiseBasis2"
+	/* type == EXPP_TEX_NOISEBASIS2 means attribute "noiseBasis2"
 	 * other types means attribute "sine", "saw", or "tri" attribute
 	 */
 
-	if( (int)type == EXPP_TEX_NOISEBASIS2 )
-		attr = PyInt_FromLong( self->texture->noisebasis2 );
+	if( GET_INT_FROM_POINTER(type) == EXPP_TEX_NOISEBASIS2 )
+		return PyInt_FromLong( self->texture->noisebasis2 );
 	else
-		attr = PyInt_FromLong( ( self->texture->noisebasis2 == (int)type ) ? 1 : 0 );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+		return PyInt_FromLong( ( self->texture->noisebasis2 == GET_INT_FROM_POINTER(type) ) ? 1 : 0 );
 }
 
 static PyObject *Texture_getNoiseDepth( BPy_Texture *self )
 {
-	PyObject *attr = PyInt_FromLong( self->texture->noisedepth );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyInt_FromLong( self->texture->noisedepth );
 }
 
 static PyObject *Texture_getNoiseSize( BPy_Texture *self )
 {
-	PyObject *attr = PyFloat_FromDouble( self->texture->noisesize );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyFloat_FromDouble( self->texture->noisesize );
 }
 
 static PyObject *Texture_getNoiseType( BPy_Texture *self )
 {
-	PyObject *attr;
-
 	if ( self->texture->noisetype == TEX_NOISESOFT )
-		attr = PyString_FromString( "soft" );
+		return PyString_FromString( "soft" );
 	else
-		attr = PyString_FromString( "hard" );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+		return PyString_FromString( "hard" );
 }
 
 static PyObject *Texture_getOcts( BPy_Texture *self )
 {
-	PyObject *attr = PyFloat_FromDouble( self->texture->mg_octaves );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyFloat_FromDouble( self->texture->mg_octaves );
 }
 
 static PyObject *Texture_getRepeat( BPy_Texture *self )
 {
-	PyObject *attr;
-
-	attr = Py_BuildValue( "(i,i)", self->texture->xrepeat,
+	return Py_BuildValue( "(i,i)", self->texture->xrepeat,
 									self->texture->yrepeat );
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
 }
 
 static PyObject *Texture_getRGBCol( BPy_Texture *self )
 {
-	PyObject *attr;
-
-	attr = Py_BuildValue( "(f,f,f)", self->texture->rfac,
+	return Py_BuildValue( "(f,f,f)", self->texture->rfac,
 									self->texture->gfac, self->texture->bfac );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
 }
 
 static PyObject *Texture_getSType( BPy_Texture *self )
 {
-	PyObject *attr;
-
 	if( self->texture->type == TEX_VORONOI )
-		attr = PyInt_FromLong( self->texture->vn_coltype );
+		return PyInt_FromLong( self->texture->vn_coltype );
 #if 0
-	else if( self->texture->type == TEX_MUSGRAVE )
-		attr = PyInt_FromLong( self->texture->noisebasis );
+	if( self->texture->type == TEX_MUSGRAVE )
+		return PyInt_FromLong( self->texture->noisebasis );
 #endif
-	else if( self->texture->type == TEX_ENVMAP )
-		attr = PyInt_FromLong( self->texture->env->stype );
-	else 
-		attr = PyInt_FromLong( self->texture->stype );
+	if( self->texture->type == TEX_ENVMAP )
+		return PyInt_FromLong( self->texture->env->stype );
 
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyInt_FromLong( self->texture->stype );
 }
 
 static PyObject *Texture_getTurbulence( BPy_Texture *self )
 {
-	PyObject *attr = PyFloat_FromDouble( self->texture->turbul );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyFloat_FromDouble( self->texture->turbul );
 }
 
 static PyObject *Texture_getType( BPy_Texture *self )
 {
-	PyObject *attr = PyInt_FromLong( self->texture->type );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
-}
-
-static PyObject *Texture_getUsers( BPy_Texture *self )
-{
-	PyObject *attr = PyInt_FromLong( self->texture->id.us );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyInt_FromLong( self->texture->type );
 }
 
 static PyObject *Texture_getWeight1( BPy_Texture *self )
 {
-	PyObject *attr = PyFloat_FromDouble( self->texture->vn_w1 );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyFloat_FromDouble( self->texture->vn_w1 );
 }
 
 static PyObject *Texture_getWeight2( BPy_Texture *self )
 {
-	PyObject *attr = PyFloat_FromDouble( self->texture->vn_w2 );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyFloat_FromDouble( self->texture->vn_w2 );
 }
 
 static PyObject *Texture_getWeight3( BPy_Texture *self )
 {
-	PyObject *attr = PyFloat_FromDouble( self->texture->vn_w3 );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyFloat_FromDouble( self->texture->vn_w3 );
 }
 
 static PyObject *Texture_getWeight4( BPy_Texture *self )
 {
-	PyObject *attr = PyFloat_FromDouble( self->texture->vn_w4 );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"couldn't get attribute" );
-
-	return attr;
+	return PyFloat_FromDouble( self->texture->vn_w4 );
 }
 
 /* #####DEPRECATED###### */
@@ -2714,20 +2478,34 @@ int Texture_setColorband( BPy_Texture * self, PyObject * value)
 	return EXPP_Colorband_fromPyList( &self->texture->coba, value );
 }
 
-static PyObject *Texture_evaluate( BPy_Texture * self, PyObject * args )
+static PyObject *Texture_evaluate( BPy_Texture * self, PyObject * value )
 {
-	VectorObject *vec_in = NULL;
 	TexResult texres= {0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0, NULL};
 	float vec[4];
 	/* int rgbnor; dont use now */
 	
-	if(!PyArg_ParseTuple(args, "O!", &vector_Type, &vec_in) || vec_in->size < 3)
-		return EXPP_ReturnPyObjError(PyExc_TypeError, 
-			"expects a 3D vector object");
-	
-	/* rgbnor = .. we don't need this now */
-	multitex_ext(self->texture, vec_in->vec, NULL, NULL, 1, &texres);
-	
+	if (VectorObject_Check(value)) {
+		if(((VectorObject *)value)->size < 3)
+			return EXPP_ReturnPyObjError(PyExc_TypeError, 
+					"expects a 3D vector object or a tuple of 3 numbers");
+		
+		/* rgbnor = .. we don't need this now */
+		multitex_ext(self->texture, ((VectorObject *)value)->vec, NULL, NULL, 1, &texres);
+	} else {
+		float vec_in[3];
+		if (!PyTuple_Check(value) || PyTuple_Size(value) < 3)
+			return EXPP_ReturnPyObjError(PyExc_TypeError, 
+					"expects a 3D vector object or a tuple of 3 numbers");
+		
+		vec_in[0] = PyFloat_AsDouble(PyTuple_GET_ITEM(value, 0));
+		vec_in[1] = PyFloat_AsDouble(PyTuple_GET_ITEM(value, 1));
+		vec_in[2] = PyFloat_AsDouble(PyTuple_GET_ITEM(value, 2));
+		if (PyErr_Occurred())
+			return EXPP_ReturnPyObjError(PyExc_TypeError, 
+					"expects a 3D vector object or a tuple of 3 numbers");
+		
+		multitex_ext(self->texture, vec_in, NULL, NULL, 1, &texres);
+	}
 	vec[0] = texres.tr;
 	vec[1] = texres.tg;
 	vec[2] = texres.tb;
