@@ -1,14 +1,11 @@
 /* 
- * $Id: Types.c,v 1.41 2007/04/02 21:04:53 khughes Exp $
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * $Id: Types.c 14444 2008-04-16 22:40:48Z hos $
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +23,7 @@
  *
  * Contributor(s): Willian P. Germano, Alex Mole, Joseph Gilbert
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
 */
 
 #include "Types.h" 
@@ -121,14 +118,14 @@ void types_InitAll( void )
 	Text_Type.ob_type = &PyType_Type;
 	Text3d_Type.ob_type = &PyType_Type;
 	Texture_Type.ob_type = &PyType_Type;
-	TimeLine_Type.ob_type = &PyType_Type;
+	//TimeLine_Type.ob_type = &PyType_Type;
 	World_Type.ob_type = &PyType_Type;
 	buffer_Type.ob_type = &PyType_Type;
 	constant_Type.ob_type = &PyType_Type;
 	euler_Type.ob_type = &PyType_Type;
 	matrix_Type.ob_type = &PyType_Type;
 	quaternion_Type.ob_type = &PyType_Type;
-	rgbTuple_Type.ob_type = &PyType_Type;
+	PyType_Ready( &rgbTuple_Type );
 	vector_Type.ob_type = &PyType_Type;
 	property_Type.ob_type = &PyType_Type;
 	point_Type.ob_type = &PyType_Type;
