@@ -1,15 +1,12 @@
 /* 
- * $Id: World.c,v 1.48 2007/03/31 15:31:37 khughes Exp $
+ * $Id: World.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): Jacques Guignot, Johnny Matthews
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
 */
 
 /**
@@ -98,7 +95,7 @@ static int       World_setStar( BPy_World * self, PyObject * args );
 static PyObject *World_getMist( BPy_World * self );
 static PyObject *World_oldsetMist( BPy_World * self, PyObject * args );
 static int       World_setMist( BPy_World * self, PyObject * args );
-static PyObject *World_getScriptLinks( BPy_World * self, PyObject * args );
+static PyObject *World_getScriptLinks( BPy_World * self, PyObject * value );
 static PyObject *World_addScriptLink( BPy_World * self, PyObject * args );
 static PyObject *World_clearScriptLinks( BPy_World * self, PyObject * args );
 static PyObject *World_setCurrent( BPy_World * self );
@@ -209,7 +206,7 @@ static PyMethodDef BPy_World_methods[] = {
 	 "() - Return World Data mist"},
 	{"setMist", ( PyCFunction ) World_oldsetMist, METH_VARARGS,
 	 "() - Return World Data mist"},
-	{"getScriptLinks", ( PyCFunction ) World_getScriptLinks, METH_VARARGS,
+	{"getScriptLinks", ( PyCFunction ) World_getScriptLinks, METH_O,
 	 "(eventname) - Get a list of this world's scriptlinks (Text names) "
 	 "of the given type\n"
 	 "(eventname) - string: FrameChanged, Redraw or Render."},
@@ -439,7 +436,8 @@ static PyObject *M_World_Get( PyObject * self, PyObject * args )
 							      &World_Type );
 			found_world->world = world_iter;
 			PyList_Append( worldlist, ( PyObject * ) found_world );
-
+			Py_DECREF(found_world);
+			
 			world_iter = world_iter->id.next;
 		}
 		return ( worldlist );
@@ -560,13 +558,7 @@ static PyObject *World_clearIpo( BPy_World * self )
 
 static PyObject *World_getSkytype( BPy_World * self )
 {
-	PyObject *attr = PyInt_FromLong( ( long ) self->world->skytype );
-
-	if( attr )
-		return attr;
-
-	return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					"couldn't get World.skytype attribute" ) );
+	return PyInt_FromLong( ( long ) self->world->skytype );
 }
 
 
@@ -578,7 +570,7 @@ static PyObject *World_getSkytype( BPy_World * self )
 
 static int World_setSkytype( BPy_World * self, PyObject * value )
 {
-	if( !PyInt_CheckExact(value) )
+	if( !PyInt_Check(value) )
 		return ( EXPP_ReturnIntError( PyExc_TypeError,
 						"expected int argument" ) );
 	self->world->skytype = (short)PyInt_AsLong(value);
@@ -599,13 +591,7 @@ static PyObject *World_oldsetSkytype( BPy_World * self, PyObject * args )
 
 static PyObject *World_getMode( BPy_World * self )
 {
-	PyObject *attr = PyInt_FromLong( ( long ) self->world->mode );
-
-	if( attr )
-		return attr;
-
-	return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					"couldn't get World.mode attribute" ) );
+	return PyInt_FromLong( ( long ) self->world->mode );
 }
 
 
@@ -617,7 +603,7 @@ static PyObject *World_getMode( BPy_World * self )
 
 static int World_setMode( BPy_World * self, PyObject * value )
 {
-	if( !PyInt_CheckExact(value) )
+	if( !PyInt_Check(value) )
 		return ( EXPP_ReturnIntError( PyExc_TypeError,
 						"expected int argument" ) );
 	self->world->mode = (short)PyInt_AsLong(value);
@@ -639,13 +625,7 @@ static PyObject *World_oldsetMode( BPy_World * self, PyObject * args )
 
 static PyObject *World_getMistype( BPy_World * self )
 {
-	PyObject *attr = PyInt_FromLong( ( long ) self->world->mistype );
-
-	if( attr )
-		return attr;
-
-	return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					"couldn't get World.mistype attribute" ) );
+	return PyInt_FromLong( ( long ) self->world->mistype );
 }
 
 
@@ -657,7 +637,7 @@ static PyObject *World_getMistype( BPy_World * self )
 
 static int World_setMistype( BPy_World * self, PyObject * value )
 {
-	if( !PyInt_CheckExact(value) )
+	if( !PyInt_Check(value) )
 		return ( EXPP_ReturnIntError( PyExc_TypeError,
 						"expected int argument" ) );
 	self->world->mistype = (short)PyInt_AsLong(value);
@@ -673,13 +653,13 @@ static PyObject *World_oldsetMistype( BPy_World * self, PyObject * args )
 
 static PyObject *World_getHor( BPy_World * self )
 {
-	PyObject *attr = PyList_New( 0 );
+	PyObject *attr = PyList_New( 3 );
 	if( !attr )
 		return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
 						"couldn't create list" ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->horr ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->horg ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->horb ) );
+	PyList_SET_ITEM( attr, 0, PyFloat_FromDouble( self->world->horr ) );
+	PyList_SET_ITEM( attr, 1, PyFloat_FromDouble( self->world->horg ) );
+	PyList_SET_ITEM( attr, 2, PyFloat_FromDouble( self->world->horb ) );
 	return attr;
 }
 
@@ -705,13 +685,13 @@ static PyObject *World_oldsetHor( BPy_World * self, PyObject * args )
 
 static PyObject *World_getZen( BPy_World * self )
 {
-	PyObject *attr = PyList_New( 0 );
+	PyObject *attr = PyList_New( 3 );
 	if( !attr )
 		return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
 						"couldn't create list" ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->zenr ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->zeng ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->zenb ) );
+	PyList_SET_ITEM( attr, 0, PyFloat_FromDouble( self->world->zenr ) );
+	PyList_SET_ITEM( attr, 1, PyFloat_FromDouble( self->world->zeng ) );
+	PyList_SET_ITEM( attr, 2, PyFloat_FromDouble( self->world->zenb ) );
 	return attr;
 }
 
@@ -738,13 +718,13 @@ static PyObject *World_oldsetZen( BPy_World * self, PyObject * args )
 
 static PyObject *World_getAmb( BPy_World * self )
 {
-	PyObject *attr = PyList_New( 0 );
+	PyObject *attr = PyList_New( 3 );
 	if( !attr )
 		return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
 						"couldn't create list" ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->ambr ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->ambg ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->ambb ) );
+	PyList_SET_ITEM( attr, 0, PyFloat_FromDouble( self->world->ambr ) );
+	PyList_SET_ITEM( attr, 1, PyFloat_FromDouble( self->world->ambg ) );
+	PyList_SET_ITEM( attr, 2, PyFloat_FromDouble( self->world->ambb ) );
 	return attr;
 }
 
@@ -770,17 +750,17 @@ static PyObject *World_oldsetAmb( BPy_World * self, PyObject * args )
 
 static PyObject *World_getStar( BPy_World * self )
 {
-	PyObject *attr = PyList_New( 0 );
+	PyObject *attr = PyList_New( 7 );
 	if( !attr )
 		return ( EXPP_ReturnPyObjError
 			 ( PyExc_RuntimeError, "couldn't create list" ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->starr ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->starg ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->starb ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->starsize ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->starmindist ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->stardist ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->starcolnoise ) );
+	PyList_SET_ITEM( attr, 0, PyFloat_FromDouble( self->world->starr ) );
+	PyList_SET_ITEM( attr, 1, PyFloat_FromDouble( self->world->starg ) );
+	PyList_SET_ITEM( attr, 2, PyFloat_FromDouble( self->world->starb ) );
+	PyList_SET_ITEM( attr, 3, PyFloat_FromDouble( self->world->starsize ) );
+	PyList_SET_ITEM( attr, 4, PyFloat_FromDouble( self->world->starmindist ) );
+	PyList_SET_ITEM( attr, 5, PyFloat_FromDouble( self->world->stardist ) );
+	PyList_SET_ITEM( attr, 6, PyFloat_FromDouble( self->world->starcolnoise ) );
 	return attr;
 }
 
@@ -815,18 +795,17 @@ static PyObject *World_oldsetStar( BPy_World * self, PyObject * args )
 
 static PyObject *World_getMist( BPy_World * self )
 {
-	PyObject *attr = PyList_New( 0 );
+	PyObject *attr = PyList_New( 4 );
 	if( !attr )
 		return ( EXPP_ReturnPyObjError
 			 ( PyExc_RuntimeError, "couldn't create list" ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->misi ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->miststa ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->mistdist ) );
-	PyList_Append( attr, PyFloat_FromDouble( self->world->misthi ) );
+	PyList_SET_ITEM( attr, 0, PyFloat_FromDouble( self->world->misi ) );
+	PyList_SET_ITEM( attr, 1, PyFloat_FromDouble( self->world->miststa ) );
+	PyList_SET_ITEM( attr, 2, PyFloat_FromDouble( self->world->mistdist ) );
+	PyList_SET_ITEM( attr, 3, PyFloat_FromDouble( self->world->misthi ) );
 	return attr;
 }
 
-
 static int World_setMist( BPy_World * self, PyObject * value )
 {
 	if( !PyList_Check( value ) )
@@ -874,7 +853,7 @@ static PyObject *World_clearScriptLinks( BPy_World * self, PyObject * args )
 }
 
 /* world.getScriptLinks */
-static PyObject *World_getScriptLinks( BPy_World * self, PyObject * args )
+static PyObject *World_getScriptLinks( BPy_World * self, PyObject * value )
 {
 	World *world = self->world;
 	ScriptLink *slink = NULL;
@@ -882,7 +861,7 @@ static PyObject *World_getScriptLinks( BPy_World * self, PyObject * args )
 
 	slink = &( world )->scriptlink;
 
-	ret = EXPP_getScriptLinks( slink, args, 0 );
+	ret = EXPP_getScriptLinks( slink, value, 0 );
 
 	if( ret )
 		return ret;
@@ -1009,37 +988,37 @@ static PyObject *World_insertIpoKey( BPy_World * self, PyObject * args )
 	map = texchannel_to_adrcode(self->world->texact);
 
 	if(key == IPOKEY_ZENITH) {
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_ZEN_R);
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_ZEN_G);
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_ZEN_B);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_ZEN_R, 0);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_ZEN_G, 0);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_ZEN_B, 0);
 	}
 	if(key == IPOKEY_HORIZON) {
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_HOR_R);
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_HOR_G);
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_HOR_B);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_HOR_R, 0);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_HOR_G, 0);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_HOR_B, 0);
 	}
 	if(key == IPOKEY_MIST) {
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_MISI);
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_MISTDI);
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_MISTSTA);
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_MISTHI);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_MISI, 0);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_MISTDI, 0);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_MISTSTA, 0);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_MISTHI, 0);
 	}
 	if(key == IPOKEY_STARS) {
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STAR_R);
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STAR_G);
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STAR_B);
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STARDIST);
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STARSIZE);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STAR_R, 0);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STAR_G, 0);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STAR_B, 0);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STARDIST, 0);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STARSIZE, 0);
 	}
 	if(key == IPOKEY_OFFSET) {
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_OFS_X);
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_OFS_Y);
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_OFS_Z);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_OFS_X, 0);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_OFS_Y, 0);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_OFS_Z, 0);
 	}
 	if(key == IPOKEY_SIZE) {
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_SIZE_X);
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_SIZE_Y);
-		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_SIZE_Z);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_SIZE_X, 0);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_SIZE_Y, 0);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_SIZE_Z, 0);
 	}
 
 	allspace(REMAKEIPO, 0);
