@@ -50,6 +50,7 @@ Example::
 		- ONLYSHADOW   - Let alpha be determined on the degree of shadow.
 		- + HALOXALPHA - Use extreme alpha.
 		- TEXFACE      - UV-Editor assigned texture gives color and texture info for faces.
+		- TEXFACE_ALPHA - When TEXFACE is enabled, use the alpha as well.
 		- + HALOSTAR   - Render halo as a star.
 		- NOMIST       - Set the Material insensitive to mist.
 		- + HALOSHADED - Let halo receive light.
@@ -236,6 +237,18 @@ class Material:
 	@ivar rayMirr:  Mirror reflection amount for raytrace.
 	Value is clamped to the range [0.0,1.0].
 	@type rayMirr:  float
+	@ivar glossMir: Amount of reflection glossy.
+	Value is clamped to the range [0.0,1.0].
+	@type glossMir: float
+	@ivar sampGloss_mir: Reflection glossy samples.
+	Value is clamped to the range [1,1024].
+	@type sampGloss_mir: int
+	@ivar glossTra: Amount of refraction glossy.
+	Value is clamped to the range [0.0,1.0].
+	@type glossTra: float
+	@ivar sampGlossTra: Refraction glossy samples.
+	Value is clamped to the range [1,1024].
+	@type sampGlossTra: int
 	@ivar rayMirrDepth:  Amount of raytrace inter-reflections.
 	Value is clamped to the range [0,10].
 	@type rayMirrDepth:  int
