@@ -105,6 +105,7 @@ Example::
 		- SIZE
 		- LOCROT
 		- LOCROTSIZE
+		- LAYER
 		- PI_STRENGTH
 		- PI_FALLOFF
 		- PI_SURFACEDAMP
@@ -366,6 +367,8 @@ class Object:
 	@ivar matrixLocal: The matrix of the object relative to its parent; if there is no parent,
 	returns the world matrix (L{matrixWorld<Object.Object.matrixWorld>}).
 	@type matrixLocal: Matrix
+	@ivar matrixParentInverse: The inverse if the parents local matrix, set when the objects parent is set (wrapped).
+	@type matrixParentInverse: Matrix
 	@ivar matrixOldWorld: Old-type worldspace matrix (prior to Blender 2.34).
 		Read-only.
 	@type matrixOldWorld: Matrix
@@ -380,7 +383,7 @@ class Object:
 	@ivar sel: The selection state of the object in the current scene. 
 		True is selected, False is unselected. Setting makes the object active.
 	@type sel: boolean
-	@ivar effects: The list of particle effects associated with the object.
+	@ivar effects: The list of particle effects associated with the object. (depricated, will always return an empty list)
 		Read-only.
 	@type effects: list of Effect objects
 	@ivar parentbonename: The string name of the parent bone (if defined).
@@ -414,7 +417,7 @@ class Object:
 				empty_ob.setMatrix(dupe_matrix)
 			Blender.Redraw()
 	@type DupObjects: list of tuples containing (object, matrix)
-	@ivar enableNLAOverride: Whether the object uses NLA or active Action for animation.
+	@ivar enableNLAOverride: Whether the object uses NLA or active Action for animation. When True the NLA is used.
 	@type enableNLAOverride: boolean
 	@ivar enableDupVerts: The DupliVerts status of the object.
 		Does not indicate that this object has any dupliVerts,
@@ -426,6 +429,8 @@ class Object:
 	@type enableDupFaces: boolean
 	@ivar enableDupFacesScale: The DupliFacesScale status of the object.
 	@type enableDupFacesScale: boolean
+	@ivar dupFacesScaleFac: Scale factor for dupliface instance, 1.0 by default.
+	@type dupFacesScaleFac: float	
 	@ivar enableDupFrames: The DupliFrames status of the object.
 		Does not indicate that this object has any dupliFrames,
 		(as returned by L{DupObjects}) just that dupliFrames are enabled.
@@ -465,6 +470,17 @@ class Object:
 
 		Value is clamped to [1,len(ob.materials)]. - [0,0] when there is no materials applied to the object.
 	@type activeMaterial: int
+	@ivar activeShape: The active shape key index for this object.
+
+		The active index is used to select the material to edit in the material buttons,
+		new data created will also use the active material.
+
+		Value is clamped to [1,len(ob.data.key.blocks)]. - [0,0] when there are no keys.
+
+	@type activeShape: int
+	
+	@ivar pinShape: If True, only the activeShape will be displayed.
+	@type pinShape: bool
 	@ivar drawSize: The size to display the Empty.
 	Value clamped to [0.01,10.0].
 	@type drawSize: float
@@ -620,12 +636,25 @@ class Object:
 	@ivar rbShapeBoundType: Rigid body shape bound type.  See L{RBShapes}
 		const dict for values.
 	@type rbShapeBoundType: int
+	@ivar trackAxis: Track axis. Return string 'X' | 'Y' | 'Z' | '-X' | '-Y' | '-Z' (readonly)
+	@type trackAxis: string 
+	@ivar upAxis: Up axis. Return string 'Y' | 'Y' | 'Z' (readonly)
+	@type upAxis: string
 	"""
-
+	def getParticleSystems():
+		"""
+		Return a list of particle systems linked to this object (see Blender.Particle).
+		"""
+		
+	def newParticleSystem():
+		"""
+		Link a new particle system (see Blender.Particle).
+		"""
+		
 	def buildParts():
 		"""
 		Recomputes the particle system. This method only applies to an Object of
-		the type Effect.
+		the type Effect. (depricated, does nothing now, use makeDisplayList instead to update the modifier stack)
 		"""
 
 	def insertShapeKey():
@@ -1162,10 +1191,12 @@ class Object:
 				- 1  - selected
 		"""
 	
-	def getBoundBox():
+	def getBoundBox(worldspace=1):
 		"""
 		Returns the worldspace bounding box of this object.  This works for meshes (out of
 		edit mode) and curves.
+		@type worldspace: int
+		@param worldspace: An optional argument. When zero, the bounding values will be localspace.
 		@rtype: list of 8 (x,y,z) float coordinate vectors (WRAPPED DATA)
 		@return: The coordinates of the 8 corners of the bounding box. Data is wrapped when
 		bounding box is present.
@@ -1173,12 +1204,9 @@ class Object:
 
 	def makeDisplayList():
 		"""
-		Updates this object's display list.  Blender uses display lists to store
-		already transformed data (like a mesh with its vertices already modified
-		by coordinate transformations and armature deformation).  If the object
-		isn't modified, there's no need to recalculate this data.  This method is
-		here for the *few cases* where a script may need it, like when toggling
-		the "SubSurf" mode for a mesh:
+		Forces an update to the objects display data. If the object isn't modified,
+		there's no need to recalculate this data.
+		This method is here for the *few cases* where it is needed.
 
 		Example::
 			import Blender
