@@ -59,6 +59,10 @@ Example::
 		- NEGALPHA - Reverse the alpha value
 		- CHECKER_ODD - Fill the "odd" checkerboard tiles
 		- CHECKER_EVEN - Fill the "even" checkerboard tiles
+		- COLORBAND - Enable colorband for this texture
+        - PREVIEW_ALPHA - Show alpha in preview
+        - REPEAT_XMIR - Mirrors X direction repeat
+        - REPEAT_YMIR - Mirrors Y direction repeat
 
 @type ImageFlags: readonly dictionary
 @var ImageFlags: The available image flags for Texture.imageFlags:
@@ -76,7 +80,7 @@ Example::
 
 @type ExtendModes: readonly dictionary
 @var ExtendModes: Extend, clip, repeat or checker modes for image textures
-		- EXTEND - Extends the colour of the edge
+		- EXTEND - Extends the color of the edge
 		- CLIP - Return alpha 0.0 outside image
 		- CLIPCUBE - Return alpha 0.0 around cube-shaped area around image
 		- REPEAT - Repeat image vertically and horizontally
@@ -205,10 +209,10 @@ Example::
 @type TexCo: readonly dictionary
 
 @var MapTo: Flags for MTex.mapto.
-		- COL - Make the texture affect the basic colour of the material
+		- COL - Make the texture affect the basic color of the material
 		- NOR - Make the texture affect the rendered normal
-		- CSP - Make the texture affect the specularity colour
-		- CMIR - Make the texture affect the mirror colour
+		- CSP - Make the texture affect the specularity color
+		- CMIR - Make the texture affect the mirror color
 		- REF - Make the texture affect the diffuse reflectivity value
 		- SPEC - Make the texture affect the specularity value
 		- HARD - Make the texture affect the hardness value
@@ -428,7 +432,7 @@ class Texture:
 		"""
 		Set the Image of this texture.
 		@param image: The new Image.
-		@type image: Blender Image
+		@type image: Blender Image or None.
 		@warning: This sets the texture's type to 'Image' if it is not already.
 		"""
 
@@ -476,7 +480,7 @@ class Texture:
 		
 		The return value is a 4D vector where (x,y,z,w) are (red, green, blue, intensity)
 		For greyscale textures, often intensity only will be used.
-		@type coord: vector
+		@type coord: vector or tuple of 3 numbers
 		"""
 		
 import id_generics
@@ -511,6 +515,8 @@ class MTex:
 	@ivar neg: Negate texture values mode
 	@ivar noRGB: Convert texture RGB values to intensity values
 	@ivar correctNor: Correct normal mapping for Texture space and Object space
+	@ivar fromDupli: Dupli's instanced from verts, faces or particles, inherit texture coordinate from their parent
+	@ivar fromOrig: Dupli's derive their object coordinates from the original objects transformation
 	@ivar xproj: Projection of X axis to Texture space. L{Proj}
 	@ivar yproj: Projection of Y axis to Texture space. L{Proj}
 	@ivar zproj: Projection of Z axis to Texture space. L{Proj}
@@ -528,6 +534,8 @@ class MTex:
 	@ivar mtAmb: How texture maps to ambient value
 	@ivar mtDisp: How texture maps to displacement
 	@ivar mtWarp: How texture maps to warp
+	@ivar uvlayer: The name of the UV Layer this texture is mapped to (when left blank uses render layer)
+	@type uvlayer: string
 	"""
 
 	def getIpo():
