@@ -1,15 +1,12 @@
 /*
- * $Id: euler.c,v 1.18 2007/03/16 11:38:02 campbellbarton Exp $
+ * $Id: euler.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +23,7 @@
  * 
  * Contributor(s): Joseph Gilbert
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include "Mathutils.h"
@@ -347,6 +344,7 @@ static PyObject *Euler_slice(EulerObject * self, int begin, int end)
 	int count;
 
 	CLAMP(begin, 0, 3);
+	if (end<0) end= 4+end;
 	CLAMP(end, 0, 3);
 	begin = MIN2(begin,end);
 
@@ -368,6 +366,7 @@ static int Euler_ass_slice(EulerObject * self, int begin, int end,
 	PyObject *e, *f;
 
 	CLAMP(begin, 0, 3);
+	if (end<0) end= 4+end;
 	CLAMP(end, 0, 3);
 	begin = MIN2(begin,end);
 
