@@ -1,15 +1,12 @@
 /* 
- * $Id: gen_library.h 11876 2007-08-29 01:57:48Z campbellbarton $
+ * $Id: gen_library.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): Michel Selten, Willian P. Germano, Alex Mole, Joseph Gilbert
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
 */
 
 #ifndef EXPP_gen_library_h
@@ -62,7 +59,7 @@
 	 NULL},\
 	{"tag",\
 	 (getter)GenericLib_getTag, (setter)GenericLib_setTag,\
-	 "temproary tag",\
+	 "temporary tag",\
 	 NULL}
 
 /* Dummy struct for getting the ID from a libdata BPyObject */
