@@ -1,15 +1,12 @@
 /* 
- * $Id: gen_utils.c 11459 2007-08-01 12:47:12Z campbellbarton $
+ * $Id: gen_utils.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,7 +25,7 @@
  * Contributor(s): Michel Selten, Willian P. Germano, Alex Mole, Ken Hughes,
  * Campbell Barton
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
 */
 
 #include "gen_utils.h" /*This must come first*/
@@ -238,6 +235,10 @@ char *event_to_name( short event )
 	switch ( event ) {
 	case SCRIPT_FRAMECHANGED:
 		return "FrameChanged";
+	case SCRIPT_OBJECTUPDATE:
+		return "ObjectUpdate";
+	case SCRIPT_OBDATAUPDATE:
+		return "ObDataUpdate";
 	case SCRIPT_ONLOAD:
 		return "OnLoad";
 	case SCRIPT_ONSAVE:
@@ -383,15 +384,15 @@ void EXPP_allqueue(unsigned short event, short val)
 /************************************************************************/
 /* Scriptlink-related functions, used by scene, object, etc. bpyobjects */
 /************************************************************************/
-PyObject *EXPP_getScriptLinks( ScriptLink * slink, PyObject * args,
+PyObject *EXPP_getScriptLinks( ScriptLink * slink, PyObject * value,
 			       int is_scene )
 {
 	PyObject *list = NULL, *tmpstr;
-	char *eventname = NULL;
+	char *eventname = PyString_AsString(value);
 	int i, event = 0;
 
 
-	if( !PyArg_ParseTuple( args, "s", &eventname ) )
+	if( !eventname )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected event name (string) as argument" );
 	
@@ -406,6 +407,10 @@ PyObject *EXPP_getScriptLinks( ScriptLink * slink, PyObject * args,
 	
 	if( !strcmp( eventname, "FrameChanged" ) )
 		event = SCRIPT_FRAMECHANGED;
+	else if( !strcmp( eventname, "ObjectUpdate" ) )
+		event = SCRIPT_OBJECTUPDATE;
+	else if( !strcmp( eventname, "ObDataUpdate" ) )
+		event = SCRIPT_OBDATAUPDATE;
 	else if( !strcmp( eventname, "Redraw" ) )
 		event = SCRIPT_REDRAW;
 	else if( !strcmp( eventname, "Render" ) )
@@ -562,6 +567,10 @@ PyObject *EXPP_addScriptLink(ScriptLink *slink, PyObject *args, int is_scene)
 
 	if( !strcmp( eventname, "FrameChanged" ) )
 		event = SCRIPT_FRAMECHANGED;
+	else if( !strcmp( eventname, "ObjectUpdate" ) )
+		event = SCRIPT_OBJECTUPDATE;
+	else if( !strcmp( eventname, "ObDataUpdate" ) )
+		event = SCRIPT_OBDATAUPDATE;
 	else if( !strcmp( eventname, "Redraw" ) )
 		event = SCRIPT_REDRAW;
 	else if( !strcmp( eventname, "Render" ) )
@@ -918,3 +927,31 @@ int EXPP_dict_set_item_str( PyObject *dict, char *key, PyObject *value)
 	Py_DECREF( value ); /* delete original */
 	return ret;
 }
+
+/*
+ * Helper function for subtypes that what the base types methods.
+ * The command below needs to have args modified to have 'self' added at the start
+ * ret = PyObject_Call(PyDict_GetItemString(PyList_Type.tp_dict, "sort"), newargs, keywds);
+ * 
+ * This is not easy with the python API so adding a function here,
+ * remember to Py_DECREF the tuple after
+ */
+
+PyObject * EXPP_PyTuple_New_Prepend(PyObject *tuple, PyObject *value)
+{
+	PyObject *item;
+	PyObject *new_tuple;
+	int i;
+	
+	i = PyTuple_Size(tuple);
+	new_tuple = PyTuple_New(i+1);
+	PyTuple_SetItem(new_tuple, 0, value);
+	Py_INCREF(value);
+	while (i) {
+		i--;
+		item = PyTuple_GetItem(tuple, i);
+		PyTuple_SetItem(new_tuple, i+1, item);
+		Py_INCREF(item);
+	}
+	return new_tuple;
+}
