@@ -1,15 +1,12 @@
 /* 
- * $Id: point.c,v 1.6 2006/08/21 14:36:11 khughes Exp $
+ * $Id: point.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): Joseph Gilbert
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
 */
 #include "Mathutils.h"
 
@@ -172,7 +169,7 @@ static PyObject *Point_item(PointObject * self, int i)
 		return EXPP_ReturnPyObjError(PyExc_IndexError,
 		"point[attribute]: array index out of range\n");
 
-	return Py_BuildValue("f", self->coord[i]);
+	return PyFloat_FromDouble( (double)self->coord[i] );
 
 }
 //----------------------------object[]-------------------------
