@@ -1,15 +1,12 @@
 /*
- * $Id: quat.c 10455 2007-04-04 13:18:41Z campbellbarton $
+ * $Id: quat.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +23,7 @@
  * 
  * Contributor(s): Joseph Gilbert
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include "Mathutils.h"
@@ -184,13 +181,13 @@ static PyObject *Quaternion_getattr(QuaternionObject * self, char *name)
 	}
 	if(STREQ(name, "angle")) {
 		mag = self->quat[0];
-		mag = 2 * (acos(mag));
+		mag = 2 * (saacos(mag));
 		mag *= (180 / Py_PI);
 		return PyFloat_FromDouble(mag);
 	}
 	if(STREQ(name, "axis")) {
 		mag = self->quat[0] * (Py_PI / 180);
-		mag = 2 * (acos(mag));
+		mag = 2 * (saacos(mag));
 		mag = sin(mag / 2);
 		for(x = 0; x < 3; x++) {
 			vec[x] = (float)(self->quat[x + 1] / mag);
@@ -350,6 +347,7 @@ static PyObject *Quaternion_slice(QuaternionObject * self, int begin, int end)
 	int count;
 
 	CLAMP(begin, 0, 4);
+	if (end<0) end= 5+end;
 	CLAMP(end, 0, 4);
 	begin = MIN2(begin,end);
 
@@ -371,6 +369,7 @@ static int Quaternion_ass_slice(QuaternionObject * self, int begin, int end,
 	PyObject *q, *f;
 
 	CLAMP(begin, 0, 4);
+	if (end<0) end= 5+end;
 	CLAMP(end, 0, 4);
 	begin = MIN2(begin,end);
 
