@@ -1,13 +1,10 @@
 /*
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +22,7 @@
  *
  * Contributor(s): Campbell Barton
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include "sceneSequence.h" /* This must come first */
@@ -57,7 +54,9 @@ enum seq_consts {
 	EXPP_SEQ_ATTR_LENGTH,
 	EXPP_SEQ_ATTR_START,
 	EXPP_SEQ_ATTR_STARTOFS,
-	EXPP_SEQ_ATTR_ENDOFS
+	EXPP_SEQ_ATTR_ENDOFS,
+	EXPP_SEQ_ATTR_STARTSTILL,
+	EXPP_SEQ_ATTR_ENDSTILL
 };
 
 
@@ -126,7 +125,7 @@ static PyObject *NewSeq_internal(ListBase *seqbase, PyObject * args, Scene *sce)
 	
 	seq = alloc_sequence(seqbase, start, machine); /* warning, this sets last */
 	
-	if (PyTuple_Check(py_data)) {
+	if (PyTuple_Check(py_data) && PyTuple_GET_SIZE(py_data) == 2) {
 		/* Image */
 		PyObject *list;
 		char *name;
@@ -154,9 +153,34 @@ static PyObject *NewSeq_internal(ListBase *seqbase, PyObject * args, Scene *sce)
 		for(a=0; a<seq->len; a++) {
 			name = PyString_AsString(PyList_GetItem( list, a ));
 			strncpy(se->name, name, FILE_MAXFILE-1);
-			se->ok= 1;
 			se++;
 		}		
+	} else if (PyTuple_Check(py_data) && PyTuple_GET_SIZE(py_data) == 3) {
+		float r,g,b;
+		SolidColorVars *colvars;
+		seq->effectdata = MEM_callocN(sizeof(struct SolidColorVars), "solidcolor");
+		colvars = (SolidColorVars *)seq->effectdata;
+		
+		if (!PyArg_ParseTuple( py_data, "fff", &r, &g, &b)) {
+			return EXPP_ReturnPyObjError( PyExc_ValueError,
+					"color needs to be a tuple of 3 floats - (r,g,b)" );
+		}
+		
+		seq->type= SEQ_COLOR;
+		
+		CLAMP(r,0,1);
+		CLAMP(g,0,1);
+		CLAMP(b,0,1);
+		
+		colvars->col[0] = r;
+		colvars->col[1] = b;
+		colvars->col[2] = g;
+		
+		/* basic defaults */
+		seq->strip= strip= MEM_callocN(sizeof(Strip), "strip");
+		strip->len = seq->len = 1;
+		strip->us= 1;
+		strip->stripdata= se= MEM_callocN(seq->len*sizeof(StripElem), "stripelem");
 		
 	} else if (BPy_Sound_Check(py_data)) {
 		/* sound */
@@ -167,7 +191,7 @@ static PyObject *NewSeq_internal(ListBase *seqbase, PyObject * args, Scene *sce)
 		seq->type= SEQ_RAM_SOUND;
 		seq->sound = sound;
 		
-		totframe= (int) ( ((float)(sound->streamlen-1)/( (float)sce->audio.mixrate*4.0 ))* (float)sce->r.frs_sec);
+		totframe= (int) ( ((float)(sound->streamlen-1)/( (float)sce->audio.mixrate*4.0 ))* (float)sce->r.frs_sec / sce->r.frs_sec_base);
 		
 		sound->flags |= SOUND_FLAGS_SEQUENCE;
 		
@@ -177,16 +201,10 @@ static PyObject *NewSeq_internal(ListBase *seqbase, PyObject * args, Scene *sce)
 		strip->len= totframe;
 		strip->us= 1;
 		strncpy(strip->dir, sound->name, FILE_MAXDIR-1);
-		strip->stripdata= se= MEM_callocN(totframe*sizeof(StripElem), "stripelem");
+		strip->stripdata= se= MEM_callocN(sizeof(StripElem), "stripelem");
 
 		/* name sound in first strip */
 		strncpy(se->name, sound->name, FILE_MAXFILE-1);
-
-		for(a=1; a<=totframe; a++, se++) {
-			se->ok= 2; /* why? */
-			se->ibuf= 0;
-			se->nr= a;
-		}
 		
 	} else if (BPy_Scene_Check(py_data)) {
 		/* scene */
@@ -203,8 +221,6 @@ static PyObject *NewSeq_internal(ListBase *seqbase, PyObject * args, Scene *sce)
 			sizeof(seq->name) - 2);
 		strip->len= seq->len;
 		strip->us= 1;
-		if(seq->len>0) strip->stripdata= MEM_callocN(seq->len*sizeof(StripElem), "stripelem");
-		
 	} else {
 		/* movie, pydata is a path to a movie file */
 		char *name = PyString_AsString ( py_data );
@@ -368,6 +384,7 @@ static PyObject *SceneSeq_nextIter( BPy_Sequence * self )
 
 
 
+
 static PyObject *Sequence_getName( BPy_Sequence * self )
 {
 	return PyString_FromString( self->seq->name+2 );
@@ -386,6 +403,32 @@ static int Sequence_setName( BPy_Sequence * self, PyObject * value )
 	return 0;
 }
 
+static PyObject *Sequence_getProxyDir( BPy_Sequence * self )
+{
+	return PyString_FromString( self->seq->strip->proxy ? self->seq->strip->proxy->dir : "" );
+}
+
+static int Sequence_setProxyDir( BPy_Sequence * self, PyObject * value )
+{
+	char *name = NULL;
+	
+	name = PyString_AsString ( value );
+	if( !name ) {
+		return EXPP_ReturnIntError( PyExc_TypeError,
+					      "expected string argument" );
+	}
+	
+	if (strlen(name) == 0) {
+		if (self->seq->strip->proxy) {
+			MEM_freeN(self->seq->strip->proxy); 
+		}
+	} else {
+		self->seq->strip->proxy = MEM_callocN(sizeof(struct StripProxy), "StripProxy");
+		strncpy(self->seq->strip->proxy->dir, name, sizeof(struct StripProxy));
+	}
+	return 0;
+}
+
 
 static PyObject *Sequence_getSound( BPy_Sequence * self )
 {
@@ -512,26 +555,72 @@ static PyObject *Sequence_getImages( BPy_Sequence * self )
 	Strip *strip;
 	StripElem *se;
 	int i;
-	PyObject *attr;
+	PyObject *list, *ret;
+	
+	if (self->seq->type != SEQ_IMAGE) {
+		list = PyList_New(0);
+		ret= Py_BuildValue( "sO", "", list);
+		Py_DECREF(list);
+		return ret;
+	}
 	
-	if (self->seq->type != SEQ_IMAGE)
-		return PyList_New(0);
 			/*return EXPP_ReturnPyObjError( PyExc_TypeError,
 					"Sequence is not an image type" );*/
 	
 	
 	strip = self->seq->strip;
 	se = strip->stripdata;
-	attr = PyList_New(strip->len);
+	list = PyList_New(strip->len);
 	
 	for (i=0; i<strip->len; i++, se++) {
-		PyList_SetItem( attr, i, PyString_FromString(se->name) );
+		PyList_SetItem( list, i, PyString_FromString(se->name) );
 	}
-
-	return attr;
+	
+	ret= Py_BuildValue( "sO", strip->dir, list);
+	Py_DECREF(list);
+	return ret;
 }
 
-
+static int Sequence_setImages( BPy_Sequence * self, PyObject *value )
+{
+	Strip *strip;
+	StripElem *se;
+	int i;
+	PyObject *list;
+	char *basepath, *name;
+	
+	if (self->seq->type != SEQ_IMAGE) {
+		return EXPP_ReturnIntError( PyExc_TypeError,
+				"Sequence is not an image type" );
+	}
+	
+	if( !PyArg_ParseTuple
+	    ( value, "sO!", &basepath, &PyList_Type, &list ) )
+		return EXPP_ReturnIntError( PyExc_TypeError,
+					      "expected string and optional list argument" );
+	
+	strip = self->seq->strip;
+	se = strip->stripdata;
+	
+	/* for now dont support different image list sizes */
+	if (PyList_Size(list) != strip->len) {
+		return EXPP_ReturnIntError( PyExc_TypeError,
+				"at the moment only image lista with the same number of images as the strip are supported" );
+	}
+	
+	strncpy(strip->dir, basepath, sizeof(strip->dir));
+	
+	for (i=0; i<strip->len; i++, se++) {
+		name = PyString_AsString(PyList_GetItem(list, i));
+		if (name) {
+			strncpy(se->name, name, sizeof(se->name));
+		} else {
+			PyErr_Clear();
+		}
+	}
+	
+	return 0;
+}
 
 /*
  * get floating point attributes
@@ -542,7 +631,7 @@ static PyObject *getIntAttr( BPy_Sequence *self, void *type )
 	struct Sequence *seq= self->seq;
 	
 	/*printf("%i %i %i %i %i %i %i %i %i\n", seq->len, seq->start, seq->startofs, seq->endofs, seq->startstill, seq->endstill, seq->startdisp, seq->enddisp, seq->depth );*/
-	switch( (int)type ) {
+	switch( GET_INT_FROM_POINTER(type) ) {
 	case EXPP_SEQ_ATTR_TYPE: 
 		param = seq->type;
 		break;
@@ -561,20 +650,24 @@ static PyObject *getIntAttr( BPy_Sequence *self, void *type )
 	case EXPP_SEQ_ATTR_ENDOFS:
 		param = seq->endofs;
 		break;
+	case EXPP_SEQ_ATTR_STARTSTILL:
+		param = seq->startstill;
+		break;
+	case EXPP_SEQ_ATTR_ENDSTILL:
+		param = seq->endstill;
+		break;
 	default:
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError, 
-				"undefined type in getFloatAttr" );
+				"undefined type in getIntAttr" );
 	}
 
 	return PyInt_FromLong( param );
 }
 
-
 /* internal functions for recursivly updating metastrip locatons */
 static void intern_pos_update(Sequence * seq) {
 	/* update startdisp and enddisp */
-	seq->startdisp = seq->start + seq->startofs - seq->startstill;
-	seq->enddisp = ((seq->start + seq->len) - seq->endofs )+ seq->endstill;
+	calc_sequence_disp(seq);
 }
 
 void intern_recursive_pos_update(Sequence * seq, int offset) {
@@ -592,17 +685,18 @@ void intern_recursive_pos_update(Sequence * seq, int offset) {
 static int setIntAttrClamp( BPy_Sequence *self, PyObject *value, void *type )
 {
 	struct Sequence *seq= self->seq;
-	int number, origval=0;
+	int number, origval=0, regen_data;
 
-	if( !PyInt_CheckExact ( value ) )
+	if( !PyInt_Check( value ) )
 		return EXPP_ReturnIntError( PyExc_TypeError, "expected an int value" );
 	
 	number = PyInt_AS_LONG( value );
 		
-	switch( (int)type ) {
+	switch( GET_INT_FROM_POINTER(type) ) {
 	case EXPP_SEQ_ATTR_CHAN:
 		CLAMP(number, 1, 1024);
 		seq->machine = number;
+		regen_data = 0;
 		break;
 	case EXPP_SEQ_ATTR_START:
 		if (self->seq->type == SEQ_EFFECT)
@@ -611,39 +705,76 @@ static int setIntAttrClamp( BPy_Sequence *self, PyObject *value, void *type )
 		CLAMP(number, -MAXFRAME, MAXFRAME);
 		origval = seq->start;
 		seq->start = number;
+		regen_data = 0;
 		break;
 	
 	case EXPP_SEQ_ATTR_STARTOFS:
-		CLAMP(number, 0, seq->len - seq->endofs);
+		if (self->seq->type == SEQ_EFFECT)
 			return EXPP_ReturnIntError( PyExc_RuntimeError,
 				"This property dosnt apply to an effect" );
+		CLAMP(number, 0, seq->len - seq->endofs);
+		origval = seq->startofs;
 		seq->startofs = number;
+		regen_data = 1;
 		break;
 	case EXPP_SEQ_ATTR_ENDOFS:
 		if (self->seq->type == SEQ_EFFECT)
 			return EXPP_ReturnIntError( PyExc_RuntimeError,
 				"This property dosnt apply to an effect" );
 		CLAMP(number, 0, seq->len - seq->startofs);
+		origval = seq->endofs;
 		seq->endofs = number;
+		regen_data = 1;
+		break;
+	case EXPP_SEQ_ATTR_STARTSTILL:
+		if (self->seq->type == SEQ_EFFECT)
+			return EXPP_ReturnIntError( PyExc_RuntimeError,
+				"This property dosnt apply to an effect" );
+		CLAMP(number, 1, MAXFRAME);
+		origval = seq->startstill;
+		seq->startstill = number;
+		regen_data = 1;
+		break;
+	case EXPP_SEQ_ATTR_ENDSTILL:
+		if (self->seq->type == SEQ_EFFECT)
+			return EXPP_ReturnIntError( PyExc_RuntimeError,
+				"This property dosnt apply to an effect" );
+		CLAMP(number, seq->startstill+1, MAXFRAME);
+		origval = seq->endstill;
+		seq->endstill = number;
+		regen_data = 1;
+		break;
+	case EXPP_SEQ_ATTR_LENGTH:
+		if (self->seq->type == SEQ_EFFECT)
+			return EXPP_ReturnIntError( PyExc_RuntimeError,
+				"cannot set the length of an effect directly" );
+		CLAMP(number, 1, MAXFRAME);
+		origval = seq->len;
+		seq->len = number;
+		regen_data = 1;
 		break;
-	
 	default:
 		return EXPP_ReturnIntError( PyExc_RuntimeError,
 				"undefined type in setFloatAttrClamp" );
 	}
-	
-	intern_pos_update(seq);
-	
-	if ((int)type == EXPP_SEQ_ATTR_START && number != origval )
-		intern_recursive_pos_update(seq, origval - seq->start);
-	
+
+	if (number != origval) {
+		intern_pos_update(seq);
+		
+		if (GET_INT_FROM_POINTER(type) == EXPP_SEQ_ATTR_START)
+			intern_recursive_pos_update(seq, origval - seq->start);
+		
+		if (regen_data) {
+			new_tstripdata(seq);
+		}
+	}
 	return 0;
 }
 
 
 static PyObject *getFlagAttr( BPy_Sequence *self, void *type )
 {
-	if (self->seq->flag & (int)type)
+	if (self->seq->flag & GET_INT_FROM_POINTER(type))
 		Py_RETURN_TRUE;
 	else
 		Py_RETURN_FALSE;
@@ -656,12 +787,16 @@ static PyObject *getFlagAttr( BPy_Sequence *self, void *type )
 
 static int setFlagAttr( BPy_Sequence *self, PyObject *value, void *type )
 {
-	int t = (int)type;
+	int t = GET_INT_FROM_POINTER(type);
+	int param = PyObject_IsTrue( value );
 	
-	if (PyObject_IsTrue(value))
+	if( param == -1 )
+		return EXPP_ReturnIntError( PyExc_TypeError,
+				"expected True/False or 0/1" );
+	
+	if (param)
 		self->seq->flag |= t;
 	else {
-		
 		/* dont allow leftsel and rightsel when its not selected */
 		if (t == SELECT)
 			t = t + SEQ_LEFTSEL + SEQ_RIGHTSEL;
@@ -680,6 +815,10 @@ static PyGetSetDef BPy_Sequence_getseters[] = {
 	 (getter)Sequence_getName, (setter)Sequence_setName,
 	 "Sequence name",
 	  NULL},
+	{"proxyDir",
+	 (getter)Sequence_getProxyDir, (setter)Sequence_setProxyDir,
+	 "Sequence proxy directory",
+	  NULL},
 	{"ipo",
 	 (getter)Sequence_getIpo, (setter)Sequence_setIpo,
 	 "Sequence ipo",
@@ -694,7 +833,7 @@ static PyGetSetDef BPy_Sequence_getseters[] = {
 	 "Sequence name",
 	  NULL},
 	{"images",
-	 (getter)Sequence_getImages, (setter)NULL,
+	 (getter)Sequence_getImages, (setter)Sequence_setImages,
 	 "Sequence scene",
 	  NULL},
 	  
@@ -723,7 +862,15 @@ static PyGetSetDef BPy_Sequence_getseters[] = {
 	 (getter)getIntAttr, (setter)setIntAttrClamp,
 	 "",
 	 (void *) EXPP_SEQ_ATTR_ENDOFS},
-
+	{"startStill",
+	 (getter)getIntAttr, (setter)setIntAttrClamp,
+	 "",
+	 (void *) EXPP_SEQ_ATTR_STARTSTILL},
+	{"endStill",
+	 (getter)getIntAttr, (setter)setIntAttrClamp,
+	 "",
+	 (void *) EXPP_SEQ_ATTR_ENDSTILL},
+	 
 	{"sel",
 	 (getter)getFlagAttr, (setter)setFlagAttr,
 	 "Sequence audio mute option",
@@ -740,10 +887,30 @@ static PyGetSetDef BPy_Sequence_getseters[] = {
 	 (getter)getFlagAttr, (setter)setFlagAttr,
 	 "",
 	 (void *)SEQ_FILTERY},
+	{"flipX",
+	 (getter)getFlagAttr, (setter)setFlagAttr,
+	 "",
+	 (void *)SEQ_FLIPX},
+	{"flipY",
+	 (getter)getFlagAttr, (setter)setFlagAttr,
+	 "",
+	 (void *)SEQ_FLIPY},
 	{"mute",
 	 (getter)getFlagAttr, (setter)setFlagAttr,
 	 "",
 	 (void *)SEQ_MUTE},
+	{"floatBuffer",
+	 (getter)getFlagAttr, (setter)setFlagAttr,
+	 "",
+	 (void *)SEQ_MAKE_FLOAT},
+	{"lock",
+	 (getter)getFlagAttr, (setter)setFlagAttr,
+	 "",
+	 (void *)SEQ_LOCK},
+	{"useProxy",
+	 (getter)getFlagAttr, (setter)setFlagAttr,
+	 "",
+	 (void *)SEQ_USE_PROXY},
 	{"premul",
 	 (getter)getFlagAttr, (setter)setFlagAttr,
 	 "",
@@ -756,10 +923,6 @@ static PyGetSetDef BPy_Sequence_getseters[] = {
 	 (getter)getFlagAttr, (setter)setFlagAttr,
 	 "",
 	 (void *)SEQ_IPO_FRAME_LOCKED},
-	{"ipoLocked",
-	 (getter)getFlagAttr, (setter)setFlagAttr,
-	 "",
-	 (void *)SEQ_IPO_FRAME_LOCKED},
 	 
 	{NULL,NULL,NULL,NULL,NULL}  /* Sentinel */
 };
@@ -1022,6 +1185,12 @@ PyObject *SceneSeq_CreatePyObject( struct Scene * scn, struct Sequence * iter)
 	if( !scn )
 		Py_RETURN_NONE;
 
+	if ( !scn->ed ) {
+		Editing *ed;
+		ed= scn->ed= MEM_callocN( sizeof(Editing), "addseq");
+		ed->seqbasep= &ed->seqbase;
+	}
+	
 	pysceseq =
 		( BPy_SceneSeq * ) PyObject_NEW( BPy_SceneSeq, &SceneSeq_Type );
 
