@@ -1,14 +1,11 @@
 /*
- * $Id: vector.c 10426 2007-03-30 12:39:21Z campbellbarton $
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * $Id: vector.c 14444 2008-04-16 22:40:48Z hos $
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +22,7 @@
  * 
  * Contributor(s): Willian P. Germano & Joseph Gilbert, Ken Hughes
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include "Mathutils.h"
@@ -168,7 +165,7 @@ PyObject *Vector_Resize4D(VectorObject * self)
   extract a quaternion from the vector and the track and up axis */
 PyObject *Vector_ToTrackQuat( VectorObject * self, PyObject * args )
 {
-	float vec[3];
+	float vec[3], quat[4];
 	char *strack, *sup;
 	short track = 2, up = 1;
 
@@ -271,7 +268,9 @@ PyObject *Vector_ToTrackQuat( VectorObject * self, PyObject * args )
 	vec[1] = -self->vec[1];
 	vec[2] = -self->vec[2];
 
-	return newQuaternionObject(vectoquat(vec, track, up), Py_NEW);
+	vectoquat(vec, track, up, quat);
+
+	return newQuaternionObject(quat, Py_NEW);
 }
 
 
@@ -359,6 +358,7 @@ static PyObject *Vector_slice(VectorObject * self, int begin, int end)
 	int count;
 
 	CLAMP(begin, 0, self->size);
+	if (end<0) end= self->size+end+1;
 	CLAMP(end, 0, self->size);
 	begin = MIN2(begin,end);
 
@@ -380,6 +380,7 @@ static int Vector_ass_slice(VectorObject * self, int begin, int end,
 	PyObject *v;
 
 	CLAMP(begin, 0, self->size);
+	if (end<0) end= self->size+end+1;
 	CLAMP(end, 0, self->size);
 	begin = MIN2(begin,end);
 
