@@ -1,18 +1,15 @@
 /**
- * $Id: quicktime_import.c 8397 2006-08-28 09:30:33Z ton $
+ * $Id: quicktime_import.c 14444 2008-04-16 22:40:48Z hos $
  *
  * quicktime_import.c
  *
  * Code to use Quicktime to load images/movies as texture.
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,7 +25,7 @@
  *
  * Contributor(s): Stefan Gartner (sgefant)
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 #ifdef WITH_QUICKTIME
 
@@ -244,6 +241,7 @@ static OSErr QT_get_frameIndexes(struct anim *anim)
 	TimeValue nextTime = 0;
 	TimeValue	startPoint;
 	TimeValue	tmpstartPoint;
+	long sampleCount = 0;
 
 	startPoint = -1;
 
@@ -254,12 +252,12 @@ static OSErr QT_get_frameIndexes(struct anim *anim)
 
 	anim->qtime->framecount = 0;
 
-	while(tmpstartPoint != -1) {
-		nextTime = 0;
-		GetMovieNextInterestingTime(anim->qtime->movie, nextTimeMediaSample, 1, &media, tmpstartPoint, 0, &nextTime, NULL);
-		tmpstartPoint = nextTime;
-		anim->qtime->framecount ++;
-	}
+	sampleCount = GetMediaSampleCount(anim->qtime->theMedia);
+	anErr = GetMoviesError();
+	if (anErr != noErr) return anErr;
+
+	anim->qtime->framecount = sampleCount;
+
 	anim->qtime->frameIndex = (TimeValue *) MEM_callocN(sizeof(TimeValue) * anim->qtime->framecount, "qtframeindex");
 
 	//rewind
@@ -403,6 +401,7 @@ int startquicktime (struct anim *anim)
 	char		*qtname;
 	Str255		dst;
 #endif
+	short depth = 0;
 
 	anim->qtime = MEM_callocN (sizeof(QuicktimeMovie),"animqt");
 	anim->qtime->have_gw = FALSE;
@@ -483,6 +482,9 @@ int startquicktime (struct anim *anim)
 				 anim->qtime->offscreenGWorld,
 				 GetGWorldDevice(anim->qtime->offscreenGWorld));
 		SetMoviePlayHints(anim->qtime->movie, hintsHighQuality, hintsHighQuality);
+		
+		// sets Media and Track!
+		depth = GetFirstVideoTrackPixelDepth(anim);
 
 		QT_get_frameIndexes(anim);
 	}
@@ -491,7 +493,7 @@ int startquicktime (struct anim *anim)
 	LockPixels(anim->qtime->offscreenPixMap);
 
 	//fill blender's anim struct
-	anim->qtime->depth = GetFirstVideoTrackPixelDepth(anim);
+	anim->qtime->depth = depth;
 	
 	anim->duration = anim->qtime->framecount;
 	anim->params = 0;
