@@ -1,14 +1,11 @@
 /* ***************************************
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,13 +23,13 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
 
 
 
     formfactors.c	nov/dec 1992
 
-    $Id: radfactors.c 10455 2007-04-04 13:18:41Z campbellbarton $
+    $Id: radfactors.c 14444 2008-04-16 22:40:48Z hos $
 
  *************************************** */
 
@@ -744,11 +741,9 @@ void subdivideshootElements(int it)
 					}
 				}
 				if(rn) {
-					min[0]= min[1]= min[2]= 1.0e10;
-					max[0]= max[1]= max[2]= -1.0e10;
+					INIT_MINMAX(min, max);
 					/* errmin and max are the filtered colors */
-					errmin[0]= errmin[1]= errmin[2]= 1.0e10;
-					errmax[0]= errmax[1]= errmax[2]= -1.0e10;
+					INIT_MINMAX(errmin, errmax);
 					minmaxradelemfilt(rp->first, min, max, errmin, errmax);
 					
 					/* if small difference between colors: no subdiv */
@@ -912,8 +907,8 @@ void inithemiwindows()
 	memset(vw, 0, sizeof(RadView));
 	vw->rectx= RG.hemires;
 	vw->recty= RG.hemires;
-	vw->rectz= MEM_mallocN(4*vw->rectx*vw->recty, "initwindows");
-	vw->rect= MEM_mallocN(4*vw->rectx*vw->recty, "initwindows");
+	vw->rectz= MEM_mallocN(sizeof(int)*vw->rectx*vw->recty, "initwindows");
+	vw->rect= MEM_mallocN(sizeof(int)*vw->rectx*vw->recty, "initwindows");
 	vw->mynear= RG.maxsize/2000.0;
 	vw->myfar= 2.0*RG.maxsize;
 	vw->wx1= -vw->mynear;
