@@ -1,5 +1,5 @@
 /*
- * $Id: pixelshading.h 10455 2007-04-04 13:18:41Z campbellbarton $
+ * $Id: pixelshading.h 14421 2008-04-14 19:48:14Z blendix $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -45,7 +45,7 @@
  * mask is pixel coverage in bits
  * @return pointer to the object
  */
-void shadeHaloFloat(HaloRen *har, 
+int shadeHaloFloat(HaloRen *har, 
 					float *col, int zz, 
 					float dist, float xn, 
 					float yn, short flarec);
