@@ -1,5 +1,5 @@
 /**
- * $Id: initrender.c 10683 2007-05-08 10:24:39Z blendix $
+ * $Id: initrender.c 14863 2008-05-16 10:20:30Z theeth $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -459,7 +459,10 @@ void RE_SetCamera(Render *re, Object *camera)
 		
 		if(cam->type==CAM_ORTHO) re->r.mode |= R_ORTHO;
 		
-		/* solve this too... all time depending stuff is in convertblender.c? */
+		/* solve this too... all time depending stuff is in convertblender.c?
+		 * Need to update the camera early because it's used for projection matrices
+		 * and other stuff BEFORE the animation update loop is done 
+		 * */
 		if(cam->ipo) {
 			calc_ipo(cam->ipo, frame_to_float(re->r.cfra));
 			execute_ipo(&cam->id, cam->ipo);
@@ -543,7 +546,7 @@ void RE_SetCamera(Render *re, Object *camera)
 	
 	re->viewdx= pixsize;
 	re->viewdy= re->ycor*pixsize;
-	
+
 	if(re->r.mode & R_ORTHO)
 		RE_SetOrtho(re, &viewplane, clipsta, clipend);
 	else 
@@ -557,6 +560,13 @@ void RE_SetPixelSize(Render *re, float pixsize)
 	re->viewdy= re->ycor*pixsize;
 }
 
+void RE_GetCameraWindow(struct Render *re, struct Object *camera, int frame, float mat[][4])
+{
+	re->r.cfra= frame;
+	RE_SetCamera(re, camera);
+	Mat4CpyMat4(mat, re->winmat);
+}
+
 /* ~~~~~~~~~~~~~~~~ part (tile) calculus ~~~~~~~~~~~~~~~~~~~~~~ */
 
 
@@ -596,24 +606,20 @@ void initparts(Render *re)
 	/* mininum part size, but for exr tile saving it was checked already */
 	if(!(re->r.scemode & R_EXR_TILE_FILE)) {
 		if(re->r.mode & R_PANORAMA) {
-			if(re->rectx/xparts < 8) 
+			if(ceil(re->rectx/(float)xparts) < 8) 
 				xparts= 1 + re->rectx/8;
 		}
 		else
-			if(re->rectx/xparts < 64) 
+			if(ceil(re->rectx/(float)xparts) < 64) 
 				xparts= 1 + re->rectx/64;
 		
-		if(re->recty/yparts < 64) 
+		if(ceil(re->recty/(float)yparts) < 64) 
 			yparts= 1 + re->recty/64;
 	}
 	
 	/* part size */
-	partx= re->rectx/xparts;
-	party= re->recty/yparts;
-	
-	/* if remainder pixel, add one, then parts are more equal in size for large panoramas */
-	if(re->rectx % partx)
-		partx++;
+	partx= ceil(re->rectx/(float)xparts);
+	party= ceil(re->recty/(float)yparts);
 	
 	re->xparts= xparts;
 	re->yparts= yparts;
