@@ -78,6 +78,7 @@
 #include "envmap.h"
 #include "initrender.h"
 #include "shadbuf.h"
+#include "pixelblending.h"
 #include "zbuf.h"
 
 
@@ -116,7 +117,7 @@ static struct ListBase RenderList= {NULL, NULL};
 Render R;
 
 /* commandline thread override */
-static int commandline_threads= 0;
+static int commandline_threads= -1;
 
 /* ********* alloc and free ******** */
 
@@ -141,26 +142,23 @@ static void stats_background(RenderStats *rs)
 	float megs_used_memory= mem_in_use/(1024.0*1024.0);
 	char str[400], *spos= str;
 	
-	if(rs->convertdone) {
-		
-		spos+= sprintf(spos, "Fra:%d Mem:%.2fM ", G.scene->r.cfra, megs_used_memory);
-		
-		if(rs->curfield)
-			spos+= sprintf(spos, "Field %d ", rs->curfield);
-		if(rs->curblur)
-			spos+= sprintf(spos, "Blur %d ", rs->curblur);
-		
-		if(rs->infostr) {
-			spos+= sprintf(spos, "| %s", rs->infostr);
-		}
-		else {
-			if(rs->tothalo)
-				spos+= sprintf(spos, "Sce: %s Ve:%d Fa:%d Ha:%d La:%d", G.scene->id.name+2, rs->totvert, rs->totface, rs->tothalo, rs->totlamp);
-			else 
-				spos+= sprintf(spos, "Sce: %s Ve:%d Fa:%d La:%d", G.scene->id.name+2, rs->totvert, rs->totface, rs->totlamp);
-		}
-		printf(str); printf("\n");
-	}	
+	spos+= sprintf(spos, "Fra:%d Mem:%.2fM ", G.scene->r.cfra, megs_used_memory);
+	
+	if(rs->curfield)
+		spos+= sprintf(spos, "Field %d ", rs->curfield);
+	if(rs->curblur)
+		spos+= sprintf(spos, "Blur %d ", rs->curblur);
+	
+	if(rs->infostr) {
+		spos+= sprintf(spos, "| %s", rs->infostr);
+	}
+	else {
+		if(rs->tothalo)
+			spos+= sprintf(spos, "Sce: %s Ve:%d Fa:%d Ha:%d La:%d", G.scene->id.name+2, rs->totvert, rs->totface, rs->tothalo, rs->totlamp);
+		else 
+			spos+= sprintf(spos, "Sce: %s Ve:%d Fa:%d La:%d", G.scene->id.name+2, rs->totvert, rs->totface, rs->totlamp);
+	}
+	printf(str); printf("\n");
 }
 
 void RE_FreeRenderResult(RenderResult *res)
@@ -171,8 +169,9 @@ void RE_FreeRenderResult(RenderResult *res)
 		RenderLayer *rl= res->layers.first;
 		
 		if(rl->rectf) MEM_freeN(rl->rectf);
-		/* acolrect is optionally allocated in shade_tile, only free here since it can be used for drawing */
+		/* acolrect and scolrect are optionally allocated in shade_tile, only free here since it can be used for drawing */
 		if(rl->acolrect) MEM_freeN(rl->acolrect);
+		if(rl->scolrect) MEM_freeN(rl->scolrect);
 		
 		while(rl->passes.first) {
 			RenderPass *rpass= rl->passes.first;
@@ -194,6 +193,22 @@ void RE_FreeRenderResult(RenderResult *res)
 	MEM_freeN(res);
 }
 
+/* version that's compatible with fullsample buffers */
+static void free_render_result(ListBase *lb, RenderResult *rr)
+{
+	RenderResult *rrnext;
+	
+	for(; rr; rr= rrnext) {
+		rrnext= rr->next;
+		
+		if(lb && lb->first)
+			BLI_remlink(lb, rr);
+		
+		RE_FreeRenderResult(rr);
+	}
+}
+
+
 /* all layers except the active one get temporally pushed away */
 static void push_render_result(Render *re)
 {
@@ -330,6 +345,10 @@ static char *get_pass_name(int passtype, int channel)
 		if(channel==-1) return "IndexOB";
 		return "IndexOB.X";
 	}
+	if(passtype == SCE_PASS_MIST) {
+		if(channel==-1) return "Mist";
+		return "Mist.Z";
+	}
 	return "Unknown";
 }
 
@@ -378,23 +397,25 @@ static int passtype_from_name(char *str)
 	if(strcmp(str, "IndexOB")==0)
 		return SCE_PASS_INDEXOB;
 
+	if(strcmp(str, "Mist")==0)
+		return SCE_PASS_MIST;
+	
 	return 0;
 }
 
-
-
-static void render_unique_exr_name(Render *re, char *str)
+static void render_unique_exr_name(Render *re, char *str, int sample)
 {
 	char di[FILE_MAX], name[FILE_MAXFILE], fi[FILE_MAXFILE];
 	
 	BLI_strncpy(di, G.sce, FILE_MAX);
 	BLI_splitdirstring(di, fi);
-	sprintf(name, "%s_%s.exr", fi, re->scene->id.name+2);
-	if(G.background)
-		BLI_make_file_string("/", str, "/tmp/", name);
+	
+	if(sample==0)
+		sprintf(name, "%s_%s.exr", fi, re->scene->id.name+2);
 	else
-		BLI_make_file_string("/", str, U.tempdir, name);
-		
+		sprintf(name, "%s_%s%d.exr", fi, re->scene->id.name+2, sample);
+
+	BLI_make_file_string("/", str, btempdir, name);
 }
 
 static void render_layer_add_pass(RenderResult *rr, RenderLayer *rl, int channels, int passtype)
@@ -413,17 +434,22 @@ static void render_layer_add_pass(RenderResult *rr, RenderLayer *rl, int channel
 			IMB_exr_add_channel(rr->exrhandle, rl->name, get_pass_name(passtype, a), 0, 0, NULL);
 	}
 	else {
+		float *rect;
+		int x;
+		
+		rpass->rect= MEM_mapallocN(sizeof(float)*rectsize, typestr);
+		
 		if(passtype==SCE_PASS_VECTOR) {
-			float *rect;
-			int x;
-			
 			/* initialize to max speed */
-			rect= rpass->rect= MEM_mapallocN(sizeof(float)*rectsize, typestr);
+			rect= rpass->rect;
 			for(x= rectsize-1; x>=0; x--)
 				rect[x]= PASS_VECTOR_MAX;
 		}
-		else
-			rpass->rect= MEM_mapallocN(sizeof(float)*rectsize, typestr);
+		else if(passtype==SCE_PASS_Z) {
+			rect= rpass->rect;
+			for(x= rectsize-1; x>=0; x--)
+				rect[x]= 10e10;
+		}
 	}
 }
 
@@ -497,6 +523,7 @@ static RenderResult *new_render_result(Render *re, rcti *partrct, int crop, int
 		
 		strcpy(rl->name, srl->name);
 		rl->lay= srl->lay;
+		rl->lay_zmask= srl->lay_zmask;
 		rl->layflag= srl->layflag;
 		rl->passflag= srl->passflag;
 		rl->pass_xor= srl->pass_xor;
@@ -526,22 +553,20 @@ static RenderResult *new_render_result(Render *re, rcti *partrct, int crop, int
 			render_layer_add_pass(rr, rl, 3, SCE_PASS_DIFFUSE);
 		if(srl->passflag  & SCE_PASS_SPEC)
 			render_layer_add_pass(rr, rl, 3, SCE_PASS_SPEC);
-		if(re->r.mode & R_SHADOW)
-			if(srl->passflag  & SCE_PASS_SHADOW)
-				render_layer_add_pass(rr, rl, 3, SCE_PASS_SHADOW);
-		if(re->r.mode & R_RAYTRACE) {
-			if(srl->passflag  & SCE_PASS_AO)
-				render_layer_add_pass(rr, rl, 3, SCE_PASS_AO);
-			if(srl->passflag  & SCE_PASS_REFLECT)
-				render_layer_add_pass(rr, rl, 3, SCE_PASS_REFLECT);
-			if(srl->passflag  & SCE_PASS_REFRACT)
-				render_layer_add_pass(rr, rl, 3, SCE_PASS_REFRACT);
-		}
-		if(re->r.mode & R_RADIO)
-			if(srl->passflag  & SCE_PASS_RADIO)
-				render_layer_add_pass(rr, rl, 3, SCE_PASS_RADIO);
+		if(srl->passflag  & SCE_PASS_AO)
+			render_layer_add_pass(rr, rl, 3, SCE_PASS_AO);
+		if(srl->passflag  & SCE_PASS_SHADOW)
+			render_layer_add_pass(rr, rl, 3, SCE_PASS_SHADOW);
+		if(srl->passflag  & SCE_PASS_REFLECT)
+			render_layer_add_pass(rr, rl, 3, SCE_PASS_REFLECT);
+		if(srl->passflag  & SCE_PASS_REFRACT)
+			render_layer_add_pass(rr, rl, 3, SCE_PASS_REFRACT);
+		if(srl->passflag  & SCE_PASS_RADIO)
+			render_layer_add_pass(rr, rl, 3, SCE_PASS_RADIO);
 		if(srl->passflag  & SCE_PASS_INDEXOB)
 			render_layer_add_pass(rr, rl, 1, SCE_PASS_INDEXOB);
+		if(srl->passflag  & SCE_PASS_MIST)
+			render_layer_add_pass(rr, rl, 1, SCE_PASS_MIST);
 		
 	}
 	/* sss, previewrender and envmap don't do layers, so we make a default one */
@@ -576,14 +601,13 @@ static RenderResult *new_render_result(Render *re, rcti *partrct, int crop, int
 
 static int render_scene_needs_vector(Render *re)
 {
-	if((re->r.scemode & R_DOCOMP) || re->r.imtype==R_MULTILAYER) {
-		SceneRenderLayer *srl;
+	SceneRenderLayer *srl;
 	
-		for(srl= re->scene->r.layers.first; srl; srl= srl->next)
-			if(!(srl->layflag & SCE_LAY_DISABLE))
-				if(srl->passflag & SCE_PASS_VECTOR)
-					return 1;
-	}
+	for(srl= re->scene->r.layers.first; srl; srl= srl->next)
+		if(!(srl->layflag & SCE_LAY_DISABLE))
+			if(srl->passflag & SCE_PASS_VECTOR)
+				return 1;
+
 	return 0;
 }
 
@@ -641,9 +665,8 @@ static void merge_render_result(RenderResult *rr, RenderResult *rrpart)
 }
 
 
-static void save_render_result_tile(Render *re, RenderPart *pa)
+static void save_render_result_tile(RenderResult *rr, RenderResult *rrpart)
 {
-	RenderResult *rrpart= pa->result;
 	RenderLayer *rlp;
 	RenderPass *rpassp;
 	int offs, partx, party;
@@ -663,23 +686,23 @@ static void save_render_result_tile(Render *re, RenderPart *pa)
 		if(rlp->rectf) {
 			int a, xstride= 4;
 			for(a=0; a<xstride; a++)
-				IMB_exr_set_channel(re->result->exrhandle, rlp->name, get_pass_name(SCE_PASS_COMBINED, a), 
-								xstride, xstride*pa->rectx, rlp->rectf+a + xstride*offs);
+				IMB_exr_set_channel(rr->exrhandle, rlp->name, get_pass_name(SCE_PASS_COMBINED, a), 
+								xstride, xstride*rrpart->rectx, rlp->rectf+a + xstride*offs);
 		}
 		
 		/* passes are allocated in sync */
 		for(rpassp= rlp->passes.first; rpassp; rpassp= rpassp->next) {
 			int a, xstride= rpassp->channels;
 			for(a=0; a<xstride; a++)
-				IMB_exr_set_channel(re->result->exrhandle, rlp->name, get_pass_name(rpassp->passtype, a), 
-									xstride, xstride*pa->rectx, rpassp->rect+a + xstride*offs);
+				IMB_exr_set_channel(rr->exrhandle, rlp->name, get_pass_name(rpassp->passtype, a), 
+									xstride, xstride*rrpart->rectx, rpassp->rect+a + xstride*offs);
 		}
 		
 	}
 
 	party= rrpart->tilerect.ymin + rrpart->crop;
 	partx= rrpart->tilerect.xmin + rrpart->crop;
-	IMB_exrtile_write_channels(re->result->exrhandle, partx, party);
+	IMB_exrtile_write_channels(rr->exrhandle, partx, party, 0);
 
 	BLI_unlock_thread(LOCK_IMAGE);
 
@@ -688,14 +711,17 @@ static void save_render_result_tile(Render *re, RenderPart *pa)
 static void save_empty_result_tiles(Render *re)
 {
 	RenderPart *pa;
+	RenderResult *rr;
 	
-	IMB_exrtile_clear_channels(re->result->exrhandle);
+	for(rr= re->result; rr; rr= rr->next) {
+		IMB_exrtile_clear_channels(rr->exrhandle);
 		
-	for(pa= re->parts.first; pa; pa= pa->next) {
-		if(pa->ready==0) {
-			int party= pa->disprect.ymin - re->disprect.ymin + pa->crop;
-			int partx= pa->disprect.xmin - re->disprect.xmin + pa->crop;
-			IMB_exrtile_write_channels(re->result->exrhandle, partx, party);
+		for(pa= re->parts.first; pa; pa= pa->next) {
+			if(pa->ready==0) {
+				int party= pa->disprect.ymin - re->disprect.ymin + pa->crop;
+				int partx= pa->disprect.xmin - re->disprect.xmin + pa->crop;
+				IMB_exrtile_write_channels(rr->exrhandle, partx, party, 0);
+			}
 		}
 	}
 }
@@ -723,6 +749,8 @@ void RE_WriteRenderResult(RenderResult *rr, char *filename, int compress)
 	RenderLayer *rl;
 	RenderPass *rpass;
 	void *exrhandle= IMB_exr_get_handle();
+
+	BLI_make_existing_file(filename);
 	
 	/* composite result */
 	if(rr->rectf) {
@@ -822,7 +850,7 @@ static void renderresult_add_names(RenderResult *rr)
 
 
 /* only for temp buffer files, makes exact copy of render result */
-static void read_render_result(Render *re)
+static void read_render_result(Render *re, int sample)
 {
 	RenderLayer *rl;
 	RenderPass *rpass;
@@ -833,14 +861,16 @@ static void read_render_result(Render *re)
 	RE_FreeRenderResult(re->result);
 	re->result= new_render_result(re, &re->disprect, 0, RR_USEMEM);
 
-	render_unique_exr_name(re, str);
+	render_unique_exr_name(re, str, sample);
 	if(IMB_exr_begin_read(exrhandle, str, &rectx, &recty)==0) {
 		IMB_exr_close(exrhandle);
 		printf("cannot read: %s\n", str);
 		return;
 	}
 	
-	if(rectx!=re->result->rectx || recty!=re->result->recty) {
+	printf("read exr tmp file: %s\n", str);
+	
+	if(re->result == NULL || rectx!=re->result->rectx || recty!=re->result->recty) {
 		printf("error in reading render result\n");
 	}
 	else {
@@ -1023,7 +1053,7 @@ void RE_FreeAllRender(void)
 
 /* what doesn't change during entire render sequence */
 /* disprect is optional, if NULL it assumes full window render */
-void RE_InitState(Render *re, RenderData *rd, int winx, int winy, rcti *disprect)
+void RE_InitState(Render *re, Render *source, RenderData *rd, int winx, int winy, rcti *disprect)
 {
 	re->ok= TRUE;	/* maybe flag */
 	
@@ -1045,18 +1075,37 @@ void RE_InitState(Render *re, RenderData *rd, int winx, int winy, rcti *disprect
 		re->recty= winy;
 	}
 	
-	if(re->rectx < 2 || re->recty < 2) {
+	if(re->rectx < 2 || re->recty < 2 || (BKE_imtype_is_movie(rd->imtype) &&
+										  (re->rectx < 16 || re->recty < 16) )) {
 		re->error("Image too small");
 		re->ok= 0;
 	}
 	else {
-		/* check state variables, osa? */
-		if(re->r.mode & (R_OSA)) {
-			re->osa= re->r.osa;
-			if(re->osa>16) re->osa= 16;
-		}
-		else re->osa= 0;
+#ifndef WITH_OPENEXR
+		/* can't do this without openexr support */
+		re->r.scemode &= ~R_EXR_TILE_FILE;
+#endif
 		
+		if(!(re->r.scemode & R_EXR_TILE_FILE))
+			re->r.scemode &= ~R_FULL_SAMPLE;	/* clear, so we can use this flag for test both */
+		
+		/* fullsample wants uniform osa levels */
+		if(source && (re->r.scemode & R_FULL_SAMPLE)) {
+			/* but, if source has no full sample we disable it */
+			if((source->r.scemode & R_FULL_SAMPLE)==0)
+				re->r.scemode &= ~R_FULL_SAMPLE;
+			else
+				re->r.osa= re->osa= source->osa;
+		}
+		else {
+			/* check state variables, osa? */
+			if(re->r.mode & (R_OSA)) {
+				re->osa= re->r.osa;
+				if(re->osa>16) re->osa= 16;
+			}
+			else re->osa= 0;
+		}
+
 		/* always call, checks for gamma, gamma tables and jitter too */
 		make_sample_tables(re);	
 		
@@ -1069,11 +1118,11 @@ void RE_InitState(Render *re, RenderData *rd, int winx, int winy, rcti *disprect
 		/* we clip faces with a minimum of 2 pixel boundary outside of image border. see zbuf.c */
 		re->clipcrop= 1.0f + 2.0f/(float)(re->winx>re->winy?re->winy:re->winx);
 		
-		if(commandline_threads>0 && commandline_threads<=BLENDER_MAX_THREADS)
-			re->r.threads= commandline_threads;
+		RE_init_threadcount(re);
 	}
 }
 
+/* part of external api, not called for regular render pipeline */
 void RE_SetDispRect (struct Render *re, rcti *disprect)
 {
 	re->disprect= *disprect;
@@ -1174,13 +1223,36 @@ static int render_display_draw_enabled(Render *re)
 		return 1;
 }
 
+/* allocate osa new results for samples */
+static RenderResult *new_full_sample_buffers(Render *re, ListBase *lb, rcti *partrct, int crop)
+{
+	int a;
+	
+	if(re->osa==0)
+		return new_render_result(re, partrct, crop, RR_USEMEM);
+	
+	for(a=0; a<re->osa; a++) {
+		RenderResult *rr= new_render_result(re, partrct, crop, RR_USEMEM);
+		BLI_addtail(lb, rr);
+		rr->sample_nr= a;
+	}
+	
+	return lb->first;
+}
+
+
+/* the main thread call, renders an entire part */
 static void *do_part_thread(void *pa_v)
 {
 	RenderPart *pa= pa_v;
 	
 	/* need to return nicely all parts on esc */
 	if(R.test_break()==0) {
-		pa->result= new_render_result(&R, &pa->disprect, pa->crop, RR_USEMEM);
+		
+		if(!R.sss_points && (R.r.scemode & R_FULL_SAMPLE))
+			pa->result= new_full_sample_buffers(&R, &pa->fullresult, &pa->disprect, pa->crop);
+		else
+			pa->result= new_render_result(&R, &pa->disprect, pa->crop, RR_USEMEM);
 
 		if(R.sss_points)
 			zbufshade_sss_tile(pa);
@@ -1190,8 +1262,13 @@ static void *do_part_thread(void *pa_v)
 			zbufshade_tile(pa);
 		
 		/* merge too on break! */
-		if(R.result->exrhandle)
-			save_render_result_tile(&R, pa);
+		if(R.result->exrhandle) {
+			RenderResult *rr, *rrpart;
+			
+			for(rr= R.result, rrpart= pa->result; rr && rrpart; rr= rr->next, rrpart= rrpart->next)
+				save_render_result_tile(rr, rrpart);
+			
+		}
 		else if(render_display_draw_enabled(&R))
 			merge_render_result(R.result, pa->result);
 	}
@@ -1307,7 +1384,7 @@ static RenderPart *find_next_pano_slice(Render *re, int *minx, rctf *viewplane)
 		Mat4CpyMat4(R.winmat, re->winmat);
 		
 		/* rotate database according to part coordinates */
-		project_renderdata(re, projectverto, 1, -R.panodxp*phi);
+		project_renderdata(re, projectverto, 1, -R.panodxp*phi, 1);
 		R.panosi= sin(R.panodxp*phi);
 		R.panoco= cos(R.panodxp*phi);
 	}
@@ -1365,33 +1442,54 @@ static void print_part_stats(Render *re, RenderPart *pa)
 	re->i.infostr= NULL;
 }
 
+/* make osa new results for samples */
+static RenderResult *new_full_sample_buffers_exr(Render *re)
+{
+	int a;
+	
+	for(a=0; a<re->osa; a++) {
+		RenderResult *rr= new_render_result(re, &re->disprect, 0, 1);
+		BLI_addtail(&re->fullresult, rr);
+		rr->sample_nr= a;
+	}
+	
+	return re->fullresult.first;
+}
+
 static void threaded_tile_processor(Render *re)
 {
 	ListBase threads;
 	RenderPart *pa, *nextpa;
-	RenderResult *rr;
 	rctf viewplane= re->viewplane;
 	int rendering=1, counter= 1, drawtimer=0, hasdrawn, minx=0;
 	
-	/* first step; the entire render result, or prepare exr buffer saving */
+	/* first step; free the entire render result, make new, and/or prepare exr buffer saving */
 	RE_FreeRenderResult(re->result);
-	rr= re->result= new_render_result(re, &re->disprect, 0, re->r.scemode & R_EXR_TILE_FILE);
 	
-	if(rr==NULL)
+	if(re->sss_points)
+		re->result= new_render_result(re, &re->disprect, 0, 0);
+	else if(re->r.scemode & R_FULL_SAMPLE)
+		re->result= new_full_sample_buffers_exr(re);
+	else
+		re->result= new_render_result(re, &re->disprect, 0, re->r.scemode & R_EXR_TILE_FILE);
+	
+	if(re->result==NULL)
 		return;
+	
 	/* warning; no return here without closing exr file */
-//	if(re->re->test_break())
-//		return;
 	
 	initparts(re);
 	
-	if(rr->exrhandle) {
+	if(re->result->exrhandle) {
+		RenderResult *rr;
 		char str[FILE_MAX];
 		
-		render_unique_exr_name(re, str);
+		for(rr= re->result; rr; rr= rr->next) {
+			render_unique_exr_name(re, str, rr->sample_nr);
 		
-		printf("write exr tmp file, %dx%d, %s\n", rr->rectx, rr->recty, str);
-		IMB_exrtile_begin_write(rr->exrhandle, str, rr->rectx, rr->recty, rr->rectx/re->xparts, rr->recty/re->yparts);
+			printf("write exr tmp file, %dx%d, %s\n", rr->rectx, rr->recty, str);
+			IMB_exrtile_begin_write(rr->exrhandle, str, 0, rr->rectx, rr->recty, re->partx, re->party);
+		}
 	}
 	
 	BLI_init_threads(&threads, do_part_thread, re->r.threads);
@@ -1446,7 +1544,7 @@ static void threaded_tile_processor(Render *re)
 						re->display_draw(pa->result, NULL);
 					print_part_stats(re, pa);
 					
-					RE_FreeRenderResult(pa->result);
+					free_render_result(&pa->fullresult, pa->result);
 					pa->result= NULL;
 					re->i.partsdone++;
 					hasdrawn= 1;
@@ -1470,11 +1568,20 @@ static void threaded_tile_processor(Render *re)
 		
 	}
 	
-	if(rr->exrhandle) {
+	if(re->result->exrhandle) {
+		RenderResult *rr;
+
 		save_empty_result_tiles(re);
-		IMB_exr_close(rr->exrhandle);
-		rr->exrhandle= NULL;
-		read_render_result(re);
+		
+		for(rr= re->result; rr; rr= rr->next) {
+			IMB_exr_close(rr->exrhandle);
+			rr->exrhandle= NULL;
+		}
+		
+		free_render_result(&re->fullresult, re->result);
+		re->result= NULL;
+		
+		read_render_result(re, 0);
 	}
 	
 	/* unset threadsafety */
@@ -1483,7 +1590,6 @@ static void threaded_tile_processor(Render *re)
 	BLI_end_threads(&threads);
 	freeparts(re);
 	re->viewplane= viewplane; /* restore viewplane, modified by pano render */
-
 }
 
 /* currently only called by preview renders and envmap */
@@ -1493,14 +1599,16 @@ void RE_TileProcessor(Render *re, int firsttile, int threaded)
 	
 	re->i.partsdone= firsttile;
 
-	re->i.starttime= PIL_check_seconds_timer();
+	if(!re->sss_points)
+		re->i.starttime= PIL_check_seconds_timer();
 
 	if(threaded)
 		threaded_tile_processor(re);
 	else
 		render_tile_processor(re, firsttile);
 		
-	re->i.lastframetime= PIL_check_seconds_timer()- re->i.starttime;
+	if(!re->sss_points)
+		re->i.lastframetime= PIL_check_seconds_timer()- re->i.starttime;
 	re->stats_draw(&re->i);
 }
 
@@ -1723,7 +1831,8 @@ static void load_backbuffer(Render *re)
 		char name[256];
 		
 		strcpy(name, re->r.backbuf);
-		BLI_convertstringcode(name, G.sce, re->r.cfra);
+		BLI_convertstringcode(name, G.sce);
+		BLI_convertstringframe(name, re->r.cfra);
 		
 		if(re->backbuf) {
 			re->backbuf->id.us--;
@@ -1808,14 +1917,18 @@ static void do_render_fields_blur_3d(Render *re)
 static void render_scene(Render *re, Scene *sce, int cfra)
 {
 	Render *resc= RE_NewRender(sce->id.name);
+	int winx= re->winx, winy= re->winy;
 	
 	sce->r.cfra= cfra;
 		
-	/* initial setup */
-	RE_InitState(resc, &sce->r, re->winx, re->winy, &re->disprect);
+	/* exception: scene uses own size (unfinished code) */
+	if(0) {
+		winx= (sce->r.size*sce->r.xsch)/100;
+		winy= (sce->r.size*sce->r.ysch)/100;
+	}
 	
-	/* this to enable this scene to create speed vectors */
-	resc->r.scemode |= R_DOCOMP;
+	/* initial setup */
+	RE_InitState(resc, re, &sce->r, winx, winy, &re->disprect);
 	
 	/* still unsure entity this... */
 	resc->scene= sce;
@@ -1831,10 +1944,15 @@ static void render_scene(Render *re, Scene *sce, int cfra)
 	do_render_fields_blur_3d(resc);
 }
 
-static void ntree_render_scenes(Render *re)
+static void tag_scenes_for_render(Render *re)
 {
 	bNode *node;
-	int cfra= re->scene->r.cfra;
+	Scene *sce;
+	
+	for(sce= G.main->scene.first; sce; sce= sce->id.next)
+		sce->id.flag &= ~LIB_DOIT;
+	
+	re->scene->id.flag |= LIB_DOIT;
 	
 	if(re->scene->nodetree==NULL) return;
 	
@@ -1844,12 +1962,21 @@ static void ntree_render_scenes(Render *re)
 			if(node->id) {
 				if(node->id != (ID *)re->scene)
 					node->id->flag |= LIB_DOIT;
-				else
-					node->id->flag &= ~LIB_DOIT;
 			}
 		}
 	}
 	
+}
+
+static void ntree_render_scenes(Render *re)
+{
+	bNode *node;
+	int cfra= re->scene->r.cfra;
+	
+	if(re->scene->nodetree==NULL) return;
+	
+	tag_scenes_for_render(re);
+	
 	/* now foreach render-result node tagged we do a full render */
 	/* results are stored in a way compisitor will find it */
 	for(node= re->scene->nodetree->nodes.first; node; node= node->next) {
@@ -1895,6 +2022,115 @@ static void render_composit_stats(char *str)
 	R.i.infostr= NULL;
 }
 
+
+/* reads all buffers, calls optional composite, merges in first result->rectf */
+static void do_merge_fullsample(Render *re, bNodeTree *ntree)
+{
+	float *rectf, filt[3][3];
+	int sample;
+	
+	/* filtmask needs it */
+	R= *re;
+	
+	/* we accumulate in here */
+	rectf= MEM_mapallocN(re->rectx*re->recty*sizeof(float)*4, "fullsample rgba");
+	
+	for(sample=0; sample<re->r.osa; sample++) {
+		RenderResult rres;
+		int x, y, mask;
+		
+		/* set all involved renders on the samplebuffers (first was done by render itself) */
+		/* also function below assumes this */
+		if(sample) {
+			Render *re1;
+			
+			tag_scenes_for_render(re);
+			for(re1= RenderList.first; re1; re1= re1->next) {
+				if(re1->scene->id.flag & LIB_DOIT)
+					if(re1->r.scemode & R_FULL_SAMPLE)
+						read_render_result(re1, sample);
+			}
+		}
+
+		/* composite */
+		if(ntree) {
+			ntreeCompositTagRender(re->scene);
+			ntreeCompositTagAnimated(ntree);
+			
+			ntreeCompositExecTree(ntree, &re->r, G.background==0);
+		}
+		
+		/* ensure we get either composited result or the active layer */
+		RE_GetResultImage(re, &rres);
+		
+		/* accumulate with filter, and clip */
+		mask= (1<<sample);
+		mask_array(mask, filt);
+
+		for(y=0; y<re->recty; y++) {
+			float *rf= rectf + 4*y*re->rectx;
+			float *col= rres.rectf + 4*y*re->rectx;
+				
+			for(x=0; x<re->rectx; x++, rf+=4, col+=4) {
+				if(col[0]<0.0f) col[0]=0.0f; else if(col[0] > 1.0f) col[0]= 1.0f;
+				if(col[1]<0.0f) col[1]=0.0f; else if(col[1] > 1.0f) col[1]= 1.0f;
+				if(col[2]<0.0f) col[2]=0.0f; else if(col[2] > 1.0f) col[2]= 1.0f;
+				
+				add_filt_fmask_coord(filt, col, rf, re->rectx, re->recty, x, y);
+			}
+		}
+		
+		/* show stuff */
+		if(sample!=re->osa-1) {
+			/* weak... the display callback wants an active renderlayer pointer... */
+			re->result->renlay= render_get_active_layer(re, re->result);
+			re->display_draw(re->result, NULL);
+		}
+		
+		if(re->test_break())
+			break;
+	}
+	
+	if(re->result->rectf) 
+		MEM_freeN(re->result->rectf);
+	re->result->rectf= rectf;
+}
+
+void RE_MergeFullSample(Render *re, Scene *sce, bNodeTree *ntree)
+{
+	Scene *scene;
+	bNode *node;
+	
+	/* first call RE_ReadRenderResult on every renderlayer scene. this creates Render structs */
+	
+	/* tag scenes unread */
+	for(scene= G.main->scene.first; scene; scene= scene->id.next) 
+		scene->id.flag |= LIB_DOIT;
+	
+	for(node= ntree->nodes.first; node; node= node->next) {
+		if(node->type==CMP_NODE_R_LAYERS) {
+			Scene *nodescene= (Scene *)node->id;
+			
+			if(nodescene==NULL) nodescene= sce;
+			if(nodescene->id.flag & LIB_DOIT) {
+				nodescene->r.mode |= R_OSA;	/* render struct needs tables */
+				RE_ReadRenderResult(sce, nodescene);
+				nodescene->id.flag &= ~LIB_DOIT;
+			}
+		}
+	}
+	
+	/* own render result should be read/allocated */
+	if(G.scene->id.flag & LIB_DOIT)
+		RE_ReadRenderResult(G.scene, G.scene);
+	
+	/* and now we can draw (result is there) */
+	re->display_init(re->result);
+	re->display_clear(re->result);
+	
+	do_merge_fullsample(re, ntree);
+}
+
 /* returns fully composited render-result on given time step (in RenderData) */
 static void do_render_composite_fields_blur_3d(Render *re)
 {
@@ -1914,28 +2150,41 @@ static void do_render_composite_fields_blur_3d(Render *re)
 	if(re->r.scemode & R_SINGLE_LAYER)
 		pop_render_result(re);
 	
-	if(!re->test_break() && ntree) {
-		ntreeCompositTagRender(re->scene);
-		ntreeCompositTagAnimated(ntree);
+	if(!re->test_break()) {
 		
-		if(re->r.scemode & R_DOCOMP) {
-			/* checks if there are render-result nodes that need scene */
-			if((re->r.scemode & R_SINGLE_LAYER)==0)
-				ntree_render_scenes(re);
-			
-			if(!re->test_break()) {
-				ntree->stats_draw= render_composit_stats;
-				ntree->test_break= re->test_break;
-				/* in case it was never initialized */
-				R.stats_draw= re->stats_draw;
+		if(ntree) {
+			ntreeCompositTagRender(re->scene);
+			ntreeCompositTagAnimated(ntree);
+		}
+		
+		if(1 || !(re->r.scemode & R_COMP_RERENDER)) {
+			if(ntree && re->r.scemode & R_DOCOMP) {
+				/* checks if there are render-result nodes that need scene */
+				if((re->r.scemode & R_SINGLE_LAYER)==0)
+					ntree_render_scenes(re);
 				
-				ntreeCompositExecTree(ntree, &re->r, G.background==0);
-				ntree->stats_draw= NULL;
-				ntree->test_break= NULL;
+				if(!re->test_break()) {
+					ntree->stats_draw= render_composit_stats;
+					ntree->test_break= re->test_break;
+					/* in case it was never initialized */
+					R.stats_draw= re->stats_draw;
+					
+					if(re->r.scemode & R_FULL_SAMPLE) 
+						do_merge_fullsample(re, ntree);
+					else
+						ntreeCompositExecTree(ntree, &re->r, G.background==0);
+					
+					ntree->stats_draw= NULL;
+					ntree->test_break= NULL;
+				}
 			}
+			else if(re->r.scemode & R_FULL_SAMPLE)
+				do_merge_fullsample(re, NULL);
 		}
 	}
 
+	/* weak... the display callback wants an active renderlayer pointer... */
+	re->result->renlay= render_get_active_layer(re, re->result);
 	re->display_draw(re->result, NULL);
 }
 
@@ -1998,8 +2247,17 @@ static void yafrayRender(Render *re)
 	RE_Database_Free(re);
 }
 
+
+
 #endif /* disable yafray */
 
+static void renderresult_stampinfo()
+{
+	RenderResult rres;
+	/* this is the basic trick to get the displayed float or char rect from render result */
+	RE_GetResultImage(RE_GetRender(G.scene->id.name), &rres);
+	BKE_stamp_buf((unsigned char *)rres.rect32, rres.rectf, rres.rectx, rres.recty, 4);
+}
 
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
 
@@ -2035,7 +2293,14 @@ static void do_render_all_options(Render *re)
 	renderresult_add_names(re->result);
 	
 	re->i.lastframetime= PIL_check_seconds_timer()- re->i.starttime;
+	
 	re->stats_draw(&re->i);
+	
+	/* stamp image info here */
+	if((G.scene->r.scemode & R_STAMP_INFO) && (G.scene->r.stamp & R_STAMP_DRAW)) {
+		renderresult_stampinfo();
+		re->display_draw(re->result, NULL);
+	}
 }
 
 static int is_rendering_allowed(Render *re)
@@ -2069,14 +2334,20 @@ static int is_rendering_allowed(Render *re)
 	if(re->r.scemode & R_EXR_TILE_FILE) {
 		char str[FILE_MAX];
 		
-		render_unique_exr_name(re, str);
+		render_unique_exr_name(re, str, 0);
 		
 		if (BLI_is_writable(str)==0) {
 			re->error("Can not save render buffers, check the temp default path");
 			return 0;
 		}
 		
+		/* no osa + fullsample won't work... */
+		if(re->osa==0)
+			re->r.scemode &= ~R_FULL_SAMPLE;
+		
 	}
+	else
+		re->r.scemode &= ~R_FULL_SAMPLE;	/* clear to be sure */
 	
 	if(re->r.scemode & R_DOCOMP) {
 		if(re->scene->use_nodes) {
@@ -2161,29 +2432,21 @@ static int render_initialize_from_scene(Render *re, Scene *scene)
 		disprect.ymax= winy;
 	}
 	
-	if(scene->r.scemode & R_EXR_TILE_FILE) {
-		int partx= winx/scene->r.xparts, party= winy/scene->r.yparts;
-		
-		/* stupid exr tiles dont like different sizes */
-		if(winx != partx*scene->r.xparts || winy != party*scene->r.yparts) {
-			re->error("Sorry... exr tile saving only allowed with equally sized parts");
-			return 0;
-		}
-		if((scene->r.mode & R_FIELDS) && (party & 1)) {
-			re->error("Sorry... exr tile saving only allowed with equally sized parts");
-			return 0;
-		}
-	}
+	re->scene= scene;
+	
+	/* check all scenes involved */
+	tag_scenes_for_render(re);
 	
 	if(scene->r.scemode & R_SINGLE_LAYER)
 		push_render_result(re);
 	
-	RE_InitState(re, &scene->r, winx, winy, &disprect);
+	RE_InitState(re, NULL, &scene->r, winx, winy, &disprect);
+	if(!re->ok)  /* if an error was printed, abort */
+		return 0;
 	
 	/* initstate makes new result, have to send changed tags around */
 	ntreeCompositTagRender(re->scene);
 	
-	re->scene= scene;
 	if(!is_rendering_allowed(re))
 		return 0;
 	
@@ -2314,17 +2577,37 @@ void RE_BlenderAnim(Render *re, Scene *scene, int sfra, int efra)
 			}
 		}
 	} else {
-		for(scene->r.cfra= sfra; 
-		    scene->r.cfra<=efra; scene->r.cfra++) {
-			re->r.cfra= scene->r.cfra;	   /* weak.... */
-		
-			do_render_all_options(re);
+		for(scene->r.cfra= sfra; scene->r.cfra<=efra; scene->r.cfra++) {
+			char name[FILE_MAX];
+			if (scene->r.mode & (R_NO_OVERWRITE | R_TOUCH) ) {
+				BKE_makepicstring(name, scene->r.pic, scene->r.cfra, scene->r.imtype);
+			}
+			
+			if (scene->r.mode & R_NO_OVERWRITE && BLI_exist(name)) {
+				printf("skipping existing frame \"%s\"\n", name);
+				continue;
+			}
+			if (scene->r.mode & R_TOUCH && !BLI_exist(name)) {
+				BLI_make_existing_file(name); /* makes the dir if its not there */
+				BLI_touch(name);
+			}
 
+			re->r.cfra= scene->r.cfra;	   /* weak.... */
+			
+			do_render_all_options(re);
+			
 			if(re->test_break() == 0) {
 				do_write_image_or_movie(re, scene, mh);
 			}
 		
-			if(G.afbreek==1) break;
+			if(G.afbreek==1) {
+				/* remove touched file */
+				if (scene->r.mode & R_TOUCH && BLI_exist(name) && BLI_filepathsize(name) == 0) {
+					BLI_delete(name, 0, 0);
+				}
+				
+				break;
+			}
 		}
 	}
 	
@@ -2340,7 +2623,7 @@ void RE_BlenderAnim(Render *re, Scene *scene, int sfra, int efra)
 
 /* note; repeated win/disprect calc... solve that nicer, also in compo */
 
-/* only temp file! */
+/* only the temp file! */
 void RE_ReadRenderResult(Scene *scene, Scene *scenode)
 {
 	Render *re;
@@ -2368,17 +2651,32 @@ void RE_ReadRenderResult(Scene *scene, Scene *scenode)
 	if(scenode)
 		scene= scenode;
 	
-	re= RE_NewRender(scene->id.name);
-	RE_InitState(re, &scene->r, winx, winy, &disprect);
+	/* get render: it can be called from UI with draw callbacks */
+	re= RE_GetRender(scene->id.name);
+	if(re==NULL)
+		re= RE_NewRender(scene->id.name);
+	RE_InitState(re, NULL, &scene->r, winx, winy, &disprect);
 	re->scene= scene;
 	
-	read_render_result(re);
+	read_render_result(re, 0);
 }
 
 void RE_set_max_threads(int threads)
 {
-	if(threads>0 && threads<=BLENDER_MAX_THREADS)
+	if (threads==0) {
+		commandline_threads = BLI_system_thread_count();
+	} else if(threads>=1 && threads<=BLENDER_MAX_THREADS) {
 		commandline_threads= threads;
-	else
+	} else {
 		printf("Error, threads has to be in range 1-%d\n", BLENDER_MAX_THREADS);
+	}
+}
+
+void RE_init_threadcount(Render *re) 
+{
+	if ((re->r.mode & R_FIXED_THREADS)==0 || commandline_threads == 0) { /* Automatic threads */
+		re->r.threads = BLI_system_thread_count();
+	} else if(commandline_threads >= 1 && commandline_threads<=BLENDER_MAX_THREADS) {
+		re->r.threads= commandline_threads;
+	}
 }
