@@ -1,15 +1,12 @@
 /**
  * $Id: 
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <time.h>
@@ -39,6 +36,7 @@
 #include <config.h>
 #endif
 
+#include "blendef.h"
 #include "MEM_guardedalloc.h"
 
 #include "DNA_color_types.h"
@@ -159,6 +157,25 @@ void test_obpoin_but(char *name, ID **idpp)
 	*idpp= NULL;
 }
 
+/* tests for an object of type OB_MESH */
+void test_meshobpoin_but(char *name, ID **idpp)
+{
+	ID *id;
+
+	id = G.main->object.first;
+	while(id) {
+		Object *ob = (Object *)id;
+		if(ob->type == OB_MESH && strcmp(name, id->name + 2) == 0) {
+			*idpp = id;
+			/* checks lib data, sets correct flag for saving then */
+			id_lib_extern(id);
+			return;
+		}
+		id = id->next;
+	}
+	*idpp = NULL;
+}
+
 void test_meshpoin_but(char *name, ID **idpp)
 {
 	ID *id;
@@ -299,6 +316,7 @@ static void curvemap_buttons_zoom_out(void *cumap_v, void *unused)
 				d1= cumap->curr.xmin - cumap->clipr.xmin;
 		cumap->curr.xmin-= d1;
 		
+		d1= d;
 		if(cumap->flag & CUMA_DO_CLIP) 
 			if(cumap->curr.xmax+d > cumap->clipr.xmax)
 				d1= -cumap->curr.xmax + cumap->clipr.xmax;
@@ -311,6 +329,7 @@ static void curvemap_buttons_zoom_out(void *cumap_v, void *unused)
 				d1= cumap->curr.ymin - cumap->clipr.ymin;
 		cumap->curr.ymin-= d1;
 		
+		d1= d;
 		if(cumap->flag & CUMA_DO_CLIP) 
 			if(cumap->curr.ymax+d > cumap->clipr.ymax)
 				d1= -cumap->curr.ymax + cumap->clipr.ymax;
@@ -552,6 +571,9 @@ void do_butspace(unsigned short event)
 	else if(event<=B_RENDERBUTS) {
 		do_render_panels(event);	// buttons_scene.c
 	}
+	else if(event<=B_SEQUENCERBUTS) {
+		do_sequencer_panels(event);
+	}
 	else if(event<=B_COMMONEDITBUTS) {
 		do_common_editbuts(event);
 	}
@@ -679,18 +701,23 @@ void redraw_test_buttons(Object *new)
 /* callback */
 void drawbutspace(ScrArea *sa, void *spacedata)
 {
+	ID *id, *idfrom;
 	SpaceButs *sbuts= sa->spacedata.first;
 	View2D *v2d= &sbuts->v2d;
 	float col[3];
 	int tab, align=0;
 	
+	/* context */
+	buttons_active_id(&id, &idfrom);
+	G.buts->lockpoin= id;
+	
 	myortho2(v2d->cur.xmin, v2d->cur.xmax, v2d->cur.ymin, v2d->cur.ymax);
 
 	BIF_GetThemeColor3fv(TH_BACK, col);
 	glClearColor(col[0], col[1], col[2], 0.0); 
 	glClear(GL_COLOR_BUFFER_BIT);
 
-	uiSetButLock(G.scene->id.lib!=0, "Can't edit library data");	
+	uiSetButLock(G.scene->id.lib!=0, ERROR_LIBDATA_MESSAGE);	
 	uiFreeBlocksWin(&sa->uiblocks, sa->win);
  
 	/* select the context to be drawn, per contex/tab the actual context is tested */
@@ -700,6 +727,8 @@ void drawbutspace(ScrArea *sa, void *spacedata)
 
 		if(tab== TAB_SCENE_RENDER) 
 			render_panels();
+		else if(tab == TAB_SCENE_SEQUENCER)
+			sequencer_panels();
 		else if(tab == TAB_SCENE_ANIM) 
 			anim_panels();
 		else if(tab == TAB_SCENE_SOUND) 
@@ -713,6 +742,8 @@ void drawbutspace(ScrArea *sa, void *spacedata)
 		   object_panels();
 		else if(tab==TAB_OBJECT_PHYSICS)
 			physics_panels();
+		else if(tab==TAB_OBJECT_PARTICLE)
+			particle_panels();
 		   
 		break;
 	case CONTEXT_SHADING:
