@@ -1,15 +1,12 @@
 /**
- * $Id: drawmesh.c 10949 2007-06-16 21:03:11Z bdiego $
+ * $Id: drawmesh.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <string.h>
@@ -38,6 +35,7 @@
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
 #include "BLI_edgehash.h"
+#include "BLI_editVert.h"
 
 #include "IMB_imbuf.h"
 #include "IMB_imbuf_types.h"
@@ -51,6 +49,7 @@
 #include "DNA_property_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_view3d_types.h"
+#include "DNA_userdef_types.h"
 
 #include "BKE_bmfont.h"
 #include "BKE_displist.h"
@@ -65,7 +64,7 @@
 #include "BKE_property.h"
 #include "BKE_utildefines.h"
 
-#include "BIF_resources.h"
+#include "BIF_editmesh.h"
 #include "BIF_gl.h"
 #include "BIF_glutil.h"
 #include "BIF_mywindow.h"
@@ -110,6 +109,18 @@ static int smaller_pow2(int num) {
 	return num;	
 }
 
+/* These are used to enable texture clamping */
+static int is_pow2_limit(int num) {
+	if (U.glreslimit != 0 && num > U.glreslimit) return 0;
+	return ((num)&(num-1))==0;
+}
+
+static int smaller_pow2_limit(int num) {
+	if (U.glreslimit != 0 && num > U.glreslimit)
+		return U.glreslimit;
+	return smaller_pow2(num);
+}
+
 static int fCurtile=0, fCurmode=0,fCurtileXRep=0,fCurtileYRep=0;
 static Image *fCurpage=0;
 static short fTexwindx, fTexwindy, fTexwinsx, fTexwinsy;
@@ -208,7 +219,7 @@ int set_tpage(MTFace *tface)
 		glDisable(GL_TEXTURE_2D);
 		glDisable(GL_TEXTURE_GEN_S);
 		glDisable(GL_TEXTURE_GEN_T);
-
+		glDisable ( GL_ALPHA_TEST );
 		return 0;
 	}
 	lasttface= tface;
@@ -225,14 +236,26 @@ int set_tpage(MTFace *tface)
 			}
 			else if(alphamode==TF_ALPHA) {
 				glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+				
+				/* added after 2.45 to clip alpha */
+				
+				/*if U.glalphaclip == 1.0, some cards go bonkers... turn off alpha test in this case*/
+				if(U.glalphaclip == 1.0) glDisable(GL_ALPHA_TEST);
+				else{
+					glEnable ( GL_ALPHA_TEST );
+					glAlphaFunc ( GL_GREATER, U.glalphaclip );
+				}
+				
 			/* 	glBlendEquationEXT(GL_FUNC_ADD_EXT); */
 			}
 			/* else { */
 			/* 	glBlendFunc(GL_ONE, GL_ONE); */
 			/* 	glBlendEquationEXT(GL_FUNC_REVERSE_SUBTRACT_EXT); */
 			/* } */
+		} else {
+			glDisable(GL_BLEND);
+			glDisable ( GL_ALPHA_TEST );
 		}
-		else glDisable(GL_BLEND);
 	}
 
 	ima= tface->tpage;
@@ -307,7 +330,6 @@ int set_tpage(MTFace *tface)
 	else fCurtile= tface->tile;
 
 	if(tilemode) {
-
 		if(ima->repbind==0) make_repbind(ima);
 		
 		if(fCurtile>=ima->totbind) fCurtile= 0;
@@ -371,9 +393,9 @@ int set_tpage(MTFace *tface)
 			rect= tilerect;
 		}
 #endif
-		if (!is_pow2(rectw) || !is_pow2(recth)) {
-			rectw= smaller_pow2(rectw);
-			recth= smaller_pow2(recth);
+		if (!is_pow2_limit(rectw) || !is_pow2_limit(recth)) {
+			rectw= smaller_pow2_limit(rectw);
+			recth= smaller_pow2_limit(recth);
 			
 			scalerect= MEM_mallocN(rectw*recth*sizeof(*scalerect), "scalerect");
 			gluScaleImage(GL_RGBA, tpx, tpy, GL_UNSIGNED_BYTE, rect, rectw, recth, GL_UNSIGNED_BYTE, scalerect);
@@ -584,10 +606,6 @@ void update_realtime_textures()
 enum {
 	eEdge_Visible = (1<<0),
 	eEdge_Select = (1<<1),
-	eEdge_Active = (1<<2),
-	eEdge_SelectAndActive = (1<<3),
-	eEdge_ActiveFirst = (1<<4),
-	eEdge_ActiveLast = (1<<5)
 };
 
 	/* Creates a hash of edges to flags indicating
@@ -609,19 +627,18 @@ EdgeHash *get_tface_mesh_marked_edge_info(Mesh *me)
 {
 	EdgeHash *eh = BLI_edgehash_new();
 	int i;
-
+	MFace *mf;
+	MTFace *tf = NULL;
+	
 	for (i=0; i<me->totface; i++) {
-		MFace *mf = &me->mface[i];
-		MTFace *tf = &me->mtface[i];
+		mf = &me->mface[i];
+		if (me->mtface)
+			tf = &me->mtface[i];
 		
 		if (mf->v3) {
 			if (!(mf->flag&ME_HIDE)) {
 				unsigned int flags = eEdge_Visible;
 				if (mf->flag&ME_FACE_SEL) flags |= eEdge_Select;
-				if (tf->flag&TF_ACTIVE) {
-					flags |= eEdge_Active;
-					if (mf->flag&ME_FACE_SEL) flags |= eEdge_SelectAndActive;
-				}
 
 				get_marked_edge_info__orFlags(eh, mf->v1, mf->v2, flags);
 				get_marked_edge_info__orFlags(eh, mf->v2, mf->v3, flags);
@@ -631,11 +648,6 @@ EdgeHash *get_tface_mesh_marked_edge_info(Mesh *me)
 				} else {
 					get_marked_edge_info__orFlags(eh, mf->v3, mf->v1, flags);
 				}
-
-				if (tf->flag&TF_ACTIVE) {
-					get_marked_edge_info__orFlags(eh, mf->v1, mf->v2, eEdge_ActiveFirst);
-					get_marked_edge_info__orFlags(eh, mf->v1, mf->v4?mf->v4:mf->v3, eEdge_ActiveLast);
-				}
 			}
 		}
 	}
@@ -692,17 +704,7 @@ static int draw_tfaces3D__setActiveOpts(void *userData, int index)
 	MEdge *med = &data->me->medge[index];
 	unsigned long flags = (long) BLI_edgehash_lookup(data->eh, med->v1, med->v2);
 
-	if (flags & eEdge_Active) {
-		if (flags & eEdge_ActiveLast) {
-			glColor3ub(255, 0, 0);
-		} else if (flags & eEdge_ActiveFirst) {
-			glColor3ub(0, 255, 0);
-		} else if (flags & eEdge_SelectAndActive) {
-			glColor3ub(255, 255, 0);
-		} else {
-			glColor3ub(255, 0, 255);
-		}
-
+	if (flags & eEdge_Select) {
 		return 1;
 	} else {
 		return 0;
@@ -712,13 +714,10 @@ static int draw_tfaces3D__drawFaceOpts(void *userData, int index)
 {
 	Mesh *me = (Mesh*)userData;
 
-	if (me->mtface) {
-		MFace *mface = &me->mface[index];
-		if (!(mface->flag&ME_HIDE) && (mface->flag&ME_FACE_SEL))
-			return 2; /* Don't set color */
-		else
-			return 0;
-	} else
+	MFace *mface = &me->mface[index];
+	if (!(mface->flag&ME_HIDE) && (mface->flag&ME_FACE_SEL))
+		return 2; /* Don't set color */
+	else
 		return 0;
 }
 static void draw_tfaces3D(Object *ob, Mesh *me, DerivedMesh *dm)
@@ -746,7 +745,7 @@ static void draw_tfaces3D(Object *ob, Mesh *me, DerivedMesh *dm)
 		glLineWidth(1);
 	}
 
-		/* Draw Selected Faces */
+	/* Draw Selected Faces */
 	if(G.f & G_DRAWFACES) {
 		glEnable(GL_BLEND);
 		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -929,25 +928,80 @@ static int set_draw_settings_cached(int clearcache, int textured, MTFace *texfac
 
 /* Icky globals, fix with userdata parameter */
 
-static Object *g_draw_tface_mesh_ob = NULL;
-static int g_draw_tface_mesh_islight = 0;
-static int g_draw_tface_mesh_istex = 0;
-static unsigned char g_draw_tface_mesh_obcol[4];
+struct TextureDrawState {
+	Object *ob;
+	int islit, istex;
+	unsigned char obcol[4];
+} Gtexdraw = {NULL, 0, 0, {0, 0, 0, 0}};
+
+static void draw_textured_begin(Object *ob)
+{
+	unsigned char obcol[4];
+	int istex, solidtex= 0;
+
+	if(G.vd->drawtype==OB_SOLID || (ob==G.obedit && G.vd->drawtype!=OB_TEXTURE)) {
+		/* draw with default lights in solid draw mode and edit mode */
+		solidtex= 1;
+		Gtexdraw.islit= -1;
+	}
+	else
+		/* draw with lights in the scene otherwise */
+		Gtexdraw.islit= set_gl_light(ob);
+	
+	obcol[0]= CLAMPIS(ob->col[0]*255, 0, 255);
+	obcol[1]= CLAMPIS(ob->col[1]*255, 0, 255);
+	obcol[2]= CLAMPIS(ob->col[2]*255, 0, 255);
+	obcol[3]= CLAMPIS(ob->col[3]*255, 0, 255);
+	
+	glCullFace(GL_BACK); glEnable(GL_CULL_FACE);
+	if(solidtex || G.vd->drawtype==OB_TEXTURE) istex= 1;
+	else istex= 0;
+
+	Gtexdraw.ob = ob;
+	Gtexdraw.istex = istex;
+	memcpy(Gtexdraw.obcol, obcol, sizeof(obcol));
+	set_draw_settings_cached(1, 0, 0, Gtexdraw.islit, 0, 0, 0);
+	glShadeModel(GL_SMOOTH);
+}
+
+static void draw_textured_end()
+{
+	/* switch off textures */
+	set_tpage(0);
+
+	glShadeModel(GL_FLAT);
+	glDisable(GL_CULL_FACE);
+
+	/* XXX, bad patch - default_gl_light() calls
+	 * glLightfv(GL_LIGHT_POSITION, ...) which
+	 * is transformed by the current matrix... we
+	 * need to make sure that matrix is identity.
+	 * 
+	 * It would be better if drawmesh.c kept track
+	 * of and restored the light settings it changed.
+	 *  - zr
+	 */
+	glPushMatrix();
+	glLoadIdentity();	
+	default_gl_light();
+	glPopMatrix();
+}
+
 
 static int draw_tface__set_draw(MTFace *tface, MCol *mcol, int matnr)
 {
 	if (tface && (tface->mode&TF_INVISIBLE)) return 0;
 
-	if (tface && set_draw_settings_cached(0, g_draw_tface_mesh_istex, tface, g_draw_tface_mesh_islight, g_draw_tface_mesh_ob, matnr, TF_TWOSIDE)) {
+	if (tface && set_draw_settings_cached(0, Gtexdraw.istex, tface, Gtexdraw.islit, Gtexdraw.ob, matnr, TF_TWOSIDE)) {
 		glColor3ub(0xFF, 0x00, 0xFF);
 		return 2; /* Don't set color */
 	} else if (tface && tface->mode&TF_OBCOL) {
-		glColor3ubv(g_draw_tface_mesh_obcol);
+		glColor3ubv(Gtexdraw.obcol);
 		return 2; /* Don't set color */
 	} else if (!mcol) {
 		if (tface) glColor3f(1.0, 1.0, 1.0);
 		else {
-			Material *ma= give_current_material(g_draw_tface_mesh_ob, matnr+1);
+			Material *ma= give_current_material(Gtexdraw.ob, matnr+1);
 			if(ma) glColor3f(ma->r, ma->g, ma->b);
 			else glColor3f(1.0, 1.0, 1.0);
 		}
@@ -968,215 +1022,192 @@ static int draw_tface_mapped__set_draw(void *userData, int index)
 	return draw_tface__set_draw(tface, mcol, matnr);
 }
 
+static int draw_em_tf_mapped__set_draw(void *userData, int index)
+{
+	EditMesh *em = userData;
+	EditFace *efa = EM_get_face_for_index(index);
+	MTFace *tface;
+	MCol *mcol;
+	int matnr;
+
+	if (efa==NULL || efa->h)
+		return 0;
+
+	tface = CustomData_em_get(&em->fdata, efa->data, CD_MTFACE);
+	mcol = CustomData_em_get(&em->fdata, efa->data, CD_MCOL);
+	matnr = efa->mat_nr;
+
+	return draw_tface__set_draw(tface, mcol, matnr);
+}
+
 static int wpaint__setSolidDrawOptions(void *userData, int index, int *drawSmooth_r)
 {
 	Mesh *me = (Mesh*)userData;
 	MTFace *tface = (me->mtface)? &me->mtface[index]: NULL;
 	MFace *mface = (me->mface)? &me->mface[index]: NULL;
 	
-	if (tface) {
-		if ((mface->flag&ME_HIDE) || (tface->mode&TF_INVISIBLE)) 
+	if ((mface->flag&ME_HIDE) || (tface && (tface->mode&TF_INVISIBLE))) 
 			return 0;
-	}
+	
 	*drawSmooth_r = 1;
 	return 1;
 }
 
-void draw_tface_mesh(Object *ob, Mesh *me, int dt)
-/* maximum dt (drawtype): exactly according values that have been set */
+static void draw_game_text_mesh(Object *ob, Mesh *me)
 {
-	unsigned char obcol[4];
-	int a;
-	short istex, solidtex=0;
-	DerivedMesh *dm;
+	DerivedMesh *ddm = mesh_get_derived_deform(ob, CD_MASK_BAREMESH);
+	MFace *mface= me->mface;
+	MTFace *tface= me->mtface;
+	MCol *mcol= me->mcol;	/* why does mcol exist? */
+	bProperty *prop = get_property(ob, "Text");
+	int a, start= 0, totface= me->totface;
+
+	tface+= start;
+	mcol+= start*4;
+	for (a=start; a<totface; a++, tface++, mcol+=4) {
+		MFace *mf= &mface[a];
+		int mode= tface->mode;
+		int matnr= mf->mat_nr;
+		int mf_smooth= mf->flag & ME_SMOOTH;
+
+		if (!(mf->flag&ME_HIDE) && !(mode&TF_INVISIBLE) && (mode&TF_BMFONT)) {
+			int badtex= set_draw_settings_cached(0, Gtexdraw.istex, tface, Gtexdraw.islit, Gtexdraw.ob, matnr, TF_TWOSIDE);
+			float v1[3], v2[3], v3[3], v4[3];
+			char string[MAX_PROPSTRING];
+			int characters, index;
+			ImBuf *ibuf;
+			float curpos;
+
+			if (badtex)
+				continue;
+
+			ddm->getVertCo(ddm, mf->v1, v1);
+			ddm->getVertCo(ddm, mf->v2, v2);
+			ddm->getVertCo(ddm, mf->v3, v3);
+			if (mf->v4) ddm->getVertCo(ddm, mf->v4, v4);
+
+			// The BM_FONT handling code is duplicated in the gameengine
+			// Search for 'Frank van Beek' ;-)
+			// string = "Frank van Beek";
+
+			set_property_valstr(prop, string);
+			characters = strlen(string);
+			
+			ibuf= BKE_image_get_ibuf(tface->tpage, NULL);
+			if (ibuf == NULL) {
+				characters = 0;
+			}
+
+			if (!mf_smooth) {
+				float nor[3];
+
+				CalcNormFloat(v1, v2, v3, nor);
+
+				glNormal3fv(nor);
+			}
+
+			curpos= 0.0;
+			glBegin(mf->v4?GL_QUADS:GL_TRIANGLES);
+			for (index = 0; index < characters; index++) {
+				float centerx, centery, sizex, sizey, transx, transy, movex, movey, advance;
+				int character = string[index];
+				char *cp= NULL;
+
+				// lets calculate offset stuff
+				// space starts at offset 1
+				// character = character - ' ' + 1;
+				
+				matrixGlyph(ibuf, character, & centerx, &centery, &sizex, &sizey, &transx, &transy, &movex, &movey, &advance);
+				movex+= curpos;
+
+				if (tface->mode & TF_OBCOL)
+					glColor3ubv(Gtexdraw.obcol);
+				else if (me->mcol) cp= (char *)mcol;
+				else glColor3ub(255, 255, 255);
+
+				glTexCoord2f((tface->uv[0][0] - centerx) * sizex + transx, (tface->uv[0][1] - centery) * sizey + transy);
+				if (cp) glColor3ub(cp[3], cp[2], cp[1]);
+				glVertex3f(sizex * v1[0] + movex, sizey * v1[1] + movey, v1[2]);
+				
+				glTexCoord2f((tface->uv[1][0] - centerx) * sizex + transx, (tface->uv[1][1] - centery) * sizey + transy);
+				if (cp) glColor3ub(cp[7], cp[6], cp[5]);
+				glVertex3f(sizex * v2[0] + movex, sizey * v2[1] + movey, v2[2]);
 	
-	if(me==NULL) return;
+				glTexCoord2f((tface->uv[2][0] - centerx) * sizex + transx, (tface->uv[2][1] - centery) * sizey + transy);
+				if (cp) glColor3ub(cp[11], cp[10], cp[9]);
+				glVertex3f(sizex * v3[0] + movex, sizey * v3[1] + movey, v3[2]);
+	
+				if(mf->v4) {
+					glTexCoord2f((tface->uv[3][0] - centerx) * sizex + transx, (tface->uv[3][1] - centery) * sizey + transy);
+					if (cp) glColor3ub(cp[15], cp[14], cp[13]);
+					glVertex3f(sizex * v4[0] + movex, sizey * v4[1] + movey, v4[2]);
+				}
 
-	dm = mesh_get_derived_final(ob, get_viewedit_datamask());
-
-	glShadeModel(GL_SMOOTH);
-
-	/* option to draw solid texture with default lights */
-	if(dt>OB_WIRE && G.vd->drawtype==OB_SOLID) {
-		solidtex= 1;
-		g_draw_tface_mesh_islight= -1;
+				curpos+= advance;
+			}
+			glEnd();
+		}
 	}
-	else
-		g_draw_tface_mesh_islight= set_gl_light(ob);
-	
-	obcol[0]= CLAMPIS(ob->col[0]*255, 0, 255);
-	obcol[1]= CLAMPIS(ob->col[1]*255, 0, 255);
-	obcol[2]= CLAMPIS(ob->col[2]*255, 0, 255);
-	obcol[3]= CLAMPIS(ob->col[3]*255, 0, 255);
-	
-	/* first all texture polys */
+
+	ddm->release(ddm);
+}
+
+void draw_mesh_textured(Object *ob, DerivedMesh *dm, int faceselect)
+{
+	Mesh *me= ob->data;
+	int editing= 0;
 	
+	/* correct for negative scale */
 	if(ob->transflag & OB_NEG_SCALE) glFrontFace(GL_CW);
 	else glFrontFace(GL_CCW);
 	
-	glCullFace(GL_BACK); glEnable(GL_CULL_FACE);
-	if(solidtex || G.vd->drawtype==OB_TEXTURE) istex= 1;
-	else istex= 0;
-
-	g_draw_tface_mesh_ob = ob;
-	g_draw_tface_mesh_istex = istex;
-	memcpy(g_draw_tface_mesh_obcol, obcol, sizeof(obcol));
-	set_draw_settings_cached(1, 0, 0, g_draw_tface_mesh_islight, 0, 0, 0);
-
-	if(dt > OB_SOLID || g_draw_tface_mesh_islight==-1) {
-		bProperty *prop = get_property(ob, "Text");
-		int editing= (G.f & (G_VERTEXPAINT+G_FACESELECT+G_TEXTUREPAINT+G_WEIGHTPAINT)) && (ob==((G.scene->basact) ? (G.scene->basact->object) : 0));
+	/* draw the textured mesh */
+	draw_textured_begin(ob);
 
 #ifdef WITH_VERSE
-		if(me->vnode) {
-			/* verse-blender doesn't support uv mapping of textures yet */
-			dm->drawFacesTex(dm, NULL);
-		}
-		else if(ob==OBACT && (G.f & G_FACESELECT) && me && me->mtface) {
-#else
-		if(ob==OBACT && (G.f & G_FACESELECT) && me && me->mtface) {
-#endif		
+	if(me->vnode) {
+		/* verse-blender doesn't support uv mapping of textures yet */
+		dm->drawFacesTex(dm, NULL);
+	}
+	else {
+#endif
+		if(ob==G.obedit) {
+			dm->drawMappedFacesTex(dm, draw_em_tf_mapped__set_draw, G.editMesh);
+		} else if(faceselect) {
 			if(G.f & G_WEIGHTPAINT)
-				dm->drawMappedFaces(dm, wpaint__setSolidDrawOptions, (void*)me, 1);
+				dm->drawMappedFaces(dm, wpaint__setSolidDrawOptions, me, 1);
 			else
-				dm->drawMappedFacesTex(dm, draw_tface_mapped__set_draw, (void*)me);
+				dm->drawMappedFacesTex(dm, draw_tface_mapped__set_draw, me);
 		}
 		else
 			dm->drawFacesTex(dm, draw_tface__set_draw);
-
-		/* drawing game engine text hack */
-		if (!editing && prop && me->mtface) {
-			DerivedMesh *ddm = mesh_get_derived_deform(ob, CD_MASK_BAREMESH);
-			MFace *mface= me->mface;
-			MTFace *tface= me->mtface;
-			MCol *mcol= me->mcol;	/* why does mcol exist? */
-			int start= 0, totface= me->totface;
-
-			tface+= start;
-			mcol+= start*4;
-			for (a=start; a<totface; a++, tface++, mcol+=4) {
-				MFace *mf= &mface[a];
-				int mode= tface->mode;
-				int matnr= mf->mat_nr;
-				int mf_smooth= mf->flag & ME_SMOOTH;
-
-				if (!(mf->flag&ME_HIDE) && !(mode&TF_INVISIBLE) && (mode&TF_BMFONT)) {
-					int badtex= set_draw_settings_cached(0, g_draw_tface_mesh_istex, tface, g_draw_tface_mesh_islight, g_draw_tface_mesh_ob, matnr, TF_TWOSIDE);
-					float v1[3], v2[3], v3[3], v4[3];
-					char string[MAX_PROPSTRING];
-					int characters, index;
-					ImBuf *ibuf;
-					float curpos;
-
-					if (badtex)
-						continue;
-
-					ddm->getVertCo(ddm, mf->v1, v1);
-					ddm->getVertCo(ddm, mf->v2, v2);
-					ddm->getVertCo(ddm, mf->v3, v3);
-					if (mf->v4) ddm->getVertCo(ddm, mf->v4, v4);
-
-					// The BM_FONT handling code is duplicated in the gameengine
-					// Search for 'Frank van Beek' ;-)
-					// string = "Frank van Beek";
-
-					set_property_valstr(prop, string);
-					characters = strlen(string);
-					
-					ibuf= BKE_image_get_ibuf(tface->tpage, NULL);
-					if (ibuf == NULL) {
-						characters = 0;
-					}
-
-					if (!mf_smooth) {
-						float nor[3];
-
-						CalcNormFloat(v1, v2, v3, nor);
-
-						glNormal3fv(nor);
-					}
-
-					curpos= 0.0;
-					glBegin(mf->v4?GL_QUADS:GL_TRIANGLES);
-					for (index = 0; index < characters; index++) {
-						float centerx, centery, sizex, sizey, transx, transy, movex, movey, advance;
-						int character = string[index];
-						char *cp= NULL;
-
-						// lets calculate offset stuff
-						// space starts at offset 1
-						// character = character - ' ' + 1;
-						
-						matrixGlyph(ibuf, character, & centerx, &centery, &sizex, &sizey, &transx, &transy, &movex, &movey, &advance);
-						movex+= curpos;
-
-						if (tface->mode & TF_OBCOL) glColor3ubv(obcol);
-						else if (mcol) cp= (char *)mcol;
-						else glColor3ub(255, 255, 255);
-
-						glTexCoord2f((tface->uv[0][0] - centerx) * sizex + transx, (tface->uv[0][1] - centery) * sizey + transy);
-						if (cp) glColor3ub(cp[3], cp[2], cp[1]);
-						glVertex3f(sizex * v1[0] + movex, sizey * v1[1] + movey, v1[2]);
-						
-						glTexCoord2f((tface->uv[1][0] - centerx) * sizex + transx, (tface->uv[1][1] - centery) * sizey + transy);
-						if (cp) glColor3ub(cp[7], cp[6], cp[5]);
-						glVertex3f(sizex * v2[0] + movex, sizey * v2[1] + movey, v2[2]);
-			
-						glTexCoord2f((tface->uv[2][0] - centerx) * sizex + transx, (tface->uv[2][1] - centery) * sizey + transy);
-						if (cp) glColor3ub(cp[11], cp[10], cp[9]);
-						glVertex3f(sizex * v3[0] + movex, sizey * v3[1] + movey, v3[2]);
-			
-						if(mf->v4) {
-							glTexCoord2f((tface->uv[3][0] - centerx) * sizex + transx, (tface->uv[3][1] - centery) * sizey + transy);
-							if (cp) glColor3ub(cp[15], cp[14], cp[13]);
-							glVertex3f(sizex * v4[0] + movex, sizey * v4[1] + movey, v4[2]);
-						}
-
-						curpos+= advance;
-					}
-					glEnd();
-				}
-			}
-
-			ddm->release(ddm);
-		}
-
-		/* switch off textures */
-		set_tpage(0);
+#ifdef WITH_VERSE
 	}
-	glShadeModel(GL_FLAT);
-	glDisable(GL_CULL_FACE);
+#endif
+
+	/* draw game engine text hack - but not if we are editing the mesh */
+	if (me->mtface && get_property(ob, "Text")) {
+		if(ob==G.obedit)
+			editing= 1;
+		else if(ob==OBACT)
+			if(FACESEL_PAINT_TEST)
+				editing= 1;
+
+		if(!editing)
+			draw_game_text_mesh(ob, me);
+	}
+
+	draw_textured_end();
 	
-	if(ob==OBACT && (G.f & G_FACESELECT) && me && me->mtface) {
+	/* draw edges and selected faces over textured mesh */
+	if(!G.obedit && faceselect)
 		draw_tfaces3D(ob, me, dm);
-	}
-	
-		/* XXX, bad patch - default_gl_light() calls
-		 * glLightfv(GL_LIGHT_POSITION, ...) which
-		 * is transformed by the current matrix... we
-		 * need to make sure that matrix is identity.
-		 * 
-		 * It would be better if drawmesh.c kept track
-		 * of and restored the light settings it changed.
-		 *  - zr
-		 */
-	glPushMatrix();
-	glLoadIdentity();	
-	default_gl_light();
-	glPopMatrix();
-	
+
+	/* reset from negative scale correction */
 	glFrontFace(GL_CCW);
-
-	if(dt > OB_SOLID && !(ob==OBACT && (G.f & G_FACESELECT) && me && me->mtface)) {
-		if(ob->flag & SELECT) {
-			BIF_ThemeColor((ob==OBACT)?TH_ACTIVE:TH_SELECT);
-		} else {
-			BIF_ThemeColor(TH_WIRE);
-		}
-		dm->drawLooseEdges(dm);
-	}
-
-	dm->release(dm);
+	
+	/* in editmode, the blend mode needs to be set incase it was ADD */
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 }
 
 void init_realtime_GL(void)
@@ -1184,6 +1215,5 @@ void init_realtime_GL(void)
 	glMatrixMode(GL_TEXTURE);
 	glLoadIdentity();
 	glMatrixMode(GL_MODELVIEW);
-
 }
 
