@@ -1,15 +1,12 @@
 /**
- * $Id: drawseq.c 10455 2007-04-04 13:18:41Z campbellbarton $
+ * $Id: drawseq.c 14777 2008-05-10 12:43:53Z campbellbarton $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifdef HAVE_CONFIG_H
@@ -51,6 +48,7 @@
 #include "DNA_screen_types.h"
 #include "DNA_space_types.h"
 #include "DNA_view2d_types.h"
+#include "DNA_userdef_types.h"
 
 #include "BKE_global.h"
 #include "BKE_plugin_types.h"
@@ -75,6 +73,7 @@
 #include "BSE_seqeffects.h"
 #include "BSE_seqscopes.h"
 #include "BSE_seqaudio.h"
+#include "BSE_time.h"
 
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"
@@ -90,43 +89,15 @@
 #define SEQ_STRIP_OFSBOTTOM		0.2
 #define SEQ_STRIP_OFSTOP		0.8
 
+/* Note, Dont use WHILE_SEQ while drawing! - it messes up transform, - Campbell */
+
 int no_rightbox=0, no_leftbox= 0;
-static void draw_seq_handle(Sequence *seq, SpaceSeq *sseq, short direction);
+static void draw_seq_handle(Sequence *seq, SpaceSeq *sseq, float pixelx, short direction);
 static void draw_seq_extensions(Sequence *seq, SpaceSeq *sseq);
 static void draw_seq_text(Sequence *seq, float x1, float x2, float y1, float y2);
 static void draw_shadedstrip(Sequence *seq, char *col, float x1, float y1, float x2, float y2);
-static void draw_seq_strip(struct Sequence *seq, struct ScrArea *sa, struct SpaceSeq *sseq);
+static void draw_seq_strip(struct Sequence *seq, struct ScrArea *sa, struct SpaceSeq *sseq, int outline_tint, float pixelx);
 
-static char *give_seqname(Sequence *seq)
-{
-	if(seq->type==SEQ_META) return "Meta";
-	else if(seq->type==SEQ_IMAGE) return "Image";
-	else if(seq->type==SEQ_SCENE) return "Scene";
-	else if(seq->type==SEQ_MOVIE) return "Movie";
-	else if(seq->type==SEQ_RAM_SOUND) return "Audio (RAM)";
-	else if(seq->type==SEQ_HD_SOUND) return "Audio (HD)";
-	else if(seq->type<SEQ_EFFECT) return seq->strip->dir;
-	else if(seq->type==SEQ_CROSS) return "Cross";
-	else if(seq->type==SEQ_GAMCROSS) return "Gamma Cross";
-	else if(seq->type==SEQ_ADD) return "Add";
-	else if(seq->type==SEQ_SUB) return "Sub";
-	else if(seq->type==SEQ_MUL) return "Mul";
-	else if(seq->type==SEQ_ALPHAOVER) return "Alpha Over";
-	else if(seq->type==SEQ_ALPHAUNDER) return "Alpha Under";
-	else if(seq->type==SEQ_OVERDROP) return "Over Drop";
-	else if(seq->type==SEQ_WIPE) return "Wipe";
-	else if(seq->type==SEQ_GLOW) return "Glow";
-	else if(seq->type==SEQ_TRANSFORM) return "Transform";
-	else if(seq->type==SEQ_COLOR) return "Color";
-	else if(seq->type==SEQ_SPEED) return "Speed";
-	else if(seq->type==SEQ_PLUGIN) {
-		if(!(seq->flag & SEQ_EFFECT_NOT_LOADED) &&
-		   seq->plugin && seq->plugin->doit) return seq->plugin->pname;
-		return "Plugin";
-	}
-	else return "Effect";
-
-}
 static void draw_cfra_seq(void)
 {
 	glColor3ub(0x30, 0x90, 0x50);
@@ -235,20 +206,26 @@ static void get_seq_color3ubv(Sequence *seq, char *col)
 
 static void drawmeta_contents(Sequence *seqm, float x1, float y1, float x2, float y2)
 {
+	/* Note, this used to use WHILE_SEQ, but it messes up the seq->depth value, (needed by transform when doing overlap checks)
+	 * so for now, just use the meta's immediate children, could be fixed but its only drawing - Campbell */
 	Sequence *seq;
 	float dx;
 	int nr;
 	char col[3];
 	
-	nr= 0;
-	WHILE_SEQ(&seqm->seqbase) {
-		nr++;
-	}
-	END_SEQ
+	nr= BLI_countlist(&seqm->seqbase);
 
 	dx= (x2-x1)/nr;
 
-	WHILE_SEQ(&seqm->seqbase) {
+	if (seqm->flag & SEQ_MUTE) {
+		glEnable(GL_POLYGON_STIPPLE);
+		glPolygonStipple(stipple_halftone);
+		
+		glEnable(GL_LINE_STIPPLE);
+		glLineStipple(1, 0x8888);
+	}
+	
+	for (seq= seqm->seqbase.first; seq; seq= seq->next) {
 		get_seq_color3ubv(seq, col);
 		
 		glColor3ubv((GLubyte *)col);
@@ -262,7 +239,11 @@ static void drawmeta_contents(Sequence *seqm, float x1, float y1, float x2, floa
 		
 		x1+= dx;
 	}
-	END_SEQ
+	
+	if (seqm->flag & SEQ_MUTE) {
+		glDisable(GL_POLYGON_STIPPLE);
+		glDisable(GL_LINE_STIPPLE);
+	}
 }
 
 static void drawseqwave(Sequence *seq, float x1, float y1, float x2, float y2, int winx)
@@ -306,8 +287,8 @@ static void drawseqwave(Sequence *seq, float x1, float y1, float x2, float y2, i
 	
 	if (seq->flag & SEQ_MUTE) glColor3ub(0x70, 0x80, 0x80); else glColor3ub(0x70, 0xc0, 0xc0);
 	
-	sofs = ((int)( (((float)(seq->startdisp-seq->start))/(float)G.scene->r.frs_sec)*(float)G.scene->audio.mixrate*4.0 )) & (~3);
-	eofs = ((int)( (((float)(seq->enddisp-seq->start))/(float)G.scene->r.frs_sec)*(float)G.scene->audio.mixrate*4.0 )) & (~3);
+	sofs = ((int)( FRA2TIME(seq->startdisp-seq->start+seq->anim_startofs)*(float)G.scene->audio.mixrate*4.0 )) & (~3);
+	eofs = ((int)( FRA2TIME(seq->enddisp-seq->start+seq->anim_startofs)*(float)G.scene->audio.mixrate*4.0 )) & (~3);
 	
 	/* clip the drawing area to the screen bounds to save time */
 	sample_step= (G.v2d->cur.xmax - G.v2d->cur.xmin)/winx;
@@ -368,14 +349,13 @@ static void drawseqwave(Sequence *seq, float x1, float y1, float x2, float y2, i
 }
 
 /* draw a handle, for each end of a sequence strip */
-static void draw_seq_handle(Sequence *seq, SpaceSeq *sseq, short direction)
+static void draw_seq_handle(Sequence *seq, SpaceSeq *sseq, float pixelx, short direction)
 {
 	float v1[2], v2[2], v3[2], rx1=0, rx2=0; //for triangles and rect
 	float x1, x2, y1, y2;
-	float pixelx;
 	float handsize;
 	float minhandle, maxhandle;
-	char str[120];
+	char str[32];
 	unsigned int whichsel=0;
 	View2D *v2d;
 	
@@ -386,12 +366,11 @@ static void draw_seq_handle(Sequence *seq, SpaceSeq *sseq, short direction)
 	y2= seq->machine+SEQ_STRIP_OFSTOP;
 	
 	v2d = &sseq->v2d;
-	pixelx = (v2d->cur.xmax - v2d->cur.xmin)/(v2d->mask.xmax - v2d->mask.xmin);
 	
 	/* clamp handles to defined size in pixel space */
 	handsize = seq->handsize;
 	minhandle = 7;
-	maxhandle = 28;
+	maxhandle = 40;
 	CLAMP(handsize, minhandle*pixelx, maxhandle*pixelx);
 	
 	/* set up co-ordinates/dimensions for either left or right handle */
@@ -447,9 +426,9 @@ static void draw_seq_handle(Sequence *seq, SpaceSeq *sseq, short direction)
 			glRasterPos3f(rx1,  y1-0.15, 0.0);
 		} else {
 			sprintf(str, "%d", seq->enddisp - 1);
-			glRasterPos3f((x2-BMF_GetStringWidth(G.font, str)*pixelx),  y2+0.05, 0.0);
+			glRasterPos3f((x2-BMF_GetStringWidth(G.fonts, str)*pixelx),  y2+0.05, 0.0);
 		}
-		BMF_DrawString(G.font, str);
+		BMF_DrawString(G.fonts, str);
 	}	
 }
 
@@ -634,6 +613,11 @@ static void draw_shadedstrip(Sequence *seq, char *col, float x1, float y1, float
 {
 	float ymid1, ymid2;
 	
+	if (seq->flag & SEQ_MUTE) {
+		glEnable(GL_POLYGON_STIPPLE);
+		glPolygonStipple(stipple_halftone);
+	}
+	
 	ymid1 = (y2-y1)*0.25 + y1;
 	ymid2 = (y2-y1)*0.65 + y1;
 	
@@ -675,6 +659,9 @@ static void draw_shadedstrip(Sequence *seq, char *col, float x1, float y1, float
 	
 	glEnd();
 	
+	if (seq->flag & SEQ_MUTE) {
+		glDisable(GL_POLYGON_STIPPLE);
+	}
 }
 
 /*
@@ -682,12 +669,14 @@ Draw a sequence strip, bounds check alredy made
 ScrArea is currently only used to get the windows width in pixels
 so wave file sample drawing precission is zoom adjusted
 */
-static void draw_seq_strip(Sequence *seq, ScrArea *sa, SpaceSeq *sseq)
+static void draw_seq_strip(Sequence *seq, ScrArea *sa, SpaceSeq *sseq, int outline_tint, float pixelx)
 {
 	float x1, x2, y1, y2;
-	char col[3];
-	Sequence *last_seq = get_last_seq();
+	char col[3], is_single_image;
 
+	/* we need to know if this is a single image/color or not for drawing */
+	is_single_image = (char)check_single_seq(seq);
+	
 	/* body */
 	if(seq->startstill) x1= seq->start;
 	else x1= seq->startdisp;
@@ -701,13 +690,20 @@ static void draw_seq_strip(Sequence *seq, ScrArea *sa, SpaceSeq *sseq)
 	get_seq_color3ubv(seq, col);
 	
 	/* draw the main strip body */
-	draw_shadedstrip(seq, col, x1, y1, x2, y2);
+	if (is_single_image) /* single image */
+		draw_shadedstrip(seq, col, seq_tx_get_final_left(seq), y1, seq_tx_get_final_right(seq), y2);
+	else /* normal operation */
+		draw_shadedstrip(seq, col, x1, y1, x2, y2);
 	
 	/* draw additional info and controls */
-	if (seq->type == SEQ_RAM_SOUND) drawseqwave(seq, x1, y1, x2, y2, sa->winx);
-	draw_seq_extensions(seq, sseq);
-	draw_seq_handle(seq, sseq, SEQ_LEFTHANDLE);
-	draw_seq_handle(seq, sseq, SEQ_RIGHTHANDLE);
+	if (seq->type == SEQ_RAM_SOUND)
+		drawseqwave(seq, x1, y1, x2, y2, sa->winx);
+	
+	if (!is_single_image)
+		draw_seq_extensions(seq, sseq);
+	
+	draw_seq_handle(seq, sseq, pixelx, SEQ_LEFTHANDLE);
+	draw_seq_handle(seq, sseq, pixelx, SEQ_RIGHTHANDLE);
 	
 	/* draw the strip outline */
 	x1= seq->startdisp;
@@ -719,13 +715,21 @@ static void draw_seq_strip(Sequence *seq, ScrArea *sa, SpaceSeq *sseq)
 			col[0]= 255; col[1]= col[2]= 40;
 		} else BIF_GetColorPtrBlendShade3ubv(col, col, col, 0.0, 120);
 	}
-	else if (seq == last_seq) BIF_GetColorPtrBlendShade3ubv(col, col, col, 0.0, 120);
-	else if (seq->flag & SELECT) BIF_GetColorPtrBlendShade3ubv(col, col, col, 0.0, -150);
-	else BIF_GetColorPtrBlendShade3ubv(col, col, col, 0.0, -60);
 
+	BIF_GetColorPtrBlendShade3ubv(col, col, col, 0.0, outline_tint);
+	
 	glColor3ubv((GLubyte *)col);
+	
+	if (seq->flag & SEQ_MUTE) {
+		glEnable(GL_LINE_STIPPLE);
+		glLineStipple(1, 0x8888);
+	}
+	
 	gl_round_box_shade(GL_LINE_LOOP, x1, y1, x2, y2, 0.0, 0.1, 0.0);
-
+	
+	if (seq->flag & SEQ_MUTE) {
+		glDisable(GL_LINE_STIPPLE);
+	}
 	
 	/* calculate if seq is long enough to print a name */
 	x1= seq->startdisp+seq->handsize;
@@ -740,10 +744,10 @@ static void draw_seq_strip(Sequence *seq, ScrArea *sa, SpaceSeq *sseq)
 	if(x2<G.v2d->cur.xmin) x2= G.v2d->cur.xmin;
 	else if(x2>G.v2d->cur.xmax) x2= G.v2d->cur.xmax;
 
-	if(x1 != x2) {
+	/* nice text here would require changing the view matrix for texture text */
+	if( (x2-x1) / pixelx > 32) {
 		draw_seq_text(seq, x1, x2, y1, y2);
 	}
-		
 }
 
 static Sequence *special_seq_update= 0;
@@ -763,12 +767,12 @@ void set_special_seq_update(int val)
 static void draw_image_seq(ScrArea *sa)
 {
 	SpaceSeq *sseq;
-	StripElem *se;
 	struct ImBuf *ibuf;
 	int x1, y1, rectx, recty;
 	int free_ibuf = 0;
 	static int recursive= 0;
 	float zoom;
+	float zoomx, zoomy;
 
 	glClearColor(0.0, 0.0, 0.0, 0.0);
 	glClear(GL_COLOR_BUFFER_BIT);
@@ -788,7 +792,19 @@ static void draw_image_seq(ScrArea *sa)
 		return;
 	else {
 		recursive= 1;
-		ibuf= (ImBuf *)give_ibuf_seq(rectx, recty, (G.scene->r.cfra), sseq->chanshown);
+		if (special_seq_update) {
+			ibuf= give_ibuf_seq_direct(
+				rectx, recty, (G.scene->r.cfra),
+				special_seq_update);
+		} else if (!U.prefetchframes || (G.f & G_PLAYANIM) == 0) {
+			ibuf= (ImBuf *)give_ibuf_seq(
+				rectx, recty, (G.scene->r.cfra), 
+				sseq->chanshown);
+		} else {
+			ibuf= (ImBuf *)give_ibuf_seq_threaded(
+				rectx, recty, (G.scene->r.cfra), 
+				sseq->chanshown);
+		}
 		recursive= 0;
 		
 		/* HURMF! the give_ibuf_seq can call image display in this window */
@@ -799,31 +815,40 @@ static void draw_image_seq(ScrArea *sa)
 		}
 	}
 	
-	if(special_seq_update) {
-		se = special_seq_update->curelem;
-		if(se) {
-			if(se->ok==2) {
-				if(se->se1)
-					ibuf= se->se1->ibuf;
-			}
-			else ibuf= se->ibuf;
-		}
-	}
 	if(ibuf==NULL) 
 		return;
-	if(ibuf->rect_float && ibuf->rect==NULL)
-		IMB_rect_from_float(ibuf);
-	if(ibuf->rect==NULL) 
+
+	if(ibuf->rect==NULL && ibuf->rect_float == NULL) 
 		return;
 
-	if (sseq->mainb == SEQ_DRAW_IMG_WAVEFORM) {
-		ibuf = make_waveform_view_from_ibuf(ibuf);
+	switch(sseq->mainb) {
+	case SEQ_DRAW_IMG_IMBUF:
+		if (sseq->zebra != 0) {
+			ibuf = make_zebra_view_from_ibuf(ibuf, sseq->zebra);
+			free_ibuf = 1;
+		}
+		break;
+	case SEQ_DRAW_IMG_WAVEFORM:
+		if ((sseq->flag & SEQ_DRAW_COLOR_SEPERATED) != 0) {
+			ibuf = make_sep_waveform_view_from_ibuf(ibuf);
+		} else {
+			ibuf = make_waveform_view_from_ibuf(ibuf);
+		}
 		free_ibuf = 1;
-	} else if (sseq->mainb == SEQ_DRAW_IMG_VECTORSCOPE) {
+		break;
+	case SEQ_DRAW_IMG_VECTORSCOPE:
 		ibuf = make_vectorscope_view_from_ibuf(ibuf);
 		free_ibuf = 1;
+		break;
+	case SEQ_DRAW_IMG_HISTOGRAM:
+		ibuf = make_histogram_view_from_ibuf(ibuf);
+		free_ibuf = 1;
+		break;
 	}
 
+	if(ibuf->rect_float && ibuf->rect==NULL)
+		IMB_rect_from_float(ibuf);
+
 	if (sseq->zoom > 0) {
 		zoom = sseq->zoom;
 	} else if (sseq->zoom == 0) {
@@ -832,136 +857,123 @@ static void draw_image_seq(ScrArea *sa)
 		zoom = -1.0/sseq->zoom;
 	}
 
-	/* calc location */
-	x1= (sa->winx-zoom*ibuf->x)/2 + sseq->xof;
-	y1= (sa->winy-zoom*ibuf->y)/2 + sseq->yof;
-
 	/* needed for gla draw */
 	glaDefine2DArea(&curarea->winrct);
-	glPixelZoom(zoom, zoom);
+	if (sseq->mainb == SEQ_DRAW_IMG_IMBUF) {
+		zoomx = zoom * ((float)G.scene->r.xasp / (float)G.scene->r.yasp);
+		zoomy = zoom;
+	} else {
+		zoomx = zoomy = zoom;
+	}
 
+	/* calc location */
+	x1= (sa->winx-zoomx*ibuf->x)/2 + sseq->xof;
+	y1= (sa->winy-zoomy*ibuf->y)/2 + sseq->yof;
+	
+	glPixelZoom(zoomx, zoomy);
+	
 	glaDrawPixelsSafe(x1, y1, ibuf->x, ibuf->y, ibuf->x, GL_RGBA, GL_UNSIGNED_BYTE, ibuf->rect);
 	
 	glPixelZoom(1.0, 1.0);
 
+	/* safety border */
+	if (sseq->mainb == SEQ_DRAW_IMG_IMBUF && 
+	    (sseq->flag & SEQ_DRAW_SAFE_MARGINS) != 0) {
+		float fac= 0.1;
+		float x2 = x1 + ibuf->x * zoomx;
+		float y2 = y1 + ibuf->y * zoomy;
+		
+		float a= fac*(x2-x1);
+		x1+= a; 
+		x2-= a;
+	
+		a= fac*(y2-y1);
+		y1+= a;
+		y2-= a;
+	
+		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); 
+		setlinestyle(3);
+
+		BIF_ThemeColorBlendShade(TH_WIRE, TH_BACK, 1.0, 0);
+		
+		uiSetRoundBox(15);
+		gl_round_box(GL_LINE_LOOP, x1, y1, x2, y2, 12.0);
+
+		setlinestyle(0);
+		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
+	}
+
+
 	if (free_ibuf) {
 		IMB_freeImBuf(ibuf);
-	}
+	} 
 
 	sa->win_swap= WIN_BACK_OK;
 }
 
-static void draw_extra_seqinfo(void)
-{
-	Sequence *last_seq = get_last_seq();
-	StripElem *se, *last;
-	float xco, xfac;
-	int sta, end;
-	char str[256];
-
-	if(last_seq==0) return;
-
-	/* xfac: size of 1 pixel */
-	xfac= G.v2d->cur.xmax - G.v2d->cur.xmin;
-	xfac/= (float)(G.v2d->mask.xmax-G.v2d->mask.xmin);
-	xco= G.v2d->cur.xmin+40*xfac;
-
-	BIF_ThemeColor(TH_TEXT);
-
-	/* NAME */
-	glRasterPos3f(xco,  0.3, 0.0);
-	strncpy(str, give_seqname(last_seq), 255);
-	BMF_DrawString(G.font, str);
-	xco += xfac*BMF_GetStringWidth(G.font, str) +30.0*xfac;
-
-	if(last_seq->type==SEQ_SCENE && last_seq->scene) {
-		glRasterPos3f(xco,  0.3, 0.0);
-		BMF_DrawString(G.font, last_seq->scene->id.name+2);
-		xco += xfac*BMF_GetStringWidth(G.font, last_seq->scene->id.name+2) +30.0*xfac;
-	}
-
-	/* LEN */
-	if(last_seq->type & SEQ_EFFECT)
-		sprintf(str, "len: %d   From %d - %d", last_seq->len, last_seq->startdisp, last_seq->enddisp-1);
-	else
-		sprintf(str, "len: %d (%d)", last_seq->enddisp-last_seq->startdisp, last_seq->len);
-
-	glRasterPos3f(xco,  0.3, 0.0);
-	BMF_DrawString(G.font, str);
-	xco += xfac*BMF_GetStringWidth(G.font, str) +30.0*xfac;
-
-	if(last_seq->type==SEQ_IMAGE) {
-
-		/* CURRENT */
-		se= (StripElem *)give_stripelem(last_seq,  (G.scene->r.cfra));
-		if(se) {
-			sprintf(str, "Cur: %s", se->name);
-			glRasterPos3f(xco,  0.3, 0.0);
-			BMF_DrawString(G.font, str);
-			xco += xfac*BMF_GetStringWidth(G.font, str) +30.0*xfac;
-		}
-
-		/* FIRST AND LAST */
-
-		if(last_seq->strip) {
-			se= last_seq->strip->stripdata;
-			last= se+last_seq->len-1;
-			if(last_seq->startofs) se+= last_seq->startofs;
-			if(last_seq->endofs) last-= last_seq->endofs;
-
-			sprintf(str, "First: %s at %d   Last: %s at %d", se->name, last_seq->startdisp, last->name, last_seq->enddisp-1);
-			glRasterPos3f(xco,  0.3, 0.0);
-			BMF_DrawString(G.font, str);
-			xco += xfac*BMF_GetStringWidth(G.font, str) +30.0*xfac;
-
-			/* orig size */
-			sprintf(str, "OrigSize: %d x %d", last_seq->strip->orx, last_seq->strip->ory);
-			glRasterPos3f(xco,  0.3, 0.0);
-			BMF_DrawString(G.font, str);
-			xco += xfac*BMF_GetStringWidth(G.font, str) +30.0*xfac;
-		}
-	}
-	else if(last_seq->type==SEQ_MOVIE) {
-
-		sta= last_seq->startofs;
-		end= last_seq->len-1-last_seq->endofs;
-
-		sprintf(str, "%s   %s%s  First: %d at %d   Last: %d at %d   Cur: %d",
-				last_seq->name+2, last_seq->strip->dir, last_seq->strip->stripdata->name,
-				sta, last_seq->startdisp, end, last_seq->enddisp-1,  (G.scene->r.cfra)-last_seq->startdisp);
-
-		glRasterPos3f(xco,  0.3, 0.0);
-		BMF_DrawString(G.font, str);
-	}
-	else if(last_seq->type==SEQ_SCENE) {
-		se= (StripElem *)give_stripelem(last_seq,  (G.scene->r.cfra));
-		if(se && last_seq->scene) {
-			sprintf(str, "Cur: %d  First: %d  Last: %d", last_seq->sfra+se->nr, last_seq->sfra, last_seq->sfra+last_seq->len-1); 
-			glRasterPos3f(xco,  0.3, 0.0);
-			BMF_DrawString(G.font, str);
-		}
-	}
-	else if(last_seq->type==SEQ_RAM_SOUND
-		|| last_seq->type == SEQ_HD_SOUND) {
-
-		sta= last_seq->startofs;
-		end= last_seq->len-1-last_seq->endofs;
-
-		sprintf(str, "%s   %s%s  First: %d at %d   Last: %d at %d   Cur: %d   Gain: %.2f dB   Pan: %.2f",
-				last_seq->name+2, last_seq->strip->dir, last_seq->strip->stripdata->name,
-				sta, last_seq->startdisp, end, last_seq->enddisp-1,  (G.scene->r.cfra)-last_seq->startdisp,
-				last_seq->level, last_seq->pan);
-
-		glRasterPos3f(xco,  0.3, 0.0);
-		BMF_DrawString(G.font, str);
-	}
-}
-
 void seq_reset_imageofs(SpaceSeq *sseq)
 {
 	sseq->xof = sseq->yof = sseq->zoom = 0;
 }
 
+void seq_home(void)
+{
+	SpaceSeq *sseq= curarea->spacedata.first;
+	
+	if (!sseq->mainb) {
+		G.v2d->cur= G.v2d->tot;
+		test_view2d(G.v2d, curarea->winx, curarea->winy);
+		view2d_do_locks(curarea, V2D_LOCK_COPY);
+	} else {
+		float zoomX, zoomY;
+		int width, height, imgwidth, imgheight;	
+		
+		width = curarea->winx;
+		height = curarea->winy;
+		
+		seq_reset_imageofs(sseq);
+		
+		imgwidth= (G.scene->r.size*G.scene->r.xsch)/100;
+		imgheight= (G.scene->r.size*G.scene->r.ysch)/100;
+		
+		/* Apply aspect, dosnt need to be that accurate */
+		imgwidth= (int)(imgwidth * ((float)G.scene->r.xasp / (float)G.scene->r.yasp));
+		
+		if (((imgwidth >= width) || (imgheight >= height)) && 
+			((width > 0) && (height > 0))) {
+			
+			/* Find the zoom value that will fit the image in the image space */
+			zoomX = ((float)width) / ((float)imgwidth);
+			zoomY = ((float)height) / ((float)imgheight);
+			sseq->zoom= (zoomX < zoomY) ? zoomX : zoomY;
+	
+			sseq->zoom = 1.0f / power_of_2(1/ MIN2(zoomX, zoomY) );
+		}
+		else {
+			sseq->zoom= 1.0f;
+		}
+	}
+	scrarea_queue_winredraw(curarea);
+}
+
+void seq_viewzoom(unsigned short event, int invert)
+{
+	SpaceSeq *sseq= curarea->spacedata.first;
+
+	if(event==PAD1)
+		sseq->zoom= 1.0;
+	else if(event==PAD2)
+		sseq->zoom= (invert)? 2.0: 0.5;
+	else if(event==PAD4)
+		sseq->zoom= (invert)? 4.0: 0.25;
+	else if(event==PAD8)
+		sseq->zoom= (invert)? 8.0: 0.125;
+	
+	/* ensure pixel exact locations for draw */
+	sseq->xof= (int)sseq->xof;
+	sseq->yof= (int)sseq->yof;
+}
+
 void seq_viewmove(SpaceSeq *sseq)
 {	
 	ScrArea *sa;
@@ -1011,232 +1023,7 @@ void seq_viewmove(SpaceSeq *sseq)
 	window_set_cursor(win, oldcursor);
 }
 
-#define SEQ_BUT_PLUGIN	1
-#define SEQ_BUT_RELOAD	2
-#define SEQ_BUT_EFFECT	3
-#define SEQ_BUT_RELOAD_ALL 4
 
-void do_seqbuttons(short val)
-{
-	Sequence *last_seq = get_last_seq();
-
-	switch(val) {
-	case SEQ_BUT_PLUGIN:
-	case SEQ_BUT_EFFECT:
-		update_changed_seq_and_deps(last_seq, 0, 1);
-		break;
-
-	case SEQ_BUT_RELOAD:
-	case SEQ_BUT_RELOAD_ALL:
-		update_seq_ipo_rect(last_seq);
-		update_seq_icu_rects(last_seq);
-
-		free_imbuf_seq();	// frees all
-
-		break;
-	}
-
-	if (val == SEQ_BUT_RELOAD_ALL) {
-		allqueue(REDRAWALL, 0);
-	} else {
-		allqueue(REDRAWSEQ, 0);
-	}
-}
-
-static void seq_panel_properties(short cntrl)	// SEQ_HANDLER_PROPERTIES
-{
-	Sequence *last_seq = get_last_seq();
-	uiBlock *block;
-
-	block= uiNewBlock(&curarea->uiblocks, "seq_panel_properties", UI_EMBOSS, UI_HELV, curarea->win);
-	uiPanelControl(UI_PNL_SOLID | UI_PNL_CLOSE | cntrl);
-	uiSetPanelHandler(SEQ_HANDLER_PROPERTIES);  // for close and esc
-	if(uiNewPanel(curarea, block, "Strip Properties", "Seq", 10, 230, 318, 204)==0) return;
-
-	if(last_seq==NULL) return;
-
-	if(last_seq->type==SEQ_PLUGIN) {
-		PluginSeq *pis;
-		VarStruct *varstr;
-		int a, xco, yco;
-
-		get_sequence_effect(last_seq);/* make sure, plugin is loaded */
-
-		uiDefBut(block, LABEL, 0, "Type: Plugin", 10,50,70,20, 0, 0, 0, 0, 0, "");
-
-		pis= last_seq->plugin;
-		if(pis->vars==0) return;
-
-		varstr= pis->varstr;
-		if(varstr) {
-			for(a=0; a<pis->vars; a++, varstr++) {
-				xco= 150*(a/6)+10;
-				yco= 125 - 20*(a % 6)+1;
-				uiDefBut(block, varstr->type, SEQ_BUT_PLUGIN, varstr->name, xco,yco,150,19, &(pis->data[a]), varstr->min, varstr->max, 100, 0, varstr->tip);
-
-			}
-		}
-		uiDefButBitS(block, TOG, SEQ_IPO_FRAME_LOCKED,
-			     SEQ_BUT_RELOAD_ALL, "IPO Frame locked",
-			     10,-40,150,19, &last_seq->flag,
-			     0.0, 1.0, 0, 0,
-			     "Lock the IPO coordinates to the "
-			     "global frame counter.");
-
-	}
-	else if(last_seq->type==SEQ_IMAGE) {
-
-		uiDefBut(block, LABEL, 0, "Type: Image", 10,140,150,20, 0, 0, 0, 0, 0, "");
-		uiDefBut(block, TEX, B_NOP, "Name: ", 10,120,150,19, last_seq->name+2, 0.0, 21.0, 100, 0, "");
-		
-		uiBlockBeginAlign(block);
-		uiDefButBitS(block, TOG, SEQ_MAKE_PREMUL, SEQ_BUT_RELOAD, "Convert to Premul", 10,90,150,19, &last_seq->flag, 0.0, 21.0, 100, 0, "Converts RGB values to become premultiplied with Alpha");
-		uiDefButBitS(block, TOG, SEQ_FILTERY, SEQ_BUT_RELOAD, "FilterY",	10,70,150,19, &last_seq->flag, 0.0, 21.0, 100, 0, "For video movies to remove fields");
-		uiDefButF(block, NUM, SEQ_BUT_RELOAD, "Mul:",			10,50,150,19, &last_seq->mul, 0.001, 5.0, 100, 0, "Multiply colors");
-		uiDefButS(block, TOG|BIT|7, SEQ_BUT_RELOAD, "Reverse Frames", 10,30,150,19, &last_seq->flag, 0.0, 21.0, 100, 0, "Reverse frame order");
-		uiDefButF(block, NUM, SEQ_BUT_RELOAD, "Strobe:",			10,10,150,19, &last_seq->strobe, 1.0, 30.0, 100, 0, "Only display every nth frame");
-		uiBlockEndAlign(block);
-	}
-	else if(last_seq->type==SEQ_META) {
-
-		uiDefBut(block, LABEL, 0, "Type: Meta", 10,140,150,20, 0, 0, 0, 0, 0, "");
-		uiDefBut(block, TEX, B_NOP, "Name: ", 10,120,150,19, last_seq->name+2, 0.0, 21.0, 100, 0, "");
-
-	}
-	else if(last_seq->type==SEQ_SCENE) {
-
-		uiDefBut(block, LABEL, 0, "Type: Scene", 10,140,150,20, 0, 0, 0, 0, 0, "");
-		uiDefBut(block, TEX, B_NOP, "Name: ", 10,120,150,19, last_seq->name+2, 0.0, 21.0, 100, 0, "");
-		uiDefButS(block, TOG|BIT|7, SEQ_BUT_RELOAD, "Reverse Frames", 10,90,150,19, &last_seq->flag, 0.0, 21.0, 100, 0, "Reverse frame order");
-	}
-	else if(last_seq->type==SEQ_MOVIE) {
-
-		if(last_seq->mul==0.0) last_seq->mul= 1.0;
-
-		uiDefBut(block, LABEL, 0, "Type: Movie", 10,140,150,20, 0, 0, 0, 0, 0, "");
-		uiDefBut(block, TEX, B_NOP, "Name: ", 10,120,150,19, last_seq->name+2, 0.0, 21.0, 100, 0, "");
-		
-		uiBlockBeginAlign(block);
-		uiDefButBitS(block, TOG, SEQ_MAKE_PREMUL, SEQ_BUT_RELOAD, "Make Premul Alpha ", 10,90,150,19, &last_seq->flag, 0.0, 21.0, 100, 0, "Converts RGB values to become premultiplied with Alpha");
-		uiDefButBitS(block, TOG, SEQ_FILTERY, SEQ_BUT_RELOAD, "FilterY ",	10,70,150,19, &last_seq->flag, 0.0, 21.0, 100, 0, "For video movies to remove fields");
-		uiDefButF(block, NUM, SEQ_BUT_RELOAD, "Mul:",			10,50,150,19, &last_seq->mul, 0.001, 5.0, 100, 0, "Multiply colors");
-		
-		uiDefButS(block, TOG|BIT|7, SEQ_BUT_RELOAD, "Reverse Frames", 10,30,150,19, &last_seq->flag, 0.0, 21.0, 100, 0, "Reverse frame order");
-		uiDefButF(block, NUM, SEQ_BUT_RELOAD, "Strobe:",			10,10,150,19, &last_seq->strobe, 1.0, 30.0, 100, 0, "Only display every nth frame");
-		uiDefButI(block, NUM, SEQ_BUT_RELOAD, "Preseek:",			10,-10,150,19, &last_seq->anim_preseek, 0.0, 50.0, 100, 0, "On MPEG-seeking preseek this many frames");
-		uiBlockEndAlign(block);
-	}
-	else if(last_seq->type==SEQ_RAM_SOUND || 
-		last_seq->type==SEQ_HD_SOUND) {
-
-		uiDefBut(block, LABEL, 0, "Type: Audio", 10,140,150,20, 0, 0, 0, 0, 0, "");
-		uiDefBut(block, TEX, 0, "Name: ", 10,120,150,19, last_seq->name+2, 0.0, 21.0, 100, 0, "");
-		
-		uiBlockBeginAlign(block);
-		uiDefButBitS(block, TOG, SEQ_IPO_FRAME_LOCKED,
-			     SEQ_BUT_RELOAD_ALL, "IPO Frame locked",
-			     10,90,150,19, &last_seq->flag, 
-			     0.0, 1.0, 0, 0, 
-			     "Lock the IPO coordinates to the "
-			     "global frame counter.");
-
-		uiDefButBitS(block, TOG, SEQ_MUTE, B_NOP, "Mute", 10,70,120,19, &last_seq->flag, 0.0, 21.0, 100, 0, "");
-		uiDefButF(block, NUM, SEQ_BUT_RELOAD, "Gain (dB):", 10,50,150,19, &last_seq->level, -96.0, 6.0, 100, 0, "");
-		uiDefButF(block, NUM, SEQ_BUT_RELOAD, "Pan:", 	10,30,150,19, &last_seq->pan, -1.0, 1.0, 100, 0, "");
-		uiBlockEndAlign(block);
-	}
-	else if(last_seq->type>=SEQ_EFFECT) {
-		uiDefBut(block, LABEL, 0, "Type: Effect", 10,140,150,20, 0, 0, 0, 0, 0, "");
-		uiDefBut(block, TEX, B_NOP, "Name: ", 10,120,150,19, last_seq->name+2, 0.0, 21.0, 100, 0, "");
-		
-		uiDefButBitS(block, TOG, SEQ_IPO_FRAME_LOCKED,
-			     SEQ_BUT_RELOAD_ALL, "IPO Frame locked",
-			     10,90,150,19, &last_seq->flag, 
-			     0.0, 1.0, 0, 0, 
-			     "Lock the IPO coordinates to the "
-			     "global frame counter.");
-		
-		uiBlockBeginAlign(block);
-		if(last_seq->type==SEQ_WIPE){
-			WipeVars *wipe = (WipeVars *)last_seq->effectdata;
-			char formatstring[256];
-			
-			strncpy(formatstring, "Transition Type %t|Single Wipe%x0|Double Wipe %x1|Iris Wipe %x4|Clock Wipe %x5", 255);
-			uiDefButS(block, MENU,SEQ_BUT_EFFECT, formatstring,	10,65,220,22, &wipe->wipetype, 0, 0, 0, 0, "What type of wipe should be performed");
-			uiDefButF(block, NUM,SEQ_BUT_EFFECT,"Blur:",	10,40,220,22, &wipe->edgeWidth,0.0,1.0, 1, 2, "The percent width of the blur edge");
-			switch(wipe->wipetype){ /*Skip Types that do not require angle*/
-				case DO_IRIS_WIPE:
-				case DO_CLOCK_WIPE:
-				break;
-				
-				default:
-					uiDefButF(block, NUM,SEQ_BUT_EFFECT,"Angle:",	10,15,220,22, &wipe->angle,-90.0,90.0, 1, 2, "The Angle of the Edge");
-			}
-			uiDefButS(block, TOG,SEQ_BUT_EFFECT,"Wipe In",  10,-10,220,22, &wipe->forward,0,0, 0, 0, "Controls Primary Direction of Wipe");				
-		}
-		else if(last_seq->type==SEQ_GLOW){
-			GlowVars *glow = (GlowVars *)last_seq->effectdata;
-
-			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "Threshold:", 	10,70,150,19, &glow->fMini, 0.0, 1.0, 0, 0, "Trigger Intensity");
-			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "Clamp:",			10,50,150,19, &glow->fClamp, 0.0, 1.0, 0, 0, "Brightness limit of intensity");
-			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "Boost factor:", 	10,30,150,19, &glow->fBoost, 0.0, 10.0, 0, 0, "Brightness multiplier");
-			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "Blur distance:", 	10,10,150,19, &glow->dDist, 0.5, 20.0, 0, 0, "Radius of glow effect");
-			uiDefButI(block, NUM, B_NOP, "Quality:", 10,-5,150,19, &glow->dQuality, 1.0, 5.0, 0, 0, "Accuracy of the blur effect");
-			uiDefButI(block, TOG, B_NOP, "Only boost", 10,-25,150,19, &glow->bNoComp, 0.0, 0.0, 0, 0, "Show the glow buffer only");
-		}
-		else if(last_seq->type==SEQ_TRANSFORM){
-			TransformVars *transform = (TransformVars *)last_seq->effectdata;
-
-			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "xScale Start:", 	10,70,150,19, &transform->ScalexIni, 0.0, 10.0, 0, 0, "X Scale Start");
-			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "xScale End:", 	160,70,150,19, &transform->ScalexFin, 0.0, 10.0, 0, 0, "X Scale End");
-			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "yScale Start:",	10,50,150,19, &transform->ScaleyIni, 0.0, 10.0, 0, 0, "Y Scale Start");
-			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "yScale End:", 	160,50,150,19, &transform->ScaleyFin, 0.0, 10.0, 0, 0, "Y Scale End");
-			
-			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "x Start:", 	10,30,150,19, &transform->xIni, -1000.0, 1000.0, 0, 0, "X Position Start");
-			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "x End:", 	160,30,150,19, &transform->xFin, -1000.0, 1000.0, 0, 0, "X Position End");
-
-			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "y Start:", 	10,10,150,19, &transform->yIni, -1000.0, 1000.0, 0, 0, "Y Position Start");
-			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "y End:", 	160,10,150,19, &transform->yFin, -1000.0, 1000.0, 0, 0, "Y Position End");
-
-			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "rot Start:",10,-10,150,19, &transform->rotIni, 0.0, 360.0, 0, 0, "Rotation Start");
-			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "rot End:",160,-10,150,19, &transform->rotFin, 0.0, 360.0, 0, 0, "Rotation End");
-		} else if(last_seq->type==SEQ_COLOR) {
-			SolidColorVars *colvars = (SolidColorVars *)last_seq->effectdata;
-			uiDefButF(block, COL, SEQ_BUT_RELOAD, "",10,90,150,19, colvars->col, 0, 0, 0, 0, "");
-		} else if(last_seq->type==SEQ_SPEED){
-			SpeedControlVars *sp = 
-				(SpeedControlVars *)last_seq->effectdata;
-
-			uiDefButF(block, NUM, SEQ_BUT_RELOAD, "Global Speed:", 	10,70,150,19, &sp->globalSpeed, 0.0, 100.0, 0, 0, "Global Speed");
-
-			uiDefButBitI(block, TOG, SEQ_SPEED_INTEGRATE,
-				     SEQ_BUT_RELOAD, 
-				     "IPO is velocity",
-				     10,50,150,19, &sp->flags, 
-				     0.0, 1.0, 0, 0, 
-				     "Interpret the IPO value as a "
-				     "velocity instead of a frame number");
-
-			uiDefButBitI(block, TOG, SEQ_SPEED_BLEND,
-				     SEQ_BUT_RELOAD, 
-				     "Enable frame blending",
-				     10,30,150,19, &sp->flags, 
-				     0.0, 1.0, 0, 0, 
-				     "Blend two frames into the "
-				     "target for a smoother result");
-
-			uiDefButBitI(block, TOG, SEQ_SPEED_COMPRESS_IPO_Y,
-				     SEQ_BUT_RELOAD, 
-				     "IPO value runs from [0..1]",
-				     10,10,150,19, &sp->flags, 
-				     0.0, 1.0, 0, 0, 
-				     "Scale IPO value to get the "
-				     "target frame number.");
-		}
-
-		uiBlockEndAlign(block);
-	}
-}
 
 static void seq_blockhandlers(ScrArea *sa)
 {
@@ -1247,13 +1034,6 @@ static void seq_blockhandlers(ScrArea *sa)
 	uiFreeBlocksWin(&sa->uiblocks, sa->win);
 
 	for(a=0; a<SPACE_MAXHANDLER; a+=2) {
-		switch(sseq->blockhandler[a]) {
-
-		case SEQ_HANDLER_PROPERTIES:
-			seq_panel_properties(sseq->blockhandler[a+1]);
-			break;
-
-		}
 		/* clear action value for event */
 		sseq->blockhandler[a+1]= 0;
 	}
@@ -1261,6 +1041,20 @@ static void seq_blockhandlers(ScrArea *sa)
 
 }
 
+void drawprefetchseqspace(ScrArea *sa, void *spacedata)
+{
+	SpaceSeq *sseq= sa->spacedata.first;
+	int rectx, recty;
+
+	rectx= (G.scene->r.size*G.scene->r.xsch)/100;
+	recty= (G.scene->r.size*G.scene->r.ysch)/100;
+
+	if(sseq->mainb) {
+		give_ibuf_prefetch_request(
+			rectx, recty, (G.scene->r.cfra), sseq->chanshown);
+	}
+}
+
 void drawseqspace(ScrArea *sa, void *spacedata)
 {
 	SpaceSeq *sseq= sa->spacedata.first;
@@ -1308,6 +1102,7 @@ void drawseqspace(ScrArea *sa, void *spacedata)
 	boundbox_seq();
 	calc_ipogrid();
 	
+	/* Alternating horizontal stripes */
 	i= MAX2(1, ((int)G.v2d->cur.ymin)-1);
 
 	glBegin(GL_QUADS);
@@ -1344,41 +1139,41 @@ void drawseqspace(ScrArea *sa, void *spacedata)
 
 	/* sequences: first deselect */
 	if(ed) {
-		seq= ed->seqbasep->first;
-		while(seq) { /* bound box test, dont draw outside the view */
-			if (seq->flag & SELECT ||
-					MIN2(seq->startdisp, seq->start) > v2d->cur.xmax ||
-					MAX2(seq->enddisp, seq->start+seq->len) < v2d->cur.xmin ||
-					seq->machine+1.0 < v2d->cur.ymin ||
-					seq->machine > v2d->cur.ymax)
-			{
-				/* dont draw */
-			} else {
-				draw_seq_strip(seq, sa, sseq);
+		Sequence *last_seq = get_last_seq();
+		int sel = 0, j;
+		int outline_tint;
+		float pixelx = (v2d->cur.xmax - v2d->cur.xmin)/(v2d->mask.xmax - v2d->mask.xmin);
+		/* loop through twice, first unselected, then selected */
+		for (j=0; j<2; j++) {
+			seq= ed->seqbasep->first;
+			if (j==0)	outline_tint = -150;
+			else		outline_tint = -60;
+			
+			while(seq) { /* bound box test, dont draw outside the view */
+				if (  ((seq->flag & SELECT) == sel) ||
+						seq == last_seq ||
+						MIN2(seq->startdisp, seq->start) > v2d->cur.xmax ||
+						MAX2(seq->enddisp, seq->start+seq->len) < v2d->cur.xmin ||
+						seq->machine+1.0 < v2d->cur.ymin ||
+						seq->machine > v2d->cur.ymax)
+				{
+					/* dont draw */
+				} else {
+					draw_seq_strip(seq, sa, sseq, outline_tint, pixelx);
+				}
+				seq= seq->next;
 			}
-			seq= seq->next;
+			sel= SELECT; /* draw selected next time round */
 		}
-	}
-	ed= G.scene->ed;
-	if(ed) {
-		seq= ed->seqbasep->first;
-		while(seq) { /* bound box test, dont draw outside the view */
-			if (!(seq->flag & SELECT) ||
-					MIN2(seq->startdisp, seq->start) > v2d->cur.xmax ||
-					MAX2(seq->enddisp, seq->start+seq->len) < v2d->cur.xmin ||
-					seq->machine+1.0 < v2d->cur.ymin ||
-					seq->machine > v2d->cur.ymax)
-			{
-				/* dont draw */
-			} else {
-				draw_seq_strip(seq, sa, sseq);
-			}
-			seq= seq->next;
+		/* draw the last selected last, removes some overlapping error */
+		if (last_seq) {
+			draw_seq_strip(last_seq, sa, sseq, 120, pixelx);
 		}
 	}
 
-	draw_extra_seqinfo();
-
+	/* Draw markers */
+	draw_markers_timespace(SCE_MARKERS, DRAW_MARKERS_LINES);
+	
 	/* restore viewport */
 	mywinset(sa->win);
 
