@@ -1,15 +1,12 @@
 /**
- * $Id: editdeform.c 11124 2007-06-29 13:41:20Z campbellbarton $
+ * $Id: editdeform.c 14776 2008-05-10 12:33:15Z blendix $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  * Creator-specific support for vertex deformation groups
  * Added: apply deform function (ton)
  */
@@ -36,11 +33,15 @@
 
 #include "MEM_guardedalloc.h"
 
+#include "DNA_cloth_types.h"
 #include "DNA_curve_types.h"
 #include "DNA_lattice_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
+#include "DNA_modifier_types.h"
 #include "DNA_object_types.h"
+#include "DNA_object_force.h"
+#include "DNA_particle_types.h"
 
 #include "BLI_blenlib.h"
 #include "BLI_editVert.h"
@@ -55,12 +56,16 @@
 #include "BKE_mesh.h"
 #include "BKE_utildefines.h"
 
+#include "BIF_interface.h"
 #include "BIF_editdeform.h"
 #include "BIF_editmesh.h"
+#include "BIF_space.h"
 #include "BIF_toolbox.h"
 
 #include "BSE_edit.h"
 
+#include "butspace.h"
+#include "mydevice.h"
 #include "editmesh.h"
 #include "multires.h"
 
@@ -216,19 +221,25 @@ void duplicate_defgroup ( Object *ob )
 	dg = BLI_findlink (&ob->defbase, (ob->actdef-1));
 	if (!dg)
 		return;
-
-	BLI_snprintf (name, 32, "%s_copy", dg->name);
-	while (get_named_vertexgroup (ob, name)) {
-		if ((strlen (name) + 6) > 32) {
-			error ("Error: the name for the new group is > 32 characters");
-			return;
+	
+	if (strstr(dg->name, "_copy")) {
+		BLI_strncpy (name, dg->name, 32); /* will be renamed _copy.001... etc */
+	} else {
+		BLI_snprintf (name, 32, "%s_copy", dg->name);
+		while (get_named_vertexgroup (ob, name)) {
+			if ((strlen (name) + 6) > 32) {
+				error ("Error: the name for the new group is > 32 characters");
+				return;
+			}
+			strcpy (s, name);
+			BLI_snprintf (name, 32, "%s_copy", s);
 		}
-		strcpy (s, name);
-		BLI_snprintf (name, 32, "%s_copy", s);
-	}
+	}		
 
 	cdg = copy_defgroup (dg);
 	strcpy (cdg->name, name);
+	unique_vertexgroup_name(cdg, ob);
+	
 	BLI_addtail (&ob->defbase, cdg);
 
 	idg = (ob->actdef-1);
@@ -249,6 +260,66 @@ void duplicate_defgroup ( Object *ob )
 	}
 }
 
+static void del_defgroup_update_users(Object *ob, int id)
+{
+	ExplodeModifierData *emd;
+	ModifierData *md;
+	ParticleSystem *psys;
+	ClothModifierData *clmd;
+	ClothSimSettings *clsim;
+	int a;
+
+	/* these cases don't use names to refer to vertex groups, so when
+	 * they get deleted the numbers get out of synce, this corrects that */
+
+	if(ob->soft) {
+		if(ob->soft->vertgroup == id)
+			ob->soft->vertgroup= 0;
+		else if(ob->soft->vertgroup > id)
+			ob->soft->vertgroup--;
+	}
+
+	for(md=ob->modifiers.first; md; md=md->next) {
+		if(md->type == eModifierType_Explode) {
+			emd= (ExplodeModifierData*)md;
+
+			if(emd->vgroup == id)
+				emd->vgroup= 0;
+			else if(emd->vgroup > id)
+				emd->vgroup--;
+		}
+		else if(md->type == eModifierType_Cloth) {
+			clmd= (ClothModifierData*)md;
+			clsim= clmd->sim_parms;
+
+			if(clsim) {
+				if(clsim->vgroup_mass == id)
+					clsim->vgroup_mass= 0;
+				else if(clsim->vgroup_mass > id)
+					clsim->vgroup_mass--;
+
+				if(clsim->vgroup_bend == id)
+					clsim->vgroup_bend= 0;
+				else if(clsim->vgroup_bend > id)
+					clsim->vgroup_bend--;
+
+				if(clsim->vgroup_struct == id)
+					clsim->vgroup_struct= 0;
+				else if(clsim->vgroup_struct > id)
+					clsim->vgroup_struct--;
+			}
+		}
+	}
+
+	for(psys=ob->particlesystem.first; psys; psys=psys->next) {
+		for(a=0; a<PSYS_TOT_VG; a++)
+			if(psys->vgroup[a] == id)
+				psys->vgroup[a]= 0;
+			else if(psys->vgroup[a] > id)
+				psys->vgroup[a]--;
+	}
+}
+
 void del_defgroup_in_object_mode ( Object *ob )
 {
 	bDeformGroup *dg;
@@ -284,6 +355,8 @@ void del_defgroup_in_object_mode ( Object *ob )
 		}
 	}
 
+	del_defgroup_update_users(ob, ob->actdef);
+
 	/* Update the active deform index if necessary */
 	if (ob->actdef == BLI_countlist(&ob->defbase))
 		ob->actdef--;
@@ -341,6 +414,8 @@ void del_defgroup (Object *ob)
 		}
 	}
 
+	del_defgroup_update_users(ob, ob->actdef);
+
 	/* Update the active deform index if necessary */
 	if (ob->actdef==BLI_countlist(&ob->defbase))
 		ob->actdef--;
@@ -700,6 +775,53 @@ void remove_vert_defgroup (Object *ob, bDeformGroup	*dg, int vertnum)
 	remove_vert_def_nr (ob, def_nr, vertnum);
 }
 
+/* for mesh in object mode lattice can be in editmode */
+static float get_vert_def_nr (Object *ob, int def_nr, int vertnum)
+{
+	MDeformVert *dvert= NULL;
+	int i;
+
+	/* get the deform vertices corresponding to the
+	 * vertnum
+	 */
+	if(ob->type==OB_MESH) {
+		if( ((Mesh*)ob->data)->dvert )
+			dvert = ((Mesh*)ob->data)->dvert + vertnum;
+	}
+	else if(ob->type==OB_LATTICE) {
+		Lattice *lt= ob->data;
+		
+		if(ob==G.obedit)
+			lt= editLatt;
+		
+		if(lt->dvert)
+			dvert = lt->dvert + vertnum;
+	}
+	
+	if(dvert==NULL)
+		return 0.0f;
+	
+	for(i=dvert->totweight-1 ; i>=0 ; i--)
+		if(dvert->dw[i].def_nr == def_nr)
+			return dvert->dw[i].weight;
+
+	return 0.0f;
+}
+
+/* mesh object mode, lattice can be in editmode */
+float get_vert_defgroup (Object *ob, bDeformGroup *dg, int vertnum)
+{
+	int def_nr;
+
+	if(!ob)
+		return 0.0f;
+
+	def_nr = get_defgroup_num(ob, dg);
+	if(def_nr < 0) return 0.0f;
+
+	return get_vert_def_nr (ob, def_nr, vertnum);
+}
+
 /* Only available in editmode */
 /* removes from active defgroup, if allverts==0 only selected vertices */
 void remove_verts_defgroup (int allverts)
@@ -728,7 +850,7 @@ void remove_verts_defgroup (int allverts)
 	case OB_MESH:
 		for (eve=G.editMesh->verts.first; eve; eve=eve->next){
 			dvert= CustomData_em_get(&G.editMesh->vdata, eve->data, CD_MDEFORMVERT);
-
+		
 			if (dvert && dvert->dw && ((eve->f & 1) || allverts)){
 				for (i=0; i<dvert->totweight; i++){
 					/* Find group */
@@ -774,6 +896,36 @@ void remove_verts_defgroup (int allverts)
 	}
 }
 
+/* Only available in editmode */
+/* removes from all defgroup, if allverts==0 only selected vertices */
+void remove_verts_defgroups(int allverts)
+{
+	Object *ob;
+	int actdef, defCount;
+	
+	if (multires_level1_test()) return;
+
+	ob= G.obedit;
+	if (ob == NULL) return;
+	
+	actdef= ob->actdef;
+	defCount= BLI_countlist(&ob->defbase);
+	
+	if (defCount == 0) {
+		error("Object has no vertex groups");
+		return;
+	}
+	
+	/* To prevent code redundancy, we just use remove_verts_defgroup, but that
+	 * only operates on the active vgroup. So we iterate through all groups, by changing
+	 * active group index
+	 */
+	for (ob->actdef= 1; ob->actdef <= defCount; ob->actdef++)
+		remove_verts_defgroup(allverts);
+		
+	ob->actdef= actdef;
+}
+
 void vertexgroup_select_by_name(Object *ob, char *name)
 {
 	bDeformGroup *curdef;
@@ -790,6 +942,104 @@ void vertexgroup_select_by_name(Object *ob, char *name)
 	ob->actdef=0;	// this signals on painting to create a new one, if a bone in posemode is selected */
 }
 
+/* This function provides a shortcut for adding/removing verts from 
+ * vertex groups. It is called by the Ctrl-G hotkey in EditMode for Meshes
+ * and Lattices. (currently only restricted to those two)
+ * It is only responsible for 
+ */
+void vgroup_assign_with_menu(void)
+{
+	Object *ob= G.obedit;
+	int defCount;
+	int mode;
+	
+	/* prevent crashes */
+	if (ob==NULL) return;
+	
+	defCount= BLI_countlist(&ob->defbase);
+	
+	/* give user choices of adding to current/new or removing from current */
+	if (defCount && ob->actdef)
+		mode = pupmenu("Vertex Groups %t|Add Selected to New Group %x1|Add Selected to Active Group %x2|Remove Selected from Active Group %x3|Remove Selected from All Groups %x4");
+	else
+		mode= pupmenu("Vertex Groups %t|Add Selected to New Group %x1");
+	
+	/* handle choices */
+	switch (mode) {
+		case 1: /* add to new group */
+			add_defgroup(ob);
+			assign_verts_defgroup();
+			allqueue(REDRAWVIEW3D, 1);
+			BIF_undo_push("Assign to vertex group");
+			break;
+		case 2: /* add to current group */
+			assign_verts_defgroup();
+			allqueue(REDRAWVIEW3D, 1);
+			BIF_undo_push("Assign to vertex group");
+			break;
+		case 3:	/* remove from current group */
+			remove_verts_defgroup(0);
+			allqueue(REDRAWVIEW3D, 1);
+			BIF_undo_push("Remove from vertex group");
+			break;
+		case 4: /* remove from all groups */
+			remove_verts_defgroups(0);
+			allqueue(REDRAWVIEW3D, 1);
+			BIF_undo_push("Remove from all vertex groups");
+			break;
+	}
+}
+
+/* This function provides a shortcut for commonly used vertex group 
+ * functions - change weight (not implemented), change active group, delete active group,
+ * when Ctrl-Shift-G is used in EditMode, for Meshes and Lattices (only for now).
+ */
+void vgroup_operation_with_menu(void)
+{
+	Object *ob= G.obedit;
+	int defCount;
+	int mode;
+	
+	/* prevent crashes and useless cases */
+	if (ob==NULL) return;
+	
+	defCount= BLI_countlist(&ob->defbase);
+	if (defCount == 0) return;
+	
+	/* give user choices of adding to current/new or removing from current */
+	if (ob->actdef)
+		mode = pupmenu("Vertex Groups %t|Change Active Group%x1|Delete Active Group%x2");
+	else
+		mode= pupmenu("Vertex Groups %t|Change Active Group%x1");
+	
+	/* handle choices */
+	switch (mode) {
+		case 1: /* change active group*/
+			{
+				char *menustr= get_vertexgroup_menustr(ob);
+				short nr;
+				
+				if (menustr) {
+					nr= pupmenu(menustr);
+					
+					if ((nr >= 1) && (nr <= defCount)) 
+						ob->actdef= nr;
+						
+					MEM_freeN(menustr);
+				}
+				allqueue(REDRAWBUTSALL, 0);
+			}
+			break;
+		case 2: /* delete active group  */
+			{
+				del_defgroup(ob);
+				allqueue (REDRAWVIEW3D, 1);
+				allqueue(REDRAWOOPS, 0);
+				BIF_undo_push("Delete vertex group");
+			}
+			break;
+	}
+}
 
 /* ******************* other deform edit stuff ********** */
 
