@@ -1,15 +1,12 @@
 /**
- * $Id: editmesh.c,v 1.205 2007/04/29 13:39:45 campbellbarton Exp $
+ * $Id: editmesh.c 14479 2008-04-19 19:19:39Z campbellbarton $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 
@@ -65,6 +62,7 @@
 
 #include "BKE_DerivedMesh.h"
 #include "BKE_depsgraph.h"
+#include "BKE_cloth.h"
 #include "BKE_customdata.h"
 #include "BKE_global.h"
 #include "BKE_key.h"
@@ -72,7 +70,11 @@
 #include "BKE_main.h"
 #include "BKE_material.h"
 #include "BKE_mesh.h"
+#include "BKE_modifier.h"
+#include "BKE_multires.h"
 #include "BKE_object.h"
+#include "BKE_pointcache.h"
+#include "BKE_softbody.h"
 #include "BKE_texture.h"
 #include "BKE_utildefines.h"
 
@@ -159,10 +161,13 @@ EditVert *addvertlist(float *vec, EditVert *example)
 	createVerseVert(eve);
 #endif
 
-	if(example)
+	if(example) {
 		CustomData_em_copy_data(&em->vdata, &em->vdata, example->data, &eve->data);
-	else
+		eve->bweight = example->bweight;
+	}
+	else {
 		CustomData_em_set_default(&em->vdata, &eve->data);
+	}
 
 	return eve;
 }
@@ -295,6 +300,7 @@ EditEdge *addedgelist(EditVert *v1, EditVert *v2, EditEdge *example)
 		   rule is to do this with addedgelist call, before addfacelist */
 		if(example) {
 			eed->crease= example->crease;
+			eed->bweight= example->bweight;
 			eed->sharp = example->sharp;
 			eed->seam = example->seam;
 			eed->h |= (example->h & EM_FGON);
@@ -333,6 +339,10 @@ void free_editface(EditFace *efa)
 	}
 #endif
 	EM_remove_selection(efa, EDITFACE);
+	
+	if (G.editMesh->act_face==efa)
+		EM_set_actFace(NULL);
+		
 	CustomData_em_free_block(&G.editMesh->fdata, &efa->data);
 	if(efa->fast==0)
 		free(efa);
@@ -784,6 +794,78 @@ static void edge_drawflags(void)
 	}
 }
 
+static int editmesh_pointcache_edit(Object *ob, int totvert, PTCacheID *pid_p, float mat[][4], int load)
+{
+	Cloth *cloth;
+	SoftBody *sb;
+	ClothModifierData *clmd;
+	PTCacheID pid, tmpid;
+	int cfra= (int)G.scene->r.cfra, found= 0;
+
+	pid.cache= NULL;
+
+	/* check for cloth */
+	if(modifiers_isClothEnabled(ob)) {
+		clmd= (ClothModifierData*)modifiers_findByType(ob, eModifierType_Cloth);
+		cloth= clmd->clothObject;
+		
+		BKE_ptcache_id_from_cloth(&tmpid, ob, clmd);
+
+		/* verify vertex count and baked status */
+		if(cloth && (totvert == cloth->numverts)) {
+			if((tmpid.cache->flag & PTCACHE_BAKED) && (tmpid.cache->flag & PTCACHE_BAKE_EDIT)) {
+				pid= tmpid;
+
+				if(load && (pid.cache->flag & PTCACHE_BAKE_EDIT_ACTIVE))
+					found= 1;
+			}
+		}
+	}
+
+	/* check for softbody */
+	if(!found && ob->soft) {
+		sb= ob->soft;
+
+		BKE_ptcache_id_from_softbody(&tmpid, ob, sb);
+
+		/* verify vertex count and baked status */
+		if(sb->bpoint && (totvert == sb->totpoint)) {
+			if((tmpid.cache->flag & PTCACHE_BAKED) && (tmpid.cache->flag & PTCACHE_BAKE_EDIT)) {
+				pid= tmpid;
+
+				if(load && (pid.cache->flag & PTCACHE_BAKE_EDIT_ACTIVE))
+					found= 1;
+			}
+		}
+	}
+
+	/* if not making editmesh verify editing was active for this point cache */
+	if(load) {
+		if(found)
+			pid.cache->flag &= ~PTCACHE_BAKE_EDIT_ACTIVE;
+		else
+			return 0;
+	}
+
+	/* check if we have cache for this frame */
+	if(pid.cache && BKE_ptcache_id_exist(&pid, cfra)) {
+		*pid_p = pid;
+		
+		if(load) {
+			Mat4CpyMat4(mat, ob->obmat);
+		}
+		else {
+			pid.cache->editframe= cfra;
+			pid.cache->flag |= PTCACHE_BAKE_EDIT_ACTIVE;
+			Mat4Invert(mat, ob->obmat); /* ob->imat is not up to date */
+		}
+
+		return 1;
+	}
+
+	return 0;
+}
+
 /* turns Mesh into editmesh */
 void make_editMesh()
 {
@@ -797,7 +879,11 @@ void make_editMesh()
 	EditFace *efa;
 	EditEdge *eed;
 	EditSelection *ese;
-	int tot, a, eekadoodle= 0;
+	PTCacheID pid;
+	Cloth *cloth;
+	SoftBody *sb;
+	float cacheco[3], cachemat[4][4], *co;
+	int tot, a, cacheedit= 0, eekadoodle= 0;
 
 #ifdef WITH_VERSE
 	if(me->vnode){
@@ -809,10 +895,11 @@ void make_editMesh()
 	/* because of reload */
 	free_editMesh(em);
 	
+	em->act_face = NULL;
 	G.totvert= tot= me->totvert;
 	G.totedge= me->totedge;
 	G.totface= me->totface;
-
+	
 	if(tot==0) {
 		countall();
 		return;
@@ -832,13 +919,32 @@ void make_editMesh()
 	CustomData_copy(&me->vdata, &em->vdata, CD_MASK_EDITMESH, CD_CALLOC, 0);
 	mvert= me->mvert;
 
+	cacheedit= editmesh_pointcache_edit(G.obedit, tot, &pid, cachemat, 0);
+
 	evlist= (EditVert **)MEM_mallocN(tot*sizeof(void *),"evlist");
 	for(a=0; a<tot; a++, mvert++) {
-		eve= addvertlist(mvert->co, NULL);
+		
+		if(cacheedit) {
+			if(pid.type == PTCACHE_TYPE_CLOTH) {
+				cloth= ((ClothModifierData*)pid.data)->clothObject;
+				VECCOPY(cacheco, cloth->verts[a].x)
+			}
+			else if(pid.type == PTCACHE_TYPE_SOFTBODY) {
+				sb= (SoftBody*)pid.data;
+				VECCOPY(cacheco, sb->bpoint[a].pos)
+			}
+
+			Mat4MulVecfl(cachemat, cacheco);
+			co= cacheco;
+		}
+		else
+			co= mvert->co;
+
+		eve= addvertlist(co, NULL);
 		evlist[a]= eve;
 		
 		// face select sets selection in next loop
-		if( (G.f & G_FACESELECT)==0 )
+		if( (FACESEL_PAINT_TEST)==0 )
 			eve->f |= (mvert->flag & 1);
 		
 		if (mvert->flag & ME_HIDE) eve->h= 1;		
@@ -846,6 +952,8 @@ void make_editMesh()
 		eve->no[1]= mvert->no[1]/32767.0;
 		eve->no[2]= mvert->no[2]/32767.0;
 
+		eve->bweight= ((float)mvert->bweight)/255.0f;
+
 		/* lets overwrite the keyindex of the editvert
 		 * with the order it used to be in before
 		 * editmode
@@ -859,12 +967,14 @@ void make_editMesh()
 	else {
 		MEdge *medge= me->medge;
 		
+		CustomData_copy(&me->edata, &em->edata, CD_MASK_EDITMESH, CD_CALLOC, 0);
 		/* make edges */
 		for(a=0; a<me->totedge; a++, medge++) {
 			eed= addedgelist(evlist[medge->v1], evlist[medge->v2], NULL);
 			/* eed can be zero when v1 and v2 are identical, dxf import does this... */
 			if(eed) {
-				eed->crease= ((float)medge->crease)/255.0;
+				eed->crease= ((float)medge->crease)/255.0f;
+				eed->bweight= ((float)medge->bweight)/255.0f;
 				
 				if(medge->flag & ME_SEAM) eed->seam= 1;
 				if(medge->flag & ME_SHARP) eed->sharp = 1;
@@ -873,6 +983,7 @@ void make_editMesh()
 				if(medge->flag & ME_HIDE) eed->h |= 1;
 				if(G.scene->selectmode==SCE_SELECT_EDGE) 
 					EM_select_edge(eed, eed->f & SELECT);		// force edge selection to vertices, seems to be needed ...
+				CustomData_to_em_block(&me->edata,&em->edata, a, &eed->data);
 			}
 		}
 		
@@ -897,13 +1008,22 @@ void make_editMesh()
 				efa->flag= mface->flag & ~ME_HIDE;
 				
 				/* select and hide face flag */
-				if(mface->flag & ME_FACE_SEL) {
-					efa->f |= SELECT;
+				if(mface->flag & ME_HIDE) {
+					efa->h= 1;
+				} else {
+					if (a==me->act_face) {
+						EM_set_actFace(efa);
+					}
+					
+					/* dont allow hidden and selected */
+					if(mface->flag & ME_FACE_SEL) {
+						efa->f |= SELECT;
+						
+						if(FACESEL_PAINT_TEST) {
+							EM_select_face(efa, 1); /* flush down */
+						}
+					}
 				}
-				if(mface->flag & ME_HIDE) efa->h= 1;
-
-				if((G.f & G_FACESELECT) && (efa->f & SELECT))
-					EM_select_face(efa, 1); /* flush down */
 			}
 		}
 	}
@@ -922,7 +1042,7 @@ void make_editMesh()
 		
 		for(a=0; a<me->totselect; a++, mselect++){
 			/*check if recorded selection is still valid, if so copy into editmesh*/
-			if( (mselect->type == EDITVERT && me->mvert[mselect->index].flag & SELECT) || (mselect->type == EDITEDGE && me->medge[mselect->index].flag & SELECT) || (mselect->type == EDITFACE && me->mface[mselect->index].flag & SELECT) ){
+			if( (mselect->type == EDITVERT && me->mvert[mselect->index].flag & SELECT) || (mselect->type == EDITEDGE && me->medge[mselect->index].flag & SELECT) || (mselect->type == EDITFACE && me->mface[mselect->index].flag & ME_FACE_SEL) ){
 				ese = MEM_callocN(sizeof(EditSelection), "Edit Selection");
 				ese->type = mselect->type;	
 				if(ese->type == EDITVERT) ese->data = EM_get_vert_for_index(mselect->index); else
@@ -939,9 +1059,12 @@ void make_editMesh()
 	EM_hide_reset();
 	/* sets helper flags which arent saved */
 	EM_fgon_flags();
+
+	/* vertex coordinates change with cache edit, need to recalc */
+	if(cacheedit)
+		recalc_editnormals();
 	
 	countall();
-	
 }
 
 /* makes Mesh out of editmesh */
@@ -954,12 +1077,16 @@ void load_editMesh(void)
 	MFace *mface;
 	MSelect *mselect;
 	EditVert *eve;
-	EditFace *efa;
+	EditFace *efa, *efa_act;
 	EditEdge *eed;
 	EditSelection *ese;
-	float *fp, *newkey, *oldkey, nor[3];
-	int i, a, ototvert, totedge=0;
-
+	SoftBody *sb;
+	Cloth *cloth;
+	ClothModifierData *clmd;
+	PTCacheID pid;
+	float *fp, *newkey, *oldkey, nor[3], cacheco[3], cachemat[4][4];
+	int i, a, ototvert, totedge=0, cacheedit= 0;
+	
 #ifdef WITH_VERSE
 	if(em->vnode) {
 		struct VNode *vnode = (VNode*)em->vnode;
@@ -1014,6 +1141,7 @@ void load_editMesh(void)
 	me->totface= G.totface;
 
 	CustomData_copy(&em->vdata, &me->vdata, CD_MASK_MESH, CD_CALLOC, me->totvert);
+	CustomData_copy(&em->edata, &me->edata, CD_MASK_MESH, CD_CALLOC, me->totedge);
 	CustomData_copy(&em->fdata, &me->fdata, CD_MASK_MESH, CD_CALLOC, me->totface);
 
 	CustomData_add_layer(&me->vdata, CD_MVERT, CD_ASSIGN, mvert, me->totvert);
@@ -1025,8 +1153,49 @@ void load_editMesh(void)
 	eve= em->verts.first;
 	a= 0;
 
+	/* check for point cache editing */
+	cacheedit= editmesh_pointcache_edit(G.obedit, G.totvert, &pid, cachemat, 1);
+
 	while(eve) {
-		VECCOPY(mvert->co, eve->co);
+		if(cacheedit) {
+			if(pid.type == PTCACHE_TYPE_CLOTH) {
+				clmd= (ClothModifierData*)pid.data;
+				cloth= clmd->clothObject;
+
+				/* assign position */
+				VECCOPY(cacheco, cloth->verts[a].x)
+				VECCOPY(cloth->verts[a].x, eve->co);
+				Mat4MulVecfl(cachemat, cloth->verts[a].x);
+
+				/* find plausible velocity, not physical correct but gives
+				 * nicer results when commented */
+				VECSUB(cacheco, cloth->verts[a].x, cacheco);
+				VecMulf(cacheco, clmd->sim_parms->stepsPerFrame*10.0f);
+				VECADD(cloth->verts[a].v, cloth->verts[a].v, cacheco);
+			}
+			else if(pid.type == PTCACHE_TYPE_SOFTBODY) {
+				sb= (SoftBody*)pid.data;
+
+				/* assign position */
+				VECCOPY(cacheco, sb->bpoint[a].pos)
+				VECCOPY(sb->bpoint[a].pos, eve->co);
+				Mat4MulVecfl(cachemat, sb->bpoint[a].pos);
+
+				/* changing velocity for softbody doesn't seem to give
+				 * good results? */
+#if 0
+				VECSUB(cacheco, sb->bpoint[a].pos, cacheco);
+				VecMulf(cacheco, sb->minloops*10.0f);
+				VECADD(sb->bpoint[a].vec, sb->bpoint[a].pos, cacheco);
+#endif
+			}
+
+			if(oldverts)
+				VECCOPY(mvert->co, oldverts[a].co)
+		}
+		else
+			VECCOPY(mvert->co, eve->co);
+
 		mvert->mat_nr= 255;  /* what was this for, halos? */
 		
 		/* vertex normal */
@@ -1043,7 +1212,8 @@ void load_editMesh(void)
 		mvert->flag= 0;
 		if(eve->f1==1) mvert->flag |= ME_SPHERETEST;
 		mvert->flag |= (eve->f & SELECT);
-		if (eve->h) mvert->flag |= ME_HIDE;			
+		if (eve->h) mvert->flag |= ME_HIDE;
+		mvert->bweight= (char)(255.0*eve->bweight);
 
 #ifdef WITH_VERSE
 		if(eve->vvert) {
@@ -1054,6 +1224,14 @@ void load_editMesh(void)
 		eve= eve->next;
 		mvert++;
 	}
+	
+	/* write changes to cache */
+	if(cacheedit) {
+		if(pid.type == PTCACHE_TYPE_CLOTH)
+			cloth_write_cache(G.obedit, pid.data, pid.cache->editframe);
+		else if(pid.type == PTCACHE_TYPE_SOFTBODY)
+			sbWriteCache(G.obedit, pid.cache->editframe);
+	}
 
 	/* the edges */
 	a= 0;
@@ -1071,7 +1249,9 @@ void load_editMesh(void)
 		if(eed->h & 1) medge->flag |= ME_HIDE;
 		
 		medge->crease= (char)(255.0*eed->crease);
-		
+		medge->bweight= (char)(255.0*eed->bweight);
+		CustomData_from_em_block(&em->edata, &me->edata, eed->data, a);		
+
 		eed->tmp.l = a++;
 		
 		medge++;
@@ -1081,7 +1261,9 @@ void load_editMesh(void)
 	/* the faces */
 	a = 0;
 	efa= em->faces.first;
+	efa_act= EM_get_actFace(0);
 	i = 0;
+	me->act_face = -1;
 	while(efa) {
 		mface= &((MFace *) me->mface)[i];
 		
@@ -1136,6 +1318,9 @@ void load_editMesh(void)
 
 		/* no index '0' at location 3 or 4 */
 		test_index_face(mface, &me->fdata, i, efa->v4?4:3);
+		
+		if (efa_act == efa)
+			me->act_face = a;
 
 #ifdef WITH_VERSE
 		if(efa->vface) {
@@ -1307,12 +1492,9 @@ void load_editMesh(void)
 	/* remake softbody of all users */
 	if(me->id.us>1) {
 		Base *base;
-		for(base= G.scene->base.first; base; base= base->next) {
-			if(base->object->data==me) {
-				base->object->softflag |= OB_SB_REDO;
+		for(base= G.scene->base.first; base; base= base->next)
+			if(base->object->data==me)
 				base->object->recalc |= OB_RECALC_DATA;
-			}
-		}
 	}
 
 	mesh_calc_normals(me->mvert, me->totvert, me->mface, me->totface, NULL);
@@ -1322,6 +1504,7 @@ void remake_editMesh(void)
 {
 	make_editMesh();
 	allqueue(REDRAWVIEW3D, 0);
+	allqueue(REDRAWBUTSOBJECT, 0); /* needed to have nice cloth panels */
 	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 	BIF_undo_push("Undo all changes");
 }
@@ -1787,6 +1970,7 @@ typedef struct EditVertC
 	float no[3];
 	float co[3];
 	unsigned char f, h;
+	short bweight;
 	int keyindex;
 } EditVertC;
 
@@ -1794,7 +1978,7 @@ typedef struct EditEdgeC
 {
 	int v1, v2;
 	unsigned char f, h, seam, sharp, pad;
-	short crease, fgoni;
+	short crease, bweight, fgoni;
 } EditEdgeC;
 
 typedef struct EditFaceC
@@ -1823,7 +2007,7 @@ typedef struct UndoMesh {
 	short selectmode;
 	RetopoPaintData *retopo_paint_data;
 	char retopo_mode;
-	CustomData vdata, fdata;
+	CustomData vdata, edata, fdata;
 	EM_MultiresUndo *mru;
 } UndoMesh;
 
@@ -1839,6 +2023,7 @@ static void free_undoMesh(void *umv)
 	if(um->selected) MEM_freeN(um->selected);
 	if(um->retopo_paint_data) retopo_free_paint_data(um->retopo_paint_data);
 	CustomData_free(&um->vdata, um->totvert);
+	CustomData_free(&um->edata, um->totedge);
 	CustomData_free(&um->fdata, um->totface);
 	if(um->mru) {
 		--um->mru->users;
@@ -1881,6 +2066,7 @@ static void *editMesh_to_undoMesh(void)
 	if(um->totsel) esec= um->selected= MEM_callocN(um->totsel*sizeof(EditSelectionC), "allselections");
 
 	if(um->totvert) CustomData_copy(&em->vdata, &um->vdata, CD_MASK_EDITMESH, CD_CALLOC, um->totvert);
+	if(um->totedge) CustomData_copy(&em->edata, &um->edata, CD_MASK_EDITMESH, CD_CALLOC, um->totedge);
 	if(um->totface) CustomData_copy(&em->fdata, &um->fdata, CD_MASK_EDITMESH, CD_CALLOC, um->totface);
 	
 	/* now copy vertices */
@@ -1893,6 +2079,7 @@ static void *editMesh_to_undoMesh(void)
 		evec->h= eve->h;
 		evec->keyindex= eve->keyindex;
 		eve->tmp.l = a; /*store index*/
+		evec->bweight= (short)(eve->bweight*255.0);
 
 		CustomData_from_em_block(&em->vdata, &um->vdata, eve->data, a);
 	}
@@ -1907,8 +2094,11 @@ static void *editMesh_to_undoMesh(void)
 		eedc->seam= eed->seam;
 		eedc->sharp= eed->sharp;
 		eedc->crease= (short)(eed->crease*255.0);
+		eedc->bweight= (short)(eed->bweight*255.0);
 		eedc->fgoni= eed->fgoni;
 		eed->tmp.l = a; /*store index*/
+		CustomData_from_em_block(&em->edata, &um->edata, eed->data, a);
+	
 	}
 	
 	/* copy faces */
@@ -2000,9 +2190,11 @@ static void undoMesh_to_editMesh(void *umv)
 #endif
 
 	CustomData_free(&em->vdata, 0);
+	CustomData_free(&em->edata, 0);
 	CustomData_free(&em->fdata, 0);
 
 	CustomData_copy(&um->vdata, &em->vdata, CD_MASK_EDITMESH, CD_CALLOC, 0);
+	CustomData_copy(&um->edata, &em->edata, CD_MASK_EDITMESH, CD_CALLOC, 0);
 	CustomData_copy(&um->fdata, &em->fdata, CD_MASK_EDITMESH, CD_CALLOC, 0);
 
 	/* now copy vertices */
@@ -2016,6 +2208,7 @@ static void undoMesh_to_editMesh(void *umv)
 		eve->f= evec->f;
 		eve->h= evec->h;
 		eve->keyindex= evec->keyindex;
+		eve->bweight= ((float)evec->bweight)/255.0f;
 
 		CustomData_to_em_block(&um->vdata, &em->vdata, a, &eve->data);
 	}
@@ -2029,7 +2222,9 @@ static void undoMesh_to_editMesh(void *umv)
 		eed->seam= eedc->seam;
 		eed->sharp= eedc->sharp;
 		eed->fgoni= eedc->fgoni;
-		eed->crease= ((float)eedc->crease)/255.0;
+		eed->crease= ((float)eedc->crease)/255.0f;
+		eed->bweight= ((float)eedc->bweight)/255.0f;
+		CustomData_to_em_block(&um->edata, &em->edata, a, &eed->data);
 	}
 	
 	/* copy faces */
@@ -2089,7 +2284,7 @@ static void undoMesh_to_editMesh(void *umv)
 /* and this is all the undo system needs to know */
 void undo_push_mesh(char *name)
 {
-	undo_editmode_push(name, free_undoMesh, undoMesh_to_editMesh, editMesh_to_undoMesh);
+	undo_editmode_push(name, free_undoMesh, undoMesh_to_editMesh, editMesh_to_undoMesh, NULL);
 }
 
 
@@ -2153,3 +2348,30 @@ EditFace *EM_get_face_for_index(int index)
 {
 	return g_em_face_array?g_em_face_array[index]:NULL;
 }
+
+/* can we edit UV's for this mesh?*/
+int EM_texFaceCheck(void)
+{
+	/* some of these checks could be a touch overkill */
+	if (	(G.obedit) &&
+			(G.obedit->type == OB_MESH) &&
+			(G.editMesh) &&
+			(G.editMesh->faces.first) &&
+			(CustomData_has_layer(&G.editMesh->fdata, CD_MTFACE)))
+		return 1;
+	return 0;
+}
+
+/* can we edit colors for this mesh?*/
+int EM_vertColorCheck(void)
+{
+	/* some of these checks could be a touch overkill */
+	if (	(G.obedit) &&
+			(G.obedit->type == OB_MESH) &&
+			(G.editMesh) &&
+			(G.editMesh->faces.first) &&
+			(CustomData_has_layer(&G.editMesh->fdata, CD_MCOL)))
+		return 1;
+	return 0;
+}
+
