@@ -1,15 +1,12 @@
 /**
  * $Id: 
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 /*
@@ -72,6 +69,69 @@ editmesh_lib: generic (no UI, no menus) operations/evaluators for editmesh data
 
 #include "editmesh.h"
 
+/* this replaces the active flag used in uv/face mode */
+void EM_set_actFace(EditFace *efa)
+{
+	G.editMesh->act_face = efa;
+}
+
+EditFace * EM_get_actFace(int sloppy)
+{
+	if (G.editMesh->act_face) {
+		return G.editMesh->act_face;
+	} else if (sloppy) {
+		EditFace *efa= NULL;
+		EditSelection *ese;
+		
+		ese = G.editMesh->selected.last;
+		for (; ese; ese=ese->prev){
+			if(ese->type == EDITFACE) {
+				efa = (EditFace *)ese->data;
+				
+				if (efa->h)	efa= NULL;
+				else		break;
+			}
+		}
+		if (efa==NULL) {
+			for (efa= G.editMesh->faces.first; efa; efa= efa->next) {
+				if (efa->f & SELECT)
+					break;
+			}
+		}
+		return efa; /* can still be null */
+	}
+	return NULL;
+}
+
+int EM_get_actSelection(EditSelection *ese)
+{
+	EditSelection *ese_last = G.editMesh->selected.last;
+	EditFace *efa = EM_get_actFace(0);
+
+	ese->next = ese->prev = NULL;
+	
+	if (ese_last) {
+		if (ese_last->type == EDITFACE) { /* if there is an active face, use it over the last selected face */
+			if (efa) {
+				ese->data = (void *)efa;
+			} else {
+				ese->data = ese_last->data;
+			}
+			ese->type = EDITFACE;
+		} else {
+			ese->data = ese_last->data;
+			ese->type = ese_last->type;
+		}
+	} else if (efa) { /* no */
+		ese->data = (void *)efa;
+		ese->type = EDITFACE;
+	} else {
+		ese->data = NULL;
+		return 0;
+	}
+	return 1;
+}
+
 /* ********* Selection History ************ */
 static int EM_check_selection(void *data)
 {
@@ -820,7 +880,7 @@ void EM_free_data_layer(CustomData *data, int type)
 
 static void add_normal_aligned(float *nor, float *add)
 {
-	if( INPR(nor, add) < 0.0 ) 
+	if( INPR(nor, add) < -0.9999f)
 		VecSubf(nor, nor, add);
 	else
 		VecAddf(nor, nor, add);
@@ -1066,7 +1126,7 @@ static short extrudeflag_edge(short flag, float *nor)
 	EditMesh *em = G.editMesh;
 	EditVert *eve, *nextve;
 	EditEdge *eed, *nexted;
-	EditFace *efa, *nextfa;
+	EditFace *efa, *nextfa, *efan;
 	short del_old= 0;
 	ModifierData *md;
 	
@@ -1131,26 +1191,39 @@ static short extrudeflag_edge(short flag, float *nor)
 			MirrorModifierData *mmd = (MirrorModifierData*) md;	
 		
 			if(mmd->flag & MOD_MIR_CLIPPING) {
+				float mtx[4][4];
+				if (mmd->mirror_ob) {
+					float imtx[4][4];
+					Mat4Invert(imtx, mmd->mirror_ob->obmat);
+					Mat4MulMat4(mtx, G.obedit->obmat, imtx);
+				}
+
 				for (eed= em->edges.first; eed; eed= eed->next) {
 					if(eed->f2 == 1) {
+						float co1[3], co2[3];
 
-						switch(mmd->axis){
-							case 0:
-								if ( (fabs(eed->v1->co[0]) < mmd->tolerance) &&
-									 (fabs(eed->v2->co[0]) < mmd->tolerance) )
-									++eed->f2;
-								break;
-							case 1:
-								if ( (fabs(eed->v1->co[1]) < mmd->tolerance) &&
-									 (fabs(eed->v2->co[1]) < mmd->tolerance) )
-									++eed->f2;
-								break;
-							case 2:
-								if ( (fabs(eed->v1->co[2]) < mmd->tolerance) &&
-									 (fabs(eed->v2->co[2]) < mmd->tolerance) )
-									++eed->f2;
-								break;
+						VecCopyf(co1, eed->v1->co);
+						VecCopyf(co2, eed->v2->co);
+
+						if (mmd->mirror_ob) {
+							VecMat4MulVecfl(co1, mtx, co1);
+							VecMat4MulVecfl(co2, mtx, co2);
 						}
+
+						if (mmd->flag & MOD_MIR_AXIS_X)
+							if ( (fabs(co1[0]) < mmd->tolerance) &&
+								 (fabs(co2[0]) < mmd->tolerance) )
+								++eed->f2;
+
+						if (mmd->flag & MOD_MIR_AXIS_Y)
+							if ( (fabs(co1[1]) < mmd->tolerance) &&
+								 (fabs(co2[1]) < mmd->tolerance) )
+								++eed->f2;
+
+						if (mmd->flag & MOD_MIR_AXIS_Z)
+							if ( (fabs(co1[2]) < mmd->tolerance) &&
+								 (fabs(co2[2]) < mmd->tolerance) )
+								++eed->f2;
 					}
 				}
 			}
@@ -1207,27 +1280,32 @@ static short extrudeflag_edge(short flag, float *nor)
 			
 			if(del_old==0) {	// keep old faces means flipping normal
 				if(efa->v4)
-					addfacelist(efa->v4->tmp.v, efa->v3->tmp.v, 
+					efan = addfacelist(efa->v4->tmp.v, efa->v3->tmp.v, 
 								efa->v2->tmp.v, efa->v1->tmp.v, efa, efa);
 				else
-					addfacelist(efa->v3->tmp.v, efa->v2->tmp.v, 
+					efan = addfacelist(efa->v3->tmp.v, efa->v2->tmp.v, 
 								efa->v1->tmp.v, NULL, efa, efa);
 			}
 			else {
 				if(efa->v4)
-					addfacelist(efa->v1->tmp.v, efa->v2->tmp.v, 
+					efan = addfacelist(efa->v1->tmp.v, efa->v2->tmp.v, 
 								efa->v3->tmp.v, efa->v4->tmp.v, efa, efa);
 				else
-					addfacelist(efa->v1->tmp.v, efa->v2->tmp.v, 
+					efan = addfacelist(efa->v1->tmp.v, efa->v2->tmp.v, 
 								efa->v3->tmp.v, NULL, efa, efa);
 			}
-	
+			
+			if (G.editMesh->act_face == efa) {
+				G.editMesh->act_face = efan; 
+			}
+			
 			/* for transform */
 			add_normal_aligned(nor, efa->n);
 		}
 	}
 	
 	if(del_old) {
+		
 		/* step 4: remove old faces, if del_old */
 		efa= em->faces.first;
 		while(efa) {
@@ -1238,7 +1316,8 @@ static short extrudeflag_edge(short flag, float *nor)
 			}
 			efa= nextfa;
 		}
-	
+		
+		
 		/* step 5: remove selected unused edges */
 		/* start tagging again */
 		for(eed= em->edges.first; eed; eed= eed->next) eed->f1=0;
@@ -1400,26 +1479,38 @@ short extrudeflag_vert(short flag, float *nor)
 			MirrorModifierData *mmd = (MirrorModifierData*) md;	
 		
 			if(mmd->flag & MOD_MIR_CLIPPING) {
+				float mtx[4][4];
+				if (mmd->mirror_ob) {
+					float imtx[4][4];
+					Mat4Invert(imtx, mmd->mirror_ob->obmat);
+					Mat4MulMat4(mtx, G.obedit->obmat, imtx);
+				}
+
 				for (eed= em->edges.first; eed; eed= eed->next) {
 					if(eed->f2 == 2) {
+						float co1[3], co2[3];
 
-						switch(mmd->axis){
-							case 0:
-								if ( (fabs(eed->v1->co[0]) < mmd->tolerance) &&
-									 (fabs(eed->v2->co[0]) < mmd->tolerance) )
-									++eed->f2;
-								break;
-							case 1:
-								if ( (fabs(eed->v1->co[1]) < mmd->tolerance) &&
-									 (fabs(eed->v2->co[1]) < mmd->tolerance) )
-									++eed->f2;
-								break;
-							case 2:
-								if ( (fabs(eed->v1->co[2]) < mmd->tolerance) &&
-									 (fabs(eed->v2->co[2]) < mmd->tolerance) )
-									++eed->f2;
-								break;
+						VecCopyf(co1, eed->v1->co);
+						VecCopyf(co2, eed->v2->co);
+
+						if (mmd->mirror_ob) {
+							VecMat4MulVecfl(co1, mtx, co1);
+							VecMat4MulVecfl(co2, mtx, co2);
 						}
+
+						if (mmd->flag & MOD_MIR_AXIS_X)
+							if ( (fabs(co1[0]) < mmd->tolerance) &&
+								 (fabs(co2[0]) < mmd->tolerance) )
+								++eed->f2;
+
+						if (mmd->flag & MOD_MIR_AXIS_Y)
+							if ( (fabs(co1[1]) < mmd->tolerance) &&
+								 (fabs(co2[1]) < mmd->tolerance) )
+								++eed->f2;
+						if (mmd->flag & MOD_MIR_AXIS_Z)
+							if ( (fabs(co1[2]) < mmd->tolerance) &&
+								 (fabs(co2[2]) < mmd->tolerance) )
+								++eed->f2;
 					}
 				}
 			}
@@ -1641,7 +1732,7 @@ void adduplicateflag(int flag)
 	EditMesh *em = G.editMesh;
 	EditVert *eve, *v1, *v2, *v3, *v4;
 	EditEdge *eed, *newed;
-	EditFace *efa, *newfa;
+	EditFace *efa, *newfa, *act_efa = EM_get_actFace(0);
 
 	EM_clear_flag_all(128);
 	EM_selectmode_set();	// paranoia check, selection now is consistant
@@ -1688,6 +1779,10 @@ void adduplicateflag(int flag)
 			
 			newfa= addfacelist(v1, v2, v3, v4, efa, efa); 
 			
+			if (efa==act_efa) {
+				EM_set_actFace(newfa);
+			}
+			
 			newfa->f= efa->f;
 			efa->f -= flag;
 			efa->f |= 128;
@@ -2070,3 +2165,134 @@ void EM_fgon_flags(void)
 	}
 	
 }
+
+/* editmesh vertmap, copied from intern.mesh.c
+ * if do_face_idx_array is 0 it means we need to run it as well as freeing
+ * */
+
+UvVertMap *make_uv_vert_map_EM(int selected, int do_face_idx_array, float *limit)
+{
+	EditMesh *em = G.editMesh;
+	EditVert *ev;
+	EditFace *efa;
+	int totverts;
+	
+	/* vars from original func */
+	UvVertMap *vmap;
+	UvMapVert *buf;
+	MTFace *tf;
+	unsigned int a;
+	int	i, totuv, nverts;
+	
+	if (do_face_idx_array)
+		EM_init_index_arrays(0, 0, 1);
+	
+	/* we need the vert */
+	for (ev= em->verts.first, totverts=0; ev; ev= ev->next, totverts++) {
+		ev->tmp.l = totverts;
+	}
+	
+	totuv = 0;
+
+	/* generate UvMapVert array */
+	for (efa= em->faces.first; efa; efa= efa->next)
+		if(!selected || ((!efa->h) && (efa->f & SELECT)))
+			totuv += (efa->v4)? 4: 3;
+		
+	if(totuv==0)
+		return NULL;
+	
+	vmap= (UvVertMap*)MEM_callocN(sizeof(*vmap), "UvVertMap");
+	if (!vmap)
+		return NULL;
+
+	vmap->vert= (UvMapVert**)MEM_callocN(sizeof(*vmap->vert)*totverts, "UvMapVert*");
+	buf= vmap->buf= (UvMapVert*)MEM_callocN(sizeof(*vmap->buf)*totuv, "UvMapVert");
+
+	if (!vmap->vert || !vmap->buf) {
+		free_uv_vert_map(vmap);
+		return NULL;
+	}
+
+	for (a=0, efa= em->faces.first; efa; a++, efa= efa->next) {
+		if(!selected || ((!efa->h) && (efa->f & SELECT))) {
+			nverts= (efa->v4)? 4: 3;
+			
+			for(i=0; i<nverts; i++) {
+				buf->tfindex= i;
+				buf->f= a;
+				buf->separate = 0;
+				
+				buf->next= vmap->vert[(*(&efa->v1 + i))->tmp.l];
+				vmap->vert[(*(&efa->v1 + i))->tmp.l]= buf;
+				
+				buf++;
+			}
+		}
+	}
+	
+	/* sort individual uvs for each vert */
+	for(a=0, ev=em->verts.first; ev; a++, ev= ev->next) {
+		UvMapVert *newvlist= NULL, *vlist=vmap->vert[a];
+		UvMapVert *iterv, *v, *lastv, *next;
+		float *uv, *uv2, uvdiff[2];
+
+		while(vlist) {
+			v= vlist;
+			vlist= vlist->next;
+			v->next= newvlist;
+			newvlist= v;
+
+			efa = EM_get_face_for_index(v->f);
+			tf = CustomData_em_get(&em->fdata, efa->data, CD_MTFACE);
+			uv = tf->uv[v->tfindex]; 
+			
+			lastv= NULL;
+			iterv= vlist;
+
+			while(iterv) {
+				next= iterv->next;
+				efa = EM_get_face_for_index(iterv->f);
+				tf = CustomData_em_get(&em->fdata, efa->data, CD_MTFACE);
+				uv2 = tf->uv[iterv->tfindex];
+				
+				Vec2Subf(uvdiff, uv2, uv);
+
+				if(fabs(uv[0]-uv2[0]) < limit[0] && fabs(uv[1]-uv2[1]) < limit[1]) {
+					if(lastv) lastv->next= next;
+					else vlist= next;
+					iterv->next= newvlist;
+					newvlist= iterv;
+				}
+				else
+					lastv=iterv;
+
+				iterv= next;
+			}
+
+			newvlist->separate = 1;
+		}
+
+		vmap->vert[a]= newvlist;
+	}
+	
+	if (do_face_idx_array)
+		EM_free_index_arrays();
+	
+	return vmap;
+}
+
+UvMapVert *get_uv_map_vert_EM(UvVertMap *vmap, unsigned int v)
+{
+	return vmap->vert[v];
+}
+
+void free_uv_vert_map_EM(UvVertMap *vmap)
+{
+	if (vmap) {
+		if (vmap->vert) MEM_freeN(vmap->vert);
+		if (vmap->buf) MEM_freeN(vmap->buf);
+		MEM_freeN(vmap);
+	}
+}
+
