@@ -1,15 +1,12 @@
 /**
  * $Id: 
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 /*
@@ -488,11 +485,11 @@ static CutCurve *get_mouse_trail(int *len, char mode, char cutmode, struct GHash
 {
 	CutCurve *curve,*temp;
 	EditVert *snapvert;
-	float *scr, mval[2], lastx=0, lasty=0;
+	float *scr, mval[2]={0.0,0.0}, lastx=0, lasty=0, lockx=0, locky=0;
 	int i=0, j, blocks=1, lasti=0;
 	int dist, tolerance;
 	short event, val, qual, vsnap=0, ldown=0, restart=0, rubberband=0;
-	short mval1[2], lockaxis=0, lockx=0, locky=0, oldmode; 
+	short mval1[2], lockaxis=0, oldmode; 
 	
 	*len=0;
 	tolerance = 75;
@@ -515,13 +512,12 @@ static CutCurve *get_mouse_trail(int *len, char mode, char cutmode, struct GHash
 			G.scene->selectmode = oldmode;
 		}
 		glDrawBuffer(GL_FRONT);
-		headerprint("LMB to draw, CTRL while drawing for vertex snap. Enter to finish (with CTRL to leave only the "
-					"cut line selected), ESC to abort.");
+		headerprint("(LMB) draw, (Ctrl held while drawing) snap to vertex, (MMB) constrain to x/y screen axis, (Enter) cut "
+					"(with Ctrl to select cut line), (Esc) cancel");
 	}
 	else{
 		glDrawBuffer(GL_FRONT);
-		headerprint("LMB to draw, Enter to finish (with CTRL to leave only the "
-					"cut line selected), ESC to abort.");
+		headerprint("(LMB) draw, (MMB) constrain to x/y screen axis, (Enter) cut (with Ctrl to select cut line), (Esc) cancel");
 	}
 	
 	persp(PERSP_WIN);
