@@ -1,15 +1,12 @@
 /**
- * $Id: editscreen.c 11607 2007-08-15 04:32:40Z bdiego $
+ * $Id: editscreen.c 14669 2008-05-04 09:41:15Z campbellbarton $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  * All screen functions that are related to the interface
  * handling and drawing. Might be split up as well later...
  */
@@ -176,8 +173,9 @@ static int choose_cursor(ScrArea *sa)
 		if(G.obedit) return CURSOR_EDIT;
 		else if(G.f & (G_VERTEXPAINT|G_WEIGHTPAINT|G_TEXTUREPAINT))
 				return CURSOR_VPAINT;
-		else if(G.f & G_FACESELECT) return CURSOR_FACESEL;
+		else if(FACESEL_PAINT_TEST) return CURSOR_FACESEL;
 		else if(G.f & G_SCULPTMODE) return CURSOR_EDIT;
+		else if(G.f & G_PARTICLEEDIT) return CURSOR_EDIT;
 		else return CURSOR_STD;
 	}
 	else if (sa->spacetype==SPACE_TEXT) {
@@ -358,6 +356,11 @@ void areawinset(short win)
 			G.v2d= &snode->v2d;
 		}
 			break;
+		case SPACE_IMASEL:
+		{
+			SpaceImaSel *simasel= curarea->spacedata.first;
+			G.v2d= &simasel->v2d;
+		}
 		default:
 			break;
 		}
@@ -485,6 +488,8 @@ static void addqueue_ext(short win, unsigned short event, short val, char ascii)
 	} 
 	else {
 		BWinEvent evt;
+
+		memset(&evt, 0, sizeof(evt));
 		evt.event= event;
 		evt.val= val;
 		evt.ascii= ascii;
@@ -523,7 +528,7 @@ static void scrarea_dispatch_header_events(ScrArea *sa)
 	
 	while(bwin_qread(sa->headwin, &evt)) {
 		if(evt.val) {
-			if( uiDoBlocks(&curarea->uiblocks, evt.event)!=UI_NOTHING ) evt.event= 0;
+			if( uiDoBlocks(&curarea->uiblocks, evt.event, 1)!=UI_NOTHING ) evt.event= 0;
 
 			switch(evt.event) {
 			case UI_BUT_EVENT:
@@ -885,12 +890,6 @@ int afterqtest(short win, unsigned short evt)
 
 
 static char ext_load_str[256]= {0, 0};
-void add_readfile_event(char *filename)
-{	
-	mainqenter(LOAD_FILE, 1);
-	strcpy(ext_load_str, filename);
-	BLI_convertstringcode(ext_load_str, G.sce, G.scene->r.cfra);
-}
 
 static short ext_reshape= 0, ext_redraw=0, ext_inputchange=0, ext_mousemove=0, ext_undopush=0;
 
@@ -1078,7 +1077,7 @@ int has_screenhandler(bScreen *sc, short eventcode)
 
 static void animated_screen(bScreen *sc, short val)
 {
-	if (U.mixbufsize && (val & TIME_WITH_SEQ_AUDIO)) {
+	if ((val & TIME_WITH_SEQ_AUDIO)) {
 		if(CFRA>=PEFRA) {
 			CFRA= PSFRA;
 			audiostream_stop();
@@ -1114,7 +1113,9 @@ static void animated_screen(bScreen *sc, short val)
 	}
 	if(val & TIME_ALL_ANIM_WIN) allqueue(REDRAWANIM, 0);
 	if(val & TIME_ALL_BUTS_WIN) allqueue(REDRAWBUTSALL, 0);
-	if(val & TIME_SEQ) allqueue(REDRAWSEQ, 0);
+	if(val & TIME_SEQ) {
+		allqueue(REDRAWSEQ, 0);
+	}
 	
 	allqueue(REDRAWTIME, 0);
 }
@@ -1131,7 +1132,7 @@ int do_screenhandlers(bScreen *sc)
 	short a, done= 0;
 	
 	time = PIL_check_seconds_timer();
-	swaptime= 1.0/(float)G.scene->r.frs_sec;
+	swaptime= 1.0/FPS;
 	
 	/* only now do the handlers */
 	if(swaptime < time-ltime || ltime==0.0) {
@@ -1387,9 +1388,14 @@ void screenmain(void)
 			towin= 0;
 		}
 		else if (event==QKEY) {
-			if((G.obedit && G.obedit->type==OB_FONT && g_activearea->spacetype==SPACE_VIEW3D)||g_activearea->spacetype==SPACE_TEXT||g_activearea->spacetype==SPACE_SCRIPT);
+			/* Temp place to print mem debugging info ctrl+alt+shift + qkey */
+			if ( G.qual == (LR_SHIFTKEY | LR_ALTKEY | LR_CTRLKEY) ) {
+				MEM_printmemlist_stats();
+			}
+			
+			else if((G.obedit && G.obedit->type==OB_FONT && g_activearea->spacetype==SPACE_VIEW3D)||g_activearea->spacetype==SPACE_TEXT||g_activearea->spacetype==SPACE_SCRIPT);
 			else {
-				if(val && (G.qual & LR_CTRLKEY)) {
+				if(val && (G.qual == LR_CTRLKEY)) {
 					if(okee("Quit Blender")) exit_usiblender();
 				}
 				towin= 0;
@@ -1400,7 +1406,8 @@ void screenmain(void)
 				bScreen *sc= G.curscreen->id.next;
 
 				/* if screen is last, set it to first */
-				if(sc == NULL) sc= G.main->screen.first;
+				if(sc == NULL) 
+					sc= G.main->screen.first;
 				
 				if(is_allowed_to_change_screen(sc)) setscreen(sc);
 				g_activearea= NULL;
@@ -1412,7 +1419,8 @@ void screenmain(void)
 				bScreen *sc= G.curscreen->id.prev;
 				
 				/* if screen is first, set it to last */
-				if(sc == NULL) sc= G.main->screen.last;
+				if(sc == NULL) 
+					sc= G.main->screen.last;
 				
 				if(is_allowed_to_change_screen(sc)) setscreen(sc);
 				g_activearea= NULL;
@@ -1575,7 +1583,7 @@ void mainwindow_set_filename_to_title(char *filename)
 	char dir[FILE_MAXDIR];
 	char file[FILE_MAXFILE];
 
-	BLI_split_dirfile(filename, dir, file);
+	BLI_split_dirfile_basic(filename, dir, file);
 
 	if(BLI_streq(file, ".B.blend") || filename[0] =='\0')
 		sprintf(str, "Blender");
@@ -2161,6 +2169,25 @@ short get_activedevice(void)
 	return window_get_activedevice(mainwin);
 }
 
+void getndof(float *sbval)
+{
+    winlay_process_events(0);
+    window_get_ndof(mainwin, sbval);
+}
+
+void filterNDOFvalues(float *sbval)
+{
+	int i=0;
+	float max  = 0.0;
+	
+	for (i =0; i<6;i++)
+		if (fabs(sbval[i]) > max)
+			max = fabs(sbval[i]);
+	for (i =0; i<6;i++)
+		if (fabs(sbval[i]) != max )
+			sbval[i]=0.0;
+}
+
 void add_to_mainqueue(Window *win, void *user_data, short evt, short val, char ascii)
 {
 
@@ -2225,6 +2252,7 @@ static bScreen *addscreen(char *name)		/* use setprefsize() if you want somethin
 		}
 		
 		window_set_handler(mainwin, add_to_mainqueue, NULL);
+		window_open_ndof(mainwin); /* needs to occur once the mainwin handler is set */
 		init_mainwin();
 		mywinset(1);
 	
@@ -2431,9 +2459,6 @@ void area_fullscreen(void)	/* with curarea */
 		wich_cursor(newa);
 	}
 	
-	if(curarea->full)
-		retopo_force_update();
-
 	/* there's also events in queue for this, but we call fullscreen for render output
 	now, and that doesn't go back to queue. Bad code, but doesn't hurt... (ton) */
 	for(sa= G.curscreen->areabase.first; sa; sa= sa->next) {
@@ -2443,8 +2468,7 @@ void area_fullscreen(void)	/* with curarea */
 	/* bad code #2: setscreen() ends with first area active. fullscreen render assumes this too */
 	curarea= sc->areabase.first;
 	
-	if(!curarea->full)
-		retopo_force_update();
+	retopo_force_update();
 }
 
 static void area_autoplayscreen(void)
@@ -3067,7 +3091,7 @@ static void splitarea_interactive(ScrArea *area, ScrEdge *onedge)
 	ScrArea *scr, *sa= area;
 	float fac= 0.0;
 	unsigned short event;
-	short ok= 0, val, split = 0, mval[2], mvalo[2], first= 1;
+	short ok= 0, val, split = 0, mval[2], mvalo[2]= {-1, -1}, first= 1;
 	char dir;
 	
 	if(sa->win==0) return;
