@@ -1,15 +1,12 @@
 /**
- * $Id: ghostwinlay.c,v 1.48 2007/04/12 06:21:31 djcapelis Exp $
+ * $Id: ghostwinlay.c 14673 2008-05-04 11:00:33Z campbellbarton $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,10 +24,11 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -55,14 +53,21 @@
 #include "BIF_usiblender.h"
 #include "BIF_cursors.h"
 
+#include "PIL_dynlib.h"
+
 #include "mydevice.h"
 #include "blendef.h"
 
 #include "winlay.h"
 
+#include <math.h>
+
+
 #ifdef __APPLE__
 #include <OpenGL/OpenGL.h>
 #define __CARBONSOUND__
+  /* XXX BIG WARNING: carbon.h should not be included in blender/src code, it conflicts with struct ID */
+#define ID ID_
 #include <Carbon/Carbon.h>
 
 /*declarations*/
@@ -107,6 +112,12 @@ struct _Window {
 	 */
 	int		faked_mbut;
 
+		/* Last known ndof device state
+         * note that the ghost device manager 
+         * can handle any number of devices, but ghostwinlay can't
+         */
+    float   ndof[7];    /* tx, ty, tz, rx, ry, rz, dt */
+
 	GHOST_TimerTaskHandle	timer;
 	int						timer_event;
 };
@@ -334,6 +345,7 @@ Window *window_open(char *title, int posx, int posy, int sizex, int sizey, int s
 	GHOST_WindowHandle ghostwin;
 	GHOST_TWindowState inital_state;
 	int scr_w, scr_h;
+    int i;
 
 	winlay_get_screensize(&scr_w, &scr_h);
 	posy= (scr_h-posy-sizey);
@@ -366,7 +378,10 @@ Window *window_open(char *title, int posx, int posy, int sizex, int sizey, int s
 			
 			win->lmouse[0]= win->size[0]/2;
 			win->lmouse[1]= win->size[1]/2;
-			
+
+            for (i = 0; i < 7; ++i)
+                win->ndof[i] = 0;
+		
 			
 		} else {
 			GHOST_DisposeWindow(g_system, ghostwin);
@@ -494,7 +509,7 @@ static int change_bit(int val, int bit, int to_on) {
 	return to_on?(val|bit):(val&~bit);
 }
 
-static void update_tablet_data(Window *win, Window *ghostwin) {
+static void update_tablet_data(Window *win, GHOST_WindowHandle ghostwin) {
 	const GHOST_TabletData *td= GHOST_GetTabletData(ghostwin);
 	
 	/* if there's tablet data from an active tablet device then use it,
@@ -506,7 +521,7 @@ static void update_tablet_data(Window *win, Window *ghostwin) {
 		win->ytilt = td->Ytilt;
 	} else {
 		win->activedevice = DEV_MOUSE;
-		win->pressure = 0.0;
+		win->pressure = 1.0;
 		win->xtilt = win->ytilt = 0.0;
 	}
 }
@@ -538,6 +553,46 @@ static int event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private)
 		}
 		
 		switch (type) {
+
+        case GHOST_kEventNDOFMotion: {
+            // update ndof device data, and dispatch motion event
+
+            GHOST_TEventNDOFData *sb= data;
+			
+				// no scaling per sfgoros patch
+            win->ndof[0] = sb->tx;
+            win->ndof[1] = sb->ty;
+            win->ndof[2] = sb->tz;
+            win->ndof[3] = sb->rx;
+            win->ndof[4] = sb->ry;
+            win->ndof[5] = sb->rz;
+            win->ndof[6] = sb->delta;
+ //        	printf(" motion capted %f %f %f %f %f %f %f \n", win->ndof[0], win->ndof[1], win->ndof[2],
+ //        							 win->ndof[3], win->ndof[4], win->ndof[5], win->ndof[6]);
+
+
+ //          window_handle(win, NDOFMOTION, win->ndof[6]);
+
+ //       start interaction for larger than teeny-tiny motions
+ //         if (fabs(win->ndof[0] > 0.003f) ||
+ //             fabs(win->ndof[1] > 0.003f) ||
+ //          fabs(win->ndof[2] > 0.003f) ||
+ //               fabs(win->ndof[3] > 0.003f) ||
+ //               fabs(win->ndof[4] > 0.003f) ||
+ //              fabs(win->ndof[5] > 0.003f)) {
+                    window_handle(win, NDOFMOTION, 1);
+ //    printf("ok\n");
+ //    }
+;
+          break;
+        }
+        case GHOST_kEventNDOFButton: {
+			GHOST_TEventNDOFData *sb= data;
+			
+//			printf("this is a button %i\n", sb->buttons);
+			window_handle(win, NDOFBUTTON, sb->buttons);
+			break;
+        }
 		case GHOST_kEventCursorMove: {
 			if(win->active == 1) {
 				GHOST_TEventCursorData *cd= data;
@@ -705,6 +760,13 @@ static int event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private)
 	return 1;
 }
 
+void window_get_ndof(Window* win, float* sbval) {
+    int i;
+    for (i = 0; i < 7; ++i) {
+        *sbval++ = win->ndof[i];
+    }
+}
+
 char *window_get_title(Window *win) {
 	char *title= GHOST_GetTitle(win->ghostwin);
 	char *mem_title= BLI_strdup(title);
@@ -831,3 +893,100 @@ void winlay_get_screensize(int *width_r, int *height_r) {
 Window *winlay_get_active_window(void) {
 	return active_gl_window;
 }
+
+#ifdef _WIN32
+#define PATH_SEP		"\\"
+#else
+#define PATH_SEP		"/"
+#endif
+
+
+void window_open_ndof(Window* win)
+{
+	char *inst_path, *plug_path;
+	const char *plug_dir = "plugins";
+	const char *plug_name = "3DxNdofBlender.plug";
+	PILdynlib *ndofLib;
+	
+	// build the plugin path
+	plug_path = NULL;
+	inst_path = get_install_dir(); // path to main blender exec/bundle
+	if (inst_path) {
+		// assume the ndof plugin is located in the plug-in dir
+		size_t len = strlen(inst_path) + strlen(plug_dir) + strlen(PATH_SEP)*2
+		             + strlen(plug_name) + 1;
+		plug_path = MEM_mallocN(len, "ndofpluginpath");
+		if (plug_path) {
+			strncpy(plug_path, inst_path, len);
+			strcat(plug_path, PATH_SEP);
+			strcat(plug_path, plug_dir);
+			strcat(plug_path, PATH_SEP);
+			strcat(plug_path, plug_name);
+		}
+		MEM_freeN(inst_path);
+	}
+	
+	ndofLib	= PIL_dynlib_open(plug_path);
+	
+	/* On systems where blender is installed in /usr/bin/blender, ~/.blender/plugins/ is a better place to look */
+	if (ndofLib==NULL) {
+		
+		if (plug_path) {
+			MEM_freeN(plug_path);
+		}
+		
+		inst_path = BLI_gethome();
+		if (inst_path) {
+			size_t len = strlen(inst_path) + strlen(plug_dir) + strlen(PATH_SEP)*2
+					 + strlen(plug_name) + 1;
+			
+			if (!strstr(inst_path, ".blender")) {
+				len += strlen(".blender") + strlen(PATH_SEP);
+			}
+			
+			plug_path = MEM_mallocN(len, "ndofpluginpath");
+			if (plug_path) {
+				strncpy(plug_path, inst_path, len);
+				strcat(plug_path, PATH_SEP);
+				if (!strstr(inst_path, ".blender")) {
+					strcat(plug_path, ".blender");
+					strcat(plug_path, PATH_SEP);
+				}
+				strcat(plug_path, plug_dir);
+				strcat(plug_path, PATH_SEP);
+				strcat(plug_path, plug_name);
+			}
+		}
+		
+		ndofLib	= PIL_dynlib_open(plug_path);
+	}
+	
+	
+	
+#if 0
+	fprintf(stderr, "plugin path=%s; ndofLib=%p\n", plug_path, (void*)ndofLib);
+#endif
+	
+	if (plug_path)
+		MEM_freeN(plug_path);
+	
+	if (ndofLib) {
+		G.ndofdevice = 0 - GHOST_OpenNDOF(g_system, win->ghostwin, 
+		               PIL_dynlib_find_symbol(ndofLib, "ndofInit"),
+		               PIL_dynlib_find_symbol(ndofLib, "ndofShutdown"),
+		               PIL_dynlib_find_symbol(ndofLib, "ndofOpen"));
+		
+		}
+    else {
+        GHOST_OpenNDOF(g_system, win->ghostwin, 0, 0, 0);
+        G.ndofdevice = -1;
+    }
+ }
+
+char *getClipboard(int flag) {
+	return (char*)GHOST_getClipboard(flag);
+}
+
+void putClipboard(char *buffer, int flag) {
+	GHOST_putClipboard((GHOST_TInt8*)buffer, flag);
+}
