@@ -1,15 +1,12 @@
 /**
- * $Id: hddaudio.c,v 1.9 2007/01/07 19:22:22 schlaile Exp $
+ * $Id: hddaudio.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,7 +19,7 @@
  *
  * Contributor(s): Peter Schlaile <peter [at] schlaile [dot] de> 2005
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <stdio.h>
@@ -200,7 +197,7 @@ struct hdaudio * sound_copy_hdaudio(struct hdaudio * c)
 #endif
 }
 
-long sound_hdaudio_get_duration(struct hdaudio * hdaudio, int frame_rate)
+long sound_hdaudio_get_duration(struct hdaudio * hdaudio, double frame_rate)
 {
 #ifdef WITH_FFMPEG
 	return hdaudio->pFormatCtx->duration * frame_rate / AV_TIME_BASE;
@@ -283,58 +280,61 @@ static void sound_hdaudio_extract_small_block(
 		
 		decode_pos -= bl_size;
 
-		memset(hdaudio->decode_cache + decode_pos, 0,
-		       (hdaudio->decode_cache_size - decode_pos) 
-		       * sizeof(short));
-		       
+		if (decode_pos < hdaudio->decode_cache_size) {
+			memset(hdaudio->decode_cache + decode_pos, 0,
+			       (hdaudio->decode_cache_size - decode_pos) 
+			       * sizeof(short));
 
-		while(av_read_frame(hdaudio->pFormatCtx, &packet) >= 0) {
-			int data_size;
-			int len;
-			uint8_t *audio_pkt_data;
-			int audio_pkt_size;
-
-			if(packet.stream_index != hdaudio->audioStream) {
-				av_free_packet(&packet);
-				continue;
-			}
-
-			audio_pkt_data = packet.data;
-			audio_pkt_size = packet.size;
-
-			while (audio_pkt_size > 0) {
-				len = avcodec_decode_audio(
-					hdaudio->pCodecCtx, 
-					hdaudio->decode_cache 
-					+ decode_pos, 
-					&data_size, 
-					audio_pkt_data, 
-					audio_pkt_size);
-				if (len <= 0) {
-					audio_pkt_size = 0;
-					break;
-				}
+			while(av_read_frame(
+				      hdaudio->pFormatCtx, &packet) >= 0) {
+				int data_size;
+				int len;
+				uint8_t *audio_pkt_data;
+				int audio_pkt_size;
 				
-				audio_pkt_size -= len;
-				audio_pkt_data += len;
-
-				if (data_size <= 0) {
+				if(packet.stream_index 
+				   != hdaudio->audioStream) {
+					av_free_packet(&packet);
 					continue;
 				}
+
+				audio_pkt_data = packet.data;
+				audio_pkt_size = packet.size;
+
+				while (audio_pkt_size > 0) {
+					len = avcodec_decode_audio(
+						hdaudio->pCodecCtx, 
+						hdaudio->decode_cache 
+						+ decode_pos, 
+						&data_size, 
+						audio_pkt_data, 
+						audio_pkt_size);
+					if (len <= 0) {
+						audio_pkt_size = 0;
+						break;
+					}
+					
+					audio_pkt_size -= len;
+					audio_pkt_data += len;
+					
+					if (data_size <= 0) {
+						continue;
+					}
+					
+					decode_pos += data_size / sizeof(short);
+					if (decode_pos + data_size
+					    / sizeof(short)
+					    > hdaudio->decode_cache_size) {
+						break;
+					}
+				}
+				av_free_packet(&packet);
 				
-				decode_pos += data_size / sizeof(short);
-				if (decode_pos + data_size
-				    / sizeof(short)
+				if (decode_pos + data_size / sizeof(short)
 				    > hdaudio->decode_cache_size) {
 					break;
 				}
 			}
-			av_free_packet(&packet);
-
-			if (decode_pos + data_size / sizeof(short)
-			    > hdaudio->decode_cache_size) {
-				break;
-			}
 		}
 
 		if (rate_conversion) {
