@@ -4,17 +4,14 @@
  * Functions to draw the "UV/Image Editor" window header
  * and handle user events sent to it.
  * 
- * $Id: header_image.c 10690 2007-05-08 19:31:39Z bebraw $
+ * $Id: header_image.c 14824 2008-05-13 06:12:33Z campbellbarton $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,7 +29,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <stdlib.h>
@@ -50,7 +47,9 @@
 #include "DNA_space_types.h"
 #include "DNA_texture_types.h"
 #include "DNA_userdef_types.h"
+#include "DNA_customdata_types.h" /* for UV layer menu */
 
+#include "BLI_arithb.h"
 #include "BLI_blenlib.h"
 
 #include "BDR_drawmesh.h"
@@ -62,21 +61,26 @@
 #include "BKE_image.h"
 #include "BKE_main.h"
 #include "BKE_utildefines.h"
+#include "BLI_editVert.h" /* for UV layer menu */
+#include "BKE_customdata.h" /* ditto */
 
 #include "BIF_butspace.h"
 #include "BIF_drawimage.h"
 #include "BIF_editsima.h"
+#include "BIF_imasel.h"
 #include "BIF_interface.h"
 #include "BIF_resources.h"
 #include "BIF_screen.h"
 #include "BIF_space.h"
 #include "BIF_transform.h"
 #include "BIF_toolbox.h"
+#include "BIF_editmesh.h"
 
 #include "BSE_drawview.h"
 #include "BSE_filesel.h"
 #include "BSE_headerbuttons.h"
 #include "BSE_trans_types.h"
+#include "BSE_edit.h"
 
 #include "BPY_extern.h"
 #include "BPY_menus.h"
@@ -104,14 +108,22 @@ void do_image_buttons(unsigned short event)
 	}
 	
 	switch(event) {
+	case B_SIMAPIN:
+		allqueue (REDRAWIMAGE, 0);
+		break;
 	case B_SIMAGEHOME:
 		image_home();
 		break;
 
 	case B_SIMABROWSE:	
 		if(G.sima->imanr== -2) {
-			activate_databrowse((ID *)G.sima->image, ID_IM, 0, B_SIMABROWSE,
+			if(G.qual & LR_CTRLKEY) {
+				activate_databrowse_imasel((ID *)G.sima->image, ID_IM, 0, B_SIMABROWSE,
 											&G.sima->imanr, do_image_buttons);
+			} else {
+				activate_databrowse((ID *)G.sima->image, ID_IM, 0, B_SIMABROWSE,
+											&G.sima->imanr, do_image_buttons);
+			}
 			return;
 		}
 		if(G.sima->imanr < 0) break;
@@ -131,31 +143,22 @@ void do_image_buttons(unsigned short event)
 			allqueue(REDRAWIMAGE, 0);
 		}
 		/* also when image is the same: assign! 0==no tileflag: */
-		image_changed(G.sima, 0);
+		image_changed(G.sima, (Image *)idtest);
 		BIF_undo_push("Assign image UV");
 
 		break;
-		
-	case B_SIMAGEDRAW:
-		if(G.f & G_FACESELECT) {
-			make_repbind(G.sima->image);
-			image_changed(G.sima, 1);
-		}
-		/* XXX might be another event needed for this? */
-		if(G.sima->image)
-			if(ELEM(G.sima->image->source, IMA_SRC_MOVIE, IMA_SRC_SEQUENCE))
-				if(G.sima->iuser.flag & IMA_ANIM_ALWAYS)
-					BKE_image_user_calc_imanr(&G.sima->iuser, G.scene->r.cfra, 0);
+	case B_SIMAGETILE:
+		image_set_tile(G.sima, 1);		/* 1: only tileflag */
 		allqueue(REDRAWVIEW3D, 0);
 		allqueue(REDRAWIMAGE, 0);
 		break;
-		
-	case B_SIMAGEDRAW1:
-		image_changed(G.sima, 2);		/* 2: only tileflag */
+	case B_SIMA3DVIEWDRAW:
+		allqueue(REDRAWVIEW3D, 0);
+		break;
+	case B_SIMA_REDR_IMA_3D:
 		allqueue(REDRAWVIEW3D, 0);
 		allqueue(REDRAWIMAGE, 0);
 		break;
-		
 	case B_SIMAGEPAINTTOOL:
 		if(G.sima->flag & SI_DRAWTOOL)
 			/* add new brush if none exists */
@@ -204,12 +207,8 @@ void do_image_buttons(unsigned short event)
 			BLI_strncpy(str, G.sima->image->name, sizeof(str));
 			ima= BKE_add_image_file(str);
 			if(ima) {
-				
-				G.sima->image= ima;
-				
 				BKE_image_signal(ima, &G.sima->iuser, IMA_SIGNAL_RELOAD);
-				image_changed(G.sima, 0);
-				
+				image_changed(G.sima, ima);
 			}
 			BIF_undo_push("Load image");
 			allqueue(REDRAWIMAGE, 0);
@@ -224,6 +223,9 @@ void do_image_buttons(unsigned short event)
 	case B_TRANS_IMAGE:
 		image_editvertex_buts(NULL);
 		break;
+	case B_CURSOR_IMAGE:
+		image_editcursor_buts(NULL);
+		break;
 		
 	case B_TWINANIM:
 	{
@@ -291,7 +293,7 @@ void do_image_buttons(unsigned short event)
 		}
 		break;
 	case B_KEEPDATA:
-		brush_toggle_fake_user(settings->imapaint.brush);
+		brush_toggled_fake_user(settings->imapaint.brush);
 		allqueue(REDRAWIMAGE, 0);
 		allqueue(REDRAWBUTSEDIT, 0);
 		break;
@@ -311,7 +313,11 @@ void do_image_buttons(unsigned short event)
 			if(G.sima->menunr==-2) {
 				MTex *mtex= brush->mtex[brush->texact];
 				ID *id= (ID*)((mtex)? mtex->tex: NULL);
-				activate_databrowse(id, ID_TE, 0, B_SIMABTEXBROWSE, &G.sima->menunr, do_global_buttons);
+				if(G.qual & LR_CTRLKEY) {
+					activate_databrowse_imasel(id, ID_TE, 0, B_SIMABTEXBROWSE, &G.sima->menunr, do_image_buttons);
+				} else {
+					activate_databrowse(id, ID_TE, 0, B_SIMABTEXBROWSE, &G.sima->menunr, do_image_buttons);
+				}
 				break;
 			}
 			else if(G.sima->menunr < 0) break;
@@ -343,6 +349,16 @@ void do_image_buttons(unsigned short event)
 	}
 }
 
+static void do_image_buttons_set_uvlayer_callback(void *act, void *data)
+{
+	CustomData_set_layer_active(&G.editMesh->fdata, CD_MTFACE, *((int *)act));
+	
+	BIF_undo_push("Set Active UV Texture");
+	allqueue(REDRAWVIEW3D, 0);
+	allqueue(REDRAWBUTSEDIT, 0);
+	allqueue(REDRAWIMAGE, 0);
+}
+
 static void do_image_view_viewnavmenu(void *arg, int event)
 {
 	switch(event) {
@@ -418,19 +434,8 @@ static void do_image_viewmenu(void *arg, int event)
 	case 2: /* Maximize Window */
 		/* using event B_FULL */
 		break;
-	case 5: /* Draw Shadow Mesh */
-		if(G.sima->flag & SI_DRAWSHADOW)
-			G.sima->flag &= ~SI_DRAWSHADOW;
-		else
-			G.sima->flag |= SI_DRAWSHADOW;
-		allqueue(REDRAWIMAGE, 0);
-		break;
-	case 6: /* Draw Faces */
-		if(G.f & G_DRAWFACES)
-			G.f &= ~G_DRAWFACES;
-		else
-			G.f |= G_DRAWFACES;
-		allqueue(REDRAWIMAGE, 0);
+	case 4: /* Realtime Panel... */
+		add_blockhandler(curarea, IMAGE_HANDLER_VIEW_PROPERTIES, UI_PNL_UNSTOW);
 		break;
 	case 7: /* Properties  Panel */
 		add_blockhandler(curarea, IMAGE_HANDLER_PROPERTIES, UI_PNL_UNSTOW);
@@ -440,9 +445,6 @@ static void do_image_viewmenu(void *arg, int event)
 		break;
 	case 9:
 		image_viewcenter();
-	case 10: /* Display Normalized Coordinates */
-		G.sima->flag ^= SI_COORDFLOATS;
-		allqueue(REDRAWIMAGE, 0);
 		break;
 	case 11: /* Curves Panel... */
 		add_blockhandler(curarea, IMAGE_HANDLER_CURVES, UI_PNL_UNSTOW);
@@ -454,7 +456,10 @@ static void do_image_viewmenu(void *arg, int event)
 	case 13: /* Realtime Panel... */
 		add_blockhandler(curarea, IMAGE_HANDLER_GAME_PROPERTIES, UI_PNL_UNSTOW);
 		break;
-		
+	case 14: /* Draw active image UV's only*/
+		G.sima->flag ^= SI_LOCAL_UV;
+		allqueue(REDRAWIMAGE, 0);
+		break;
 	}
 	allqueue(REDRAWVIEW3D, 0);
 }
@@ -468,20 +473,22 @@ static uiBlock *image_viewmenu(void *arg_unused)
 	block= uiNewBlock(&curarea->uiblocks, "image_viewmenu", UI_EMBOSSP, UI_HELV, curarea->headwin);
 	uiBlockSetButmFunc(block, do_image_viewmenu, NULL);
 
-	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "Properties...",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "View Properties...",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 4, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "Image Properties...|N",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "Real-time Properties...",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 13, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "Paint Tool...|C",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 8, "");
+	if (G.sima->image && (G.sima->flag & SI_DRAWTOOL)) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "Paint Tool...|C",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 8, "");
+	}
 	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "Curves Tool...",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 11, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "Composite Preview...|Shift P",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 12, "");
-
-	if(G.sima->flag & SI_COORDFLOATS) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Display Normalized Coordinates|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 10, "");
-	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Display Normalized Coordinates|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 10, "");
-	if(G.f & G_DRAWFACES) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Draw Faces", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 6, "");
-	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Draw Faces|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 6, "");
-	if(G.sima->flag & SI_DRAWSHADOW) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Draw Shadow Mesh", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
-	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Draw Shadow Mesh|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
-
+	
+	if(G.sima->flag & SI_LOCAL_UV) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "UV Local View|NumPad /",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 14, "");
+	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "UV Local View|NumPad /", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 14, "");
+	if(!(G.sima->flag & SI_LOCAL_UV)) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "UV Global View|NumPad /",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 14, "");
+	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "UV Global View|NumPad /",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 14, "");
+	
 	uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
+	
 	uiDefIconTextBlockBut(block, image_view_viewnavmenu, NULL, ICON_RIGHTARROW_THIN, "View Navigation", 0, yco-=20, 120, 19, "");
 
 	if(G.sima->lock) {
@@ -524,32 +531,15 @@ static void do_image_selectmenu(void *arg, int event)
 	case 1: /* Select/Deselect All */
 		select_swap_tface_uv();
 		break;
+	case 9: /* Select Inverse */
+		select_invert_tface_uv();
+		break;
 	case 2: /* Unlink Selection */
 		unlink_selection();
 		break;
 	case 3: /* Linked UVs */
 		select_linked_tface_uv(2);
 		break;
-	case 4: /* Toggle Local UVs Stick to Vertex in Mesh */
-		if(G.sima->flag & SI_LOCALSTICKY)
-			G.sima->flag &= ~SI_LOCALSTICKY;
-		else {
-			G.sima->flag |= SI_LOCALSTICKY;
-			G.sima->flag &= ~SI_STICKYUVS;
-		}
-		allqueue(REDRAWIMAGE, 0);
-		break;  
-	case 5: /* Toggle UVs Stick to Vertex in Mesh */
-		if(G.sima->flag & SI_STICKYUVS) {
-			G.sima->flag &= ~SI_STICKYUVS;
-			G.sima->flag |= SI_LOCALSTICKY;
-		}
-		else {
-			G.sima->flag |= SI_STICKYUVS;
-			G.sima->flag &= ~SI_LOCALSTICKY;
-		}
-		allqueue(REDRAWIMAGE, 0);
-		break;  
 	case 6: /* Toggle Active Face Select */
 		if(G.sima->flag & SI_SELACTFACE)
 			G.sima->flag &= ~SI_SELACTFACE;
@@ -570,20 +560,15 @@ static uiBlock *image_selectmenu(void *arg_unused)
 
 	block= uiNewBlock(&curarea->uiblocks, "image_selectmenu", UI_EMBOSSP, UI_HELV, curarea->headwin);
 	uiBlockSetButmFunc(block, do_image_selectmenu, NULL);
-
-	if(G.sima->flag & SI_SELACTFACE) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Active Face Select|C", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 6, "");
-	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Active Face Select|C", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 6, "");
-
-	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
-
-	if(G.sima->flag & SI_LOCALSTICKY) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Stick Local UVs to Mesh Vertex|Shift C", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 4, "");
-	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Stick Local UVs to Mesh Vertex|Shift C", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 4, "");
-
-	if(G.sima->flag & SI_STICKYUVS) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Stick UVs to Mesh Vertex|Ctrl C", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
-	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Stick UVs to Mesh Vertex|Ctrl C", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
-
-	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
-
+	
+	
+	if ((G.sima->flag & SI_SYNC_UVSEL)==0 || (G.sima->flag & SI_SYNC_UVSEL && (G.scene->selectmode != SCE_SELECT_FACE))) {
+		if(G.sima->flag & SI_SELACTFACE) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Active Face Select|C", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 6, "");
+		else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Active Face Select|C", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 6, "");
+	
+		uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
+	}
+	
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Border Select|B", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 0, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Border Select Pinned|Shift B", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 8, "");
 
@@ -591,6 +576,8 @@ static uiBlock *image_selectmenu(void *arg_unused)
 
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Select/Deselect All|A", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 1, "");
 
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Inverse|Ctrl I", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 9, "");
+
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Unlink Selection|Alt L", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 2, "");
 	
 	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
@@ -864,12 +851,20 @@ static uiBlock *image_uvs_transformmenu(void *arg_unused)
 
 static void do_image_uvs_mirrormenu(void *arg, int event)
 {
+	float mat[3][3];
+	
+	Mat3One(mat);
+	
 	switch(event) {
 	case 0: /* X axis */
-		mirror_tface_uv('x');
+		initTransform(TFM_MIRROR, CTX_NO_PET|CTX_AUTOCONFIRM);
+		BIF_setSingleAxisConstraint(mat[0], " on global X axis");
+		Transform();
 		break;
 	case 1: /* Y axis */
-		mirror_tface_uv('y');
+		initTransform(TFM_MIRROR, CTX_NO_PET|CTX_AUTOCONFIRM);
+		BIF_setSingleAxisConstraint(mat[1], " on global Y axis");
+		Transform();
 		break;
 	}
 	
@@ -898,16 +893,19 @@ static void do_image_uvs_weldalignmenu(void *arg, int event)
 	case 0: /* Weld */
 		weld_align_tface_uv('w');
 		break;
-	case 1: /* Align X */
+	case 1: /* Align Auto */
+		weld_align_tface_uv('a');
+		break;
+	case 2: /* Align X */
 		weld_align_tface_uv('x');
 		break;
-	case 2: /* Align Y */
+	case 3: /* Align Y */
 		weld_align_tface_uv('y');
 		break;
 	}
 	
 	if(event==0) BIF_undo_push("Weld UV");
-	else if(event==1 || event==2) BIF_undo_push("Align UV");
+	else if(ELEM3(event, 1, 2, 3)) BIF_undo_push("Align UV");
 }
 
 static uiBlock *image_uvs_weldalignmenu(void *arg_unused)
@@ -919,8 +917,9 @@ static uiBlock *image_uvs_weldalignmenu(void *arg_unused)
 	uiBlockSetButmFunc(block, do_image_uvs_weldalignmenu, NULL);
 	
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Weld|W, 1", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 0, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Align X|W, 2", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 1, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Align Y|W, 3", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 2, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Align Auto|W, 2", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 1, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Align X|W, 3", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 2, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Align Y|W, 4", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 3, "");
 
 	uiBlockSetDirection(block, UI_RIGHT);
 	uiTextBoundsBlock(block, 60);
@@ -962,6 +961,9 @@ static void do_image_uvsmenu(void *arg, int event)
 {
 
 	switch(event) {
+//	case 0: /* UV Transform Properties Panel... */
+//		add_blockhandler(curarea, IMAGE_HANDLER_TRANSFORM_PROPERTIES, UI_PNL_UNSTOW);
+//		break;
 	case 1: /* UVs Constrained Rectangular */
 		if(G.sima->flag & SI_BE_SQUARE) G.sima->flag &= ~SI_BE_SQUARE;
 		else G.sima->flag |= SI_BE_SQUARE;
@@ -971,10 +973,10 @@ static void do_image_uvsmenu(void *arg, int event)
 		else G.sima->flag |= SI_CLIP_UV;
 		break;
 	case 3: /* Limit Stitch UVs */
-		stitch_uv_tface(1);
+		stitch_limit_uv_tface();
 		break;
 	case 4: /* Stitch UVs */
-		stitch_uv_tface(0);
+		stitch_vert_uv_tface();
 		break;
 	case 5: /* Proportional Edit (toggle) */
 		if(G.scene->proportional)
@@ -1004,6 +1006,9 @@ static void do_image_uvsmenu(void *arg, int event)
 	case 13:
 		pack_charts_tface_uv();
 		break;
+	case 14:
+		average_charts_tface_uv();
+		break;
 	}
 }
 
@@ -1014,9 +1019,9 @@ static uiBlock *image_uvsmenu(void *arg_unused)
 
 	block= uiNewBlock(&curarea->uiblocks, "image_uvsmenu", UI_EMBOSSP, UI_HELV, curarea->headwin);
 	uiBlockSetButmFunc(block, do_image_uvsmenu, NULL);
-
-	// uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Transform Properties...|N", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 0, "");
-	// uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
+	
+	//uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "Transform Properties...|N", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 0, "");
+	uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 	if(G.sima->flag & SI_PIXELSNAP) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Snap to Pixels|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
 	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Snap to Pixels|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
 
@@ -1036,7 +1041,8 @@ static uiBlock *image_uvsmenu(void *arg_unused)
 
 	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
 
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Pack Charts|Ctrl P", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 13, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Pack Islands|Ctrl P", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 13, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Average Islands Scale|Ctrl A", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 14, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Minimize Stretch|Ctrl V", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 12, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Limit Stitch...|Shift V", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 3, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Stitch|V", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 4, "");
@@ -1084,8 +1090,15 @@ void image_buttons(void)
 	uiBlock *block;
 	short xco, xmax;
 	char naam[256], *menuname;
+	char is_render; /* true if the image is a render or composite */
+	
+	int allow_pin= B_SIMAPIN;
+	
 	/* This should not be a static var */
 	static int headerbuttons_packdummy;
+	
+	
+	is_render = ((G.sima->image!=NULL) && ((G.sima->image->type == IMA_TYPE_R_RESULT) || (G.sima->image->type == IMA_TYPE_COMPOSITE)));
 
 	headerbuttons_packdummy = 0;
 		
@@ -1127,7 +1140,7 @@ void image_buttons(void)
 		uiDefPulldownBut(block, image_viewmenu, NULL, "View", xco, -2, xmax-3, 24, "");
 		xco+= xmax;
 		
-		if((G.f & G_FACESELECT) && !(ima && (G.sima->flag & SI_DRAWTOOL))) {
+		if((EM_texFaceCheck()) && !(ima && (G.sima->flag & SI_DRAWTOOL))) {
 			xmax= GetButStringLength("Select");
 			uiDefPulldownBut(block, image_selectmenu, NULL, "Select", xco, -2, xmax-3, 24, "");
 			xco+= xmax;
@@ -1140,8 +1153,7 @@ void image_buttons(void)
 		xmax= GetButStringLength(menuname);
 		uiDefPulldownBut(block, image_imagemenu, NULL, menuname, xco, -2, xmax-3, 24, "");
 		xco+= xmax;
-		
-		if((G.f & G_FACESELECT) && !(ima && (G.sima->flag & SI_DRAWTOOL))) {
+		if((EM_texFaceCheck()) && !(ima && (G.sima->flag & SI_DRAWTOOL))) {
 			xmax= GetButStringLength("UVs");
 			uiDefPulldownBut(block, image_uvsmenu, NULL, "UVs", xco, -2, xmax-3, 24, "");
 			xco+= xmax;
@@ -1151,8 +1163,107 @@ void image_buttons(void)
 	/* other buttons: */
 	uiBlockSetEmboss(block, UI_EMBOSS);
 
-	xco= std_libbuttons(block, xco, 0, 0, NULL, B_SIMABROWSE, ID_IM, 0, (ID *)ima, 0, &(G.sima->imanr), 0, 0, B_IMAGEDELETE, 0, 0);
+	if (is_render)
+		allow_pin = 0;
+	
+	xco= 8 + std_libbuttons(block, xco, 0, allow_pin, &G.sima->pin, B_SIMABROWSE, ID_IM, 0, (ID *)ima, 0, &(G.sima->imanr), 0, 0, B_IMAGEDELETE, 0, 0);
+	
+	if( ima && !ELEM3(ima->source, IMA_SRC_SEQUENCE, IMA_SRC_MOVIE, IMA_SRC_VIEWER) && ima->ok) {
 
+		if (ima->packedfile) {
+			headerbuttons_packdummy = 1;
+		}
+		if (ima->packedfile && ibuf && (ibuf->userflags & IB_BITMAPDIRTY))
+			uiDefIconButBitI(block, TOG, 1, B_SIMA_REPACK, ICON_UGLYPACKAGE,	xco,0,XIC,YIC, &headerbuttons_packdummy, 0, 0, 0, 0, "Re-Pack this image as PNG");
+		else
+			uiDefIconButBitI(block, TOG, 1, B_SIMAPACKIMA, ICON_PACKAGE,	xco,0,XIC,YIC, &headerbuttons_packdummy, 0, 0, 0, 0, "Pack/Unpack this image");
+			
+		xco+= XIC+8;
+	}
+	
+	/* UV EditMode buttons, not painting or rencering or compositing */
+	if ( EM_texFaceCheck() && (G.sima->flag & SI_DRAWTOOL)==0 && !is_render) {
+		uiBut *ubut;
+		int layercount;
+		
+		uiDefIconTextButS(block, ICONTEXTROW, B_NOP, ICON_ROTATE,
+				"Pivot: %t|Bounding Box Center %x0|Median Point %x3|2D Cursor %x1",
+				xco,0,XIC+10,YIC, &(G.v2d->around), 0, 3.0, 0, 0,
+				"Rotation/Scaling Pivot (Hotkeys: Comma, Shift Comma, Period)");
+		xco+= XIC + 18;
+		
+		
+		uiDefIconButBitI(block, TOG, SI_SYNC_UVSEL, B_REDR, ICON_EDIT, xco,0,XIC,YIC, &G.sima->flag, 0, 0, 0, 0, "Sync UV and Mesh Selection");
+		xco+= XIC+8;
+		if (G.sima->flag & SI_SYNC_UVSEL) {
+			uiBlockBeginAlign(block);
+			
+			/* B_SEL_VERT & B_SEL_FACE are not defined here which is a bit bad, BUT it works even if image editor is fullscreen */
+			uiDefIconButBitS(block, TOG, SCE_SELECT_VERTEX, B_SEL_VERT, ICON_VERTEXSEL, xco,0,XIC,YIC, &G.scene->selectmode, 1.0, 0.0, 0, 0, "Vertex select mode (Ctrl Tab 1)");
+			xco+= XIC;
+			/* no edge */
+			/*uiDefIconButBitS(block, TOG, SCE_SELECT_EDGE, B_SEL_EDGE, ICON_EDGESEL, xco,0,XIC,YIC, &G.scene->selectmode, 1.0, 0.0, 0, 0, "Edge select mode (Ctrl Tab 2)");
+			xco+= XIC; */
+			uiDefIconButBitS(block, TOG, SCE_SELECT_FACE, B_SEL_FACE, ICON_FACESEL, xco,0,XIC,YIC, &G.scene->selectmode, 1.0, 0.0, 0, 0, "Face select mode (Ctrl Tab 3)");
+			xco+= XIC+8;
+			uiBlockEndAlign(block);
+			
+		} else {
+			uiBlockBeginAlign(block);
+			uiDefIconButBitI(block, TOGN, SI_SELACTFACE, B_REDR, ICON_VERTEXSEL, xco,0,XIC,YIC, &G.sima->flag, 1.0, 0.0, 0, 0, "UV Vertex select mode");
+			xco+= XIC;
+			uiDefIconButBitI(block, TOG, SI_SELACTFACE, B_REDR, ICON_FACESEL, xco,0,XIC,YIC, &G.sima->flag, 0, 0, 0, 0, "UV Face select mode");
+			xco+= XIC+8;
+			uiBlockEndAlign(block);
+			
+			/* would use these if const's could go in strings 
+			 * SI_STICKY_LOC SI_STICKY_DISABLE SI_STICKY_VERTEX */
+			ubut = uiDefIconTextButC(block, ICONTEXTROW, B_REDR, ICON_STICKY_UVS_LOC,
+					"Sticky UV Selection: %t|Disable%x1|Shared Location%x0|Shared Vertex%x2",
+					xco,0,XIC+10,YIC, &(G.sima->sticky), 0, 3.0, 0, 0,
+					"Sticky UV Selection (Hotkeys: Shift C, Alt C, Ctrl C)");
+			
+		}
+		xco+= XIC + 16;
+		
+		/* Snap copied right out of view3d header */
+			uiBlockBeginAlign(block);
+
+			if (G.scene->snap_flag & SCE_SNAP) {
+				uiDefIconButBitS(block, TOG, SCE_SNAP, B_REDR, ICON_SNAP_GEO,xco,0,XIC,YIC, &G.scene->snap_flag, 0, 0, 0, 0, "Use Snap or Grid (Shift Tab)");	
+				xco+= XIC;
+				uiDefButS(block, MENU, B_NOP, "Mode%t|Closest%x0|Center%x1|Median%x2",xco,0,70,YIC, &G.scene->snap_target, 0, 0, 0, 0, "Snap Target Mode");
+				xco+= 70;
+			} else {
+				uiDefIconButBitS(block, TOG, SCE_SNAP, B_REDR, ICON_SNAP_GEAR,xco,0,XIC,YIC, &G.scene->snap_flag, 0, 0, 0, 0, "Snap while Ctrl is held during transform (Shift Tab)");	
+				xco+= XIC;
+			}
+
+			uiBlockEndAlign(block);
+			xco+= 10;
+		/* end snap */
+			
+		/* Layer Menu */
+		layercount = CustomData_number_of_layers(&G.editMesh->fdata, CD_MTFACE); 
+		if (layercount>1 && layercount < 12) { /* could allow any number but limit of 11 means no malloc needed */
+			static int act;
+			char str_menu[384], *str_pt; /*384 allows for 11 layers */
+			
+			
+			act = CustomData_get_active_layer(&G.editMesh->fdata, CD_MTFACE);
+			
+			/*str_pt = (char *)MEM_mallocN(layercount*40 , "uvmenu"); str[0]='\0';*/
+			str_pt = str_menu;
+			str_pt[0]='\0';
+			mesh_layers_menu_concat(&G.editMesh->fdata, CD_MTFACE, str_pt);
+			ubut = uiDefButI(block, MENU, B_NOP, str_menu ,xco,0,85,YIC, &act, 0, 0, 0, 0, "Active UV Layer for editing");
+			uiButSetFunc(ubut, do_image_buttons_set_uvlayer_callback, &act, NULL);
+			
+			/*MEM_freeN(str);*/
+			xco+= 90;
+		}
+	}
+	
 	if (ima) {
 		RenderResult *rr= BKE_image_get_renderresult(ima);
 		
@@ -1164,20 +1275,8 @@ void image_buttons(void)
 			uiBlockEndAlign(block);
 			xco+= 166;
 		}
-		if( !ELEM3(ima->source, IMA_SRC_SEQUENCE, IMA_SRC_MOVIE, IMA_SRC_VIEWER) && ima->ok) {
-
-			if (ima->packedfile) {
-				headerbuttons_packdummy = 1;
-			}
-			if (ima->packedfile && ibuf && (ibuf->userflags & IB_BITMAPDIRTY))
-				uiDefIconButBitI(block, TOG, 1, B_SIMA_REPACK, ICON_UGLYPACKAGE,	xco,0,XIC,YIC, &headerbuttons_packdummy, 0, 0, 0, 0, "Re-Pack this image as PNG");
-			else
-				uiDefIconButBitI(block, TOG, 1, B_SIMAPACKIMA, ICON_PACKAGE,	xco,0,XIC,YIC, &headerbuttons_packdummy, 0, 0, 0, 0, "Pack/Unpack this image");
-				
-			xco+= XIC+8;
-		}
-		
 		uiDefIconButBitI(block, TOG, SI_DRAWTOOL, B_SIMAGEPAINTTOOL, ICON_TPAINT_HLT, xco,0,XIC,YIC, &G.sima->flag, 0, 0, 0, 0, "Enables painting textures on the image with left mouse button");
+		
 		xco+= XIC+8;
 
 		uiBlockBeginAlign(block);
@@ -1189,10 +1288,9 @@ void image_buttons(void)
 		}
 		if(ibuf) {
 			if(ibuf->zbuf || ibuf->zbuf_float || (ibuf->channels==1)) {
-				uiDefIconButBitI(block, TOG, SI_SHOW_ZBUF, B_SIMA_SHOW_ZBUF, ICON_SOLID, xco,0,XIC,YIC, &G.sima->flag, 0, 0, 0, 0, "Draws zbuffer values");
+				uiDefIconButBitI(block, TOG, SI_SHOW_ZBUF, B_SIMA_SHOW_ZBUF, ICON_SOLID, xco,0,XIC,YIC, &G.sima->flag, 0, 0, 0, 0, "Draws zbuffer values (mapped from camera clip start to end)");
 				xco+= XIC;
 			}
-			else G.sima->flag &= ~SI_SHOW_ZBUF;	/* no confusing display for non-zbuf images */
 		}		
 		xco+= 8;
 		
@@ -1208,7 +1306,7 @@ void image_buttons(void)
 		uiBlockEndAlign(block);
 		xco+= 8;
 	}
-
+	
 	/* draw LOCK */
 	uiDefIconButS(block, ICONTOG, 0, ICON_UNLOCKED,	xco,0,XIC,YIC, &(G.sima->lock), 0, 0, 0, 0, "Updates other affected window spaces automatically to reflect changes in real time");
 
