@@ -4,17 +4,14 @@
  * Functions to draw the "Text Editor" window header
  * and handle user events sent to it.
  * 
- * $Id: header_text.c,v 1.35 2007/03/11 16:25:17 campbellbarton Exp $
+ * $Id: header_text.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,7 +29,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <stdlib.h>
@@ -52,6 +49,8 @@
 #include "DNA_screen_types.h"
 #include "DNA_space_types.h"
 #include "DNA_text_types.h"
+#include "DNA_constraint_types.h"
+#include "DNA_action_types.h"
 
 #include "BIF_drawtext.h"
 #include "BIF_interface.h"
@@ -59,11 +58,14 @@
 #include "BIF_screen.h"
 #include "BIF_space.h"
 #include "BIF_toolbox.h"
+
 #include "BKE_global.h"
 #include "BKE_library.h"
 #include "BKE_main.h"
 #include "BKE_sca.h"
 #include "BKE_text.h"
+#include "BKE_depsgraph.h"
+
 #include "BSE_filesel.h"
 
 #include "BPY_extern.h"
@@ -74,13 +76,12 @@
 
 void do_text_buttons(unsigned short event)
 {
-	SpaceText *st= curarea->spacedata.first;
+	SpaceText *st= curarea->spacedata.first; /* bad but cant pass as an arg here */
 	ID *id, *idtest;
 	int nr= 1;
 	Text *text;
-		
-	if (!st) return;
-	if (st->spacetype != SPACE_TEXT) return;
+	
+	if (st==NULL || st->spacetype != SPACE_TEXT) return;
 	
 	switch (event) {
 	case B_TEXTBROWSE:
@@ -127,7 +128,7 @@ void do_text_buttons(unsigned short event)
 				st->top= 0;
 				
 				pop_space_text(st);
-				if (st->showsyntax) get_format_string();
+				if (st->showsyntax) get_format_string(st);
 				allqueue(REDRAWTEXT, 0);
 				allqueue(REDRAWHEADERS, 0);
 			}
@@ -135,27 +136,36 @@ void do_text_buttons(unsigned short event)
 		break;
 		
 	case B_TEXTDELETE:
-		
-		text= st->text;
-		if (!text) return;
-		
-		/* make the previous text active, if its not there make the next text active */
-		if (st->text->id.prev) {
-			st->text = st->text->id.prev;
-			pop_space_text(st);
-		} else if (st->text->id.next) {
-			st->text = st->text->id.next;
-			pop_space_text(st);
-		}
+		{
+			text= st->text;
+			if (!text) return;
 			
-		BPY_clear_bad_scriptlinks(text);
-		free_text_controllers(text);
-		
-		unlink_text(text);
-		free_libblock(&G.main->text, text);
-		
-		allqueue(REDRAWTEXT, 0);
-		allqueue(REDRAWHEADERS, 0);
+			/* make the previous text active, if its not there make the next text active */
+			if (st->text->id.prev) {
+				st->text = st->text->id.prev;
+				pop_space_text(st);
+			} else if (st->text->id.next) {
+				st->text = st->text->id.next;
+				pop_space_text(st);
+			}
+			
+			BPY_clear_bad_scriptlinks(text);
+			BPY_free_pyconstraint_links(text);
+			free_text_controllers(text);
+			
+			unlink_text(text);
+			free_libblock(&G.main->text, text);
+			
+			allqueue(REDRAWTEXT, 0);
+			allqueue(REDRAWHEADERS, 0);
+			
+			/*for if any object constraints were changed.*/
+			allqueue(REDRAWVIEW3D, 0);
+			allqueue(REDRAWBUTSOBJECT, 0);
+			allqueue(REDRAWBUTSEDIT, 0);
+			
+			BIF_undo_push("Delete Text");
+		}
 		break;
 		
 /*
@@ -184,13 +194,13 @@ void do_text_buttons(unsigned short event)
 
 		break;
 	case B_TAB_NUMBERS:
-		if (st->showsyntax) get_format_string();
+		if (st->showsyntax) get_format_string(st);
 		allqueue(REDRAWTEXT, 0);
 		allqueue(REDRAWHEADERS, 0);
 		break;
 	case B_SYNTAX:
 		if (st->showsyntax) {
-			get_format_string();
+			get_format_string(st);
 		}
 		allqueue(REDRAWTEXT, 0);
 		allqueue(REDRAWHEADERS, 0);
@@ -232,15 +242,19 @@ static uiBlock *text_template_scriptsmenu (void *args_unused)
 /* action executed after clicking in File menu */
 static void do_text_filemenu(void *arg, int event)
 {
-	SpaceText *st= curarea->spacedata.first;
-	Text *text= st->text;
+	SpaceText *st= curarea->spacedata.first; /* bad but cant pass as an arg here */
+	Text *text;
 	ScrArea *sa;
-
+	
+	if (st==NULL || st->spacetype != SPACE_TEXT) return;
+	
+	text= st->text;
+	
 	switch(event) {
 	case 1:
 		st->text= add_empty_text( "Text" );
 		st->top=0;
-
+		
 		allqueue(REDRAWTEXT, 0);
 		allqueue(REDRAWHEADERS, 0);
 		break;
@@ -254,7 +268,7 @@ static void do_text_filemenu(void *arg, int event)
 				if (!reopen_text(text)) {
 					error("Could not reopen file");
 				}
-			if (st->showsyntax) get_format_string();
+			if (st->showsyntax) get_format_string(st);
 			}
 		break;
 	case 5:
@@ -265,6 +279,42 @@ static void do_text_filemenu(void *arg, int event)
 	case 6:
 		run_python_script(st);
 		break;
+	case 7:
+	{
+		Object *ob;
+		bConstraint *con;
+		short update;
+		
+		/* check all pyconstraints */
+		for (ob= G.main->object.first; ob; ob= ob->id.next) {
+			update = 0;
+			if (ob->type==OB_ARMATURE && ob->pose) {
+				bPoseChannel *pchan;
+				for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
+					for (con = pchan->constraints.first; con; con= con->next) {
+						if (con->type==CONSTRAINT_TYPE_PYTHON) {
+							bPythonConstraint *data = con->data;
+							if (data->text==text) BPY_pyconstraint_update(ob, con);
+							update = 1;
+							
+						}
+					}
+				}
+			}
+			for (con = ob->constraints.first; con; con= con->next) {
+				if (con->type==CONSTRAINT_TYPE_PYTHON) {
+					bPythonConstraint *data = con->data;
+					if (data->text==text) BPY_pyconstraint_update(ob, con);
+					update = 1;
+				}
+			}
+			
+			if (update) {
+				DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+			}
+		}
+	}
+		break;
 	default:
 		break;
 	}
@@ -279,10 +329,14 @@ static void do_text_filemenu(void *arg, int event)
 /* action executed after clicking in Edit menu */
 static void do_text_editmenu(void *arg, int event)
 {
-	SpaceText *st= curarea->spacedata.first;
-	Text *text= st->text;
+	SpaceText *st= curarea->spacedata.first; /* bad but cant pass as an arg here */
+	Text *text;
 	ScrArea *sa;
 	
+	if (st==NULL || st->spacetype != SPACE_TEXT) return;
+	
+	text= st->text;
+	
 	switch(event) {
 	case 1:
 		txt_do_undo(text);
@@ -291,15 +345,17 @@ static void do_text_editmenu(void *arg, int event)
 		txt_do_redo(text);
 		break;
 	case 3:
+		txt_copy_clipboard(text);
 		txt_cut_sel(text);
 		pop_space_text(st);
 		break;
 	case 4:
-		txt_copy_sel(text);
+		//txt_copy_sel(text);
+		txt_copy_clipboard(text);
 		break;
 	case 5:
-		txt_paste(text);
-		if (st->showsyntax) get_format_string();
+		txt_paste_clipboard(text);
+		if (st->showsyntax) get_format_string(st);
 		break;
 	case 6:
 		txt_print_cutbuffer();
@@ -328,10 +384,14 @@ static void do_text_editmenu(void *arg, int event)
 /* action executed after clicking in View menu */
 static void do_text_editmenu_viewmenu(void *arg, int event)
 {
-	SpaceText *st= curarea->spacedata.first;
-	Text *text= st->text;
+	SpaceText *st= curarea->spacedata.first; /* bad but cant pass as an arg here */
+	Text *text;
 	ScrArea *sa;
 	
+	if (st==NULL || st->spacetype != SPACE_TEXT) return;
+	
+	text = st->text;
+	
 	switch(event) {
 		case 1:
 			txt_move_bof(text, 0);
@@ -356,10 +416,14 @@ static void do_text_editmenu_viewmenu(void *arg, int event)
 /* action executed after clicking in Select menu */
 static void do_text_editmenu_selectmenu(void *arg, int event)
 {
-	SpaceText *st= curarea->spacedata.first;
-	Text *text= st->text;
+	SpaceText *st= curarea->spacedata.first; /* bad but cant pass as an arg here */
+	Text *text;
 	ScrArea *sa;
 	
+	if (st==NULL || st->spacetype != SPACE_TEXT) return;
+	
+	text = st->text;
+	
 	switch(event) {
 	case 1:
 		txt_sel_all(text);
@@ -382,10 +446,14 @@ static void do_text_editmenu_selectmenu(void *arg, int event)
 /* action executed after clicking in Format menu */
 static void do_text_formatmenu(void *arg, int event)
 {
-	SpaceText *st= curarea->spacedata.first;
-	Text *text= st->text;
+	SpaceText *st= curarea->spacedata.first; /* bad but cant pass as an arg here */
+	Text *text;
 	ScrArea *sa;
 	
+	if (st==NULL || st->spacetype != SPACE_TEXT) return;
+	
+	text = st->text;
+	
 	switch(event) {
 	case 3:
 		if (txt_has_sel(text)) {
@@ -408,7 +476,7 @@ static void do_text_formatmenu(void *arg, int event)
 		if ( txt_has_sel(text)) {
 			txt_order_cursors(text);
 			comment(text);
-			if (st->showsyntax) get_format_string();
+			if (st->showsyntax) get_format_string(st);
 			break;
 		}
 		break;
@@ -416,7 +484,7 @@ static void do_text_formatmenu(void *arg, int event)
 		if ( txt_has_sel(text)) {
 			txt_order_cursors(text);
 			uncomment(text);
-			if (st->showsyntax) get_format_string();
+			if (st->showsyntax) get_format_string(st);
 			break;
 		}
 		break;
@@ -470,7 +538,9 @@ static uiBlock *text_editmenu_selectmenu(void *arg_unused)
 
 void do_text_formatmenu_convert(void *arg, int event)
 {
-	SpaceText *st= curarea->spacedata.first;
+	SpaceText *st= curarea->spacedata.first; /* bad but cant pass as an arg here */
+	
+	if (st==NULL || st->spacetype != SPACE_TEXT) return;
 	
 	switch(event) {
 	case 1: convert_tabs(st, 0); break;
@@ -529,8 +599,11 @@ static uiBlock *text_formatmenu(void *arg_unused)
 /* action executed after clicking in Object to 3d Sub Menu */
 void do_text_editmenu_to3dmenu(void *arg, int event)
 {
-	SpaceText *st= curarea->spacedata.first;
-	Text *text= st->text;
+	SpaceText *st= curarea->spacedata.first; /* bad but cant pass as an arg here */
+	Text *text;
+	if (st==NULL || st->spacetype != SPACE_TEXT) return;
+	
+	text = st->text;
 	
 	switch(event) {
 	case 1: txt_export_to_object(text); break;
@@ -598,7 +671,7 @@ static uiBlock *text_editmenu(void *arg_unused)
 /* File menu */
 static uiBlock *text_filemenu(void *arg_unused)
 {
-	SpaceText *st= curarea->spacedata.first;
+	SpaceText *st= curarea->spacedata.first; /* bad but cant pass as an arg here */
 	Text *text= st->text;
 	uiBlock *block;
 	short yco= 0, menuwidth=120;
@@ -608,13 +681,23 @@ static uiBlock *text_filemenu(void *arg_unused)
 
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "New|Alt N", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 1, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Open...|Alt O", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 2, "");
+	
 	if(text) {
 		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Reopen|Alt R", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 3, "");
+		
 		uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
+		
 		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Save|Alt S", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 4, "");
 		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Save As...", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
+		
 		uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
+		
 		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Run Python Script|Alt P", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 6, "");
+		
+		if (BPY_is_pyconstraint(text))
+			uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Refresh All PyConstraints", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
+			
+		uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 	}
 	
 	uiDefIconTextBlockBut(block, text_template_scriptsmenu, NULL, ICON_RIGHTARROW_THIN, "Script Templates", 0, yco-=20, 120, 19, "");
@@ -636,11 +719,13 @@ void text_buttons(void)
 {
 	uiBlock *block;
 	SpaceText *st= curarea->spacedata.first;
-	Text *text= st->text;
+	Text *text;
 	short xco, xmax;
 	char naam[256];
 	
-	if (!st || st->spacetype != SPACE_TEXT) return;
+	if (st==NULL || st->spacetype != SPACE_TEXT) return;
+	
+	text = st->text;
 
 	sprintf(naam, "header %d", curarea->headwin);
 	block= uiNewBlock(&curarea->uiblocks, naam, UI_EMBOSS, UI_HELV, curarea->headwin);
