@@ -1,18 +1,15 @@
 /**
- * $Id: imagepaint.c,v 1.23 2007/04/05 12:42:07 broken Exp $
+ * $Id: imagepaint.c 14444 2008-04-16 22:40:48Z hos $
  * imagepaint.c
  *
  * Functions to paint images in 2D and 3D.
  * 
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,7 +27,7 @@
  *
  * Contributor(s): Jens Ole Wund (bjornmose)
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <string.h>
@@ -559,6 +556,9 @@ static int imapaint_paint_sub_stroke(ImagePaintState *s, BrushPainter *painter,
 	ImBuf *ibuf= BKE_image_get_ibuf(image, G.sima?&G.sima->iuser:NULL);
 	float pos[2];
 
+	if(!ibuf)
+		return 0;
+
 	pos[0] = uv[0]*ibuf->x;
 	pos[1] = uv[1]*ibuf->y;
 
@@ -590,8 +590,10 @@ static void imapaint_paint_stroke(ImagePaintState *s, BrushPainter *painter, sho
 
 			if(ibuf && ibuf->rect)
 				texpaint_pick_uv(s->ob, s->me, newfaceindex, mval, newuv);
-			else
+			else {
 				newimage = NULL;
+				newuv[0] = newuv[1] = 0.0f;
+			}
 		}
 		else
 			newuv[0] = newuv[1] = 0.0f;
