@@ -1,15 +1,12 @@
 /**
- * $Id: oops.c 5339 2005-09-24 13:49:55Z ton $
+ * $Id: oops.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <math.h> 
@@ -50,6 +47,7 @@
 #include "DNA_texture_types.h"
 #include "DNA_texture_types.h"
 #include "DNA_key_types.h"
+#include "DNA_group_types.h"
 
 #include "BKE_utildefines.h"
 #include "BKE_global.h"
@@ -656,8 +654,21 @@ void add_material_oopslinks(Material *ma, Oops *oops, short flag)
 			if(ma->mtex[a]) add_oopslink("ob", oops, ID_OB, &(ma->mtex[a]->object), 0.0, (float)(0.2*OOPSY));
 		}
 	}
-	if(flag & OOPS_IP) {
+	if(flag & OOPS_IP)
 		if(ma->ipo) add_oopslink("ipo", oops, ID_IP, &(ma->ipo), OOPSX, (float)(0.5*OOPSY));
+	
+	if (ma->group)
+		add_oopslink("group", oops, ID_GR, &(ma->group), OOPSX, (float)(0.5*OOPSY));
+}
+
+
+void add_group_oopslinks(Group *gp, Oops *oops, short flag)
+{
+	GroupObject *gob;
+	if(flag & OOPS_GR) {
+		for(gob=gp->gobject.first; gob; gob= gob->next) {
+			add_oopslink("group", oops, ID_GR, &(gob->ob), (float)(0.5*OOPSX), (float)OOPSY);
+		}
 	}
 }
 
@@ -701,6 +712,9 @@ void add_object_oopslinks(Object *ob, Oops *oops, short flag)
 	}
 	
 	if(flag & OOPS_IP) add_oopslink("ipo", oops, ID_IP, &ob->ipo, OOPSX, (float)(0.5*OOPSY));
+	
+	if (ob->dup_group && (ob->transflag & OB_DUPLIGROUP))
+		add_oopslink("group", oops, ID_GR, &ob->dup_group, OOPSX, (float)(0.5*OOPSY));
 }
 
 void add_mesh_oopslinks(Mesh *me, Oops *oops, short flag)
@@ -817,6 +831,9 @@ Oops *add_test_oops(void *id)	/* incl links */
 	case ID_MA:
 		add_material_oopslinks((Material *)id, oops, G.soops->visiflag);
 		break;
+	case ID_GR:
+		add_group_oopslinks((Group *)id, oops, G.soops->visiflag);
+		break;
 	case ID_TE:
 		tex= (Tex *)id;
 		if(tex->ima) if(G.soops->visiflag & OOPS_IM) add_oopslink("image", oops, ID_IM, &tex->ima, OOPSX, (float)(0.3*OOPSY));
@@ -962,7 +979,13 @@ void build_oops()
 				ima= ima->id.next;
 			}
 		}
-		
+		if(G.soops->visiflag & OOPS_GR) {
+			Group *gp= G.main->group.first;
+			while(gp) {
+				oops= add_test_oops(gp);
+				gp= gp->id.next;
+			}
+		}
 	}
 	else {
 		
