@@ -5,6 +5,7 @@
 #include "BLI_arithb.h"
 #include "BLI_rand.h"
 #include "BLI_heap.h"
+#include "BLI_boxpack2d.h"
 
 #include "BKE_utildefines.h"
 
@@ -144,8 +145,8 @@ static float p_vec_angle_cos(float *v1, float *v2, float *v3)
 	d2[1] = v3[1] - v2[1];
 	d2[2] = v3[2] - v2[2];
 
-	Normalise(d1);
-	Normalise(d2);
+	Normalize(d1);
+	Normalize(d2);
 
 	return d1[0]*d2[0] + d1[1]*d2[1] + d1[2]*d2[2];
 }
@@ -211,23 +212,6 @@ static float p_face_uv_area_signed(PFace *f)
 	            ((v3->uv[0] - v1->uv[0])*(v2->uv[1] - v1->uv[1])));
 }
 
-static float p_face_uv_area(PFace *f)
-{
-	return fabs(p_face_uv_area_signed(f));
-}
-
-static void p_chart_area(PChart *chart, float *uv_area, float *area)
-{
-	PFace *f;
-
-	*uv_area = *area = 0.0f;
-
-	for (f=chart->faces; f; f=f->nextlink) {
-		*uv_area += p_face_uv_area(f);
-		*area += p_face_area(f);
-	}
-}
-
 static float p_edge_length(PEdge *e)
 {
     PVert *v1 = e->vert, *v2 = e->next->vert;
@@ -272,6 +256,16 @@ static void p_chart_uv_scale(PChart *chart, float scale)
 	}
 }
 
+static void p_chart_uv_scale_xy(PChart *chart, float x, float y)
+{
+	PVert *v;
+
+	for (v=chart->verts; v; v=v->nextlink) {
+		v->uv[0] *= x;
+		v->uv[1] *= y;
+	}
+}
+
 static void p_chart_uv_translate(PChart *chart, float trans[2])
 {
 	PVert *v;
@@ -395,7 +389,7 @@ static void p_chart_topological_sanity_check(PChart *chart)
 
 /* Loading / Flushing */
 
-static void p_vert_load_pin_select_uvs(PVert *v)
+static void p_vert_load_pin_select_uvs(PHandle *handle, PVert *v)
 {
 	PEdge *e;
 	int nedges = 0, npins = 0;
@@ -410,13 +404,13 @@ static void p_vert_load_pin_select_uvs(PVert *v)
 				v->flag |= PVERT_SELECT;
 
  			if (e->flag & PEDGE_PIN) {
-				pinuv[0] += e->orig_uv[0];
-				pinuv[1] += e->orig_uv[1];
+				pinuv[0] += e->orig_uv[0]*handle->aspx;
+				pinuv[1] += e->orig_uv[1]*handle->aspy;
 				npins++;
 			}
 			else {
-				v->uv[0] += e->orig_uv[0];
-				v->uv[1] += e->orig_uv[1];
+				v->uv[0] += e->orig_uv[0]*handle->aspx;
+				v->uv[1] += e->orig_uv[1]*handle->aspy;
 			}
 
 			nedges++;
@@ -436,27 +430,27 @@ static void p_vert_load_pin_select_uvs(PVert *v)
 	}
 }
 
-static void p_flush_uvs(PChart *chart)
+static void p_flush_uvs(PHandle *handle, PChart *chart)
 {
 	PEdge *e;
 
 	for (e=chart->edges; e; e=e->nextlink) {
 		if (e->orig_uv) {
-			e->orig_uv[0] = e->vert->uv[0];
-			e->orig_uv[1] = e->vert->uv[1];
+			e->orig_uv[0] = e->vert->uv[0]/handle->aspx;
+			e->orig_uv[1] = e->vert->uv[1]/handle->aspy;
 		}
 	}
 }
 
-static void p_flush_uvs_blend(PChart *chart, float blend)
+static void p_flush_uvs_blend(PHandle *handle, PChart *chart, float blend)
 {
 	PEdge *e;
 	float invblend = 1.0f - blend;
 
 	for (e=chart->edges; e; e=e->nextlink) {
 		if (e->orig_uv) {
-			e->orig_uv[0] = blend*e->old_uv[0] + invblend*e->vert->uv[0];
-			e->orig_uv[1] = blend*e->old_uv[1] + invblend*e->vert->uv[1];
+			e->orig_uv[0] = blend*e->old_uv[0] + invblend*e->vert->uv[0]/handle->aspx;
+			e->orig_uv[1] = blend*e->old_uv[1] + invblend*e->vert->uv[1]/handle->aspy;
 		}
 	}
 }
@@ -2229,7 +2223,7 @@ static PBool p_abf_matrix_invert(PAbfSystem *sys, PChart *chart)
 	nlBegin(NL_MATRIX);
 
 	for (i = 0; i < nvar; i++)
-		nlRightHandSideAdd(i, sys->bInterior[i]);
+		nlRightHandSideAdd(0, i, sys->bInterior[i]);
 
 	for (f=chart->faces; f; f=f->nextlink) {
 		float wi1, wi2, wi3, b, si, beta[3], j2[3][3], W[3][3];
@@ -2275,8 +2269,8 @@ static PBool p_abf_matrix_invert(PAbfSystem *sys, PChart *chart)
 			sys->J2dt[e2->u.id][0] = j2[1][0] = p_abf_compute_sin_product(sys, v1, e2->u.id)*wi2;
 			sys->J2dt[e3->u.id][0] = j2[2][0] = p_abf_compute_sin_product(sys, v1, e3->u.id)*wi3;
 
-			nlRightHandSideAdd(v1->u.id, j2[0][0]*beta[0]);
-			nlRightHandSideAdd(ninterior + v1->u.id, j2[1][0]*beta[1] + j2[2][0]*beta[2]);
+			nlRightHandSideAdd(0, v1->u.id, j2[0][0]*beta[0]);
+			nlRightHandSideAdd(0, ninterior + v1->u.id, j2[1][0]*beta[1] + j2[2][0]*beta[2]);
 
 			row1[0] = j2[0][0]*W[0][0];
 			row2[0] = j2[0][0]*W[1][0];
@@ -2295,8 +2289,8 @@ static PBool p_abf_matrix_invert(PAbfSystem *sys, PChart *chart)
 			sys->J2dt[e2->u.id][1] = j2[1][1] = 1.0*wi2;
 			sys->J2dt[e3->u.id][1] = j2[2][1] = p_abf_compute_sin_product(sys, v2, e3->u.id)*wi3;
 
-			nlRightHandSideAdd(v2->u.id, j2[1][1]*beta[1]);
-			nlRightHandSideAdd(ninterior + v2->u.id, j2[0][1]*beta[0] + j2[2][1]*beta[2]);
+			nlRightHandSideAdd(0, v2->u.id, j2[1][1]*beta[1]);
+			nlRightHandSideAdd(0, ninterior + v2->u.id, j2[0][1]*beta[0] + j2[2][1]*beta[2]);
 
 			row1[1] = j2[1][1]*W[0][1];
 			row2[1] = j2[1][1]*W[1][1];
@@ -2315,8 +2309,8 @@ static PBool p_abf_matrix_invert(PAbfSystem *sys, PChart *chart)
 			sys->J2dt[e2->u.id][2] = j2[1][2] = p_abf_compute_sin_product(sys, v3, e2->u.id)*wi2;
 			sys->J2dt[e3->u.id][2] = j2[2][2] = 1.0*wi3;
 
-			nlRightHandSideAdd(v3->u.id, j2[2][2]*beta[2]);
-			nlRightHandSideAdd(ninterior + v3->u.id, j2[0][2]*beta[0] + j2[1][2]*beta[1]);
+			nlRightHandSideAdd(0, v3->u.id, j2[2][2]*beta[2]);
+			nlRightHandSideAdd(0, ninterior + v3->u.id, j2[0][2]*beta[0] + j2[1][2]*beta[1]);
 
 			row1[2] = j2[2][2]*W[0][2];
 			row2[2] = j2[2][2]*W[1][2];
@@ -2373,24 +2367,24 @@ static PBool p_abf_matrix_invert(PAbfSystem *sys, PChart *chart)
 			pre[0] = pre[1] = pre[2] = 0.0;
 
 			if (v1->flag & PVERT_INTERIOR) {
-				float x = nlGetVariable(v1->u.id);
-				float x2 = nlGetVariable(ninterior + v1->u.id);
+				float x = nlGetVariable(0, v1->u.id);
+				float x2 = nlGetVariable(0, ninterior + v1->u.id);
 				pre[0] += sys->J2dt[e1->u.id][0]*x;
 				pre[1] += sys->J2dt[e2->u.id][0]*x2;
 				pre[2] += sys->J2dt[e3->u.id][0]*x2;
 			}
 
 			if (v2->flag & PVERT_INTERIOR) {
-				float x = nlGetVariable(v2->u.id);
-				float x2 = nlGetVariable(ninterior + v2->u.id);
+				float x = nlGetVariable(0, v2->u.id);
+				float x2 = nlGetVariable(0, ninterior + v2->u.id);
 				pre[0] += sys->J2dt[e1->u.id][1]*x2;
 				pre[1] += sys->J2dt[e2->u.id][1]*x;
 				pre[2] += sys->J2dt[e3->u.id][1]*x2;
 			}
 
 			if (v3->flag & PVERT_INTERIOR) {
-				float x = nlGetVariable(v3->u.id);
-				float x2 = nlGetVariable(ninterior + v3->u.id);
+				float x = nlGetVariable(0, v3->u.id);
+				float x2 = nlGetVariable(0, ninterior + v3->u.id);
 				pre[0] += sys->J2dt[e1->u.id][2]*x2;
 				pre[1] += sys->J2dt[e2->u.id][2]*x2;
 				pre[2] += sys->J2dt[e3->u.id][2]*x;
@@ -2421,8 +2415,8 @@ static PBool p_abf_matrix_invert(PAbfSystem *sys, PChart *chart)
 		}
 
 		for (i = 0; i < ninterior; i++) {
-			sys->lambdaPlanar[i] += nlGetVariable(i);
-			sys->lambdaLength[i] += nlGetVariable(ninterior + i);
+			sys->lambdaPlanar[i] += nlGetVariable(0, i);
+			sys->lambdaLength[i] += nlGetVariable(0, ninterior + i);
 		}
 	}
 
@@ -2754,8 +2748,8 @@ static void p_chart_lscm_load_solution(PChart *chart)
 	PVert *v;
 
 	for (v=chart->verts; v; v=v->nextlink) {
-		v->uv[0] = nlGetVariable(2*v->u.id);
-		v->uv[1] = nlGetVariable(2*v->u.id + 1);
+		v->uv[0] = nlGetVariable(0, 2*v->u.id);
+		v->uv[1] = nlGetVariable(0, 2*v->u.id + 1);
 	}
 }
 
@@ -2787,8 +2781,6 @@ static void p_chart_lscm_begin(PChart *chart, PBool live, PBool abf)
 #endif
 
 		if (abf) {
-			PBool p_chart_abf_solve(PChart *chart);
-
 			if (!p_chart_abf_solve(chart))
 				param_warning("ABF solving failed: falling back to LSCM.\n");
 		}
@@ -2814,17 +2806,19 @@ static void p_chart_lscm_begin(PChart *chart, PBool live, PBool abf)
 
 		nlNewContext();
 		nlSolverParameteri(NL_NB_VARIABLES, 2*chart->nverts);
+		nlSolverParameteri(NL_NB_ROWS, 2*chart->nfaces);
 		nlSolverParameteri(NL_LEAST_SQUARES, NL_TRUE);
 
 		chart->u.lscm.context = nlGetCurrent();
 	}
 }
 
-static PBool p_chart_lscm_solve(PChart *chart)
+static PBool p_chart_lscm_solve(PHandle *handle, PChart *chart)
 {
 	PVert *v, *pin1 = chart->u.lscm.pin1, *pin2 = chart->u.lscm.pin2;
 	PFace *f;
 	float *alpha = chart->u.lscm.abf_alpha;
+	int row;
 
 	nlMakeCurrent(chart->u.lscm.context);
 
@@ -2836,7 +2830,7 @@ static PBool p_chart_lscm_solve(PChart *chart)
 
 	for (v=chart->verts; v; v=v->nextlink)
 		if (v->flag & PVERT_PIN)
-			p_vert_load_pin_select_uvs(v); /* reload for live */
+			p_vert_load_pin_select_uvs(handle, v); /* reload for live */
 
 	if (chart->u.lscm.pin1) {
 		nlLockVariable(2*pin1->u.id);
@@ -2844,10 +2838,10 @@ static PBool p_chart_lscm_solve(PChart *chart)
 		nlLockVariable(2*pin2->u.id);
 		nlLockVariable(2*pin2->u.id + 1);
 	
-		nlSetVariable(2*pin1->u.id, pin1->uv[0]);
-		nlSetVariable(2*pin1->u.id + 1, pin1->uv[1]);
-		nlSetVariable(2*pin2->u.id, pin2->uv[0]);
-		nlSetVariable(2*pin2->u.id + 1, pin2->uv[1]);
+		nlSetVariable(0, 2*pin1->u.id, pin1->uv[0]);
+		nlSetVariable(0, 2*pin1->u.id + 1, pin1->uv[1]);
+		nlSetVariable(0, 2*pin2->u.id, pin2->uv[0]);
+		nlSetVariable(0, 2*pin2->u.id + 1, pin2->uv[1]);
 	}
 	else {
 		/* set and lock the pins */
@@ -2856,8 +2850,8 @@ static PBool p_chart_lscm_solve(PChart *chart)
 				nlLockVariable(2*v->u.id);
 				nlLockVariable(2*v->u.id + 1);
 
-				nlSetVariable(2*v->u.id, v->uv[0]);
-				nlSetVariable(2*v->u.id + 1, v->uv[1]);
+				nlSetVariable(0, 2*v->u.id, v->uv[0]);
+				nlSetVariable(0, 2*v->u.id + 1, v->uv[1]);
 			}
 		}
 	}
@@ -2866,6 +2860,7 @@ static PBool p_chart_lscm_solve(PChart *chart)
 
 	nlBegin(NL_MATRIX);
 
+	row = 0;
 	for (f=chart->faces; f; f=f->nextlink) {
 		PEdge *e1 = f->edge, *e2 = e1->next, *e3 = e2->next;
 		PVert *v1 = e1->vert, *v2 = e2->vert, *v3 = e3->vert;
@@ -2888,9 +2883,6 @@ static PBool p_chart_lscm_solve(PChart *chart)
 		sinmax = MAX3(sina1, sina2, sina3);
 
 		/* shift vertices to find most stable order */
-		#define SHIFT3(type, a, b, c) \
-			{ type tmp; tmp = a; a = c; c = b; b = tmp; }
-
 		if (sina3 != sinmax) {
 			SHIFT3(PVert*, v1, v2, v3);
 			SHIFT3(float, a1, a2, a3);
@@ -2908,6 +2900,7 @@ static PBool p_chart_lscm_solve(PChart *chart)
 		cosine = cos(a1)*ratio;
 		sine = sina1*ratio;
 
+#if 0
 		nlBegin(NL_ROW);
 		nlCoefficient(2*v1->u.id,   cosine - 1.0);
 		nlCoefficient(2*v1->u.id+1, -sine);
@@ -2923,6 +2916,21 @@ static PBool p_chart_lscm_solve(PChart *chart)
 		nlCoefficient(2*v2->u.id+1, -cosine);
 		nlCoefficient(2*v3->u.id+1, 1.0);
 		nlEnd(NL_ROW);
+#else
+		nlMatrixAdd(row, 2*v1->u.id,   cosine - 1.0);
+		nlMatrixAdd(row, 2*v1->u.id+1, -sine);
+		nlMatrixAdd(row, 2*v2->u.id,   -cosine);
+		nlMatrixAdd(row, 2*v2->u.id+1, sine);
+		nlMatrixAdd(row, 2*v3->u.id,   1.0);
+		row++;
+
+		nlMatrixAdd(row, 2*v1->u.id,   sine);
+		nlMatrixAdd(row, 2*v1->u.id+1, cosine - 1.0);
+		nlMatrixAdd(row, 2*v2->u.id,   -sine);
+		nlMatrixAdd(row, 2*v2->u.id+1, -cosine);
+		nlMatrixAdd(row, 2*v3->u.id+1, 1.0);
+		row++;
+#endif
 	}
 
 	nlEnd(NL_MATRIX);
@@ -3104,144 +3112,6 @@ static void p_chart_stretch_minimize(PChart *chart, RNG *rng)
 	}
 }
 
-/* Packing */
-
-static int p_compare_chart_area(const void *a, const void *b)
-{
-	PChart *ca = *((PChart**)a);
-	PChart *cb = *((PChart**)b);
-
-    if (ca->u.pack.area > cb->u.pack.area)
-		return -1;
-	else if (ca->u.pack.area == cb->u.pack.area)
-		return 0;
-	else
-		return 1;
-}
-
-static PBool p_pack_try(PHandle *handle, float side)
-{
-	PChart *chart;
-	float packx, packy, rowh, groupw, w, h;
-	int i;
-
-	packx= packy= 0.0;
-	rowh= 0.0;
-	groupw= 1.0/sqrt(handle->ncharts);
-
-	for (i = 0; i < handle->ncharts; i++) {
-		chart = handle->charts[i];
-
-		if (chart->flag & PCHART_NOPACK)
-			continue;
-
-		w = chart->u.pack.size[0];
-		h = chart->u.pack.size[1];
-
-		if(w <= (side-packx)) {
-			chart->u.pack.trans[0] = packx;
-			chart->u.pack.trans[1] = packy;
-
-			packx += w;
-			rowh= MAX2(rowh, h);
-		}
-		else {
-			packy += rowh;
-			packx = w;
-			rowh = h;
-
-			chart->u.pack.trans[0] = 0.0;
-			chart->u.pack.trans[1] = packy;
-		}
-
-		if (packy+rowh > side)
-			return P_FALSE;
-	}
-
-	return P_TRUE;
-}
-
-#define PACK_SEARCH_DEPTH 15
-
-void p_charts_pack(PHandle *handle)
-{
-	PChart *chart;
-	float uv_area, area, trans[2], minside, maxside, totarea, side;
-	int i;
-
-	/* very simple rectangle packing */
-
-	if (handle->ncharts == 0)
-		return;
-
-	totarea = 0.0f;
-	maxside = 0.0f;
-
-	for (i = 0; i < handle->ncharts; i++) {
-		chart = handle->charts[i];
-
-		if (chart->flag & PCHART_NOPACK) {
-			chart->u.pack.area = 0.0f;
-			continue;
-		}
-
-		p_chart_area(chart, &uv_area, &area);
-		p_chart_uv_bbox(chart, trans, chart->u.pack.size);
-
-		/* translate to origin and make area equal to 3d area */
-		chart->u.pack.rescale = (uv_area > 0.0f)? sqrt(area)/sqrt(uv_area): 0.0f;
-		chart->u.pack.area = area;
-		totarea += area;
-
-		trans[0] = -trans[0];
-		trans[1] = -trans[1];
-		p_chart_uv_translate(chart, trans);
-		p_chart_uv_scale(chart, chart->u.pack.rescale);
-
-		/* compute new dimensions for packing */
-		chart->u.pack.size[0] += trans[0];
-		chart->u.pack.size[1] += trans[1];
-		chart->u.pack.size[0] *= chart->u.pack.rescale;
-		chart->u.pack.size[1] *= chart->u.pack.rescale;
-
-		maxside = MAX3(maxside, chart->u.pack.size[0], chart->u.pack.size[1]);
-	}
-
-	/* sort by chart area, largest first */
-	qsort(handle->charts, handle->ncharts, sizeof(PChart*), p_compare_chart_area);
-
-	/* binary search over pack region size */
-	minside = MAX2(sqrt(totarea), maxside);
-	maxside = (((int)sqrt(handle->ncharts-1))+1)*maxside;
-
-	if (minside < maxside) { /* should always be true */
-
-		for (i = 0; i < PACK_SEARCH_DEPTH; i++) {
-			if (p_pack_try(handle, (minside+maxside)*0.5f + 1e-5))
-				maxside = (minside+maxside)*0.5f;
-			else
-				minside = (minside+maxside)*0.5f;
-		}
-	}
-
-	/* do the actual packing */
-	side = maxside + 1e-5;
-	if (!p_pack_try(handle, side))
-		param_warning("packing failed.\n");
-
-	for (i = 0; i < handle->ncharts; i++) {
-		chart = handle->charts[i];
-
-		if (chart->flag & PCHART_NOPACK)
-			continue;
-
-		p_chart_uv_scale(chart, 1.0f/side);
-		trans[0] = chart->u.pack.trans[0]/side;
-		trans[1] = chart->u.pack.trans[1]/side;
-		p_chart_uv_translate(chart, trans);
-	}
-}
-
 /* Minimum area enclosing rectangle for packing */
 
 static int p_compare_geometric_uv(const void *a, const void *b)
@@ -3990,6 +3860,8 @@ ParamHandle *param_construct_begin()
 	handle->construction_chart = p_chart_new(handle);
 	handle->state = PHANDLE_STATE_ALLOCATED;
 	handle->arena = BLI_memarena_new((1<<16));
+	handle->aspx = 1.0f;
+	handle->aspy = 1.0f;
 
 	handle->hash_verts = phash_new((PHashLink**)&handle->construction_chart->verts, 1);
 	handle->hash_edges = phash_new((PHashLink**)&handle->construction_chart->edges, 1);
@@ -3998,6 +3870,14 @@ ParamHandle *param_construct_begin()
 	return (ParamHandle*)handle;
 }
 
+void param_aspect_ratio(ParamHandle *handle, float aspx, float aspy)
+{
+	PHandle *phandle = (PHandle*)handle;
+
+	phandle->aspx = aspx;
+	phandle->aspy = aspy;
+}
+
 void param_delete(ParamHandle *handle)
 {
 	PHandle *phandle = (PHandle*)handle;
@@ -4086,7 +3966,7 @@ void param_construct_end(ParamHandle *handle, ParamBool fill, ParamBool impl)
 
 		p_chart_boundaries(chart, &nboundaries, &outer);
 
-		if (nboundaries == 0) {
+		if (!impl && nboundaries == 0) {
 			p_chart_delete(chart);
 			continue;
 		}
@@ -4098,7 +3978,7 @@ void param_construct_end(ParamHandle *handle, ParamBool fill, ParamBool impl)
 			p_chart_fill_boundaries(chart, outer);
 
 		for (v=chart->verts; v; v=v->nextlink)
-			p_vert_load_pin_select_uvs(v);
+			p_vert_load_pin_select_uvs(handle, v);
 	}
 
 	phandle->ncharts = j;
@@ -4135,7 +4015,7 @@ void param_lscm_solve(ParamHandle *handle)
 		chart = phandle->charts[i];
 
 		if (chart->u.lscm.context) {
-			result = p_chart_lscm_solve(chart);
+			result = p_chart_lscm_solve(phandle, chart);
 
 			if (result && !(chart->flag & PCHART_NOPACK))
 				p_chart_rotate_minimum_area(chart);
@@ -4242,10 +4122,139 @@ void param_smooth_area(ParamHandle *handle)
 		p_smooth(chart);
 	}
 }
-
+ 
 void param_pack(ParamHandle *handle)
 {
-	p_charts_pack((PHandle*)handle);
+	/* box packing variables */
+	boxPack *boxarray, *box;
+	float tot_width, tot_height, scale;
+	 
+	PChart *chart;
+	int i, unpacked=0;
+	float trans[2];
+	
+	PHandle *phandle = (PHandle*)handle;
+	
+	if (phandle->ncharts == 0)
+		return;
+	
+	if(phandle->aspx != phandle->aspy)
+		param_scale(handle, 1.0f/phandle->aspx, 1.0f/phandle->aspy);
+	
+	/* we may not use all these boxes */
+	boxarray = MEM_mallocN( phandle->ncharts*sizeof(boxPack), "boxPack box");
+	
+	for (i = 0; i < phandle->ncharts; i++) {
+		chart = phandle->charts[i];
+		
+		if (chart->flag & PCHART_NOPACK) {
+			unpacked++;
+			continue;
+		}
+		
+		box = boxarray+(i-unpacked);
+		
+		p_chart_uv_bbox(chart, trans, chart->u.pack.size);
+		
+		trans[0] = -trans[0];
+		trans[1] = -trans[1];
+		
+		p_chart_uv_translate(chart, trans);
+		
+		box->w =  chart->u.pack.size[0] + trans[0];
+		box->h =  chart->u.pack.size[1] + trans[1];
+		box->index = i; /* warning this index skips PCHART_NOPACK boxes */
+	}
+	
+	boxPack2D(boxarray, phandle->ncharts-unpacked, &tot_width, &tot_height);
+	
+	if (tot_height>tot_width)
+		scale = 1.0/tot_height;
+	else
+		scale = 1.0/tot_width;
+	
+	for (i = 0; i < phandle->ncharts-unpacked; i++) {
+		box = boxarray+i;
+		trans[0] = box->x;
+		trans[1] = box->y;
+		
+		chart = phandle->charts[box->index];
+		p_chart_uv_translate(chart, trans);
+		p_chart_uv_scale(chart, scale);
+	}
+	MEM_freeN(boxarray);
+
+	if(phandle->aspx != phandle->aspy)
+		param_scale(handle, phandle->aspx, phandle->aspy);
+}
+
+void param_average(ParamHandle *handle)
+{
+	PChart *chart;
+	int i;
+	float tot_uvarea = 0.0f, tot_facearea = 0.0f;
+	float tot_fac, fac;
+	float minv[2], maxv[2], trans[2];
+	PHandle *phandle = (PHandle*)handle;
+	
+	if (phandle->ncharts == 0)
+		return;
+	
+	for (i = 0; i < phandle->ncharts; i++) {
+		PFace *f;
+		chart = phandle->charts[i];
+		
+		chart->u.pack.area = 0.0f; /* 3d area */
+		chart->u.pack.rescale = 0.0f; /* UV area, abusing rescale for tmp storage, oh well :/ */
+		
+		for (f=chart->faces; f; f=f->nextlink) {
+			chart->u.pack.area += p_face_area(f);
+			chart->u.pack.rescale += fabs(p_face_uv_area_signed(f));
+		}
+		
+		tot_facearea += chart->u.pack.area;
+		tot_uvarea += chart->u.pack.rescale;
+	}
+	
+	if (tot_facearea == tot_uvarea || tot_facearea==0.0f || tot_uvarea==0.0f) {
+		/* nothing to do */
+		return;
+	}
+	
+	tot_fac = tot_facearea/tot_uvarea;
+	
+	for (i = 0; i < phandle->ncharts; i++) {
+		chart = phandle->charts[i];
+		if (chart->u.pack.area != 0.0f && chart->u.pack.rescale != 0.0f) {
+			fac = chart->u.pack.area / chart->u.pack.rescale;
+			
+			/* Get the island center */
+			p_chart_uv_bbox(chart, minv, maxv);
+			trans[0] = (minv[0] + maxv[0]) /-2.0f;
+			trans[1] = (minv[1] + maxv[1]) /-2.0f;
+			
+			/* Move center to 0,0 */
+			p_chart_uv_translate(chart, trans);
+			p_chart_uv_scale(chart, sqrt(fac / tot_fac));
+			
+			/* Move to original center */
+			trans[0] = -trans[0];
+			trans[1] = -trans[1];
+			p_chart_uv_translate(chart, trans);
+		}
+	}
+}
+
+void param_scale(ParamHandle *handle, float x, float y)
+{
+	PHandle *phandle = (PHandle*)handle;
+	PChart *chart;
+	int i;
+
+	for (i = 0; i < phandle->ncharts; i++) {
+		chart = phandle->charts[i];
+		p_chart_uv_scale_xy(chart, x, y);
+	}
 }
 
 void param_flush(ParamHandle *handle)
@@ -4261,9 +4270,9 @@ void param_flush(ParamHandle *handle)
 			continue;
 
 		if (phandle->blend == 0.0f)
-			p_flush_uvs(chart);
+			p_flush_uvs(phandle, chart);
 		else
-			p_flush_uvs_blend(chart, phandle->blend);
+			p_flush_uvs_blend(phandle, chart, phandle->blend);
 	}
 }
 
