@@ -1,15 +1,12 @@
 /*
- *  $Id: seqaudio.c,v 1.26 2007/04/12 14:51:04 khughes Exp $
+ *  $Id: seqaudio.c 14669 2008-05-04 09:41:15Z campbellbarton $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): intrr, Peter Schlaile
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <math.h>
@@ -95,6 +92,8 @@ static int audio_pos;
 static int audio_scrub=0;
 static int audio_playing=0;
 static int audio_initialised=0;
+static int audio_startframe=0;
+static double audio_starttime = 0.0;
 /////
 //
 /* local protos ------------------- */
@@ -107,7 +106,7 @@ void makewavstring (char *string)
 	if (string==0) return;
 
 	strcpy(string, G.scene->r.pic);
-	BLI_convertstringcode(string, G.sce, G.scene->r.cfra);
+	BLI_convertstringcode(string, G.sce);
 
 	BLI_make_existing_file(string);
 
@@ -139,7 +138,7 @@ void audio_mixdown()
 
 	strcpy(buf, "RIFFlengWAVEfmt fmln01ccRATEbsecBP16dataDLEN");
 	totframe = (EFRA - SFRA + 1);
-	totlen = (int) ( ((float)totframe / (float)G.scene->r.frs_sec) * (float)G.scene->audio.mixrate * 4.0);
+	totlen = (int) ( FRA2TIME(totframe) * (float)G.scene->audio.mixrate * 4.0);
 	printf(" totlen %d\n", totlen+36+8);
 	
 	totlen+= 36;	/* len is filesize-8 in WAV spec, total header is 44 bytes */
@@ -182,7 +181,7 @@ void audio_mixdown()
 		
 		memset(buf+i, 0, 64);
 		
-		CFRA=(int) ( ((float)(audio_pos-64)/( G.scene->audio.mixrate*4 ))*(float)G.scene->r.frs_sec );
+		CFRA=(int) ( ((float)(audio_pos-64)/( G.scene->audio.mixrate*4 ))*FPS );
 			
 		audio_fill(buf+i, NULL, 64);
 		if (G.order == B_ENDIAN) {
@@ -216,7 +215,7 @@ void audiostream_fill(Uint8 *mixdown, int len)
 	for (i = 0; i < len; i += 64) {
 		CFRA = (int) ( ((float)(audio_pos-64)
 				/( G.scene->audio.mixrate*4 ))
-			       *(float)G.scene->r.frs_sec );
+			       * FPS );
 
 		audio_fill(mixdown + i, NULL, 
 			   (len - i) > 64 ? 64 : (len - i));
@@ -302,7 +301,7 @@ static void audio_fill_ram_sound(Sequence *seq, void * mixdown,
 	    (seq->startdisp <= CFRA) && ((seq->enddisp) > CFRA))
 	{
 		if(seq->ipo && seq->ipo->curve.first) {
-			do_seq_ipo(seq);
+			do_seq_ipo(seq, CFRA);
 			facf = seq->facf0;
 		} else {
 			facf = 1.0;
@@ -331,7 +330,7 @@ static void audio_fill_hd_sound(Sequence *seq,
 	    (seq->startdisp <= CFRA) && ((seq->enddisp) > CFRA))
 	{
 		if(seq->ipo && seq->ipo->curve.first) {
-			do_seq_ipo(seq);
+			do_seq_ipo(seq, CFRA);
 			facf = seq->facf0; 
 		} else {
 			facf = 1.0;
@@ -357,36 +356,52 @@ static void audio_fill_hd_sound(Sequence *seq,
 }
 
 static void audio_fill_seq(Sequence * seq, void * mixdown,
-			   Uint8 *sstream, int len)
+			   Uint8 *sstream, int len, int advance_only)
 {
 	while(seq) {
-		if (seq->type == SEQ_META) {
-			audio_fill_seq(seq->seqbase.first,
-				       mixdown, sstream, len);
+		if (seq->type == SEQ_META &&
+		    (!(seq->flag & SEQ_MUTE))) {
+			if (seq->startdisp <= CFRA && seq->enddisp > CFRA) {
+				audio_fill_seq(seq->seqbase.first,
+					       mixdown, sstream, len, 
+					       advance_only);
+			} else {
+				audio_fill_seq(seq->seqbase.first,
+					       mixdown, sstream, len, 
+					       1);
+			}
 		}
 		if ( (seq->type == SEQ_RAM_SOUND) &&
 		     (seq->sound) &&
 		     (!(seq->flag & SEQ_MUTE))) {
-			audio_fill_ram_sound(seq, mixdown, sstream, len);
+			if (advance_only) {
+				seq->curpos += len;
+			} else {
+				audio_fill_ram_sound(
+					seq, mixdown, sstream, len);
+			}
 		}
 		if ( (seq->type == SEQ_HD_SOUND) &&
 		     (!(seq->flag & SEQ_MUTE)))	{
-			if (!seq->hdaudio) {
-				char name[FILE_MAXDIR+FILE_MAXFILE];
+			if (advance_only) {
+				seq->curpos += len;
+			} else {
+				if (!seq->hdaudio) {
+					char name[FILE_MAXDIR+FILE_MAXFILE];
 
-				strncpy(name, seq->strip->dir, 
-					FILE_MAXDIR-1);
-				strncat(name, 
-					seq->strip->stripdata->name, 
-					FILE_MAXFILE-1);
-				BLI_convertstringcode(name, G.sce, 
-						      G.scene->r.cfra);
+					strncpy(name, seq->strip->dir, 
+						FILE_MAXDIR-1);
+					strncat(name, 
+						seq->strip->stripdata->name, 
+						FILE_MAXFILE-1);
+					BLI_convertstringcode(name, G.sce);
 				
-				seq->hdaudio= sound_open_hdaudio(name);
-			}
-			if (seq->hdaudio) {
-				audio_fill_hd_sound(seq, mixdown, 
-						    sstream, len);
+					seq->hdaudio= sound_open_hdaudio(name);
+				}
+				if (seq->hdaudio) {
+					audio_fill_hd_sound(seq, mixdown, 
+							    sstream, len);
+				}
 			}
 		}
 		seq = seq->next;
@@ -401,7 +416,7 @@ void audio_fill(void *mixdown, Uint8 *sstream, int len)
 	ed = G.scene->ed;
 	if((ed) && (!(G.scene->audio.flag & AUDIO_MUTE))) {
 		seq = ed->seqbasep->first;
-		audio_fill_seq(seq, mixdown, sstream, len);
+		audio_fill_seq(seq, mixdown, sstream, len, 0);
 	}
        
 	audio_pos += len;    
@@ -452,9 +467,11 @@ static int audiostream_play_seq(Sequence * seq, Uint32 startframe)
 		}
 		if ((seq->type == SEQ_RAM_SOUND) && (seq->sound)) {
 			have_sound = 1;
-			seq->curpos = (int)( (((float)((float)startframe
-						       -(float)seq->start)
-					       / (float)G.scene->r.frs_sec)
+			seq->curpos = (int)( (FRA2TIME(
+						      (double) startframe -
+						      (double) seq->start +
+						      (double) 
+						      seq->anim_startofs)
 					      * ((float)G.scene->audio.mixrate)
 					      * 4 ));
 		}
@@ -467,9 +484,10 @@ static int audiostream_play_seq(Sequence * seq, Uint32 startframe)
 				
 				seq->hdaudio = sound_open_hdaudio(name);
 			}
-			seq->curpos = (int)( (((float)((float)startframe
-						       - (float)seq->start)
-					       / (float)G.scene->r.frs_sec)
+			seq->curpos = (int)( (FRA2TIME((double) startframe - 
+						       (double) seq->start +
+						       (double)
+						       seq->anim_startofs)
 					      * ((float)G.scene->audio.mixrate)
 					      * 4 ));
 		}
@@ -495,22 +513,28 @@ void audiostream_play(Uint32 startframe, Uint32 duration, int mixdown)
 		sound_init_audio();
 	}
 
-   	if (!audio_initialised && !(duration + mixdown)) {
+   	if (U.mixbufsize && !audio_initialised && !mixdown) {
    		desired.freq=G.scene->audio.mixrate;
 		desired.format=AUDIO_S16SYS;
    		desired.channels=2;
    		desired.samples=U.mixbufsize;
-   		desired.userdata=0;	
+   		desired.userdata=0;
+
    		if (audio_init(&desired)==0) {
    			U.mixbufsize = 0;	/* no audio */
    		}
    	}
 
-	audio_pos = ( ((int)( (((float)startframe)
-			       /(float)G.scene->r.frs_sec)
+	audio_startframe = startframe;
+	audio_pos = ( ((int)( FRA2TIME(startframe)
 			      *(G.scene->audio.mixrate)*4 )) & (~3) );
-	
-	audio_scrub = duration;
+	audio_starttime = PIL_check_seconds_timer();
+
+	/* if audio already is playing, just reseek, otherwise
+	   remember scrub-duration */
+	if (!(audio_playing && !audio_scrub)) {
+		audio_scrub = duration;
+	}
 	if (!mixdown) {
 		SDL_PauseAudio(0);
 		audio_playing++;
@@ -536,9 +560,17 @@ void audiostream_stop(void)
 int audiostream_pos(void) 
 {
 	int pos;
-	
-	pos = (int) ( ((float)(audio_pos-U.mixbufsize)/( G.scene->audio.mixrate*4 ))*(float)G.scene->r.frs_sec );
-	if (pos<1) pos=1;
+
+	if (U.mixbufsize) {
+		pos = (int) (((double)(audio_pos-U.mixbufsize)
+			      / ( G.scene->audio.mixrate*4 ))
+			     * FPS );
+	} else { /* fallback to seconds_timer when no audio available */
+		pos = (int) ((PIL_check_seconds_timer() - audio_starttime) 
+			     * FPS);
+	}
+
+	if (pos < audio_startframe) pos = audio_startframe;
 	return ( pos );
 }
 
