@@ -1,15 +1,12 @@
 /**
- * $Id: seqeffects.c,v 1.16 2007/04/04 13:18:33 campbellbarton Exp $
+ * $Id: seqeffects.c 14517 2008-04-22 14:53:15Z sirdude $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +22,7 @@
  *
  * Contributor(s): Peter Schlaile <peter [at] schlaile [dot] de> 2005/2006
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <string.h>
@@ -35,6 +32,7 @@
 #include "MEM_guardedalloc.h"
 #include "PIL_dynlib.h"
 
+#include "DNA_scene_types.h"
 #include "DNA_sequence_types.h"
 
 #include "BLI_blenlib.h"
@@ -103,7 +101,7 @@ static void open_plugin_seq(PluginSeq *pis, const char *seqname)
 
 		if (version != 0) {
 			pis->version= version();
-			if (pis->version >= 2 && pis->version <= 5) {
+			if (pis->version >= 2 && pis->version <= 6) {
 				int (*info_func)(PluginInfo *);
 				PluginInfo *info= (PluginInfo*) MEM_mallocN(sizeof(PluginInfo), "plugin_info");
 
@@ -785,9 +783,9 @@ static void do_cross_effect(Sequence * seq,int cfra,
    ********************************************************************** */
 
 /* copied code from initrender.c */
-static unsigned short *gamtab = 0;
-static unsigned short *igamtab1 = 0;
-static int gamma_tabs_refcount = 0;
+static unsigned short gamtab[65536];
+static unsigned short igamtab1[256];
+static int gamma_tabs_init = FALSE;
 
 #define RE_GAMMA_TABLE_SIZE 400
 
@@ -881,9 +879,6 @@ static void gamtabs(float gamma)
 	float val, igamma= 1.0f/gamma;
 	int a;
 	
-	gamtab= MEM_mallocN(65536*sizeof(short), "initGaus2");
-	igamtab1= MEM_mallocN(256*sizeof(short), "initGaus2");
-
 	/* gamtab: in short, out short */
 	for(a=0; a<65536; a++) {
 		val= a;
@@ -906,36 +901,25 @@ static void gamtabs(float gamma)
 
 }
 
-static void alloc_or_ref_gammatabs()
+static void build_gammatabs()
 {
-	if (gamma_tabs_refcount == 0) {
+	if (gamma_tabs_init == FALSE) {
 		gamtabs(2.0f);
 		makeGammaTables(2.0f);
+		gamma_tabs_init = TRUE;
 	}
-	gamma_tabs_refcount++;
 }
 
 static void init_gammacross(Sequence * seq)
 {
-	alloc_or_ref_gammatabs();
 }
 
 static void load_gammacross(Sequence * seq)
 {
-	alloc_or_ref_gammatabs();
 }
 
 static void free_gammacross(Sequence * seq)
 {
-	if (--gamma_tabs_refcount == 0) {
-		MEM_freeN(gamtab);
-		MEM_freeN(igamtab1);
-		gamtab = 0;
-		igamtab1 = 0;
-	}
-	if (gamma_tabs_refcount < 0) {
-		fprintf(stderr, "seqeffects: free_gammacross double free!\n");
-	}
 }
 
 static void do_gammacross_effect_byte(float facf0, float facf1, 
@@ -999,7 +983,7 @@ static void do_gammacross_effect_float(float facf0, float facf1,
 				       float *rect1, float *rect2, 
 				       float *out)
 {
-	float fac1, fac2, col;
+	float fac1, fac2;
 	int xo;
 	float *rt1, *rt2, *rt;
 
@@ -1028,7 +1012,7 @@ static void do_gammacross_effect_float(float facf0, float facf1,
 		x= xo * 4;
 		while(x--) {
 
-			col= gammaCorrect(
+			*rt= gammaCorrect(
 				fac1*invGammaCorrect(*rt1) 
 				+ fac2*invGammaCorrect(*rt2));
 
@@ -1042,6 +1026,8 @@ static void do_gammacross_effect(Sequence * seq,int cfra,
 				 struct ImBuf *ibuf1, struct ImBuf *ibuf2, 
 				 struct ImBuf *ibuf3, struct ImBuf *out)
 {
+	build_gammatabs();
+
 	if (out->rect_float) {
 		do_gammacross_effect_float(
 			facf0, facf1, x, y,
@@ -1512,12 +1498,38 @@ static void do_mul_effect(Sequence * seq,int cfra,
    WIPE
    ********************************************************************** */
 
+typedef struct WipeZone {
+	float angle;
+	int flip;
+	int xo, yo;
+	int width;
+	float invwidth;
+	float pythangle;
+} WipeZone;
+
+static void precalc_wipe_zone(WipeZone *wipezone, WipeVars *wipe, int xo, int yo)
+{
+	wipezone->flip = (wipe->angle < 0);
+	wipezone->angle = pow(fabs(wipe->angle)/45.0f, log(xo)/log(2.0f));
+	wipezone->xo = xo;
+	wipezone->yo = yo;
+	wipezone->width = (int)(wipe->edgeWidth*((xo+yo)/2.0f));
+	wipezone->pythangle = 1.0f/sqrt(wipe->angle*wipe->angle + 1.0f);
+
+	if(wipe->wipetype == DO_SINGLE_WIPE)
+		wipezone->invwidth = 1.0f/wipezone->width;
+	else
+		wipezone->invwidth = 1.0f/(0.5f*wipezone->width);
+}
+
 // This function calculates the blur band for the wipe effects
-static float in_band(float width,float dist, float perc,int side,int dir){
-	
+static float in_band(WipeZone *wipezone,float width,float dist,float perc,int side,int dir)
+{
 	float t1,t2,alpha,percwidth;
+
 	if(width == 0)
 		return (float)side;
+
 	if(side == 1)
 		percwidth = width * perc;
 	else
@@ -1526,8 +1538,8 @@ static float in_band(float width,float dist, float perc,int side,int dir){
 	if(width < dist)
 		return side;
 	
-	t1 = dist / width;  //percentange of width that is
-	t2 = 1 / width;  //amount of alpha per % point
+	t1 = dist * wipezone->invwidth;  //percentange of width that is
+	t2 = wipezone->invwidth;  //amount of alpha per % point
 	
 	if(side == 1)
 		alpha = (t1*t2*100) + (1-perc); // add point's alpha contrib to current position in wipe
@@ -1536,31 +1548,31 @@ static float in_band(float width,float dist, float perc,int side,int dir){
 	
 	if(dir == 0)
 		alpha = 1-alpha;
+
 	return alpha;
 }
 
-static float check_zone(int x, int y, int xo, int yo, 
-			Sequence *seq, float facf0) 
+static float check_zone(WipeZone *wipezone, int x, int y,
+	Sequence *seq, float facf0) 
 {
 	float posx, posy,hyp,hyp2,angle,hwidth,b1,b2,b3,pointdist;
 /*some future stuff
 float hyp3,hyp4,b4,b5	   
 */
 	float temp1,temp2,temp3,temp4; //some placeholder variables
-	float halfx = xo/2;
-	float halfy = yo/2;
+	int xo = wipezone->xo;
+	int yo = wipezone->yo;
+	float halfx = xo*0.5f;
+	float halfy = yo*0.5f;
 	float widthf,output=0;
 	WipeVars *wipe = (WipeVars *)seq->effectdata;
 	int width;
 
- 	angle = wipe->angle;
- 	if(angle < 0){
- 		x = xo-x;
- 		//y = yo-y
- 		}
- 	angle = pow(fabs(angle)/45,log(xo)/log(2));
+	if(wipezone->flip) x = xo - x;
+	angle = wipezone->angle;
 
 	posy = facf0 * yo;
+
 	if(wipe->forward){
 		posx = facf0 * xo;
 		posy = facf0 * yo;
@@ -1568,64 +1580,83 @@ float hyp3,hyp4,b4,b5
 		posx = xo - facf0 * xo;
 		posy = yo - facf0 * yo;
 	}
+
 	switch (wipe->wipetype) {
 		case DO_SINGLE_WIPE:
-			width = (int)(wipe->edgeWidth*((xo+yo)/2.0));
-			hwidth = (float)width/2.0;
+			width = wipezone->width;
+			hwidth = width*0.5f;
 
-			if (angle == 0.0)angle = 0.000001;
-			b1 = posy - (-angle)*posx;
-			b2 = y - (-angle)*x;
-			hyp = fabs(angle*x+y+(-posy-angle*posx))/sqrt(angle*angle+1);
-			if(angle < 0){
+			if(angle == 0.0f) {
+				b1 = posy;
+				b2 = y;
+				hyp = fabs(y - posy);
+			}
+			else {
+				b1 = posy - (-angle)*posx;
+				b2 = y - (-angle)*x;
+				hyp = fabs(angle*x+y+(-posy-angle*posx))*wipezone->pythangle;
+			}
+
+			if(angle < 0) {
 				temp1 = b1;
 				b1 = b2;
 				b2 = temp1;
 			}
-			if(wipe->forward){	 
+
+			if(wipe->forward) {
 				if(b1 < b2)
-					output = in_band(width,hyp,facf0,1,1);
+					output = in_band(wipezone,width,hyp,facf0,1,1);
 				else
-					output = in_band(width,hyp,facf0,0,1);
-		 	} else{	 
+					output = in_band(wipezone,width,hyp,facf0,0,1);
+		 	}
+			else {
 				if(b1 < b2)
-					output = in_band(width,hyp,facf0,0,1);
+					output = in_band(wipezone,width,hyp,facf0,0,1);
 				else
-					output = in_band(width,hyp,facf0,1,1);
+					output = in_band(wipezone,width,hyp,facf0,1,1);
 		 	}
 		break;
 	 
-	 
 		case DO_DOUBLE_WIPE:
-			if(!wipe->forward)facf0 = 1-facf0;   // Go the other direction
+			if(!wipe->forward)
+				facf0 = 1.0f-facf0;   // Go the other direction
 
-			width = (int)(wipe->edgeWidth*((xo+yo)/2.0));  // calculate the blur width
-			hwidth = (float)width/2.0;
-			if (angle == 0)angle = 0.000001;
-			b1 = posy/2 - (-angle)*posx/2;
-			b3 = (yo-posy/2) - (-angle)*(xo-posx/2);
-			b2 = y - (-angle)*x;
+			width = wipezone->width;  // calculate the blur width
+			hwidth = width*0.5f;
+			if (angle == 0) {
+				b1 = posy*0.5f;
+				b3 = yo-posy*0.5f;
+				b2 = y;
 
-			hyp = abs(angle*x+y+(-posy/2-angle*posx/2))/sqrt(angle*angle+1);
-			hyp2 = abs(angle*x+y+(-(yo-posy/2)-angle*(xo-posx/2)))/sqrt(angle*angle+1);
+				hyp = abs(y - posy*0.5f);
+				hyp2 = abs(y - (yo-posy*0.5f));
+			}
+			else {
+				b1 = posy*0.5f - (-angle)*posx*0.5f;
+				b3 = (yo-posy*0.5f) - (-angle)*(xo-posx*0.5f);
+				b2 = y - (-angle)*x;
 
-			temp1 = xo*(1-facf0/2)-xo*facf0/2;
-			temp2 = yo*(1-facf0/2)-yo*facf0/2;
+				hyp = abs(angle*x+y+(-posy*0.5f-angle*posx*0.5f))*wipezone->pythangle;
+				hyp2 = abs(angle*x+y+(-(yo-posy*0.5f)-angle*(xo-posx*0.5f)))*wipezone->pythangle;
+			}
+
+			temp1 = xo*(1-facf0*0.5f)-xo*facf0*0.5f;
+			temp2 = yo*(1-facf0*0.5f)-yo*facf0*0.5f;
 			pointdist = sqrt(temp1*temp1 + temp2*temp2);
 
 			if(b2 < b1 && b2 < b3 ){
 				if(hwidth < pointdist)
-					output = in_band(hwidth,hyp,facf0,0,1);
+					output = in_band(wipezone,hwidth,hyp,facf0,0,1);
 			} else if(b2 > b1 && b2 > b3 ){
 				if(hwidth < pointdist)
-					output = in_band(hwidth,hyp2,facf0,0,1);	
+					output = in_band(wipezone,hwidth,hyp2,facf0,0,1);	
 			} else {
 				if(  hyp < hwidth && hyp2 > hwidth )
-					output = in_band(hwidth,hyp,facf0,1,1);
+					output = in_band(wipezone,hwidth,hyp,facf0,1,1);
 				else if( hyp > hwidth && hyp2 < hwidth )
-				 	 output = in_band(hwidth,hyp2,facf0,1,1);
+				 	 output = in_band(wipezone,hwidth,hyp2,facf0,1,1);
 				else
-				 	 output = in_band(hwidth,hyp2,facf0,1,1) * in_band(hwidth,hyp,facf0,1,1);
+				 	 output = in_band(wipezone,hwidth,hyp2,facf0,1,1) * in_band(wipezone,hwidth,hyp,facf0,1,1);
 			}
 			if(!wipe->forward)output = 1-output;
 	 	break;
@@ -1636,31 +1667,31 @@ float hyp3,hyp4,b4,b5
 	 	 		temp3: angle of low side of blur
 	 	 		temp4: angle of high side of blur
 	 	 	*/
-		 	output = 1-facf0;
-	 		widthf = wipe->edgeWidth*2*3.14159;
- 	 		temp1 = 2 * 3.14159 * facf0;
+		 	output = 1.0f - facf0;
+	 		widthf = wipe->edgeWidth*2.0f*(float)M_PI;
+ 	 		temp1 = 2.0f * (float)M_PI * facf0;
 	 	 	
  			if(wipe->forward){
- 				temp1 = 2*3.14159-temp1;
+ 				temp1 = 2.0f*(float)M_PI - temp1;
  			}
  	 		
  	 		x = x - halfx;
  	 		y = y - halfy;
 
  	 		temp2 = asin(abs(y)/sqrt(x*x + y*y));
- 	 		if(x <= 0 && y >= 0) temp2 = 3.14159 - temp2;
- 	 		else if(x<=0 && y <= 0) temp2 += 3.14159;
- 	 		else if(x >= 0 && y <= 0) temp2 = 2*3.14159 - temp2;
+ 	 		if(x <= 0 && y >= 0) temp2 = (float)M_PI - temp2;
+ 	 		else if(x<=0 && y <= 0) temp2 += (float)M_PI;
+ 	 		else if(x >= 0 && y <= 0) temp2 = 2.0f*(float)M_PI - temp2;
 
  	 		if(wipe->forward){
-	 	 		temp3 = temp1-(widthf/2)*facf0;
-	 	 		temp4 = temp1+(widthf/2)*(1-facf0);
+	 	 		temp3 = temp1-(widthf*0.5f)*facf0;
+	 	 		temp4 = temp1+(widthf*0.5f)*(1-facf0);
  	 		} else{
-	 	 		temp3 = temp1-(widthf/2)*(1-facf0);
-	 	 		temp4 = temp1+(widthf/2)*facf0;
+	 	 		temp3 = temp1-(widthf*0.5f)*(1-facf0);
+	 	 		temp4 = temp1+(widthf*0.5f)*facf0;
 			}
  	 		if (temp3 < 0) temp3 = 0;
- 	 		if (temp4 > 2*3.14159) temp4 = 2*3.14159;
+ 	 		if (temp4 > 2.0f*(float)M_PI) temp4 = 2.0f*(float)M_PI;
  	 		
  	 		
  	 		if(temp2 < temp3) output = 0;
@@ -1684,8 +1715,8 @@ float hyp3,hyp4,b4,b5
 			b3 = (yo-posy/2) - (-angle)*(xo-posx/2);
 			b2 = y - (-angle)*x;
 
-			hyp = abs(angle*x+y+(-posy/2-angle*posx/2))/sqrt(angle*angle+1);
-			hyp2 = abs(angle*x+y+(-(yo-posy/2)-angle*(xo-posx/2)))/sqrt(angle*angle+1);
+			hyp = abs(angle*x+y+(-posy/2-angle*posx/2))*wipezone->pythangle;
+			hyp2 = abs(angle*x+y+(-(yo-posy/2)-angle*(xo-posx/2)))*wipezone->pythangle;
 
 			temp1 = xo*(1-facf0/2)-xo*facf0/2;
 			temp2 = yo*(1-facf0/2)-yo*facf0/2;
@@ -1693,17 +1724,17 @@ float hyp3,hyp4,b4,b5
 
 			if(b2 < b1 && b2 < b3 ){
 				if(hwidth < pointdist)
-					output = in_band(hwidth,hyp,facf0,0,1);
+					output = in_band(wipezone,hwidth,hyp,facf0,0,1);
 			} else if(b2 > b1 && b2 > b3 ){
 				if(hwidth < pointdist)
-					output = in_band(hwidth,hyp2,facf0,0,1);	
+					output = in_band(wipezone,hwidth,hyp2,facf0,0,1);	
 			} else {
 				if( hyp < hwidth && hyp2 > hwidth )
-					output = in_band(hwidth,hyp,facf0,1,1);
+					output = in_band(wipezone,hwidth,hyp,facf0,1,1);
 				else if( hyp > hwidth && hyp2 < hwidth )
-				 	output = in_band(hwidth,hyp2,facf0,1,1);
+				 	output = in_band(wipezone,hwidth,hyp2,facf0,1,1);
 				else
-				 	output = in_band(hwidth,hyp2,facf0,1,1) * in_band(hwidth,hyp,facf0,1,1);
+				 	output = in_band(wipezone,hwidth,hyp2,facf0,1,1) * in_band(wipezone,hwidth,hyp,facf0,1,1);
 			}
 
 			if(invert)facf0 = 1-facf0;
@@ -1712,22 +1743,22 @@ float hyp3,hyp4,b4,b5
 			b3 = (yo-posy/2) - (-angle)*(xo-posx/2);
 			b2 = y - (-angle)*x;
 
-			hyp = abs(angle*x+y+(-posy/2-angle*posx/2))/sqrt(angle*angle+1);
-			hyp2 = abs(angle*x+y+(-(yo-posy/2)-angle*(xo-posx/2)))/sqrt(angle*angle+1);
+			hyp = abs(angle*x+y+(-posy/2-angle*posx/2))*wipezone->pythangle;
+			hyp2 = abs(angle*x+y+(-(yo-posy/2)-angle*(xo-posx/2)))*wipezone->pythangle;
 
 			if(b2 < b1 && b2 < b3 ){
 				if(hwidth < pointdist)
-					output *= in_band(hwidth,hyp,facf0,0,1);
+					output *= in_band(wipezone,hwidth,hyp,facf0,0,1);
 			} else if(b2 > b1 && b2 > b3 ){
 				if(hwidth < pointdist)
-					output *= in_band(hwidth,hyp2,facf0,0,1);	
+					output *= in_band(wipezone,hwidth,hyp2,facf0,0,1);	
 			} else {
 				if( hyp < hwidth && hyp2 > hwidth )
-					output *= in_band(hwidth,hyp,facf0,1,1);
+					output *= in_band(wipezone,hwidth,hyp,facf0,1,1);
 				else if( hyp > hwidth && hyp2 < hwidth )
-					output *= in_band(hwidth,hyp2,facf0,1,1);
+					output *= in_band(wipezone,hwidth,hyp2,facf0,1,1);
 				else
-					output *= in_band(hwidth,hyp2,facf0,1,1) * in_band(hwidth,hyp,facf0,1,1);
+					output *= in_band(wipezone,hwidth,hyp2,facf0,1,1) * in_band(wipezone,hwidth,hyp,facf0,1,1);
 			}
 
 		break;
@@ -1738,15 +1769,15 @@ float hyp3,hyp4,b4,b5
 
 			if(!wipe->forward) facf0 = 1-facf0;
 
-			width = (int)(wipe->edgeWidth*((xo+yo)/2.0));
-			hwidth = (float)width/2.0; 
+			width = wipezone->width;
+			hwidth = width*0.5f;
 
 			temp1 = (halfx-(halfx)*facf0);
 		 	pointdist = sqrt(temp1*temp1 + temp1*temp1);
 		 
 		 	temp2 = sqrt((halfx-x)*(halfx-x) + (halfy-y)*(halfy-y));
-		 	if(temp2 > pointdist) output = in_band(hwidth,fabs(temp2-pointdist),facf0,0,1);
-		 	else output = in_band(hwidth,fabs(temp2-pointdist),facf0,1,1);
+		 	if(temp2 > pointdist) output = in_band(wipezone,hwidth,fabs(temp2-pointdist),facf0,0,1);
+		 	else output = in_band(wipezone,hwidth,fabs(temp2-pointdist),facf0,1,1);
 		 
 			if(!wipe->forward) output = 1-output;
 			
@@ -1784,8 +1815,13 @@ static void do_wipe_effect_byte(Sequence *seq, float facf0, float facf1,
 				unsigned char *rect1, 
 				unsigned char *rect2, unsigned char *out)
 {
+	WipeZone wipezone;
+	WipeVars *wipe = (WipeVars *)seq->effectdata;
 	int xo, yo;
 	char *rt1, *rt2, *rt;
+
+	precalc_wipe_zone(&wipezone, wipe, x, y);
+
 	rt1 = (char *)rect1;
 	rt2 = (char *)rect2;
 	rt = (char *)out;
@@ -1794,7 +1830,7 @@ static void do_wipe_effect_byte(Sequence *seq, float facf0, float facf1,
 	yo = y;
 	for(y=0;y<yo;y++) {
 		for(x=0;x<xo;x++) {
-			float check = check_zone(x,y,xo,yo,seq,facf0);
+			float check = check_zone(&wipezone,x,y,seq,facf0);
 			if (check) {
 				if (rt1) {
 					rt[0] = (int)(rt1[0]*check)+ (int)(rt2[0]*(1-check));
@@ -1837,8 +1873,13 @@ static void do_wipe_effect_float(Sequence *seq, float facf0, float facf1,
 				 float *rect1, 
 				 float *rect2, float *out)
 {
+	WipeZone wipezone;
+	WipeVars *wipe = (WipeVars *)seq->effectdata;
 	int xo, yo;
 	float *rt1, *rt2, *rt;
+
+	precalc_wipe_zone(&wipezone, wipe, x, y);
+
 	rt1 = rect1;
 	rt2 = rect2;
 	rt = out;
@@ -1847,7 +1888,7 @@ static void do_wipe_effect_float(Sequence *seq, float facf0, float facf1,
 	yo = y;
 	for(y=0;y<yo;y++) {
 		for(x=0;x<xo;x++) {
-			float check = check_zone(x,y,xo,yo,seq,facf0);
+			float check = check_zone(&wipezone,x,y,seq,facf0);
 			if (check) {
 				if (rt1) {
 					rt[0] = rt1[0]*check+ rt2[0]*(1-check);
@@ -1926,6 +1967,8 @@ static void init_transform_effect(Sequence *seq)
 	scale->rotIni=0;
 	scale->rotFin=0;
 	
+	scale->interpolation=1;
+	scale->percent=1;
 }
 
 static int num_inputs_transform()
@@ -1944,49 +1987,8 @@ static void copy_transform_effect(Sequence *dst, Sequence *src)
 	dst->effectdata = MEM_dupallocN(src->effectdata);
 }
 
-/* function assumes out to be zero'ed, only does RGBA */
-static void bilinear_interpolation_transform_float(ImBuf *in, float *out, float u, float v)
-{
-	float *row1, *row2, *row3, *row4, a, b;
-	float a_b, ma_b, a_mb, ma_mb;
-	float empty[4]= {0.0f, 0.0f, 0.0f, 0.0f};
-	int y1, y2, x1, x2;
-
-	x1= (int)floor(u);
-	x2= (int)ceil(u);
-	y1= (int)floor(v);
-	y2= (int)ceil(v);
-
-	/* sample area entirely outside image? */
-	if(x2<0 || x1>in->x-1 || y2<0 || y1>in->y-1)
-		return;
-	
-	/* sample including outside of edges of image */
-	if(x1<0 || y1<0) row1= empty;
-	else row1= in->rect_float + in->x * y1 * 4 + 4*x1;
-	
-	if(x1<0 || y2>in->y-1) row2= empty;
-	else row2= in->rect_float + in->x * y2 * 4 + 4*x1;
-	
-	if(x2>in->x-1 || y1<0) row3= empty;
-	else row3= in->rect_float + in->x * y1 * 4 + 4*x2;
-	
-	if(x2>in->x-1 || y2>in->y-1) row4= empty;
-	else row4= in->rect_float + in->x * y2 * 4 + 4*x2;
-	
-	a= u-floor(u);
-	b= v-floor(v);
-	a_b= a*b; ma_b= (1.0f-a)*b; a_mb= a*(1.0f-b); ma_mb= (1.0f-a)*(1.0f-b);
-	
-	out[0]= ma_mb*row1[0] + a_mb*row3[0] + ma_b*row2[0]+ a_b*row4[0];
-	out[1]= ma_mb*row1[1] + a_mb*row3[1] + ma_b*row2[1]+ a_b*row4[1];
-	out[2]= ma_mb*row1[2] + a_mb*row3[2] + ma_b*row2[2]+ a_b*row4[2];
-	out[3]= ma_mb*row1[3] + a_mb*row3[3] + ma_b*row2[3]+ a_b*row4[3];
-}
-
-
-static void do_transform_effect_float(Sequence * seq,float facf0, int x, int y, 
-			   struct ImBuf *ibuf1,float *out)
+static void do_transform(Sequence * seq,float facf0, int x, int y, 
+			  struct ImBuf *ibuf1,struct ImBuf *out)
 {
 	int xo, yo, xi, yi;
 	float xs,ys,factxScale,factyScale,tx,ty,rad,s,c,xaux,yaux,factRot,px,py;
@@ -1996,15 +1998,20 @@ static void do_transform_effect_float(Sequence * seq,float facf0, int x, int y,
 	xo = x;
 	yo = y;
 
-	/*factor scale*/
+	//factor scale
 	factxScale = scale->ScalexIni + (scale->ScalexFin - scale->ScalexIni) * facf0;
 	factyScale = scale->ScaleyIni + (scale->ScaleyFin - scale->ScaleyIni) * facf0;
 
-	/*Factor translate*/
-	tx = scale->xIni+(xo / 2.0f) + (scale->xFin-(xo / 2.0f) - scale->xIni+(xo / 2.0f)) * facf0;
-	ty = scale->yIni+(yo / 2.0f) + (scale->yFin-(yo / 2.0f) - scale->yIni+(yo / 2.0f)) * facf0;
+	//Factor translate
+	if(!scale->percent){
+		tx = scale->xIni+(xo / 2.0f) + (scale->xFin-(xo / 2.0f) - scale->xIni+(xo / 2.0f)) * facf0;
+		ty = scale->yIni+(yo / 2.0f) + (scale->yFin-(yo / 2.0f) - scale->yIni+(yo / 2.0f)) * facf0;
+	}else{
+		tx = xo*(scale->xIni/100.0)+(xo / 2.0f) + (xo*(scale->xFin/100.0)-(xo / 2.0f) - xo*(scale->xIni/100.0)+(xo / 2.0f)) * facf0;
+		ty = yo*(scale->yIni/100.0)+(yo / 2.0f) + (yo*(scale->yFin/100.0)-(yo / 2.0f) - yo*(scale->yIni/100.0)+(yo / 2.0f)) * facf0;
+	}
 
-	/*factor Rotate*/
+	//factor Rotate
 	factRot = scale->rotIni + (scale->rotFin - scale->rotIni) * facf0;
 	rad = (M_PI * factRot) / 180.0f;
 	s= sin(rad);
@@ -2012,162 +2019,29 @@ static void do_transform_effect_float(Sequence * seq,float facf0, int x, int y,
 
 	for (yi = 0; yi < yo; yi++) {
 		for (xi = 0; xi < xo; xi++) {
-			/*tranlate point*/
+			//tranlate point
 			px = xi-tx;
 			py = yi-ty;
 
-			/*rotate point with center ref*/
+			//rotate point with center ref
 			xaux = c*px + py*s ;
 			yaux = -s*px + c*py;
 
-			/*scale point with center ref*/
+			//scale point with center ref
 			xs = xaux / factxScale;
 			ys = yaux / factyScale;
 
-			/*undo reference center point */
+			//undo reference center point 
 			xs += (xo / 2.0f);
 			ys += (yo / 2.0f);
 
-			/*interpolate*/
-			bilinear_interpolation_transform_float(ibuf1,out, xs,ys);
-			
-			out+=4;
-		}
-	}	
-	
-}
-
-/* function assumes out to be zero'ed, only does RGBA */
-static void bilinear_interpolation_transform_byte(unsigned char *in, unsigned char *out, float u, float v,int x,int y)
-{
-	float a, b;
-	float a_b, ma_b, a_mb, ma_mb;
-	int y1, y2, x1, x2;
-
-	int row1R,row1G,row1B,row1A;
-	int row2R,row2G,row2B,row2A;
-	int row3R,row3G,row3B,row3A;
-	int row4R,row4G,row4B,row4A;
-
-	x1= (int)floor(u);
-	x2= (int)ceil(u);
-	y1= (int)floor(v);
-	y2= (int)ceil(v);
-
-	/* sample area entirely outside image? */
-	if(x2<0 || x1>x-1 || y2<0 || y1>y-1)
-		return;
-	
-	/* sample including outside of edges of image */
-	if(x1<0 || y1<0){
-		row1R = 0;
-		row1G = 0;
-		row1B = 0;
-		row1A = 0;
-	}
-	else{ 
-		row1R= in[x * y1 * 4 + 4 * x1];
-		row1G= in[x * y1 * 4 + 4 * x1 + 1];
-		row1B= in[x * y1 * 4 + 4 * x1 + 2];
-		row1A= in[x * y1 * 4 + 4 * x1 + 3];
-	}
-	
-	if(x1<0 || y2>y-1){
-		row2R = 0;
-		row2G = 0;
-		row2B = 0;
-		row2A = 0;
-	}
-	else{ 
-		row2R= in[x * y2 * 4 + 4 * x1];
-		row2G= in[x * y2 * 4 + 4 * x1 + 1];
-		row2B= in[x * y2 * 4 + 4 * x1 + 2];
-		row2A= in[x * y2 * 4 + 4 * x1 + 3];
-	}
-	
-	if(x2>x-1 || y1<0){
-		row3R = 0;
-		row3G = 0;
-		row3B = 0;
-		row3A = 0;
-	}
-	else{ 
-		row3R= in[x * y1 * 4 + 4 * x2];
-		row3G= in[x * y1 * 4 + 4 * x2 + 1];
-		row3B= in[x * y1 * 4 + 4 * x2 + 2];
-		row3A= in[x * y1 * 4 + 4 * x2 + 3];
-	}
-	
-	if(x2>x-1 || y2>y-1){
-		row4R = 0;
-		row4G = 0;
-		row4B = 0;
-		row4A = 0;
-	}
-	else{ 
-		row4R= in[x * y2 * 4 + 4 * x2];
-		row4G= in[x * y2 * 4 + 4 * x2 + 1];
-		row4B= in[x * y2 * 4 + 4 * x2 + 2];
-		row4A= in[x * y2 * 4 + 4 * x2 + 3];
-	}
-	
-	a= u-floor(u);
-	b= v-floor(v);
-	a_b= a*b; ma_b= (1-a)*b; a_mb= a*(1-b); ma_mb= (1-a)*(1-b);
-	
-	out[0]= (int)(ma_mb*row1R + a_mb*row3R + ma_b*row2R + a_b*row4R);
-	out[1]= (int)(ma_mb*row1G + a_mb*row3G + ma_b*row2G + a_b*row4G);
-	out[2]= (int)(ma_mb*row1B + a_mb*row3B + ma_b*row2B + a_b*row4B);
-	out[3]= (int)(ma_mb*row1A + a_mb*row3A + ma_b*row2A + a_b*row4A);
-}
-
-static void do_transform_effect_byte(Sequence * seq,float facf0, int x, int y, 
-			  unsigned char *ibuf1,unsigned char *out)
-{
-	int xo, yo, xi, yi;
-	float xs,ys,factxScale,factyScale,tx,ty,rad,s,c,xaux,yaux,factRot,px,py;
-	TransformVars *scale;
-	
-	scale = (TransformVars *)seq->effectdata;
-	xo = x;
-	yo = y;
-
-	/*factor scale*/
-	factxScale = scale->ScalexIni + (scale->ScalexFin - scale->ScalexIni) * facf0;
-	factyScale = scale->ScaleyIni + (scale->ScaleyFin - scale->ScaleyIni) * facf0;
-
-	/*Factor translate*/
-	tx = scale->xIni+(xo / 2.0f) + (scale->xFin-(xo / 2.0f) - scale->xIni+(xo / 2.0f)) * facf0;
-	ty = scale->yIni+(yo / 2.0f) + (scale->yFin-(yo / 2.0f) - scale->yIni+(yo / 2.0f)) * facf0;
-
-	/*factor Rotate*/
-	factRot = scale->rotIni + (scale->rotFin - scale->rotIni) * facf0;
-	rad = (M_PI * factRot) / 180.0f;
-	s= sin(rad);
-	c= cos(rad);
-
-	for (yi = 0; yi < yo; yi++) {
-		for (xi = 0; xi < xo; xi++) {
-			/*tranlate point*/
-			px = xi-tx;
-			py = yi-ty;
-
-			/*rotate point with center ref*/
-			xaux = c*px + py*s ;
-			yaux = -s*px + c*py;
-
-			/*scale point with center ref*/
-			xs = xaux / factxScale;
-			ys = yaux / factyScale;
-
-			/*undo reference center point */
-			xs += (xo / 2.0f);
-			ys += (yo / 2.0f);
-
-			/*interpolate*/
-			bilinear_interpolation_transform_byte(ibuf1,out, xs,ys,x,y);
-			
-			out+=4;
+			//interpolate
+			if(scale->interpolation==0)
+				neareast_interpolation(ibuf1,out, xs,ys,xi,yi);
+			if(scale->interpolation==1)
+				bilinear_interpolation(ibuf1,out, xs,ys,xi,yi);
+			if(scale->interpolation==2)
+				bicubic_interpolation(ibuf1,out, xs,ys,xi,yi);
 		}
 	}	
 
@@ -2177,17 +2051,7 @@ static void do_transform_effect(Sequence * seq,int cfra,
 			   struct ImBuf *ibuf1, struct ImBuf *ibuf2, 
 			   struct ImBuf *ibuf3, struct ImBuf *out)
 {
-	if (out->rect_float) {
-		do_transform_effect_float(seq,
-			facf0,x, y,
-			ibuf1,
-			out->rect_float);
-	} else {
-		do_transform_effect_byte(seq,
-			facf0,x, y,
-			(unsigned char*) ibuf1->rect,
-			(unsigned char*) out->rect);
-	}
+	do_transform(seq, facf0, x, y, ibuf1, out);
 }
 
 
@@ -2671,9 +2535,10 @@ static void do_glow_effect_byte(Sequence *seq, float facf0, float facf1,
 	unsigned char *outbuf=(unsigned char *)out;
 	unsigned char *inbuf=(unsigned char *)rect1;
 	GlowVars *glow = (GlowVars *)seq->effectdata;
+	struct RenderData *rd = &G.scene->r;
 
-	RVIsolateHighlights_byte(inbuf, outbuf , x, y, glow->fMini*765, glow->fBoost, glow->fClamp);
-	RVBlurBitmap2_byte (outbuf, x, y, glow->dDist,glow->dQuality);
+	RVIsolateHighlights_byte(inbuf, outbuf , x, y, glow->fMini*765, glow->fBoost * facf0, glow->fClamp);
+	RVBlurBitmap2_byte (outbuf, x, y, glow->dDist * (rd->size / 100.0f),glow->dQuality);
 	if (!glow->bNoComp)
 		RVAddBitmaps_byte (inbuf , outbuf, outbuf, x, y);
 }
@@ -2685,9 +2550,10 @@ static void do_glow_effect_float(Sequence *seq, float facf0, float facf1,
 	float *outbuf = out;
 	float *inbuf = rect1;
 	GlowVars *glow = (GlowVars *)seq->effectdata;
+	struct RenderData *rd = &G.scene->r;
 
-	RVIsolateHighlights_float(inbuf, outbuf , x, y, glow->fMini*3.0f, glow->fBoost, glow->fClamp);
-	RVBlurBitmap2_float (outbuf, x, y, glow->dDist,glow->dQuality);
+	RVIsolateHighlights_float(inbuf, outbuf , x, y, glow->fMini*3.0f, glow->fBoost * facf0, glow->fClamp);
+	RVBlurBitmap2_float (outbuf, x, y, glow->dDist * (rd->size / 100.0f),glow->dQuality);
 	if (!glow->bNoComp)
 		RVAddBitmaps_float (inbuf , outbuf, outbuf, x, y);
 }
@@ -2758,26 +2624,67 @@ static void do_solid_color(Sequence * seq,int cfra,
 	float *rect_float;
 
 	if (out->rect) {
+		unsigned char col0[3];
+		unsigned char col1[3];
+
+		col0[0] = facf0 * cv->col[0] * 255;
+		col0[1] = facf0 * cv->col[1] * 255;
+		col0[2] = facf0 * cv->col[2] * 255;
+
+		col1[0] = facf1 * cv->col[0] * 255;
+		col1[1] = facf1 * cv->col[1] * 255;
+		col1[2] = facf1 * cv->col[2] * 255;
+
 		rect = (unsigned char *)out->rect;
 		
 		for(y=0; y<out->y; y++) {	
 			for(x=0; x<out->x; x++, rect+=4) {
-				rect[0]= (char)(cv->col[0]*255);
-				rect[1]= (char)(cv->col[1]*255);
-				rect[2]= (char)(cv->col[2]*255);
+				rect[0]= col0[0];
+				rect[1]= col0[1];
+				rect[2]= col0[2];
 				rect[3]= 255;
-			}	
+			}
+			y++;
+			if (y<out->y) {
+				for(x=0; x<out->x; x++, rect+=4) {
+					rect[0]= col1[0];
+					rect[1]= col1[1];
+					rect[2]= col1[2];
+					rect[3]= 255;
+				}	
+			}
 		}
+
 	} else if (out->rect_float) {
+		float col0[3];
+		float col1[3];
+
+		col0[0] = facf0 * cv->col[0];
+		col0[1] = facf0 * cv->col[1];
+		col0[2] = facf0 * cv->col[2];
+
+		col1[0] = facf1 * cv->col[0];
+		col1[1] = facf1 * cv->col[1];
+		col1[2] = facf1 * cv->col[2];
+
 		rect_float = out->rect_float;
 		
 		for(y=0; y<out->y; y++) {	
 			for(x=0; x<out->x; x++, rect_float+=4) {
-				rect_float[0]= cv->col[0];
-				rect_float[1]= cv->col[1];
-				rect_float[2]= cv->col[2];
+				rect_float[0]= col0[0];
+				rect_float[1]= col0[1];
+				rect_float[2]= col0[2];
 				rect_float[3]= 1.0;
 			}
+			y++;
+			if (y<out->y) {
+				for(x=0; x<out->x; x++, rect_float+=4) {
+					rect_float[0]= col1[0];
+					rect_float[1]= col1[1];
+					rect_float[2]= col1[2];
+					rect_float[3]= 1.0;
+				}
+			}
 		}
 	}
 }
@@ -2907,6 +2814,7 @@ void sequence_effect_speed_rebuild_map(struct Sequence * seq, int force)
 		float cursor = 0;
 
 		v->frameMap[0] = 0;
+		v->lastValidFrame = 0;
 
 		for (cfra = 1; cfra < v->length; cfra++) {
 			if(seq->ipo) {
@@ -2933,9 +2841,11 @@ void sequence_effect_speed_rebuild_map(struct Sequence * seq, int force)
 				v->frameMap[cfra] = v->length - 1;
 			} else {
 				v->frameMap[cfra] = cursor;
+				v->lastValidFrame = cfra;
 			}
 		}
 	} else {
+		v->lastValidFrame = 0;
 		for (cfra = 0; cfra < v->length; cfra++) {
 			if(seq->ipo) {
 				if((seq->flag & SEQ_IPO_FRAME_LOCKED) != 0) {
@@ -2961,6 +2871,8 @@ void sequence_effect_speed_rebuild_map(struct Sequence * seq, int force)
 			seq->facf0 *= v->globalSpeed;
 			if (seq->facf0 >= v->length) {
 				seq->facf0 = v->length - 1;
+			} else {
+				v->lastValidFrame = cfra;
 			}
 			v->frameMap[cfra] = seq->facf0;
 		}
@@ -3183,11 +3095,33 @@ static struct SeqEffectHandle get_sequence_effect_impl(int seq_type)
 
 struct SeqEffectHandle get_sequence_effect(Sequence * seq)
 {
-	struct SeqEffectHandle rval = get_sequence_effect_impl(seq->type);
+	struct SeqEffectHandle rval;
 
-	if ((seq->flag & SEQ_EFFECT_NOT_LOADED) != 0) {
-		rval.load(seq);
-		seq->flag &= ~SEQ_EFFECT_NOT_LOADED;
+	memset(&rval, 0, sizeof(struct SeqEffectHandle));
+
+	if (seq->type & SEQ_EFFECT) {
+		rval = get_sequence_effect_impl(seq->type);
+		if ((seq->flag & SEQ_EFFECT_NOT_LOADED) != 0) {
+			rval.load(seq);
+			seq->flag &= ~SEQ_EFFECT_NOT_LOADED;
+		}
+	}
+
+	return rval;
+}
+
+struct SeqEffectHandle get_sequence_blend(Sequence * seq)
+{
+	struct SeqEffectHandle rval;
+
+	memset(&rval, 0, sizeof(struct SeqEffectHandle));
+
+	if (seq->blend_mode != 0) {
+		rval = get_sequence_effect_impl(seq->blend_mode);
+		if ((seq->flag & SEQ_EFFECT_NOT_LOADED) != 0) {
+			rval.load(seq);
+			seq->flag &= ~SEQ_EFFECT_NOT_LOADED;
+		}
 	}
 
 	return rval;
@@ -3197,5 +3131,9 @@ int get_sequence_effect_num_inputs(int seq_type)
 {
 	struct SeqEffectHandle rval = get_sequence_effect_impl(seq_type);
 
-	return rval.num_inputs();
+	int cnt = rval.num_inputs();
+	if (rval.execute) {
+		return cnt;
+	}
+	return 0;
 }
