@@ -1,15 +1,12 @@
 /**
- * $Id: spacetypes.c 8774 2006-11-07 11:24:11Z jesterking $
+ * $Id: spacetypes.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <stdlib.h>
@@ -52,6 +49,7 @@
 struct _SpaceType {
 	char			name[32];
 	
+	SpacePrefetchDrawFP winprefetchdraw;
 	SpaceDrawFP		windraw;
 	SpaceChangeFP	winchange;
 	SpaceHandleFP	winhandle;
@@ -70,8 +68,9 @@ SpaceType *spacetype_new(char *name)
 	return st;
 }
 
-void spacetype_set_winfuncs(SpaceType *st, SpaceDrawFP draw, SpaceChangeFP change, SpaceHandleFP handle) 
+void spacetype_set_winfuncs(SpaceType *st, SpacePrefetchDrawFP prefetchdraw, SpaceDrawFP draw, SpaceChangeFP change, SpaceHandleFP handle) 
 {
+	st->winprefetchdraw = prefetchdraw;
 	st->windraw= draw;
 	st->winchange= change;
 	st->winhandle= handle;
@@ -105,6 +104,17 @@ static SpaceType *spacetype_from_area(ScrArea *area)
 	}
 }
 
+void scrarea_do_winprefetchdraw(ScrArea *area)
+{
+	SpaceType *st= spacetype_from_area(area);
+	
+	areawinset(area->win);
+
+	if(area->win && st->winprefetchdraw) {
+		st->winprefetchdraw(area, area->spacedata.first);
+	}
+}
+
 void scrarea_do_windraw(ScrArea *area)
 {
 	SpaceType *st= spacetype_from_area(area);
