@@ -1,15 +1,12 @@
 /**
- * $Id: transform_constraints.c,v 1.74 2007/04/04 13:18:34 campbellbarton Exp $
+ * $Id: transform_constraints.c 14826 2008-05-13 10:17:46Z theeth $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <stdlib.h>
@@ -218,7 +215,9 @@ static void planeProjection(TransInfo *t, float in[3], float out[3]) {
 	VecSubf(vec, out, in);
 
 	factor = Inpf(vec, norm);
-	if (factor == 0.0f) return; /* prevent divide by zero */
+	if (fabs(factor) <= 0.001) {
+		return; /* prevent divide by zero */
+	}
 	factor = Inpf(vec, vec) / factor;
 
 	VECCOPY(vec, norm);
@@ -243,8 +242,7 @@ static void applyAxisConstraintVec(TransInfo *t, TransData *td, float in[3], flo
 		Mat3MulVecfl(t->con.pmtx, out);
 		
 		// With snap, a projection is alright, no need to correct for view alignment
-		if ((t->tsnap.status & SNAP_ON) == 0)
-			{
+		if ((t->tsnap.status & SNAP_ON) == 0) {
 			if (getConstraintSpaceDimension(t) == 2) {
 				if (out[0] != 0.0f || out[1] != 0.0f || out[2] != 0.0f) {
 					planeProjection(t, in, out);
@@ -414,7 +412,8 @@ static void applyAxisConstraintRot(TransInfo *t, TransData *td, float vec[3])
 			VECCOPY(vec, t->con.mtx[2]);
 			break;
 		}
-		if (!(mode & CON_NOFLIP)) {
+		/* don't flip axis if asked to or if num input */
+		if (!(mode & CON_NOFLIP) && hasNumInput(&t->num) == 0) {
 			if (Inpf(vec, t->viewinv[2]) > 0.0f) {
 				VecMulf(vec, -1.0f);
 			}
@@ -537,6 +536,10 @@ void setUserConstraint(TransInfo *t, int mode, const char ftext[]) {
 		sprintf(text, ftext, "view");
 		setConstraint(t, t->spacemtx, mode, text);
 		break;
+	default: /* V3D_MANIP_CUSTOM */
+		sprintf(text, ftext, t->spacename);
+		setConstraint(t, t->spacemtx, mode, text);
+		break;
 	}
 
 	t->con.mode |= CON_USER;
@@ -592,12 +595,21 @@ void BIF_setSingleAxisConstraint(float vec[3], char *text) {
 	Mat3Ortho(space);
 
 	Mat3CpyMat3(t->con.mtx, space);
-	t->con.mode = (CON_AXIS0|CON_APPLY);
+	t->con.mode = CON_AXIS0;
+	
 	getConstraintMatrix(t);
 
+	startConstraint(t);
+	
 	/* start copying with an offset of 1, to reserve a spot for the SPACE char */
-	if(text) strncpy(t->con.text+1, text, 48);	// 50 in struct
-
+	if(text)
+	{
+		strncpy(t->con.text+1, text, 48);	/* 50 in struct */
+	}
+	else
+	{
+		t->con.text[1] = '\0'; /* No text */
+	}
 	
 	t->con.drawExtra = NULL;
 	t->con.applyVec = applyAxisConstraintVec;
@@ -616,11 +628,21 @@ void BIF_setDualAxisConstraint(float vec1[3], float vec2[3], char *text) {
 	Mat3Ortho(space);
 	
 	Mat3CpyMat3(t->con.mtx, space);
-	t->con.mode = (CON_AXIS0|CON_AXIS1|CON_APPLY);
+	t->con.mode = CON_AXIS0|CON_AXIS1;
+
 	getConstraintMatrix(t);
+
+	startConstraint(t);
 	
 	/* start copying with an offset of 1, to reserve a spot for the SPACE char */
-	if(text) strncpy(t->con.text+1, text, 48);	// 50 in struct
+	if(text)
+	{
+		strncpy(t->con.text+1, text, 48);	/* 50 in struct */
+	}
+	else
+	{
+		t->con.text[1] = '\0'; /* No text */
+	}
 
 	t->con.drawExtra = NULL;
 	t->con.applyVec = applyAxisConstraintVec;
@@ -644,7 +666,7 @@ void BIF_drawConstraint(void)
 		return;
 	
 	/* nasty exception for Z constraint in camera view */
-	if((t->flag & T_OBJECT) && G.vd->camera==OBACT && G.vd->persp>1) 
+	if((t->flag & T_OBJECT) && G.vd->camera==OBACT && G.vd->persp==V3D_CAMOB) 
 		return;
 
 	if (tc->drawExtra) {
@@ -698,7 +720,8 @@ void BIF_drawPropCircle()
 		BIF_ThemeColor(TH_GRID);
 		
 		/* if editmode we need to go into object space */
-		if(G.obedit) mymultmatrix(G.obedit->obmat);
+		if(G.obedit && t->spacetype == SPACE_VIEW3D)
+			mymultmatrix(G.obedit->obmat);
 		
 		mygetmatrix(tmat);
 		Mat4Invert(imat, tmat);
@@ -708,7 +731,8 @@ void BIF_drawPropCircle()
 		set_inverted_drawing(0);
 		
 		/* if editmode we restore */
-		if(G.obedit) myloadmatrix(G.vd->viewmat);
+		if(G.obedit && t->spacetype == SPACE_VIEW3D)
+			myloadmatrix(G.vd->viewmat);
 	}
 }
 
@@ -838,24 +862,40 @@ void postSelectConstraint(TransInfo *t)
 	t->redraw = 1;
 }
 
-void setNearestAxis(TransInfo *t)
+static void setNearestAxis2d(TransInfo *t)
+{
+	short mval[2];
+	short ival[2];
+	
+	getmouseco_areawin(mval);
+	ival[0]= t->imval[0];
+	ival[1]= t->imval[1];
+	
+	/* no correction needed... just use whichever one is lower */
+	if ( abs(mval[0]-ival[0]) < abs(mval[1]-ival[1]) ) {
+		t->con.mode |= CON_AXIS1;
+		sprintf(t->con.text, " along Y axis");
+	}
+	else {
+		t->con.mode |= CON_AXIS0;
+		sprintf(t->con.text, " along X axis");
+	}
+}
+
+static void setNearestAxis3d(TransInfo *t)
 {
 	float zfac;
 	float mvec[3], axis[3], proj[3];
 	float len[3];
 	int i, icoord[2];
 	short coord[2];
-
-	t->con.mode &= ~CON_AXIS0;
-	t->con.mode &= ~CON_AXIS1;
-	t->con.mode &= ~CON_AXIS2;
-
+	
+	/* calculate mouse movement */
 	getmouseco_areawin(coord);
 	mvec[0] = (float)(coord[0] - t->con.imval[0]);
 	mvec[1] = (float)(coord[1] - t->con.imval[1]);
 	mvec[2] = 0.0f;
-
-		
+	
 	/* we need to correct axis length for the current zoomlevel of view,
 	   this to prevent projected values to be clipped behind the camera
 	   and to overflow the short integers.
@@ -918,6 +958,25 @@ void setNearestAxis(TransInfo *t)
 			sprintf(t->con.text, " along %s Z axis", t->spacename);
 		}
 	}
+}
+
+void setNearestAxis(TransInfo *t)
+{
+	/* clear any prior constraint flags */
+	t->con.mode &= ~CON_AXIS0;
+	t->con.mode &= ~CON_AXIS1;
+	t->con.mode &= ~CON_AXIS2;
+
+	/* constraint setting - depends on spacetype */
+	if (t->spacetype == SPACE_VIEW3D) {
+		/* 3d-view */
+		setNearestAxis3d(t);	
+	}
+	else {
+		/* assume that this means a 2D-Editor */
+		setNearestAxis2d(t);
+	}
+	
 	getConstraintMatrix(t);
 }
 
