@@ -1,15 +1,12 @@
 /**
- * $Id: transform_generics.c,v 1.106 2007/04/29 18:32:40 artificer Exp $
+ * $Id: transform_generics.c 14799 2008-05-11 19:58:46Z campbellbarton $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <string.h>
@@ -42,8 +39,10 @@
 #include "DNA_lattice_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_modifier_types.h"
+#include "DNA_nla_types.h"
 #include "DNA_object_types.h"
 #include "DNA_object_force.h"
+#include "DNA_particle_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_space_types.h"
 #include "DNA_scene_types.h"
@@ -54,12 +53,18 @@
 #include "BIF_resources.h"
 #include "BIF_mywindow.h"
 #include "BIF_gl.h"
+#include "BIF_editaction.h"
 #include "BIF_editarmature.h"
 #include "BIF_editmesh.h"
+#include "BIF_editnla.h"
 #include "BIF_editsima.h"
+#include "BIF_editparticle.h"
 #include "BIF_meshtools.h"
 #include "BIF_retopo.h"
 
+#include "BSE_editipo.h"
+#include "BSE_editipo_types.h"
+
 #ifdef WITH_VERSE
 #include "BIF_verse.h"
 #endif
@@ -67,6 +72,7 @@
 #include "BKE_action.h"
 #include "BKE_anim.h"
 #include "BKE_armature.h"
+#include "BKE_cloth.h"
 #include "BKE_curve.h"
 #include "BKE_depsgraph.h"
 #include "BKE_displist.h"
@@ -75,6 +81,7 @@
 #include "BKE_group.h"
 #include "BKE_ipo.h"
 #include "BKE_lattice.h"
+#include "BKE_key.h"
 #include "BKE_mesh.h"
 #include "BKE_modifier.h"
 #include "BKE_object.h"
@@ -85,6 +92,7 @@
 #endif
 
 #include "BSE_view.h"
+#include "BSE_editaction_types.h"
 #include "BDR_unwrapper.h"
 
 #include "BLI_arithb.h"
@@ -110,7 +118,7 @@ void getViewVector(float coord[3], float vec[3])
 {
 	TransInfo *t = BIF_GetTransInfo();
 
-	if (t->persp)
+	if (t->persp != V3D_ORTHO)
 	{
 		float p1[4], p2[4];
 
@@ -139,7 +147,7 @@ void getViewVector(float coord[3], float vec[3])
 static void clipMirrorModifier(TransInfo *t, Object *ob)
 {
 	ModifierData *md= ob->modifiers.first;
-	float tolerance[3];
+	float tolerance[3] = {0.0f, 0.0f, 0.0f};
 	int axis = 0;
 
 	for (; md; md=md->next) {
@@ -147,6 +155,7 @@ static void clipMirrorModifier(TransInfo *t, Object *ob)
 			MirrorModifierData *mmd = (MirrorModifierData*) md;	
 		
 			if(mmd->flag & MOD_MIR_CLIPPING) {
+				axis = 0;
 				if(mmd->flag & MOD_MIR_AXIS_X) {
 					axis |= 1;
 					tolerance[0] = mmd->tolerance;
@@ -159,28 +168,71 @@ static void clipMirrorModifier(TransInfo *t, Object *ob)
 					axis |= 4;
 					tolerance[2] = mmd->tolerance;
 				}
-			}
-		}
-	}
-	if (axis) {
-		TransData *td = t->data;
-		int i;
+				if (axis) {
+					float mtx[4][4], imtx[4][4];
+					int i;
+					TransData *td = t->data;
 		
-		for(i = 0 ; i < t->total; i++, td++) {
-			if (td->flag & TD_NOACTION)
-				break;
-			if (td->loc==NULL)
-				break;
+					if (mmd->mirror_ob) {
+						float obinv[4][4];
+
+						Mat4Invert(obinv, mmd->mirror_ob->obmat);
+						Mat4MulMat4(mtx, ob->obmat, obinv);
+						Mat4Invert(imtx, mtx);
+					}
+
+					for(i = 0 ; i < t->total; i++, td++) {
+						int clip;
+						float loc[3], iloc[3];
+
+						if (td->flag & TD_NOACTION)
+							break;
+						if (td->loc==NULL)
+							break;
+							
+						if (td->flag & TD_SKIP)
+							continue;
 			
-			if(axis & 1) {
-				if(fabs(td->iloc[0])<=tolerance[0] || td->loc[0]*td->iloc[0]<0.0f) td->loc[0]= 0.0f;
-			}
+						VecCopyf(loc,  td->loc);
+						VecCopyf(iloc, td->iloc);
+
+						if (mmd->mirror_ob) {
+							VecMat4MulVecfl(loc, mtx, loc);
+							VecMat4MulVecfl(iloc, mtx, iloc);
+						}
+
+						clip = 0;
+						if(axis & 1) {
+							if(fabs(iloc[0])<=tolerance[0] || 
+							   loc[0]*iloc[0]<0.0f) {
+								loc[0]= 0.0f;
+								clip = 1;
+							}
+						}
 			
-			if(axis & 2) {
-				if(fabs(td->iloc[1])<=tolerance[1] || td->loc[1]*td->iloc[1]<0.0f) td->loc[1]= 0.0f;
-			}
-			if(axis & 4) {
-				if(fabs(td->iloc[2])<=tolerance[2] || td->loc[2]*td->iloc[2]<0.0f) td->loc[2]= 0.0f;
+						if(axis & 2) {
+							if(fabs(iloc[1])<=tolerance[1] || 
+							   loc[1]*iloc[1]<0.0f) {
+								loc[1]= 0.0f;
+								clip = 1;
+							}
+						}
+						if(axis & 4) {
+							if(fabs(iloc[2])<=tolerance[2] || 
+							   loc[2]*iloc[2]<0.0f) {
+								loc[2]= 0.0f;
+								clip = 1;
+							}
+						}
+						if (clip) {
+							if (mmd->mirror_ob) {
+								VecMat4MulVecfl(loc, imtx, loc);
+							}
+							VecCopyf(td->loc, loc);
+						}
+					}
+				}
+
 			}
 		}
 	}
@@ -198,8 +250,10 @@ static void editmesh_apply_to_mirror(TransInfo *t)
 			break;
 		if (td->loc==NULL)
 			break;
+		if (td->flag & TD_SKIP)
+			continue;
 		
-		eve= td->tdmir;
+		eve = td->tdmir;
 		if(eve) {
 			eve->co[0]= -td->loc[0];
 			eve->co[1]= td->loc[1];
@@ -215,33 +269,190 @@ void recalcData(TransInfo *t)
 #ifdef WITH_VERSE
 	struct TransData *td;
 #endif
+	
+	if (t->spacetype == SPACE_ACTION) {
+		Object *ob= OBACT;
+		void *data;
+		short context;
 		
-	if (G.obedit) {
+		/* determine what type of data we are operating on */
+		data = get_action_context(&context);
+		if (data == NULL) return;
+		
+		if (G.saction->lock) {
+			if (context == ACTCONT_ACTION) {
+				if(ob) {
+					ob->ctime= -1234567.0f;
+					if(ob->pose || ob_get_key(ob))
+						DAG_object_flush_update(G.scene, ob, OB_RECALC);
+					else
+						DAG_object_flush_update(G.scene, ob, OB_RECALC_OB);
+				}
+			}
+			else if (context == ACTCONT_SHAPEKEY) {
+				DAG_object_flush_update(G.scene, OBACT, OB_RECALC_OB|OB_RECALC_DATA);
+			}
+		}
+	}	
+	else if (t->spacetype == SPACE_NLA) {
+		if (G.snla->lock) {
+			for (base=G.scene->base.first; base; base=base->next) {
+				if (base->flag & BA_HAS_RECALC_OB)
+					base->object->recalc |= OB_RECALC_OB;
+				if (base->flag & BA_HAS_RECALC_DATA)
+					base->object->recalc |= OB_RECALC_DATA;
+				
+				if (base->object->recalc) 
+					base->object->ctime= -1234567.0f;	// eveil! 
+				
+				/* recalculate scale of selected nla-strips */
+				if (base->object->nlastrips.first) {
+					Object *bob= base->object;
+					bActionStrip *strip;
+					
+					for (strip= bob->nlastrips.first; strip; strip= strip->next) {
+						if (strip->flag & ACTSTRIP_SELECT) {
+							float actlen= strip->actend - strip->actstart;
+							float len= strip->end - strip->start;
+							
+							strip->scale= len / (actlen * strip->repeat);
+						}
+					}
+				}
+			}
+			
+			DAG_scene_flush_update(G.scene, screen_view3d_layers(), 0);
+		}
+		else {
+			for (base=G.scene->base.first; base; base=base->next) {
+				/* recalculate scale of selected nla-strips */
+				if (base->object->nlastrips.first) {
+					Object *bob= base->object;
+					bActionStrip *strip;
+					
+					for (strip= bob->nlastrips.first; strip; strip= strip->next) {
+						if (strip->flag & ACTSTRIP_SELECT) {
+							float actlen= strip->actend - strip->actstart;
+							float len= strip->end - strip->start;
+							
+							/* prevent 'negative' scaling */
+							if (len < 0) {
+								SWAP(float, strip->start, strip->end);
+								len= fabs(len);
+							}
+							
+							/* calculate new scale */
+							strip->scale= len / (actlen * strip->repeat);
+						}
+					}
+				}
+			}
+		}
+	}
+	else if (t->spacetype == SPACE_IPO) {
+		EditIpo *ei;
+		int dosort = 0;
+		int a;
+		
+		/* do the flush first */
+		flushTransIpoData(t);
+		
+		/* now test if there is a need to re-sort */
+		ei= G.sipo->editipo;
+		for (a=0; a<G.sipo->totipo; a++, ei++) {
+			if (ISPOIN(ei, flag & IPO_VISIBLE, icu)) {
+				
+				/* watch it: if the time is wrong: do not correct handles */
+				if (test_time_ipocurve(ei->icu)) {
+					dosort++;
+				} else {
+					calchandles_ipocurve(ei->icu);
+				}
+			}
+		}
+		
+		/* do resort and other updates? */
+		if (dosort) remake_ipo_transdata(t);
+		if (G.sipo->showkey) update_ipokey_val();
+		
+		calc_ipo(G.sipo->ipo, (float)CFRA);
+		
+		/* update realtime - not working? */
+		if (G.sipo->lock) {
+			if (G.sipo->blocktype==ID_MA || G.sipo->blocktype==ID_TE) {
+				do_ipo(G.sipo->ipo);
+			}
+			else if(G.sipo->blocktype==ID_CA) {
+				do_ipo(G.sipo->ipo);
+			}
+			else if(G.sipo->blocktype==ID_KE) {
+				Object *ob= OBACT;
+				if(ob) {
+					ob->shapeflag &= ~OB_SHAPE_TEMPLOCK;
+					DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+				}
+			}
+			else if(G.sipo->blocktype==ID_PO) {
+				Object *ob= OBACT;
+				if(ob && ob->pose) {
+					DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+				}
+			}
+			else if(G.sipo->blocktype==ID_OB) {
+				Base *base= FIRSTBASE;
+				
+				while(base) {
+					if(base->object->ipo==G.sipo->ipo) {
+						do_ob_ipo(base->object);
+						base->object->recalc |= OB_RECALC_OB;
+					}
+					base= base->next;
+				}
+				DAG_scene_flush_update(G.scene, screen_view3d_layers(), 0);
+			}
+		}
+	}
+	else if (G.obedit) {
 		if (G.obedit->type == OB_MESH) {
-			retopo_do_all();
-
-			/* mirror modifier clipping? */
-			if(t->state != TRANS_CANCEL)
-				clipMirrorModifier(t, G.obedit);
-			
-			if(G.scene->toolsettings->editbutflag & B_MESH_X_MIRROR)
-				editmesh_apply_to_mirror(t);
-			
-			DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);  /* sets recalc flags */
-			
-			recalc_editnormals();
+			if(t->spacetype==SPACE_IMAGE) {
+				flushTransUVs(t);
+				if (G.sima->flag & SI_LIVE_UNWRAP)
+					unwrap_lscm_live_re_solve();
+			} else {
+				/* mirror modifier clipping? */
+				if(t->state != TRANS_CANCEL) {
+					if ((G.qual & LR_CTRLKEY)==0) {
+						/* Only retopo if not snapping, Note, this is the only case of G.qual being used, but we have no T_SHIFT_MOD - Campbell */
+						retopo_do_all();
+					}
+					clipMirrorModifier(t, G.obedit);
+				}
+				if((t->context & CTX_NO_MIRROR) == 0 && (G.scene->toolsettings->editbutflag & B_MESH_X_MIRROR))
+					editmesh_apply_to_mirror(t);
+				
+				DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);  /* sets recalc flags */
+				
+				recalc_editnormals();
+			}
 		}
 		else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) {
 			Nurb *nu= editNurb.first;
 			DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);  /* sets recalc flags */
 			
-			while(nu) {
-				test2DNurb(nu);
-				testhandlesNurb(nu); /* test for bezier too */
-				nu= nu->next;
+			if (t->state == TRANS_CANCEL) {
+				while(nu) {
+					calchandlesNurb(nu); /* Cant do testhandlesNurb here, it messes up the h1 and h2 flags */
+					nu= nu->next;
+				}
+			} else {
+				/* Normal updating */
+				while(nu) {
+					test2DNurb(nu);
+					calchandlesNurb(nu);
+					nu= nu->next;
+				}
+				retopo_do_all();
 			}
-
-			retopo_do_all();
 		}
 		else if(G.obedit->type==OB_ARMATURE){   /* no recalc flag, does pose */
 			bArmature *arm= G.obedit->data;
@@ -302,33 +513,23 @@ void recalcData(TransInfo *t)
 		/* old optimize trick... this enforces to bypass the depgraph */
 		if (!(arm->flag & ARM_DELAYDEFORM)) {
 			DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);  /* sets recalc flags */
-			
-			/* bah, softbody exception... recalcdata doesnt reset */
-			for(base= FIRSTBASE; base; base= base->next) {
-				if(base->object->recalc & OB_RECALC_DATA)
-					if(modifiers_isSoftbodyEnabled(base->object)) {
-						base->object->softflag |= OB_SB_REDO;
-				}
-			}
 		}
 		else
 			where_is_pose(ob);
 	}
-	else if(t->spacetype==SPACE_IMAGE) {
-		flushTransUVs(t);
-		if (G.sima->flag & SI_LIVE_UNWRAP)
-			unwrap_lscm_live_re_solve();
+	else if(G.f & G_PARTICLEEDIT) {
+		flushTransParticles(t);
 	}
 	else {
 		for(base= FIRSTBASE; base; base= base->next) {
 			Object *ob= base->object;
 			
-			/* this flag is from depgraph, was stored in nitialize phase, handled in drawview.c */
+			/* this flag is from depgraph, was stored in initialize phase, handled in drawview.c */
 			if(base->flag & BA_HAS_RECALC_OB)
 				ob->recalc |= OB_RECALC_OB;
 			if(base->flag & BA_HAS_RECALC_DATA)
 				ob->recalc |= OB_RECALC_DATA;
-			
+
 			/* thanks to ob->ctime usage, ipos are not called in where_is_object,
 			   unless we edit ipokeys */
 			if(base->flag & BA_DO_IPO) {
@@ -345,12 +546,6 @@ void recalcData(TransInfo *t)
 				}				
 			}
 			
-			/* softbody exception */
-			if(modifiers_isSoftbodyEnabled(ob)) {
-				if(ob->recalc & OB_RECALC_DATA)
-					ob->softflag |= OB_SB_REDO;
-			}
-			
 			/* proxy exception */
 			if(ob->proxy)
 				ob->proxy->recalc |= ob->recalc;
@@ -373,7 +568,6 @@ void recalcData(TransInfo *t)
 	/* update shaded drawmode while transform */
 	if(t->spacetype==SPACE_VIEW3D && G.vd->drawtype == OB_SHADED)
 		reshadeall_displist();
-	
 }
 
 void initTransModeFlags(TransInfo *t, int mode) 
@@ -403,6 +597,7 @@ void initTransModeFlags(TransInfo *t, int mode)
 	case TFM_CREASE:
 	case TFM_BONE_ENVELOPE:
 	case TFM_CURVE_SHRINKFATTEN:
+	case TFM_BONE_ROLL:
 		t->flag |= T_NO_CONSTRAINT;
 		break;
 	}
@@ -445,6 +640,7 @@ void initTrans (TransInfo *t)
 	
 	/* moving: is shown in drawobject() (transform color) */
 	if(G.obedit || (t->flag & T_POSE) ) G.moving= G_TRANSFORM_EDIT;
+	else if(G.f & G_PARTICLEEDIT) G.moving= G_TRANSFORM_PARTICLE;
 	else G.moving= G_TRANSFORM_OBJ;
 
 	t->data = NULL;
@@ -463,6 +659,7 @@ void initTrans (TransInfo *t)
 	t->con.imval[1] = t->imval[1];
 
 	t->transform		= NULL;
+	t->handleEvent		= NULL;
 
 	t->total			=
 		t->num.idx		=
@@ -491,11 +688,14 @@ void initTrans (TransInfo *t)
 	if(t->spacetype==SPACE_VIEW3D) {
 		if(G.vd->flag & V3D_ALIGN) t->flag |= T_V3D_ALIGN;
 		t->around = G.vd->around;
+	} else if(t->spacetype==SPACE_IMAGE) {
+		t->around = G.v2d->around;
 	}
 	else
 		t->around = V3D_CENTER;
 
 	setTransformViewMatrices(t);
+	initNDofInput(&(t->ndof));
 }
 
 /* Here I would suggest only TransInfo related issues, like free data & reset vars. Not redraws */
@@ -532,6 +732,7 @@ void postTrans (TransInfo *t)
 		/* since ipokeys are optional on objects, we mallocced them per trans-data */
 		for(a=0, td= t->data; a<t->total; a++, td++) {
 			if(td->tdi) MEM_freeN(td->tdi);
+			if (td->flag & TD_BEZTRIPLE) MEM_freeN(td->hdata); 
 		}
 		MEM_freeN(t->data);
 	}
@@ -582,7 +783,7 @@ static void restoreElement(TransData *td) {
 	if (td->val) {
 		*td->val = td->ival;
 	}
-	if (td->ext) {
+	if (td->ext && (td->flag&TD_NO_EXT)==0) {
 		if (td->ext->rot) {
 			VECCOPY(td->ext->rot, td->ext->irot);
 		}
@@ -595,6 +796,12 @@ static void restoreElement(TransData *td) {
 			}
 		}
 	}
+	
+	if (td->flag & TD_BEZTRIPLE) {
+		*(td->hdata->h1) = td->hdata->ih1;
+		*(td->hdata->h2) = td->hdata->ih2;
+	}
+	
 	if(td->tdi) {
 		TransDataIpokey *tdi= td->tdi;
 		
@@ -677,6 +884,19 @@ void calculateCenterCursor(TransInfo *t)
 	calculateCenter2D(t);
 }
 
+void calculateCenterCursor2D(TransInfo *t)
+{
+	float aspx=1.0, aspy=1.0;
+	
+	if(t->spacetype==SPACE_IMAGE) /* only space supported right now but may change */
+		transform_aspect_ratio_tface_uv(&aspx, &aspy);
+	if (G.v2d) {
+		t->center[0] = G.v2d->cursor[0] * aspx; 
+		t->center[1] = G.v2d->cursor[1] * aspy; 
+	}
+	calculateCenter2D(t);
+}
+
 void calculateCenterMedian(TransInfo *t)
 {
 	float partial[3] = {0.0f, 0.0f, 0.0f};
@@ -684,7 +904,8 @@ void calculateCenterMedian(TransInfo *t)
 	
 	for(i = 0; i < t->total; i++) {
 		if (t->data[i].flag & TD_SELECTED) {
-			VecAddf(partial, partial, t->data[i].center);
+			if (!(t->data[i].flag & TD_NOCENTER))
+				VecAddf(partial, partial, t->data[i].center);
 		}
 		else {
 			/* 
@@ -709,7 +930,8 @@ void calculateCenterBound(TransInfo *t)
 	for(i = 0; i < t->total; i++) {
 		if (i) {
 			if (t->data[i].flag & TD_SELECTED) {
-				MinMax3(min, max, t->data[i].center);
+				if (!(t->data[i].flag & TD_NOCENTER))
+					MinMax3(min, max, t->data[i].center);
 			}
 			else {
 				/* 
@@ -740,18 +962,22 @@ void calculateCenter(TransInfo *t)
 		calculateCenterMedian(t);
 		break;
 	case V3D_CURSOR:
-		calculateCenterCursor(t);
+		if(t->spacetype==SPACE_IMAGE)
+			calculateCenterCursor2D(t);
+		else
+			calculateCenterCursor(t);
 		break;
 	case V3D_LOCAL:
 		/* Individual element center uses median center for helpline and such */
 		calculateCenterMedian(t);
 		break;
 	case V3D_ACTIVE:
+		{
 		/* set median, and if if if... do object center */
-		
+		EditSelection ese;
 		/* EDIT MODE ACTIVE EDITMODE ELEMENT */
-		if (G.obedit && G.obedit->type == OB_MESH && G.editMesh->selected.last) {
-			EM_editselection_center(t->center, G.editMesh->selected.last);
+		if (G.obedit && G.obedit->type == OB_MESH && EM_get_actSelection(&ese)) {
+			EM_editselection_center(t->center, &ese);
 			calculateCenter2D(t);
 			break;
 		} /* END EDIT MODE ACTIVE ELEMENT */
@@ -764,6 +990,8 @@ void calculateCenter(TransInfo *t)
 				projectIntView(t, t->center, t->center2d);
 			}
 		}
+		
+		}
 	}
 
 	/* setting constraint center */
@@ -775,7 +1003,7 @@ void calculateCenter(TransInfo *t)
 
 	/* voor panning from cameraview */
 	if(t->flag & T_OBJECT) {
-		if( G.vd->camera==OBACT && G.vd->persp>1) {
+		if( G.vd->camera==OBACT && G.vd->persp==V3D_CAMOB) {
 			float axis[3];
 			/* persinv is nasty, use viewinv instead, always right */
 			VECCOPY(axis, t->viewinv[2]);
