@@ -1,15 +1,12 @@
 /**
- * $Id: verse_mesh.c 10594 2007-04-27 14:04:30Z jiri $
+ * $Id: verse_mesh.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,7 +19,7 @@
  *
  * Contributor(s): Jiri Hnidek.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifdef WITH_VERSE
@@ -677,7 +674,6 @@ void post_vertex_create(VerseVert *vvert)
 		if(em && (vvert->vertex)) {
 			struct EditVert *eve = (EditVert*)vvert->vertex;
 			VECCOPY(vvert->co, eve->co);
-/*			printf("\tpost_vertex_create: %d send and NOT_OBSOLETE\n", vvert->id);*/
 			send_verse_vertex(vvert);
 			verse_callback_update(0);
 
@@ -1496,8 +1492,8 @@ void create_meshdata_from_geom_node(Mesh *me, VNode *vnode)
 		MEdge *medge = me->medge = CustomData_add_layer(&me->edata, CD_MEDGE, CD_CALLOC, NULL, me->totedge);
 
 		for(i = BLI_edgehashIterator_new(edges); !BLI_edgehashIterator_isDone(i); BLI_edgehashIterator_step(i), ++medge) {
+			memset(medge, 0, sizeof(struct MEdge));
 			BLI_edgehashIterator_getKey(i, (int*)&medge->v1, (int*)&medge->v2);
-			medge->crease = medge->pad = medge->flag = 0;
 		}
 		BLI_edgehashIterator_free(i);
 	}
