@@ -1,15 +1,12 @@
 /**
- * $Id: view.c 11965 2007-09-08 00:44:41Z bdiego $
+ * $Id: view.c 14885 2008-05-18 16:00:13Z lukep $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,7 +26,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <math.h>
@@ -66,13 +63,16 @@
 #include "BKE_global.h"
 #include "BKE_main.h"
 #include "BKE_object.h"
+#include "BKE_sculpt.h"
 #include "BKE_utildefines.h"
 
+#include "BIF_transform.h"
+#include "BIF_editparticle.h"
 #include "BIF_gl.h"
-#include "BIF_space.h"
-#include "BIF_mywindow.h"
 #include "BIF_previewrender.h"
+#include "BIF_mywindow.h"
 #include "BIF_retopo.h"
+#include "BIF_space.h"
 #include "BIF_screen.h"
 #include "BIF_toolbox.h"
 
@@ -86,8 +86,10 @@
 
 #include "mydevice.h"
 #include "blendef.h"
+#include "transform.h"
 
 #include "PIL_time.h" /* smoothview */
+#include <float.h>
 
 #define TRACKBALLSIZE  (1.1)
 #define BL_NEAR_CLIP 0.001
@@ -536,23 +538,221 @@ void calctrackballvec(rcti *area, short *mval, float *vec)
 
 }
 
+
+// ndof scaling will be moved to user setting.
+// In the mean time this is just a place holder.
+
+// Note: scaling in the plugin and ghostwinlay.c
+// should be removed. With driver default setting,
+// each axis returns approx. +-200 max deflection.
+
+// The values I selected are based on the older
+// polling i/f. With event i/f, the sensistivity
+// can be increased for improved response from
+// small deflections of the device input.
+
+
+// lukep notes : i disagree on the range.
+// the normal 3Dconnection driver give +/-400
+// on defaut range in other applications
+// and up to +/- 1000 if set to maximum
+// because i remove the scaling by delta,
+// which was a bad idea as it depend of the system
+// speed and os, i changed the scaling values, but 
+// those are still not ok
+
+
+float ndof_axis_scale[6] = {
+	+0.01,	// Tx
+	+0.01,	// Tz
+	+0.01,	// Ty
+	+0.0015,	// Rx
+	+0.0015,	// Rz
+	+0.0015	// Ry
+};
+
+// statics for controlling G.vd->dist corrections.
+// viewmoveNDOF zeros and adjusts G.vd->ofs.
+// viewmove restores based on dz_flag state.
+
+int dz_flag = 0;
+float m_dist;
+
+void viewmoveNDOFfly(int mode)
+{
+    int i;
+    float phi;
+    float dval[7];
+	// static fval[6] for low pass filter; device input vector is dval[6]
+	static float fval[6];
+    float tvec[3],rvec[3];
+    float q1[4];
+	float mat[3][3];
+	float upvec[3];
+
+
+    /*----------------------------------------------------
+	 * sometimes this routine is called from headerbuttons
+     * viewmove needs to refresh the screen
+     */
+	areawinset(curarea->win);
+
+
+	// fetch the current state of the ndof device
+	getndof(dval);
+
+	if (G.vd->ndoffilter)
+		filterNDOFvalues(fval);
+
+//	for(i=0;i<7;i++) printf("%f ",dval[i]);
+//		printf("\n");
+
+
+	// Scale input values
+
+//	if(dval[6] == 0) return; // guard against divide by zero
+
+	for(i=0;i<6;i++) {
+
+		// user scaling
+		dval[i] = dval[i] * ndof_axis_scale[i];
+
+		// non-linear scaling
+		if(dval[i]<0.0f)
+			dval[i] = -1.0f * dval[i] * dval[i];
+		else
+			dval[i] = dval[i] * dval[i];
+	}
+
+
+	// low pass filter with zero crossing reset
+
+	for(i=0;i<6;i++) {
+		if((dval[i] * fval[i]) >= 0)
+			dval[i] = (fval[i] * 15 + dval[i]) / 16;
+		else
+			fval[i] = 0;
+	}
+
+
+	// force perspective mode. This is a hack and is
+	// incomplete. It doesn't actually effect the view
+	// until the first draw and doesn't update the menu
+	// to reflect persp mode.
+
+	G.vd->persp = V3D_PERSP;
+
+
+	// Correct the distance jump if G.vd->dist != 0
+
+	// This is due to a side effect of the original
+	// mouse view rotation code. The rotation point is
+	// set a distance in front of the viewport to
+	// make rotating with the mouse look better.
+	// The distance effect is written at a low level
+	// in the view management instead of the mouse
+	// view function. This means that all other view
+	// movement devices must subtract this from their
+	// view transformations.
+
+	if(G.vd->dist != 0.0) {
+		dz_flag = 1;
+		m_dist = G.vd->dist;
+		upvec[0] = upvec[1] = 0;
+		upvec[2] = G.vd->dist;
+		Mat3CpyMat4(mat, G.vd->viewinv);
+		Mat3MulVecfl(mat, upvec);
+		VecSubf(G.vd->ofs, G.vd->ofs, upvec);
+		G.vd->dist = 0.0;
+	}
+
+
+	// Apply rotation
+
+	rvec[0] = -dval[3];
+	rvec[1] = -dval[4];
+	rvec[2] = dval[5];
+
+	// rotate device x and y by view z
+
+	Mat3CpyMat4(mat, G.vd->viewinv);
+	mat[2][2] = 0.0f;
+	Mat3MulVecfl(mat, rvec);
+
+	// rotate the view
+
+	phi = Normalize(rvec);
+	if(phi != 0) {
+		VecRotToQuat(rvec,phi,q1);
+		QuatMul(G.vd->viewquat, G.vd->viewquat, q1);
+	}
+
+
+	// Apply translation
+
+	tvec[0] = dval[0];
+	tvec[1] = dval[1];
+	tvec[2] = -dval[2];
+
+	// the next three lines rotate the x and y translation coordinates
+	// by the current z axis angle
+
+	Mat3CpyMat4(mat, G.vd->viewinv);
+	mat[2][2] = 0.0f;
+	Mat3MulVecfl(mat, tvec);
+
+	// translate the view
+
+	VecSubf(G.vd->ofs, G.vd->ofs, tvec);
+
+
+	/*----------------------------------------------------
+     * refresh the screen
+     */
+    scrarea_do_windraw(curarea);
+    screen_swapbuffers();
+
+	// update render preview window
+
+	BIF_view3d_previewrender_signal(curarea, PR_DBASE|PR_DISPRECT);
+}
+
 void viewmove(int mode)
 {
+	static float lastofs[3] = {0,0,0};
 	Object *ob = OBACT;
 	float firstvec[3], newvec[3], dvec[3];
 	float reverse, oldquat[4], q1[4], si, phi, dist0;
 	float ofs[3], obofs[3]= {0.0f, 0.0f, 0.0f};
 	int firsttime=1;
-	short mvalball[2], mval[2], mvalo[2];
+	short mvalball[2], mval[2], mvalo[2], mval_area[2], mvali[2];
 	short use_sel = 0;
 	short preview3d_event= 1;
 	
-	/* 3D window may not be defined */
+	// locals for dist correction
+	float mat[3][3];
+	float upvec[3];
+
+		/* 3D window may not be defined */
 	if( !G.vd ) {
 		fprintf( stderr, "G.vd == NULL in viewmove()\n" );
 		return;
 	}
+	
+		// dist correction from other movement devices
+		
+	if((dz_flag)||G.vd->dist==0) {
+		dz_flag = 0;
+		G.vd->dist = m_dist;
+		upvec[0] = upvec[1] = 0;
+		upvec[2] = G.vd->dist;
+		Mat3CpyMat4(mat, G.vd->viewinv);
+		Mat3MulVecfl(mat, upvec);
+		VecAddf(G.vd->ofs, G.vd->ofs, upvec);
+	}
 
+
+		
 	/* sometimes this routine is called from headerbuttons */
 
 	areawinset(curarea->win);
@@ -561,9 +761,10 @@ void viewmove(int mode)
 	
 	QUATCOPY(oldquat, G.vd->viewquat);
 	
-	getmouseco_sc(mvalo);		/* work with screen coordinates because of trackball function */
-	mvalball[0]= mvalo[0];			/* needed for turntable to work */
-	mvalball[1]= mvalo[1];
+	getmouseco_areawin(mval_area);	/* for zoom to mouse loc */
+	getmouseco_sc(mvali);		/* work with screen coordinates because of trackball function */
+	mvalball[0]= mvalo[0] = mvali[0];			/* needed for turntable to work */
+	mvalball[1]= mvalo[1] = mvali[1];
 	dist0= G.vd->dist;
 	
 	calctrackballvec(&curarea->winrct, mvalo, firstvec);
@@ -574,19 +775,21 @@ void viewmove(int mode)
 		use_sel= 1;
 		VecCopyf(ofs, G.vd->ofs);
 
-		VecCopyf(obofs,&sculpt_session()->pivot.x);
+		VecCopyf(obofs, sculpt_data()->pivot);
 		Mat4MulVecfl(ob->obmat, obofs);
 		obofs[0]= -obofs[0];
 		obofs[1]= -obofs[1];
 		obofs[2]= -obofs[2];
 	}
-	else if (ob && (U.uiflag & USER_ORBIT_SELECTION)) {
+	else if (U.uiflag & USER_ORBIT_SELECTION) {
 		use_sel = 1;
 		
 		VECCOPY(ofs, G.vd->ofs);
 		
-		/* If there's no selection, obofs is unmodified, so <0,0,0> */
-		calculateTransformCenter(V3D_CENTROID, obofs);
+		/* If there's no selection, lastofs is unmodified and last value since static */
+		calculateTransformCenter(V3D_CENTROID, lastofs);
+		
+		VECCOPY(obofs, lastofs);
 		VecMulf(obofs, -1.0f);
 	}
 	else
@@ -600,7 +803,10 @@ void viewmove(int mode)
 		getmouseco_sc(mval);
 		
 		// if playanim = alt+A, screenhandlers are for animated UI, python, etc
-		if(mval[0]!=mvalo[0] || mval[1]!=mvalo[1] || (G.f & G_PLAYANIM) || do_screenhandlers(G.curscreen)) {
+		if(		(mode==2 && U.viewzoom==USER_ZOOM_CONT) || /* continues zoom always update */
+				mval[0]!=mvalo[0] || mval[1]!=mvalo[1] || /* mouse moved, so update */
+				(G.f & G_PLAYANIM) || do_screenhandlers(G.curscreen)
+			) {
 			
 			if(firsttime) {
 				
@@ -610,15 +816,15 @@ void viewmove(int mode)
 					if(G.vd->view!=0) scrarea_queue_headredraw(curarea);	/*for button */
 					G.vd->view= 0;
 				}
-				if(G.vd->persp==2 && mode!=1 && G.vd->camera) {
-					G.vd->persp= 1;
+				if(G.vd->persp==V3D_CAMOB && mode!=1 && G.vd->camera) {
+					G.vd->persp= V3D_PERSP;
 					scrarea_do_windraw(curarea);
 					scrarea_queue_headredraw(curarea);
 				}
 			}
 
 			if(mode==0) {	/* view rotate */
-				if (U.uiflag & USER_AUTOPERSP) G.vd->persp= 1;
+				if (U.uiflag & USER_AUTOPERSP) G.vd->persp= V3D_PERSP;
 
 				if (U.flag & USER_TRACKBALL) mvalball[0]= mval[0];
 				mvalball[1]= mval[1];
@@ -718,7 +924,7 @@ void viewmove(int mode)
 				}
 			}
 			else if(mode==1) {	/* translate */
-				if(G.vd->persp==2) {
+				if(G.vd->persp==V3D_CAMOB) {
 					float max= (float)MAX2(curarea->winx, curarea->winy);
 
 					G.vd->camdx += (mvalo[0]-mval[0])/(max);
@@ -733,9 +939,15 @@ void viewmove(int mode)
 				}
 			}
 			else if(mode==2) {
+				float zfac=1.0;
+
+				/* use initial value (do not use mvalo (that is used to detect mouse moviments)) */
+				mvalo[0] = mvali[0];
+				mvalo[1] = mvali[1];
+				
 				if(U.viewzoom==USER_ZOOM_CONT) {
 					// oldstyle zoom
-					G.vd->dist*= 1.0+(float)(mvalo[0]-mval[0]+mvalo[1]-mval[1])/1000.0;
+					zfac = 1.0+(float)(mvalo[0]-mval[0]+mvalo[1]-mval[1])/1000.0;
 				}
 				else if(U.viewzoom==USER_ZOOM_SCALE) {
 					int ctr[2], len1, len2;
@@ -747,25 +959,27 @@ void viewmove(int mode)
 					len1 = (int)sqrt((ctr[0] - mval[0])*(ctr[0] - mval[0]) + (ctr[1] - mval[1])*(ctr[1] - mval[1])) + 5;
 					len2 = (int)sqrt((ctr[0] - mvalo[0])*(ctr[0] - mvalo[0]) + (ctr[1] - mvalo[1])*(ctr[1] - mvalo[1])) + 5;
 					
-					G.vd->dist= dist0 * ((float)len2/len1);
+					zfac = dist0 * ((float)len2/len1) / G.vd->dist;
 				}
 				else {	/* USER_ZOOM_DOLLY */
 					float len1 = (curarea->winrct.ymax - mval[1]) + 5;
 					float len2 = (curarea->winrct.ymax - mvalo[1]) + 5;
-					
-					G.vd->dist= dist0 * (2.0*((len2/len1)-1.0) + 1.0);
+					zfac = dist0 * (2.0*((len2/len1)-1.0) + 1.0) / G.vd->dist;
 				}
+
+				if(zfac != 1.0 && zfac*G.vd->dist > 0.001*G.vd->grid && 
+					zfac*G.vd->dist < 10.0*G.vd->far)
+					view_zoom_mouseloc(zfac, mval_area);
 				
 				/* these limits are in toets.c too */
 				if(G.vd->dist<0.001*G.vd->grid) G.vd->dist= 0.001*G.vd->grid;
 				if(G.vd->dist>10.0*G.vd->far) G.vd->dist=10.0*G.vd->far;
 				
-				mval[1]= mvalo[1]; /* preserve first value */
-				mval[0]= mvalo[0];
-				
-				if(G.vd->persp==0 || G.vd->persp==2) preview3d_event= 0;
+				if(G.vd->persp==V3D_ORTHO || G.vd->persp==V3D_CAMOB) preview3d_event= 0;
 			}
 			
+			
+			
 			mvalo[0]= mval[0];
 			mvalo[1]= mval[1];
 
@@ -804,6 +1018,247 @@ void viewmove(int mode)
 		BIF_view3d_previewrender_signal(curarea, PR_PROJECTED);
 
 }
+ 
+void view_zoom_mouseloc(float dfac, short *mouseloc)
+{
+	if(U.uiflag & USER_ZOOM_TO_MOUSEPOS) {
+		short vb[2];
+		float dvec[3];
+		float tvec[3];
+		float tpos[3];
+		float new_dist;
+
+		/* find the current window width and height */
+		vb[0] = G.vd->area->winx;
+		vb[1] = G.vd->area->winy;
+		
+		tpos[0] = -G.vd->ofs[0];
+		tpos[1] = -G.vd->ofs[1];
+		tpos[2] = -G.vd->ofs[2];
+
+		/* Project cursor position into 3D space */
+		initgrabz(tpos[0], tpos[1], tpos[2]);
+		window_to_3d(dvec, mouseloc[0]-vb[0]/2, mouseloc[1]-vb[1]/2);
+
+		/* Calculate view target position for dolly */
+		tvec[0] = -(tpos[0] + dvec[0]);
+		tvec[1] = -(tpos[1] + dvec[1]);
+		tvec[2] = -(tpos[2] + dvec[2]);
+
+		/* Offset to target position and dolly */
+		new_dist = G.vd->dist * dfac;
+		
+		VECCOPY(G.vd->ofs, tvec);
+		G.vd->dist = new_dist;
+		
+		/* Calculate final offset */
+		dvec[0] = tvec[0] + dvec[0] * dfac;
+		dvec[1] = tvec[1] + dvec[1] * dfac;
+		dvec[2] = tvec[2] + dvec[2] * dfac;
+		
+		VECCOPY(G.vd->ofs, dvec);
+	} else {
+		G.vd->dist *= dfac;
+	}
+}
+
+void viewmoveNDOF(int mode)
+{
+    float fval[7];
+    float dvec[3];
+    float sbadjust = 1.0f;
+    float len;
+	short use_sel = 0;
+	Object *ob = OBACT;
+    float m[3][3];
+    float m_inv[3][3];
+    float xvec[3] = {1,0,0};
+    float yvec[3] = {0,-1,0};
+    float zvec[3] = {0,0,1};
+	float phi, si;
+    float q1[4];
+    float obofs[3];
+    float reverse;
+    float diff[4];
+    float d, curareaX, curareaY;
+
+    /* Sensitivity will control how fast the view rotates.  The value was
+     * obtained experimentally by tweaking until the author didn't get dizzy watching.
+     * Perhaps this should be a configurable user parameter. 
+     */
+    float psens = 0.005f * (float) U.ndof_pan;   /* pan sensitivity */
+    float rsens = 0.005f * (float) U.ndof_rotate;  /* rotate sensitivity */
+    float zsens = 0.3f;   /* zoom sensitivity */
+
+    const float minZoom = -30.0f;
+    const float maxZoom = 300.0f;
+
+	//reset view type
+	G.vd->view = 0;
+//printf("passing here \n");
+//
+	if (G.obedit==NULL && ob && !(ob->flag & OB_POSEMODE)) {
+		use_sel = 1;
+	}
+
+    /*----------------------------------------------------
+	 * sometimes this routine is called from headerbuttons
+     * viewmove needs to refresh the screen
+     */
+	areawinset(curarea->win);
+
+    /*----------------------------------------------------
+     * record how much time has passed. clamp at 10 Hz
+     * pretend the previous frame occured at the clamped time 
+     */
+//    now = PIL_check_seconds_timer();
+ //   frametime = (now - prevTime);
+ //   if (frametime > 0.1f){        /* if more than 1/10s */
+ //       frametime = 1.0f/60.0;      /* clamp at 1/60s so no jumps when starting to move */
+//    }
+//    prevTime = now;
+ //   sbadjust *= 60 * frametime;             /* normalize ndof device adjustments to 100Hz for framerate independence */
+
+    /* fetch the current state of the ndof device */
+    getndof(fval);
+ //           printf(" motion command %f %f %f %f %f %f %f \n", fval[0], fval[1], fval[2],
+ //           							 fval[3], fval[4], fval[5], fval[6]);
+			if (G.vd->ndoffilter)
+				filterNDOFvalues(fval);
+	
+	
+	// put scaling back here, was previously in ghostwinlay
+            fval[0] = fval[0]  * (1.0f/800.0f);
+            fval[1] = fval[1]  * (1.0f/800.0f);
+            fval[2] = fval[2] * (1.0f/800.0f);
+            fval[3] = fval[3]  * 0.00005f;
+            fval[4] = fval[4]  * 0.00005f;
+            fval[5] = fval[5] * 0.00005f;
+            fval[6] = fval[6]  / 1000000.0f;
+			
+	// scale more if not in perspective mode
+			if (G.vd->persp == V3D_ORTHO) {
+				fval[0] = fval[0] * 0.05f;
+				fval[1] = fval[1] * 0.05f;
+				fval[2] = fval[2] * 0.05f;
+				fval[3] = fval[3] * 0.9f;
+				fval[4] = fval[4] * 0.9f;
+				fval[5] = fval[5] * 0.9f;
+				zsens *= 8;
+			}
+			
+	
+    /* set object offset */
+	if (ob) {
+		obofs[0] = -ob->obmat[3][0];
+		obofs[1] = -ob->obmat[3][1];
+		obofs[2] = -ob->obmat[3][2];
+	}
+	else {
+		VECCOPY(obofs, G.vd->ofs);
+	}
+
+    /* calc an adjustment based on distance from camera */
+    if (ob) {
+        VecSubf(diff, obofs, G.vd->ofs);
+        d = VecLength(diff);
+    }
+    else {
+        d = 1.0f;
+    }
+    reverse = (G.vd->persmat[2][1] < 0.0f) ? -1.0f : 1.0f;
+
+    /*----------------------------------------------------
+     * ndof device pan 
+     */
+    psens *= 1.0f + d;
+    curareaX = sbadjust * psens * fval[0];
+    curareaY = sbadjust * psens * fval[1];
+    dvec[0] = curareaX * G.vd->persinv[0][0] + curareaY * G.vd->persinv[1][0];
+    dvec[1] = curareaX * G.vd->persinv[0][1] + curareaY * G.vd->persinv[1][1];
+    dvec[2] = curareaX * G.vd->persinv[0][2] + curareaY * G.vd->persinv[1][2];
+    VecAddf(G.vd->ofs, G.vd->ofs, dvec);
+
+    /*----------------------------------------------------
+     * ndof device dolly 
+     */
+    len = zsens * sbadjust * fval[2];
+
+    if (G.vd->persp==V3D_CAMOB) {
+        if(G.vd->persp==V3D_CAMOB) { /* This is stupid, please fix - TODO */
+            G.vd->camzoom+= 10.0f * -len;
+        }
+        if (G.vd->camzoom < minZoom) G.vd->camzoom = minZoom;
+        else if (G.vd->camzoom > maxZoom) G.vd->camzoom = maxZoom;
+    }
+    else if ((G.vd->dist> 0.001*G.vd->grid) && (G.vd->dist<10.0*G.vd->far)) {
+        G.vd->dist*=(1.0 + len);
+    }
+
+
+    /*----------------------------------------------------
+     * ndof device turntable
+     * derived from the turntable code in viewmove
+     */
+
+    /* Get the 3x3 matrix and its inverse from the quaternion */
+    QuatToMat3(G.vd->viewquat, m);
+    Mat3Inv(m_inv,m);
+
+    /* Determine the direction of the x vector (for rotating up and down) */
+    /* This can likely be compuated directly from the quaternion. */
+    Mat3MulVecfl(m_inv,xvec);
+    Mat3MulVecfl(m_inv,yvec);
+    Mat3MulVecfl(m_inv,zvec);
+
+    /* Perform the up/down rotation */
+    phi = sbadjust * rsens * /*0.5f * */ fval[3]; /* spin vertically half as fast as horizontally */
+    si = sin(phi);
+    q1[0] = cos(phi);
+    q1[1] = si * xvec[0];
+    q1[2] = si * xvec[1];
+    q1[3] = si * xvec[2];
+    QuatMul(G.vd->viewquat, G.vd->viewquat, q1);
+
+    if (use_sel) {
+        QuatConj(q1); /* conj == inv for unit quat */
+        VecSubf(G.vd->ofs, G.vd->ofs, obofs);
+        QuatMulVecf(q1, G.vd->ofs);
+        VecAddf(G.vd->ofs, G.vd->ofs, obofs);
+    }
+
+    /* Perform the orbital rotation */
+    /* Perform the orbital rotation 
+       If the seen Up axis is parallel to the zoom axis, rotation should be
+       achieved with a pure Roll motion (no Spin) on the device. When you start 
+       to tilt, moving from Top to Side view, Spinning will increasingly become 
+       more relevant while the Roll component will decrease. When a full 
+       Side view is reached, rotations around the world's Up axis are achieved
+       with a pure Spin-only motion.  In other words the control of the spinning
+       around the world's Up axis should move from the device's Spin axis to the
+       device's Roll axis depending on the orientation of the world's Up axis 
+       relative to the screen. */
+    //phi = sbadjust * rsens * reverse * fval[4];  /* spin the knob, y axis */
+    phi = sbadjust * rsens * (yvec[2] * fval[4] + zvec[2] * fval[5]);
+    q1[0] = cos(phi);
+    q1[1] = q1[2] = 0.0;
+    q1[3] = sin(phi);
+    QuatMul(G.vd->viewquat, G.vd->viewquat, q1);
+
+    if (use_sel) {
+        QuatConj(q1);
+        VecSubf(G.vd->ofs, G.vd->ofs, obofs);
+        QuatMulVecf(q1, G.vd->ofs);
+        VecAddf(G.vd->ofs, G.vd->ofs, obofs);
+    }
+
+    /*----------------------------------------------------
+     * refresh the screen
+     */
+    scrarea_do_windraw(curarea);
+    screen_swapbuffers();
+}
+
 
 /* Gets the lens and clipping values from a camera of lamp type object */
 void object_view_settings(Object *ob, float *lens, float *clipsta, float *clipend)
@@ -844,10 +1299,10 @@ int get_view3d_viewplane(int winxi, int winyi, rctf *viewplane, float *clipsta,
 
 /*	
  * Cant use this since we need the fac and x1 values set
- * if(G.vd->persp==2)
+ * if(G.vd->persp==V3D_CAMOB)
 		object_view_settings(G.vd->camera, &lens, &(*clipsta), &(*clipend));*/
 	
-	if(G.vd->persp==2) {
+	if(G.vd->persp==V3D_CAMOB) {
 		if(G.vd->camera) {
 			if(G.vd->camera->type==OB_LAMP ) {
 				Lamp *la;
@@ -870,7 +1325,7 @@ int get_view3d_viewplane(int winxi, int winyi, rctf *viewplane, float *clipsta,
 		}
 	}
 	
-	if(G.vd->persp==0) {
+	if(G.vd->persp==V3D_ORTHO) {
 		if(winx>winy) x1= -G.vd->dist;
 		else x1= -winx*G.vd->dist/winy;
 		x2= -x1;
@@ -885,7 +1340,7 @@ int get_view3d_viewplane(int winxi, int winyi, rctf *viewplane, float *clipsta,
 	}
 	else {
 		/* fac for zoom, also used for camdx */
-		if(G.vd->persp==2) {
+		if(G.vd->persp==V3D_CAMOB) {
 			fac= (1.41421+( (float)G.vd->camzoom )/50.0);
 			fac*= fac;
 		}
@@ -1001,7 +1456,7 @@ void obmat_to_viewmat(Object *ob, short smooth)
 	Mat3CpyMat4(tmat, G.vd->viewmat);
 	if (smooth) {
 		float new_quat[4];
-		if (G.vd->persp==2 && G.vd->camera) {
+		if (G.vd->persp==V3D_CAMOB && G.vd->camera) {
 			/* were from a camera view */
 			
 			float orig_ofs[3];
@@ -1012,13 +1467,13 @@ void obmat_to_viewmat(Object *ob, short smooth)
 			/* Switch from camera view */
 			Mat3ToQuat(tmat, new_quat);
 			
-			G.vd->persp=1;
+			G.vd->persp=V3D_PERSP;
 			G.vd->dist= 0.0;
 			
 			view_settings_from_ob(G.vd->camera, G.vd->ofs, NULL, NULL, &G.vd->lens);
 			smooth_view(G.vd, orig_ofs, new_quat, &orig_dist, &orig_lens);
 			
-			G.vd->persp=2; /* just to be polite, not needed */
+			G.vd->persp=V3D_CAMOB; /* just to be polite, not needed */
 			
 		} else {
 			Mat3ToQuat(tmat, new_quat);
@@ -1032,18 +1487,10 @@ void obmat_to_viewmat(Object *ob, short smooth)
 /* dont set windows active in in here, is used by renderwin too */
 void setviewmatrixview3d()
 {
-	Camera *cam;
-
-	if(G.vd->persp>=2) {	    /* obs/camera */
+	if(G.vd->persp==V3D_CAMOB) {	    /* obs/camera */
 		if(G.vd->camera) {
-			
 			where_is_object(G.vd->camera);	
 			obmat_to_viewmat(G.vd->camera, 0);
-			
-			if(G.vd->camera->type==OB_CAMERA) {
-				cam= G.vd->camera->data;
-				//if(cam->type==CAM_ORTHO) G.vd->viewmat[3][2]*= 100.0;
-			}
 		}
 		else {
 			QuatToMat4(G.vd->viewquat, G.vd->viewmat);
@@ -1053,7 +1500,7 @@ void setviewmatrixview3d()
 	else {
 		
 		QuatToMat4(G.vd->viewquat, G.vd->viewmat);
-		if(G.vd->persp==1) G.vd->viewmat[3][2]-= G.vd->dist;
+		if(G.vd->persp==V3D_PERSP) G.vd->viewmat[3][2]-= G.vd->dist;
 		if(G.vd->ob_centre) {
 			Object *ob= G.vd->ob_centre;
 			float vec[3];
@@ -1084,11 +1531,12 @@ void setcameratoview3d(void)
 	VECCOPY(ob->loc, dvec);
 	VecSubf(ob->loc, ob->loc, G.vd->ofs);
 	G.vd->viewquat[0]= -G.vd->viewquat[0];
-	if (ob->transflag & OB_QUAT) {
+	/*  */
+	/*if (ob->transflag & OB_QUAT) {
 		QUATCOPY(ob->quat, G.vd->viewquat);
-	} else {
+	} else {*/
 		QuatToEul(G.vd->viewquat, ob->rot);
-	}
+	/*}*/
 	G.vd->viewquat[0]= -G.vd->viewquat[0];
 }
 
@@ -1258,8 +1706,7 @@ void initlocalview()
 
 	if(G.vd->localvd) return;
 
-	min[0]= min[1]= min[2]= 1.0e10;
-	max[0]= max[1]= max[2]= -1.0e10;
+	INIT_MINMAX(min, max);
 
 	locallay= free_localbit();
 
@@ -1313,7 +1760,7 @@ void initlocalview()
 			G.vd->dist*= size;
 		}
 		
-		if (G.vd->persp>1) G.vd->persp= 1;
+		if (G.vd->persp==V3D_CAMOB) G.vd->persp= V3D_PERSP;
 		if (G.vd->near> 0.1) G.vd->near= 0.1;
 		
 		G.vd->cursor[0]= -G.vd->ofs[0];
@@ -1354,9 +1801,7 @@ void centerview()	/* like a localview without local! */
 	float new_ofs[3];
 	float new_dist;
 	
-	
-	min[0]= min[1]= min[2]= 1.0e10;
-	max[0]= max[1]= max[2]= -1.0e10;
+	INIT_MINMAX(min, max);
 
 	if (G.f & G_WEIGHTPAINT) {
 		/* hardcoded exception, we look for the one selected armature */
@@ -1398,9 +1843,12 @@ void centerview()	/* like a localview without local! */
 			}
 		}
 	}
-	else if (G.f & G_FACESELECT) {
+	else if (FACESEL_PAINT_TEST) {
 		ok= minmax_tface(min, max);
 	}
+	else if (G.f & G_PARTICLEEDIT) {
+		ok= PE_minmax(min, max);
+	}
 	else {
 		Base *base= FIRSTBASE;
 		while(base) {
@@ -1441,16 +1889,16 @@ void centerview()	/* like a localview without local! */
 	G.vd->cursor[1]= -new_ofs[1];
 	G.vd->cursor[2]= -new_ofs[2];
 	
-	if (G.vd->persp==2 && G.vd->camera) {
+	if (G.vd->persp==V3D_CAMOB && G.vd->camera) {
 		float orig_lens= G.vd->lens;
 		
-		G.vd->persp=1;
+		G.vd->persp=V3D_PERSP;
 		G.vd->dist= 0.0;
 		view_settings_from_ob(G.vd->camera, G.vd->ofs, NULL, NULL, &G.vd->lens);
 		smooth_view(G.vd, new_ofs, NULL, &new_dist, &orig_lens);
 	} else {
-		if(G.vd->persp>=2)
-			G.vd->persp= 1;
+		if(G.vd->persp==V3D_CAMOB)
+			G.vd->persp= V3D_PERSP;
 		
 		smooth_view(G.vd, new_ofs, NULL, &new_dist, NULL);
 	}
@@ -1532,8 +1980,7 @@ void view3d_home(int center)
 		max[0]= max[1]= max[2]= 0.0;
 	}
 	else {
-		min[0]= min[1]= min[2]= 1.0e10;
-		max[0]= max[1]= max[2]= -1.0e10;
+		INIT_MINMAX(min, max);
 	}
 	
 	for(base= FIRSTBASE; base; base= base->next) {
@@ -1566,17 +2013,17 @@ void view3d_home(int center)
 			new_dist*= size;
 		}
 		
-		if (G.vd->persp==2 && G.vd->camera) {
+		if (G.vd->persp==V3D_CAMOB && G.vd->camera) {
 			/* switch out of camera view */
 			float orig_lens= G.vd->lens;
 			
-			G.vd->persp=1;
+			G.vd->persp= V3D_PERSP;
 			G.vd->dist= 0.0;
 			view_settings_from_ob(G.vd->camera, G.vd->ofs, NULL, NULL, &G.vd->lens);
 			smooth_view(G.vd, new_ofs, NULL, &new_dist, &orig_lens);
 			
 		} else {
-			if(G.vd->persp>=2) G.vd->persp= 1;
+			if(G.vd->persp==V3D_CAMOB) G.vd->persp= V3D_PERSP;
 			smooth_view(G.vd, new_ofs, NULL, &new_dist, NULL);
 		}
 		scrarea_queue_winredraw(curarea);
@@ -1603,19 +2050,19 @@ void view3d_align_axis_to_vector(View3D *v3d, int axisidx, float vec[3])
 
 	v3d->view= 0;
 	
-	if (v3d->persp==2 && v3d->camera) {
+	if (v3d->persp==V3D_CAMOB && v3d->camera) {
 		/* switch out of camera view */
 		float orig_ofs[3];
 		float orig_dist= v3d->dist;
 		float orig_lens= v3d->lens;
 
 		VECCOPY(orig_ofs, v3d->ofs);
-		G.vd->persp=1;
+		G.vd->persp= V3D_PERSP;
 		G.vd->dist= 0.0;
 		view_settings_from_ob(v3d->camera, v3d->ofs, NULL, NULL, &v3d->lens);
 		smooth_view(G.vd, orig_ofs, new_quat, &orig_dist, &orig_lens);
 	} else {
-		if (v3d->persp>=2) v3d->persp= 1; /* switch out of camera mode */
+		if (v3d->persp==V3D_CAMOB) v3d->persp= V3D_PERSP; /* switch out of camera mode */
 		smooth_view(v3d, NULL, new_quat, NULL, NULL);
 	}
 }
@@ -1664,7 +2111,7 @@ void smooth_view(View3D *v3d, float *ofs, float *quat, float *dist, float *lens)
 				changed = 1;
 		}
 		
-		/* The new view is different from teh old one
+		/* The new view is different from the old one
 		 * so animate the view */
 		if (changed) {
 			
@@ -1790,7 +2237,7 @@ void view_settings_from_ob(Object *ob, float *ofs, float *quat, float *dist, flo
  * */
 void smooth_view_to_camera(View3D *v3d)
 {
-	if (!U.smooth_viewtx || !v3d->camera || G.vd->persp != 2) {
+	if (!U.smooth_viewtx || !v3d->camera || G.vd->persp != V3D_CAMOB) {
 		return;
 	} else {
 		Object *ob = v3d->camera;
@@ -1807,13 +2254,13 @@ void smooth_view_to_camera(View3D *v3d)
 		
 		view_settings_from_ob(ob, new_ofs, new_quat, NULL, &new_lens);
 		
-		G.vd->persp=1;
+		G.vd->persp= V3D_PERSP;
 		smooth_view(v3d, new_ofs, new_quat, &new_dist, &new_lens);
 		VECCOPY(v3d->ofs, orig_ofs);
 		v3d->lens= orig_lens;
 		v3d->dist = orig_dist; /* restore the dist */
 		
 		v3d->camera = ob;
-		v3d->persp=2;
+		v3d->persp= V3D_CAMOB;
 	}
 }
