@@ -1,14 +1,11 @@
 /**
- * $Id: KX_BlenderRenderTools.cpp 8957 2006-11-20 04:28:02Z blendix $
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * $Id: KX_BlenderRenderTools.cpp 14629 2008-04-30 19:58:44Z ben2610 $
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +23,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include "KX_BlenderRenderTools.h"
@@ -94,6 +91,10 @@ int	KX_BlenderRenderTools::ProcessLighting(int layer)
 	{
 		if (m_clientobject)
 		{
+			if (layer == RAS_LIGHT_OBJECT_LAYER)
+			{
+				layer = static_cast<KX_GameObject*>(m_clientobject)->GetLayer();
+			}
 			if (applyLights(layer))
 			{
 				EnableOpenGLLights();
@@ -452,4 +453,37 @@ RAS_IPolyMaterial* KX_BlenderRenderTools::CreateBlenderPolyMaterial(
 	return NULL;
 }
 
+void KX_BlenderRenderTools::MotionBlur(RAS_IRasterizer* rasterizer)
+{
+	int state = rasterizer->GetMotionBlurState();
+	float motionblurvalue;
+	if(state)
+	{
+		motionblurvalue = rasterizer->GetMotionBlurValue();
+		if(state==1)
+		{
+			//bugfix:load color buffer into accum buffer for the first time(state=1)
+			glAccum(GL_LOAD, 1.0);
+			rasterizer->SetMotionBlurState(2);
+		}
+		else if(motionblurvalue>=0.0 && motionblurvalue<=1.0)
+		{
+			glAccum(GL_MULT, motionblurvalue);
+			glAccum(GL_ACCUM, 1-motionblurvalue);
+			glAccum(GL_RETURN, 1.0);
+			glFlush();
+		}
+	}
+}
+
+void KX_BlenderRenderTools::Update2DFilter(RAS_2DFilterManager::RAS_2DFILTER_MODE filtermode, int pass, STR_String& text)
+{
+	m_filtermanager.EnableFilter(filtermode, pass, text);
+}
+
+void KX_BlenderRenderTools::Render2DFilters(RAS_ICanvas* canvas)
+{
+	m_filtermanager.RenderFilters(canvas);
+}
+
 unsigned int KX_BlenderRenderTools::m_numgllights;
