@@ -1,15 +1,12 @@
 /**
-* $Id: BL_ActionActuator.cpp 7334 2006-04-28 17:35:03Z erwin $
+* $Id: BL_ActionActuator.cpp 14444 2008-04-16 22:40:48Z hos $
 *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
 */
 
 #if defined (__sgi)
@@ -43,6 +40,7 @@
 #include "KX_GameObject.h"
 #include "STR_HashedString.h"
 #include "DNA_action_types.h"
+#include "DNA_nla_types.h"
 #include "DNA_actuator_types.h"
 #include "BKE_action.h"
 #include "DNA_armature_types.h"
@@ -384,7 +382,7 @@ bool BL_ActionActuator::Update(double curtime, bool frame)
 				
 				/* Find percentages */
 				newweight = (m_blendframe/(float)m_blendin);
-				blend_poses(m_pose, m_blendpose, 1.0 - newweight, POSE_BLEND);
+				blend_poses(m_pose, m_blendpose, 1.0 - newweight, ACTSTRIPMODE_BLEND);
 
 				/* Increment current blending percentage */
 				m_blendframe = (curtime - m_blendstart)*KX_FIXED_FRAME_PER_SEC;
