@@ -1,15 +1,12 @@
 /**
- * $Id: BL_ArmatureObject.cpp,v 1.11 2006/04/28 17:35:03 erwin Exp $
+ * $Id: BL_ArmatureObject.cpp 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include "BL_ArmatureObject.h"
@@ -190,8 +187,9 @@ double BL_ArmatureObject::GetLastFrame()
 
 bool BL_ArmatureObject::GetBoneMatrix(Bone* bone, MT_Matrix4x4& matrix) const
 {
-	// ton hack
-	bPoseChannel *pchan= get_pose_channel(m_pose, bone->name);
+	Object* par_arma = m_objArma;
+	where_is_pose(par_arma);
+	bPoseChannel *pchan= get_pose_channel(par_arma->pose, bone->name);
 
 	if(pchan) {
 		matrix.setValue(&pchan->pose_mat[0][0]);
