@@ -1,15 +1,12 @@
 /**
- * $Id: KX_BlenderSceneConverter.h 9335 2006-12-16 05:50:38Z erwin $
+ * $Id: KX_BlenderSceneConverter.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 #ifndef __KX_BLENDERSCENECONVERTER_H
 #define __KX_BLENDERSCENECONVERTER_H
@@ -51,10 +48,13 @@ struct SpaceIpo;
 
 class KX_BlenderSceneConverter : public KX_ISceneConverter
 {
-	vector<KX_WorldInfo*>	m_worldinfos;
-	vector<RAS_IPolyMaterial*> m_polymaterials;
-	vector<RAS_MeshObject*> m_meshobjects;
-	vector<BL_Material *>	m_materials;
+	// Use vector of pairs to allow removal of entities between scene switch
+	vector<pair<KX_Scene*,KX_WorldInfo*> >	m_worldinfos;
+	vector<pair<KX_Scene*,RAS_IPolyMaterial*> > m_polymaterials;
+	vector<pair<KX_Scene*,RAS_MeshObject*> > m_meshobjects;
+	vector<pair<KX_Scene*,BL_Material *> >	m_materials;
+	// Should also have a list of collision shapes. 
+	// For the time being this is held in KX_Scene::m_shapes
 
 	GEN_Map<CHashedPtr,struct Object*> m_map_gameobject_to_blender;
 	GEN_Map<CHashedPtr,KX_GameObject*> m_map_blender_to_gameobject;
@@ -72,6 +72,7 @@ class KX_BlenderSceneConverter : public KX_ISceneConverter
 
 	STR_String				m_newfilename;
 	class KX_KetsjiEngine*	m_ketsjiEngine;
+	class KX_Scene*			m_currentScene;	// Scene being converted
 	bool					m_alwaysUseExpandFraming;
 	bool					m_usemat;
 
@@ -99,6 +100,7 @@ public:
 						class RAS_IRenderTools* rendertools,
 						class RAS_ICanvas* canvas
 					);
+	virtual void RemoveScene(class KX_Scene *scene);
 
 	void SetNewFileName(const STR_String& filename);
 	bool TryAndLoadNewFile();
@@ -106,6 +108,7 @@ public:
 	void SetAlwaysUseExpandFraming(bool to_what);
 	
 	void RegisterGameObject(KX_GameObject *gameobject, struct Object *for_blenderobject);
+	void UnregisterGameObject(KX_GameObject *gameobject);
 	KX_GameObject *FindGameObject(struct Object *for_blenderobject);
 	struct Object *FindBlenderObject(KX_GameObject *for_gameobject);
 
