@@ -1,15 +1,12 @@
 /**
- * $Id: KX_ConvertSensors.cpp,v 1.3 2002/11/25 15:29:40 mein Exp $
+ * $Id: KX_ConvertSensors.cpp 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  * Conversion of Blender data blocks to KX sensor system
  */
 
@@ -65,9 +62,10 @@ probably misplaced */
 #include "KX_NearSensor.h"
 #include "KX_RadarSensor.h"
 #include "KX_MouseFocusSensor.h"
-
+#include "SCA_JoystickSensor.h"
 #include "KX_NetworkMessageSensor.h"
 
+
 #include "SCA_PropertySensor.h"
 #include "SCA_RandomSensor.h"
 #include "KX_RaySensor.h"
@@ -79,6 +77,7 @@ probably misplaced */
 #include "KX_BlenderKeyboardDevice.h"
 #include "KX_BlenderGL.h"
 #include "RAS_ICanvas.h"
+#include "PHY_IPhysicsEnvironment.h"
 
 #include "KX_KetsjiEngine.h"
 #include "KX_BlenderSceneConverter.h"
@@ -104,11 +103,13 @@ void BL_ConvertSensors(struct Object* blenderobject,
 	
 	/* The reverse table. In order to not confuse ourselves, we      */
 	/* immediately convert all events that come in to KX codes.      */
-	gReverseKeyTranslateTable[LEFTMOUSE			] =	SCA_IInputDevice::KX_LEFTMOUSE;	  		
-	gReverseKeyTranslateTable[MIDDLEMOUSE		] =	SCA_IInputDevice::KX_MIDDLEMOUSE;	  		
-	gReverseKeyTranslateTable[RIGHTMOUSE		] =	SCA_IInputDevice::KX_RIGHTMOUSE;	  		
-	gReverseKeyTranslateTable[MOUSEX			] = SCA_IInputDevice::KX_MOUSEX;	  		
-	gReverseKeyTranslateTable[MOUSEY			] =	SCA_IInputDevice::KX_MOUSEY;	  		
+	gReverseKeyTranslateTable[LEFTMOUSE			] =	SCA_IInputDevice::KX_LEFTMOUSE;
+	gReverseKeyTranslateTable[MIDDLEMOUSE		] =	SCA_IInputDevice::KX_MIDDLEMOUSE;
+	gReverseKeyTranslateTable[RIGHTMOUSE		] =	SCA_IInputDevice::KX_RIGHTMOUSE;
+	gReverseKeyTranslateTable[WHEELUPMOUSE		] =	SCA_IInputDevice::KX_WHEELUPMOUSE;
+	gReverseKeyTranslateTable[WHEELDOWNMOUSE	] =	SCA_IInputDevice::KX_WHEELDOWNMOUSE;
+	gReverseKeyTranslateTable[MOUSEX			] = SCA_IInputDevice::KX_MOUSEX;
+	gReverseKeyTranslateTable[MOUSEY			] =	SCA_IInputDevice::KX_MOUSEY;
 	
 	// TIMERS                                                                                                  
 	
@@ -297,15 +298,13 @@ void BL_ConvertSensors(struct Object* blenderobject,
 					(blendertouchsensor->name ? blendertouchsensor->name: ""));
 					
 					
-					//if (gameobj->GetSumoObject())
-					//{	
-					//	gamesensor = 0;
-							//new KX_TouchSensor(eventmgr,
-							//gameobj,
-							//gameobj->GetSumoObject(),
-							//bFindMaterial,
-							//touchPropOrMatName);
-					//}
+					if (gameobj->GetPhysicsController())
+					{	
+						gamesensor = new KX_TouchSensor(eventmgr,
+							gameobj,
+							bFindMaterial,
+							touchPropOrMatName);
+					}
 					
 				}
 				
@@ -324,15 +323,13 @@ void BL_ConvertSensors(struct Object* blenderobject,
 						touchpropertyname = (char*) (blendertouchsensor->ma->id.name+2);
 					}
 					bool bFindMaterial = true;
-					//if (gameobj->GetSumoObject())
-					//{
-					//	gamesensor = 0;
-						//new KX_TouchSensor(eventmgr,
-						//	gameobj,
-						//	gameobj->GetSumoObject(),
-						//	bFindMaterial,
-						//	touchpropertyname);
-					//}
+					if (gameobj->GetPhysicsController())
+					{	
+						gamesensor = new KX_TouchSensor(eventmgr,
+							gameobj,
+							bFindMaterial,
+							touchpropertyname);
+					}
 				}
 				break;
 			}
@@ -376,11 +373,29 @@ void BL_ConvertSensors(struct Object* blenderobject,
 					
 					// this sumoObject is not deleted by a gameobj, so delete it ourself
 					// later (memleaks)!
-					//SM_Object* sumoObj 		=	new SM_Object(shape,NULL,NULL,NULL);
-					//sumoObj->setMargin(blendernearsensor->dist);
-					//sumoObj->setPosition(gameobj->NodeGetWorldPosition());
+					float radius = blendernearsensor->dist;
+					PHY__Vector3 pos;
+					const MT_Vector3& wpos = gameobj->NodeGetWorldPosition();
+					pos[0] = wpos[0];
+					pos[1] = wpos[1];
+					pos[2] = wpos[2];
+					pos[3] = 0.f;
 					bool bFindMaterial = false;
-					gamesensor = 0;//new KX_NearSensor(eventmgr,gameobj,blendernearsensor->dist,blendernearsensor->resetdist,bFindMaterial,nearpropertyname,kxscene);
+					PHY_IPhysicsController* physCtrl = kxscene->GetPhysicsEnvironment()->CreateSphereController(radius,pos);
+
+					//will be done in KX_TouchEventManager::RegisterSensor()  
+					//if (isInActiveLayer)
+					//	kxscene->GetPhysicsEnvironment()->addSensor(physCtrl);
+
+						
+
+					gamesensor = new KX_NearSensor(eventmgr,gameobj,
+						blendernearsensor->dist,
+						blendernearsensor->resetdist,
+						bFindMaterial,
+						nearpropertyname,kxscene,
+						physCtrl
+						);
 					
 				}
 				break;
@@ -411,7 +426,7 @@ void BL_ConvertSensors(struct Object* blenderobject,
 		case SENS_MOUSE:
 			{
 				int keytype = SCA_MouseSensor::KX_MOUSESENSORMODE_NODEF;			
-				bool trackfocus = false;
+				int trackfocus = 0;
 				bMouseSensor *bmouse = (bMouseSensor *)sens->data;
 				
 				/* There are two main types of mouse sensors. If there is
@@ -434,12 +449,22 @@ void BL_ConvertSensors(struct Object* blenderobject,
 					case BL_SENS_MOUSE_RIGHT_BUTTON:
 						keytype = SCA_MouseSensor::KX_MOUSESENSORMODE_RIGHTBUTTON;
 						break;
+					case BL_SENS_MOUSE_WHEEL_UP:
+						keytype = SCA_MouseSensor::KX_MOUSESENSORMODE_WHEELUP;
+						break;
+					case BL_SENS_MOUSE_WHEEL_DOWN:
+						keytype = SCA_MouseSensor::KX_MOUSESENSORMODE_WHEELDOWN;
+						break;
 					case BL_SENS_MOUSE_MOVEMENT:
 						keytype = SCA_MouseSensor::KX_MOUSESENSORMODE_MOVEMENT;
 						break;
 					case BL_SENS_MOUSE_MOUSEOVER:
-						trackfocus = true;
+						trackfocus = 1;
 						break;
+					case BL_SENS_MOUSE_MOUSEOVER_ANY:
+						trackfocus = 2;
+						break;
+
 					default:
 						; /* error */
 					}
@@ -514,7 +539,7 @@ void BL_ConvertSensors(struct Object* blenderobject,
 			
 		case SENS_RADAR:
 			{
-				/*
+				
 				SCA_EventManager* eventmgr = logicmgr->FindEventManager(SCA_EventManager::TOUCH_EVENTMGR);
 				if (eventmgr)
 				{
@@ -540,33 +565,30 @@ void BL_ConvertSensors(struct Object* blenderobject,
 					//MT_Scalar coneradius = coneheight * (factor / 2);
 					MT_Scalar coneradius = coneheight * factor;
 					
-					DT_ShapeHandle shape = DT_Cone(coneradius,coneheight);
 					
 					// this sumoObject is not deleted by a gameobj, so delete it ourself
 					// later (memleaks)!
-					SM_Object* sumoObj = new SM_Object(shape,NULL,NULL,NULL);
-					sumoObj->setMargin(0.0);
-					
-					sumoObj->setPosition(gameobj->NodeGetWorldPosition());
 					MT_Scalar smallmargin = 0.0;
-					MT_Scalar largemargin = 0.1;
+					MT_Scalar largemargin = 0.0;
 					
 					bool bFindMaterial = false;
+					PHY_IPhysicsController* ctrl = kxscene->GetPhysicsEnvironment()->CreateConeController(coneradius,coneheight);
+
 					gamesensor = new KX_RadarSensor(
 						eventmgr,
 						gameobj,
+						ctrl,
 						coneradius,
 						coneheight,
 						radaraxis,
 						smallmargin,
 						largemargin,
-						sumoObj,
 						bFindMaterial,
 						radarpropertyname,
 						kxscene);
 						
 				}
-			*/
+			
 				break;
 			}
 		case SENS_RAY:
@@ -586,15 +608,13 @@ void BL_ConvertSensors(struct Object* blenderobject,
 					int axis = blenderraysensor->axisflag;
 
 					
-					gamesensor = 0;
-					/*new KX_RaySensor(eventmgr,
+					gamesensor = new KX_RaySensor(eventmgr,
 												  gameobj,
 												  checkname,
 												  bFindMaterial,
 												  distance,
 												  axis,
-												  kxscene->GetSumoScene());
-				*/
+												  kxscene);
 
 				}
 				break;
@@ -613,6 +633,62 @@ void BL_ConvertSensors(struct Object* blenderobject,
 						gamesensor = new SCA_RandomSensor(eventmgr, gameobj, randomSeed);
 					}
 				}
+				break;
+			}
+		case SENS_JOYSTICK:
+			{
+				int joysticktype = SCA_JoystickSensor::KX_JOYSENSORMODE_NODEF;
+				
+				bJoystickSensor* bjoy = (bJoystickSensor*) sens->data;
+				
+				SCA_JoystickManager *eventmgr 
+					= (SCA_JoystickManager*) logicmgr->FindEventManager(SCA_EventManager::JOY_EVENTMGR);
+				if (eventmgr) 
+				{
+					int axis	=0;
+					int axisf	=0;
+					int button	=0;
+					int buttonf =0; 
+					int hat		=0; 
+					int hatf	=0;
+					int prec	=0;
+					
+					switch(bjoy->type)
+					{
+					case SENS_JOY_AXIS:
+						axis	= bjoy->axis;
+						axisf	= bjoy->axisf;
+						prec	= bjoy->precision;
+						joysticktype  = SCA_JoystickSensor::KX_JOYSENSORMODE_AXIS;
+						break;
+					case SENS_JOY_BUTTON:
+						button	= bjoy->button;
+						buttonf	= bjoy->buttonf;
+						joysticktype  = SCA_JoystickSensor::KX_JOYSENSORMODE_BUTTON;
+						break;
+					case SENS_JOY_HAT:
+						hat		= bjoy->hat;
+						hatf	= bjoy->hatf;
+						joysticktype  = SCA_JoystickSensor::KX_JOYSENSORMODE_HAT;
+						break;
+					default:
+						printf("Error: bad case statement\n");
+						break;
+					}
+					gamesensor = new SCA_JoystickSensor(
+						eventmgr,
+						gameobj,
+						joysticktype,
+						axis,axisf,
+						prec,
+						button,buttonf,
+						hat,hatf);
+				} 
+				else
+				{
+					printf("Error there was a problem finding the event manager\n");
+				}
+
 				break;
 			}
 		default:
@@ -654,6 +730,8 @@ void BL_ConvertSensors(struct Object* blenderobject,
 					logicmgr->RegisterToSensor(gamecont,gamesensor);
 				}
 			}
+			// done with gamesensor
+			gamesensor->Release();
 			
 		}
 		sens=sens->next;
