@@ -1,14 +1,11 @@
 /**
- * $Id: KX_IpoConvert.cpp 6319 2006-01-06 03:46:54Z erwin $
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * $Id: KX_IpoConvert.cpp 14444 2008-04-16 22:40:48Z hos $
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +23,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifdef HAVE_CONFIG_H
@@ -100,9 +97,9 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 
 		ipocontr->GetIPOTransform().SetPosition(
 			MT_Point3(
-			blenderobject->loc[0]+blenderobject->dloc[0],
-			blenderobject->loc[1]+blenderobject->dloc[1],
-			blenderobject->loc[2]+blenderobject->dloc[2]
+			blenderobject->loc[0]/*+blenderobject->dloc[0]*/,
+			blenderobject->loc[1]/*+blenderobject->dloc[1]*/,
+			blenderobject->loc[2]/*+blenderobject->dloc[2]*/
 			)
 		);
 		ipocontr->GetIPOTransform().SetEulerAngles(
@@ -134,7 +131,7 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 					&(ipocontr->GetIPOTransform().GetPosition()[0]),
 					ipo);
 			ipocontr->AddInterpolator(interpolator);
-			ipocontr->SetModifyPosition(true);
+			ipocontr->SetIPOChannelActive(OB_LOC_X, true);
 	
 		}
 		
@@ -145,7 +142,7 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 					&(ipocontr->GetIPOTransform().GetPosition()[1]),
 					ipo);
 			ipocontr->AddInterpolator(interpolator);
-			ipocontr->SetModifyPosition(true);
+			ipocontr->SetIPOChannelActive(OB_LOC_Y, true);
 		}
 		
 		ipo = ipoList->GetScalarInterpolator(OB_LOC_Z);
@@ -155,7 +152,7 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 					&(ipocontr->GetIPOTransform().GetPosition()[2]),
 					ipo);
 			ipocontr->AddInterpolator(interpolator);
-			ipocontr->SetModifyPosition(true);
+			ipocontr->SetIPOChannelActive(OB_LOC_Z, true);
 		}
 		
 		// Master the art of cut & paste programming...
@@ -167,7 +164,7 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 					&(ipocontr->GetIPOTransform().GetDeltaPosition()[0]),
 					ipo);
 			ipocontr->AddInterpolator(interpolator);
-			ipocontr->SetModifyPosition(true);
+			ipocontr->SetIPOChannelActive(OB_DLOC_X, true);
 		}
 		
 		ipo = ipoList->GetScalarInterpolator(OB_DLOC_Y);
@@ -177,7 +174,7 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 					&(ipocontr->GetIPOTransform().GetDeltaPosition()[1]),
 					ipo);
 			ipocontr->AddInterpolator(interpolator);
-			ipocontr->SetModifyPosition(true);
+			ipocontr->SetIPOChannelActive(OB_DLOC_Y, true);
 		}
 		
 		ipo = ipoList->GetScalarInterpolator(OB_DLOC_Z);
@@ -187,7 +184,7 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 					&(ipocontr->GetIPOTransform().GetDeltaPosition()[2]),
 					ipo);
 			ipocontr->AddInterpolator(interpolator);
-			ipocontr->SetModifyPosition(true);
+			ipocontr->SetIPOChannelActive(OB_DLOC_Z, true);
 		}
 		
 		// Explore the finesse of reuse and slight modification
@@ -199,7 +196,7 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 					&(ipocontr->GetIPOTransform().GetEulerAngles()[0]),
 					ipo);
 			ipocontr->AddInterpolator(interpolator);
-			ipocontr->SetModifyOrientation(true);
+			ipocontr->SetIPOChannelActive(OB_ROT_X, true);
 		}
 		ipo = ipoList->GetScalarInterpolator(OB_ROT_Y);
 		if (ipo) {
@@ -208,7 +205,7 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 					&(ipocontr->GetIPOTransform().GetEulerAngles()[1]),
 					ipo);
 			ipocontr->AddInterpolator(interpolator);
-			ipocontr->SetModifyOrientation(true);
+			ipocontr->SetIPOChannelActive(OB_ROT_Y, true);
 		}
 		ipo = ipoList->GetScalarInterpolator(OB_ROT_Z);
 		if (ipo) {
@@ -217,7 +214,7 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 					&(ipocontr->GetIPOTransform().GetEulerAngles()[2]),
 					ipo);
 			ipocontr->AddInterpolator(interpolator);
-			ipocontr->SetModifyOrientation(true);
+			ipocontr->SetIPOChannelActive(OB_ROT_Z, true);
 		}
 
 		// Hmmm, the need for a macro comes to mind... 
@@ -229,7 +226,7 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 					&(ipocontr->GetIPOTransform().GetDeltaEulerAngles()[0]),
 					ipo);
 			ipocontr->AddInterpolator(interpolator);
-			ipocontr->SetModifyOrientation(true);
+			ipocontr->SetIPOChannelActive(OB_DROT_X, true);
 		}
 		ipo = ipoList->GetScalarInterpolator(OB_DROT_Y);
 		if (ipo) {
@@ -238,7 +235,7 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 					&(ipocontr->GetIPOTransform().GetDeltaEulerAngles()[1]),
 					ipo);
 			ipocontr->AddInterpolator(interpolator);
-			ipocontr->SetModifyOrientation(true);
+			ipocontr->SetIPOChannelActive(OB_DROT_Y, true);
 		}
 		ipo = ipoList->GetScalarInterpolator(OB_DROT_Z);
 		if (ipo) {
@@ -247,7 +244,7 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 					&(ipocontr->GetIPOTransform().GetDeltaEulerAngles()[2]),
 					ipo);
 			ipocontr->AddInterpolator(interpolator);
-			ipocontr->SetModifyOrientation(true);
+			ipocontr->SetIPOChannelActive(OB_DROT_Z, true);
 		}
 
 		// Hang on, almost there... 
@@ -259,7 +256,7 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 					&(ipocontr->GetIPOTransform().GetScaling()[0]),
 					ipo);
 			ipocontr->AddInterpolator(interpolator);
-			ipocontr->SetModifyScaling(true);
+			ipocontr->SetIPOChannelActive(OB_SIZE_X, true);
 		}
 		ipo = ipoList->GetScalarInterpolator(OB_SIZE_Y);
 		if (ipo) {
@@ -268,7 +265,7 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 					&(ipocontr->GetIPOTransform().GetScaling()[1]),
 					ipo);
 			ipocontr->AddInterpolator(interpolator);
-			ipocontr->SetModifyScaling(true);
+			ipocontr->SetIPOChannelActive(OB_SIZE_Y, true);
 		}
 		ipo = ipoList->GetScalarInterpolator(OB_SIZE_Z);
 		if (ipo) {
@@ -277,7 +274,7 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 					&(ipocontr->GetIPOTransform().GetScaling()[2]),
 					ipo);
 			ipocontr->AddInterpolator(interpolator);
-			ipocontr->SetModifyScaling(true);
+			ipocontr->SetIPOChannelActive(OB_SIZE_Z, true);
 		}
 
 		// The last few... 
@@ -289,7 +286,7 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 					&(ipocontr->GetIPOTransform().GetDeltaScaling()[0]),
 					ipo);
 			ipocontr->AddInterpolator(interpolator);
-			ipocontr->SetModifyScaling(true);
+			ipocontr->SetIPOChannelActive(OB_DSIZE_X, true);
 		}
 		ipo = ipoList->GetScalarInterpolator(OB_DSIZE_Y);
 		if (ipo) {
@@ -298,7 +295,7 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 					&(ipocontr->GetIPOTransform().GetDeltaScaling()[1]),
 					ipo);
 			ipocontr->AddInterpolator(interpolator);
-			ipocontr->SetModifyScaling(true);
+			ipocontr->SetIPOChannelActive(OB_DSIZE_Y, true);
 		}
 		ipo = ipoList->GetScalarInterpolator(OB_DSIZE_Z);
 		if (ipo) {
@@ -307,7 +304,7 @@ void BL_ConvertIpos(struct Object* blenderobject,KX_GameObject* gameobj,KX_Blend
 					&(ipocontr->GetIPOTransform().GetDeltaScaling()[2]),
 					ipo);
 			ipocontr->AddInterpolator(interpolator);
-			ipocontr->SetModifyScaling(true);
+			ipocontr->SetIPOChannelActive(OB_DSIZE_Z, true);
 		}
 		
 		{
