@@ -1,13 +1,10 @@
 /**
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +22,7 @@
  *
  * Contributor(s): snailrose.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 #include <SDL.h>
 
@@ -55,6 +52,34 @@ SCA_Joystick::~SCA_Joystick()
 	delete m_private;
 }
 
+SCA_Joystick *SCA_Joystick::m_instance = NULL;
+int SCA_Joystick::m_refCount = 0;
+
+SCA_Joystick *SCA_Joystick::GetInstance()
+{
+	if (m_instance == 0) 
+	{
+		m_instance = new SCA_Joystick();
+		m_instance->CreateJoystickDevice();
+		m_refCount = 1;
+	}
+	else
+	{
+		m_refCount++;
+	}
+	return m_instance;
+}
+
+void SCA_Joystick::ReleaseInstance()
+{
+	if (--m_refCount == 0)
+	{
+		DestroyJoystickDevice();
+		delete m_instance;
+		m_instance = NULL;
+	}
+}
+
 
 bool SCA_Joystick::CreateJoystickDevice()
 {
