@@ -1,15 +1,12 @@
 /**
- * $Id: SCA_IActuator.h 3271 2004-10-16 11:41:50Z kester $
+ * $Id: SCA_IActuator.h 14557 2008-04-26 20:41:25Z ben2610 $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 #ifndef __KX_IACTUATOR
 #define __KX_IACTUATOR
@@ -49,6 +46,15 @@ public:
 	SCA_IActuator(SCA_IObject* gameobj,
 				  PyTypeObject* T =&Type); 
 
+	/**
+	 * UnlinkObject(...)
+	 * Certain actuator use gameobject pointers (like TractTo actuator)
+	 * This function can be called when an object is removed to make
+	 * sure that the actuator will not use it anymore.
+	 */
+
+	virtual bool UnlinkObject(SCA_IObject* clientobj) { return false; }
+
 	/**
 	 * Update(...)
 	 * Update the actuator based upon the events received since 
