@@ -1,17 +1,14 @@
 /**
  * Property sensor
  *
- * $Id: SCA_PropertySensor.cpp 9459 2006-12-25 02:02:13Z erwin $
+ * $Id: SCA_PropertySensor.cpp 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,7 +26,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <iostream>
@@ -66,11 +63,11 @@ SCA_PropertySensor::SCA_PropertySensor(SCA_EventManager* eventmgr,
 	//CValue* resultval = m_rightexpr->Calculate();
 
 	CValue* orgprop = GetParent()->FindIdentifier(m_checkpropname);
-	if (orgprop)
+	if (!orgprop->IsError())
 	{
 		m_previoustext = orgprop->GetText();
-		orgprop->Release();
 	}
+	orgprop->Release();
 
 	if (m_checktype==KX_PROPSENSOR_INTERVAL)
 	{
@@ -82,16 +79,28 @@ SCA_PropertySensor::SCA_PropertySensor(SCA_EventManager* eventmgr,
 void SCA_PropertySensor::PrecalculateRangeExpression()
 {
 		CParser pars;
+		//The context is needed to retrieve the property at runtime but it creates
+		//loop of references
 		pars.SetContext(this->AddRef());
 		STR_String checkstr = "(" + m_checkpropval + " <= " 
 							+ m_checkpropname + ") && ( " 
 							+ m_checkpropname + " <= " 
-							+ m_checkpropmaxval;
+							+ m_checkpropmaxval + ")";
 
 		m_range_expr = pars.ProcessText(checkstr);
 }
 
-
+// Forced deletion of precalculated range expression to break reference loop
+// Use this function when you know that you won't use the sensor anymore
+void SCA_PropertySensor::Delete()
+{
+	if (m_range_expr)
+	{
+		m_range_expr->Release();
+		m_range_expr = NULL;
+	}
+	Release();
+}
 
 CValue* SCA_PropertySensor::GetReplica()
 {
@@ -164,7 +173,7 @@ bool	SCA_PropertySensor::CheckPropertyCondition()
 	case KX_PROPSENSOR_EQUAL:
 		{
 			CValue* orgprop = GetParent()->FindIdentifier(m_checkpropname);
-			if (orgprop)
+			if (!orgprop->IsError())
 			{
 				STR_String testprop = orgprop->GetText();
 				// Force strings to upper case, to avoid confusion in
@@ -177,9 +186,8 @@ bool	SCA_PropertySensor::CheckPropertyCondition()
 				} else {
 					result = (orgprop->GetText() == m_checkpropval);
 				}
-				orgprop->Release();
-
 			}
+			orgprop->Release();
 
 			if (reverse)
 				result = !result;
@@ -244,15 +252,15 @@ bool	SCA_PropertySensor::CheckPropertyCondition()
 		{
 			CValue* orgprop = GetParent()->FindIdentifier(m_checkpropname);
 				
-			if (orgprop)
+			if (!orgprop->IsError())
 			{
 				if (m_previoustext != orgprop->GetText())
 				{
 					m_previoustext = orgprop->GetText();
 					result = true;
 				}
-				orgprop->Release();
 			}
+			orgprop->Release();
 
 			//cout << " \nSens:Prop:changed!"; /* need implementation here!!! */
 			break;
@@ -388,12 +396,13 @@ PyObject* SCA_PropertySensor::PySetProperty(PyObject* self, PyObject* args, PyOb
 		return NULL;
 	}
 
-	if (FindIdentifier(STR_String(propNameArg))) {
+	CValue *prop = FindIdentifier(STR_String(propNameArg));
+	if (!prop->IsError()) {
 		m_checkpropname = propNameArg;
 	} else {
 		; /* error: bad property name */
 	}
-
+	prop->Release();
 	Py_Return;
 }
 
