@@ -1,17 +1,14 @@
 /**
  * Property sensor
  *
- * $Id: SCA_PropertySensor.h,v 1.7 2004/05/16 13:05:12 kester Exp $
+ * $Id: SCA_PropertySensor.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,7 +26,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifndef __KX_PROPERTYSENSOR
@@ -77,6 +74,7 @@ public:
 					  KX_PROPSENSOR_TYPE checktype,
 					  PyTypeObject* T=&Type );
 	
+	virtual void Delete();
 	virtual ~SCA_PropertySensor();
 	virtual CValue* GetReplica();
 	void	PrecalculateRangeExpression();
