@@ -1,17 +1,14 @@
 /**
  * Generate random pulses
  *
- * $Id: SCA_RandomSensor.h 2548 2004-05-16 13:05:15Z kester $
+ * $Id: SCA_RandomSensor.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,7 +26,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifndef __KX_RANDOMSENSOR
@@ -45,6 +42,7 @@ class SCA_RandomSensor : public SCA_ISensor
 
 	unsigned int m_currentDraw;
 	int m_iteration;
+	int m_interval;
 	SCA_RandomNumberGenerator *m_basegenerator;
 	bool m_lastdraw;
 public:
