@@ -1,15 +1,12 @@
 /**
-* $Id: GPG_ghost.cpp 11527 2007-08-09 15:05:41Z bdiego $
+* $Id: GPG_ghost.cpp 14798 2008-05-11 18:45:30Z ben2610 $
 *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
 * Start up of the Blender Player on GHOST.
 */
 
@@ -304,7 +301,6 @@ int main(int argc, char** argv)
 #endif /* __alpha__ */
 #endif /* __linux__ */
 	BLI_where_am_i(bprogname, argv[0]);
-	
 #ifdef __APPLE__
     // Can't use Carbon right now because of double defined type ID (In Carbon.h and DNA_ID.h, sigh)
     /*
@@ -566,26 +562,37 @@ int main(int argc, char** argv)
 				STR_String exitstring = "";
 				GPG_Application app(system, NULL, exitstring);
 				bool firstTimeRunning = true;
+				char *filename = get_filename(argc, argv);
+				char *titlename;
+				char pathname[160];
 				
 				do
 				{
 					// Read the Blender file
-					char *filename = get_filename(argc, argv);
-					char *titlename;
-					char pathname[160];
 					BlendFileData *bfd;
 					
 					// if we got an exitcode 3 (KX_EXIT_REQUEST_START_OTHER_GAME) load a different file
 					if (exitcode == KX_EXIT_REQUEST_START_OTHER_GAME)
 					{
-						char basedpath[160];
+						char basedpath[240];
 						
 						// base the actuator filename with respect
 						// to the original file working directory
 						strcpy(basedpath, exitstring.Ptr());
-						BLI_convertstringcode(basedpath, pathname, 0);
+						BLI_convertstringcode(basedpath, pathname);
 						
 						bfd = load_game_data(basedpath);
+
+						if (!bfd)
+						{
+							// just add "//" in front of it
+							char temppath[242];
+							strcpy(temppath, "//");
+							strcat(temppath, basedpath);
+				
+							BLI_convertstringcode(temppath, pathname);
+							bfd = load_game_data(temppath);
+						}
 					}
 					else
 					{
@@ -611,7 +618,6 @@ int main(int argc, char** argv)
 #endif // WIN32
 						Main *maggie = bfd->main;
 						Scene *scene = bfd->curscene;
-						strcpy (pathname, maggie->name);
 						char *startscenename = scene->id.name + 2;
 						G.fileflags  = bfd->fileflags;
 
@@ -655,7 +661,12 @@ int main(int argc, char** argv)
 						if (firstTimeRunning)
 						{
 							firstTimeRunning = false;
-							
+
+							// set the filename only the first time as in KetsjiEmbedded
+							strcpy (pathname, maggie->name);
+							// also copy here (used by GameLogic.getBaseDirectory)
+							strcpy (G.sce, maggie->name);
+
 							if (fullScreen)
 							{
 #ifdef WIN32
