@@ -1,15 +1,12 @@
 /**
- * $Id: UnixShell.c,v 1.4 2004/03/22 22:01:46 jesterking Exp $
+ * $Id: UnixShell.c 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,11 +24,11 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  * NS api template, adapted to link to our own internals.
  */
 
-#define UNIXSH_VERSION "$Id: UnixShell.c,v 1.4 2004/03/22 22:01:46 jesterking Exp $"
+#define UNIXSH_VERSION "$Id: UnixShell.c 14444 2008-04-16 22:40:48Z hos $"
 
 /* -*- Mode: C; tab-width: 8; c-set-style: bsd -*- */
 
@@ -157,7 +154,7 @@ NPP_GetValue(
 	case NPPVpluginDescriptionString:
 		log_entry("NPP_GetValue::NPPVpluginDescriptionString");
 		*((char **)value) =
-		  "Player for interactive 3D content.\n";
+			"Player for interactive 3D content";
 		break;
 	case NPPVpluginScriptableInstance:
 		/* This cast must succeed... otherwise we'll get nasty
