@@ -1,15 +1,12 @@
 /**
- * $Id: KXH_ketsji_hooks.cpp,v 1.9 2007/04/04 13:18:38 campbellbarton Exp $
+ * $Id: KXH_ketsji_hooks.cpp 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  * Implementation of the application hooks between browser and app, 
  * where app = Ketjsi.
  */
@@ -883,9 +880,8 @@ set_plain_color_redraw_func(ketsji_engine_data* k)
   //	KXH_log_entry("set_plain_color_redraw_func");
 	PLA_acquire_gl_context(k->plugin);
 
-	float* c = PLA_get_foreground_color(k->plugin);
-
 #ifndef _WIN32
+	float *c = PLA_get_foreground_colour(k->plugin);
  	glClearColor(c[0], c[1], c[2], 1.0);
  	glClear(GL_COLOR_BUFFER_BIT);
 #endif
