@@ -1,4 +1,5 @@
-// ------------------------------------
+
+
 #ifdef WIN32
 #include <windows.h>
 #endif // WIN32
@@ -8,7 +9,12 @@
 #include <OpenGL/glu.h>
 #else
 #include <GL/gl.h>
+/* #if defined(__sun__) && !defined(__sparc__)
+#include <mesa/glu.h>
+#else
+*/
 #include <GL/glu.h>
+/* #endif */
 #endif
 
 #include <iostream>
@@ -1422,8 +1428,8 @@ KX_PYMETHODDEF_DOC( BL_Shader, setAttrib, "setAttrib(enum)" )
 		bgl::blBindAttribLocationARB(mShader, mAttr, "Tangent");
 		Py_Return;
 	}
-	return NULL;
 #endif
+	return NULL;
 }
 
 
