@@ -1,15 +1,12 @@
 /**
- * $Id: KX_NetworkMessageSensor.cpp 9639 2007-01-07 04:35:33Z erwin $
+ * $Id: KX_NetworkMessageSensor.cpp 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  * Ketsji Logic Extenstion: Network Message Sensor generic implementation
  */
 
@@ -142,7 +139,11 @@ bool KX_NetworkMessageSensor::Evaluate(CValue* event)
 
 	result = (WasUp != m_IsUp);
 
-	// Return true if the message received state has changed. 
+	// Return always true if a message was received otherwise we can loose messages
+	if (m_IsUp)
+		return true;
+	// Is it usefull to return also true when the first frame without a message?? 
+	// This will cause a fast on/off cycle that seems useless!
 	return result;
 }
 
