@@ -1,15 +1,12 @@
 /*
- * $Id: KX_Camera.cpp,v 1.17 2007/04/04 13:18:39 campbellbarton Exp $
+ * $Id: KX_Camera.cpp 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,12 +24,13 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  * Camera in the gameengine. Cameras are also used for views.
  */
  
 #include "KX_Camera.h"
-
+#include "KX_Scene.h"
+#include "KX_PythonInit.h"
 #include "KX_Python.h"
 #include "KX_PyMath.h"
 #ifdef HAVE_CONFIG_H
@@ -57,7 +55,9 @@ KX_Camera::KX_Camera(void* sgReplicationInfo,
 	m_name = "cam";
 	m_projection_matrix.setIdentity();
 	m_modelview_matrix.setIdentity();
-	SetProperty("camera",new CIntValue(1));
+	CValue* val = new CIntValue(1);
+	SetProperty("camera",val);
+	val->Release();
 }
 
 
@@ -66,7 +66,22 @@ KX_Camera::~KX_Camera()
 }	
 
 
+CValue*	KX_Camera::GetReplica()
+{
+	KX_Camera* replica = new KX_Camera(*this);
 	
+	// this will copy properties and so on...
+	CValue::AddDataToReplica(replica);
+	ProcessReplica(replica);
+	
+	return replica;
+}
+	
+void KX_Camera::ProcessReplica(KX_Camera* replica)
+{
+	KX_GameObject::ProcessReplica(replica);
+}
+
 MT_Transform KX_Camera::GetWorldToCamera() const
 { 
 	MT_Transform camtrans;
@@ -391,6 +406,7 @@ PyMethodDef KX_Camera::Methods[] = {
 	KX_PYMETHODTABLE(KX_Camera, setProjectionMatrix),
 	KX_PYMETHODTABLE(KX_Camera, enableViewport),
 	KX_PYMETHODTABLE(KX_Camera, setViewport),
+	KX_PYMETHODTABLE(KX_Camera, setOnTop),
 	
 	{NULL,NULL} //Sentinel
 };
@@ -759,3 +775,15 @@ KX_PYMETHODDEF_DOC(KX_Camera, setViewport,
 	}
 	Py_Return;
 }
+
+KX_PYMETHODDEF_DOC(KX_Camera, setOnTop,
+"setOnTop()\n"
+"Sets this camera's viewport on top\n")
+{
+	class KX_Scene* scene;
+	
+	scene = PHY_GetActiveScene();
+	MT_assert(scene);
+	scene->SetCameraOnTop(this);
+	Py_Return;
+}
