@@ -1,17 +1,14 @@
 /**
  * KX_CameraActuator.cpp
  *
- * $Id: KX_CameraActuator.cpp 10455 2007-04-04 13:18:41Z campbellbarton $
+ * $Id: KX_CameraActuator.cpp 14737 2008-05-07 22:32:45Z ben2610 $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,7 +26,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  *
  */
 
@@ -158,14 +155,25 @@ static void Kx_VecUpMat3(float *vec, float mat[][3], short axis)
 	mat[coz][0]= vec[0];
 	mat[coz][1]= vec[1];
 	mat[coz][2]= vec[2];
-	Kx_Normalize((float *)mat[coz]);
+	if (Kx_Normalize((float *)mat[coz]) == 0.f) {
+		/* this is a very abnormal situation: the camera has reach the object center exactly
+		   We will choose a completely arbitrary direction */
+		mat[coz][0] = 1.0f;
+		mat[coz][1] = 0.0f;
+		mat[coz][2] = 0.0f;
+	}
 	
 	inp= mat[coz][2];
 	mat[coy][0]= - inp*mat[coz][0];
 	mat[coy][1]= - inp*mat[coz][1];
 	mat[coy][2]= 1.0 - inp*mat[coz][2];
 
-	Kx_Normalize((float *)mat[coy]);
+	if (Kx_Normalize((float *)mat[coy]) == 0.f) {
+		/* the camera is vertical, chose the y axis arbitrary */
+		mat[coy][0] = 0.f;
+		mat[coy][1] = 1.f;
+		mat[coy][2] = 0.f;
+	}
 	
 	Kx_Crossf(mat[cox], mat[coy], mat[coz]);
 	
