@@ -1,15 +1,12 @@
 /**
- * $Id: KX_GameObject.cpp 9460 2006-12-25 04:11:39Z erwin $
+ * $Id: KX_GameObject.cpp 14713 2008-05-06 20:55:55Z ben2610 $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  * Game object wrapper
  */
 
@@ -61,7 +58,8 @@ typedef unsigned long uint_ptr;
 #include "SG_Controller.h"
 #include "KX_ClientObjectInfo.h"
 #include "RAS_BucketManager.h"
-
+#include "KX_RayCast.h"
+#include "KX_PythonInit.h"
 #include "KX_PyMath.h"
 
 // This file defines relationships between parents and children
@@ -76,16 +74,19 @@ KX_GameObject::KX_GameObject(
 ) : 
 	SCA_IObject(T),
 	m_bDyna(false),
+	m_layer(0),
 	m_bSuspendDynamics(false),
 	m_bUseObjectColor(false),
 	m_bVisible(true),
 	m_pPhysicsController1(NULL),
-	m_isDeformable(false)
+	m_pPhysicsEnvironment(NULL),
+	m_isDeformable(false),
+	m_pHitObject(NULL)
 {
 	m_ignore_activity_culling = false;
 	m_pClient_info = new KX_ClientObjectInfo(this, KX_ClientObjectInfo::ACTOR);
 	m_pSGNode = new SG_Node(this,sgReplicationInfo,callbacks);
-	
+
 	// define the relationship between this node and it's parent.
 	
 	KX_NormalParentRelation * parent_relation = 
@@ -105,7 +106,19 @@ KX_GameObject::~KX_GameObject()
 	delete m_pClient_info;
 	//if (m_pSGNode)
 	//	delete m_pSGNode;
-	
+	if (m_pSGNode)
+	{
+		// must go through controllers and make sure they will not use us anymore
+		// This is important for KX_BulletPhysicsControllers that unregister themselves
+		// from the object when they are deleted.
+		SGControllerList::iterator contit;
+		SGControllerList& controllers = m_pSGNode->GetSGControllerList();
+		for (contit = controllers.begin();contit!=controllers.end();++contit)
+		{
+			(*contit)->ClearObject();
+		}
+		m_pSGNode->SetSGClientObject(NULL);
+	}
 }
 
 
@@ -189,7 +202,62 @@ KX_GameObject* KX_GameObject::GetParent()
 	
 }
 
+void KX_GameObject::SetParent(KX_Scene *scene, KX_GameObject* obj)
+{
+	if (obj && GetSGNode()->GetSGParent() != obj->GetSGNode())
+	{
+		// Make sure the objects have some scale
+		MT_Vector3 scale1 = NodeGetWorldScaling();
+		MT_Vector3 scale2 = obj->NodeGetWorldScaling();
+		if (fabs(scale2[0]) < FLT_EPSILON || 
+			fabs(scale2[1]) < FLT_EPSILON || 
+			fabs(scale2[2]) < FLT_EPSILON || 
+			fabs(scale1[0]) < FLT_EPSILON || 
+			fabs(scale1[1]) < FLT_EPSILON || 
+			fabs(scale1[2]) < FLT_EPSILON) { return; }
 
+		// Remove us from our old parent and set our new parent
+		RemoveParent(scene);
+		obj->GetSGNode()->AddChild(GetSGNode());
+
+		// Set us to our new scale, position, and orientation
+		scale1[0] = scale1[0]/scale2[0];
+		scale1[1] = scale1[1]/scale2[1];
+		scale1[2] = scale1[2]/scale2[2];
+		MT_Matrix3x3 invori = obj->NodeGetWorldOrientation().inverse();
+		MT_Vector3 newpos = invori*(NodeGetWorldPosition()-obj->NodeGetWorldPosition())*scale1;
+
+		NodeSetLocalScale(scale1);
+		NodeSetLocalPosition(MT_Point3(newpos[0],newpos[1],newpos[2]));
+		NodeSetLocalOrientation(invori*NodeGetWorldOrientation());
+		NodeUpdateGS(0.f,true);
+		// object will now be a child, it must be removed from the parent list
+		CListValue* rootlist = scene->GetRootParentList();
+		if (rootlist->RemoveValue(this))
+			// the object was in parent list, decrement ref count as it's now removed
+			Release();
+	}
+}
+
+void KX_GameObject::RemoveParent(KX_Scene *scene)
+{
+	if (GetSGNode()->GetSGParent())
+	{
+		// Set us to the right spot 
+		GetSGNode()->SetLocalScale(GetSGNode()->GetWorldScaling());
+		GetSGNode()->SetLocalOrientation(GetSGNode()->GetWorldOrientation());
+		GetSGNode()->SetLocalPosition(GetSGNode()->GetWorldPosition());
+
+		// Remove us from our parent
+		GetSGNode()->DisconnectFromParent();
+		NodeUpdateGS(0.f,true);
+		// the object is now a root object, add it to the parentlist
+		CListValue* rootlist = scene->GetRootParentList();
+		if (!rootlist->SearchValue(this))
+			// object was not in root list, add it now and increment ref count
+			rootlist->Add(AddRef());
+	}
+}
 
 void KX_GameObject::ProcessReplica(KX_GameObject* replica)
 {
@@ -244,12 +312,12 @@ void KX_GameObject::ApplyMovement(const MT_Vector3& dloc,bool local)
 void KX_GameObject::ApplyRotation(const MT_Vector3& drot,bool local)
 {
 	MT_Matrix3x3 rotmat(drot);
-	rotmat.transpose();
-	
-	if (m_pPhysicsController1) // (IsDynamic())
-		m_pPhysicsController1->RelativeRotate(rotmat,local); 
-	// in worldspace
+
 	GetSGNode()->RelativeRotate(rotmat,local);
+
+	if (m_pPhysicsController1) { // (IsDynamic())
+		m_pPhysicsController1->RelativeRotate(rotmat,local); 
+	}
 }
 
 
@@ -412,6 +480,22 @@ KX_GameObject::SetVisible(
 	m_bVisible = v;
 }
 
+void
+KX_GameObject::SetLayer(
+	int l
+	)
+{
+	m_layer = l;
+}
+
+int
+KX_GameObject::GetLayer(
+	void
+	)
+{
+	return m_layer;
+}
+
 // used by Python, and the actuatorshould _not_ be misused by the
 // scene!
 void 
@@ -497,16 +581,27 @@ void KX_GameObject::SetObjectColor(const MT_Vector4& rgbavec)
 
 
 
-MT_Vector3 KX_GameObject::GetLinearVelocity()
+MT_Vector3 KX_GameObject::GetLinearVelocity(bool local)
 {
 	MT_Vector3 velocity(0.0,0.0,0.0);
-	
+	MT_Matrix3x3 ori, locvel;
+	int i, j; 
 	if (m_pPhysicsController1)
 	{
 		velocity = m_pPhysicsController1->GetLinearVelocity();
+		
+		if (local)
+		{
+			ori = GetSGNode()->GetWorldOrientation();
+			
+			for(i=0; i < 3; i++)
+				for(j=0; j < 3; j++)
+					locvel[i][j]= velocity[i]*ori[i][j];
+			for(i=0; i < 3; i++)
+				velocity[i] = locvel[0][i] + locvel[1][i] + locvel[2][i];
+		}
 	}
-	return velocity;
-	
+	return velocity;	
 }
 
 
@@ -642,9 +737,12 @@ PyMethodDef KX_GameObject::Methods[] = {
 	{"enableRigidBody", (PyCFunction)KX_GameObject::sPyEnableRigidBody,METH_VARARGS},
 	{"disableRigidBody", (PyCFunction)KX_GameObject::sPyDisableRigidBody,METH_VARARGS},
 	{"getParent", (PyCFunction)KX_GameObject::sPyGetParent,METH_VARARGS},
+	{"setParent", (PyCFunction)KX_GameObject::sPySetParent,METH_VARARGS},
+	{"removeParent", (PyCFunction)KX_GameObject::sPyRemoveParent,METH_VARARGS},
 	{"getMesh", (PyCFunction)KX_GameObject::sPyGetMesh,METH_VARARGS},
 	{"getPhysicsId", (PyCFunction)KX_GameObject::sPyGetPhysicsId,METH_VARARGS},
 	KX_PYMETHODTABLE(KX_GameObject, getDistanceTo),
+	KX_PYMETHODTABLE(KX_GameObject, rayCastTo),
 	{NULL,NULL} //Sentinel
 };
 
@@ -781,6 +879,7 @@ int KX_GameObject::_setattr(const STR_String& attr, PyObject *value)	// _setattr
 				if (PyMatTo(value, rot))
 				{
 					NodeSetLocalOrientation(rot);
+					NodeUpdateGS(0.f,true);
 					return 0;
 				}
 				return 1;
@@ -793,6 +892,7 @@ int KX_GameObject::_setattr(const STR_String& attr, PyObject *value)	// _setattr
 				{
 					rot.setRotation(qrot);
 					NodeSetLocalOrientation(rot);
+					NodeUpdateGS(0.f,true);
 					return 0;
 				}
 				return 1;
@@ -805,6 +905,7 @@ int KX_GameObject::_setattr(const STR_String& attr, PyObject *value)	// _setattr
 				{
 					rot.setEuler(erot);
 					NodeSetLocalOrientation(rot);
+					NodeUpdateGS(0.f,true);
 					return 0;
 				}
 				return 1;
@@ -819,6 +920,7 @@ int KX_GameObject::_setattr(const STR_String& attr, PyObject *value)	// _setattr
 			if (PyVecTo(value, pos))
 			{
 				NodeSetLocalPosition(pos);
+				NodeUpdateGS(0.f,true);
 				return 0;
 			}
 			return 1;
@@ -830,6 +932,7 @@ int KX_GameObject::_setattr(const STR_String& attr, PyObject *value)	// _setattr
 			if (PyVecTo(value, scale))
 			{
 				NodeSetLocalScale(scale);
+				NodeUpdateGS(0.f,true);
 				return 0;
 			}
 			return 1;
@@ -845,6 +948,8 @@ int KX_GameObject::_setattr(const STR_String& attr, PyObject *value)	// _setattr
 		}
 	}
 	
+	/* Need to have parent settable here too */
+	
 	return SCA_IObject::_setattr(attr, value);
 }
 
@@ -854,7 +959,15 @@ PyObject* KX_GameObject::PyGetLinearVelocity(PyObject* self,
 											 PyObject* kwds)
 {
 	// only can get the velocity if we have a physics object connected to us...
-	return PyObjectFrom(GetLinearVelocity());
+	int local = 0;
+	if (PyArg_ParseTuple(args,"|i",&local))
+	{
+		return PyObjectFrom(GetLinearVelocity((local!=0)));
+	}
+	else
+	{
+		return NULL;
+	}
 }
 
 
@@ -969,7 +1082,31 @@ PyObject* KX_GameObject::PyGetParent(PyObject* self,
 	Py_Return;
 }
 
+PyObject* KX_GameObject::PySetParent(PyObject* self, 
+									 PyObject* args, 
+									 PyObject* kwds)
+{
+	PyObject* gameobj;
+	if (PyArg_ParseTuple(args, "O!", &KX_GameObject::Type, &gameobj))
+	{
+		// The object we want to set as parent
+		CValue *m_ob = (CValue*)gameobj;
+		KX_GameObject *obj = ((KX_GameObject*)m_ob);
+		KX_Scene *scene = PHY_GetActiveScene();
+		
+		this->SetParent(scene, obj);
+	}
+	Py_Return;
+}
 
+PyObject* KX_GameObject::PyRemoveParent(PyObject* self, 
+									 PyObject* args, 
+									 PyObject* kwds)
+{
+	KX_Scene *scene = PHY_GetActiveScene();
+	this->RemoveParent(scene);
+	Py_Return;
+}
 
 PyObject* KX_GameObject::PyGetMesh(PyObject* self, 
 								   PyObject* args, 
@@ -1100,6 +1237,7 @@ PyObject* KX_GameObject::PySetOrientation(PyObject* self,
 		if (PyObject_IsMT_Matrix(pylist, 3) && PyMatTo(pylist, matrix))
 		{
 			NodeSetLocalOrientation(matrix);
+			NodeUpdateGS(0.f,true);
 			Py_Return;
 		}
 	
@@ -1108,6 +1246,7 @@ PyObject* KX_GameObject::PySetOrientation(PyObject* self,
 		{
 			matrix.setRotation(quat);
 			NodeSetLocalOrientation(matrix);
+			NodeUpdateGS(0.f,true);
 			Py_Return;
 		}
 	}
@@ -1164,6 +1303,83 @@ KX_PYMETHODDEF_DOC(KX_GameObject, getDistanceTo,
 	return NULL;
 }
 
+bool KX_GameObject::RayHit(KX_ClientObjectInfo* client, MT_Point3& hit_point, MT_Vector3& hit_normal, void * const data)
+{
+
+	KX_GameObject* hitKXObj = client->m_gameobject;
+	
+	if (client->m_type > KX_ClientObjectInfo::ACTOR)
+	{
+		// false hit
+		return false;
+	}
+
+	if (m_testPropName.Length() == 0 || hitKXObj->GetProperty(m_testPropName) != NULL)
+	{
+		m_pHitObject = hitKXObj;
+		return true;
+	}
+
+	return false;
+	
+}
+
+KX_PYMETHODDEF_DOC(KX_GameObject, rayCastTo,
+"rayCastTo(other,dist,prop): look towards another point/KX_GameObject and return first object hit within dist that match prop\n"
+" prop = property name that object must have; can be omitted => detect any object\n"
+" dist = max distance to look (can be negative => look behind); 0 or omitted => detect up to other\n"
+" other = 3-tuple or object reference")
+{
+	MT_Point3 toPoint;
+	PyObject* pyarg;
+	float dist = 0.0f;
+	char *propName = NULL;
+
+	if (!PyArg_ParseTuple(args,"O|fs", &pyarg, &dist, &propName))
+		return NULL;
+
+	if (!PyVecTo(pyarg, toPoint))
+	{
+		KX_GameObject *other;
+		PyErr_Clear();
+		if (!PyType_IsSubtype(pyarg->ob_type, &KX_GameObject::Type))
+			return NULL;
+		other = static_cast<KX_GameObject*>(pyarg);
+		toPoint = other->NodeGetWorldPosition();
+	}
+	MT_Point3 fromPoint = NodeGetWorldPosition();
+	if (dist != 0.0f)
+	{
+		MT_Vector3 toDir = toPoint-fromPoint;
+		toDir.normalize();
+		toPoint = fromPoint + (dist) * toDir;
+	}
+
+	MT_Point3 resultPoint;
+	MT_Vector3 resultNormal;
+	PHY_IPhysicsEnvironment* pe = GetPhysicsEnvironment();
+	KX_IPhysicsController *spc = GetPhysicsController();
+	KX_GameObject *parent = GetParent();
+	if (!spc && parent)
+		spc = parent->GetPhysicsController();
+	if (parent)
+		parent->Release();
+	
+	m_pHitObject = NULL;
+	if (propName)
+		m_testPropName = propName;
+	else
+		m_testPropName.SetLength(0);
+	KX_RayCast::RayTest(spc, pe, fromPoint, toPoint, resultPoint, resultNormal, KX_RayCast::Callback<KX_GameObject>(this));
+
+    if (m_pHitObject)
+	{
+		m_pHitObject->AddRef();
+		return m_pHitObject;
+	}
+	Py_Return;
+}
+
 /* --------------------------------------------------------------------- 
  * Some stuff taken from the header
  * --------------------------------------------------------------------- */
