@@ -1,15 +1,12 @@
 /**
- * $Id: KX_PythonInit.cpp,v 1.24 2006/07/06 07:58:07 erwin Exp $
+ * $Id: KX_PythonInit.cpp 14798 2008-05-11 18:45:30Z ben2610 $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  * Initialize Python thingies.
  */
 
@@ -44,9 +41,15 @@
 #include <OpenGL/glu.h>
 #else
 #include <GL/gl.h>
+/* #if defined(__sun__) && !defined(__sparc__)
+#include <mesa/glu.h>
+#else */
 #include <GL/glu.h>
+/* #endif */
 #endif
 
+#include <stdlib.h>
+
 #ifdef WIN32
 #pragma warning (disable : 4786)
 #endif //WIN32
@@ -85,6 +88,10 @@
 //#include "BPY_extern.h"
 #endif 
 
+#include "BKE_utildefines.h"
+#include "BKE_global.h"
+#include "BLI_blenlib.h"
+
 static void setSandbox(TPythonSecurityLevel level);
 
 
@@ -137,6 +144,32 @@ static PyObject* gPySetGravity(PyObject*,
 	return NULL;
 }
 
+static char gPyExpandPath_doc[] =
+"(path) - Converts a blender internal path into a proper file system path.\n\
+path - the string path to convert.\n\n\
+Use / as directory separator in path\n\
+You can use '//' at the start of the string to define a relative path;\n\
+Blender replaces that string by the directory of the startup .blend or runtime\n\
+file to make a full path name (doesn't change during the game, even if you load\n\
+other .blend).\n\
+The function also converts the directory separator to the local file system format.";
+
+static PyObject* gPyExpandPath(PyObject*,
+								PyObject* args, 
+								PyObject*)
+{
+	char expanded[FILE_MAXDIR + FILE_MAXFILE];
+	char* filename;
+	
+	if (PyArg_ParseTuple(args,"s",&filename))
+	{
+		BLI_strncpy(expanded, filename, FILE_MAXDIR + FILE_MAXFILE);
+		BLI_convertstringcode(expanded, G.sce);
+		return PyString_FromString(expanded);
+	}
+	return NULL;
+}
+
 
 static bool usedsp = false;
 
@@ -331,16 +364,18 @@ static PyObject *pyPrintExt(PyObject *,PyObject *,PyObject *)
 		pprint("");
 	}
 #endif
-#ifdef GL_ARB_multitexture
-	support = ext._ARB_multitexture;
-	count = 1;
-	pprint(" GL_ARB_multitexture supported?         "<< (support?"yes.":"no."));
-	if(support){
-		pprint(" ----------Details----------");
-		int units=0;
-		glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, (GLint*)&units);
-		pprint("  Max texture units available.  " << units);
-		pprint("");
+#if defined(GL_ARB_multitexture) && defined(WITH_GLEXT)
+	if (!getenv("WITHOUT_GLEXT")) {
+		support = ext._ARB_multitexture;
+		count = 1;
+		pprint(" GL_ARB_multitexture supported?         "<< (support?"yes.":"no."));
+		if(support){
+			pprint(" ----------Details----------");
+			int units=0;
+			glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, (GLint*)&units);
+			pprint("  Max texture units available.  " << units);
+			pprint("");
+		}
 	}
 #endif
 #ifdef GL_ARB_texture_env_combine
@@ -356,6 +391,7 @@ static PyObject *pyPrintExt(PyObject *,PyObject *,PyObject *)
 
 
 static struct PyMethodDef game_methods[] = {
+	{"expandPath", (PyCFunction)gPyExpandPath, METH_VARARGS, gPyExpandPath_doc},
 	{"getCurrentController",
 	(PyCFunction) SCA_PythonController::sPyGetCurrentController,
 	METH_VARARGS, SCA_PythonController::sPyGetCurrentController__doc__},
@@ -618,7 +654,31 @@ static PyObject* gPyMakeScreenshot(PyObject*,
 	Py_Return;
 }
 
+static PyObject* gPyEnableMotionBlur(PyObject*,
+									PyObject* args,
+									PyObject*)
+{
+	float motionblurvalue;
+	if (PyArg_ParseTuple(args,"f",&motionblurvalue))
+	{
+		if(gp_Rasterizer)
+		{
+			gp_Rasterizer->EnableMotionBlur(motionblurvalue);
+		}
+	}
+	Py_Return;
+}
 
+static PyObject* gPyDisableMotionBlur(PyObject*,
+									PyObject* args,
+									PyObject*)
+{
+	if(gp_Rasterizer)
+	{
+		gp_Rasterizer->DisableMotionBlur();
+	}
+	Py_Return;
+}
 
 STR_String	gPyGetWindowHeight__doc__="getWindowHeight doc";
 STR_String	gPyGetWindowWidth__doc__="getWindowWidth doc";
@@ -645,6 +705,9 @@ static struct PyMethodDef rasterizer_methods[] = {
  {"setMistColor",(PyCFunction)gPySetMistColor,METH_VARARGS,"set Mist Color (rgb)"},
   {"setMistStart",(PyCFunction)gPySetMistStart,METH_VARARGS,"set Mist Start(rgb)"},
   {"setMistEnd",(PyCFunction)gPySetMistEnd,METH_VARARGS,"set Mist End(rgb)"},
+  {"enableMotionBlur",(PyCFunction)gPyEnableMotionBlur,METH_VARARGS,"enable motion blur"},
+  {"disableMotionBlur",(PyCFunction)gPyDisableMotionBlur,METH_VARARGS,"disable motion blur"},
+
   
   {"setEyeSeparation", (PyCFunction) gPySetEyeSeparation, METH_VARARGS, "set the eye separation for stereo mode"},
   {"getEyeSeparation", (PyCFunction) gPyGetEyeSeparation, METH_VARARGS, "get the eye separation for stereo mode"},
@@ -1128,3 +1191,8 @@ void PHY_SetActiveScene(class KX_Scene* scene)
 {
 	gp_KetsjiScene = scene;
 }
+
+class KX_Scene* PHY_GetActiveScene()
+{
+	return gp_KetsjiScene;
+}
