@@ -1,15 +1,12 @@
 /*
- * $Id: KX_Scene.cpp 10455 2007-04-04 13:18:41Z campbellbarton $
+ * $Id: KX_Scene.cpp 14629 2008-04-30 19:58:44Z ben2610 $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  * Ketsji scene. Holds references to all scene data.
  */
 
@@ -35,6 +32,7 @@
 #pragma warning (disable : 4786)
 #endif //WIN32
 
+
 #include "KX_Scene.h"
 #include "MT_assert.h"
 
@@ -74,10 +72,18 @@
 #include "NG_NetworkScene.h"
 #include "PHY_IPhysicsEnvironment.h"
 #include "KX_IPhysicsController.h"
+#include "KX_BlenderSceneConverter.h"
 
 #include "BL_SkinDeformer.h"
 #include "BL_DeformableGameObject.h"
 
+// to get USE_BULLET!
+#include "KX_ConvertPhysicsObject.h"
+
+#ifdef USE_BULLET
+#include "CcdPhysicsEnvironment.h"
+#include "CcdPhysicsController.h"
+#endif
 
 void* KX_SceneReplicationFunc(SG_IObject* node,void* gameobj,void* scene)
 {
@@ -112,7 +118,8 @@ KX_Scene::KX_Scene(class SCA_IInputDevice* keyboarddevice,
 	m_adi(adi),
 	m_networkDeviceInterface(ndi),
 	m_active_camera(NULL),
-	m_ueberExecutionPriority(0)
+	m_ueberExecutionPriority(0),
+	m_sceneConverter(NULL)
 {
 	m_suspendedtime = 0.0;
 	m_suspendeddelta = 0.0;
@@ -124,6 +131,7 @@ KX_Scene::KX_Scene(class SCA_IInputDevice* keyboarddevice,
 	m_objectlist = new CListValue();
 	m_parentlist = new CListValue();
 	m_lightlist= new CListValue();
+	m_inactivelist = new CListValue();
 	m_euthanasyobjects = new CListValue();
 	m_delayReleaseObjects = new CListValue();
 
@@ -170,13 +178,14 @@ KX_Scene::KX_Scene(class SCA_IInputDevice* keyboarddevice,
 
 KX_Scene::~KX_Scene()
 {
-	
-//	int numobj = m_objectlist->GetCount();
+	// The release of debug properties used to be in SCA_IScene::~SCA_IScene
+	// It's still there but we remove all properties here otherwise some
+	// reference might be hanging and causing late release of objects
+	RemoveAllDebugProperties();
 
-	//int numrootobjects = GetRootParentList()->GetCount();
-	for (int i = 0; i < GetRootParentList()->GetCount(); i++)
+	while (GetRootParentList()->GetCount() > 0) 
 	{
-		KX_GameObject* parentobj = (KX_GameObject*) GetRootParentList()->GetValue(i);
+		KX_GameObject* parentobj = (KX_GameObject*) GetRootParentList()->GetValue(0);
 		this->RemoveObject(parentobj);
 	}
 
@@ -186,6 +195,9 @@ KX_Scene::~KX_Scene()
 	if (m_parentlist)
 		m_parentlist->Release();
 	
+	if (m_inactivelist)
+		m_inactivelist->Release();
+
 	if (m_lightlist)
 		m_lightlist->Release();
 	
@@ -213,11 +225,38 @@ KX_Scene::~KX_Scene()
 	{
 		delete m_bucketmanager;
 	}
-
+#ifdef USE_BULLET
+	// This is a fix for memory leaks in bullet: the collision shapes is not destroyed 
+	// when the physical controllers are destroyed. The reason is that shapes are shared
+	// between replicas of an object. There is no reference count in Bullet so the
+	// only workaround that does not involve changes in Bullet is to save in this array
+	// the list of shapes that are created when the scene is created (see KX_ConvertPhysicsObjects.cpp)
+	class btCollisionShape* shape;
+	class btTriangleMeshShape* meshShape;
+	vector<class btCollisionShape*>::iterator it = m_shapes.begin();
+	while (it != m_shapes.end()) {
+		shape = *it;
+		if (shape->getShapeType() == TRIANGLE_MESH_SHAPE_PROXYTYPE)
+		{
+			meshShape = static_cast<btTriangleMeshShape*>(shape);
+			// shapes based on meshes use an interface that contains the vertices.
+			// Again the idea is to be able to share the interface between shapes but
+			// this is not used in Blender: each base object will have its own interface 
+			btStridingMeshInterface* meshInterface = meshShape->getMeshInterface();
+			if (meshInterface)
+				delete meshInterface;
+		}
+		delete shape;
+		it++;
+	}
+#endif
 	//Py_DECREF(m_attrlist);
 }
 
-
+void KX_Scene::AddShape(class btCollisionShape*shape)
+{
+	m_shapes.push_back(shape);
+}
 
 
 void KX_Scene::SetProjectionMatrix(MT_CmMatrix4x4& pmat)
@@ -246,6 +285,11 @@ CListValue* KX_Scene::GetRootParentList()
 	return m_parentlist;
 }
 
+CListValue* KX_Scene::GetInactiveList()
+{
+	return m_inactivelist;
+}
+
 
 
 CListValue* KX_Scene::GetLightList()
@@ -266,7 +310,7 @@ SCA_TimeEventManager* KX_Scene::GetTimeEventManager()
 
 
  
-set<class KX_Camera*>* KX_Scene::GetCameras()
+list<class KX_Camera*>* KX_Scene::GetCameras()
 {
 	return &m_cameras;
 }
@@ -367,8 +411,15 @@ void KX_Scene::EnableZBufferClearing(bool isclearingZbuffer)
 void KX_Scene::RemoveNodeDestructObject(class SG_IObject* node,class CValue* gameobj)
 {
 	KX_GameObject* orgobj = (KX_GameObject*)gameobj;	
-	NewRemoveObject(orgobj);
-
+	if (NewRemoveObject(orgobj) != 0)
+	{
+		// object is not yet deleted (this can happen when it hangs in an add object actuator
+		// last object created reference. It's a bad situation, don't know how to fix it exactly
+		// The least I can do, is remove the reference to the node in the object as the node
+		// will in any case be deleted. This ensures that the object will not try to use the node
+		// when it is finally deleted (see KX_GameObject destructor)
+		orgobj->SetSGNode(NULL);
+	}
 	if (node)
 		delete node;
 }
@@ -418,7 +469,7 @@ KX_GameObject* KX_Scene::AddNodeReplicaObject(class SG_IObject* node, class CVal
 	replicanode->SetSGClientObject(newobj);
 
 	// this is the list of object that are send to the graphics pipeline
-	m_objectlist->Add(newobj);
+	m_objectlist->Add(newobj->AddRef());
 	newobj->Bucketize();
 
 	// logic cannot be replicated, until the whole hierarchy is replicated.
@@ -574,7 +625,9 @@ SCA_IObject* KX_Scene::AddReplicaObject(class CValue* originalobject,
 		// add a timebomb to this object
 		// for now, convert between so called frames and realtime
 		m_tempObjectList->Add(replica->AddRef());
-		replica->SetProperty("::timebomb",new CFloatValue(lifespan*0.02));
+		CValue *fval = new CFloatValue(lifespan*0.02);
+		replica->SetProperty("::timebomb",fval);
+		fval->Release();
 	}
 
 	// add to 'rootparent' list (this is the list of top hierarchy objects, updated each frame)
@@ -597,6 +650,8 @@ SCA_IObject* KX_Scene::AddReplicaObject(class CValue* originalobject,
 	for (git = m_logicHierarchicalGameObjects.begin();!(git==m_logicHierarchicalGameObjects.end());++git)
 	{
 		(*git)->Relink(&m_map_gameobject_to_replica);
+		// add the object in the layer of the parent
+		(*git)->SetLayer(parentobj->GetLayer());
 	}
 
 	// now replicate logic
@@ -637,7 +692,7 @@ SCA_IObject* KX_Scene::AddReplicaObject(class CValue* originalobject,
 	replica->GetSGNode()->UpdateWorldData(0);
 	replica->GetSGNode()->SetBBox(originalobj->GetSGNode()->BBox());
 	replica->GetSGNode()->SetRadius(originalobj->GetSGNode()->Radius());
-	
+	//	don't release replica here because we are returning it, not done with it...
 	return replica;
 }
 
@@ -657,7 +712,8 @@ void KX_Scene::RemoveObject(class CValue* gameobj)
 		// recursively destruct
 		node->Destruct();
 	}
-	newobj->SetSGNode(0);
+	//no need to do that: the object is destroyed and memory released 
+	//newobj->SetSGNode(0);
 }
 
 void KX_Scene::DelayedReleaseObject(CValue* gameobj)
@@ -677,8 +733,9 @@ void KX_Scene::DelayedRemoveObject(class CValue* gameobj)
 
 
 
-void KX_Scene::NewRemoveObject(class CValue* gameobj)
+int KX_Scene::NewRemoveObject(class CValue* gameobj)
 {
+	int ret;
 	KX_GameObject* newobj = (KX_GameObject*) gameobj;
 
 	//todo: look at this
@@ -699,6 +756,7 @@ void KX_Scene::NewRemoveObject(class CValue* gameobj)
 	{
 		(*itc)->UnlinkAllSensors();
 		(*itc)->UnlinkAllActuators();
+		m_logicmgr->RemoveController(*itc);
 	}
 
 	SCA_ActuatorList& actuators = newobj->GetActuators();
@@ -707,6 +765,7 @@ void KX_Scene::NewRemoveObject(class CValue* gameobj)
 	{
 		m_logicmgr->RemoveDestroyedActuator(*ita);
 	}
+	// the sensors/controllers/actuators must also be released, this is done in ~SCA_IObject
 
 	// now remove the timer properties from the time manager
 	int numprops = newobj->GetPropertyCount();
@@ -721,20 +780,31 @@ void KX_Scene::NewRemoveObject(class CValue* gameobj)
 	}
 	
 	newobj->RemoveMeshes();
+	ret = 1;
 	if (m_objectlist->RemoveValue(newobj))
-		newobj->Release();
+		ret = newobj->Release();
 	if (m_tempObjectList->RemoveValue(newobj))
-		newobj->Release();
+		ret = newobj->Release();
 	if (m_parentlist->RemoveValue(newobj))
-		newobj->Release();
+		ret = newobj->Release();
+	if (m_inactivelist->RemoveValue(newobj))
+		ret = newobj->Release();
 	if (m_euthanasyobjects->RemoveValue(newobj))
-		newobj->Release();
+		ret = newobj->Release();
 		
 	if (newobj == m_active_camera)
 	{
-		m_active_camera->Release();
+		//no AddRef done on m_active_camera so no Release
+		//m_active_camera->Release();
 		m_active_camera = NULL;
 	}
+	// in case this is a camera
+	m_cameras.remove((KX_Camera*)newobj);
+
+	if (m_sceneConverter)
+		m_sceneConverter->UnregisterGameObject(newobj);
+	// return value will be 0 if the object is actually deleted (all reference gone)
+	return ret;
 }
 
 
@@ -850,7 +920,7 @@ MT_CmMatrix4x4& KX_Scene::GetProjectionMatrix()
 
 KX_Camera* KX_Scene::FindCamera(KX_Camera* cam)
 {
-	set<KX_Camera*>::iterator it = m_cameras.begin();
+	list<KX_Camera*>::iterator it = m_cameras.begin();
 
 	while ( (it != m_cameras.end()) 
 			&& ((*it) != cam) ) {
@@ -863,7 +933,7 @@ KX_Camera* KX_Scene::FindCamera(KX_Camera* cam)
 
 KX_Camera* KX_Scene::FindCamera(STR_String& name)
 {
-	set<KX_Camera*>::iterator it = m_cameras.begin();
+	list<KX_Camera*>::iterator it = m_cameras.begin();
 
 	while ( (it != m_cameras.end()) 
 			&& ((*it)->GetName() != name) ) {
@@ -875,9 +945,11 @@ KX_Camera* KX_Scene::FindCamera(STR_String& name)
 
 void KX_Scene::AddCamera(KX_Camera* cam)
 {
-	m_cameras.insert(cam);
+	if (!FindCamera(cam))
+		m_cameras.push_back(cam);
 }
 
+
 KX_Camera* KX_Scene::GetActiveCamera()
 {	
 	// NULL if not defined
@@ -896,6 +968,17 @@ void KX_Scene::SetActiveCamera(KX_Camera* cam)
 	m_active_camera = cam;
 }
 
+void KX_Scene::SetCameraOnTop(KX_Camera* cam)
+{
+	if (!FindCamera(cam)){
+		// adding is always done at the back, so that's all that needs to be done
+		AddCamera(cam);
+		if (cam) std::cout << "Added cam " << cam->GetName() << std::endl;
+	} else {
+		m_cameras.remove(cam);
+		m_cameras.push_back(cam);
+	}
+}
 
 
 void KX_Scene::UpdateMeshTransformations()
@@ -1099,6 +1182,8 @@ void KX_Scene::LogicEndFrame()
 	for (i = numobj - 1; i >= 0; i--)
 	{
 		KX_GameObject* gameobj = (KX_GameObject*)m_euthanasyobjects->GetValue(i);
+		// KX_Scene::RemoveObject will also remove the object from this list
+		// that's why we start from the end
 		this->RemoveObject(gameobj);
 	}
 
@@ -1106,11 +1191,11 @@ void KX_Scene::LogicEndFrame()
 	for (i = numobj-1;i>=0;i--)
 	{
 		KX_GameObject* gameobj = (KX_GameObject*)m_delayReleaseObjects->GetValue(i);
-		m_delayReleaseObjects->RemoveValue(gameobj);	
-		
+		// This list is not for object removal, but just object release
+		gameobj->Release();
 	}
-	
-
+	// empty the list as we have removed all references
+	m_delayReleaseObjects->Resize(0);	
 }
 
 
@@ -1131,9 +1216,9 @@ void KX_Scene::UpdateParents(double curtime)
 
 
 
-RAS_MaterialBucket* KX_Scene::FindBucket(class RAS_IPolyMaterial* polymat)
+RAS_MaterialBucket* KX_Scene::FindBucket(class RAS_IPolyMaterial* polymat, bool &bucketCreated)
 {
-	return m_bucketmanager->RAS_BucketManagerFindBucket(polymat);
+	return m_bucketmanager->RAS_BucketManagerFindBucket(polymat, bucketCreated);
 }
 
 
@@ -1215,6 +1300,11 @@ void KX_Scene::SetNodeTree(SG_Tree* root)
 	m_objecttree = root;
 }
 
+void KX_Scene::SetSceneConverter(class KX_BlenderSceneConverter* sceneConverter)
+{
+	m_sceneConverter = sceneConverter;
+}
+
 void KX_Scene::SetPhysicsEnvironment(class PHY_IPhysicsEnvironment* physEnv)
 {
 	m_physicsEnvironment = physEnv;
