@@ -1,17 +1,14 @@
 /**
 * Set scene/camera stuff
 *
-* $Id: KX_SceneActuator.cpp 3919 2005-03-09 19:45:59Z lukep $
+* $Id: KX_SceneActuator.cpp 14444 2008-04-16 22:40:48Z hos $
 *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,7 +26,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
 */
 
 #include "SCA_IActuator.h"
@@ -105,6 +102,15 @@ bool KX_SceneActuator::Update()
 		{
 			m_scene->SetActiveCamera(m_camera);
 		}
+		else
+		{
+			// if no camera is set and the parent object is a camera, use it as the camera
+			SCA_IObject* parent = GetParent();
+			if (parent->isA(&KX_Camera::Type))
+			{
+				m_scene->SetActiveCamera((KX_Camera*)parent);
+			}
+		}
 		break;
 	default:
 		break;
