@@ -36,6 +36,7 @@ float gAngularSleepingTreshold = 1.0f;
 
 
 btVector3 startVel(0,0,0);//-10000);
+
 CcdPhysicsController::CcdPhysicsController (const CcdConstructionInfo& ci)
 :m_cci(ci)
 {
@@ -104,12 +105,13 @@ public:
 
 };
 
+
 void CcdPhysicsController::CreateRigidbody()
 {
 
-	btTransform trans = GetTransformFromMotionState(m_cci.m_MotionState);
+	btTransform trans = GetTransformFromMotionState(m_MotionState);
 
-	m_bulletMotionState = new BlenderBulletMotionState(m_cci.m_MotionState);
+	m_bulletMotionState = new BlenderBulletMotionState(m_MotionState);
 
 	m_body = new btRigidBody(m_cci.m_mass,
 		m_bulletMotionState,
@@ -118,20 +120,27 @@ void CcdPhysicsController::CreateRigidbody()
 		m_cci.m_linearDamping,m_cci.m_angularDamping,
 		m_cci.m_friction,m_cci.m_restitution);
 
-	
-
 	//
 	// init the rigidbody properly
 	//
 	
 	//setMassProps this also sets collisionFlags
 	//convert collision flags!
-
+	//special case: a near/radar sensor controller should not be defined static or it will
+	//generate loads of static-static collision messages on the console
+	if ((m_cci.m_collisionFilterGroup & CcdConstructionInfo::SensorFilter) != 0)
+	{
+		// reset the flags that have been set so far
+		m_body->setCollisionFlags(0);
+	}
 	m_body->setCollisionFlags(m_body->getCollisionFlags() | m_cci.m_collisionFlags);
-	
 	m_body->setGravity( m_cci.m_gravity);
 	m_body->setDamping(m_cci.m_linearDamping, m_cci.m_angularDamping);
 
+	if (!m_cci.m_bRigid)
+	{
+		m_body->setAngularFactor(0.f);
+	}
 }
 
 CcdPhysicsController::~CcdPhysicsController()
@@ -140,12 +149,14 @@ CcdPhysicsController::~CcdPhysicsController()
 	if (m_cci.m_physicsEnv)
 		m_cci.m_physicsEnv->removeCcdPhysicsController(this);
 
-	delete m_MotionState;
+	if (m_MotionState)
+		delete m_MotionState;
 	if (m_bulletMotionState)
 		delete m_bulletMotionState;
 	delete m_body;
 }
 
+
 		/**
 			SynchronizeMotionStates ynchronizes dynas, kinematic and deformable entities (and do 'late binding')
 		*/
@@ -275,7 +286,7 @@ void		CcdPhysicsController::RelativeTranslate(float dlocX,float dlocY,float dloc
 
 void		CcdPhysicsController::RelativeRotate(const float rotval[9],bool local)
 {
-	if (m_body )
+	if (m_body)
 	{
 		m_body->activate(true);
 		if (m_body->isStaticObject())
@@ -283,9 +294,9 @@ void		CcdPhysicsController::RelativeRotate(const float rotval[9],bool local)
 			m_body->setCollisionFlags(m_body->getCollisionFlags() | btCollisionObject::CF_KINEMATIC_OBJECT);
 		}
 
-		btMatrix3x3 drotmat(	rotval[0],rotval[1],rotval[2],
-								rotval[4],rotval[5],rotval[6],
-								rotval[8],rotval[9],rotval[10]);
+		btMatrix3x3 drotmat(	rotval[0],rotval[4],rotval[8],
+								rotval[1],rotval[5],rotval[9],
+								rotval[2],rotval[6],rotval[10]);
 
 
 		btMatrix3x3 currentOrn;
@@ -319,33 +330,39 @@ void		CcdPhysicsController::getOrientation(float &quatImag0,float &quatImag1,flo
 }
 void		CcdPhysicsController::setOrientation(float quatImag0,float quatImag1,float quatImag2,float quatReal)
 {
-	m_body->activate(true);
-	if (m_body->isStaticObject())
+	if (m_body)
 	{
-		m_body->setCollisionFlags(m_body->getCollisionFlags() | btCollisionObject::CF_KINEMATIC_OBJECT);
-	}
+		m_body->activate(true);
+		if (m_body->isStaticObject())
+		{
+			m_body->setCollisionFlags(m_body->getCollisionFlags() | btCollisionObject::CF_KINEMATIC_OBJECT);
+		}
 
-	m_MotionState->setWorldOrientation(quatImag0,quatImag1,quatImag2,quatReal);
-	btTransform xform  = m_body->getCenterOfMassTransform();
-	xform.setRotation(btQuaternion(quatImag0,quatImag1,quatImag2,quatReal));
-	m_body->setCenterOfMassTransform(xform);
-	m_bulletMotionState->setWorldTransform(xform);
+		m_MotionState->setWorldOrientation(quatImag0,quatImag1,quatImag2,quatReal);
+		btTransform xform  = m_body->getCenterOfMassTransform();
+		xform.setRotation(btQuaternion(quatImag0,quatImag1,quatImag2,quatReal));
+		m_body->setCenterOfMassTransform(xform);
+		m_bulletMotionState->setWorldTransform(xform);
+	}
 
 }
 
 void		CcdPhysicsController::setPosition(float posX,float posY,float posZ)
 {
-	m_body->activate(true);
-	if (m_body->isStaticObject())
+	if (m_body)
 	{
-		m_body->setCollisionFlags(m_body->getCollisionFlags() | btCollisionObject::CF_KINEMATIC_OBJECT);
+		m_body->activate(true);
+		if (m_body->isStaticObject())
+		{
+			m_body->setCollisionFlags(m_body->getCollisionFlags() | btCollisionObject::CF_KINEMATIC_OBJECT);
+		}
+		
+		m_MotionState->setWorldPosition(posX,posY,posZ);
+		btTransform xform  = m_body->getCenterOfMassTransform();
+		xform.setOrigin(btVector3(posX,posY,posZ));
+		m_body->setCenterOfMassTransform(xform);
+		m_bulletMotionState->setWorldTransform(xform);
 	}
-	
-	m_MotionState->setWorldPosition(posX,posY,posZ);
-	btTransform xform  = m_body->getCenterOfMassTransform();
-	xform.setOrigin(btVector3(posX,posY,posZ));
-	m_body->setCenterOfMassTransform(xform);
-	m_bulletMotionState->setWorldTransform(xform);
 
 
 }
@@ -389,49 +406,59 @@ void		CcdPhysicsController::ApplyTorque(float torqueX,float torqueY,float torque
 {
 	btVector3 torque(torqueX,torqueY,torqueZ);
 	btTransform xform = m_body->getCenterOfMassTransform();
-	if (torque.length2() > (SIMD_EPSILON*SIMD_EPSILON))
+	if (m_body && torque.length2() > (SIMD_EPSILON*SIMD_EPSILON))
 	{
 		m_body->activate();
+		if (m_body->isStaticObject())
+		{
+			m_body->setCollisionFlags(m_body->getCollisionFlags() | btCollisionObject::CF_KINEMATIC_OBJECT);
+		}
+		if (local)
+		{
+			torque	= xform.getBasis()*torque;
+		}
+		m_body->applyTorque(torque);
 	}
-	if (local)
-	{
-		torque	= xform.getBasis()*torque;
-	}
-	m_body->applyTorque(torque);
 }
 
 void		CcdPhysicsController::ApplyForce(float forceX,float forceY,float forceZ,bool local)
 {
 	btVector3 force(forceX,forceY,forceZ);
 	
-	if (force.length2() > (SIMD_EPSILON*SIMD_EPSILON))
+	if (m_body && force.length2() > (SIMD_EPSILON*SIMD_EPSILON))
 	{
 		m_body->activate();
+		if (m_body->isStaticObject())
+		{
+			m_body->setCollisionFlags(m_body->getCollisionFlags() | btCollisionObject::CF_KINEMATIC_OBJECT);
+		}
+		{
+			btTransform xform = m_body->getCenterOfMassTransform();
+			if (local)
+			{	
+				force	= xform.getBasis()*force;
+			}
+		}
+		m_body->applyCentralForce(force);
 	}
-
-
-	btTransform xform = m_body->getCenterOfMassTransform();
-	if (local)
-	{
-		force	= xform.getBasis()*force;
-	}
-	m_body->applyCentralForce(force);
 }
 void		CcdPhysicsController::SetAngularVelocity(float ang_velX,float ang_velY,float ang_velZ,bool local)
 {
 	btVector3 angvel(ang_velX,ang_velY,ang_velZ);
-	if (angvel.length2() > (SIMD_EPSILON*SIMD_EPSILON))
+	if (m_body && angvel.length2() > (SIMD_EPSILON*SIMD_EPSILON))
 	{
 		m_body->activate(true);
-	}
-
-	{
-		btTransform xform = m_body->getCenterOfMassTransform();
-		if (local)
+		if (m_body->isStaticObject())
 		{
-			angvel	= xform.getBasis()*angvel;
+			m_body->setCollisionFlags(m_body->getCollisionFlags() | btCollisionObject::CF_KINEMATIC_OBJECT);
+		}
+		{
+			btTransform xform = m_body->getCenterOfMassTransform();
+			if (local)
+			{
+				angvel	= xform.getBasis()*angvel;
+			}
 		}
-
 		m_body->setAngularVelocity(angvel);
 	}
 
@@ -440,16 +467,20 @@ void		CcdPhysicsController::SetLinearVelocity(float lin_velX,float lin_velY,floa
 {
 
 	btVector3 linVel(lin_velX,lin_velY,lin_velZ);
-	if (linVel.length2() > (SIMD_EPSILON*SIMD_EPSILON))
+	if (m_body && linVel.length2() > (SIMD_EPSILON*SIMD_EPSILON))
 	{
 		m_body->activate(true);
-	}
-	
-	{
-		btTransform xform = m_body->getCenterOfMassTransform();
-		if (local)
+		if (m_body->isStaticObject())
 		{
-			linVel	= xform.getBasis()*linVel;
+			m_body->setCollisionFlags(m_body->getCollisionFlags() | btCollisionObject::CF_KINEMATIC_OBJECT);
+		}
+
+		{
+			btTransform xform = m_body->getCenterOfMassTransform();
+			if (local)
+			{
+				linVel	= xform.getBasis()*linVel;
+			}
 		}
 		m_body->setLinearVelocity(linVel);
 	}
@@ -458,14 +489,16 @@ void		CcdPhysicsController::applyImpulse(float attachX,float attachY,float attac
 {
 	btVector3 impulse(impulseX,impulseY,impulseZ);
 
-	if (impulse.length2() > (SIMD_EPSILON*SIMD_EPSILON))
+	if (m_body && impulse.length2() > (SIMD_EPSILON*SIMD_EPSILON))
 	{
 		m_body->activate();
+		if (m_body->isStaticObject())
+		{
+			m_body->setCollisionFlags(m_body->getCollisionFlags() | btCollisionObject::CF_KINEMATIC_OBJECT);
+		}
 		
 		btVector3 pos(attachX,attachY,attachZ);
 
-		m_body->activate();
-
 		m_body->applyImpulse(impulse,pos);
 	}
 
