@@ -46,7 +46,8 @@ struct CcdConstructionInfo
 	        StaticFilter = 2,
 	        KinematicFilter = 4,
 	        DebrisFilter = 8,
-	        AllFilter = DefaultFilter | StaticFilter | KinematicFilter | DebrisFilter,
+			SensorFilter = 16,
+	        AllFilter = DefaultFilter | StaticFilter | KinematicFilter | DebrisFilter | SensorFilter,
 	};
 
 
@@ -59,8 +60,10 @@ struct CcdConstructionInfo
 		m_linearDamping(0.1f),
 		m_angularDamping(0.1f),
 		m_collisionFlags(0),
+		m_bRigid(false),
 		m_collisionFilterGroup(DefaultFilter),
 		m_collisionFilterMask(AllFilter),
+		m_collisionShape(0),
 		m_MotionState(0),
 		m_physicsEnv(0),
 		m_inertiaFactor(1.f)
@@ -76,6 +79,7 @@ struct CcdConstructionInfo
 	btScalar	m_linearDamping;
 	btScalar	m_angularDamping;
 	int			m_collisionFlags;
+	bool		m_bRigid;
 
 	///optional use of collision group/mask:
 	///only collision with object goups that match the collision mask.
@@ -85,9 +89,8 @@ struct CcdConstructionInfo
 	short int	m_collisionFilterGroup;
 	short int	m_collisionFilterMask;
 
-
-	btCollisionShape*			m_collisionShape;
-	class	PHY_IMotionState*			m_MotionState;
+	class btCollisionShape*	m_collisionShape;
+	class PHY_IMotionState*	m_MotionState;
 	
 	CcdPhysicsEnvironment*	m_physicsEnv; //needed for self-replication
 	float	m_inertiaFactor;//tweak the inertia (hooked up to Blender 'formfactor'
@@ -96,11 +99,12 @@ struct CcdConstructionInfo
 
 class btRigidBody;
 
+
 ///CcdPhysicsController is a physics object that supports continuous collision detection and time of impact based physics resolution.
 class CcdPhysicsController : public PHY_IPhysicsController	
 {
 	btRigidBody* m_body;
-	class	PHY_IMotionState*			m_MotionState;
+	class PHY_IMotionState*		m_MotionState;
 	btMotionState* 	m_bulletMotionState;
 
 
@@ -215,7 +219,11 @@ class CcdPhysicsController : public PHY_IPhysicsController
 			return m_MotionState;
 		}
 
-
+		class CcdPhysicsEnvironment* GetPhysicsEnvironment()
+		{
+			return m_cci.m_physicsEnv;
+		}
+		
 };
 
 
