@@ -1,15 +1,12 @@
 /**
- * $Id: SumoPhysicsEnvironment.cpp,v 1.16 2006/12/16 05:50:37 erwin Exp $
+ * $Id: SumoPhysicsEnvironment.cpp 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 #include "SumoPhysicsEnvironment.h"
 #include "PHY_IMotionState.h"
@@ -206,6 +203,8 @@ void SumoPhysicsEnvironment::addTouchCallback(int response_class, PHY_ResponseCa
 	case PHY_STATIC_RESPONSE:
 		sumoRespClass = PHY_STATIC_RESPONSE;
 		break;
+	case PHY_BROADPH_RESPONSE:
+		return;
 	default:
 		assert(0);
 		return;
