@@ -19,7 +19,6 @@ subject to the following restrictions:
 
 
 
-class	PHY_ResponseTable;
 
 class PHY_Shape;
 
@@ -40,10 +39,11 @@ struct	PHY__Vector3
 typedef enum
 {
 	PHY_FH_RESPONSE,
-	PHY_SENSOR_RESPONSE,		/* Touch Sensors */
+	PHY_SENSOR_RESPONSE,	/* Touch Sensors */
 	PHY_CAMERA_RESPONSE,	/* Visibility Culling */
 	PHY_OBJECT_RESPONSE,	/* Object Dynamic Geometry Response */
 	PHY_STATIC_RESPONSE,	/* Static Geometry Response */
+	PHY_BROADPH_RESPONSE,	/* broadphase Response */
 	
 	PHY_NUM_RESPONSE
 };
@@ -81,6 +81,7 @@ typedef enum PHY_ConstraintType {
 	PHY_POINT2POINT_CONSTRAINT=1,
 	PHY_LINEHINGE_CONSTRAINT=2,
 	PHY_ANGULAR_CONSTRAINT = 3,//hinge without ball socket
+	PHY_CONE_TWIST_CONSTRAINT = 4,
 	PHY_VEHICLE_CONSTRAINT=11,//complex 'constraint' that turns a rigidbody into a vehicle
 	PHY_GENERIC_6DOF_CONSTRAINT=12,//can leave any of the 6 degree of freedom 'free' or 'locked'
 
