@@ -1,14 +1,11 @@
 /**
- * $Id: RAS_BucketManager.cpp 7252 2006-04-11 05:57:30Z erwin $
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * $Id: RAS_BucketManager.cpp 14638 2008-05-01 12:35:21Z ben2610 $
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +23,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifdef HAVE_CONFIG_H
@@ -116,6 +113,7 @@ void RAS_BucketManager::RenderAlphaBuckets(
 	std::multiset< alphamesh, backtofront>::iterator msit = alphameshset.begin();
 	for (; msit != alphameshset.end(); ++msit)
 	{
+		rendertools->SetClientObject((*(*msit).m_ms).m_clientObj);
 		while ((*msit).m_bucket->ActivateMaterial(cameratrans, rasty, rendertools, drawingmode))
 			(*msit).m_bucket->RenderMeshSlot(cameratrans, rasty, rendertools, *(*msit).m_ms, drawingmode);
 	}
@@ -156,8 +154,9 @@ void RAS_BucketManager::Renderbuckets(
 	RAS_MaterialBucket::EndFrame();
 }
 
-RAS_MaterialBucket* RAS_BucketManager::RAS_BucketManagerFindBucket(RAS_IPolyMaterial * material)
+RAS_MaterialBucket* RAS_BucketManager::RAS_BucketManagerFindBucket(RAS_IPolyMaterial * material, bool &bucketCreated)
 {
+	bucketCreated = false;
 	BucketList::iterator it;
 	for (it = m_MaterialBuckets.begin(); it != m_MaterialBuckets.end(); it++)
 	{
@@ -172,6 +171,7 @@ RAS_MaterialBucket* RAS_BucketManager::RAS_BucketManagerFindBucket(RAS_IPolyMate
 	}
 	
 	RAS_MaterialBucket *bucket = new RAS_MaterialBucket(material);
+	bucketCreated = true;
 	if (bucket->IsTransparant())
 		m_AlphaBuckets.push_back(bucket);
 	else
