@@ -1,15 +1,12 @@
 /**
- * $Id: RAS_IRasterizer.h,v 1.20 2007/04/04 13:18:40 campbellbarton Exp $
+ * $Id: RAS_IRasterizer.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 #ifndef __RAS_IRASTERIZER
 #define __RAS_IRASTERIZER
@@ -48,7 +45,6 @@ class RAS_IPolyMaterial;
  */
 class RAS_IRasterizer
 {
-
 public:
 
 	RAS_IRasterizer(RAS_ICanvas* canv){};
@@ -398,7 +394,15 @@ public:
 
 	virtual bool	QueryLists(){return false;}
 	virtual bool	QueryArrays(){return false;}
+	
+	virtual void	EnableMotionBlur(float motionblurvalue)=0;
+	virtual void	DisableMotionBlur()=0;
+	
+	virtual float	GetMotionBlurValue()=0;
+	virtual int	GetMotionBlurState()=0;
+	virtual void SetMotionBlurState(int newstate)=0;
 };
 
 #endif //__RAS_IRASTERIZER
 
+
