@@ -1,14 +1,11 @@
 /**
- * $Id: RAS_MaterialBucket.cpp 10932 2007-06-14 19:29:39Z bdiego $
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * $Id: RAS_MaterialBucket.cpp 14629 2008-04-30 19:58:44Z ben2610 $
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +23,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include "RAS_MaterialBucket.h"
@@ -69,6 +66,11 @@ bool KX_MeshSlot::Less(const KX_MeshSlot& lhs) const
 	return result;
 }
 
+KX_MeshSlot::~KX_MeshSlot()
+{
+	if (m_DisplayList)
+		m_DisplayList->Release();
+}
 
 
 RAS_MaterialBucket::RAS_MaterialBucket(RAS_IPolyMaterial* mat)
@@ -187,7 +189,7 @@ bool RAS_MaterialBucket::ActivateMaterial(const MT_Transform& cameratrans, RAS_I
 	}
 	else
 	{
-		rendertools->ProcessLighting(m_material->GetLightLayer());
+		rendertools->ProcessLighting(RAS_IRenderTools::RAS_LIGHT_OBJECT_LAYER/*m_material->GetLightLayer()*/);
 	}
 
 	drawmode = (rasty->GetDrawingMode()  < RAS_IRasterizer::KX_SOLID ? 	
@@ -202,7 +204,6 @@ void RAS_MaterialBucket::RenderMeshSlot(const MT_Transform& cameratrans, RAS_IRa
 	if (!ms.m_bVisible)
 		return;
 	
-	rendertools->SetClientObject(ms.m_clientObj);
 	m_material->ActivateMeshSlot(ms, rasty);
 
 	/* __NLA Do the deformation */
@@ -315,15 +316,12 @@ void RAS_MaterialBucket::Render(const MT_Transform& cameratrans,
 
 	//rasty->SetMaterial(*m_material);
 	
-	if (m_meshSlots.size() >0)
-	{
-		rendertools->SetClientObject((*m_meshSlots.begin()).m_clientObj);
-	}
 	
 	int drawmode;
 	for (T_MeshSlotList::const_iterator it = m_meshSlots.begin();
 	! (it == m_meshSlots.end()); ++it)
 	{
+		rendertools->SetClientObject((*it).m_clientObj);
 		while (ActivateMaterial(cameratrans, rasty, rendertools, drawmode))
 			RenderMeshSlot(cameratrans, rasty, rendertools, *it, drawmode);
 	}
