@@ -1,15 +1,12 @@
 /**
- * $Id: RAS_MaterialBucket.h 7187 2006-04-02 21:04:20Z erwin $
+ * $Id: RAS_MaterialBucket.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 #ifndef __RAS_MATERIALBUCKET
 #define __RAS_MATERIALBUCKET
@@ -69,10 +66,21 @@ public:
  */
 class KX_ListSlot
 {
+protected:
+	int m_refcount;
 public:
-	KX_ListSlot(){}
-	virtual ~KX_ListSlot(){}
-
+	KX_ListSlot(){ m_refcount=1; }
+	virtual ~KX_ListSlot() {}
+	virtual int Release() { 
+		if (--m_refcount > 0)
+			return m_refcount;
+		delete this;
+		return 0;
+	}
+	virtual KX_ListSlot* AddRef() {
+		m_refcount++;
+		return this;
+	}
 	virtual void SetModified(bool mod)=0;
 };
 
@@ -96,7 +104,7 @@ public:
 		m_DisplayList(0)
 	{
 	}
-	~KX_MeshSlot(){}
+	~KX_MeshSlot();
 	bool					Less(const KX_MeshSlot& lhs) const;
 };
 
