@@ -1,15 +1,12 @@
 /**
- * $Id: RAS_OpenGLRasterizer.h,v 1.17 2006/04/11 05:57:30 erwin Exp $
+ * $Id: RAS_OpenGLRasterizer.h 14444 2008-04-16 22:40:48Z hos $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 #ifndef __RAS_OPENGLRASTERIZER
 #define __RAS_OPENGLRASTERIZER
@@ -91,6 +88,10 @@ class RAS_OpenGLRasterizer : public RAS_IRasterizer
 	int				m_noOfScanlines;
 	bool			InterlacedStereo() const;
 
+	//motion blur
+	int	m_motionblur;
+	float	m_motionblurvalue;
+
 protected:
 	int				m_drawingmode;
 	TexCoGen		m_texco[RAS_MAX];
@@ -293,8 +294,22 @@ public:
 					const RAS_TexVert& v2,
 					const RAS_TexVert& v3,
 					const MT_Vector3 &no);
-
+	
+	virtual void	EnableMotionBlur(float motionblurvalue);
+	virtual void	DisableMotionBlur();
+	virtual float	GetMotionBlurValue(){return m_motionblurvalue;};
+	virtual int	GetMotionBlurState(){return m_motionblur;};
+	virtual void SetMotionBlurState(int newstate)
+	{
+		if(newstate<0) 
+			m_motionblur = 0;
+		else if(newstate>2)
+			m_motionblur = 2;
+		else 
+			m_motionblur = newstate;
+	};
 };
 
 #endif //__RAS_OPENGLRASTERIZER
 
+
