@@ -1,11 +1,15 @@
+
 import os
 import os.path
 import SCons.Options
-import SCons.Options.BoolOption
+try:
+    import SCons.Options.BoolOption
+except ImportError:
+    pass
 try:
     import subprocess
 except ImportError:
-	pass
+    pass
 import string
 import glob
 import shutil
@@ -29,7 +33,8 @@ def validate_arguments(args, bc):
             'BF_PTHREADS', 'BF_PTHREADS_INC', 'BF_PTHREADS_LIB', 'BF_PTHREADS_LIBPATH',
             'WITH_BF_FMOD',
             'WITH_BF_OPENEXR', 'BF_OPENEXR', 'BF_OPENEXR_INC', 'BF_OPENEXR_LIB', 'BF_OPENEXR_LIBPATH',
-            'WITH_BF_FFMPEG', 'BF_FFMPEG_LIB', 'BF_FFMPEG',  'BF_FFMPEG_INC',
+            'WITH_BF_DDS',
+            'WITH_BF_FFMPEG', 'BF_FFMPEG_LIB','BF_FFMPEG_EXTRA', 'BF_FFMPEG',  'BF_FFMPEG_INC',
             'WITH_BF_JPEG', 'BF_JPEG', 'BF_JPEG_INC', 'BF_JPEG_LIB', 'BF_JPEG_LIBPATH',
             'WITH_BF_PNG', 'BF_PNG', 'BF_PNG_INC', 'BF_PNG_LIB', 'BF_PNG_LIBPATH',
             'BF_TIFF', 'BF_TIFF_INC',
@@ -45,22 +50,30 @@ def validate_arguments(args, bc):
             'WITH_BF_QUICKTIME', 'BF_QUICKTIME', 'BF_QUICKTIME_INC', 'BF_QUICKTIME_LIB', 'BF_QUICKTIME_LIBPATH',
             'WITH_BF_STATICOPENGL', 'BF_OPENGL', 'BF_OPENGL_INC', 'BF_OPENGL_LIB', 'BF_OPENGL_LIBPATH', 'BF_OPENGL_LIB_STATIC', 'BF_OPENGL_LINKFLAGS',
             'WITH_BF_FTGL', 'BF_FTGL', 'BF_FTGL_INC', 'BF_FTGL_LIB',
-            'WITH_BF_FFMPEG',
             'WITH_BF_PLAYER',
+	    'WITH_BF_GLEXT',
+            'WITH_BF_BINRELOC',	
             'CFLAGS', 'CCFLAGS', 'CPPFLAGS', 
             'REL_CFLAGS', 'REL_CCFLAGS',
             'C_WARN', 'CC_WARN', 'LLIBS', 'PLATFORM_LINKFLAGS',
             'BF_PROFILE_FLAGS', 'LCGDIR', 'WITH_BF_VERSE', 
             'BF_VERSE_INCLUDE',
             'VERSE_BUILD_BINARY', 'VERSE_BUILD_DIR', 'VERSE_REGEN_PROTO',
-            'BF_TWEAK_MODE'
+            'BF_TWEAK_MODE', 'BF_SPLIT_SRC',
+            'WITHOUT_BF_INSTALL',
+            'WITH_BF_OPENMP',
+            'WITHOUT_BF_INSTALL',
+            'BF_FANCY', 'BF_QUIET',
+            'BF_X264_CONFIG',
+            'BF_XVIDCORE_CONFIG',
             ]
 
     arg_list = ['BF_DEBUG', 'BF_QUIET', 'BF_CROSS', 'BF_UPDATE',
             'BF_INSTALLDIR', 'BF_TOOLSET', 'BF_BINNAME',
             'BF_BUILDDIR', 'BF_FANCY', 'BF_QUICK', 'BF_PROFILE',
-            'BF_DEBUG_FLAGS', 'BF_BSC',
-            'BF_PRIORITYLIST', 'BF_BUILDINFO','CC', 'CXX', "BF_QUICKDEBUG", "BF_LISTDEBUG", 'LCGDIR']
+            'BF_DEBUG_FLAGS', 'BF_BSC', 'BF_CONFIG',
+            'BF_PRIORITYLIST', 'BF_BUILDINFO','CC', 'CXX', 'BF_QUICKDEBUG',
+            'BF_LISTDEBUG', 'LCGDIR', 'BF_X264_CONFIG', 'BF_XVIDCORE_CONFIG']
 
     all_list = opts_list + arg_list
     okdict = {}
@@ -162,9 +175,12 @@ def read_opts(cfg, args):
         ('BF_OPENEXR_LIB', 'OPENEXR library', ''),
         ('BF_OPENEXR_LIBPATH', 'OPENEXR library path', ''),
 
+        (BoolOption('WITH_BF_DDS', 'Use DDS if true', 'true')),
+
         (BoolOption('WITH_BF_FFMPEG', 'Use FFMPEG if true', 'false')),
         ('BF_FFMPEG', 'FFMPEG base path', ''),
         ('BF_FFMPEG_LIB', 'FFMPEG library', ''),
+        ('BF_FFMPEG_EXTRA', 'FFMPEG flags that must be preserved', ''),
 
         ('BF_FFMPEG_INC', 'FFMPEG includes', ''),
         ('BF_FFMPEG_LIBPATH', 'FFMPEG library path', ''),
@@ -243,6 +259,8 @@ def read_opts(cfg, args):
         ('BF_FREETYPE_LIB', 'Freetype library', ''),
         ('BF_FREETYPE_LIBPATH', 'Freetype library path', ''),
 
+        (BoolOption('WITH_BF_OPENMP', 'Use OpenMP if true', 'false')),
+
         (BoolOption('WITH_BF_QUICKTIME', 'Use QuickTime if true', 'false')),
         ('BF_QUICKTIME', 'QuickTime base path', ''),
         ('BF_QUICKTIME_INC', 'QuickTime include path', ''),
@@ -262,6 +280,7 @@ def read_opts(cfg, args):
         ('BF_FTGL_INC', 'FTGL include path', ''),
         ('BF_FTGL_LIB', 'FTGL libraries', ''),
 
+        (BoolOption('WITH_BF_GLEXT', 'Enable GL Extensions', 'true')),
         (BoolOption('WITH_BF_PLAYER', 'Build blenderplayer if true', 'false')),
 
         ('CFLAGS', 'C-compiler flags', ''),
@@ -283,7 +302,7 @@ def read_opts(cfg, args):
         ('BF_DEBUG_FLAGS', 'Debug flags', ''),
 
         (BoolOption('BF_BSC', 'Create .bsc files (msvc only)', 'true')),
-        
+
         ('BF_BUILDDIR', 'Build dir', ''),
         ('BF_INSTALLDIR', 'Installation dir', ''),
 
@@ -293,6 +312,14 @@ def read_opts(cfg, args):
         (BoolOption('BF_BUILDINFO', 'Buildtime in splash if true', 'true')),
 
         (BoolOption('BF_TWEAK_MODE', 'Enable tweak mode if true', 'false')),
+        (BoolOption('BF_SPLIT_SRC', 'Split src lib into several chunks if true', 'false')),
+        (BoolOption('WITHOUT_BF_INSTALL', 'dont install if true', 'false')),
+        (BoolOption('BF_FANCY', 'Enable fancy output if true', 'true')),
+        (BoolOption('BF_QUIET', 'Enable silent output if true', 'true')),
+        (BoolOption('WITH_BF_BINRELOC', 'Enable relocatable binary (linux only)', 'false')),
+
+        ('BF_X264_CONFIG', 'configuration flags for x264', ''),
+        ('BF_XVIDCORE_CONFIG', 'configuration flags for xvidcore', ''),
 
     ) # end of opts.AddOptions()
 
